Imports System
Imports System.Collections
Imports System.Windows.Forms
Imports System.Drawing
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Structure Personnel
    Dim Loc As RectangleF
    'This will hold the current location of the player.  X and Y included.
    'Why didn't I use point?  Because I am probably going to use the height and width for collisions later.
    Dim Direc As Integer
    'Which direction is the character facing? 
    Dim AnimCycler As Integer
    'This is the animation cycler that will count from 0 to 3 and back to 0.
    Dim IsJumping, IsLongJumping As Boolean
    'While this is true, the character will be jumping.
    Dim Veloc As Single
    'This is the velocity of the player in the Y direction.
    'Positive values point down.
    'To jump up, PlayerVeloc must be initialized to a negative value.
    Dim HorizontVeloc As Single
    'How fast the player is moving left-right on surface such as ice.
    Dim OnIce As Boolean
    'True if the player is walking on ice.
    Dim InWater As Boolean
    'Is the player swimming?
    Dim OnSlope As Boolean
    'Is the player on a slope
    Dim WKReady, WKActive As Boolean
    'Is player jumping against a wall?

    Dim LeftEnd As Single
    Dim RightEnd As Single
    Dim LH, RH As Single
    'This stores the locations of the end of the cliff that the character is currently on.
    Dim IsClimbing As Boolean, LB As Single 'LH as single

    Dim Lives As Integer

    Dim HP As Integer
    Dim TemporarilyInvulnerable As Integer
    Dim Shots() As SelfBullet
    Dim IsShooting As Boolean

    Dim BulletType As ShotTypes
    Dim WeaponType As ShotPattern

    Dim OwnedBullets() As Boolean
    Dim OwnedWeapons() As Boolean

    Dim Coincount As Integer
    Dim DefenseUp As Boolean
    Dim EvadeUp As Boolean
    Dim VisionUp As Boolean
    Dim ShotTime As Integer
    Dim ShotSpeed As Single

    Dim LongJumpEnable As Boolean
    Dim WallKickEnable As Boolean
    Dim DoubleJumpEnable As Boolean
    Dim IceStopEnable As Boolean
    Dim FlameRetardEnable As Boolean
    Dim MysticPowerEnable As Boolean
End Structure


Public Enum PeriodicDesignation
    Stable = 0  'Denotes that the object will not move in this direction.
    CosineFxn   'Moves using the cosine function.
    SineFxn     'Moves using the sine function.
    Triangle
    Hexagon
    Trapezoid
    Scalene
    TriangleforSquare   'Trianglewave with offset of 0.5
    HexagonforSquare    'Hexagonwave with offset of 1.
    TrapezoidforRightTriangle   'Trapezoid, offset 1.
    ScaleneforWedge     'Scalene, offset 1.
End Enum
Public Structure LevFloater
    Private rect As RectangleF
    Private horizfxn As PeriodicDesignation
    Private vertfxn As PeriodicDesignation
    Private xrspeed As Integer
    Private yrspeed As Integer
    Private xfreq As Integer
    Private yfreq As Integer

    Public Sub New(ByVal R As RectangleF, ByVal RadispdX As Integer, ByVal RadispdY As Integer, ByVal FreqX As Integer, ByVal FreqY As Integer, ByVal HFxn As PeriodicDesignation, ByVal VFxn As PeriodicDesignation)
        Me.horizfxn = HFxn
        Me.rect = R
        Me.vertfxn = VFxn
        Me.xfreq = FreqX
        Me.xrspeed = RadispdX
        Me.yfreq = FreqY
        Me.yrspeed = RadispdY

    End Sub

    Public Sub Offset(ByVal dX As Single, ByVal dY As Single)
        rect.Offset(dX, dY)
    End Sub

    Public Property Loc() As RectangleF
        Get
            Return rect
        End Get
        Set(ByVal Value As RectangleF)
            rect = Value
        End Set
    End Property

    Public Property FunctionHorizontal() As PeriodicDesignation
        Get
            Return horizfxn
        End Get
        Set(ByVal Value As PeriodicDesignation)
            horizfxn = Value
        End Set
    End Property

    Public Property FunctionVertical() As PeriodicDesignation
        Get
            Return vertfxn
        End Get
        Set(ByVal Value As PeriodicDesignation)
            vertfxn = Value
        End Set
    End Property

    Public Property RadispeedX() As Integer
        Get
            Return xrspeed
        End Get
        Set(ByVal Value As Integer)
            xrspeed = Value
        End Set
    End Property

    Public Property RadispeedY() As Integer
        Get
            Return yrspeed
        End Get
        Set(ByVal Value As Integer)
            yrspeed = Value
        End Set
    End Property

    Public Property FrequencyX() As Integer
        Get
            Return xfreq
        End Get
        Set(ByVal Value As Integer)
            xfreq = Value
        End Set
    End Property

    Public Property FrequencyY() As Integer
        Get
            Return yfreq
        End Get
        Set(ByVal Value As Integer)
            yfreq = Value
        End Set
    End Property
End Structure
Public Structure JumpingBoard
    Public Loc As RectangleF 'Location.
    Public ReboundVeloc As Integer  'The velocity at which the player will jump to when he lands on this board.

    Public Sub New(ByVal X As Integer, ByVal Y As Integer, ByVal W As Integer, ByVal H As Integer, ByVal Rebound As Integer) 'Quickly set our jumping board.
        Loc = New RectangleF(X, Y, W, H)
        ReboundVeloc = Rebound
    End Sub
End Structure
Public Structure DataBlock
    Public Loc As RectangleF 'Location.
    Public Data As Single   'For a conveyor, this is the horizontal belt velocity.

    Public Sub New(ByVal L As Single, ByVal T As Single, ByVal R As Single, ByVal B As Single, ByVal Info As Single) 'Quickly set our jumping board.
        Loc = RectangleF.FromLTRB(L, T, R, B)
        Data = Info
    End Sub

End Structure
Public Structure MovingWall
    Implements IWallmove
    Private rect As RectangleF  'The location of the wall.
    Private cx As Single       'How much the wall moves in the horizontal.
    Private cy As Single      'How much the wall moves in the vertical.
    Private bound As RectangleF  'The wall cannot move outside of this square.
    Public Property Loc() As RectangleF Implements IWallmove.Loc
        Get
            Return rect
        End Get
        Set(ByVal Value As RectangleF)
            rect = Value
        End Set
    End Property
    Public Property dX() As Single Implements IWallmove.dX
        Get
            Return cx
        End Get
        Set(ByVal Value As Single)
            cx = Value
        End Set
    End Property
    Public Property dY() As Single Implements IWallmove.dY
        Get
            Return cy
        End Get
        Set(ByVal Value As Single)
            cy = Value
        End Set
    End Property
    Public Property Boundary() As RectangleF Implements IWallmove.Boundary
        Get
            Return bound
        End Get
        Set(ByVal Value As RectangleF)
            bound = Value
        End Set
    End Property
    Public Sub New(ByVal R As RectangleF, ByVal dX As Integer, ByVal dY As Integer, ByVal Boundary As RectangleF)
        Me.rect = R
        Me.cx = dX
        Me.cy = dY
        Me.bound = Boundary
    End Sub
    Public Sub Offset(ByVal dX As Integer, ByVal dY As Integer) Implements IWallmove.Offset
        rect.Offset(dX, dY)
        bound.Offset(dX, dY)
    End Sub
    Public Sub MoveWall() Implements IWallmove.MoveWall
        'This subroutine coordinates the in-game functionality of the moving wall with the rest of the game.
        'Just add the collision checks against rect.
        rect.Offset(cx, cy) 'Move the wall.
        If rect.Right > bound.Right OrElse rect.Left < bound.Left Then
            'When the wall hits the edge of our boundary, then it must begin moving in the other direction.
            cx = -cx
        End If
        If rect.Bottom > bound.Bottom OrElse rect.Top < bound.Top Then
            'This is for vertical movement direction switching.
            cy = -cy
        End If
    End Sub
    Public Sub MoveHorizontal() Implements IWallmove.MoveHorizontal
        'This subroutine does just like the Movewall does, but only in the horizontal, with cx, left and right.
        rect.Offset(cx, 0) 'Move.
        If rect.Right >= bound.Right OrElse rect.Left <= bound.Left Then
            'When the wall hits the edge of our boundary, then it must begin moving in the other direction.
            cx = -cx
        End If
    End Sub
    Public Sub MoveVertical() Implements IWallmove.MoveVertical
        'Same as MoveWall, but only in the vertical, with cy, top and bottom.
        rect.Offset(0, cy) 'Move.
        If rect.Bottom >= bound.Bottom OrElse rect.Top <= bound.Top Then
            'This is for vertical movement direction switching.
            cy = -cy
        End If
    End Sub
End Structure
Public Structure MovingWallBoxCW
    Implements IWallmove
    Private rect As RectangleF  'The location of the wall.
    Private cx As Single       'How much the wall moves in the horizontal.
    Private cy As Single       'How much the wall moves in the vertical.
    Private bound As RectangleF  'The wall cannot move outside of this square.

    Public Property Loc() As RectangleF Implements IWallmove.Loc
        Get
            Return rect
        End Get
        Set(ByVal Value As RectangleF)
            rect = Value
        End Set
    End Property
    Public Property dX() As Single Implements IWallmove.dX
        Get
            Return cx
        End Get
        Set(ByVal Value As Single)
            cx = Value
        End Set
    End Property
    Public Property dY() As Single Implements IWallmove.dY
        Get
            Return cy
        End Get
        Set(ByVal Value As Single)
            cy = Value
        End Set
    End Property
    Public Property Boundary() As RectangleF Implements IWallmove.Boundary
        Get
            Return bound
        End Get
        Set(ByVal Value As RectangleF)
            bound = Value
        End Set
    End Property
    Public Sub New(ByVal R As RectangleF, ByVal dX As Integer, ByVal dY As Integer, ByVal Boundary As RectangleF)
        Me.rect = R
        Me.cx = dX
        Me.cy = dY
        Me.bound = Boundary
    End Sub
    Public Sub Offset(ByVal dX As Integer, ByVal dY As Integer) Implements IWallmove.Offset
        rect.Offset(dX, dY)
        bound.Offset(dX, dY)
    End Sub
    Public Sub MoveWall() Implements IWallmove.MoveWall
        'This subroutine coordinates the in-game functionality of the moving wall with the rest of the game.
        'Just add the collision checks against rect.
        'rect.Offset(cx, cy) 'Move the wall.
        'If rect.Right > bound.Right OrElse rect.Left < bound.Left Then
        '    'When the wall hits the edge of our boundary, then it must begin moving in the other direction.
        '    cx = -cx
        'End If
        'If rect.Bottom > bound.Bottom OrElse rect.Top < bound.Top Then
        '    'This is for vertical movement direction switching.
        '    cy = -cy
        'End If
    End Sub
    Public Sub MoveHorizontal() Implements IWallmove.MoveHorizontal
        'This subroutine does just like the Movewall does, but only in the horizontal, with cx, left and right.
        If cx <> 0 Then
            rect.Offset(cx, 0) 'Move.
            If rect.Right >= bound.Right OrElse rect.Left <= bound.Left Then
                cy = cx
                cx = 0
            End If
        End If
    End Sub
    Public Sub MoveVertical() Implements IWallmove.MoveVertical
        'Same as MoveWall, but only in the vertical, with cy, top and bottom.
        If cy <> 0 Then
            rect.Offset(0, cy) 'Move.
            If rect.Bottom >= bound.Bottom OrElse rect.Top <= bound.Top Then
                'This is for vertical movement direction switching.
                cx = -cy
                cy = 0

            End If
        End If
    End Sub
End Structure
Public Structure MovingWallBoxCCW
    Implements IWallmove
    Private rect As RectangleF  'The location of the wall.
    Private cx As Single       'How much the wall moves in the horizontal.
    Private cy As Single       'How much the wall moves in the vertical.
    Private bound As RectangleF  'The wall cannot move outside of this square.

    Public Property Loc() As RectangleF Implements IWallmove.Loc
        Get
            Return rect
        End Get
        Set(ByVal Value As RectangleF)
            rect = Value
        End Set
    End Property
    Public Property dX() As Single Implements IWallmove.dX
        Get
            Return cx
        End Get
        Set(ByVal Value As Single)
            cx = Value
        End Set
    End Property
    Public Property dY() As Single Implements IWallmove.dY
        Get
            Return cy
        End Get
        Set(ByVal Value As Single)
            cy = Value
        End Set
    End Property
    Public Property Boundary() As RectangleF Implements IWallmove.Boundary
        Get
            Return bound
        End Get
        Set(ByVal Value As RectangleF)
            bound = Value
        End Set
    End Property
    Public Sub New(ByVal R As RectangleF, ByVal dX As Integer, ByVal dY As Integer, ByVal Boundary As RectangleF)
        Me.rect = R
        Me.cx = dX
        Me.cy = dY
        Me.bound = Boundary
    End Sub
    Public Sub Offset(ByVal dX As Integer, ByVal dY As Integer) Implements IWallmove.Offset
        rect.Offset(dX, dY)
        bound.Offset(dX, dY)
    End Sub
    Public Sub MoveWall() Implements IWallmove.MoveWall
        'This subroutine coordinates the in-game functionality of the moving wall with the rest of the game.
        'Just add the collision checks against rect.
        'rect.Offset(cx, cy) 'Move the wall.
        'If rect.Right > bound.Right OrElse rect.Left < bound.Left Then
        '    'When the wall hits the edge of our boundary, then it must begin moving in the other direction.
        '    cx = -cx
        'End If
        'If rect.Bottom > bound.Bottom OrElse rect.Top < bound.Top Then
        '    'This is for vertical movement direction switching.
        '    cy = -cy
        'End If
    End Sub
    Public Sub MoveHorizontal() Implements IWallmove.MoveHorizontal
        'This subroutine does just like the Movewall does, but only in the horizontal, with cx, left and right.
        If cx <> 0 Then
            rect.Offset(cx, 0) 'Move.
            If rect.Right >= bound.Right OrElse rect.Left <= bound.Left Then
                cy = -cx
                cx = 0
            End If

        End If
    End Sub
    Public Sub MoveVertical() Implements IWallmove.MoveVertical
        'Same as MoveWall, but only in the vertical, with cy, top and bottom.
        If cy <> 0 Then
            rect.Offset(0, cy) 'Move.
            If rect.Bottom >= bound.Bottom OrElse rect.Top <= bound.Top Then
                'This is for vertical movement direction switching.
                cx = cy
                cy = 0
            End If
        End If
    End Sub
End Structure
Public Structure MovingWallCirc
    Implements IWallmove
    Private rect As RectangleF  'The location of the wall.
    Private cx As Single       'How much the wall moves in the horizontal.
    Private cy As Single        'How much the wall moves in the vertical.
    Private nextrect As RectangleF
    Private peri As Integer
    Private bound As RectangleF  'The wall may not move outside of this square - definitely not to the right or left.
    Private T As Integer
    Private ang As Single, angfreq As Single
    Const Pi As Single = 3.1415926535897931
    'frequency = 1 / Period
    'angfreq = 2 * Math.PI / Period
    'circumference = 2 * Math.PI * radius
    'speed = 2 * Math.PI * radius / Period

    Public Property Loc() As RectangleF Implements IWallmove.Loc
        Get
            Return rect
        End Get
        Set(ByVal Value As RectangleF)
            rect = Value
        End Set
    End Property
    Public Property dX() As Single Implements IWallmove.dX
        Get
            Return cx
        End Get
        Set(ByVal Value As Single)
            cx = Value
        End Set
    End Property
    Public Property dY() As Single Implements IWallmove.dY
        Get
            Return cy
        End Get
        Set(ByVal Value As Single)
            cy = Value
        End Set
    End Property
    Public Property Period() As Integer
        Get
            Return peri
        End Get
        Set(ByVal Value As Integer)
            peri = Value
            If peri = 0 Then
                peri = 1
            End If
            angfreq = 2.0F * Pi / peri
            ang = 0.0F
            UpdatePosVel()
        End Set
    End Property
    Public ReadOnly Property Radius() As Single
        Get
            Return (bound.Width - rect.Width) / 2.0F
        End Get
    End Property
    Public Property Angle() As Single
        Get
            Return ang
        End Get
        Set(ByVal Value As Single)
            ang = Value
        End Set
    End Property
    Public Property Boundary() As RectangleF Implements IWallmove.Boundary
        Get
            Return bound
        End Get
        Set(ByVal Value As RectangleF)
            bound = Value
        End Set
    End Property
    Public Sub New(ByVal R As RectangleF, ByVal Period As Integer, ByVal Angle As Single, ByVal Boundary As RectangleF)
        Me.rect = R
        Me.cx = dX
        Me.cy = dY
        Me.peri = Period
        Me.ang = Angle
        Me.bound = Boundary
        If peri = 0 Then
            peri = 1
        End If
        angfreq = 2.0F * Pi / peri
        'ang = 0.0F
        UpdatePosVel()
    End Sub
    Public Sub Offset(ByVal dX As Integer, ByVal dY As Integer) Implements IWallmove.Offset
        rect.Offset(dX, dY)
        bound.Offset(dX, dY)
    End Sub
    Public Sub MoveWall() Implements IWallmove.MoveWall
        'This subroutine coordinates the in-game functionality of the moving wall with the rest of the game.
        'Just add the collision checks against rect.
        'rect.Offset(cx, cy) 'Move the wall.
        'If rect.Right > bound.Right OrElse rect.Left < bound.Left Then
        '    'When the wall hits the edge of our boundary, then it must begin moving in the other direction.
        '    cx = -cx
        'End If
        'If rect.Bottom > bound.Bottom OrElse rect.Top < bound.Top Then
        '    'This is for vertical movement direction switching.
        '    cy = -cy
        'End If
    End Sub
    Public Sub MoveHorizontal() Implements IWallmove.MoveHorizontal
        'This subroutine does just like the Movewall does, but only in the horizontal, with cx, left and right.
        ang += angfreq
        rect.X = nextrect.X
        nextrect.X = Convert.ToInt32(bound.Left + (bound.Width - rect.Width) * (1.0R + Math.Cos(ang + angfreq)) / 2)
        cx = nextrect.X - rect.X

    End Sub
    Public Sub MoveVertical() Implements IWallmove.MoveVertical
        'Same as MoveWall, but only in the vertical, with cy, top and bottom.
        'If cy <> 0 Then
        rect.Y = nextrect.Y
        nextrect.Y = Convert.ToInt32(bound.Top + (bound.Width - rect.Width) * (1.0R + Math.Sin(ang + angfreq)) / 2)
        cy = nextrect.Y - rect.Y
    End Sub
    Private Sub UpdatePosVel()
        rect.X = Convert.ToInt32(bound.Left + (bound.Width - rect.Width) * (1.0R + Math.Cos(ang)) / 2)
        rect.Y = Convert.ToInt32(bound.Top + (bound.Width - rect.Width) * (1.0R + Math.Sin(ang + angfreq)) / 2)
        nextrect.X = Convert.ToInt32(bound.Left + (bound.Width - rect.Width) * (1.0R + Math.Cos(ang)) / 2)
        nextrect.Y = Convert.ToInt32(bound.Top + (bound.Width - rect.Width) * (1.0R + Math.Sin(ang + angfreq)) / 2)
        cx = nextrect.X - rect.X
        cy = nextrect.Y - rect.Y
    End Sub
    Private Sub ToNextPosVel()
    End Sub
End Structure
Public Structure Piston
    Implements IWallmove
    Private rect As RectangleF  'The location of the wall.
    Private cx As Single       'How much the wall moves in the horizontal.
    Private cy As Single       'How much the wall moves in the vertical.
    Private bound As RectangleF  'The wall cannot move outside of this square.
    Private up As Boolean
    Private downspeed As Integer
    Public Property Loc() As RectangleF Implements IWallmove.Loc
        Get
            Return rect
        End Get
        Set(ByVal Value As RectangleF)
            rect = Value
        End Set
    End Property
    Public Property dX() As Single Implements IWallmove.dX
        Get
            Return cx
        End Get
        Set(ByVal Value As Single)
            cx = Value
        End Set
    End Property
    Public Property dY() As Single Implements IWallmove.dY
        Get
            Return cy
        End Get
        Set(ByVal Value As Single)
            cy = Value
        End Set
    End Property
    Public Property DownSpd() As Integer
        Get
            Return downspeed
        End Get
        Set(ByVal Value As Integer)
            downspeed = Value
            cy = downspeed
            up = False
        End Set
    End Property
    Public Property Boundary() As RectangleF Implements IWallmove.Boundary
        Get
            Return bound
        End Get
        Set(ByVal Value As RectangleF)
            bound = Value
        End Set
    End Property
    Public Sub New(ByVal R As RectangleF, ByVal Downspd As Integer, ByVal Boundary As RectangleF)
        Me.rect = R
        Me.downspeed = Downspd
        Me.bound = Boundary
        cy = downspeed
        cx = 0
        up = False

    End Sub
    Public Sub Offset(ByVal dX As Integer, ByVal dY As Integer) Implements IWallmove.Offset
        rect.Offset(dX, dY)
        bound.Offset(dX, dY)
    End Sub
    Public Sub MoveWall() Implements IWallmove.MoveWall
        'This subroutine coordinates the in-game functionality of the moving wall with the rest of the game.
        'Just add the collision checks against rect.
        rect.Offset(cx, cy) 'Move the wall.
        If rect.Right > bound.Right OrElse rect.Left < bound.Left Then
            'When the wall hits the edge of our boundary, then it must begin moving in the other direction.
            cx = -cx
        End If
        If rect.Bottom > bound.Bottom OrElse rect.Top < bound.Top Then
            'This is for vertical movement direction switching.
            cy = -cy
        End If
    End Sub
    Public Sub MoveHorizontal() Implements IWallmove.MoveHorizontal
        ''This subroutine does just like the Movewall does, but only in the horizontal, with cx, left and right.
        'rect.Offset(cx, 0) 'Move.
        'If rect.Right >= bound.Right OrElse rect.Left <= bound.Left Then
        '    'When the wall hits the edge of our boundary, then it must begin moving in the other direction.
        '    cx = -cx
        'End If
    End Sub
    Public Sub MoveVertical() Implements IWallmove.MoveVertical
        'Same as MoveWall, but only in the vertical, with cy, top and bottom.
        rect.Offset(0, cy) 'Move.
        If up Then
            cy -= 1  'Increase vertical speed.
            If rect.Top <= bound.Top Then
                cy = downspeed
                up = False
            End If

        ElseIf rect.Bottom >= bound.Bottom Then
            'This is for vertical movement direction switching.
            up = True
            cy = 0
        End If

    End Sub
End Structure
Public Structure OpenerSwitch
    Public Switch As RectangleF
    Public Gate As RectangleF

    Public Data As Single
    Public OpenAt As Single
    Public OpenAbove As Boolean

    Private BottomPoint As Single
    Private FullOpenAt As Single

    Public Sub New(ByVal SX As Integer, ByVal SY As Integer, ByVal SW As Integer, ByVal SH As Integer, ByVal GX As Integer, ByVal GY As Integer, ByVal GW As Integer, ByVal GH As Integer, ByVal CritData As Integer, ByVal OpAt As Integer, ByVal WorksAbove As Boolean)
        Switch = New RectangleF(SX, SY, SW, SH)
        Gate = New RectangleF(GX, GY, GW, GH)
        Data = CritData
        OpenAt = OpAt
        OpenAbove = WorksAbove
        BottomPoint = Gate.Height
        FullOpenAt = OpenAt + 100
    End Sub

    Public Sub Offset(ByVal dx As Integer, ByVal dy As Integer)
        Switch.Offset(dx, dy)
        Gate.Offset(dx, dy)
    End Sub

    Public Sub Decay(ByVal X As Single)
        If Data > 100 Then
            Data -= X
            If Data < FullOpenAt AndAlso Data >= OpenAt Then
                Gate.Height = (FullOpenAt - Data) * BottomPoint / 100
            ElseIf Data < OpenAt Then
                Gate.Height = BottomPoint
            ElseIf Data > FullOpenAt Then
                Gate.Height = 0
            End If
        End If
    End Sub

    Public Sub PowerUp(ByVal X As Single)
        If Data < 500 Then
            Data += X
            If Data > 500 Then
                Data = 500
            End If
            If Data >= OpenAt AndAlso Data < FullOpenAt Then
                Gate.Height = (FullOpenAt - Data) * BottomPoint / 100
            ElseIf Data < OpenAt Then
                Gate.Height = BottomPoint
            ElseIf Data > FullOpenAt Then
                Gate.Height = 1
            End If
        End If

    End Sub

    Public Sub Setting(ByVal X As Single)
        Data = X
        If Data >= OpenAt AndAlso Data < FullOpenAt Then
            Gate.Height = (FullOpenAt - Data) * BottomPoint / 100
        ElseIf Data < OpenAt Then
            Gate.Height = BottomPoint
        ElseIf Data > FullOpenAt Then
            Gate.Height = 1
        End If
    End Sub
End Structure
Public Structure AutoMessage
    Public MessageID As Integer   'The message ID that leads to the message that will be displayed - see Message subroutine.
    Public X As Integer    'X value that displays the message

    Public Sub New(ByVal X As Integer, ByVal ID As Integer)
        Me.X = X
        Me.MessageID = ID
    End Sub
End Structure


Public Interface IWallmove
    Property Loc() As RectangleF
    Property dX() As Single
    Property dY() As Single
    Property Boundary() As RectangleF

    Sub Offset(ByVal dx As Integer, ByVal dy As Integer)
    Sub MoveWall()
    Sub MoveHorizontal()
    Sub MoveVertical()
End Interface

Public Enum SuiteReturnType
    NoReturn = 0
    Landing
    HeadBanger
    Crush
    AdjustSlope
    Wallkick
    SlopeClimbing
End Enum

Public Structure AutoSuiteReturn
    Public RetState As SuiteReturnType
    Public IncValue As RectangleF
    Public TrpValue As StandTrapezoid
    Public Extra As Single
    Public ExtraTwo As Single

    Public Sub New(ByVal S As SuiteReturnType, ByVal Vl As RectangleF)
        Me.RetState = S
        Me.IncValue = Vl
    End Sub
    Public Sub New(ByVal S As SuiteReturnType, ByVal Vl As RectangleF, ByVal Ex As Single)
        Me.RetState = S
        Me.IncValue = Vl
        Me.Extra = Ex
    End Sub
    Public Sub New(ByVal S As SuiteReturnType, ByVal Vl As RectangleF, ByVal Ex As Single, ByVal Ex2 As Single)
        Me.RetState = S
        Me.IncValue = Vl
        Me.Extra = Ex
        Me.ExtraTwo = Ex2
    End Sub
    Public Sub New(ByVal S As SuiteReturnType, ByVal TpVl As StandTrapezoid)
        Me.RetState = S
        Me.TrpValue = TpVl
    End Sub
    Public Sub New(ByVal S As SuiteReturnType, ByVal TpVl As StandTrapezoid, ByVal Ex As Single)
        Me.RetState = S
        Me.TrpValue = TpVl
        Me.Extra = Ex
    End Sub
End Structure

'To add a new world feature:
'Add it to the LevelSuite
'Load it in LoadLevel using RECTANGLE Fs!
'Add it to KUArea structure (if it is static... dynamic, just add it to LevelSuite)
'Determine the number of vertices needed to draw this and make a constant
'Count the new object in the PopulateKUs() if static
'Adjust vertex count in NewLevelInit() on applicable list
'Make texture for the object in TextureMesh setup (don't forget to increment the texture array size).
'Draw the object in Render() or RenderAllStatics(), as applicable
'Setup object in SetVertexBufferData() or SetDynamicBufferData()
'Allow interface with object via CharacterMovement subs (Left/Right) and IsJumping block.


Public Class LevelSuite
    Public Const AUTOMESSNUM As Integer = 4

    Public Platforms As ArrayList
    'This will store the Platforms that are in this level.
    Public Walls As ArrayList
    'Stores the Walls that are in this level.
    Public Coins As ArrayList
    'Holds all of the coin locations.
    Public Lavas As ArrayList
    'Holds all of the lava pools for this level.
    Public Floaters As ArrayList
    'Holds all of the Floaters for this level.
    Public Jumpboards As ArrayList
    'Holds all of the jump boards in the level.
    Public RTDs As ArrayList
    Public Pneumos As ArrayList
    Public Mobilewalls As ArrayList
    'Holds all of the moving Walls in the level.
    Public Ices As ArrayList
    'Holds all of the ice Platforms in the level.
    Public Waters As ArrayList
    'Holds all of the bodies of water in the level.
    Public Glasses As ArrayList
    Public Backdrops As ArrayList
    'Bunch of Backdrops.
    Public Verticals As ArrayList
    Public Bulbs As ArrayList
    Public TrapPlatforms As ArrayList
    Public Slopedwalls As ArrayList
    Public Extrawalls As ArrayList
    Public Crates As ArrayList
    Public Backcrates As ArrayList
    Public ForeFences As ArrayList
    Public AftFences As ArrayList
    Public FieldFences As ArrayList
    Public EyeBeams As ArrayList
    Public CrossStays As ArrayList
    Public Conveyors As ArrayList
    Public Ladders As ArrayList
    Public Ropes As ArrayList
    Public Groundwalls As ArrayList
    Public AutoMessages(4) As AutoMessage

    Public Things As ArrayList
    Public Videoscreens As ArrayList

    'Public Bullets As ArrayList

    Public MovingWallCount As Integer
    Public MovingWallBoxCCWcount As Integer
    Public MovingWallBoxCWcount As Integer
    Public MovingWallCircCount As Integer
    Public PistonCount As Integer

    Public PlantCount As Integer
    Public StatueCount As Integer
    Public PictureCount As Integer
    Public VideoscreenCount As Integer
    Public AshtrayCount As Integer
    Public PhoneBoothCount As Integer
    Public SignCount As Integer
    Public BenchCount As Integer

    Private ThingsAreSorted As Boolean

    Public Monsters As ArrayList
    'Holds a bunch of monsters that should all implement the IMonster interface -
    'otherwise, they are in violation of contract!
    Public SonicEvents As ArrayList
    Public Explosions As ArrayList
    Public Negations As ArrayList
    Public Shards As ArrayList

    Public Shared Function PlatformJump(ByVal Pforms As ArrayList, ByVal Source As RectangleF, ByVal VertSpd As Single, ByVal Buffer As Single) As AutoSuiteReturn
        'Q, player.loc, Player.Veloc, GRAVITYTH
        Dim Platform As RectangleF
        Dim Ret As AutoSuiteReturn

        For Each Platform In Pforms

            If VertSpd > 0 AndAlso Source.Bottom >= Platform.Top AndAlso Source.Bottom - VertSpd - Buffer <= Platform.Top Then
                If Source.Left < Platform.Right AndAlso Source.Right > Platform.Left Then
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, Platform)
                End If
            End If
        Next

        Return Ret
    End Function
    Public Shared Function WallJump(ByVal Walls As ArrayList, ByVal Source As RectangleF, ByVal VertSpd As Single, ByVal AlreadyDown As Boolean) As AutoSuiteReturn
        'Q, Player.loc Player.Veloc, OnPlatform
        Dim Wall As RectangleF
        Dim Ret As AutoSuiteReturn

        For Each Wall In Walls
            If Source.IntersectsWith(Wall) Then
                If AlreadyDown Then  'If the player has landed on a platform.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Crush, Wall)

                ElseIf VertSpd > 0 Then 'If player is falling, then the player will land on the wall.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, Wall)

                Else 'The player has jumped up into the wall.
                    Ret = New AutoSuiteReturn(SuiteReturnType.HeadBanger, Wall)

                End If
            End If

        Next

        Return Ret
    End Function
    Public Shared Function JumpBoxJump(ByVal Ws As ArrayList, ByVal Source As RectangleF, ByVal VertSpd As Single, ByVal AlreadyDown As Boolean) As AutoSuiteReturn
        'Q, Player.loc Player.Veloc, OnPlatform
        Dim JB As JumpingBoard
        Dim Ret As AutoSuiteReturn

        For Each JB In Ws
            If Source.IntersectsWith(JB.Loc) Then
                If AlreadyDown Then  'If the player has landed on a platform.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Crush, JB.Loc, JB.ReboundVeloc)

                ElseIf VertSpd > 0 Then 'If player is falling, then the player will land on the wall.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, JB.Loc, JB.ReboundVeloc)

                Else 'The player has jumped up into the wall.
                    Ret = New AutoSuiteReturn(SuiteReturnType.HeadBanger, JB.Loc, JB.ReboundVeloc)

                End If
            End If

        Next

        Return Ret
    End Function
    Public Shared Function MobileWallJump(ByVal Q As LevelSuite, ByVal Source As RectangleF, ByVal VertSpd As Single, ByVal AlreadyDown As Boolean) As AutoSuiteReturn
        Dim MobileWall As IWallmove
        Dim Ret As AutoSuiteReturn


        For Each MobileWall In Q.Mobilewalls
            If Source.IntersectsWith(MobileWall.Loc) Then
                If AlreadyDown Then  'If the player has landed on a platform.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Crush, MobileWall.Loc, MobileWall.dY)

                ElseIf VertSpd >= 0 Then
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, MobileWall.Loc, Q.Mobilewalls.IndexOf(MobileWall))

                Else
                    Ret = New AutoSuiteReturn(SuiteReturnType.HeadBanger, MobileWall.Loc, MobileWall.dY)

                End If
            End If
        Next

        Return Ret
    End Function
    Public Shared Function OpenSwitchJump(ByVal OSS As ArrayList, ByVal source As RectangleF, ByVal vertspd As Single, ByVal alreadydown As Boolean) As AutoSuiteReturn
        Dim OpSw As OpenerSwitch
        Dim Ret As AutoSuiteReturn
        Dim LV As Integer = 0

        For Each OpSw In OSS
            If source.IntersectsWith(OpSw.Switch) Then
                If alreadydown Then  'If the player has landed on a platform.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Crush, OpSw.Switch)

                ElseIf vertspd > 0 Then
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, OpSw.Switch, LV, vertspd)

                Else
                    Ret = New AutoSuiteReturn(SuiteReturnType.HeadBanger, OpSw.Switch)

                End If
            End If
            If source.IntersectsWith(OpSw.Gate) Then
                If alreadydown Then  'If the player has landed on a platform.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Crush, OpSw.Gate)

                ElseIf vertspd > 0 Then
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, OpSw.Gate, -1)

                Else
                    Ret = New AutoSuiteReturn(SuiteReturnType.HeadBanger, OpSw.Gate)

                End If
            End If
            LV += 1
        Next

        Return Ret

    End Function
    Public Shared Function ConveyorJump(ByVal Q As LevelSuite, ByVal Source As RectangleF, ByVal VertSpd As Single, ByVal AlreadyDown As Boolean) As AutoSuiteReturn
        Dim Datablk As DataBlock
        Dim Ret As AutoSuiteReturn


        For Each Datablk In Q.Conveyors
            If Source.IntersectsWith(Datablk.Loc) Then
                If AlreadyDown Then  'If the player has landed on a platform.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Crush, Datablk.Loc)

                ElseIf VertSpd > 0 Then
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, Datablk.Loc, Q.Conveyors.IndexOf(Datablk))

                Else
                    Ret = New AutoSuiteReturn(SuiteReturnType.HeadBanger, Datablk.Loc)

                End If
            End If
        Next

        Return Ret
    End Function
    Public Shared Function TrapPlatformJump(ByVal TPforms As ArrayList, ByVal Source As RectangleF, ByVal VertSpd As Single, ByVal Buffer As Single) As AutoSuiteReturn
        'Q, player.loc, Player.Veloc, GRAVITYTH
        Dim Trap As StandTrapezoid
        Dim Sh As Single
        Dim Ret As AutoSuiteReturn

        For Each Trap In TPforms
            Sh = Trap.HeightForRect(Source)

            If VertSpd > 0 AndAlso Source.Bottom >= Sh AndAlso Source.Bottom - VertSpd - Buffer <= Sh Then
                If Source.Left < Trap.R.Right AndAlso Source.Right > Trap.R.Left Then
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, Trap, Sh)

                End If
            End If
        Next

        Return Ret
    End Function
    Public Shared Function SlopeWallJump(ByVal SWs As ArrayList, ByVal Source As RectangleF, ByVal VertSpd As Single, ByVal AlreadyDown As Boolean) As AutoSuiteReturn
        Dim Slopedwall As StandTrapezoid
        Dim Sh As Single
        Dim Ret As AutoSuiteReturn
        For Each Slopedwall In SWs
            If Slopedwall.IntersectsWith(Source) Then
                If AlreadyDown Then  'If the player has landed on a platform.
                    Ret = New AutoSuiteReturn(SuiteReturnType.Crush, Slopedwall)
                ElseIf VertSpd > 0 Then
                    Sh = Slopedwall.HeightForRect(Source)
                    Ret = New AutoSuiteReturn(SuiteReturnType.Landing, Slopedwall, Sh)
                Else
                    Ret = New AutoSuiteReturn(SuiteReturnType.HeadBanger, Slopedwall)
                    'The player has jumped up into the wall.
                End If
            End If
        Next

        Return Ret
    End Function

    Public Sub ArrangeSetThings()

        Dim OLV, ILV As Integer
        Dim OutOb, InOb, Temp As DataBlock
        Dim RTE As Boolean

        If Not ThingsAreSorted Then
            'Videoscreens = New ArrayList
            PlantCount = 0
            StatueCount = 0
            PictureCount = 0
            VideoscreenCount = 0
            AshtrayCount = 0
            PhoneBoothCount = 0
            SignCount = 0
            BenchCount = 0


            For OLV = 0 To Things.Count - 1
                OutOb = DirectCast(Things(OLV), DataBlock)
                'This is the outer loop variable's object.
                For ILV = OLV + 1 To Things.Count - 1
                    InOb = DirectCast(Things(ILV), DataBlock)

                    If OutOb.Data > InOb.Data Then
                        'These are in the wrong order - switch.
                        Temp = OutOb
                        OutOb = InOb
                        InOb = Temp
                        Things(OLV) = OutOb
                        Things(ILV) = InOb
                    End If
                Next

                UpdateCounter(OutOb.Data)
                If OutOb.Data >= 7 AndAlso OutOb.Data <= 10 Then
                    'Videoscreens.Add(OutOb)
                End If
            Next
            ThingsAreSorted = True
        Else
            If Things.Count >= 1 Then  ' 2 >= 1
                OutOb = DirectCast(Things(0), DataBlock)   '[12]
                ILV = 1                                         'i = 1
                Do Until ILV >= Things.Count OrElse RTE         'until i = 2 or [12] <= [  ]
                    InOb = DirectCast(Things(ILV), DataBlock)   '[11]
                    RTE = (OutOb.Data <= InOb.Data)             'is 12 <= 11 no
                    ILV += 1                                    'i = 2
                Loop
                If RTE Then                                     'no 
                    Things.Insert(ILV - 1, OutOb)
                    Things.RemoveAt(0)
                Else
                    Things.Add(OutOb)
                    Things.RemoveAt(0)
                End If
                UpdateCounter(OutOb.Data)
            End If
        End If

    End Sub
    Private Sub UpdateCounter(ByVal Data As Single)
        Select Case Data
            Case 0
                PlantCount += 1
            Case 1, 2
                StatueCount += 1
            Case 3 To 6
                PictureCount += 1
            Case 7 To 10
                VideoscreenCount += 1
            Case 11
                AshtrayCount += 1
            Case 12
                PhoneBoothCount += 1
            Case 13 To 20
                SignCount += 1
            Case 21
                BenchCount += 1
        End Select
    End Sub
    Public Function GetNextThingIndex(ByVal Idx As Integer) As Integer
        Select Case Idx
            Case 0, 11, 12, 21
                Return Idx

            Case 1, 2
                Return 3 - Idx

            Case 3, 4, 5
                Return Idx + 1
            Case 6
                Return 3

            Case 7, 8, 9
                Return Idx + 1
            Case 10
                Return 7

            Case 13, 14, 15, 16, 17, 18, 19
                Return Idx + 1
            Case 20
                Return 13

        End Select
    End Function

    Public Sub ArrangeSetMobilewalls()
        Dim X, Y, Temp As IWallmove
        Dim NX, NY As Integer
        Dim LX, LY As Integer
        MovingWallCount = 0
        MovingWallBoxCCWcount = 0
        MovingWallBoxCWcount = 0
        MovingWallCircCount = 0
        PistonCount = 0
        For LX = 0 To Mobilewalls.Count - 1
            X = DirectCast(Mobilewalls(LX), IWallmove)
            If TypeOf X Is MovingWall Then
                NX = 0
            ElseIf TypeOf X Is MovingWallBoxCCW Then
                NX = 1
            ElseIf TypeOf X Is MovingWallBoxCW Then
                NX = 2
            ElseIf TypeOf X Is MovingWallCirc Then
                NX = 3
            ElseIf TypeOf X Is Piston Then
                NX = 4
            End If
            For LY = LX + 1 To Mobilewalls.Count - 1
                Y = DirectCast(Mobilewalls(LY), IWallmove)
                If TypeOf Y Is MovingWall Then
                    NY = 0
                ElseIf TypeOf Y Is MovingWallBoxCCW Then
                    NY = 1
                ElseIf TypeOf Y Is MovingWallBoxCW Then
                    NY = 2
                ElseIf TypeOf Y Is MovingWallCirc Then
                    NY = 3
                ElseIf TypeOf Y Is Piston Then
                    NY = 4
                End If
                If NX > NY Then
                    Temp = X
                    X = Y
                    Y = Temp
                    NX = NY
                    Mobilewalls(LX) = X
                    Mobilewalls(LY) = Y
                End If
            Next
            If NX = 0 Then
                MovingWallCount += 1
            ElseIf NX = 1 Then
                MovingWallBoxCCWcount += 1
            ElseIf NX = 2 Then
                MovingWallBoxCWcount += 1
            ElseIf NX = 3 Then
                MovingWallCircCount += 1
            ElseIf NX = 4 Then
                PistonCount += 1
            End If
        Next
    End Sub

    Public Sub AddExplosion(ByVal Xo As Single, ByVal Yo As Single)
        Dim e As Explosion
        e = New Explosion(Xo, Yo)
        Me.Monsters.Add(e)
        Me.Explosions.Add(e)
        Me.SonicEvents.Add(SonicEventType.Explosion)
    End Sub
    Public Sub AddNegation(ByVal Xo As Single, ByVal Yo As Single)
        Dim n As Negation
        n = New Negation(Xo, Yo)
        Me.Monsters.Add(n)
        Me.Negations.Add(n)
        Me.SonicEvents.Add(SonicEventType.Negate)
    End Sub
    Public Sub AddShard(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single)
        Dim s As Shard
        s = New Shard(Xo, Yo, Vxo)
        Me.Monsters.Add(s)
        Me.Shards.Add(s)
    End Sub

    Public Sub PopulateKUs(ByRef KUs() As KilounitArea)
        Dim Thg As DataBlock, Idx As Integer
        For Each Thg In Things
            Idx = Convert.ToInt32(Thg.Loc.Left) \ 1000
            Select Case Thg.Data
                Case 0
                    KUs(Idx).PlantCount += 1
                Case 1, 2
                    KUs(Idx).StatueCount += 1
                Case 3 To 6
                    KUs(Idx).PictureCount += 1
                Case 7 To 10
                    KUs(Idx).VideoscreenCount += 1
                Case 11
                    KUs(Idx).AshtrayCount += 1
                Case 12
                    KUs(Idx).PhoneBoothCount += 1
                Case 13 To 20
                    KUs(Idx).SignCount += 1
                Case 21
                    KUs(Idx).BenchCount += 1
            End Select
        Next
    End Sub
End Class

Public Structure KilounitArea
    Public PlatformCount As Integer
    Public WallCount As Integer
    Public LavaCount As Integer
    Public JumpCount As Integer
    Public IceCount As Integer
    Public BackdropCount As Integer
    Public TrapPlatformCount As Integer
    Public SlopedwallCount As Integer
    Public ColumnCount As Integer
    Public ExtrawallCount As Integer
    Public CrateCount As Integer
    Public BackcrateCount As Integer
    Public LadderCount As Integer
    Public RopeCount As Integer
    Public GroundCount As Integer
    Public BulbCount As Integer
    Public ForeFenceCount As Integer
    Public AftFenceCount As Integer
    Public FieldFenceCount As Integer
    Public EyeBeamCount As Integer
    Public CrossStayCount As Integer
    'Public KonveyCount As Integer

    Public PlantCount As Integer
    Public StatueCount As Integer
    Public PictureCount As Integer
    Public VideoscreenCount As Integer
    Public AshtrayCount As Integer
    Public PhoneBoothCount As Integer
    Public SignCount As Integer
    Public BenchCount As Integer
End Structure

'To add a new enemy:
'have the enemy class from the PFTour
'paste it into this project
'we'll fix it later.
'TextureMeshSetup - make a new bitmech texture  Bitmech = New Texture(...) {}
'load texture... check filename and color key.
'Monster class - comment the old draw (and intersectswith, save, and move) routine and
'reimplement the IMonster interface
'ID - Return ##
'The class should have constant(s) for height and width.
'and values for X, Y for position.  Z = 0.
'Convert the constructor parameters to appropriate singles - you'll get a blue error otherwise.
'Remove the constructor bitmap.
'Convert variables to singles.
'Remove bitmap variable.
'Make sure that constructor Move uses Singles and RectangleFs.
'The draw routine will take effort - start with existing one and modify it.
'SetUpBillboard should handle all of your needs and Render should draw it for you.

Public Enum DisplayMode
    IPIntroscrn
    OpenToMainscrn
    Mainscrn
    CloseMainscrn
    GameIntro
    Regular
    YouDied
    LevelIntro
    LoadScreen
    LevelTransfer
    PauseMenu
    SaveScreen
    WeaponSelect
    EndCredits
    OptionsMenu
    ControlsMenu
    VictoryCutscene
End Enum

Public Enum ShotTypes
    ShockWave = 0
    Aquabullet
    Napalm
    Rubberet
    Screwer
    ForceMod
    Corkwheel
    Stonebullet
    Plasma
    Pearl
    Homing
End Enum
Public Enum ShotPattern
    SingleShot = 0
    HurdledShot
    SpreadShot
    ParabolicShot
    VerticalShot
    WideShot
    CartridgeShot
    SplitterShot
    AlphaShot
    BackhandShot
    MeteorShot
    SprinkleShot
    BoomerangShot
    FivewayShot
    VolleyShot
    TetradShot
    AberrationShot
    SixteenShot
    HighstormShot
    ClusterShot
End Enum
'Must correspond to trx array at beginning of formD_load
Public Enum SonicEventType
    ShotHit
    ShotDefend
    ShotThru
    Explosion
    ShotRavage
    Landing
    Splash
    Shoot
    GateOpen
    BreakGlass
    HitGlass
    Negate
    Coinage
    HitOne
    HitTwo
End Enum

Public Class Form1
    Inherits System.Windows.Forms.Form

#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Form overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        '
        'Form1
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(544, 477)
        Me.StartPosition = FormStartPosition.CenterScreen
        Me.Name = "Form1"
        Me.Text = "Platform Tour"

    End Sub

#End Region

    'Ep 2: Vertex Buffers.
    'The SDK ties events to everything in a rather kooky way.
    'Player.Loc = New Rectangle(100, 0, CHARWIDTH, CHARHEIGHT)
    'Player.IsJumping = True
    'Player.Veloc = 0
    'Player.LeftEnd = 0 : Player.RightEnd = KUSize * 1000
    ''Set the current end points.


    'Platforms = New ArrayList
    ''Initialize/instantiate the platforms arraylist.
    'Platforms.Add(Rectangle.FromLTRB(50, 415, 160, LANDHEIGHT))
    'Platforms.Add(Rectangle.FromLTRB(100, 465, 185, LANDHEIGHT))
    'Platforms.Add(Rectangle.FromLTRB(150, 515, 210, LANDHEIGHT))
    'Platforms.Add(Rectangle.FromLTRB(1150, 515, 1210, LANDHEIGHT))
    'Platforms.Add(Rectangle.FromLTRB(2150, 515, 2210, LANDHEIGHT))
    ''Create three platforms.
    'Coins = New ArrayList

    'Coins.Add(New Rectangle(200, 530, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(100, 440, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(116, 440, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(132, 440, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(260, 355, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(230, 365, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(240, 510, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(20, 490, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(280, 400, COINSIZE, COINSIZE))
    'Coins.Add(New Rectangle(200, 498, COINSIZE, COINSIZE))
    ''Ten coins going into the coins arraylist.
    ''Coin
    'Walls = New ArrayList
    ''The list of walls is set up.
    'Walls.Add(Rectangle.FromLTRB(225, 415, 275, 465))
    'Walls.Add(Rectangle.FromLTRB(320, 480, 350, LANDHEIGHT))
    'Walls.Add(Rectangle.FromLTRB(350, 520, 380, LANDHEIGHT))
    ''And three walls are added.
    'Lavas = New ArrayList
    ''Create the arraylist.
    'Lavas.Add(Rectangle.FromLTRB(210, 520, 270, LANDHEIGHT))
    ''Initialize the lava location.
    'Floaters = New ArrayList
    'Floaters.Add(New LevFloater(New Rectangle(420, 400, FLOATERSIZE, FLOATERSIZE), 2, 2, 4, 4, PeriodicDesignation.Trapezoid, PeriodicDesignation.Hexagon))
    'Floaters.Add(New LevFloater(New Rectangle(550, 400, FLOATERSIZE, FLOATERSIZE), 5, 5, 16, 16, PeriodicDesignation.SineFxn, PeriodicDesignation.CosineFxn))
    ''Information about the floater.  The width and height are all determined by the floatersize constant.
    'Jumpboards = New ArrayList
    ''Initialize the jumping board arraylist.
    'Jumpboards.Add(New JumpingBoard(450, 524, JUMPBOARDSIZE, JUMPBOARDSIZE, -46))
    'Jumpboards.Add(New JumpingBoard(1650, 524, JUMPBOARDSIZE, JUMPBOARDSIZE, -42))
    ''Add this jumping board to the level.
    'MobileWalls = New ArrayList
    ''Set up the arraylist.
    'MobileWalls.Add(New MovingWall(New Rectangle(640, 510, 32, 32), 1, 0, New Rectangle(500, 0, 300, 600)))
    'MobileWalls.Add(New MovingWall(New Rectangle(800, 250, 32, 32), 0, 2, New Rectangle(800, 0, 100, 536)))
    ''Add a moving wall.
    ''Creates an ice.
    'Ices = New ArrayList
    'Ices.Add(New Rectangle(1500, 350, 400, 218))
    'Ices.Add(Rectangle.FromLTRB(1350, 150, 1520, LANDHEIGHT))
    'Ices.Add(Rectangle.FromLTRB(1620, 150, 1820, LANDHEIGHT))
    ''Create the ices arraylist and add our ice rectangle to it.            'Create the ices arraylist and add our ice rectangle to it.
    'Waters = New ArrayList
    ''Create the list of all waters.
    'Water = New Rectangle(2100, 250, 400, 300)
    'Waters.Add(Water)
    ''Add this new water to the list of all water.

#Region "DirectX/Interface Stuff"
    Dim WithEvents D9 As Device
    Dim VxB() As VertexBuffer   'Each element is 1000 units wide, containing all object that have a 
    'left property that fall in its jurisdiction.
    Dim Xlucent As VertexBuffer, DynaVxB As VertexBuffer
    Dim PsB As VertexBuffer
    Dim TextBfr As VertexBuffer, Backgrd As VertexBuffer
    Dim Billboard As VertexBuffer
    Dim SkyBuffer As VertexBuffer, SkyTx As Texture, BackTx As Texture, Exdo As Texture
    Dim IPPicture As Texture, IPBuffer As VertexBuffer
    Dim TitlePicture As Texture, TitleBuffer, OptionBuffer As VertexBuffer
    Dim BlockFont, Chopper As Texture, ChopBuffer As VertexBuffer
    '32 16 28
    Dim Vertex As CustomVertex.PositionNormalTextured, PCVertex As CustomVertex.PositionColored, TCVertex As CustomVertex.TransformedColoredTextured, PCTVertex As CustomVertex.PositionColoredTextured
    'Three dimensional plot - each vertex has a normal direction - texture coordinates to fit the texture this vertex holds.
    Dim Spots() As CustomVertex.PositionColoredTextured
    'For the coins, we need vertices.

    Dim Running As Boolean
    'Set to false to end the game loop.
    'Dim Paused As Boolean

    Dim PrT As PrimitiveType

    Dim Wallpaper, Luminous, LavaMtrl, WaterMtrl, GlasMtrl As Material

    Dim Tx(), PsTx(), Bitmech(), Fltr, Jumpbd, Mov As Texture
    'The single-texture that our project is using (soon, there will be more textures)

    Dim MM() As MeshPlus, Mx() As Matrix
    'Dim Column() As MeshPlus, ColumnMx() As Matrix

    Dim VertexCount As Integer
    Dim Camera, FocusPt, CamNormal As Vector3
    Dim AnglePl, AngleEv As Double
    'Dim LOSTravel As Boolean

    Dim Up, Lft, Rght, Dwn As Boolean

    Dim Ri As Random
    Dim Tick As Integer, FPS As Integer

    'Dim WithEvents DisplayFont As Direct3D.Font

    Dim CusText() As String, CusLines As Integer
    Dim DisplayActive As Boolean, DisplayCounter As Integer
    Dim AllFiles() As String, ViewFiles() As String, TopIdx As Integer
    Dim Textspots() As CustomVertex.TransformedColoredTextured

    Dim Bits As ArrayList
    Dim Bbx As Sonic3

    Const Pi As Single = 3.1415926535897931
    Const PiHalf As Single = Pi / 2
    Const PiThird As Single = Pi / 3
    Const PiFourth As Single = Pi / 4
    Const PiFifth As Single = Pi / 5
    Const PiSixth As Single = Pi / 6
    Const PiEighth As Single = Pi / 8
    Const PiTenth As Single = Pi / 10

    Const Sin60 As Single = 0.8660254F
    Const Cos30 As Single = Sin60
    Const Sin45 As Single = 0.707106769F
    Const Cos45 As Single = Sin45

    Const RootTwo As Single = 1.4142135623731
    Const RootThree As Single = 1.73205080756888

    '           Diagnostics.Debug.WriteLine(System.Runtime.InteropServices.Marshal.SizeOf(Vertex))  '32
    '           Diagnostics.Debug.WriteLine(System.Runtime.InteropServices.Marshal.SizeOf(TCVertex)) '28
    '           Diagnostics.Debug.WriteLine(System.Runtime.InteropServices.Marshal.SizeOf(PCVertex)) '16

    Const PNTSiez As Integer = 32
    Const TCTSize As Integer = 28
    Const PCTSize As Integer = 16
#End Region

#Region "Gameplay Variables"

    Dim SR As IO.StreamWriter

    Dim Player As Personnel
    'Holds all data for the player.
    Dim GoFast As Boolean
    'Is shift-key pressed?

    Dim Platform As RectangleF
    'A platform.
    'Dim Platforms As ArrayList
    'This will store the platforms that are in this level.
    Dim Coin As RectangleF
    'This holds the location of the coin.
    Dim Wall As RectangleF
    'Holds the location of the wall.
    'Dim Coins As ArrayList
    'Holds the list of coins for this level.
    'Dim Walls As ArrayList
    'The walls for this level.
    Dim Lava As RectangleF
    'A deadly rectangle of lava that will kill the character.
    'Dim Lavas As ArrayList
    'Holds all of the lava pools for this level.
    Dim Floater As LevFloater
    'This is the floater's location and movement information on the field.
    'Dim Floaters As ArrayList
    'Holds all of the floaters for this level.
    'Dim Jumpboards As ArrayList
    'Holds all of the jump boards in the level.
    Dim Jumpboard As JumpingBoard
    'A board that automatically boosts the player by giving him a large jumping speed.
    'Dim MobileWalls As ArrayList
    'Holds all of the moving walls in the level.
    Dim MobileWall As IWallmove
    'A moving wall structure variable declaration
    Dim Datablk As DataBlock
    'Conveyor.
    Dim OpSw As OpenerSwitch
    'Switch: RTD, Pneumo
    Dim Ladder As RectangleF
    'A ladder or rope to climb
    Dim MobileIndex As Integer
    'Index of the mobile wall that the player is on.
    Dim KonveyIndex As Integer
    'Index of the conveyor that the player is on.
    Dim RTDIndex, PNMIndex As Integer, PNMData As Single
    'Index of the RTD switch that the player is on.
    Dim KUSize As Integer
    'Number of kilounits in level.
    Dim KU() As KilounitArea

    Dim Upgrade As RectangleF

    'Dim Ices As ArrayList
    'Holds all of the ice platforms in the level.
    Dim Ice As RectangleF
    'Holds the location of the ice.
    'Dim Waters As ArrayList
    'Holds all of the bodies of water in the level.
    Dim Water As RectangleF
    'Holds the location of a body of water.

    Dim Backdrop As RectangleF
    'Holds rectangle that stores background walls.
    'Dim Backdrops As ArrayList
    'Bunch of backdrops.

    Dim TrapPlatform As StandTrapezoid
    'Dim TrapPlatforms As ArrayList

    Dim Slopedwall As StandTrapezoid
    'Dim Slopedwalls As ArrayList

    Dim Exitdoor As RectangleF  'Location of the exit.
    Dim Column As RectangleF

    Dim LevelMajor, LevelMinor As Integer

    Dim Q As LevelSuite

    Dim MyMode As DisplayMode
    Dim CtrTwo As Integer
    Dim Selexion As Integer

    Dim CurrentLevelFile As String
    Dim AutoSaveLevel As String
    Dim ActiveMessageIndex As Integer

    Dim CompletedLev() As Boolean
    Dim RowUnlocked() As Boolean
    Dim HasUpgrade() As Boolean
    Dim CoinCount() As Integer
    Dim MaxCoinCount() As Integer
    Dim BreakGlassIndex As Integer

    Dim Filename As String
    Dim Ext As String

#End Region

#Region "Gameplay Constants"
    Const MAPWIDTH As Integer = 800
    Const MAPHEIGHT As Integer = 600
    'I need to determine a default spot for my land.
    'It has to be between 0 and MAPHEIGHT, since that's the height of my backbuffer.
    'Preferably closer to 600 as more land equals less playing area.
    Const LANDHEIGHT As Integer = 568
    '600 - 32 = 568... I like 32. 
    'Constants are good for you.

    Const CHARWIDTH As Integer = 16
    Const CHARHEIGHT As Integer = 32
    'These define the width of the character in the file.
    Const HALFHEIGHT As Integer = CHARHEIGHT >> 1
    'Half of the character's height.
    Const HALFWIDTH As Integer = CHARWIDTH >> 1
    Const MOVESPEED As Single = 2
    'How much I move every tick 
    Const FASTMOVESPEED As Single = MOVESPEED * 1.5F
    'What if I'm running?
    Const SUBMOVESPEED As Single = MOVESPEED * 0.5F
    'What if I'm underwater?
    Const SUBFASTMOVESPEED As Single = MOVESPEED * 0.75F
    'What if I'm running underwater?
    Const ICEUPSPEED As Single = MOVESPEED * 0.1F

    Const WALKINGFRAMECOUNT As Integer = 4
    Const WALKINGFRAMECOUNTEX As Integer = WALKINGFRAMECOUNT - 1
    Const EFFWALKINGFRAMECOUNTEX As Integer = WALKINGFRAMECOUNT * FRAMETIME - 1
    'How many frames are in each walking cycle.  EX means -1
    Const FRAMEDIVCONST As Single = 32.0F
    Const FRAMETIME As Integer = 2
    Const REALFRAMEDIVCONST As Single = FRAMEDIVCONST * FRAMETIME

    Const BACKGROUNDDEPTH As Single = 600.0F

    Const CAMERAHIGH As Single = 20.0F
    Const CAMERADOWN As Single = CAMERAHIGH   '- 50f
    Const CAMERADEPTH As Single = -300.0F

    'Const CAMERAHIGH As Single = 80.0F
    'Const CAMERADOWN As Single = CAMERAHIGH   '- 50f
    'Const CAMERADEPTH As Single = -300.0F

    'Const CAMERAHIGH As Single = 75.0F
    'Const CAMERADOWN As Single = CAMERAHIGH - 25.0F
    'Const CAMERADEPTH As Single = -200.0F
    Const CAMERADISTANCECONSTANT As Single = (CAMERAHIGH * CAMERAHIGH + CAMERADEPTH * CAMERADEPTH) ^ 0.5

    Const FIELDTILEDEPTH As Integer = 6
    Const FIELDTILEWIDTH As Integer = 6
    Const FIELDTILEAREA As Integer = FIELDTILEDEPTH * FIELDTILEWIDTH
    Const FIELDTILEVTXCOUNT As Integer = 6        '3 vertices per triangle, 2 triangles per square.
    Const FIELDVERTEXCOUNT As Integer = FIELDTILEVTXCOUNT * FIELDTILEAREA
    'number of vertices needed by the field.

    Const SBARINITIALVELOCITY As Single = -8.38
    'The initial velocity for the player's jump using space bar.

    Const DOUBLEJUMPVELOCITY As Single = SBARINITIALVELOCITY * 1.42F
    'To jump twice the height with a given gravity, use the formula: dD = Vo * T + 0.5 * A * T * T. with dV = A * T
    'dV = -Vo -> A=-Vo/T -> H = Vo * T - 0.5 * Vo * T = 0.5 * Vo * T = 0.5 * Vo * Vo / A.
    '2H = 2(0.5 * Vo * Vo / A) = 4 * H * A = 2 * Vo * Vo = (Sqr(2) * Vo) ^ 2
    Const LOWJUMPVELOCITY As Single = SBARINITIALVELOCITY * 0.625F
    'Initial velocity for a low jump.
    Const GRAVITY As Single = 0.38F
    'Gravitational acceleration constant... how much the speed changes at each tick.
    Const GRAVITYTH As Single = GRAVITY * 1.5F
    'This represents how much the player moved due to gravity.
    Const LOWGRAVITY As Single = 0.25F

    Const PLATFORMCLOSENESS As Single = 0.0F
    Const PLATFORMDEPTH As Single = 100.0F

    Const PLATFORMVTXCOUNT As Integer = 24

    Const SHOULDER As Integer = 17
    'Looked in paint to determine the distance the shoulder is from the bottom of the foot.
    'This will be used for determining if the player gets on a platform.

    Const COINSIZE As Integer = 16
    Const COINVTXCOUNT As Integer = 4
    'The width and height of the coin.
    Const BULBSIZE As Integer = 8
    Const BULBHALF As Integer = BULBSIZE >> 1

    Const COINFRAMECOUNT As Integer = 4
    'The number of frames in the coin bitmap.

    Const WALLCLOSENESS As Single = -80.0F
    Const WALLDEPTH As Single = 120.0F
    Const WALLVTXCOUNT As Integer = 30

    Const LAVACLOSENESS As Single = -200.0F
    Const LAVADEPTH As Single = 200.0F
    Const LAVAVTXCOUNT As Integer = 36

    Const FLOATERSIZE As Integer = 16
    'The width and height of floater.
    Const FLOATERDEPTH As Integer = -20

    Const FREQUENCYDIVIDER As Integer = 128
    'The number that will divide the frequency of periodic functions.

    Const TERMINALVELOCITY As Single = 20
    'The maximum velocity that the character reaches while falling.

    Const JUMPBOARDSIZE As Integer = 16
    'Size of the jumping board.

    Const JBDCLOSENESS As Single = -8.0F
    Const JBDDEPTH As Single = 8.0F
    Const JBDVTXCOUNT As Integer = 30

    Const RTDVTXCOUNT As Integer = 96
    Const PNEUMOVTXCOUNT As Integer = 96

    Const MOVINGWALLCLOSENESS As Single = 0.0F
    Const MOVINGWALLDEPTH As Single = 100.0F
    Const MOVINGWALLVTXCOUNT As Integer = 30

    Const ICECLOSENESS As Single = PLATFORMCLOSENESS + 10.0F
    Const ICEDEPTH As Single = 100.0F
    Const ICEVTXCOUNT As Integer = 30
    Const ICEMAXSPEED As Single = MOVESPEED * 2
    'The maximum speed that the player can move on the ice - should be slightly larger than fast movement speed.

    Const WATERCLOSENESS As Single = LAVACLOSENESS
    Const WATERDEPTH As Single = LAVADEPTH
    Const WATERVTXCOUNT As Integer = LAVAVTXCOUNT
    Const GLASSCLOSENESS As Single = WALLCLOSENESS
    Const GLASSDEPTH As Single = WALLDEPTH
    Const GLASSVTXCOUNT As Integer = LAVAVTXCOUNT

    Const BACKDROPDEPTH As Single = 120.0F
    Const BACKDROPVTXCOUNT As Integer = 6

    Const TRAPPLATFORMCLOSENESS As Single = PLATFORMCLOSENESS
    Const TRAPPLATFORMDEPTH As Single = PLATFORMDEPTH
    Const TRAPPLATFORMVTXCOUNT As Integer = 12

    Const SLOPEDWALLCLOSENESS As Single = WALLCLOSENESS
    Const SLOPEDWALLDEPTH As Single = WALLDEPTH
    Const SLOPEDWALLVTXCOUNT As Integer = 30

    Const COLUMNVTXCOUNT As Integer = 36

    Const EXWALLCLOSENESS As Single = WALLCLOSENESS
    Const EXWALLDEPTH As Single = WALLDEPTH
    Const EXWALLVTXCOUNT As Integer = 30

    Const CRATECLOSENESS As Single = WALLCLOSENESS
    Const CRATEDEPTH As Single = 50
    Const CRATEVTXCOUNT As Integer = 30
    Const BACKCRATECLOSENESS As Single = CRATEDEPTH
    Const BACKCRATEDEPTH As Single = WALLDEPTH
    Const BACKCRATEVTXCOUNT As Integer = 30

    Const KONVEYCLOSENESS As Single = WALLCLOSENESS
    Const KONVEYDEPTH As Single = WALLDEPTH
    Const KONVEYVTXCOUNT As Integer = 30

    Const ROPECLOSENESS As Single = PLATFORMCLOSENESS + 40
    Const LADDERVTXCOUNT As Integer = 6
    Const ROPEVTXCOUNT2 As Integer = COLUMNVTXCOUNT

    Const CLIMBINGFRAMECOUNT As Integer = 4
    Const CLIMBINGFRAMECOUNTEX As Integer = CLIMBINGFRAMECOUNT - 1
    Const EFFCLIMBINGFRAMECOUNTEX As Integer = CLIMBINGFRAMECOUNT * FRAMETIME - 1

    Const GROUNDCLOSENESS As Single = CAMERADEPTH
    Const GROUNDDEPTH As Single = 250.0F
    Const GROUNDSECDEPTH As Single = 350.0F
    Const GROUNDFULLDEPTH As Single = BACKGROUNDDEPTH
    Const GROUNDVTXCOUNT As Integer = 60

    Const EYEBEAMCLOSENESS As Single = -10.0F
    Const EYEBEAMDEPTH As Single = 210.0F
    Const EYEBEAMVTXCOUNT As Integer = 156

    Const CROSSSTAYCLOSENESS As Single = EYEBEAMCLOSENESS
    Const CROSSSTAYDEPTH As Single = EYEBEAMDEPTH + 16.0F
    Const CROSSSTAYVTXCOUNT As Integer = 30

    Const PLANTDEPTH As Single = 40.0F
    Const PLANTVTXCOUNT As Integer = 6
    Const STATUEDEPTH As Single = 40.0F
    Const STATUEVTXCOUNT As Integer = 6
    Const PICTUREDEPTH As Single = BACKDROPDEPTH
    Const PICTUREVTXCOUNT As Integer = 6
    Const VIDEODEPTH As Single = BACKDROPDEPTH
    Const VIDEOVTXCOUNT As Integer = 6

    Const BULBDEPTH As Single = 100.0F
    Const BULBCLOSENESS As Single = 0.0F
    Const BULBVTXCOUNT As Integer = 15

    Const FENCECLOSENESS As Single = WALLCLOSENESS
    Const FENCEDEPTH As Single = 250.0F
    Const FENCEVTXCOUNT As Integer = 12
    Const FOREFENCEVTXCOUNT As Integer = 24

    Const EXITSIZE As Integer = 32

    Const SUBTERMINAL As Single = TERMINALVELOCITY * 0.45F
    'The maximum velocity that the character reaches while falling underwater.
    Const SUBGRAVITY As Single = GRAVITY * 0.45F
    'Underwater acceleration factor ... how much the speed changes at each tick while falling underwater.
    Const SUBGRAVITYTH As Single = SUBGRAVITY * 1.5F
    'This represents how much the player moved due to underwater acceleration.
    Const SUBINITIALVELOCITY As Single = SBARINITIALVELOCITY * 0.75F
    'The initial velocity for the player's jump using space bar underwater.
    Const SUBLOWJUMPVELOCITY As Single = LOWJUMPVELOCITY * 0.75F
    'Initial velocity for a low jump underwater.

    Const INVULNTIME As Integer = 30
    Const HIGHINVULN As Integer = 75

    Const CHARACTERCOUNT As Integer = 48
    Const CHARLINECOUNT As Integer = 35

    Const DISPLAYFILENUMEX As Integer = 9

    Const NUMLEVELSEX As Integer = 81

#End Region

    Dim UpKey, LeftKey, RightKey, DownKey, JumpKey, FireKey, RunKey, MenuKey, WeaponNextKey, WeaponPrevKey, BulletNextKey, BulletPrevKey, UseDoorKey As Keys
    '    Up     Left     Right     Down     Space    F        Shift   Back     Enter
    Dim ControlSet As Boolean, EquipTimer As Integer

    Dim ExitTimer As Integer
    Const EQUIPMAXTIMER As Integer = 500
    Const SPLASHCAPTION As String = "Platform Tour v1.0.4 (c) 2006-2014 Iceplug - http://www.iceplug.us"

    'X forward, Y down, Z , by RHR is IN.

    Private Sub FormD_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim LV As Integer
        Dim RV As AutoSuiteReturn
        Static TickCtr, FrameCtr As Integer
        Dim Trx As String()
        Ext = ".mid" 'Extension for the music.
        'Me.FormBorderStyle = FormBorderStyle.None
        Me.MinimizeBox = False
        Me.MaximizeBox = False
        Me.FormBorderStyle = FormBorderStyle.FixedSingle
        'Me.ControlBox = False
        Try
            Ri = New Random
            SR = New IO.StreamWriter(Application.StartupPath & "\files\error.txt")

            Trx = New String() {Application.StartupPath & "\files\metalslap.wav", Application.StartupPath & "\files\metal.wav", Application.StartupPath & "\files\swash.wav", _
                                Application.StartupPath & "\files\exposion.wav", Application.StartupPath & "\files\kank.wav", Application.StartupPath & "\files\gravstep.wav", _
                                Application.StartupPath & "\files\swim dive.wav", Application.StartupPath & "\files\Gun 5.wav", Application.StartupPath & "\files\creaky fast.wav", _
                                Application.StartupPath & "\files\hit window.wav", Application.StartupPath & "\files\Hit garbage.wav", Application.StartupPath & "\files\star.wav", _
                                Application.StartupPath & "\files\savepnt.wav", Application.StartupPath & "\files\argh.wav", Application.StartupPath & "\files\augh.wav", _
                                Application.StartupPath & "\files\Helicopter loop 1.wav"}
            'Bbx = New Sonic3(4, Me.Handle)
            Bbx = New Sonic3(Me.Handle, Trx)
            'Bbx.Prevent = True
            'Bbx.PlayMusic(Application.StartupPath & "\Vice6.mid")
            MyMode = DisplayMode.IPIntroscrn

            Me.Size = New Size(800, 600)
            Dim PsPms As PresentParameters  'So, we declare it.
            PsPms = New PresentParameters
            SetPresentParams(PsPms)
            'Now that we've set some properties, we can make our Device.
            D9 = New Device(0, DeviceType.Hardware, Me, CreateFlags.SoftwareVertexProcessing, PsPms)
            'And there's our device.
            'Cull clockwise
            D9.RenderState.CullMode = Cull.None

            D9.RenderState.ZBufferEnable = True
            D9.RenderState.Lighting = True
            D9.RenderState.Ambient = Color.DarkSlateGray        'When lighting is enabled and materials have been laid, this is the color of darkness.
            D9.RenderState.ZBufferWriteEnable = True

            D9.RenderState.PointScaleEnable = True
            D9.RenderState.PointSizeMin = COINSIZE
            D9.RenderState.PointSizeMax = 64.0F
            D9.RenderState.PointScaleB = 2.0F

            'Enable alphablending, which is transparency.
            D9.RenderState.AlphaBlendEnable = True

            'When drawing something with X transparency, all of the transparency from the source is made transparent.
            D9.RenderState.SourceBlend = Blend.SourceAlpha
            'And then, when transparency is drawn onto something else, the destination color is attenuated (interpolated)
            'to the source color by the transparency amount
            D9.RenderState.DestinationBlend = Blend.InvSourceAlpha
            'D9.RenderState.AlphaDestinationBlend = Blend.SourceAlpha
            'D9.RenderState.AlphaSourceBlend = Blend.InvSourceAlpha
            PrT = PrimitiveType.TriangleList

            Camera = New Vector3(0.0F, LANDHEIGHT - CAMERAHIGH, CAMERADEPTH)
            FocusPt = New Vector3(0.0F, LANDHEIGHT - CAMERADOWN, 0.0F)
            CamNormal = New Vector3(0.0F, -1.0F, 0.0F)

            Q = New LevelSuite

            LoadControls()
            LoadLevel(Application.StartupPath & "\files\cellar.dat")

            Q.SonicEvents = New ArrayList
            Q.Monsters = New ArrayList
            Q.Explosions = New ArrayList
            Q.Negations = New ArrayList
            Q.Shards = New ArrayList




            KUSize = 14
            VxB = New VertexBuffer(KUSize - 1) {}


            LevelMajor = 0
            LevelMinor = 1
            Player.Shots = New SelfBullet(2) {}
            For LV = 0 To Player.Shots.Length - 1
                Player.Shots(LV) = New SelfBullet(0, 0, 0, 0, 0, 0, 0)
            Next
            Diagnostics.Debug.WriteLine(Player.Shots.Length)
            Player.HP = 100
            Player.Lives = 3
            Player.ShotTime = 60
            Player.ShotSpeed = 4.0F
            CompletedLev = New Boolean(NUMLEVELSEX) {}
            HasUpgrade = New Boolean(NUMLEVELSEX) {}
            CoinCount = New Integer(NUMLEVELSEX) {}

            MaxCoinCount = New Integer() {76, 103, 86, 163, 71, 78, 137, 0, 0, _
                                          113, 82, 61, 108, 137, 125, 101, 0, 0, _
                                          73, 130, 85, 96, 140, 124, 173, 0, 0, _
                                          136, 105, 135, 121, 178, 159, 145, 0, 0, _
                                          38, 88, 74, 204, 73, 34, 100, 0, 0, _
                                          115, 121, 158, 172, 156, 155, 207, 0, 0, _
                                          188, 166, 122, 221, 68, 93, 157, 0, 0, _
                                          143, 207, 144, 172, 251, 211, 257, 0, 0, _
                                          10, 10, 10, 10, 10, 10, 10, 0}
            RowUnlocked = New Boolean(8) {}  'each to note if level *-9 available or, as index 8, to note if world is available
            ViewFiles = New String(9) {}
            Player.OwnedBullets = New Boolean(9) {}
            Player.OwnedBullets(ShotTypes.ShockWave) = False
            Player.OwnedWeapons = New Boolean(19) {}
            Player.OwnedWeapons(ShotPattern.SingleShot) = True

            Player.WeaponType = ShotPattern.SingleShot
            Player.BulletType = ShotTypes.ShockWave

            CusText = New String(CHARLINECOUNT) {}

            'CusText = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%"
            'DisplayFont = New Direct3D.Font(D9, Me.Font)

            InitBuffers()
            TextureMeshSetup()


            'Next, our vertex buffer.
            'I don't use overload #1 or #3 because I don't know how big a TransformedColored vertex is.




            'Now, with all DirectX applications, we are obliged to use a game loop.
            'The DirectDraw tutorial starts game loops on episode 3, but DirectX9 is harder... we must do
            'this now.  (Actually, we probably don't have to do this now)
            'So first, we need to make sure the form is visible before we do any drawing.
            Me.Show()
            Me.Focus()

            'And then of course the game loop.
            Running = True
            Do While Running

                If MyMode = DisplayMode.Regular Then
                    'Game processing would go in this if block.
                        Movement()

                    Render()
                    If MyMode = DisplayMode.LevelIntro Then
                        ToNextLevel()
                    End If
                ElseIf MyMode = DisplayMode.VictoryCutscene Then
                    VictoryLoop()
                    If LevelMinor = 10 Then
                        MyMode = DisplayMode.LevelIntro
                        ToNextLevel()
                    End If

                ElseIf MyMode = DisplayMode.LevelIntro Then
                    RenderLevelIntro()
                    If CtrTwo >= 180 Then
                        MyMode = DisplayMode.Regular
                    End If
                    CtrTwo += 1
                ElseIf MyMode = DisplayMode.LevelTransfer Then
                    RenderLevelXfer()
                ElseIf MyMode = DisplayMode.WeaponSelect Then
                    CusText(0) = Player.BulletType.ToString & " WITH " & Player.WeaponType.ToString & " --------------------------------------------------------------"
                    RenderWeaponSelect()
                ElseIf MyMode = DisplayMode.PauseMenu Then
                    RenderPauseMenu()
                ElseIf MyMode = DisplayMode.OptionsMenu Then
                    RenderOptions()
                ElseIf MyMode = DisplayMode.ControlsMenu Then
                    RenderControls()
                ElseIf MyMode = DisplayMode.SaveScreen Then
                    RenderSaveName()

                ElseIf MyMode = DisplayMode.LoadScreen Then
                    CusText(0) = "Select game to load."
                    CusText(1) = "--------------------"
                    For LV = 0 To 9
                        CusText(LV + 2) = ViewFiles(LV)
                    Next
                    RenderLoader()
                ElseIf MyMode = DisplayMode.YouDied Then
                    DiedLoop()
                ElseIf MyMode = DisplayMode.IPIntroscrn Then
                    IPLoop()
                ElseIf MyMode = DisplayMode.OpenToMainscrn Then
                    OpToMainLoop()
                ElseIf MyMode = DisplayMode.EndCredits Then
                    EndCreditLoop()
                ElseIf MyMode = DisplayMode.Mainscrn Then
                    MainScreenLoop()
                ElseIf MyMode = DisplayMode.CloseMainscrn Then
                    Me.RenderCloseMS()
                    If CtrTwo = 2200 Then
                        If Selexion = 0 Then
                            CurrentLevelFile = Application.StartupPath & "\files\L0.dat"
                            LoadLevel(CurrentLevelFile)
                            NewLevelInit()
                            MyMode = DisplayMode.GameIntro
                            Bbx.PlayMusic(Application.StartupPath & "\files\Vertical" & Ext)
                            CtrTwo = 0
                        ElseIf Selexion = 1 Then
                            Selexion = 0
                            AllFiles = System.IO.Directory.GetFiles(Application.StartupPath & "\files", "*.pft")
                            TopIdx = 0
                            If AllFiles.GetUpperBound(0) < 9 Then
                                For LV = 0 To AllFiles.GetUpperBound(0)
                                    ViewFiles(LV) = System.IO.Path.GetFileName(AllFiles(LV))
                                Next
                                For LV = AllFiles.GetUpperBound(0) + 1 To 9
                                    ViewFiles(LV) = "--------------"
                                Next
                            Else
                                For LV = 0 To 9
                                    ViewFiles(LV) = System.IO.Path.GetFileName(AllFiles(LV + TopIdx))
                                Next
                            End If
                            MyMode = DisplayMode.LoadScreen
                            CtrTwo = 0
                        ElseIf Selexion = 2 Then
                            MyMode = DisplayMode.OptionsMenu
                            Selexion = 0
                            CtrTwo = 0
                        End If
                    End If
                    CtrTwo += 1
                ElseIf MyMode = DisplayMode.GameIntro Then
                    If CtrTwo = 1 Then
                        SetCusText("All right, I am going to drop you off at the top of this building.  Be careful - I see something flying up ahead and movement on the rooftops.  Good luck.".ToUpper)
                        Bbx.PlayWaveLoop(15, True)
                        Player.Loc.X = 199
                        Player.Loc.Y = -1100
                    ElseIf CtrTwo <= 480 Then
                        'CusText = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%"
                        Player.Loc.Offset(5, 0)

                    ElseIf CtrTwo = 500 Then
                        'CusText = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%"
                        'Ladder = DirectCast(Q.Ropes(0), RectangleF)
                        Player.Loc.X = 2594
                        Player.IsClimbing = True
                        Player.Direc = 1
                        Player.LB = -800
                        Player.LH = -1105
                        Player.OnIce = False
                        Player.OnSlope = False
                        Player.IsJumping = False
                    ElseIf CtrTwo >= 800 AndAlso CtrTwo < 950 Then
                        CusText(0) = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%"
                        CusLines = 1
                        Player.Loc.Offset(0, MOVESPEED)
                        Player.Veloc = MOVESPEED
                        If Player.Loc.Top > Player.LB Then
                            Player.Loc.Offset(0, Player.LH - Player.Loc.Bottom)
                        End If
                        Player.AnimCycler = (Player.AnimCycler + 1) And EFFCLIMBINGFRAMECOUNTEX   'Make player animate.
                    ElseIf CtrTwo = 950 Then
                        Player.IsClimbing = False
                        Player.IsJumping = True
                        'We are not really jumping, but we are falling.
                        MobileIndex = -1
                        KonveyIndex = -1
                        RTDIndex = -1
                        PNMIndex = -1
                        BreakGlassIndex = -1
                        'We are not standing on anything now, because we are jumping.
                        Player.Veloc = 0
                        'Since we are falling from rest, the initial velocity will be zero.
                    ElseIf CtrTwo >= 951 AndAlso CtrTwo <= 978 Then
                        If Player.IsJumping Then
                            Player.Loc.Offset(0, Player.Veloc + GRAVITY)
                            'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
                            Player.Veloc += GRAVITY
                            'dV = (PlayerVeloc +=) A (GRAVITY)
                            'Moves the character according to its velocity, gravity, and location.
                            If Player.Veloc >= TERMINALVELOCITY Then  'We're in TV.
                                Player.Veloc = TERMINALVELOCITY  'Limit the velocity.
                            End If
                        End If
                        If Player.IsJumping Then
                            RV = LevelSuite.WallJump(Q.Walls, Player.Loc, Player.Veloc, False)
                            If RV.RetState > SuiteReturnType.NoReturn Then
                                If RV.RetState = SuiteReturnType.Landing Then
                                    'If player is falling, then the player will land on the wall.
                                    Player.IsJumping = False
                                    'Stop the jump.
                                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right
                                    'Set the endpoints.
                                    Player.AnimCycler = 0
                                    'Reset the cycler.
                                    Player.OnIce = False : Player.HorizontVeloc = 0
                                    'We are no longer on ice.
                                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom)
                                    'Position the player to stand on this wall.

                                End If
                            End If
                        End If
                    ElseIf CtrTwo = 1080 Then
                        Q.AutoMessages(0) = New AutoMessage(2800, 1)
                        Q.AutoMessages(1) = New AutoMessage(6000, 2)
                        Q.AutoMessages(2) = New AutoMessage(10000, 12)
                        MyMode = DisplayMode.Regular
                        Bbx.StopWave(15)
                    End If
                    D9.Lights(0).Position = New Vector3(Player.Loc.X, Player.Loc.Y, 0.0F)
                    Bits(0) = New Vector3(Player.Loc.X + HALFWIDTH, Player.Loc.Y + HALFHEIGHT, 0.0F)

                    Camera = New Vector3(Player.Loc.X + HALFWIDTH, Player.Loc.Y + HALFHEIGHT - CAMERAHIGH, CAMERADEPTH)
                    FocusPt = New Vector3(Player.Loc.X + HALFWIDTH - 1, Player.Loc.Y + HALFHEIGHT - CAMERADOWN, 0.0F)

                    Me.RenderGameIntro()
                    CtrTwo += 1
                End If

                Player.IsShooting = False

                If Environment.TickCount >= TickCtr Then
                    TickCtr = Environment.TickCount + 1000
                    FPS = FrameCtr + 1
                    FrameCtr = 0
                Else
                    FrameCtr += 1
                End If

                Application.DoEvents()
                'Of course, we MUST have a DoEvents in the loop, otherwise, Windows will have a seizure.
                Tick += 1
                ExitTimer -= 1

            Loop


            Me.Text = "Buffer Dump"
            Application.DoEvents()

            SkyBuffer.Dispose()
            IPBuffer.Dispose()
            TitleBuffer.Dispose()
            IPPicture.Dispose()
            TitlePicture.Dispose()
            BlockFont.Dispose()

            Me.Text = "Bitmech Dump"
            Application.DoEvents()

            For LV = 0 To Bitmech.GetUpperBound(0)
                Bitmech(LV).Dispose()
            Next

            SkyTx.Dispose()

            Me.Text = "Floater Dump"
            Application.DoEvents()
            For LV = 0 To Q.Floaters.Count - 1
                MM(LV).Dispose()
            Next

            'For LV = 0 To Q

            Me.Text = "PsTx Dump"
            Application.DoEvents()
            For LV = 0 To 7
                PsTx(LV).Dispose()
            Next

            Me.Text = "Tx Dump"
            Application.DoEvents()
            For LV = 0 To Tx.GetUpperBound(0)
                Tx(LV).Dispose()
            Next

            If MyMode <> DisplayMode.Mainscrn AndAlso Not (Bits Is Nothing) Then
                Me.Text = "KUs VxB Dump"
                Application.DoEvents()
                For LV = 0 To KUSize - 1
                    VxB(LV).Dispose()
                Next

            End If

            Me.Text = "DynaVxB Dump"
            Application.DoEvents()
            If Q.Mobilewalls.Count > 0 Then
                DynaVxB.Dispose()
            End If

            Me.Text = "Xlucent Dump"
            Application.DoEvents()
            If Convert.ToBoolean(Q.Lavas.Count Or Q.Waters.Count Or Q.Glasses.Count) Then
                Xlucent.Dispose()
            End If

            Me.Text = "Billbds Dump"
            Application.DoEvents()
            Exdo.Dispose()
            BackTx.Dispose()
            TextBfr.Dispose()
            Billboard.Dispose()


            Me.Text = "PsB Dump"
            Application.DoEvents()
            If Q.Coins.Count > 0 Then
                PsB.Dispose()
            End If
            Me.Text = "Removing D9 device from form, please wait."
            Application.DoEvents()
            D9.Dispose()

            Me.Text = "Stopping Music, closing form"
            Application.DoEvents()
            Bbx.StopMusic()

        Catch Dex As Microsoft.DirectX.DirectXException
            MessageBox.Show("Error in DirectX 9.  Chances are that your graphics card does not support this version.")
            MessageBox.Show(Dex.ToString())
            'You're practically obligated to add this check.
        Catch ex As Exception
            Me.Text = ex.Message
            SR.WriteLine(ex.ToString())
            Application.DoEvents()
            System.Threading.Thread.Sleep(2000)
            'Anything else I'll display in the window for 3 seconds.
        Finally
            SR.WriteLine("End Execution " & Tick.ToString)
            SR.Close()
            Me.Close()
            'And close the form when the game loop is over.
        End Try
        'And close the form when the game loop is over.

    End Sub
    Private Sub InitBuffers()

        TextBfr = New VertexBuffer(GetType(CustomVertex.TransformedColoredTextured), CHARACTERCOUNT * 6 * CHARLINECOUNT, D9, Usage.WriteOnly, CustomVertex.TransformedColoredTextured.Format, Pool.Managed)
        Billboard = New VertexBuffer(GetType(CustomVertex.PositionColoredTextured), 2048, D9, Usage.WriteOnly, CustomVertex.PositionColoredTextured.Format, Pool.Managed)

        SkyBuffer = New VertexBuffer(GetType(CustomVertex.TransformedColoredTextured), 6, D9, Usage.WriteOnly, CustomVertex.TransformedColoredTextured.Format, Pool.Managed)
        IPBuffer = New VertexBuffer(GetType(CustomVertex.TransformedColoredTextured), 6, D9, Usage.WriteOnly, CustomVertex.TransformedColoredTextured.Format, Pool.Managed)
        TitleBuffer = New VertexBuffer(GetType(CustomVertex.TransformedColoredTextured), 144, D9, Usage.WriteOnly, CustomVertex.TransformedColoredTextured.Format, Pool.Managed)
        OptionBuffer = New VertexBuffer(GetType(CustomVertex.TransformedColoredTextured), 2400, D9, Usage.WriteOnly, CustomVertex.TransformedColoredTextured.Format, Pool.Managed)
        ChopBuffer = New VertexBuffer(GetType(CustomVertex.PositionColoredTextured), 12, D9, Usage.WriteOnly, CustomVertex.PositionColoredTextured.Format, Pool.Managed)
        Backgrd = New VertexBuffer(GetType(CustomVertex.PositionNormalTextured), 12, D9, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed)

    End Sub
    Private Sub NewLevelInit()
        'The file has been loaded.  This procedure sets all values so that the game will run immediately.
        Me.Text = "Platform Tour"
        Dim LV As Integer, R As RectangleF
        Dim DVCount As Integer
        MobileIndex = -1
        KonveyIndex = -1
        RTDIndex = -1
        PNMIndex = -1
        KU = New KilounitArea(KUSize - 1) {}

        Bits = New ArrayList
        Bits.Add(New Vector3(Player.Loc.X + HALFWIDTH, Player.Loc.Y + HALFHEIGHT, 0.0F))

        Mx = New Matrix(Q.Monsters.Count + 400) {}
        '+ 100 to account for bullets

        PopulateKUs()


        For LV = 0 To KUSize - 1
            VertexCount = FIELDVERTEXCOUNT + KU(LV).PlatformCount * PLATFORMVTXCOUNT + KU(LV).WallCount * WALLVTXCOUNT + _
            KU(LV).JumpCount * JBDVTXCOUNT + KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + _
            KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + _
            KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + _
            KU(LV).GroundCount * GROUNDVTXCOUNT + KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT + KU(LV).AftFenceCount * FENCEVTXCOUNT + KU(LV).FieldFenceCount * FENCEVTXCOUNT + _
            KU(LV).EyeBeamCount * EYEBEAMVTXCOUNT + KU(LV).CrossStayCount * CROSSSTAYVTXCOUNT + _
            KU(LV).CrateCount * CRATEVTXCOUNT + _
            KU(LV).BackcrateCount * BACKCRATEVTXCOUNT + _
            KU(LV).PlantCount * PLANTVTXCOUNT + KU(LV).StatueCount * STATUEVTXCOUNT + _
            KU(LV).PictureCount * PICTUREVTXCOUNT

            VxB(LV) = New VertexBuffer(GetType(CustomVertex.PositionNormalTextured), VertexCount, D9, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed)

        Next

        If Q.Coins.Count > 0 Then
            PsB = New VertexBuffer(GetType(CustomVertex.PositionColoredTextured), Q.Coins.Count * COINVTXCOUNT, D9, Usage.WriteOnly, CustomVertex.PositionColoredTextured.Format, Pool.Managed)
        End If

        If Q.Mobilewalls.Count + Q.Conveyors.Count + Q.VideoscreenCount + Q.RTDs.Count + Q.Pneumos.Count + Upgrade.Width > 0 Then
            DVCount = MOVINGWALLVTXCOUNT * Q.Mobilewalls.Count + KONVEYVTXCOUNT * Q.Conveyors.Count + _
                        VIDEOVTXCOUNT * Q.VideoscreenCount + RTDVTXCOUNT * Q.RTDs.Count + Q.Pneumos.Count * PNEUMOVTXCOUNT + 6
            DynaVxB = New VertexBuffer(GetType(CustomVertex.PositionNormalTextured), DVCount, D9, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed)
            'That is a LOT of non-trivial parameters.
        End If

        If Convert.ToBoolean(Q.Lavas.Count Or Q.Waters.Count Or Q.Glasses.Count) Then
            Xlucent = New VertexBuffer(GetType(CustomVertex.PositionColored), LAVAVTXCOUNT * Q.Lavas.Count + WATERVTXCOUNT * Q.Waters.Count + GLASSVTXCOUNT * Q.Glasses.Count, D9, Usage.WriteOnly, CustomVertex.TransformedColored.Format, Pool.Managed)
        End If


        Lighting()

        TextureMeshSetup()
        'Set up our textures.

        SetVertexBufferData(VxB, D9)
        If Convert.ToBoolean(Q.Lavas.Count Or Q.Waters.Count) Then
            SetXlucentBuffer(Xlucent, D9)

        End If

        SetBackgrd(Backgrd, D9)
    End Sub
    Private Sub IPLoop()
        RenderIP()
        CtrTwo += 1
        If CtrTwo = 200 Then

            Bbx.PlayMusic(Application.StartupPath & "\files\IP.mid")
        ElseIf CtrTwo = 620 Then
            Bbx.StopMusic()
        ElseIf CtrTwo = 900 Then
            MyMode = DisplayMode.OpenToMainscrn
            CtrTwo = 0
        End If
    End Sub
    Private Sub EndCreditLoop()
        RenderEndCredits()
        CtrTwo += 1
        CusLines = 1
        If CtrTwo = 1 Then
            CusText(0) = "Thus bringing an end to Huerz Smith's"
        ElseIf CtrTwo = 151 Then
            CusText(0) = "diabolical plan of world domination."
        ElseIf CtrTwo = 301 Then
            CusText(0) = "Mission Accomplished!"
        ElseIf CtrTwo = 601 Then
            CusText(0) = "    - Debrief -"
        ElseIf CtrTwo = 751 Then
            CusText(0) = "I was initially dropped off"
            '             12345678..12345678..12345678..12345678..1
        ElseIf CtrTwo = 901 Then
            CusText(0) = "on the roofs of the Lucadio District."
        ElseIf CtrTwo = 1051 Then
            CusText(0) = "where all of the members of the mob"
        ElseIf CtrTwo = 1201 Then
            CusText(0) = "were secretly meeting."
        ElseIf CtrTwo = 1351 Then
            CusText(0) = "I was detected here - the mob fled"
        ElseIf CtrTwo = 1501 Then
            CusText(0) = "to the North Hills, with the exception"
        ElseIf CtrTwo = 1651 Then
            CusText(0) = "of one named Sauitek.  I learned from"
        ElseIf CtrTwo = 1801 Then
            CusText(0) = "him that they went to the North Hills"
        ElseIf CtrTwo = 1951 Then
            CusText(0) = "to find a boat that will take them all"
        ElseIf CtrTwo = 2101 Then
            CusText(0) = "across the ocean where they"
        ElseIf CtrTwo = 2251 Then
            CusText(0) = "would be able to launch their satellite."
        ElseIf CtrTwo = 2401 Then
            CusText(0) = "I found Caputaru in the North Hills,"
        ElseIf CtrTwo = 2551 Then
            CusText(0) = "and I learned that the mob is traveling"
        ElseIf CtrTwo = 2701 Then
            CusText(0) = "across the ocean within a fleet of ships."
        ElseIf CtrTwo = 2851 Then
            CusText(0) = "I found Rychyd among the fleet, and I"
        ElseIf CtrTwo = 3001 Then
            CusText(0) = "learned that the mob still needed"
        ElseIf CtrTwo = 3151 Then
            CusText(0) = "to construct a satellite fuel system"
        ElseIf CtrTwo = 3301 Then
            CusText(0) = "for long-term satellite control."
        ElseIf CtrTwo = 3451 Then
            CusText(0) = "In the factory, I prevented Bobeck from"
        ElseIf CtrTwo = 3601 Then
            CusText(0) = "building an electrical satellite engine."
        ElseIf CtrTwo = 3751 Then
            CusText(0) = "In the Clamapi construction zone, I was"
        ElseIf CtrTwo = 3901 Then
            CusText(0) = "able to stop Notel from adding the solar"
        ElseIf CtrTwo = 4051 Then
            CusText(0) = "panels to the satellite."
        ElseIf CtrTwo = 4201 Then
            CusText(0) = "I then learned that the mob was making a"
        ElseIf CtrTwo = 4351 Then
            CusText(0) = "last ditch effort to use geothermal power"
        ElseIf CtrTwo = 4501 Then
            CusText(0) = "to power the satellite."
        ElseIf CtrTwo = 4651 Then
            CusText(0) = "My attempts to stop the magma extraction"
        ElseIf CtrTwo = 4801 Then
            CusText(0) = "caused the volcano of Hexadiemme Island"
        ElseIf CtrTwo = 4951 Then
            CusText(0) = "to explode violently."
        ElseIf CtrTwo = 5101 Then
            CusText(0) = "I awoke to find that the satellite was"
        ElseIf CtrTwo = 5251 Then
            CusText(0) = "about to be launched in a few minutes."
        ElseIf CtrTwo = 5401 Then
            CusText(0) = "I arrived too late to stop the launch"
        ElseIf CtrTwo = 5551 Then
            CusText(0) = "but, I was able to stop Shelis from"
        ElseIf CtrTwo = 5701 Then
            CusText(0) = "launching the geothermal fuel unit."
        ElseIf CtrTwo = 5851 Then
            CusText(0) = "I was then able to find Huerz Smith"
        ElseIf CtrTwo = 6001 Then
            CusText(0) = "leaving for the space station in order to"
        ElseIf CtrTwo = 6151 Then
            CusText(0) = "control the ray emitter, but I was able"
        ElseIf CtrTwo = 6301 Then
            CusText(0) = "to subdue him before he used the ray"
        ElseIf CtrTwo = 6451 Then
            CusText(0) = "emitter."
        ElseIf CtrTwo = 6601 Then
            CusText(0) = "Yes, I have saved the world. :)"
        ElseIf CtrTwo = 6751 Then
            CusText(0) = "This game is brought to you by Iceplug"
        ElseIf CtrTwo = 6901 Then
            CusText(0) = "http://www.iceplug.us/"
        ElseIf CtrTwo = 7051 Then
            CusText(0) = "Thanks for playing! :)"
            CompletedLev((9 - 1) * 8 + 1 - 1) = True
            SaveGame(AutoSaveLevel)
        End If
    End Sub
    Private Sub VictoryLoop()
        RenderMainScreenTransition()
        CtrTwo += 1
        CusLines = 1
        Select Case LevelMajor
            Case 1
                If CtrTwo = 1 Then
                    CusText(0) = "Chris: Where is your boss?"
                ElseIf CtrTwo = 150 Then
                    CusText(0) = "Sauitek: Dunno what you're talking about."
                    '             12345678..12345678..12345678..12345678..1
                ElseIf CtrTwo = 300 Then
                    CusText(0) = "Chris: Tell me where Huerz is, now!"
                ElseIf CtrTwo = 450 Then
                    CusText(0) = "Sauitek: Do it!  I'll never talk."
                ElseIf CtrTwo = 600 Then
                    CusText(0) = "Chris: Hmph."
                ElseIf CtrTwo = 750 Then
                    CusText(0) = "Chris: And now, you'll never walk."
                ElseIf CtrTwo = 900 Then
                    CusText(0) = "Chris: What's this?  An address"
                ElseIf CtrTwo = 1050 Then
                    CusText(0) = "Chris: Deliver equipment to North Hills"
                ElseIf CtrTwo = 1200 Then
                    CusText(0) = "Sauitek: What?  Give me that."
                ElseIf CtrTwo = 1350 Then
                    CusText(0) = "Chris: Well, looks like I've got some..."
                ElseIf CtrTwo = 1500 Then
                    CusText(0) = "...""toys"" to break before I head north."
                ElseIf CtrTwo = 1650 Then
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                    LevelMinor = 10
                End If
            Case 2
                If CtrTwo = 1 Then
                    CusText(0) = "Caputaru: Ugh.  How did you do that?"
                ElseIf CtrTwo = 150 Then
                    CusText(0) = "Chris: Well, I think I'll be taking this."
                    '             12345678..12345678..12345678..12345678..1
                ElseIf CtrTwo = 300 Then
                    CusText(0) = "Caputaru: You're gonna take my ship?"
                ElseIf CtrTwo = 450 Then
                    CusText(0) = "Caputaru: Ho ho ho.  This ship is just"
                ElseIf CtrTwo = 600 Then
                    CusText(0) = "a dud.  It doesn't fly, either."
                ElseIf CtrTwo = 750 Then
                    CusText(0) = "Chris: You.  Shut it."
                ElseIf CtrTwo = 900 Then
                    CusText(0) = "Chris: I can fly this at least to the"
                ElseIf CtrTwo = 1050 Then
                    CusText(0) = "pier where Huerz is."
                ElseIf CtrTwo = 1200 Then
                    CusText(0) = "Whoa, there's a lot more people here than"
                ElseIf CtrTwo = 1350 Then
                    CusText(0) = "I expected.  Looks like I'll just have to"
                ElseIf CtrTwo = 1500 Then
                    CusText(0) = "approach from the outside."
                ElseIf CtrTwo = 1650 Then
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                    LevelMinor = 10
                End If
            Case 3
                If CtrTwo = 1 Then
                    CusText(0) = "Rychyd: You puny little worm.  I should"
                ElseIf CtrTwo = 150 Then
                    CusText(0) = "have crushed you underfoot.  You're a"
                    '             12345678..12345678..12345678..12345678..1
                ElseIf CtrTwo = 300 Then
                    CusText(0) = "disgrace to the very dirt that covers"
                ElseIf CtrTwo = 450 Then
                    CusText(0) = "your wretched skin."
                ElseIf CtrTwo = 600 Then
                    CusText(0) = "Chris: Whoa, nice to meet you, too."
                ElseIf CtrTwo = 750 Then
                    CusText(0) = "Chris: And what's this thing that you're"
                ElseIf CtrTwo = 900 Then
                    CusText(0) = "resting your stuff on, over here?"
                ElseIf CtrTwo = 1050 Then
                    CusText(0) = "Rychyd: It's air, you miscreant."
                ElseIf CtrTwo = 1200 Then
                    CusText(0) = "The engine and solar panels aren't here!"
                ElseIf CtrTwo = 1350 Then
                    CusText(0) = "Chris: Great, I think I'll do a quick"
                ElseIf CtrTwo = 1500 Then
                    CusText(0) = "drop test of those while I'm here."
                ElseIf CtrTwo = 1650 Then
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                    LevelMinor = 10
                End If
            Case 4
                If CtrTwo = 1 Then
                    CusText(0) = "Bobeck: Even if you defeat me, you won't"
                ElseIf CtrTwo = 150 Then
                    CusText(0) = "stop the Reckoning."
                    '             12345678..12345678..12345678..12345678..1
                ElseIf CtrTwo = 300 Then
                    CusText(0) = "Chris: You better recognize."
                ElseIf CtrTwo = 450 Then
                    CusText(0) = "Bobeck: Oh, no!  My control circuits!"
                ElseIf CtrTwo = 600 Then
                    CusText(0) = "Chris: A job well done."
                ElseIf CtrTwo = 750 Then
                    CusText(0) = "Bobeck: Such talking will only get your"
                ElseIf CtrTwo = 900 Then
                    CusText(0) = "goose cooked by the satellite first."
                ElseIf CtrTwo = 1050 Then
                    CusText(0) = "Chris: Not without your engine control."
                ElseIf CtrTwo = 1200 Then
                    CusText(0) = "Bobeck: Darn."
                ElseIf CtrTwo = 1350 Then
                    CusText(0) = "Chris: Something doesn't seem right."
                ElseIf CtrTwo = 1500 Then
                    CusText(0) = "It's time to keep going."
                ElseIf CtrTwo = 1650 Then
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                    LevelMinor = 10
                End If
            Case 5
                If CtrTwo = 1 Then
                    CusText(0) = "Notel: You must have lost your mind."
                ElseIf CtrTwo = 150 Then
                    CusText(0) = "Chris: No, I think you've lost yours."
                    '             12345678..12345678..12345678..12345678..1
                ElseIf CtrTwo = 300 Then
                    CusText(0) = "Notel: The solar panels aren't the power"
                ElseIf CtrTwo = 450 Then
                    CusText(0) = "source that you are looking for."
                ElseIf CtrTwo = 600 Then
                    CusText(0) = "Chris: Of course not.  It says so here on"
                ElseIf CtrTwo = 750 Then
                    CusText(0) = "these construction notes on the back."
                ElseIf CtrTwo = 900 Then
                    CusText(0) = "Notel: They didn't get rid of those?"
                ElseIf CtrTwo = 1050 Then
                    CusText(0) = "Anyway, you won't be able to stop the"
                ElseIf CtrTwo = 1200 Then
                    CusText(0) = "volcanic eruption, so you should just"
                ElseIf CtrTwo = 1350 Then
                    CusText(0) = "give up while you're ahead."
                ElseIf CtrTwo = 1500 Then
                    CusText(0) = "Chris: Nope, it's time to visit the"
                ElseIf CtrTwo = 1650 Then
                    CusText(0) = "volcanoes and end this madness!"
                ElseIf CtrTwo = 1800 Then
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                    LevelMinor = 10
                End If
            Case 6
                If CtrTwo = 1 Then
                    CusText(0) = "Chris: Huerz, your time is up!"
                ElseIf CtrTwo = 150 Then
                    CusText(0) = "Huerz: But you're already too late.  The"
                    '             12345678..12345678..12345678..12345678..1
                ElseIf CtrTwo = 300 Then
                    CusText(0) = "volcano ignition is already inside."
                ElseIf CtrTwo = 450 Then
                    CusText(0) = "That means that, in a few minutes, this"
                ElseIf CtrTwo = 600 Then
                    CusText(0) = "beautiful resort is going to be ash."
                ElseIf CtrTwo = 750 Then
                    CusText(0) = "Which is a shame, I'll admit.  The beach"
                ElseIf CtrTwo = 900 Then
                    CusText(0) = "is very clean... not crowded like others"
                ElseIf CtrTwo = 1050 Then
                    CusText(0) = "that I've been to.  And the surf is nice"
                ElseIf CtrTwo = 1200 Then
                    CusText(0) = "but anyway, enough chat.  Ta-ta!"
                ElseIf CtrTwo = 1350 Then
                    CusText(0) = "* rumble *"
                ElseIf CtrTwo = 1500 Then
                    CusText(0) = "Chris: No, it's too late!"
                ElseIf CtrTwo = 1650 Then
                    CusText(0) = ". . . "
                ElseIf CtrTwo = 1800 Then
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                    LevelMinor = 10
                End If
            Case 7
                If CtrTwo = 1 Then
                    CusText(0) = "Shelis: What do you hope to accomplish?"
                ElseIf CtrTwo = 150 Then
                    CusText(0) = "Chris: It's simple: destroying your"
                    '             12345678..12345678..12345678..12345678..1
                ElseIf CtrTwo = 300 Then
                    CusText(0) = "geothermal power supply, your satellite's"
                ElseIf CtrTwo = 450 Then
                    CusText(0) = "rays won't be powerful enough to do any"
                ElseIf CtrTwo = 600 Then
                    CusText(0) = "damage.  What I want to know is why your"
                ElseIf CtrTwo = 750 Then
                    CusText(0) = "boss is already in space?"
                ElseIf CtrTwo = 900 Then
                    CusText(0) = "Shelis: Oh, no, he's not."
                ElseIf CtrTwo = 1050 Then
                    CusText(0) = "Chris: Oh, yes, he just left."
                ElseIf CtrTwo = 1200 Then
                    CusText(0) = "Shelis: Well, he's probably got a backup"
                ElseIf CtrTwo = 1350 Then
                    CusText(0) = "plan that you should find out!"
                ElseIf CtrTwo = 1500 Then
                    CusText(0) = "Chris: Don't tell me what to do, or I'll"
                ElseIf CtrTwo = 1650 Then
                    CusText(0) = "leave you here!"
                ElseIf CtrTwo = 1800 Then
                    CusText(0) = "Shelis: You were gonna do that, anyway."
                ElseIf CtrTwo = 1950 Then
                    CusText(0) = "Chris: I'm glad you figured that one out."
                ElseIf CtrTwo = 2100 Then
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                    LevelMinor = 10
                End If
        End Select

        'MyMode = DisplayMode.LevelIntro
        'CtrTwo = 0
        'LevelMinor = 10

    End Sub
    Private Sub OpToMainLoop()
        RenderMainScreenTransition()
        CtrTwo += 1
        CusLines = 1
        If CtrTwo = 1 Then
            CusText(0) = "After three years of waiting..."
        ElseIf CtrTwo = 150 Then
            CusText(0) = "it is now or never."
        ElseIf CtrTwo = 300 Then
            CusText(0) = "Three years ago, an organized mob, led by"
            '             12345678..12345678..12345678..12345678..1
        ElseIf CtrTwo = 450 Then
            CusText(0) = "Huerz Smith, leader of the mob of eight,"
        ElseIf CtrTwo = 600 Then
            CusText(0) = "stole and modified the plans for a ray"
        ElseIf CtrTwo = 750 Then
            CusText(0) = "emitter mounted on a satellite"
        ElseIf CtrTwo = 900 Then
            CusText(0) = "The ray emitter modification now allows"
        ElseIf CtrTwo = 1050 Then
            CusText(0) = "disruption of the atmosphere above Earth,"
        ElseIf CtrTwo = 1200 Then
            CusText(0) = "altering global weather patterns and"
        ElseIf CtrTwo = 1350 Then
            CusText(0) = "allowing solar radiation to fry"
        ElseIf CtrTwo = 1500 Then
            CusText(0) = "all existing life on Earth."
        ElseIf CtrTwo = 1650 Then
            CusText(0) = "This dangerous mob remained hidden for"
        ElseIf CtrTwo = 1800 Then
            CusText(0) = "the last three years, until recently,"
        ElseIf CtrTwo = 1950 Then
            CusText(0) = "when we determined their dangerous plans."
        ElseIf CtrTwo = 2100 Then
            CusText(0) = "The time has come to stop these plans!"
        ElseIf CtrTwo = 2250 Then
            CusText(0) = "Smith has begun preparations to launch"
        ElseIf CtrTwo = 2400 Then
            CusText(0) = "the satellite - time is running out."
        ElseIf CtrTwo = 2550 Then
            CusText(0) = "I have been training"
        ElseIf CtrTwo = 2700 Then
            CusText(0) = "and waiting to take them down."
        ElseIf CtrTwo = 2850 Then
            CusText(0) = "My name is Chris Sumult..."
        ElseIf CtrTwo = 3000 Then
            CusText(0) = "and I will save the world."
        ElseIf CtrTwo = 3150 Then
            MyMode = DisplayMode.Mainscrn
            CtrTwo = 0

        End If
    End Sub
    Private Sub MainScreenLoop()
        RenderMainScreen()
        If CtrTwo = 0 Then
            Bbx.PlayMusic(Application.StartupPath & "\files\PlatformTour" & Ext)
            NewShot()
            Me.Text = SPLASHCAPTION
        End If

        CtrTwo += 1
    End Sub
    Private Sub DiedLoop()
        RenderDeath()
    End Sub

    Private Sub FormD_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
        Dim LV, Ctr As Integer
        If MyMode = DisplayMode.Regular Then

            If e.KeyCode = Keys.Escape Then
                If ExitTimer > 0 Then
                    Running = False
                    'End the game loop when Escape is pressed.
                Else
                    ExitTimer = 120
                End If

            End If

            If e.KeyCode = WeaponPrevKey Then
                LV = CType(Player.WeaponType, Integer)
                Do
                    Ctr += 1
                    LV -= 1
                    If LV = -1 Then
                        LV = 19
                    End If
                Loop Until (Ctr = 20 OrElse Player.OwnedWeapons(CType(LV, ShotPattern)))
                If Player.OwnedWeapons(CType(LV, ShotPattern)) Then
                    Player.WeaponType = CType(LV, ShotPattern)
                End If
                If CusLines <= 2 Then
                    CusText(1) = Player.BulletType.ToString & " WITH " & Player.WeaponType.ToString & " --------------------------------------------------------------"
                    CusLines = 2
                    EquipTimer = EQUIPMAXTIMER
                End If
            ElseIf e.KeyCode = WeaponNextKey Then
                LV = CType(Player.WeaponType, Integer)
                Do
                    Ctr += 1
                    LV += 1
                    If LV = 20 Then
                        LV = 0
                    End If
                Loop Until Ctr = 20 OrElse Player.OwnedWeapons(CType(LV, ShotPattern))
                If Player.OwnedWeapons(CType(LV, ShotPattern)) Then
                    Player.WeaponType = CType(LV, ShotPattern)
                End If
                If CusLines <= 2 Then
                    CusText(1) = Player.BulletType.ToString & " WITH " & Player.WeaponType.ToString & " --------------------------------------------------------------"
                    CusLines = 2
                    EquipTimer = EQUIPMAXTIMER
                End If
            End If
            If e.KeyCode = BulletPrevKey Then
                LV = CType(Player.BulletType, Integer)
                Do
                    Ctr += 1
                    LV -= 1
                    If LV = -1 Then
                        LV = 9
                    End If
                Loop Until Ctr = 10 OrElse Player.OwnedBullets(CType(LV, ShotTypes))
                If Player.OwnedBullets(CType(LV, ShotTypes)) Then
                    Player.BulletType = CType(LV, ShotTypes)
                End If
                If CusLines <= 2 Then
                    CusText(1) = Player.BulletType.ToString & " WITH " & Player.WeaponType.ToString & " --------------------------------------------------------------"
                    CusLines = 2
                    EquipTimer = EQUIPMAXTIMER
                End If
            ElseIf e.KeyCode = BulletNextKey Then
                LV = CType(Player.BulletType, Integer)
                Do
                    Ctr += 1
                    LV += 1
                    If LV = 10 Then
                        LV = 0
                    End If
                Loop Until Ctr = 10 OrElse Player.OwnedBullets(CType(LV, ShotTypes))
                If Player.OwnedBullets(CType(LV, ShotTypes)) Then
                    Player.BulletType = CType(LV, ShotTypes)
                End If
                If CusLines <= 2 Then
                    CusText(1) = Player.BulletType.ToString & " WITH " & Player.WeaponType.ToString & " --------------------------------------------------------------"
                    CusLines = 2
                    EquipTimer = EQUIPMAXTIMER
                End If
            End If

            If e.KeyCode = DownKey Then
                Dwn = True
            ElseIf e.KeyCode = LeftKey Then
                Lft = True
                'ElseIf e.KeyCode = Keys.NumPad2 Then
                '    NDwn = True
                'ElseIf e.KeyCode = Keys.NumPad4 Then
                '    NLft = True
                'ElseIf e.KeyCode = Keys.NumPad6 Then
                '    NRght = True
                'ElseIf e.KeyCode = Keys.NumPad8 Then
                '    NUp = True
            ElseIf e.KeyCode = RightKey Then
                Rght = True
            ElseIf e.KeyCode = UpKey Then
                Up = True
            ElseIf e.KeyCode = MenuKey Then
                MyMode = DisplayMode.PauseMenu
                CusText(0) = "PAUSE MENU"
                CusText(1) = "--------------"
                CusText(2) = "CHANGE EQUIP"
                CusText(3) = "RETURN TO GAME"
                CusText(4) = "GO TO LEVEL"
                CusText(5) = "SAVE THIS GAME"
                CusText(6) = "LOAD NEW GAME"
                CusText(7) = "QUIT"
                Selexion = 0
                'If LevelMajor >= 1 Then
                '    Selexion = (LevelMajor - 1) * 8 + LevelMinor - 1
                '    MyMode = DisplayMode.LevelTransfer
                '    CusText(0) = "SELECT."
                'End If
            ElseIf e.KeyCode = RunKey Then
                GoFast = True
            ElseIf e.KeyCode = JumpKey Then

                If Player.InWater Then
                    Player.IsJumping = True
                    'The jump is initialized.
                    Player.OnSlope = False
                    'Not on a slopy surface anymore.
                    MobileIndex = -1
                    KonveyIndex = -1
                    RTDIndex = -1
                    PNMIndex = -1
                    BreakGlassIndex = -1
                    Player.IsClimbing = False

                    If Dwn Then
                        Player.Veloc = Me.SUBLOWJUMPVELOCITY
                        'Use low jump velocity if down is pressed.
                    ElseIf Up AndAlso Player.DoubleJumpEnable Then
                        Player.Veloc = Me.SUBINITIALVELOCITY * 1.42F
                    Else
                        Player.Veloc = Me.SUBINITIALVELOCITY
                        'Use space bar initial velocity.


                    End If
                ElseIf Player.WKReady AndAlso Player.WallKickEnable Then
                    Player.WKReady = False
                    Player.WKActive = True
                    Player.IsJumping = True
                    Player.OnSlope = False
                    Player.IsLongJumping = False
                    MobileIndex = -1
                    KonveyIndex = -1
                    RTDIndex = -1
                    PNMIndex = -1
                    Player.IsClimbing = False
                    If Dwn Then
                        Player.Veloc = Me.LOWJUMPVELOCITY
                    Else
                        Player.Veloc = Me.SBARINITIALVELOCITY
                    End If
                    Player.Direc = 1 - Player.Direc
                    If BreakGlassIndex >= 0 Then
                        Wall = DirectCast(Q.Glasses(BreakGlassIndex), RectangleF)
                        Q.AddShard(Wall.Left, 0.25F * Wall.Top + 0.75F * Wall.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                        Q.AddShard(Wall.Right, 0.75F * Wall.Top + 0.25F * Wall.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                        Q.AddShard(0.25F * Wall.Left + 0.75F * Wall.Right, Wall.Top, Ri.Next(11) * 0.5F - 2.5F)
                        Q.AddShard(0.75F * Wall.Left + 0.25F * Wall.Right, Wall.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                        Wall = New RectangleF(-1.0F, 1000.0F, 0.0F, 0.0F)
                        Bbx.PlayWave(9, True)
                        Q.Glasses(BreakGlassIndex) = Wall
                        BreakGlassIndex = -1

                    End If
                ElseIf (Not Player.IsJumping) Then
                    Player.IsJumping = True
                    'The jump is initialized.
                    'Bbx.PlayWave(Application.StartupPath & "\Invader1.wav", 1, True)
                    Player.OnSlope = False
                    'Not on a slopy surface anymore.

                    MobileIndex = -1
                    KonveyIndex = -1
                    RTDIndex = -1
                    PNMIndex = -1
                    BreakGlassIndex = -1
                    Player.IsClimbing = False

                    'We are not standing on anything now, because we are jumping.
                    If Dwn AndAlso (Lft OrElse Rght) Then
                        If Player.LongJumpEnable Then
                            Player.IsLongJumping = True
                            Player.Veloc = Me.LOWJUMPVELOCITY
                            'Player.HP -= 1
                        Else
                            Player.Veloc = Me.LOWJUMPVELOCITY
                            'Use space bar initial velocity.
                        End If
                    ElseIf Dwn Then
                        If Player.LongJumpEnable Then
                            Player.IsLongJumping = True
                            Player.Veloc = Me.LOWJUMPVELOCITY
                            'Player.HP -= 1
                        Else
                            Player.Veloc = Me.LOWJUMPVELOCITY
                            'Use space bar initial velocity.
                        End If
                    ElseIf Up AndAlso Player.DoubleJumpEnable Then
                        Player.Veloc = Me.SBARINITIALVELOCITY * 1.42F
                    Else
                        Player.Veloc = Me.SBARINITIALVELOCITY
                        'Use space bar initial velocity.
                    End If


                End If
                'ElseIf e.KeyCode = Keys.Y Then
                '    Player.Loc = Exitdoor
            ElseIf e.KeyCode = UseDoorKey Then
                If Player.Loc.IntersectsWith(Exitdoor) Then
                    If LevelMajor >= 1 Then
                        If Player.Coincount > CoinCount((LevelMajor - 1) * 9 + LevelMinor - 1) Then
                            CoinCount((LevelMajor - 1) * 9 + LevelMinor - 1) = Player.Coincount
                        End If
                        UpdateUnlockedRows()
                    End If
                    ToNextLevel()
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                End If
            End If
        ElseIf MyMode = DisplayMode.Mainscrn Then
            If CtrTwo < 800 Then
                CtrTwo = 800
            End If
            If e.KeyCode = Keys.Up Then
                Selexion -= 1
                If Selexion < 0 Then
                    Selexion = 0
                End If
            ElseIf e.KeyCode = Keys.Down Then
                Selexion += 1
                If Selexion > 3 Then
                    Selexion = 3
                End If
            ElseIf e.KeyCode = Keys.Enter Then
                If Selexion = 3 Then
                    Running = False
                    'End the game loop.
                ElseIf Selexion = 0 OrElse Selexion = 1 OrElse Selexion = 2 Then
                    MyMode = DisplayMode.CloseMainscrn
                    CtrTwo = 2000
                End If
            ElseIf e.KeyCode = Keys.Escape Then
                Running = False
                'End the game loop when Escape is pressed.
            End If
        ElseIf MyMode = DisplayMode.OptionsMenu Then
            If e.KeyCode = Keys.Escape Then
                Running = False
            ElseIf e.KeyCode = Keys.Up Then
                Selexion -= 1
                If Selexion < 0 Then
                    Selexion = 0
                End If
            ElseIf e.KeyCode = Keys.Down Then
                Selexion += 1
                If Selexion > 2 Then
                    Selexion = 2
                End If
            ElseIf e.KeyCode = Keys.Left Then
                If Selexion = 0 Then
                    If Bbx.WaveVol > 0 Then
                        Bbx.SetWaveVolume(Bbx.WaveVol - 1)
                    End If
                ElseIf Selexion = 1 Then
                    If Bbx.MusicVol > 0 Then
                        Bbx.SetMusicVolume(Bbx.MusicVol - 1)
                    End If
                End If
            ElseIf e.KeyCode = Keys.Right Then
                If Selexion = 0 Then
                    If Bbx.WaveVol < 100 Then
                        Bbx.SetWaveVolume(Bbx.WaveVol + 1)
                    End If
                ElseIf Selexion = 1 Then
                    If Bbx.MusicVol < 100 Then
                        Bbx.SetMusicVolume(Bbx.MusicVol + 1)
                    End If
                End If
            ElseIf e.KeyCode = Keys.Enter Then
                If Selexion = 0 Then
                    Bbx.PlayWave(7, True)
                ElseIf Selexion = 2 Then
                    CtrTwo = 800
                    MyMode = DisplayMode.ControlsMenu
                    Selexion = 0
                End If
            ElseIf e.KeyCode = Keys.Back Then
                CtrTwo = 800
                MyMode = DisplayMode.Mainscrn
            End If
        ElseIf MyMode = DisplayMode.ControlsMenu Then

            If e.KeyCode = Keys.Escape Then
                Running = False
            ElseIf ControlSet Then
                Select Case Selexion
                    Case 0
                        UpKey = e.KeyCode
                    Case 1
                        DownKey = e.KeyCode
                    Case 2
                        LeftKey = e.KeyCode
                    Case 3
                        RightKey = e.KeyCode
                    Case 4
                        JumpKey = e.KeyCode
                    Case 5
                        FireKey = e.KeyCode
                    Case 6
                        RunKey = e.KeyCode
                    Case 7
                        MenuKey = e.KeyCode
                    Case 8
                        UseDoorKey = e.KeyCode
                    Case 9
                        WeaponNextKey = e.KeyCode
                    Case 10
                        WeaponPrevKey = e.KeyCode
                    Case 11
                        BulletNextKey = e.KeyCode
                    Case 12
                        BulletPrevKey = e.KeyCode
                End Select
                ControlSet = False
            Else
                If e.KeyCode = Keys.Up Then
                    Selexion -= 1
                    If Selexion < 0 Then
                        Selexion = 0
                    End If
                ElseIf e.KeyCode = Keys.Down Then
                    Selexion += 1
                    If Selexion > 14 Then
                        Selexion = 14
                    End If
                ElseIf e.KeyCode = Keys.Right OrElse e.KeyCode = Keys.Enter Then
                    If Selexion < 13 Then
                        ControlSet = True
                    ElseIf Selexion = 14 Then
                        UpKey = Keys.Up : DownKey = Keys.Down : LeftKey = Keys.Left : RightKey = Keys.Right
                        FireKey = Keys.F : RunKey = Keys.ShiftKey : JumpKey = Keys.Space : MenuKey = Keys.Back : UseDoorKey = Keys.Enter
                        WeaponPrevKey = Keys.Q : WeaponNextKey = Keys.Z : BulletPrevKey = Keys.W : BulletNextKey = Keys.X
                    ElseIf Selexion = 13 Then
                        SaveControls()
                        MyMode = DisplayMode.OptionsMenu
                        Selexion = 2
                    End If
                ElseIf e.KeyCode = Keys.Left OrElse e.KeyCode = Keys.Back Then
                    SaveControls()
                    MyMode = DisplayMode.OptionsMenu
                    Selexion = 2
                End If
            End If
        ElseIf MyMode = DisplayMode.YouDied Then
            If e.KeyCode = Keys.Escape Then
                Running = False
                'End the game loop when Escape is pressed.
            ElseIf e.KeyCode = Keys.Enter Then
                SetLevel()
                Player.HP = 100
                MyMode = DisplayMode.Regular

            End If
        ElseIf MyMode = DisplayMode.LoadScreen Then
            If e.KeyCode = Keys.Down Then
                Selexion += 1
                If Selexion > AllFiles.GetUpperBound(0) Then
                    Selexion = AllFiles.GetUpperBound(0)
                End If
                If Selexion > TopIdx + DISPLAYFILENUMEX Then
                    TopIdx += 1
                    For LV = 0 To DISPLAYFILENUMEX
                        ViewFiles(LV) = System.IO.Path.GetFileName(AllFiles(LV + TopIdx))
                    Next
                End If
            ElseIf e.KeyCode = Keys.Up Then
                Selexion -= 1
                If Selexion < 0 Then
                    Selexion = 0
                End If
                If Selexion < TopIdx Then
                    TopIdx -= 1
                    For LV = 0 To DISPLAYFILENUMEX
                        ViewFiles(LV) = System.IO.Path.GetFileName(AllFiles(LV + TopIdx))
                    Next
                End If
            ElseIf e.KeyCode = Keys.Escape Then
                Running = False
            ElseIf e.KeyCode = Keys.Enter Then
                LoadGame(Application.StartupPath & "\files\" & System.IO.Path.GetFileName(AllFiles(Selexion)))
                LevelMajor = 1
                LevelMinor = 1
                Selexion = 0
                MyMode = DisplayMode.LevelTransfer
                CusText(0) = "SELECT."
                Player.WeaponType = ShotPattern.SingleShot
                Player.BulletType = ShotTypes.ShockWave
            End If
        ElseIf MyMode = DisplayMode.PauseMenu Then
            If e.KeyCode = Keys.Escape Then
                Running = False
            ElseIf e.KeyCode = Keys.Up Then
                Selexion -= 1
                If Selexion < 0 Then
                    Selexion = 0
                End If
            ElseIf e.KeyCode = Keys.Down Then
                Selexion += 1
                If Selexion > 5 Then
                    Selexion = 5
                End If
            ElseIf e.KeyCode = Keys.Enter Then
                If Selexion = 5 Then
                    Running = False
                ElseIf Selexion = 0 Then
                    MyMode = DisplayMode.WeaponSelect
                    Selexion = 0
                ElseIf Selexion = 1 Then
                    MyMode = DisplayMode.Regular
                ElseIf Selexion = 2 Then
                    If LevelMajor >= 1 Then
                        Selexion = (LevelMajor - 1) * 9 + LevelMinor - 1
                        MyMode = DisplayMode.LevelTransfer
                        CusText(0) = "SELECT."
                    End If
                ElseIf Selexion = 3 Then
                    If LevelMajor >= 1 Then
                        Filename = ""
                        MyMode = DisplayMode.SaveScreen
                    End If
                ElseIf Selexion = 4 Then
                    Selexion = 0
                    AllFiles = System.IO.Directory.GetFiles(Application.StartupPath, "*.pft")
                    TopIdx = 0
                    If AllFiles.GetUpperBound(0) < 9 Then
                        For LV = 0 To AllFiles.GetUpperBound(0)
                            ViewFiles(LV) = System.IO.Path.GetFileName(AllFiles(LV))
                        Next
                        For LV = AllFiles.GetUpperBound(0) + 1 To 9
                            ViewFiles(LV) = "--------------"
                        Next
                    Else
                        For LV = 0 To 9
                            ViewFiles(LV) = System.IO.Path.GetFileName(AllFiles(LV + TopIdx))
                        Next
                    End If
                    MyMode = DisplayMode.LoadScreen
                    CtrTwo = 0

                End If
            End If
        ElseIf MyMode = DisplayMode.LevelTransfer Then
            If e.KeyCode = Keys.Escape Then
                Running = False
            ElseIf e.KeyCode = Keys.Enter Then
                If Me.IsLevelClear(ZoneFromSelexionEx(Selexion) + 1, SubsecFromSelexionEx(Selexion) + 1) Then
                    LevelMajor = ZoneFromSelexionEx(Selexion) + 1
                    LevelMinor = SubsecFromSelexionEx(Selexion) + 1
                    SetLevel()
                    MyMode = DisplayMode.LevelIntro
                    CtrTwo = 0
                End If
            ElseIf e.KeyCode = Keys.Up Then
                If Selexion <= 8 Then
                    Selexion += 72
                Else
                    Selexion -= 9
                End If
                If Selexion = 80 Then
                    Selexion = 71
                End If
                If Selexion >= 72 Then
                    If Not RowUnlocked(8) Then
                        Selexion -= 9
                    End If
                End If
                If Selexion Mod 9 = 8 Then
                    If Not RowUnlocked(ZoneFromSelexionEx(Selexion)) Then
                        Selexion -= 10
                    End If
                End If
            ElseIf e.KeyCode = Keys.Left Then
                If Selexion Mod 9 = 0 Then
                    Selexion += 8
                Else
                    Selexion -= 1
                End If
                If Selexion = 80 Then
                    Selexion = 79
                End If
                If Selexion Mod 9 = 8 Then
                    If Not RowUnlocked(ZoneFromSelexionEx(Selexion)) Then
                        Selexion -= 1
                    End If
                End If
            ElseIf e.KeyCode = Keys.Right Then
                If Selexion Mod 9 = 8 Then
                    Selexion -= 8
                Else
                    Selexion += 1
                End If
                If Selexion = 80 Then
                    Selexion = 72
                End If
                If Selexion Mod 9 = 8 Then
                    If Not RowUnlocked(ZoneFromSelexionEx(Selexion)) Then
                        Selexion -= 8
                    End If
                End If
            ElseIf e.KeyCode = Keys.Down Then
                If Selexion >= 72 Then
                    Selexion -= 72
                Else
                    Selexion += 9
                End If
                If Selexion = 80 Then
                    Selexion = 0
                End If
                If Selexion >= 72 Then
                    If Not RowUnlocked(8) Then
                        Selexion -= 72
                    End If
                End If
                If Selexion Mod 9 = 8 Then
                    If Not RowUnlocked(ZoneFromSelexionEx(Selexion)) Then
                        Selexion -= 8
                    End If
                End If
            End If
            CusText(0) = "LEVEL " & ((Selexion \ 9) + 1).ToString() & "-" & ((Selexion Mod 9) + 1).ToString() & "  "
            If Selexion Mod 9 <> 7 Then
                If HasUpgrade(Selexion) Then
                    CusText(0) = CusText(0) & "UPGRADE FOUND "
                Else
                    CusText(0) = CusText(0) & "UPGRADE NOT FOUND "
                End If
                CusText(0) &= CoinCount(Selexion) & "/" & MaxCoinCount(Selexion)
            End If
        ElseIf MyMode = DisplayMode.WeaponSelect Then
            If e.KeyCode = Keys.Left Then
                Selexion -= 1
                If Selexion = 1 Then
                    Player.WeaponType = CType(Player.WeaponType - 10, ShotPattern)
                ElseIf Selexion < 0 Then
                    Selexion = 0
                End If
            ElseIf e.KeyCode = Keys.Right Then
                Selexion += 1
                If Selexion = 2 Then
                    Player.WeaponType = CType(Player.WeaponType + 10, ShotPattern)
                ElseIf Selexion > 2 Then
                    Selexion = 2
                ElseIf Selexion = 1 Then
                    If Player.WeaponType >= 10 Then
                        Player.WeaponType = CType(Player.WeaponType - 10, ShotPattern)
                    End If
                End If
            ElseIf e.KeyCode = Keys.Up Then
                If Selexion = 0 Then
                    LV = Player.BulletType - 1
                    If LV < 0 Then
                        Player.BulletType = 0
                    Else
                        Player.BulletType = CType(LV, ShotTypes)
                    End If
                ElseIf Selexion = 1 Then
                    LV = Player.WeaponType - 1
                    If LV < 0 Then
                        Player.WeaponType = 0
                    Else
                        Player.WeaponType = CType(LV, ShotPattern)
                    End If
                ElseIf Selexion = 2 Then
                    LV = Player.WeaponType - 1
                    If LV < 10 Then
                        Player.WeaponType = CType(10, ShotPattern)
                    Else
                        Player.WeaponType = CType(LV, ShotPattern)
                    End If
                End If
            ElseIf e.KeyCode = Keys.Down Then
                If Selexion = 0 Then
                    LV = Player.BulletType + 1
                    If LV > 9 Then
                        Player.BulletType = CType(9, ShotTypes)
                    Else
                        Player.BulletType = CType(LV, ShotTypes)
                    End If
                ElseIf Selexion = 1 Then
                    LV = Player.WeaponType + 1
                    If LV > 9 Then
                        Player.WeaponType = CType(9, ShotPattern)
                    Else
                        Player.WeaponType = CType(LV, ShotPattern)
                    End If
                ElseIf Selexion = 2 Then
                    LV = Player.WeaponType + 1
                    If LV > 19 Then
                        Player.WeaponType = CType(19, ShotPattern)
                    Else
                        Player.WeaponType = CType(LV, ShotPattern)
                    End If
                End If
            ElseIf e.KeyCode = Keys.Enter Then
                If Not Player.OwnedBullets(Player.BulletType) Then
                    Player.BulletType = ShotTypes.ShockWave
                End If
                If Not Player.OwnedWeapons(Player.WeaponType) Then
                    Player.WeaponType = ShotPattern.SingleShot
                End If
                MyMode = DisplayMode.Regular
            End If
        ElseIf MyMode = DisplayMode.IPIntroscrn OrElse MyMode = DisplayMode.OpenToMainscrn Then
            CtrTwo = 800
            MyMode = DisplayMode.Mainscrn
            Bbx.PlayMusic(Application.StartupPath & "\files\PlatformTour" & Ext)
            NewShot()
            Me.Text = SPLASHCAPTION
        ElseIf MyMode = DisplayMode.VictoryCutscene Then
            If e.KeyCode = Keys.Back Then
                CtrTwo = 0
                LevelMinor = 10
            End If
        ElseIf MyMode = DisplayMode.EndCredits Then
            If e.KeyCode = Keys.Escape Then
                Running = False
            End If
        ElseIf MyMode = DisplayMode.SaveScreen Then
            If (e.KeyCode >= Keys.A AndAlso e.KeyCode <= Keys.Z) OrElse (e.KeyCode >= Keys.NumPad0 AndAlso e.KeyCode <= Keys.NumPad9) Then
                If Filename.Length < 8 Then
                    Filename &= e.KeyCode.ToString()
                    CusText(0) = "TYPE FILENAME :" & Filename
                End If
            ElseIf e.KeyCode = Keys.Back Then
                If Filename.Length <= 0 Then
                    MyMode = DisplayMode.Regular
                Else
                    Filename = Filename.Substring(0, Filename.Length - 1)
                    CusText(0) = "TYPE FILENAME :" & Filename
                End If
            ElseIf e.KeyCode = Keys.Enter Then
                SaveGame(Application.StartupPath & "\files\" & Filename & ".pft")
                MyMode = DisplayMode.Regular
            End If
        End If

    End Sub
    Private Sub FormD_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyUp
        If e.KeyCode = DownKey Then
            Dwn = False
        ElseIf e.KeyCode = LeftKey Then
            Lft = False
            'ElseIf e.KeyCode = Keys.NumPad2 Then
            '    NDwn = False
            'ElseIf e.KeyCode = Keys.NumPad4 Then
            '    NLft = False
            'ElseIf e.KeyCode = Keys.NumPad6 Then
            '    NRght = False
            'ElseIf e.KeyCode = Keys.NumPad8 Then
            '    NUp = False
        ElseIf e.KeyCode = RightKey Then
            Rght = False
        ElseIf e.KeyCode = UpKey Then
            Up = False
        ElseIf e.KeyCode = RunKey Then
            GoFast = False
        ElseIf e.KeyCode = FireKey Then
            If MyMode = DisplayMode.Regular Then
                If Player.OwnedBullets(ShotTypes.ShockWave) Then
                    NewShot()
                End If
            End If
        End If
    End Sub

    Private Sub ToNextLevel()
        LevelMinor += 1
        SetLevel()
    End Sub
    Private Sub SetLevel()
        Dim S As String, LV As Integer
        Upgrade = RectangleF.Empty
        Q.Explosions.Clear()
        Q.Shards.Clear()
        For LV = 0 To Q.AUTOMESSNUM
            Q.AutoMessages(LV) = New AutoMessage(0, 0)
        Next
        Player.Coincount = 0
        Select Case (LevelMajor * 10) + LevelMinor
            Case 1
                S = "\files\L0.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\Vertical" & Ext)
                Q.AutoMessages(0) = New AutoMessage(2800, 1)
                Q.AutoMessages(1) = New AutoMessage(6000, 2)
                Q.AutoMessages(2) = New AutoMessage(10000, 12)
            Case 2, 11
                LevelMajor = 1
                LevelMinor = 1
                S = "\files\L11.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
                Q.AutoMessages(1) = New AutoMessage(2000, 4)
            Case 12
                S = "\files\L12.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
                Q.AutoMessages(0) = New AutoMessage(1400, 5)
            Case 13
                S = "\files\L13.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
            Case 14
                S = "\files\L14.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
            Case 15
                S = "\files\L15.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
            Case 16
                S = "\files\L16.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
            Case 17
                S = "\files\L17.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
                Q.AutoMessages(0) = New AutoMessage(5800, 3)
            Case 18
                S = "\files\b1.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 19
                If RowUnlocked(0) Then
                    S = "\files\L19.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG1" & Ext)
                    CompletedLev((2 - 1) * 8 + 1 - 1) = True
                    Q.AutoMessages(0) = New AutoMessage(0, 8)
                    Q.AutoMessages(1) = New AutoMessage(1500, 9)
                    Q.AutoMessages(2) = New AutoMessage(3800, 10)
                    Q.AutoMessages(3) = New AutoMessage(10500, 11)
                Else
                    LevelMajor = 2
                    LevelMinor = 1
                    S = "\files\L21.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
                End If
            Case 20, 21
                LevelMajor = 2
                LevelMinor = 1
                S = "\files\L21.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
                Q.AutoMessages(0) = New AutoMessage(2200, 6)
            Case 22
                S = "\files\L22.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
            Case 23
                S = "\files\L23.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
            Case 24
                S = "\files\L24.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
            Case 25
                S = "\files\L25.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
            Case 26
                S = "\files\L26.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
            Case 27
                S = "\files\L27.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
            Case 28
                S = "\files\b2.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 29
                If RowUnlocked(1) Then
                    S = "\files\L29.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG2" & Ext)
                    CompletedLev((3 - 1) * 8 + 1 - 1) = True
                Else
                    LevelMajor = 3
                    LevelMinor = 1
                    S = "\files\L31.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
                End If
            Case 30, 31
                LevelMajor = 3
                LevelMinor = 1
                S = "\files\L31.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
                Q.AutoMessages(0) = New AutoMessage(3300, 7)
            Case 32
                S = "\files\L32.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
            Case 33
                S = "\files\L33.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
            Case 34
                S = "\files\L34.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
            Case 35
                S = "\files\L35.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
            Case 36
                S = "\files\L36.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
            Case 37
                S = "\files\L37.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
            Case 38
                S = "\files\b3.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 39
                If RowUnlocked(2) Then
                    S = "\files\L39.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG3" & Ext)
                    CompletedLev((4 - 1) * 8 + 1 - 1) = True
                Else
                    LevelMajor = 4
                    LevelMinor = 1
                    S = "\files\L41.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
                End If
            Case 40, 41
                LevelMajor = 4
                LevelMinor = 1
                S = "\files\L41.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
            Case 42
                S = "\files\L42.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
            Case 43
                S = "\files\L43.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
            Case 44
                S = "\files\L44.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
            Case 45
                S = "\files\L45.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
            Case 46
                S = "\files\L46.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
            Case 47
                S = "\files\L47.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
            Case 48
                S = "\files\b4.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 49
                If RowUnlocked(3) Then
                    S = "\files\L49.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG4" & Ext)
                    CompletedLev((5 - 1) * 8 + 1 - 1) = True
                Else
                    LevelMajor = 5
                    LevelMinor = 1
                    S = "\files\L51.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
                End If
            Case 50, 51
                LevelMajor = 5
                LevelMinor = 1
                S = "\files\L51.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
            Case 52
                S = "\files\L52.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
            Case 53
                S = "\files\L53.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
            Case 54
                S = "\files\L54.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
            Case 55
                S = "\files\L55.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
            Case 56
                S = "\files\L56.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
            Case 57
                S = "\files\L57.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
            Case 58
                S = "\files\b5.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 59
                If RowUnlocked(4) Then
                    S = "\files\L59.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG5" & Ext)
                    CompletedLev((6 - 1) * 8 + 1 - 1) = True
                Else
                    LevelMajor = 6
                    LevelMinor = 1
                    S = "\files\L61.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
                End If
            Case 60, 61
                LevelMajor = 6
                LevelMinor = 1
                S = "\files\L61.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
            Case 62
                S = "\files\L62.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
            Case 63
                S = "\files\L63.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
            Case 64
                S = "\files\L64.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
            Case 65
                S = "\files\L65.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
            Case 66
                S = "\files\L66.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
            Case 67
                S = "\files\L67.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
            Case 68
                S = "\files\b6.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 69
                If RowUnlocked(5) Then
                    S = "\files\L69.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG6" & Ext)
                    CompletedLev((7 - 1) * 8 + 1 - 1) = True
                Else
                    LevelMajor = 7
                    LevelMinor = 1
                    S = "\files\L71.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
                End If
            Case 70, 71
                LevelMajor = 7
                LevelMinor = 1
                S = "\files\L71.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
            Case 72
                S = "\files\L72.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
            Case 73
                S = "\files\L73.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
            Case 74
                S = "\files\L74.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
            Case 75
                S = "\files\L75.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
            Case 76
                S = "\files\L76.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
            Case 77
                S = "\files\L77.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
            Case 78
                S = "\files\b7.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 79
                If RowUnlocked(6) Then
                    S = "\files\L79.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG7" & Ext)
                    CompletedLev((8 - 1) * 8 + 1 - 1) = True
                Else
                    LevelMajor = 8
                    LevelMinor = 1
                    S = "\files\L81.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
                End If
            Case 80, 81
                LevelMajor = 8
                LevelMinor = 1
                S = "\files\L81.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
            Case 82
                S = "\files\L82.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
            Case 83
                S = "\files\L83.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
            Case 84
                S = "\files\L84.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
            Case 85
                S = "\files\L85.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
            Case 86
                S = "\files\L86.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
            Case 87
                S = "\files\L87.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
                CompletedLev((8 - 1) * 9 + 1 - 1) = True
            Case 88
                S = "\files\b8.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\TR" & Ext)
            Case 89
                If RowUnlocked(7) Then
                    S = "\files\L89.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
                Else
                    LevelMajor = 8
                    LevelMinor = 1
                    S = "\files\L81.dat"
                    Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)
                End If
            Case 90, 91
                LevelMajor = 8
                LevelMinor = 1
                S = "\files\L81.dat"
                Bbx.PlayMusic(Application.StartupPath & "\files\BG8" & Ext)

        End Select
        CusText(0) = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  "
        CusLines = 1
        CurrentLevelFile = Application.StartupPath & S
        If LevelMajor > 0 Then
            CompletedLev((LevelMajor - 1) * 8 + LevelMinor - 1) = True
        End If
        LoadLevel(CurrentLevelFile)
        If LevelMinor = 8 Then
            If LevelMajor = 1 Then
                Q.Monsters.Add(New BossOne(632.0F, 520.0F))
            ElseIf LevelMajor = 2 Then
                Q.Monsters.Add(New BossTwo(1400.0F, 440.0F))
            ElseIf LevelMajor = 3 Then
                Q.Monsters.Add(New BossThree(1400.0F, 168.0F))
            ElseIf LevelMajor = 4 Then
                Q.Monsters.Add(New BossFour(1168.0F, 160.0F))
            ElseIf LevelMajor = 5 Then
                Q.Monsters.Add(New BossFive(1204.0F, -224.0F, Ri))
            ElseIf LevelMajor = 6 Then
                Q.Monsters.Add(New BossSix(144.0F, 176.0F))
            ElseIf LevelMajor = 7 Then
                Q.Monsters.Add(New BossSeven(580.0F, 384.0F, Ri))
            ElseIf LevelMajor = 8 Then
                Q.Monsters.Add(New PhaseOne(1686.0F, 448.0F))
            End If
        End If
        For LV = 0 To Player.Shots.Length - 1
            Player.Shots(LV) = New SelfBullet(0, 0, 0, 0, 0, 0, 0)
        Next
        NewLevelInit()
        If LevelMajor >= 1 Then
            If HasUpgrade(LevelMinor - 1 + 9 * (LevelMajor - 1)) Then
                Upgrade = RectangleF.Empty
            End If
        End If
        Player.HP = 100
    End Sub
    Public Sub Movement()
        Dim dX, dY, dZ As Single
        Dim LV, Idx As Integer, Sh As Single
        Dim OldPL As RectangleF = Player.Loc  'Where the player was before we called this sub.
        Dim Vec As Vector3
        Dim MW As IWallmove
        Dim OnPlatform As Boolean
        Dim TouchingWall As Boolean
        Static ReturnTouch As Boolean         'This is true when the player has touched a wall
        Dim RV As AutoSuiteReturn
        Dim tempHP, tempMHP As Integer

        If Player.IsJumping Then
            BreakGlassIndex = -1
        End If

        '-------------------------
        '|       R I G H T       |
        '-------------------------
        '|  F A S T  |  S L O W  |
        '-------------------------
        '|ICE|WTR|n/a|ICE|WTR|n/a|
        '-------------------------
        '| 3 |1.5| 3 | 2 |0.5| 2 |
        '-------------------------

        '================================================================================================ Horizontal motion is handled first.

        Player.WKReady = False

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process wall kicks - when wall kicking, player cannot move until half of the wallkick trajectory is done.
        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ This is to prevent wall climbing.
        If Player.WKActive Then
            If Player.Direc = 1 Then
                Player.Loc.Offset(FASTMOVESPEED, 0)
                AdjustPlayerHeight(OldPL)
                RightMovementChecks()
            Else
                Player.Loc.Offset(-FASTMOVESPEED, 0)
                AdjustPlayerHeight(OldPL)
                LeftMovementChecks()
            End If
            If Player.Veloc >= 0 Then
                Player.WKActive = False
            End If

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process long jumps - when long jumping, player moves forward at enhanced rate with little vertical speed.
        ElseIf Player.IsLongJumping Then
            If Player.Direc = 1 Then
                If Player.InWater Then
                    Player.Loc.Offset(1.75F * MOVESPEED, 0)
                Else
                    Player.Loc.Offset(3.5F * MOVESPEED, 0)
                End If
                AdjustPlayerHeight(OldPL)
                RightMovementChecks()
            ElseIf Player.Direc = 0 Then
                If Player.InWater Then
                    Player.Loc.Offset(-1.75F * MOVESPEED, 0)
                Else
                    Player.Loc.Offset(-3.5F * MOVESPEED, 0)
                End If
                AdjustPlayerHeight(OldPL)
                LeftMovementChecks()
            End If

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process moving right, which can be done at any time.  If on ladder, rope, player falls off.  Shift makes
            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ the player move faster, and ice will cause player to accelerate to the right.  Player will also look right.
        ElseIf Rght Then
            If GoFast Then
                If Player.OnIce Then
                    If Player.IceStopEnable AndAlso Player.HorizontVeloc < 0 Then
                        Player.HorizontVeloc = ICEUPSPEED
                    Else
                        Player.HorizontVeloc += ICEUPSPEED
                    End If
                    If Player.HorizontVeloc > ICEMAXSPEED Then
                        Player.HorizontVeloc = ICEMAXSPEED
                    End If
                ElseIf Player.InWater Then
                    Player.Loc.Offset(SUBFASTMOVESPEED, 0)
                    'Halfspeed underwater.
                Else
                    Player.Loc.Offset(FASTMOVESPEED, 0)
                    'Runs to the right.
                End If
            Else
                If Player.OnIce Then
                    If Player.IceStopEnable AndAlso Player.HorizontVeloc < 0 Then
                        Player.HorizontVeloc = ICEUPSPEED
                    Else
                        Player.HorizontVeloc += ICEUPSPEED
                    End If
                    If Player.HorizontVeloc > ICEMAXSPEED Then
                        Player.HorizontVeloc = ICEMAXSPEED
                    End If
                ElseIf Player.InWater Then
                    Player.Loc.Offset(SUBMOVESPEED, 0)
                    'Halfspeed underwater.
                Else
                    Player.Loc.Offset(MOVESPEED, 0)
                    'Move player right.
                End If

            End If
            Player.Direc = 1   'Look right.
            Player.AnimCycler = (Player.AnimCycler + 1) And EFFWALKINGFRAMECOUNTEX   'Make player animate.

            AdjustPlayerHeight(OldPL)
            RightMovementChecks()

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process movement to the left.
        ElseIf Lft Then
            If GoFast Then
                If Player.OnIce Then
                    If Player.IceStopEnable AndAlso Player.HorizontVeloc > 0 Then
                        Player.HorizontVeloc = -ICEUPSPEED
                    Else
                        Player.HorizontVeloc -= ICEUPSPEED
                    End If
                    If -Player.HorizontVeloc > ICEMAXSPEED Then
                        Player.HorizontVeloc = -ICEMAXSPEED
                    End If
                ElseIf Player.InWater Then
                    Player.Loc.Offset(-SUBFASTMOVESPEED, 0)
                    'Halfspeed underwater.
                Else
                    Player.Loc.Offset(-FASTMOVESPEED, 0)
                    'When Shift is pressed the player runs.
                    'Runs to the left.
                End If
            Else
                If Player.OnIce Then
                    If Player.IceStopEnable AndAlso Player.HorizontVeloc > 0 Then
                        Player.HorizontVeloc = -ICEUPSPEED
                    Else
                        Player.HorizontVeloc -= ICEUPSPEED
                    End If
                    If -Player.HorizontVeloc > ICEMAXSPEED Then
                        Player.HorizontVeloc = -ICEMAXSPEED
                    End If
                ElseIf Player.InWater Then
                    Player.Loc.Offset(-SUBMOVESPEED, 0)
                    'Halfspeed underwater
                Else
                    Player.Loc.Offset(-MOVESPEED, 0)
                    'Move player left.
                End If

            End If
            Player.Direc = 0   'Look left.
            Player.AnimCycler = (Player.AnimCycler + 1) And EFFWALKINGFRAMECOUNTEX

            AdjustPlayerHeight(OldPL)
            LeftMovementChecks()

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Stop animation if player is not moving.  Apply ice brake upgrade.
        Else
            If Not Up AndAlso Not Dwn Then
                Player.AnimCycler = 0
            End If
            If Player.OnIce AndAlso Player.IceStopEnable Then
                Player.HorizontVeloc = 0
            End If
        End If
        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process player ice sliding.
        If Player.OnIce Then
            Player.Loc.Offset(Player.HorizontVeloc, 0)  'Slide to some direction.
            If Player.HorizontVeloc < 0 Then
                LeftMovementChecks()   'Check if we went too far left while sliding.
            ElseIf Player.HorizontVeloc > 0 Then
                RightMovementChecks()  'Check if we went too far right while sliding.
            End If
            If Not Lft OrElse Not Rght Then
                Player.AnimCycler = (Player.AnimCycler + 1) Mod WALKINGFRAMECOUNT
            End If
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process movement up or down.  I suppose that this could be moved to the vertical section.
        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ but it does not interfere with any other method (jumping, walking slopes, riding lifts, etc.) and is self-contained.
        If Up Then
            If Player.IsClimbing Then
                Player.Loc.Offset(0, -MOVESPEED)
                Player.Veloc = -MOVESPEED
                If Player.Loc.Bottom < Player.LH Then
                    Player.Loc.Offset(0, Player.LH - Player.Loc.Bottom)
                End If
                Player.AnimCycler = (Player.AnimCycler + 1) And EFFCLIMBINGFRAMECOUNTEX   'Make player animate.
            Else
                For Each Ladder In Q.Ropes
                    If Player.Loc.IntersectsWith(Ladder) Then
                        Player.Loc.X = Ladder.Left + (Ladder.Width / 2) - (CHARWIDTH \ 2)
                        Player.IsClimbing = True
                        Player.LB = Ladder.Bottom
                        Player.LH = Ladder.Top
                        Player.OnIce = False
                        Player.OnSlope = False
                        Player.IsJumping = False
                        Player.IsLongJumping = False
                    End If
                Next
                For Each Ladder In Q.Ladders
                    If Player.Loc.IntersectsWith(Ladder) Then
                        If Player.Loc.Left < Ladder.Left Then
                            Player.Loc.X = Ladder.Left
                        ElseIf Player.Loc.Right > Ladder.Right Then
                            Player.Loc.X = Ladder.Right - Player.Loc.Width
                        End If
                        Player.IsClimbing = True
                        Player.LB = Ladder.Bottom
                        Player.LH = Ladder.Top
                        Player.OnIce = False
                        Player.OnSlope = False
                        Player.IsJumping = False
                        Player.IsLongJumping = False
                    End If
                Next
            End If

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process climbing down.
        ElseIf Dwn Then
            If Player.IsClimbing Then
                Player.Loc.Offset(0, MOVESPEED)
                Player.Veloc = MOVESPEED
                If Player.Loc.Bottom > Player.LB Then
                    Player.Loc.Offset(0, Player.LB - Player.Loc.Bottom)
                End If
                Player.AnimCycler = (Player.AnimCycler + 1) And EFFCLIMBINGFRAMECOUNTEX   'Make player animate.
            Else
                Player.Loc.Y += 4   'Allows player to grab ladders from the floor beneath.
                For Each Ladder In Q.Ropes
                    If Player.Loc.IntersectsWith(Ladder) Then
                        Player.Loc.X = Ladder.Left + (Ladder.Width / 2) - (CHARWIDTH \ 2)
                        Player.IsClimbing = True
                        Player.LB = Ladder.Bottom
                        Player.LH = Ladder.Top
                        Player.OnIce = False
                        Player.OnSlope = False
                        Player.IsJumping = False
                        Player.IsLongJumping = False
                    End If

                Next
                For Each Ladder In Q.Ladders
                    If Player.Loc.IntersectsWith(Ladder) Then
                        If Player.Loc.Left < Ladder.Left Then
                            Player.Loc.X = Ladder.Left
                        ElseIf Player.Loc.Right > Ladder.Right Then
                            Player.Loc.X = Ladder.Right - Player.Loc.Width
                        End If
                        Player.IsClimbing = True
                        Player.LB = Ladder.Bottom
                        Player.LH = Ladder.Top
                        Player.OnIce = False
                        Player.OnSlope = False
                        Player.IsJumping = False
                        Player.IsLongJumping = False
                    End If
                Next
                Player.Loc.Y -= 4
            End If
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Move player left or right according to the moving platform or conveyor belt that they are on.
        If MobileIndex >= 0 Then
            MobileWall = DirectCast(Q.Mobilewalls(MobileIndex), IWallmove)
            Player.LeftEnd += MobileWall.dX
            Player.RightEnd += MobileWall.dX
            Player.Loc.Offset(MobileWall.dX, 0)
            dX = MobileWall.dX
            dY = MobileWall.dY
            If dX > 0 Then 'Move right, check for right movement collision as priority.
                RightMovementChecks()
                LeftMovementChecks()
            Else
                LeftMovementChecks()
                RightMovementChecks()
            End If
        End If
        If KonveyIndex >= 0 Then
            Datablk = DirectCast(Q.Conveyors(KonveyIndex), DataBlock)
            Player.Loc.Offset(Datablk.Data, 0)
            dX = Datablk.Data

            If dX < 0 Then
                LeftMovementChecks()
            ElseIf dX > 0 Then
                RightMovementChecks()
            End If
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Move the moving wall and push/smush the player if he is in the way.
        Dim R As Integer
        For R = 0 To Q.Mobilewalls.Count - 1

            MobileWall = DirectCast(Q.Mobilewalls(R), IWallmove)  'We have to use DirectCast since arraylist contains structures and we are changing the property values.

            If MobileWall.dX < 0 Then   'The wall is moving left.
                MobileWall.MoveHorizontal() 'This does horizontal movement for the wall.
                If MobileWall.Loc.IntersectsWith(Player.Loc) Then
                    If Not TouchingWall Then 'Move the player to the left side of this wall.
                        Player.Loc.Offset(MobileWall.Loc.Left - Player.Loc.Right, 0)
                        TouchingWall = True
                        If Player.Loc.Right < Player.LeftEnd AndAlso Not Player.IsJumping Then
                            Player.IsJumping = True 'We are not really jumping, but we are falling.
                            MobileIndex = -1 'We are not standing on anything now, because we are jumping.
                            Player.Veloc = 0 'Since we are falling from rest, the initial velocity will be zero.
                        End If
                    Else
                        Player.HP = 0 'The player has been smushed.  Lose a life.
                    End If
                End If

            ElseIf MobileWall.dX > 0 Then 'The wall is moving right.
                MobileWall.MoveHorizontal() 'This does all of the movement for the wall.
                If MobileWall.Loc.IntersectsWith(Player.Loc) Then
                    If Not TouchingWall Then 'Move the player to the right side of this wall.
                        Player.Loc.Offset(MobileWall.Loc.Right - Player.Loc.Left, 0)
                        TouchingWall = True
                        If Player.Loc.Left > Player.RightEnd AndAlso Not Player.IsJumping Then
                            Player.IsJumping = True 'We are falling.
                            MobileIndex = -1 'We are not standing on anything now, because we are jumping.
                            Player.Veloc = 0 'A fall from rest.
                        End If
                    Else
                        Player.HP = 0 'The player has been smushed.  Lose a life.
                    End If
                End If
            Else
                MobileWall.MoveHorizontal() 'This does all of the movement for the wall.
            End If
            Q.Mobilewalls(R) = MobileWall
        Next


        '================================================================================================ Vertical motion is handled afterwards.
        If MobileIndex >= 0 Then
            Player.Loc.Offset(0, dY)

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Moves the player vertically based on the wall that he is on.
            For Each Wall In Q.Walls
                If Player.Loc.IntersectsWith(Wall) Then
                    If dY < 0 Then 'The player has been smushed.  Lose a life.
                        Player.HP = 0
                    Else 'The player has 'landed' on this wall.
                        Player.IsJumping = False
                        Player.IsLongJumping = False 'Stop the jump.
                        Player.OnIce = False : Player.HorizontVeloc = 0 'No longer on ice.
                        Player.LeftEnd = Wall.Left : Player.RightEnd = Wall.Right 'Set the endpoints.
                        Player.AnimCycler = 0 'Reset the cycler.
                        Player.Loc.Offset(0, Wall.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                        MobileIndex = -1
                        KonveyIndex = -1
                        RTDIndex = -1
                        PNMIndex = -1
                    End If
                End If
            Next    'Why is this only checking walls and mobile walls instead of extrawalls, groundwalls, , etc.?

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Move the moving walls and check for player interaction.
            'I used Q.mobilewalls.indexof(MW) when it's clearly R.
            For R = 0 To Q.Mobilewalls.Count - 1
                MW = DirectCast(Q.Mobilewalls(R), IWallmove)

                If MW.dY > dY Then  'Smushed between two moving walls.
                    MW.MoveVertical()
                    If Player.Loc.IntersectsWith(MW.Loc) AndAlso R <> MobileIndex Then
                        Player.HP = 0 'The player has been smushed.  Lose a life.
                    End If
                Else
                    MW.MoveVertical()
                    If Player.Loc.IntersectsWith(MW.Loc) Then 'Deleted  AndAlso R <> MobileIndex from here
                        Player.IsJumping = False 'If player is falling, then the player will land on the wall.
                        Player.IsLongJumping = False 'Stop the jump.
                        Player.OnIce = False : Player.HorizontVeloc = 0 : Player.OnSlope = False 'No longer on ice.
                        Player.LeftEnd = MW.Loc.Left : Player.RightEnd = MW.Loc.Right 'Set the endpoints.
                        MobileIndex = Q.Mobilewalls.IndexOf(MW) 'Set the index of this mobile wall in the array.
                        Player.AnimCycler = 0 'Reset the cycler.
                        Player.Loc.Offset(0, MW.Loc.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    End If
                End If
                Q.Mobilewalls(R) = MW
            Next

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Just move walls if player is not on a wall.
        Else
            For R = 0 To Q.Mobilewalls.Count - 1
                MW = DirectCast(Q.Mobilewalls(R), IWallmove)
                MW.MoveVertical()
                If Player.Loc.IntersectsWith(MW.Loc) Then
                    If MW.dY > 0 Then
                        If Not Player.IsJumping Then
                            Player.HP = 0 'The player has been smushed.  Lose a life.
                        End If
                    Else
                        If Player.IsJumping AndAlso Player.Veloc > Me.SBARINITIALVELOCITY Then
                            'Lifts the player off of his current wall and make him 'land' on this wall.
                            Player.IsJumping = False
                            Player.IsLongJumping = False 'Stop the jump.
                            Bbx.PlayWave(5, Player.Veloc > 5)
                            Player.OnIce = False : Player.HorizontVeloc = 0 : Player.OnSlope = False 'No longer on ice.
                            Player.LeftEnd = MW.Loc.Left : Player.RightEnd = MW.Loc.Right 'Set the endpoints.
                            MobileIndex = Q.Mobilewalls.IndexOf(MW) 'Set the index of this mobile wall in the array.
                            Player.AnimCycler = 0 'Reset the cycler.
                            Player.Loc.Offset(0, MW.Loc.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                        End If
                    End If
                End If
                Q.Mobilewalls(R) = MW
            Next
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Why was process player ice sliding here?


        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Check for getting smushed due to movement from a moving wall.
        If TouchingWall Then
            For Each Wall In Q.Walls
                If Player.Loc.IntersectsWith(Wall) Then
                    Player.HP = 0 'The player has been smushed.  Lose a life.
                End If
            Next
            For Each Wall In Q.Extrawalls
                If Player.Loc.IntersectsWith(Wall) Then
                    Player.HP = 0
                End If
            Next
            For Each Wall In Q.Groundwalls
                If Player.Loc.IntersectsWith(Wall) Then
                    Player.HP = 0
                End If
            Next
            For Each Wall In Q.CrossStays
                If Player.Loc.IntersectsWith(Wall) Then
                    Player.HP = 0
                End If
            Next
            For Each Wall In Q.Crates
                If Player.Loc.IntersectsWith(Wall) Then
                    Player.HP = 0
                End If
            Next
            For Each Wall In Q.EyeBeams
                If Player.Loc.IntersectsWith(Wall) Then
                    Player.HP = 0
                End If
            Next
            For Each Datablk In Q.Conveyors
                If Player.Loc.IntersectsWith(Datablk.Loc) Then
                    Player.HP = 0
                End If
            Next
        End If
        TouchingWall = False 'Reset this boolean for use in vertical checks.


        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Move player vertically according to jumps.
        If Player.IsJumping OrElse Player.IsClimbing Then
            If Player.IsJumping Then
                If Player.InWater Then
                    Player.Loc.Offset(0, Player.Veloc + SUBGRAVITY) 'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
                    Player.Veloc += SUBGRAVITY 'dV = (PlayerVeloc +=) A (GRAVITY) ; 'Moves the character according to its velocity, gravity, and location.
                    If Player.Veloc >= SUBTERMINAL Then  'We're in TV.
                        Player.Veloc = SUBTERMINAL  'Limit the velocity.
                    End If
                Else
                    If LevelMajor = 8 Then
                        Player.Loc.Offset(0, Player.Veloc + LOWGRAVITY)
                        Player.Veloc += LOWGRAVITY
                    Else
                        Player.Loc.Offset(0, Player.Veloc + GRAVITY) 'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
                        Player.Veloc += GRAVITY 'Moves the character according to its velocity, gravity, and location.
                    End If
                    If Player.Veloc >= TERMINALVELOCITY Then  'We're in TV.
                        Player.Veloc = TERMINALVELOCITY  'Limit the velocity.
                    End If
                End If
            End If

            Player.Loc.X = Convert.ToSingle(Math.Round(Player.Loc.Left * 1000.0F, 0)) / 1000.0F
            Player.Loc.Y = Convert.ToSingle(Math.Round(Player.Loc.Top * 1000.0F, 0)) / 1000.0F 'rounding prevents clipping through platforms and small surfaces

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Determine if player lands on object, bangs head, or is being crushed by object.
            If Player.IsJumping Then

                RV = LevelSuite.PlatformJump(Q.Platforms, Player.Loc, Player.Veloc, GRAVITYTH)
                If RV.RetState > SuiteReturnType.NoReturn Then
                    If RV.RetState = SuiteReturnType.Landing Then 'The player has landed on this platform.  He needs to be updated. (See how many advantages there are to using a Rectangle?)
                        Player.IsJumping = False
                        Player.IsLongJumping = False 'Hit the ground, we are not jumping.
                        Bbx.PlayWave(5, Player.Veloc > 5)
                        Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the current end points.
                        Player.AnimCycler = 0 'Reset cycler.
                        Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                        Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Move the player onto this platform.
                        OnPlatform = True 'This is used to fix other problems with landing on platforms and colliding with walls.
                    End If
                End If
                RV = LevelSuite.TrapPlatformJump(Q.TrapPlatforms, Player.Loc, Player.Veloc, GRAVITYTH)
                If RV.RetState = SuiteReturnType.Landing Then
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Hit the ground, we are not jumping.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.LeftEnd = RV.TrpValue.R.Left : Player.RightEnd = RV.TrpValue.R.Right 'Set the endpoints for the platform.
                    Player.AnimCycler = 0 'Reset cycler.
                    Player.Loc.Offset(0, RV.Extra - Player.Loc.Bottom) 'Move the player onto this platform.
                    Player.OnSlope = True 'Is player walking on something sloped?
                    Player.LH = RV.TrpValue.LeftHeight
                    Player.RH = RV.TrpValue.RightHeight 'The interpolation will occur in the character movement places, after the movement checks.
                    OnPlatform = True 'This is used to fix other problems with landing on platforms and colliding with walls.
                End If

                RV = LevelSuite.PlatformJump(Q.Ices, Player.Loc, Player.Veloc, GRAVITYTH)
                If RV.RetState > SuiteReturnType.NoReturn Then
                    If RV.RetState = SuiteReturnType.Landing Then 'The player has landed on this platform.  He needs to be updated.
                        Player.IsJumping = False
                        Player.IsLongJumping = False 'Hit the ground, we are not jumping.
                        Bbx.PlayWave(5, Player.Veloc > 5)
                        Player.OnIce = True 'Just landed on some ice.
                        Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints for the platform.
                        Player.AnimCycler = 0 'Reset cycler.
                        If Lft Then
                            If GoFast Then
                                Player.HorizontVeloc = -FASTMOVESPEED
                            Else
                                Player.HorizontVeloc = -MOVESPEED
                            End If
                        ElseIf Rght Then
                            If GoFast Then
                                Player.HorizontVeloc = FASTMOVESPEED
                            Else
                                Player.HorizontVeloc = MOVESPEED
                            End If
                        End If 'The above lines set the initial movement speed when the player initially lands on the ice.
                        Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Move the player onto this platform.
                    End If
                End If
            End If

            RV = LevelSuite.WallJump(Q.Walls, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player fall.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    If ReturnTouch Then
                        Player.HP = 0  'This eliminates continuous jumping upward into movingwalls.
                    End If
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                    TouchingWall = True
                End If
            End If

            RV = LevelSuite.WallJump(Q.Extrawalls, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player fall.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    If ReturnTouch Then
                        Player.HP = 0  'This eliminates continuous jumping upward into movingwalls.
                    End If
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                    TouchingWall = True
                End If
            End If
            RV = LevelSuite.WallJump(Q.Groundwalls, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    If ReturnTouch Then
                        Player.HP = 0  'This eliminates continuous jumping upward into movingwalls.
                    End If
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                    TouchingWall = True
                End If
            End If
            RV = LevelSuite.WallJump(Q.Crates, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    If ReturnTouch Then
                        Player.HP = 0  'This eliminates continuous jumping upward into movingwalls.
                    End If
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                    TouchingWall = True
                End If
            End If

            RV = LevelSuite.WallJump(Q.EyeBeams, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player fall.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    If ReturnTouch Then
                        Player.HP = 0  'This eliminates continuous jumping upward into movingwalls.
                    End If
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                    TouchingWall = True
                End If
            End If
            RV = LevelSuite.WallJump(Q.CrossStays, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    If ReturnTouch Then
                        Player.HP = 0  'This eliminates continuous jumping upward into movingwalls.
                    End If
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                    TouchingWall = True
                End If
            End If
            RV = LevelSuite.WallJump(Q.Glasses, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player
                    Bbx.PlayWave(10, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    If Player.Veloc >= 15.0F Then
                        Player.Veloc = 0
                        Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom)
                        Bbx.PlayWave(9, True)
                        LV = Q.Glasses.IndexOf(RV.IncValue)
                        Q.AddShard(RV.IncValue.Left, 0.25F * RV.IncValue.Top + 0.75F * RV.IncValue.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                        Q.AddShard(RV.IncValue.Right, 0.75F * RV.IncValue.Top + 0.25F * RV.IncValue.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                        Q.AddShard(0.25F * RV.IncValue.Left + 0.75F * RV.IncValue.Right, RV.IncValue.Top, Ri.Next(11) * 0.5F - 2.5F)
                        Q.AddShard(0.75F * RV.IncValue.Left + 0.25F * RV.IncValue.Right, RV.IncValue.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                        RV.IncValue = New RectangleF(-1.0F, 1000.0F, 0.0F, 0.0F)
                        Q.Glasses(LV) = RV.IncValue
                    Else
                        Player.IsJumping = False
                        Player.IsLongJumping = False 'Stop the jump.
                        Bbx.PlayWave(10, Player.Veloc > 5)
                        Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                        Player.AnimCycler = 0 'Reset the cycler.
                        Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                        Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                        BreakGlassIndex = Q.Glasses.IndexOf(RV.IncValue)
                        If ReturnTouch Then
                            Player.HP = 0  'This eliminates continuous jumping upward into movingwalls.
                        End If
                    End If
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                    TouchingWall = True
                End If


            End If
            Player.Loc.X = Convert.ToSingle(Math.Round(Player.Loc.Left * 1000.0F, 0)) / 1000.0F
            Player.Loc.Y = Convert.ToSingle(Math.Round(Player.Loc.Top * 1000.0F, 0)) / 1000.0F   'rounding prevents clipping through platforms and small surfaces

            ReturnTouch = False

            RV = LevelSuite.MobileWallJump(Q, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player die
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = RV.Extra 'Make the player start falling.
                    Player.HP = 0 'Dead
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    MobileIndex = Convert.ToInt32(RV.Extra) 'Set the index of this mobile wall in the array.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    'For killing the player when banging head on wall while jumping while riding upwards on this lift and clipping through this lift.
                    For Each Wall In Q.Walls
                        If Player.Loc.IntersectsWith(Wall) Then
                            If DirectCast(Q.Mobilewalls(Convert.ToInt32(RV.Extra)), MovingWall).dY < 0 Then
                                Player.HP = 0
                            End If
                        End If
                    Next
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + Math.Abs(RV.Extra)) 'Move the player to under the wall.
                    Player.Veloc = RV.Extra + 1 'Make the player start falling down.
                    If TouchingWall Then
                        Player.HP = 0 'Smushed while jumping?
                    Else
                        ReturnTouch = True
                    End If
                End If
            End If

            RV = LevelSuite.ConveyorJump(Q, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player fall.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    KonveyIndex = Convert.ToInt32(RV.Extra) 'Set the index of this mobile wall in the array.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.OnIce = False : Player.HorizontVeloc = 0 'We are no longer on ice.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                End If
            End If

            RV = LevelSuite.SlopeWallJump(Q.Slopedwalls, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, Slopedwall.R.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Player.IsJumping = False
                    Player.IsLongJumping = False 'Stop the jump.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnIce = False : Player.HorizontVeloc = 0 'No longer on ice.
                    Player.LeftEnd = RV.TrpValue.R.Left : Player.RightEnd = RV.TrpValue.R.Right 'Set the endpoints.
                    Player.AnimCycler = 0 'Reset the cycler.
                    Player.Loc.Offset(0, RV.Extra - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    Player.OnSlope = True 'Is player walking on something sloped?
                    Player.LH = RV.TrpValue.LeftHeight
                    Player.RH = RV.TrpValue.RightHeight 'The interpolation will occur in the character movement places, after the movement checks.
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.TrpValue.R.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.   
                End If
            End If

            RV = LevelSuite.JumpBoxJump(Q.Jumpboards, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True 'Knock the player off the platform and make the player fall
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False 'Not on a slopy surface anymore.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH) 'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
                    Player.Veloc = 0 'Make the player start falling.
                ElseIf RV.RetState = SuiteReturnType.Landing Then 'If player is falling, then the player will land on the wall.
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.Veloc = RV.Extra / 3.2F 'Bounce the player off of this board.
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.
                    Player.LH = RV.IncValue.Left
                    Player.RH = RV.IncValue.Right
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                End If
            End If
            RV = LevelSuite.OpenSwitchJump(Q.RTDs, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH)
                    Player.Veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    Player.IsJumping = False
                    Player.IsLongJumping = False
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnIce = False : Player.HorizontVeloc = 0
                    Player.AnimCycler = 0
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.Veloc = 0
                    RTDIndex = Convert.ToInt32(RV.Extra)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                End If
            End If
            RV = LevelSuite.OpenSwitchJump(Q.Pneumos, Player.Loc, Player.Veloc, OnPlatform)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    Player.IsJumping = True
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnSlope = False
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top + GRAVITYTH)
                    Player.Veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    Player.IsJumping = False
                    Player.IsLongJumping = False
                    Bbx.PlayWave(5, Player.Veloc > 5)
                    Player.OnIce = False : Player.HorizontVeloc = 0
                    Player.AnimCycler = 0
                    Player.Loc.Offset(0, RV.IncValue.Top - Player.Loc.Bottom)
                    Player.LeftEnd = RV.IncValue.Left : Player.RightEnd = RV.IncValue.Right 'Set the endpoints.
                    Player.Veloc = 0
                    PNMIndex = Convert.ToInt32(RV.Extra)  'Useless due to operation of this gate.
                    PNMData = 10 * RV.ExtraTwo
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then 'The player has jumped up into the wall.
                    Player.Loc.Offset(0, RV.IncValue.Bottom - Player.Loc.Top) 'Move the player to under the wall.
                    Player.Veloc = 0 'Make the player start falling down.
                End If
            End If

            '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Check for lifting/smushing the player if the player is not jumping/climbing.
        Else
            For Each MW In Q.Mobilewalls
                If Player.Loc.IntersectsWith(MW.Loc) Then
                    If MW.dY > 0 Then
                        Player.HP = 0 'The player has been smushed.  Lose a life.
                    Else
                        'Lifts the player off of his current wall and make him 'land' on this wall.
                        Player.IsJumping = False
                        Player.IsLongJumping = False 'Stop the jump.
                        Bbx.PlayWave(5, Player.Veloc > 5)
                        Player.OnIce = False : Player.HorizontVeloc = 0 : Player.OnSlope = False 'No longer on ice.
                        Player.LeftEnd = MW.Loc.Left : Player.RightEnd = MW.Loc.Right 'Set the endpoints.
                        MobileIndex = Q.Mobilewalls.IndexOf(MW) 'Set the index of this mobile wall in the array.
                        Player.AnimCycler = 0 'Reset the cycler.
                        Player.Loc.Offset(0, MW.Loc.Top - Player.Loc.Bottom) 'Position the player to stand on this wall.

                    End If
                End If
            Next

        End If

        Dim tool As IMonster, Ex As Explosion, Nt As Negation, Sd As Shard

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process RTD gate opening/shutting/operation via explosives.
        For LV = 0 To Q.RTDs.Count - 1
            OpSw = DirectCast(Q.RTDs(LV), OpenerSwitch)
            If RTDIndex = LV Then
                Sh = OpSw.Data
                OpSw.PowerUp(4.0F)
                Bbx.PlayWave(8, OpSw.Data = 500 AndAlso Sh <> 500)
            Else
                OpSw.Decay(0.25F)
                If Player.Loc.IntersectsWith(OpSw.Gate) AndAlso Not Player.IsJumping Then
                    Player.HP = 0   'Kill player when smushed by gate.
                End If
            End If
            For Each Ex In Q.Explosions
                If OpSw.Switch.IntersectsWith(Ex.Rect) Then
                    OpSw.PowerUp(15.0F)
                    Bbx.PlayWave(8, OpSw.Data = 500)
                End If
            Next
            Q.RTDs(LV) = OpSw
        Next
        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process pneumatic gate.
        If PNMIndex >= 0 Then
            OpSw = DirectCast(Q.Pneumos(PNMIndex), OpenerSwitch)
            OpSw.Setting(PNMData)
            Q.Pneumos(PNMIndex) = OpSw
        End If


        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Confirm player is in water , process player entry into water.
        LV = 0 'Initialize value for use in Do Loop.
        If Player.InWater Then
            Player.InWater = False 'Initialize value - confirm player is in water after every frame.
            Do Until LV = Q.Waters.Count OrElse Player.InWater = True
                Water = DirectCast(Q.Waters.Item(LV), RectangleF)
                Water.Y += HALFHEIGHT
                Water.Height -= HALFHEIGHT 'This adjustment is necessary for walking so that the player doesn't walk to the edge of the water and suddenly be in water.
                If Player.Loc.IntersectsWith(Water) Then
                    Player.InWater = True
                    Player.IsLongJumping = False
                End If
                LV += 1
            Loop
        Else
            Do Until (LV = Q.Waters.Count OrElse Player.InWater = True)
                Water = DirectCast(Q.Waters.Item(LV), RectangleF)
                Water.Y += HALFHEIGHT
                Water.Height -= HALFHEIGHT 'This adjustment is necessary for walking so that the player doesn't walk to the edge of the water and suddenly be in water.
                If Player.Loc.IntersectsWith(Water) Then
                    Player.InWater = True
                    Player.IsLongJumping = False 'Move the player to the edge of this wall.
                    Player.Veloc /= 4.0F 'Wet impulse, we've splashed into the water (play splash sound effect here).
                    Bbx.PlayWave(6, Player.Veloc > 2)
                End If
                LV += 1
            Loop
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Detect if player has collected a coin.
        For LV = 0 To Q.Coins.Count - 1
            Coin = DirectCast(Q.Coins(LV), RectangleF)
            If Coin.IntersectsWith(Player.Loc) Then
                Coin = RectangleF.Empty 'Set this coin to be an empty rectangle; it is collected.
                Player.Coincount += 1
                Bbx.PlayWave(12, True)
                Q.Coins(LV) = Coin
            End If
        Next

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Detect if player has passed an automessage trip point.
        For LV = 0 To Q.AUTOMESSNUM
            If Q.AutoMessages(LV).MessageID > 0 AndAlso Player.Loc.X >= Q.AutoMessages(LV).X Then
                ActiveMessageIndex = Q.AutoMessages(LV).MessageID
                Q.AutoMessages(LV) = New AutoMessage(0, 0)
            End If
        Next
        If ActiveMessageIndex > 0 Then
            Messages(ActiveMessageIndex)
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Determine if player has touched an upgrade block.
        If Upgrade.Width > 0 Then
            If Upgrade.IntersectsWith(Player.Loc) Then
                GetUpgrade()
            End If
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Set position of player and camera.
        Bits(0) = New Vector3(Player.Loc.X + HALFWIDTH, Player.Loc.Y + HALFHEIGHT, 0.0F)

        Camera = New Vector3(Player.Loc.X + HALFWIDTH, Player.Loc.Y + HALFHEIGHT - CAMERAHIGH, CAMERADEPTH)
        FocusPt = New Vector3(Player.Loc.X + HALFWIDTH - 1, Player.Loc.Y + HALFHEIGHT - CAMERADOWN, 0.0F)




        '=========================================================================================================================== Not Really Movement Stuff vvv
        Dim ILV As Integer, Ret As Boolean, RR As RectangleF

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Set player's evasion hitbox.
        If Player.EvadeUp Then
            RR = New RectangleF(Player.Loc.Left + 6, Player.Loc.Top + 6, Player.Loc.Width - 12, Player.Loc.Height - 12) 'dramastically shrink player's hitbox.
        Else
            RR = Player.Loc
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process monster deaths / bosses load the next level / explosions have to be removed from the Q.
        LV = 0
        Do Until LV >= Q.Monsters.Count
            tool = DirectCast(Q.Monsters(LV), IMonster)
            If tool.Dead Then
                If LevelMinor = 8 Then
                    If TypeOf tool Is BossOne Then
                        MyMode = DisplayMode.VictoryCutscene
                        CtrTwo = 0
                    ElseIf TypeOf tool Is BossTwo Then
                        MyMode = DisplayMode.VictoryCutscene
                        CtrTwo = 0
                    ElseIf TypeOf tool Is BossThree Then
                        MyMode = DisplayMode.VictoryCutscene
                        CtrTwo = 0
                    ElseIf TypeOf tool Is BossFour Then
                        MyMode = DisplayMode.VictoryCutscene
                        CtrTwo = 0
                    ElseIf TypeOf tool Is BossFive Then
                        MyMode = DisplayMode.VictoryCutscene
                        CtrTwo = 0
                    ElseIf TypeOf tool Is BossSix Then
                        MyMode = DisplayMode.VictoryCutscene
                        CtrTwo = 0
                    ElseIf TypeOf tool Is BossSeven Then
                        MyMode = DisplayMode.VictoryCutscene
                        CtrTwo = 0
                    ElseIf TypeOf tool Is PhaseOne Then
                        Bbx.PlayMusic(Application.StartupPath & "\files\iktru" & Ext)
                    ElseIf TypeOf tool Is PhaseTwo Then
                        Bbx.PlayMusic(Application.StartupPath & "\files\PlatformTour" & Ext)
                    ElseIf TypeOf tool Is PhaseThree Then
                        Bbx.PlayMusic(Application.StartupPath & "\files\lurkalurk" & Ext)
                    End If
                End If
                If TypeOf tool Is Explosion Then
                    Q.Explosions.Remove(tool)
                ElseIf TypeOf tool Is Negation Then
                    Q.Negations.Remove(tool)
                ElseIf TypeOf tool Is Shard Then
                    Q.Shards.Remove(tool)
                End If
                Q.Monsters.Remove(tool)
                '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Process all living monster's subroutines.
            Else
                tool.Move(Player.Loc, Q)
                LV += 1
                If tool.IntersectsWith(RR) AndAlso Player.TemporarilyInvulnerable = 0 AndAlso Not (TypeOf tool Is SelfBullet) Then
                    If Not (TypeOf tool Is Explosion AndAlso Player.FlameRetardEnable) Then

                        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ damage player, make invulnerable, play sound of pain.
                        If Player.DefenseUp Then
                            Player.HP -= 4
                        Else
                            Player.HP -= 5
                        End If
                        If Player.EvadeUp Then
                            Player.TemporarilyInvulnerable = HIGHINVULN
                        Else
                            Player.TemporarilyInvulnerable = INVULNTIME
                        End If
                        If Tick Mod 2 = 0 Then
                            Bbx.PlayWave(14, True) 'argh
                        Else
                            Bbx.PlayWave(13, True)  'augh
                        End If
                    End If

                    '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Nullify available player shots for 1000 frames.
                    If TypeOf tool Is Negation Then
                        For ILV = 0 To Player.Shots.Length - 1
                            If Not Player.Shots(ILV).Flying Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 4.0F, 0.25F * (Ri.Next(41) - 20), 0, 0.8F, 1000)
                                Player.Shots(ILV).Flying = True
                                Q.Monsters.Add(Player.Shots(ILV))
                            End If
                        Next
                    End If
                End If

                '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Allow bullets to harm monsters, attenuate bullets that interact with monster hitbox (Rect).
                'Below code in else block moved into else block to minimize operations on dead monsters.
                For ILV = 0 To Player.Shots.Length - 1
                    If Player.Shots(ILV).Flying Then
                        If tool.Rect.Contains(Player.Shots(ILV).Loc) Then
                            If TypeOf Player.Shots(ILV) Is SelfAquabullet Then
                                Ret = tool.Harm(Q, ShotTypes.Aquabullet, Player.Shots(ILV).Veloc.X)
                            ElseIf TypeOf Player.Shots(ILV) Is SelfScrewer Then
                                Ret = tool.Harm(Q, ShotTypes.Screwer, Player.Shots(ILV).Veloc.X)
                            ElseIf TypeOf Player.Shots(ILV) Is SelfRubberet Then
                                Ret = tool.Harm(Q, ShotTypes.Rubberet, Player.Shots(ILV).Veloc.X)
                            ElseIf TypeOf Player.Shots(ILV) Is SelfCorkwheel Then
                                Ret = tool.Harm(Q, ShotTypes.Corkwheel, Player.Shots(ILV).Veloc.X)
                            ElseIf TypeOf Player.Shots(ILV) Is SelfForceMod Then
                                Ret = tool.Harm(Q, ShotTypes.ForceMod, Player.Shots(ILV).Veloc.X)
                            ElseIf TypeOf Player.Shots(ILV) Is SelfNapalm Then
                                Ret = tool.Harm(Q, ShotTypes.Napalm, Player.Shots(ILV).Veloc.X)
                            ElseIf TypeOf Player.Shots(ILV) Is SelfStoneBullet Then
                                Ret = tool.Harm(Q, ShotTypes.Stonebullet, Player.Shots(ILV).Veloc.X)
                            ElseIf TypeOf Player.Shots(ILV) Is SelfPlasma Then
                                tool.Harm(Q, ShotTypes.Plasma, Player.Shots(ILV).Veloc.X)
                                Ret = False  'Do not attenuate plasma - it goes through enemies
                            ElseIf TypeOf Player.Shots(ILV) Is SelfPearl Then
                                Ret = tool.Harm(Q, ShotTypes.Pearl, Player.Shots(ILV).Veloc.X)
                            Else
                                Ret = tool.Harm(Q, ShotTypes.ShockWave, Player.Shots(ILV).Veloc.X)
                            End If
                            If Ret Then
                                Player.Shots(ILV).Flying = False
                                Q.Monsters.Remove(Player.Shots(ILV))
                            End If
                        End If

                    End If
                Next

                '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Damage monsters with explosions, negation explosions, and glass shards.
                If Not (TypeOf tool Is Explosion) Then
                    ILV = 0
                    Do Until ILV >= Q.Explosions.Count
                        Ex = DirectCast(Q.Explosions(ILV), Explosion)
                        If Ex.IntersectsWith(tool.Rect) Then
                            tool.Harm(Q, ShotTypes.Napalm, 0.0F)
                        End If
                        ILV += 1
                    Loop
                End If
                If Not (TypeOf tool Is Negation) Then
                    ILV = 0
                    Do Until ILV >= Q.Negations.Count
                        Nt = DirectCast(Q.Negations(ILV), Negation)
                        If Nt.IntersectsWith(tool.Rect) Then
                            tool.Harm(Q, ShotTypes.Plasma, 0.0F)
                        End If
                        ILV += 1
                    Loop
                End If
                If Not (TypeOf tool Is Shard) Then
                    ILV = 0
                    Do Until ILV >= Q.Shards.Count
                        Sd = DirectCast(Q.Shards(ILV), Shard)
                        If Sd.IntersectsWith(tool.Rect) Then
                            tool.Harm(Q, ShotTypes.Rubberet, 0.0F)
                        End If
                        ILV += 1
                    Loop
                End If

                '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Allow stonebullets and forcemodifiers to break glass.

                If (TypeOf tool Is SelfForceMod) OrElse (TypeOf tool Is ForceMod) OrElse (TypeOf tool Is SelfStoneBullet) OrElse (TypeOf tool Is StoneBullet) Then
                    ILV = 0
                    Do Until ILV >= Q.Glasses.Count
                        Wall = DirectCast(Q.Glasses(ILV), RectangleF)
                        If Wall.IntersectsWith(tool.Rect) Then
                            Bbx.PlayWave(9, True)
                            Q.AddShard(Wall.Left, 0.25F * Wall.Top + 0.75F * Wall.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                            Q.AddShard(Wall.Right, 0.75F * Wall.Top + 0.25F * Wall.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                            Q.AddShard(0.25F * Wall.Left + 0.75F * Wall.Right, Wall.Top, Ri.Next(11) * 0.5F - 2.5F)
                            Q.AddShard(0.75F * Wall.Left + 0.25F * Wall.Right, Wall.Bottom, Ri.Next(11) * 0.5F - 2.5F)
                            Wall = New RectangleF(-1.0F, 1000.0F, 0.0F, 0.0F)
                            Q.Glasses(ILV) = Wall
                            If BreakGlassIndex = ILV Then
                                If Not Player.IsJumping Then
                                    Player.IsJumping = True
                                    Player.Veloc = 0
                                End If
                            End If
                        End If
                        ILV += 1
                    Loop
                End If
                If (TypeOf tool Is BossOne) Then
                    tempHP = DirectCast(tool, BossOne).HP
                    tempMHP = 40
                ElseIf (TypeOf tool Is BossTwo) Then
                    tempHP = DirectCast(tool, BossTwo).HP
                    tempMHP = 180
                ElseIf (TypeOf tool Is BossThree) Then
                    tempHP = DirectCast(tool, BossThree).HP
                    tempMHP = 200
                ElseIf (TypeOf tool Is BossFour) Then
                    tempHP = DirectCast(tool, BossFour).HP
                    tempMHP = 400
                ElseIf (TypeOf tool Is BossFive) Then
                    tempHP = DirectCast(tool, BossFive).HP
                    tempMHP = 450
                ElseIf (TypeOf tool Is BossSix) Then
                    tempHP = DirectCast(tool, BossSix).HP
                    tempMHP = 400
                ElseIf (TypeOf tool Is BossSeven) Then
                    tempHP = DirectCast(tool, BossSeven).HP
                    tempMHP = 1440
                ElseIf (TypeOf tool Is PhaseOne) Then
                    tempHP = DirectCast(tool, PhaseOne).HP
                    tempMHP = 150
                ElseIf (TypeOf tool Is PhaseTwo) Then
                    tempHP = DirectCast(tool, PhaseTwo).HP
                    tempMHP = 2450
                ElseIf (TypeOf tool Is PhaseThree) Then
                    tempHP = DirectCast(tool, PhaseThree).HP
                    tempMHP = 400
                End If
            End If

        Loop

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Play all sounds.
        ILV = Q.SonicEvents.Count - 1
        Do Until ILV < 0

            Bbx.PlayWave(CType(Q.SonicEvents(ILV), SonicEventType), True)
            Q.SonicEvents.RemoveAt(ILV)
            ILV -= 1
        Loop

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Reduce player HP by one each frame while unprotected and in lava.
        If Not Player.FlameRetardEnable Then
            For Each Lava In Q.Lavas
                If Lava.IntersectsWith(Player.Loc) Then
                    Player.HP -= 1
                End If
            Next
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Kill player if falling into a pit.
        If Player.Loc.Bottom > Me.MAPHEIGHT Then
            Player.HP = 0
        End If

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Display death prompt.
        If Player.HP <= 0 Then
            MyMode = DisplayMode.YouDied
            Bbx.StopMusic()
            CusText(0) = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%" & " Coin: " & Player.Coincount & "/" & Q.Coins.Count.ToString("0") & " FPS" & FPS.ToString()
            CusText(1) = ""
            CusText(2) = ""
            CusText(3) = ""
            CusText(4) = ""
            CusText(5) = ""
            CusText(6) = ""
            CusText(7) = ""
            CusText(8) = ""
            CusText(9) = ""
            CusText(10) = ""
            CusText(11) = ""
            CusText(12) = ""
            CusText(13) = ""
            CusText(14) = ""
            CusText(15) = ""
            CusText(16) = ""
            CusText(17) = "                    FAILURE!"
            CusText(18) = "PRESS ENTER TO RETRY OR ESC TO QUIT"
            CusLines = 19
            ReturnTouch = False
        Else
            'CusText(0) = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%" & " LOC:" & Player.Loc.X.ToString("0.0") & " " & Player.Loc.Y.ToString("0.0")
            'CusText(0) = "LOC:" & Player.Loc.X.ToString() & " " & Player.Loc.Y.ToString()
            If LevelMinor = 8 Then
                CusText(0) = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%" & " Boss: " & tempHP.ToString("0") & "/" & tempMHP.ToString("0") & " FPS" & FPS.ToString()
            Else
                CusText(0) = "LEVEL " & LevelMajor.ToString() & "-" & LevelMinor.ToString() & "  - - - HEALTH " & Player.HP.ToString("000") & "%" & " Coin: " & Player.Coincount & "/" & Q.Coins.Count.ToString("0") & " FPS" & FPS.ToString()
            End If
            If DisplayActive Then
                DisplayCounter -= 1
                If DisplayCounter <= 0 Then
                    DisplayActive = False
                    If ActiveMessageIndex = 15 Then
                        MyMode = DisplayMode.EndCredits
                        CtrTwo = 0
                    End If
                    ActiveMessageIndex = 0
                End If
            Else
                If EquipTimer = 0 Then
                    CusLines = 1
                Else
                    CusLines = 2
                    EquipTimer -= 1
                End If
            End If
            If Player.TemporarilyInvulnerable > 0 Then
                Player.TemporarilyInvulnerable -= 1
            End If

        End If


        D9.Lights(0).Position = New Vector3(Player.Loc.X, Player.Loc.Y, 0)
    End Sub
    Private Sub NewShot()
        Dim Ctr As Integer
        Dim ILV As Integer = 0, Done As Boolean = False
        Do Until Done OrElse ILV >= Player.Shots.Length
            If Player.WeaponType = ShotPattern.SingleShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Player.BulletType = ShotTypes.ShockWave Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Napalm Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Rubberet Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Screwer Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.ForceMod Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Plasma Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Pearl Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Done = True
                    Player.IsShooting = True
                    Bbx.PlayWave(7, True)
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.HurdledShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 84, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 84, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If

                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 2 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.SplitterShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 24, -Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 24, Player.ShotSpeed, 0, 0, 0.1F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 8, -Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 8, Player.ShotSpeed, 0, 0, -0.1F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 2 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.SpreadShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, -4, 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 4, 0, 0, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 3 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.ParabolicShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Player.BulletType = ShotTypes.ShockWave Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Napalm Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Rubberet Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Screwer Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.ForceMod Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Plasma Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    ElseIf Player.BulletType = ShotTypes.Pearl Then
                        If Player.Direc = 0 Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        Else
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -5.0F, 0, 0.2F, Player.ShotTime)
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Done = True
                    Player.IsShooting = True
                    Bbx.PlayWave(7, True)
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.VerticalShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Player.BulletType = ShotTypes.ShockWave Then
                        Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                        Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Napalm Then
                        Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Rubberet Then
                        Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Screwer Then
                        Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.ForceMod Then
                        Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                        Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                        Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Plasma Then
                        Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    ElseIf Player.BulletType = ShotTypes.Pearl Then
                        Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0, -Player.ShotSpeed, 0, 0, Player.ShotTime)
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Done = True
                    Player.IsShooting = True
                    Bbx.PlayWave(7, True)
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.CartridgeShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 3 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.WideShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 32, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 32, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0, 0, 0, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 3 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.AlphaShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1, 0, -0.125F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1, 0, 0.125F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 2 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.BackhandShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 1, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -1, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 2, 0, -Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -2, 0, Player.ShotSpeed / 32.0F, 0, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 3 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.MeteorShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 32, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 16, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 8, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 8, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 40, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 40, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 16, Player.Loc.Y + 16, -Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 32, Player.Loc.Y + 16, Player.ShotSpeed * 0.7F, Player.ShotSpeed * 0.7F, 0, 0, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 4 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.SprinkleShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 1.5F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed / 3.0F, 5.0F, 0.0F, -0.2F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 3 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.BoomerangShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, 0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, Player.ShotSpeed * 0.38F, -0.25F * 0.92F, -0.25F * 0.38F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, 0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.92F, -Player.ShotSpeed * 0.38F, -0.25F * 0.92F, 0.25F * 0.38F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.25F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, -0.25F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 3 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.FivewayShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 4 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 5 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.VolleyShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 1.0F - Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed - 1.0F, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 4 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.TetradShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 34, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 34, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 42, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 58, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 4 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.AberrationShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 6, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 6, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 26, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 26, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 36, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 36, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 4 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 4, -Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 4, Player.ShotSpeed, 0.0F, 0.0F, 0.5F - Convert.ToSingle(Ri.NextDouble), Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 5 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.SixteenShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 4 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 5 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 6 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 7 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.707F, -Player.ShotSpeed * 0.707F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 8 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 9 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, -Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 10 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 11 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.924F, Player.ShotSpeed * 0.383F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 12 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 13 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, -Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 14 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    ElseIf Ctr = 15 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed * 0.383F, Player.ShotSpeed * 0.924F, 0.0F, 0.0F, Player.ShotTime)
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, 0.0F, -Player.ShotSpeed, 0.0F, 0.0F, Player.ShotTime)
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 16 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.HighstormShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, 1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, -Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, Player.ShotSpeed, -1.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 4 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, 0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 5 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, -0.08F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y - 66, 0.0F, -0.4F, 0.08F, 0.0F, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, -Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 8, Player.Loc.Y + 16, Player.ShotSpeed, 0.0F, 0.0F, 0.0F, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 6 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1
            ElseIf Player.WeaponType = ShotPattern.ClusterShot Then
                If Not Player.Shots(ILV).Flying Then
                    If Ctr = 1 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 18, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 18, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 2 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 18, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 18, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 5 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 2, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 2, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                        Bbx.PlayWave(7, True)
                    ElseIf Ctr = 4 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 2, Player.Loc.Y, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 2, Player.Loc.Y, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                    ElseIf Ctr = 3 Then
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 2, Player.Loc.Y + 32, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X - 2, Player.Loc.Y + 32, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                    Else
                        If Player.BulletType = ShotTypes.ShockWave Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfBullet(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Aquabullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfAquabullet(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Napalm Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfNapalm(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Rubberet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfRubberet(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Screwer Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfScrewer(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.ForceMod Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfForceMod(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Corkwheel Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfCorkwheel(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Stonebullet Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfStoneBullet(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Plasma Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPlasma(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        ElseIf Player.BulletType = ShotTypes.Pearl Then
                            If Player.Direc = 0 Then
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 18, Player.Loc.Y + 16, -Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            Else
                                Player.Shots(ILV) = New SelfPearl(Player.Loc.X + 18, Player.Loc.Y + 16, Player.ShotSpeed, Convert.ToSingle(2.0R * Ri.NextDouble() - 1.0R), 0, 0, Player.ShotTime)
                            End If
                        End If
                    End If
                    Player.Shots(ILV).Flying = True
                    Q.Monsters.Add(Player.Shots(ILV))
                    Ctr += 1
                    If Ctr = 6 Then
                        Done = True
                    End If
                    Player.IsShooting = True
                End If
                ILV += 1

            End If
        Loop
        'Do Until Done OrElse ILV >= Player.Shots.Length
        '   If Player.WeaponType = ShotPattern.SingleShot Then
        '       If Not Player.Shots(ILV).Flying Then
        '           If Ctr = 1 Then
        '               If Player.BulletType = ShotTypes.ShockWave Then

        '           End If
        '           Player.Shots(ILV).Flying = True
        '           Q.Monsters.Add(Player.Shots(ILV))
        '           Ctr += 1
        '           If Ctr = MAX Then
        '               Done = True
        '           End If
        '           Player.IsShooting = True
        '       End If
        '   ILV += 1
        '   End If
        'Loop


    End Sub
    Private Sub GetUpgrade()
        Dim LV, Idx As Integer
        Idx = (LevelMajor - 1) * 9 + LevelMinor - 1
        Upgrade = RectangleF.Empty
        HasUpgrade(Idx) = True
        DisplayActive = True
        Select Case Idx
            Case 0
                Player.OwnedWeapons(ShotPattern.HurdledShot) = True
                Messages(20)
            Case 1
                Player.VisionUp = True
                Messages(21)
            Case 2
                Player.OwnedWeapons(ShotPattern.SixteenShot) = True
                Messages(22)
            Case 3
                Player.OwnedWeapons(ShotPattern.ParabolicShot) = True
                Messages(23)
            Case 5
                Player.OwnedBullets(ShotTypes.Rubberet) = True
                Messages(24)
            Case 6
                Player.OwnedBullets(ShotTypes.ShockWave) = True
                Messages(25)
            Case 4, 9, 14, 18, 28, 29, 31, 36, 37, 49, 58, 65
                LV = Player.Shots.Length 'makes array bigger by 1.
                Player.Shots = New SelfBullet(Player.Shots.Length) {}
                For Idx = 0 To LV
                    Player.Shots(Idx) = New SelfBullet(0, 0, 0, 0, 0, 0, 0)
                Next
                Messages(26)
            Case 10
                Player.IceStopEnable = True
                Messages(27)
            Case 11
                Player.OwnedWeapons(ShotPattern.SpreadShot) = True
                Messages(28)
            Case 12, 45, 67
                LV = Player.Shots.Length + 4 'makes array bigger by 1.
                Player.Shots = New SelfBullet(Player.Shots.Length + 4) {}
                For Idx = 0 To LV
                    Player.Shots(Idx) = New SelfBullet(0, 0, 0, 0, 0, 0, 0)
                Next
                Messages(29)
            Case 13
                Player.OwnedBullets(ShotTypes.Aquabullet) = True
                Messages(30)
            Case 15
                Player.OwnedWeapons(ShotPattern.VerticalShot) = True
                Messages(31)
            Case 19
                Player.OwnedWeapons(ShotPattern.WideShot) = True
                Messages(32)
            Case 20
                Player.ShotTime = 120
                Messages(33)
            Case 21
                Player.OwnedWeapons(ShotPattern.CartridgeShot) = True
                Messages(34)
            Case 22
                Player.OwnedBullets(ShotTypes.Screwer) = True
                Messages(35)
            Case 23
                Player.DefenseUp = True
                Messages(36)
            Case 24
                Player.OwnedWeapons(ShotPattern.SplitterShot) = True
                Messages(37)
            Case 27
                Player.LongJumpEnable = True
                Messages(38)
            Case 30
                Player.OwnedWeapons(ShotPattern.AlphaShot) = True
                Messages(39)
            Case 32
                Player.OwnedWeapons(ShotPattern.BackhandShot) = True
                Messages(40)
            Case 33
                Player.OwnedBullets(ShotTypes.ForceMod) = True
                Messages(41)
            Case 38
                Player.OwnedWeapons(ShotPattern.MeteorShot) = True
                Messages(42)
            Case 39
                Player.WallKickEnable = True
                Messages(43)
            Case 40, 54
                Player.ShotSpeed = Player.ShotSpeed + 2
                Messages(44)
            Case 41
                Player.OwnedBullets(ShotTypes.Corkwheel) = True
                Messages(45)
            Case 42
                Player.OwnedWeapons(ShotPattern.BoomerangShot) = True
                Messages(46)
            Case 46
                Player.OwnedWeapons(ShotPattern.SprinkleShot) = True
                Messages(47)
            Case 47
                Player.OwnedBullets(ShotTypes.Napalm) = True
                Messages(48)
            Case 48
                Player.EvadeUp = True
                Messages(49)
            Case 50
                Player.OwnedWeapons(ShotPattern.FivewayShot) = True
                Messages(50)
            Case 51
                Player.OwnedWeapons(ShotPattern.VolleyShot) = True
                Messages(51)
            Case 55
                Player.OwnedBullets(ShotTypes.Stonebullet) = True
                Messages(52)
            Case 56
                Player.OwnedWeapons(ShotPattern.TetradShot) = True
                Messages(53)
            Case 57
                Player.OwnedWeapons(ShotPattern.AberrationShot) = True
                Messages(54)
            Case 59
                Player.OwnedWeapons(ShotPattern.HighstormShot) = True
                Messages(55)
            Case 60
                Player.FlameRetardEnable = True
                Messages(56)
            Case 63
                Player.DoubleJumpEnable = True
                Messages(57)
            Case 64
                Player.OwnedBullets(ShotTypes.Plasma) = True
                Messages(58)
            Case 66
                Player.OwnedWeapons(ShotPattern.ClusterShot) = True
                Messages(59)
            Case 68
                Player.OwnedBullets(ShotTypes.Pearl) = True
                Messages(60)
            Case 69
                Player.MysticPowerEnable = True
                Messages(61)
        End Select
    End Sub
    Private Sub Messages(ByVal ID As Integer)
        Dim DC As Integer
        Select Case ID
            Case 1
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = ""
                CusText(2) = "       Press " & JumpKey.ToString() & " To Jump"
                CusLines = 3
                DC = 300
            Case 2
                CusText(1) = ""
                CusText(2) = "       Hold " & RunKey.ToString() & " To Run."
                CusText(3) = "       " & RunKey.ToString() & " + " & JumpKey.ToString() & " + " & LeftKey.ToString() & " or " & RightKey.ToString()
                CusText(4) = "       To Perform a RUNNING JUMP"
                CusLines = 5
                DC = 600
            Case 3
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = ""
                CusText(2) = "       Press " & FireKey.ToString() & " To Shoot Your Weapon"
                CusText(3) = "       Find Shockwave Bullets Here In Level 1-7"
                CusText(4) = "       Before Going to Level 1-8"
                CusLines = 5
                DC = 700
            Case 4
                CusText(1) = ""
                CusText(2) = "       Press " & MenuKey.ToString() & " To Open The Menu"
                CusLines = 3
                DC = 300
            Case 5
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = ""
                CusText(2) = "       Stand On the RTD Circuit To Warm The"
                CusText(3) = "       Switch Which Will Open A Gate"
                CusText(4) = "       Stepping Off Of The RTD Circuit Will Cool"
                CusText(5) = "       Off The Switch And Close The Gate"
                CusLines = 6
                DC = 900
            Case 6
                CusText(1) = ""
                CusText(2) = "       Press " & DownKey.ToString() & " + " & JumpKey.ToString() & " To Perform A Low Jump"
                '             12345678..12345678..12345678..12345678..12345678
                CusLines = 3
                DC = 1200
            Case 7
                CusText(1) = ""
                CusText(2) = "       Find 90% of the Coins in Each Zone"
                CusText(3) = "       To Unlock Level -9.  Each Level -9 Has"
                CusText(4) = "       Extra + Optional Content"
                CusLines = 5
                DC = 600
            Case 8
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = ""
                CusText(2) = "       This First Area Is Called The Rotunda"
                CusText(3) = "       This Is Where I Did Experiments With"
                CusText(4) = "       Moving Walls, As You Will See Further Up"
                CusLines = 5
                DC = 600
            Case 9  '1500
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = ""
                CusText(2) = "       This Area Is Based On A Level That I Made"
                CusText(3) = "       Over Ten Years Ago - Unfortunately The"
                CusText(4) = "       Design Is Incompatible With This Game"
                CusLines = 5
                DC = 600
            Case 10 '3800
                CusText(1) = ""
                CusText(2) = "       This Area Was Initially Going To Be"
                CusText(3) = "       Level 1-1.  I Removed It Because There"
                CusText(4) = "       Is Too Much Empty Area Between Buildings"
                CusLines = 5
                DC = 600
            Case 11 '10500
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = ""
                CusText(2) = "       This Area Is Called The Crib"
                CusText(3) = "       I Filled It With RTD Circuit Gates"
                CusLines = 4
                DC = 600
            Case 12
                CusText(1) = ""
                CusText(2) = "       Press " & UseDoorKey.ToString() & " To Enter The Exit Doors"
                CusLines = 3
                DC = 300
            Case 13
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = ""
                CusText(2) = "Grr... You may be able to beat me one-on-one,"
                CusText(3) = "...but I don't play by your little ethics."
                CusText(4) = "I'm gonna kill you with my emperor robot"
                CusText(5) = "and end this little rampage that you're on"
                CusText(6) = "right here and right now."
                CusLines = 7
                DC = 1500
            Case 14
                '             12345678..12345678..12345678..12345678..12345678
                CusText(1) = "Wha... what?  My robot?  I can't believe it."
                CusText(2) = "You destroyed my emperor robot?"
                CusText(3) = "Argh... ... ..."
                CusText(4) = "No matter, I can finish you off with"
                CusText(5) = "the robot's detachable head... it already has"
                CusText(6) = "All the necessary features installed"
                CusText(7) = "and after you're dead, the world will be mine!"
                CusText(8) = "Now die, punk!"
                CusLines = 9
                DC = 2000
            Case 15
                CusText(1) = ""
                CusText(2) = "Noooooooo!  I can't believe this is happening."
                CusText(3) = "I spent three years trying to realize my dreams"
                CusText(4) = "and this is what it comes to?"
                CusText(5) = "*Cough* *Cough* How could you do this?"
                CusLines = 6
                DC = 1500
            Case 16
                CusText(1) = ""
                CusText(2) = "To swim, press " & JumpKey.ToString() & "."
                CusLines = 3
                DC = 250
            Case 20
                CusText(1) = ""
                CusText(2) = "This is the hurdledshot.  It shoots two bullets:"
                CusText(3) = "The first bullet fires in front of you normally,"
                CusText(4) = "but the second one is fired from above your head"
                CusText(5) = "You need bullets in order to use it, though..."
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 21
                CusText(1) = ""
                CusText(2) = "This is a vision upgrade.  It dramatically"
                CusText(3) = "increases your field of view, which will help"
                CusText(4) = "you avoid incoming attacks."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 22
                CusText(1) = ""
                CusText(2) = "This is the sixteenshot.  It shoots sixteen"
                CusText(3) = "bullets outward from you in a circle."
                CusLines = 4
                DC = 500
                '             12345678..12345678..12345678..12345678..12345678
            Case 23
                CusText(1) = ""
                CusText(2) = "This is the parabolic shot.  It shoots a bullet"
                CusText(3) = "on a parabolic trajectory, almost as if it were"
                CusText(4) = "being pulled down by gravity."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 24
                CusText(1) = ""
                CusText(2) = "These are rubberets (short for rubber bullets)."
                CusText(3) = "They bounce around rebounding off of walls,"
                CusText(4) = "and continue until they hit an enemy."
                CusText(5) = "Load into your favorite weapon and enjoy!"
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 25
                CusText(1) = ""
                CusText(2) = "This is the shockwave.  You can now load bullets"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "and fire shockwave bullets from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Now, you can defeat the boss of Level 1-8"
                CusLines = 7
                DC = 1250
                '             12345678..12345678..12345678..12345678..12345678
            Case 26
                CusText(1) = ""
                CusText(2) = "This is an ammunition upgrade.  It allows you to"
                CusText(3) = "have an extra bullet on the field."
                CusText(4) = "(Now you can have " & Player.Shots.Length & " bullets active.)"
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 27
                CusText(1) = ""
                CusText(2) = "This is an ice-braking upgrade.  Now, when you"
                CusText(3) = "release " & LeftKey.ToString() & " or " & RightKey.ToString() & " on the ice,"
                CusText(4) = "you will be stopped automatically.  It does not"
                CusText(5) = "stop you from speeding up on ice."
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 28
                CusText(1) = ""
                CusText(2) = "This is the spreadshot.  This weapon will shoot"
                CusText(3) = "three bullets in front of you, much like a"
                CusText(4) = "normal three-way shooting weapon."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 29
                CusText(1) = ""
                CusText(2) = "This is an ammunition upgrade.  It allows you to"
                CusText(3) = "have five extra bullets on the field."
                CusText(4) = "(Now you can have " & Player.Shots.Length & " bullets active.)"
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 30
                CusText(1) = ""
                CusText(2) = "This is the aquabullet.  This can be loaded"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "to fire aqua bullets from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Aquabullets are very effective on machines and"
                CusText(7) = "fire-elemental monsters."
                CusLines = 8
                DC = 1500
                '             12345678..12345678..12345678..12345678..12345678
            Case 31
                CusText(1) = ""
                CusText(2) = "This is the verticalshot.  This shoots a single"
                CusText(3) = "bullet straight upwards.  Handy for things that"
                CusText(4) = "are flying overhead."
                CusLines = 5
                DC = 750
            Case 32
                CusText(1) = ""
                CusText(2) = "This is the wideshot.  This shoots three bullets"
                CusText(3) = "in a straight line in front of you.  It's good"
                CusText(4) = "for larger enemies."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 33
                CusText(1) = ""
                CusText(2) = "This is the bullet time upgrade.  Now, your"
                CusText(3) = "bullets can stay active for 2 seconds before"
                CusText(4) = "they lose their power and disappear from the"
                CusText(5) = "field."
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 34
                CusText(1) = ""
                CusText(2) = "This is the cartridgeshot.  It shoots three"
                CusText(3) = "bullets in a randomized shotgun pattern in"
                CusText(4) = "front of you.  It's quite useful.  Try it out."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 35
                CusText(1) = ""
                CusText(2) = "This is the screwer.  It is a large metal slug"
                CusText(3) = "bullet that causes a varying effect on machines,"
                CusText(4) = "leading to weapons failure or an explosion."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 36
                CusText(1) = ""
                CusText(2) = "This is a defensive upgrade.  You now can resist"
                CusText(3) = "all enemy damage by 80% and now can survive 24"
                CusText(4) = "collisions with enemies and their attacks."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 37
                CusText(1) = ""
                CusText(2) = "This is the splittershot.  It shoots two bullets"
                CusText(3) = "that fly apart from either due to repulsive"
                CusText(4) = "electric forces.  Maybe there's some use for it?"
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 38
                CusText(1) = ""
                CusText(2) = "This is the longjump upgrade.  To do a longjump"
                CusText(3) = "press " & DownKey.ToString() & " and " & JumpKey.ToString() & "."
                CusText(4) = "You will rapidly vault ahead.  A longjump does"
                CusText(5) = "not allow you to jump upwards very high, or to"
                CusText(6) = "slow down mid-jump, so use with care."
                CusLines = 7
                DC = 1250
                '             12345678..12345678..12345678..12345678..12345678
            Case 39
                CusText(1) = ""
                CusText(2) = "This is the alphashot.  This weapon shoots two"
                CusText(3) = "bullets that will trace out an alpha symbol."
                CusText(4) = "It works better at point-blank range."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 40
                CusText(1) = ""
                CusText(2) = "This is the backhandshot.  This shoots three"
                CusText(3) = "bullets behind you that will accelerate forward"
                CusText(4) = "in a straight-line.  Great for long-distance"
                CusText(5) = "engagements."
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 41
                CusText(1) = ""
                CusText(2) = "This is the forcemod.  This can be loaded"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "to fire force bullets from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Forcemods deal a concussive blow to animals and"
                CusText(7) = "can break certain objects."
                CusLines = 8
                DC = 1500
                '             12345678..12345678..12345678..12345678..12345678
            Case 42
                CusText(1) = ""
                CusText(2) = "This is the meteorshot.  This shoots four"
                CusText(3) = "bullets diagonally downward in a square"
                CusText(4) = "formation.  Great for monsters that wait ahead"
                CusText(5) = "but are lower than you."
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 43
                CusText(1) = ""
                CusText(2) = "This is the wallkick upgrade.  To bounce off of"
                CusText(3) = "a wall, press " & LeftKey.ToString() & " or " & RightKey.ToString()
                CusText(4) = "while jumping and then press " & JumpKey.ToString() & "."
                CusText(5) = "You will bounce off of the wall in the opposite"
                CusText(6) = "direction.  Use this to climb up to high ledges"
                CusText(7) = "that are near vertical surfaces."
                CusLines = 8
                DC = 1500
                '             12345678..12345678..12345678..12345678..12345678
            Case 44
                CusText(1) = ""
                CusText(2) = "This is the shooting speed upgrade.  This causes"
                CusText(3) = "bullets that you shoot ahead to travel forward"
                CusText(4) = "an additional 2 mps (mile per second) faster."
                CusText(5) = "Current speed: " & Player.ShotSpeed.ToString() & " mps"
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 45
                CusText(1) = ""
                CusText(2) = "This is the corkwheel.  This can be loaded"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "to fire corkwheels from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Corkwheels are significantly large bullets that"
                CusText(7) = "roll when they land on a surface."
                CusLines = 8
                DC = 1500
                '             12345678..12345678..12345678..12345678..12345678
            Case 46
                CusText(1) = ""
                CusText(2) = "This is the boomerangshot.  It shoots three"
                CusText(3) = "bullets ahead that will then accelerate behind"
                CusText(4) = "you.  I'm sure you can find a use for this"
                CusText(5) = "weapon in the name of crowd control."
                CusLines = 6
                DC = 1000
                '             12345678..12345678..12345678..12345678..12345678
            Case 47
                CusText(1) = ""
                CusText(2) = "This is the sprinkleshot.  It shoots three"
                CusText(3) = "bullets below you that will then rise upwards in"
                CusText(4) = "front of you."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 48
                CusText(1) = ""
                CusText(2) = "These are napalm bullets.  These can be loaded"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "to fire napalm bullets from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Napalm bullets explode when they come in contact"
                CusText(7) = "with any rigid surface."
                CusLines = 8
                DC = 1500
                '             12345678..12345678..12345678..12345678..12345678
            Case 49
                CusText(1) = ""
                CusText(2) = "This is the evasion upgrade.  This upgrade"
                CusText(3) = "dramastically shrinks your hitbox and allows you"
                CusText(4) = "to remain invulnerable for longer after being"
                CusText(5) = "hit from an enemy."
                CusLines = 6
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 50
                CusText(1) = ""
                CusText(2) = "This is the fiveway shot.  This weapon shoots"
                CusText(3) = "five bullets: three in the shape of the 3-way,"
                CusText(4) = "one directly above, and one directly below."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 51
                CusText(1) = ""
                CusText(2) = "This is the volleyshot.  This weapon shoots"
                CusText(3) = "four bullets in a small rectangular pattern"
                CusText(4) = "in front of you.  Good for close encounters."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 52
                CusText(1) = ""
                CusText(2) = "These are stone bullets.  These can be loaded"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "to fire stone bullets from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Stone bullets levitate in place when they come"
                CusText(7) = "in contact with any rigid surface."
                CusLines = 8
                DC = 1500
                '             12345678..12345678..12345678..12345678..12345678
            Case 53
                CusText(1) = ""
                CusText(2) = "This is the tetradshot.  This weapon shoots"
                CusText(3) = "four bullets in a T-shaped pattern in front of"
                CusText(4) = "you.  Similar to the 3-way shot."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 54
                CusText(1) = ""
                CusText(2) = "This is the aberrationshot.  This shoots five"
                CusText(3) = "bullets in a bizarre pattern in front of you."
                CusLines = 4
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 55
                CusText(1) = ""
                CusText(2) = "This is the highstormshot.  This shoots six"
                CusText(3) = "bullets: one directly in front of you, and five"
                CusText(4) = "over your head in a sweeping pattern."
                CusLines = 4
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 56
                CusText(1) = ""
                CusText(2) = "This is the flame retardant.  This will keep you"
                CusText(3) = "protected from lava damage and explosions."
                CusLines = 4
                DC = 500
                '             12345678..12345678..12345678..12345678..12345678
            Case 57
                CusText(1) = ""
                CusText(2) = "This is the highjump upgrade.  To perform, press"
                CusText(3) = UpKey.ToString() & " and " & JumpKey.ToString() & "."
                CusText(4) = "This will allow you to jump twice your normal"
                CusText(5) = "height, but does not work on long jumps and"
                CusText(6) = "wall kicks."
                CusLines = 7
                DC = 1250
                '             12345678..12345678..12345678..12345678..12345678
            Case 58
                CusText(1) = ""
                CusText(2) = "These are plasma bullets.  These can be loaded"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "to fire plasma bullets from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Plasma bullets travel through all enemies,"
                CusText(7) = "damaging them as long they maintain contact."
                CusLines = 8
                DC = 1500
            Case 59
                CusText(1) = ""
                CusText(2) = "This is the clustershot.  This shoots six"
                CusText(3) = "bullets, in two sets of three, as a pair of"
                CusText(4) = "cartridge shots."
                CusLines = 5
                DC = 750
                '             12345678..12345678..12345678..12345678..12345678
            Case 60
                CusText(1) = ""
                CusText(2) = "These are pearl bullets.  These can be loaded"
                CusText(3) = "into any weapon (icons with blue background)"
                CusText(4) = "to fire pearl bullets from it by pressing"
                CusText(5) = FireKey.ToString() & "."
                CusText(6) = "Pearl bullets hit the target with holy power"
                CusText(7) = "which can defeat undead monsters."
                CusLines = 8
                DC = 1500
            Case 61
                CusText(1) = ""
                CusText(2) = "This is a mystical item, indeed.  Perhaps you'll"
                CusText(3) = "find what it does sometime."
                CusLines = 4
                DC = 500
                '             12345678..12345678..12345678..12345678..12345678
        End Select
        'ActiveMessageIndex = ID
        If DisplayCounter = 0 Then
            DisplayCounter = DC
        Else
            DisplayActive = True
        End If
    End Sub
    Private Function IsLevelClear(ByVal Mjr As Integer, ByVal Mnr As Integer) As Boolean
        Dim Idx As Integer = (Mjr - 1) * 8 + Mnr - 1
        Return CompletedLev(Idx)
    End Function
    Private Function ZoneFromSelexionEx(ByVal S As Integer) As Integer
        Return (S \ 9)
    End Function
    Private Function SubsecFromSelexionEx(ByVal S As Integer) As Integer
        Return (S Mod 9)
    End Function

    Private Sub LeftMovementChecks()
        Dim Rcv As AutoSuiteReturn
        If Player.Loc.Left < 0 Then
            'If the player goes off the edge of the form, then we will move him back to the edge.
            Player.Loc.Offset(-Player.Loc.Left, 0)
        ElseIf Player.Loc.Right < Player.LeftEnd AndAlso Not Player.IsJumping Then
            Player.IsJumping = True
            'We are not really jumping, but we are falling.
            Player.IsClimbing = False
            Player.OnSlope = False
            'Not on a slopy surface anymore.
            Player.Veloc = 0
            'Since we are falling from rest, the initial velocity will be zero.
            MobileIndex = -1
            KonveyIndex = -1
            RTDIndex = -1
            PNMIndex = -1


        ElseIf Player.IsClimbing Then
            Player.IsClimbing = False
            Player.IsJumping = True
            'We are not really jumping, but we are falling.

            MobileIndex = -1
            KonveyIndex = -1
            RTDIndex = -1
            PNMIndex = -1
            'We are not standing on anything now, because we are jumping.
            Player.Veloc = 0
            'Since we are falling from rest, the initial velocity will be zero.

        End If
        Rcv = CheckForWalkingLeftIntoWall(Player.Loc, Player.IsJumping)  'Source ByRef
        If Rcv.RetState = SuiteReturnType.AdjustSlope Then
            AdjustPlayerHeight(Rcv.IncValue)
        ElseIf Rcv.RetState = SuiteReturnType.SlopeClimbing Then
            'If player is falling, then the player will land on the wall.
            Player.IsJumping = False
            Player.IsLongJumping = False
            'Stop the jump.
            Bbx.PlayWave(5, Player.Veloc > 5)
            Player.OnIce = False : Player.HorizontVeloc = 0
            'No longer on ice.
            Player.LeftEnd = Rcv.TrpValue.R.Left : Player.RightEnd = Rcv.TrpValue.R.Right
            'Set the endpoints.
            Player.AnimCycler = 0
            'Reset the cycler.
            Player.Loc.Offset(0, Rcv.TrpValue.MiddleValue - Player.Loc.Bottom)
            'Position the player to stand on this wall.
            Player.OnSlope = True
            'Is player walking on something sloped?
            Player.LH = Rcv.TrpValue.LeftHeight
            Player.RH = Rcv.TrpValue.RightHeight
            'The interpolation will occur in the character movement places, after the movement checks.
        End If

        Player.WKReady = (Rcv.RetState = SuiteReturnType.Wallkick AndAlso Player.IsJumping)

        If Player.Loc.Right < Player.LeftEnd AndAlso Not Player.IsJumping Then
            Player.IsJumping = True
            'Falling off the edge of a platform.
            Player.OnSlope = False
            'Not on a slopy surface anymore.
            Player.Veloc = 0
            'Free-fall.
        End If


    End Sub
    Private Sub RightMovementChecks()
        Dim Rcv As AutoSuiteReturn
        If Player.Loc.Left > Player.RightEnd AndAlso Not Player.IsJumping Then
            Player.IsJumping = True
            'We are falling.
            Player.IsClimbing = False
            Player.OnSlope = False
            'Not on a slopy surface anymore.
            Player.Veloc = 0
            'A fall from rest.
            MobileIndex = -1
            KonveyIndex = -1
            RTDIndex = -1
            PNMIndex = -1
        ElseIf Player.IsClimbing Then
            Player.IsClimbing = False
            Player.IsJumping = True
            'We are not really jumping, but we are falling.

            MobileIndex = -1
            KonveyIndex = -1
            RTDIndex = -1
            PNMIndex = -1
            'We are not standing on anything now, because we are jumping.
            Player.Veloc = 0
            'Since we are falling from rest, the initial velocity will be zero.
            'Rcv = CheckForWalkingRightIntoWall(Player.Loc)
            'If Rcv.RetState = SuiteReturnType.AdjustSlope Then
            '    AdjustPlayerHeight(Rcv.IncValue)
            'End If

            'If Rcv.RetState = SuiteReturnType.Wallkick Then
            '    Player.WKReady = (Rcv.RetState = SuiteReturnType.Wallkick AndAlso Player.IsJumping)

            'End If 
        End If

        ' *** The below line was in a final Else block with the following 2 if blocks
        ' I moved it out because there was no check to see if the player ran into a wall while falling.
        Rcv = CheckForWalkingRightIntoWall(Player.Loc, Player.IsJumping)
        If Rcv.RetState = SuiteReturnType.AdjustSlope Then
            AdjustPlayerHeight(Rcv.IncValue)
        ElseIf Rcv.RetState = SuiteReturnType.SlopeClimbing Then
            'If player is falling, then the player will land on the wall.
            Player.IsJumping = False
            Player.IsLongJumping = False
            'Stop the jump.
            Bbx.PlayWave(5, Player.Veloc > 5)
            Player.OnIce = False : Player.HorizontVeloc = 0
            'No longer on ice.
            Player.LeftEnd = Rcv.TrpValue.R.Left : Player.RightEnd = Rcv.TrpValue.R.Right
            'Set the endpoints.
            Player.AnimCycler = 0
            'Reset the cycler.
            Player.Loc.Offset(0, Rcv.TrpValue.MiddleValue - Player.Loc.Bottom)
            'Position the player to stand on this wall.
            Player.OnSlope = True
            'Is player walking on something sloped?
            Player.LH = Rcv.TrpValue.LeftHeight
            Player.RH = Rcv.TrpValue.RightHeight
            'The interpolation will occur in the character movement places, after the movement checks.
        End If
        Player.WKReady = (Rcv.RetState = SuiteReturnType.Wallkick AndAlso Player.IsJumping)



        If Player.Loc.Left > Player.RightEnd AndAlso Not Player.IsJumping Then
            Player.IsJumping = True
            Player.Veloc = 0
            'Falling from the edge of platform.
            Player.OnSlope = False
            'Not on a slopy surface anymore.

        End If
    End Sub

    Private Function CheckForWalkingLeftIntoWall(ByRef Source As RectangleF, ByVal CheckGlass As Boolean) As AutoSuiteReturn
        Dim HzV As Single
        Dim Ret As AutoSuiteReturn

        For Each Wall In Q.Walls
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Right - Source.Left, 0)
                'Move the player back to the right edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Right - Source.Left, 0)
                'Move the player back to the right edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Right - Source.Left, 0)
                'Move the player back to the right edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.Crates
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Right - Source.Left, 0)
                'Move the player back to the right edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.EyeBeams
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Right - Source.Left, 0)
                'Move the player back to the right edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.CrossStays
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Right - Source.Left, 0)
                'Move the player back to the right edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            'Check with each moving wall this time.
            If Source.IntersectsWith(MobileWall.Loc) Then
                Source.Offset(MobileWall.Loc.Right - Source.Left, 0)
                'Move the player to the edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, MobileWall.Loc)
            End If
        Next
        For Each OpSw In Q.RTDs
            'Check with each moving wall this time.
            If Source.IntersectsWith(OpSw.Gate) Then
                Source.Offset(OpSw.Gate.Right - Source.Left, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Gate)
                'Move the player to the edge of this wall.
            ElseIf Source.IntersectsWith(OpSw.Switch) Then
                Source.Offset(OpSw.Switch.Right - Source.Left, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Switch)
            End If
        Next
        For Each OpSw In Q.Pneumos
            'Check with each moving wall this time.
            If Source.IntersectsWith(OpSw.Gate) Then
                Source.Offset(OpSw.Gate.Right - Source.Left, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Gate)
                'Move the player to the edge of this wall.
            ElseIf Source.IntersectsWith(OpSw.Switch) Then
                Source.Offset(OpSw.Switch.Right - Source.Left, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Switch)
            End If
        Next
        For Each Jumpboard In Q.Jumpboards
            'Check with each wall.
            If Source.IntersectsWith(Jumpboard.Loc) Then
                Source.Offset(Jumpboard.Loc.Right - Source.Left, 0)
                'Move the player to the edge of this jumping board.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Jumpboard.Loc)
            End If
        Next
        For Each Datablk In Q.Conveyors 'Variable reuse
            If Source.IntersectsWith(Datablk.Loc) Then
                Source.Offset(Datablk.Loc.Right - Source.Left, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Datablk.Loc)
            End If
        Next
        For Each Slopedwall In Q.Slopedwalls
            'Check with each wall.
            If Slopedwall.IntersectsWith(Player.Loc) Then
                If Slopedwall.Diag = Diagonals.UpRight AndAlso Source.Bottom = Slopedwall.MiddleValue Then
                    Source.Offset(Slopedwall.R.Right - Source.Left, 0)
                    Ret = New AutoSuiteReturn(SuiteReturnType.SlopeClimbing, Slopedwall, Slopedwall.MiddleValue)
                Else
                    HzV = Slopedwall.HorValueForRect(Source, False)  'You wouldn't hit the left side of the wall
                    'by walking to the left.
                    Source.Offset(HzV - Source.Left, 0)
                    'Move the player to the edge of this wall.
                    Ret = New AutoSuiteReturn(SuiteReturnType.AdjustSlope, Source)
                    'AdjustPlayerHeight(Source)
                End If

            End If
        Next
        For Each Wall In Q.Glasses
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Right - Source.Left, 0)
                'Move the player back to the right edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
                If CheckGlass Then
                    BreakGlassIndex = Q.Glasses.IndexOf(Wall)
                End If
            End If
        Next

        Return Ret
    End Function
    Private Function CheckForWalkingRightIntoWall(ByRef Source As RectangleF, ByVal CheckGlass As Boolean) As AutoSuiteReturn
        Dim HzV As Single
        Dim Ret As AutoSuiteReturn

        For Each Wall In Q.Walls
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.Crates
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.FieldFences
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.EyeBeams
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each Wall In Q.CrossStays
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If Source.IntersectsWith(MobileWall.Loc) Then
                Source.Offset(MobileWall.Loc.Left - Source.Right, 0)
                'Move the player to the edge of this wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, MobileWall.Loc)
            End If
        Next
        For Each OpSw In Q.RTDs
            'Check with each moving wall this time.
            If Source.IntersectsWith(OpSw.Gate) Then
                Source.Offset(OpSw.Gate.Left - Source.Right, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Gate)
                'Move the player to the edge of this wall.
            ElseIf Source.IntersectsWith(OpSw.Switch) Then
                Source.Offset(OpSw.Switch.Left - Source.Right, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Switch)
            End If
        Next
        For Each OpSw In Q.Pneumos
            'Check with each moving wall this time.
            If Source.IntersectsWith(OpSw.Gate) Then
                Source.Offset(OpSw.Gate.Left - Source.Right, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Gate)
                'Move the player to the edge of this wall.
            ElseIf Source.IntersectsWith(OpSw.Switch) Then
                Source.Offset(OpSw.Switch.Left - Source.Right, 0)
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, OpSw.Switch)
            End If
        Next
        For Each Jumpboard In Q.Jumpboards
            If Source.IntersectsWith(Jumpboard.Loc) Then
                Source.Offset(Jumpboard.Loc.Left - Source.Right, 0)
                'Move the player to the edge of this jumping board..
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Jumpboard.Loc)
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Source.IntersectsWith(Datablk.Loc) Then
                Source.Offset(Datablk.Loc.Left - Source.Right, 0)
                'Move the player to the edge of this jumping board..
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Datablk.Loc)
            End If
        Next
        For Each Slopedwall In Q.Slopedwalls
            'Check with each wall.

            If Slopedwall.IntersectsWith(Source) Then
                If Slopedwall.Diag = Diagonals.UpLeft AndAlso Source.Bottom = Slopedwall.MiddleValue Then
                    Source.Offset(Slopedwall.R.Left - Source.Right, 0)
                    Ret = New AutoSuiteReturn(SuiteReturnType.SlopeClimbing, Slopedwall, Slopedwall.MiddleValue)

                Else
                    HzV = Slopedwall.HorValueForRect(Source, True) 'You'd hit the left side of the wall first when walking right.
                    Source.Offset(HzV - Source.Right, 0)
                    'Move the player to the edge of this wall.
                    Ret = New AutoSuiteReturn(SuiteReturnType.AdjustSlope, Source)
                    'AdjustPlayerHeight(Source)

                    'AdjustPlayerHeight(Player.Loc)
                End If

            End If
        Next
        For Each Wall In Q.Glasses
            If Source.IntersectsWith(Wall) Then
                Source.Offset(Wall.Left - Source.Right, 0)
                'Move player back to left edge of wall.
                Ret = New AutoSuiteReturn(SuiteReturnType.Wallkick, Wall)
                If CheckGlass Then
                    BreakGlassIndex = Q.Glasses.IndexOf(Wall)
                End If
            End If
        Next

        Return Ret
    End Function

    Private Sub AdjustPlayerHeight(ByVal OldPL As RectangleF)
        Dim L As Single, Percent As Double, OldPct As Double
        If Player.OnSlope Then
            If Player.LH > Player.RH Then       'If slope is higher on the right
                L = Player.Loc.Right            'Use Right side of player location to adjust height
                Percent = (L - Player.LeftEnd) / (Player.RightEnd - Player.LeftEnd)    'Interpolation: percentage progress to the right side.
                If Percent < 0.0R Then
                    Player.Loc.Y = Player.LH - Player.Loc.Height
                ElseIf Percent < 1.0R Then          'Player is on slope
                    Player.Loc.Y = Player.LH - Player.Loc.Height + Convert.ToInt32(Percent * (Player.RH - Player.LH))   'Interpolation.
                Else
                    OldPct = (OldPL.Right - Player.LeftEnd) / (Player.RightEnd - Player.LeftEnd)            'Where the player was.
                    If OldPct < 1.0R Then               'If player was on the slope, player has moved off of the slope.
                        'i have redirected all of this code to place the player on a platform which contains the top of the slope - no jumping.
                        'Player.IsJumping = True
                        ''We are not really jumping, but we are falling.
                        'Player.OnSlope = False
                        ''Not on a slopy surface anymore.
                        'MobileIndex = -1
                        ''We are not standing on anything now, because we are jumping.
                        'If GoFast Then
                        '    Player.Veloc = FASTMOVESPEED * (Player.RH - Player.LH) / (Player.RightEnd - Player.LeftEnd) - 4 * GRAVITY
                        '    'Since we are falling from rest, the initial velocity will be zero.
                        'Else
                        '    Player.Veloc = MOVESPEED * (Player.RH - Player.LH) / (Player.RightEnd - Player.LeftEnd) - 4 * GRAVITY
                        '    'Since we are falling from rest, the initial velocity will be zero.
                        'End If
                        Player.Loc.Offset(0, Player.RH - Player.Loc.Bottom)
                        Player.LeftEnd = Player.RightEnd - CHARWIDTH
                        Player.OnSlope = False

                    Else
                        Player.Loc.Y = Player.RH - Player.Loc.Height  'This places the player on top of the slope.
                    End If
                End If
            Else
                L = Player.Loc.Left
                Percent = (L - Player.LeftEnd) / (Player.RightEnd - Player.LeftEnd)
                If Percent > 1.0R Then
                    Player.Loc.Y = Player.RH - Player.Loc.Height
                ElseIf Percent > 0.0R Then
                    Player.Loc.Y = Player.LH - Player.Loc.Height + Convert.ToInt32(Percent * (Player.RH - Player.LH))
                Else
                    OldPct = (OldPL.Left - Player.LeftEnd) / (Player.RightEnd - Player.LeftEnd)
                    If OldPct > 0.0R Then
                        'Player.IsJumping = True
                        ''We are not really jumping, but we are falling.
                        'Player.OnSlope = False
                        ''Not on a slopy surface anymore.
                        'MobileIndex = -1
                        ''We are not standing on anything now, because we are jumping.
                        'If GoFast Then
                        '    Player.Veloc = FASTMOVESPEED * (Player.LH - Player.RH) / (Player.RightEnd - Player.LeftEnd) - 4 * GRAVITY
                        '    'Since we are falling from rest, the initial velocity will be zero.
                        'Else
                        '    Player.Veloc = MOVESPEED * (Player.LH - Player.RH) / (Player.RightEnd - Player.LeftEnd) - 4 * GRAVITY
                        '    'Since we are falling from rest, the initial velocity will be zero.
                        'End If
                        Player.Loc.Offset(0, Player.LH - Player.Loc.Bottom)
                        Player.RightEnd = Player.LeftEnd + CHARWIDTH
                        Player.OnSlope = False
                    Else
                        Player.Loc.Y = Player.LH - Player.Loc.Height
                    End If
                End If
            End If

        End If
    End Sub

    Private Sub FormD_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        Running = False
        'End the game loop so that the device doesn't draw again.
    End Sub

    Private Sub SetPresentParams(ByVal PP As PresentParameters)
        PP.DeviceWindow = Me
        'This looks important.
        PP.Windowed = True
        'We'll let it be windowed for now. :)
        PP.SwapEffect = SwapEffect.Discard
        'This one is easy to set... Discard is what the documentation uses, but I figure this just draws over the
        'last drawing.
        PP.EnableAutoDepthStencil = True
        'When we present, we draw from back to front.
        PP.AutoDepthStencilFormat = DepthFormat.D16
        'dunno what D16 stands for.
        'Now that we've set some properties, we can make our Device.
        'PP.PresentationInterval = PresentInterval.Immediate
    End Sub

    Private Sub SetVertexBufferData(ByVal vxbf() As VertexBuffer, ByVal D9 As Device)
        Dim LX, LZ As Single, Ctr() As Integer, LV As Integer, Idx As Integer
        Dim TheBack As Single

        Dim UL As Boolean

        Const DEPTHBEGIN As Single = -100.0F
        Const DEPTHEND As Single = 256.0F
        Const WIDTHBEGIN As Single = 0.0F
        Const WIDTHEND As Single = 1000.0F

        Const BDDIV As Single = 64.0F

        Const dLz As Single = (DEPTHEND - DEPTHBEGIN) / FIELDTILEDEPTH, dLx As Single = (WIDTHEND - WIDTHBEGIN) / FIELDTILEWIDTH


        Dim vs()() As CustomVertex.PositionNormalTextured = New CustomVertex.PositionNormalTextured(KUSize - 1)() {}

        Ctr = New Integer(KUSize - 1) {}
        Dim LDep As Single = +20.0F
        Const dLDep As Single = -0.1F

        Const TrapSpacing As Single = 32.0F

        'After that workout, we now have to actually set the vertices.
        'We do this by locking the surface out of D9 while we "operate" on it.
        Dim A As Array

        For LV = 0 To KUSize - 1

            A = vxbf(LV).Lock(0, LockFlags.None)
            'It returns a vanilla array...
            vs(LV) = DirectCast(A, CustomVertex.PositionNormalTextured())
            'So we have to convert it to an array of the appropriate vertices.



            'Z = F(X, Y), but this time with texture.
            'Our texture should be 0,0 at X,Y = -3,-3 and 1,1 at X,Y = 3,3
            'So, we will simply do an interpolation during this process.
            'An interpolation is, again, like this:
            'Vf = L + Percent * (H - L), where Vf is the texture value, L is the lowest texture value, and H is the highest texture value.
            'Percent would also be an interpolation: Percent = (Vo - L) / (H - L)
            'where Vo is the loop variable value, L is the lowest loop value, and H is the highest loop value.
            'The first formula would be Vf = 0 + Percent * (1 - 0) = Percent.
            'The second formula would be Percent = (Vo - -3) / (3 - -3) = (Vo + 3) / 6,
            'Therefore, Vf = (Vo + 3) / 6
            'with L + dL
            'Vf = (Vo + dVo + 3) / 6

            Try
                For LZ = DEPTHBEGIN To DEPTHEND - 1.0F Step dLz
                    For LX = WIDTHBEGIN To WIDTHEND - 1.0F Step dLx
                        'He he... I'm using a number as an index to an array of indexes to an array of arrays.
                        'triangle 1:
                        vs(LV)(Ctr(LV)) = New CustomVertex.PositionNormalTextured(LV * 1000 + LX, LANDHEIGHT, LZ, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                        vs(LV)(Ctr(LV) + 1) = New CustomVertex.PositionNormalTextured(LV * 1000 + LX + dLx, LANDHEIGHT, LZ, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                        vs(LV)(Ctr(LV) + 2) = New CustomVertex.PositionNormalTextured(LV * 1000 + LX, LANDHEIGHT, LZ + dLz, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                        'triangle 2:
                        vs(LV)(Ctr(LV) + 3) = New CustomVertex.PositionNormalTextured(LV * 1000 + LX + dLx, LANDHEIGHT, LZ + dLz, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
                        vs(LV)(Ctr(LV) + 4) = vs(LV)(Ctr(LV) + 2)
                        vs(LV)(Ctr(LV) + 5) = vs(LV)(Ctr(LV) + 1)
                        Ctr(LV) += 6
                    Next
                Next
            Catch ex As Exception
                System.Diagnostics.Debug.WriteLine(String.Format("X = {0}, Z = {1}, Ctr = {2}", LX, LZ, Ctr(LV)))
            End Try

        Next

        'Now, we draw the platforms:

        '***************************************************
        ' PLATFORMS
        '***************************************************
        For Each Platform In Q.Platforms
            Idx = Convert.ToInt32(Platform.X) \ 1000

            'Front face.
            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Bottom, PLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Bottom, PLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Bottom, PLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            'Top face
            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Top, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Top, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            'Left Face
            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Top, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Bottom, PLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Bottom, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Bottom, PLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Platform.Left, Platform.Top, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'Right Face
            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Bottom, PLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Bottom, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Top, PLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Platform.Right, Platform.Bottom, PLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            Ctr(Idx) += PLATFORMVTXCOUNT
            LDep += dLDep
        Next
        LDep = 0

        '***************************************************
        ' WALL
        '***************************************************
        If LevelMajor = 5 Then
            For Each Wall In Q.Walls
                Idx = Convert.ToInt32(Wall.X) \ 1000

                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + 32, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                Ctr(Idx) += WALLVTXCOUNT
                'LDep += dLDep
            Next
            'LDep = 0
        Else
            For Each Wall In Q.Walls
                Idx = Convert.ToInt32(Wall.X) \ 1000

                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, WALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                Ctr(Idx) += WALLVTXCOUNT
                LDep += dLDep
            Next
            LDep = 0
        End If

        '***************************************************
        ' Jumpboards
        '***************************************************

        For Each Jumpboard In Q.Jumpboards
            Idx = Convert.ToInt32(Jumpboard.Loc.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Right, Jumpboard.Loc.Bottom, JBDCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Jumpboard.Loc.Left, Jumpboard.Loc.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

            Ctr(Idx) += JBDVTXCOUNT
            LDep += dLDep
        Next

        ' ****************************************************
        '  ICE
        ' ****************************************************
        LDep = 20.0F
        For Each Ice In Q.Ices
            Idx = Convert.ToInt32(Ice.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Top, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Top, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Top, ICEDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICEDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Top, ICEDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Bottom, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Top, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Bottom, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Ice.Right, Ice.Bottom, ICECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Ice.Left, Ice.Bottom, ICEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

            Ctr(Idx) += ICEVTXCOUNT
            LDep -= dLDep
        Next

        ' ****************************************************
        '  BACKDROP
        ' ****************************************************
        LDep = 0
        For Each Backdrop In Q.Backdrops
            Idx = Convert.ToInt32(Backdrop.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Backdrop.Left, Backdrop.Top, BACKDROPDEPTH, 0.0F, 0.0F, 0.0F, Backdrop.Left / BDDIV, Backdrop.Top / BDDIV)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Backdrop.Left, Backdrop.Bottom, BACKDROPDEPTH, 0.0F, 0.0F, 0.0F, Backdrop.Left / BDDIV, Backdrop.Bottom / BDDIV)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Backdrop.Right, Backdrop.Top, BACKDROPDEPTH, 0.0F, 0.0F, 0.0F, Backdrop.Right / BDDIV, Backdrop.Top / BDDIV)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Backdrop.Right, Backdrop.Bottom, BACKDROPDEPTH, 0.0F, 0.0F, 0.0F, Backdrop.Right / BDDIV, Backdrop.Bottom / BDDIV)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Backdrop.Right, Backdrop.Top, BACKDROPDEPTH, 0.0F, 0.0F, 0.0F, Backdrop.Right / BDDIV, Backdrop.Top / BDDIV)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Backdrop.Left, Backdrop.Bottom, BACKDROPDEPTH, 0.0F, 0.0F, 0.0F, Backdrop.Left / BDDIV, Backdrop.Bottom / BDDIV)

            Ctr(Idx) += BACKDROPVTXCOUNT
            LDep -= dLDep
        Next

        ' *******************************************************
        '   TRAPEZOID PLATFORM
        ' *******************************************************
        For Each TrapPlatform In Q.TrapPlatforms
            Idx = Convert.ToInt32(TrapPlatform.R.X) \ 1000

            UL = (TrapPlatform.Diag = Diagonals.UpLeft)

            If UL Then
                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.MiddleValue, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Left / TrapSpacing, 0.5F)
            Else
                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.R.Top, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Left / TrapSpacing, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.R.Bottom, TRAPPLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Left / TrapSpacing, 1.0F)
            If UL Then
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.R.Top, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Right / TrapSpacing, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.MiddleValue, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Right / TrapSpacing, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.R.Bottom, TRAPPLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Right / TrapSpacing, 1.0F)
            If UL Then
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.R.Top, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Right / TrapSpacing, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.MiddleValue, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Right / TrapSpacing, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.R.Bottom, TRAPPLATFORMCLOSENESS, 0.0F, 0.0F, 0.0F, TrapPlatform.R.Left / TrapSpacing, 1.0F)



            If UL Then
                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.MiddleValue, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.R.Top, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.R.Top, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.MiddleValue, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.MiddleValue, TRAPPLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.R.Top, TRAPPLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.R.Top, TRAPPLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.MiddleValue, TRAPPLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.MiddleValue, TRAPPLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Left, TrapPlatform.R.Top, TRAPPLATFORMDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.R.Top, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(TrapPlatform.R.Right, TrapPlatform.MiddleValue, TRAPPLATFORMCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            End If



            Ctr(Idx) += TRAPPLATFORMVTXCOUNT
            LDep += dLDep
        Next


        '*****************************
        ' SLOPED WALL
        '*****************************
        LDep = 0

        For Each Slopedwall In Q.Slopedwalls
            Idx = Convert.ToInt32(Slopedwall.R.X) \ 1000

            UL = (Slopedwall.Diag = Diagonals.UpLeft)


            If UL Then
                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            If UL Then
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            If UL Then
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            If UL Then
                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.MiddleValue, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Top, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.MiddleValue, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Top, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            End If

            If UL Then
                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            End If
            If UL Then
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.MiddleValue, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Top, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            End If
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            If UL Then
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.MiddleValue, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            Else
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Top, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            End If

            If UL Then
                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            If UL Then
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Bottom, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            If UL Then
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Top, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            Else
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.MiddleValue, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            End If
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Bottom, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Right, Slopedwall.R.Bottom, SLOPEDWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Slopedwall.R.Left, Slopedwall.R.Bottom, SLOPEDWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            Ctr(Idx) += SLOPEDWALLVTXCOUNT
            LDep += dLDep
        Next
        LDep = 0

        ' ****************************************************
        '  COLUMN
        ' ****************************************************
        LDep = 0
        For Each Column In Q.Verticals
            Idx = Convert.ToInt32(Column.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Top, WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Bottom, WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Bottom, WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            'Column.Left + Column.Width * Sin60
            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Top, WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Bottom, WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Top, WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Top, -WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Bottom, -WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Bottom, -WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            'Column.Left + Column.Width * Sin60
            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)

            vs(Idx)(Ctr(Idx) + 30) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 31) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)
            vs(Idx)(Ctr(Idx) + 32) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Top, -WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 33) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Bottom, -WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 34) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Top, -WALLCLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 35) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, -WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)

            Ctr(Idx) += COLUMNVTXCOUNT
            LDep -= dLDep
        Next

        ' *******************************
        ' EXTRA WALLS
        ' *******************************

        For Each Wall In Q.Extrawalls
            Idx = Convert.ToInt32(Wall.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EXWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EXWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            Ctr(Idx) += EXWALLVTXCOUNT
            LDep += dLDep
        Next

        ' *******************************
        ' BULB
        ' *******************************

        For Each Backdrop In Q.Bulbs
            Idx = Convert.ToInt32(Backdrop.X) \ 1000


            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Backdrop.Left, Backdrop.Top, BULBCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Backdrop.Left, Backdrop.Top, BULBDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Backdrop.Left + BULBHALF, Backdrop.Bottom, BULBCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Backdrop.Left + BULBHALF, Backdrop.Bottom, BULBDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Backdrop.Left + BULBHALF, Backdrop.Bottom, BULBCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Backdrop.Left, Backdrop.Top, BULBDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Backdrop.Right, Backdrop.Top, BULBCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Backdrop.Right - BULBHALF, Backdrop.Bottom, BULBCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Backdrop.Right, Backdrop.Top, BULBDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Backdrop.Right - BULBHALF, Backdrop.Bottom, BULBDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Backdrop.Right, Backdrop.Top, BULBDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Backdrop.Right - BULBHALF, Backdrop.Bottom, BULBCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Backdrop.Left, Backdrop.Top, BULBCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Backdrop.Left + BULBHALF, Backdrop.Bottom, BULBCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Backdrop.Right, Backdrop.Top, BULBCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)

            Ctr(Idx) += BULBVTXCOUNT
            'LDep += dLDep
        Next

        For Each Ladder In Q.Ladders
            Idx = Convert.ToInt32(Ladder.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, ROPECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, ROPECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, Ladder.Height / 32.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, ROPECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Bottom, ROPECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, Ladder.Height / 32.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, ROPECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, ROPECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, Ladder.Height / 32.0F)


            Ctr(Idx) += LADDERVTXCOUNT
            LDep += dLDep
        Next

        ' ****************************************************
        '  ROPE
        ' ****************************************************
        LDep = 0
        For Each Column In Q.Ropes
            Idx = Convert.ToInt32(Column.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Top, ROPECLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Bottom, ROPECLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Column.Left, Column.Bottom, ROPECLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            'Column.Left + Column.Width * Sin60
            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Top, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width / 4, Column.Bottom, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.333333343F, 1.0F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Top, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Top, ROPECLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Bottom, ROPECLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width, Column.Top, ROPECLOSENESS + Column.Width / 2, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Column.Left + Column.Width * 0.75F, Column.Bottom, ROPECLOSENESS, 0.0F, 0.0F, 0.0F, 0.6666667F, 1.0F)

            Ctr(Idx) += ROPEVTXCOUNT2
            LDep -= dLDep
        Next

        ' *******************************
        ' GROUND
        ' *******************************

        For Each Wall In Q.Groundwalls
            Idx = Convert.ToInt32(Wall.X) \ 1000


            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 30) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 31) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 32) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 33) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 34) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 35) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 36) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 37) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 38) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 39) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 40) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 41) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)

            vs(Idx)(Ctr(Idx) + 42) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 43) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 44) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 45) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 46) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.75F)
            vs(Idx)(Ctr(Idx) + 47) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 48) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 49) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 50) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 51) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 52) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 53) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.25F)

            vs(Idx)(Ctr(Idx) + 54) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 55) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.25F)
            vs(Idx)(Ctr(Idx) + 56) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 57) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 58) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, GROUNDFULLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 59) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, GROUNDSECDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.25F)


            Ctr(Idx) += GROUNDVTXCOUNT
            LDep += dLDep
        Next

        '**************************************
        '  FENCE
        '**************************************
        For Each Ladder In Q.ForeFences
            Idx = Convert.ToInt32(Ladder.X) \ 1000
            LDep = Ladder.Height / 8.0F

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top + (Ladder.Height / 8.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top + (Ladder.Height / 8.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top + (Ladder.Height / 8.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top + (Ladder.Height / 2.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top + (Ladder.Height * 5.0F / 8.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top + (Ladder.Height / 2.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top + (Ladder.Height * 5.0F / 8.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top + (Ladder.Height / 2.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top + (Ladder.Height * 5.0F / 8.0F), FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width / 4.0F), Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width / 4.0F), Ladder.Bottom, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width / 4.0F) + LDep, Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width / 4.0F) + LDep, Ladder.Bottom, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width / 4.0F) + LDep, Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width / 4.0F), Ladder.Bottom, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width * 3.0F / 4.0F) - LDep, Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width * 3.0F / 4.0F) - LDep, Ladder.Bottom, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width * 3.0F / 4.0F), Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width * 3.0F / 4.0F), Ladder.Bottom, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width * 3.0F / 4.0F), Ladder.Top, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Ladder.Left + (Ladder.Width * 3.0F / 4.0F) - LDep, Ladder.Bottom, FENCECLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            Ctr(Idx) += FOREFENCEVTXCOUNT
        Next
        For Each Ladder In Q.AftFences
            Idx = Convert.ToInt32(Ladder.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Ladder.Right, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)


            Ctr(Idx) += FENCEVTXCOUNT
            LDep += dLDep
        Next
        For Each Ladder In Q.FieldFences
            Idx = Convert.ToInt32(Ladder.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Bottom, FENCECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Ladder.Left, Ladder.Top, FENCEDEPTH + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)


            Ctr(Idx) += FENCEVTXCOUNT
            LDep += dLDep
        Next



        '***************************************************
        ' CRATE
        '***************************************************

        For Each Wall In Q.Crates
            Idx = Convert.ToInt32(Wall.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CRATEDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            Ctr(Idx) += CRATEVTXCOUNT
            LDep += dLDep
        Next
        LDep = 0


        '***************************************************
        ' BACKCRATE
        '***************************************************

        If LevelMajor = 1 OrElse LevelMajor = 8 Then
            TheBack = BACKCRATEDEPTH
            For Each Wall In Q.Backcrates
                Idx = Convert.ToInt32(Wall.X) \ 1000

                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                Ctr(Idx) += BACKCRATEVTXCOUNT
                LDep += dLDep
            Next
            LDep = 0
        Else
            TheBack = BACKCRATEDEPTH
            For Each Wall In Q.Backcrates
                Idx = Convert.ToInt32(Wall.X) \ 1000

                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, Wall.Left / BDDIV, Wall.Top / BDDIV)
                vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, Wall.Left / BDDIV, Wall.Bottom / BDDIV)
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, Wall.Right / BDDIV, Wall.Top / BDDIV)
                vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, Wall.Right / BDDIV, Wall.Bottom / BDDIV)
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, Wall.Right / BDDIV, Wall.Top / BDDIV)
                vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, Wall.Left / BDDIV, Wall.Bottom / BDDIV)

                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, BACKCRATECLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, TheBack, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                Ctr(Idx) += BACKCRATEVTXCOUNT
                LDep += dLDep
            Next
            LDep = 0
        End If




        '***************************************************
        ' EYEBEAM
        '***************************************************

        For Each Wall In Q.EyeBeams
            Idx = Convert.ToInt32(Wall.X) \ 1000
            If Wall.Height < Wall.Width Then
                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.6F)
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)

                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.7F)
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.8F)
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)

                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.9F)
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 30) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 31) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 32) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 33) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 34) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 35) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 36) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 37) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 38) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 39) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 40) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 41) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)

                vs(Idx)(Ctr(Idx) + 42) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 43) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 44) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 45) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 46) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 47) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)

                vs(Idx)(Ctr(Idx) + 48) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 49) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 50) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 51) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 52) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 53) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)

                vs(Idx)(Ctr(Idx) + 54) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 55) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 56) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 57) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 58) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 59) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)

                vs(Idx)(Ctr(Idx) + 60) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 61) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 62) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 63) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 64) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 65) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                vs(Idx)(Ctr(Idx) + 66) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 67) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 68) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 69) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 70) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 71) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 72) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 73) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 74) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 75) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 76) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 77) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMCLOSENESS + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                vs(Idx)(Ctr(Idx) + 78) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 79) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)
                vs(Idx)(Ctr(Idx) + 80) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 81) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.6F)
                vs(Idx)(Ctr(Idx) + 82) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 83) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)

                vs(Idx)(Ctr(Idx) + 84) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.7F)
                vs(Idx)(Ctr(Idx) + 85) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)
                vs(Idx)(Ctr(Idx) + 86) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 87) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.8F)
                vs(Idx)(Ctr(Idx) + 88) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 89) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)

                vs(Idx)(Ctr(Idx) + 90) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.9F)
                vs(Idx)(Ctr(Idx) + 91) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 92) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 93) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 94) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 95) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 96) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 97) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 98) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 99) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 100) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 101) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 102) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 103) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 104) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 105) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 106) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 107) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 108) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 109) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 110) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 111) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 112) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 113) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 114) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 115) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 116) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 117) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 118) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 119) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)

                vs(Idx)(Ctr(Idx) + 120) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 121) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 122) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 123) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 124) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 125) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)

                vs(Idx)(Ctr(Idx) + 126) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 127) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 128) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 129) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 130) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 131) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)

                vs(Idx)(Ctr(Idx) + 132) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 133) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 134) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 135) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 136) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 137) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)

                vs(Idx)(Ctr(Idx) + 138) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 139) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 140) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 141) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 142) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 143) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + Wall.Height, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                vs(Idx)(Ctr(Idx) + 144) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 145) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 146) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 147) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 148) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 149) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom - (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 150) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 151) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 152) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 153) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 154) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 155) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top + (Wall.Height / 4.0F), EYEBEAMDEPTH + (Wall.Height * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                Ctr(Idx) += EYEBEAMVTXCOUNT
            Else
                vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)
                vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.6F)
                vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)

                vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.7F)
                vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)
                vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.8F)
                vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)

                vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.9F)
                vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 30) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 31) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 32) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 33) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 34) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 35) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 36) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 37) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 38) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 39) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 40) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 41) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)

                vs(Idx)(Ctr(Idx) + 42) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 43) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 44) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 45) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 46) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 47) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)

                vs(Idx)(Ctr(Idx) + 48) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 49) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 50) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 51) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 52) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 53) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)

                vs(Idx)(Ctr(Idx) + 54) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 55) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 56) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 57) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 58) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 59) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)

                vs(Idx)(Ctr(Idx) + 60) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 61) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 62) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 63) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 64) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 65) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMCLOSENESS + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                vs(Idx)(Ctr(Idx) + 66) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 67) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 68) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 69) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 70) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 71) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 72) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 73) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 74) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 75) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 76) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 77) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMCLOSENESS + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                vs(Idx)(Ctr(Idx) + 78) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 79) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)
                vs(Idx)(Ctr(Idx) + 80) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 81) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.6F)
                vs(Idx)(Ctr(Idx) + 82) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 83) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.6F)

                vs(Idx)(Ctr(Idx) + 84) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.7F)
                vs(Idx)(Ctr(Idx) + 85) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)
                vs(Idx)(Ctr(Idx) + 86) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 87) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.8F)
                vs(Idx)(Ctr(Idx) + 88) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.7F)
                vs(Idx)(Ctr(Idx) + 89) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.0F, 0.8F)

                vs(Idx)(Ctr(Idx) + 90) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.9F)
                vs(Idx)(Ctr(Idx) + 91) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
                vs(Idx)(Ctr(Idx) + 92) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 93) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
                vs(Idx)(Ctr(Idx) + 94) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.9F)
                vs(Idx)(Ctr(Idx) + 95) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

                vs(Idx)(Ctr(Idx) + 96) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 97) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 98) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 99) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 100) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 101) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 102) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 103) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 104) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 105) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 106) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 107) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 108) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 109) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 110) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 111) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 112) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 113) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

                vs(Idx)(Ctr(Idx) + 114) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 115) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 116) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 117) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 118) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 119) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)

                vs(Idx)(Ctr(Idx) + 120) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 121) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 122) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 123) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 124) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 125) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)

                vs(Idx)(Ctr(Idx) + 126) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(Idx)(Ctr(Idx) + 127) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 128) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.125F)
                vs(Idx)(Ctr(Idx) + 129) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 130) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 6.0F / 8.0F), 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 131) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)

                vs(Idx)(Ctr(Idx) + 132) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.375F)
                vs(Idx)(Ctr(Idx) + 133) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 134) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 135) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 136) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 1.0F, 0.375F)
                vs(Idx)(Ctr(Idx) + 137) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)

                vs(Idx)(Ctr(Idx) + 138) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 139) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 140) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 141) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 142) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 143) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, EYEBEAMDEPTH + Wall.Width, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                vs(Idx)(Ctr(Idx) + 144) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 145) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 146) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 147) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 148) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 149) = New CustomVertex.PositionNormalTextured(Wall.Right - (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

                vs(Idx)(Ctr(Idx) + 150) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(Idx)(Ctr(Idx) + 151) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(Idx)(Ctr(Idx) + 152) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 153) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
                vs(Idx)(Ctr(Idx) + 154) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Bottom, EYEBEAMDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
                vs(Idx)(Ctr(Idx) + 155) = New CustomVertex.PositionNormalTextured(Wall.Left + (Wall.Width / 4.0F), Wall.Top, EYEBEAMDEPTH + (Wall.Width * 2.0F / 8.0F), 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

                Ctr(Idx) += EYEBEAMVTXCOUNT

            End If

            'LDep += dLDep
        Next
        LDep = 0

        '***************************************************
        ' CROSSSTAY
        '***************************************************

        For Each Wall In Q.CrossStays
            Idx = Convert.ToInt32(Wall.X) \ 1000

            vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 6) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 7) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 8) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 9) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 10) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 11) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)

            vs(Idx)(Ctr(Idx) + 12) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 13) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 14) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 15) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 16) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 17) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Top, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(Idx)(Ctr(Idx) + 18) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 19) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(Idx)(Ctr(Idx) + 20) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 21) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(Idx)(Ctr(Idx) + 22) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Top, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 23) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(Idx)(Ctr(Idx) + 24) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(Idx)(Ctr(Idx) + 25) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(Idx)(Ctr(Idx) + 26) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 27) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.5F)
            vs(Idx)(Ctr(Idx) + 28) = New CustomVertex.PositionNormalTextured(Wall.Right, Wall.Bottom, CROSSSTAYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(Idx)(Ctr(Idx) + 29) = New CustomVertex.PositionNormalTextured(Wall.Left, Wall.Bottom, CROSSSTAYDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            Ctr(Idx) += CROSSSTAYVTXCOUNT
            'LDep += dLDep
        Next
        LDep = 0


        For Each Datablk In Q.Things
            Select Case Datablk.Data
                Case 0  'Plant
                    vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, PLANTDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                    vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, PLANTDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0625F)
                    vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, PLANTDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F, 0.0F)
                    vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, PLANTDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F, 0.0625F)
                    vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, PLANTDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F, 0.0F)
                    vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, PLANTDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0625F)

                    Ctr(Idx) += PLANTVTXCOUNT
                    LDep += dLDep
                Case 1, 2   'Statue
                    vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, STATUEDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F + 0.0625F * (Datablk.Data - 1), 0.0F)
                    vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, STATUEDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F + 0.0625F * (Datablk.Data - 1), 0.0625F)
                    vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, STATUEDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F + 0.0625F * Datablk.Data, 0.0F)
                    vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, STATUEDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F + 0.0625F * Datablk.Data, 0.0625F)
                    vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, STATUEDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F + 0.0625F * Datablk.Data, 0.0F)
                    vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, STATUEDEPTH, 0.0F, 0.0F, 0.0F, 0.03125F + 0.0625F * (Datablk.Data - 1), 0.0625F)

                    Ctr(Idx) += STATUEVTXCOUNT
                    LDep += dLDep
                Case 3 To 6  'Picture
                    vs(Idx)(Ctr(Idx)) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, PICTUREDEPTH, 0.0F, 0.0F, 0.0F, 0.15625F + 0.09375F * (Datablk.Data - 3), 0.0F)
                    vs(Idx)(Ctr(Idx) + 1) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, PICTUREDEPTH, 0.0F, 0.0F, 0.0F, 0.15625F + 0.09375F * (Datablk.Data - 3), 0.0625F)
                    vs(Idx)(Ctr(Idx) + 2) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, PICTUREDEPTH, 0.0F, 0.0F, 0.0F, 0.15625F + 0.09375F * (Datablk.Data - 2), 0.0F)
                    vs(Idx)(Ctr(Idx) + 3) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, PICTUREDEPTH, 0.0F, 0.0F, 0.0F, 0.15625F + 0.09375F * (Datablk.Data - 2), 0.0625F)
                    vs(Idx)(Ctr(Idx) + 4) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, PICTUREDEPTH, 0.0F, 0.0F, 0.0F, 0.15625F + 0.09375F * (Datablk.Data - 2), 0.0F)
                    vs(Idx)(Ctr(Idx) + 5) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, PICTUREDEPTH, 0.0F, 0.0F, 0.0F, 0.15625F + 0.09375F * (Datablk.Data - 3), 0.0625F)

                    Ctr(Idx) += PICTUREVTXCOUNT
                    LDep += dLDep
            End Select
        Next


        For LV = 0 To KUSize - 1
            NormalizeTriangles(vs(LV), PrT)
            vxbf(LV).Unlock()

        Next

        'It's important to unlock the vertex buffer, so that D9 can see what we put in there.

    End Sub
    Private Sub SetCoinSprites(ByVal vs() As CustomVertex.PositionColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Const COINDEPTH As Single = -30.0F
        Dim N As Single = 1.0F
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        'It returns a vanilla array...
        vs = DirectCast(A, CustomVertex.PositionColoredTextured())

        Dim LV As Integer = 0
        For Each Coin In Q.Coins
            If Not Coin.IsEmpty Then
                vs(LV) = New CustomVertex.PositionColoredTextured(Coin.Left, Coin.Top, COINDEPTH, &HFFFFFFFF, 0.0F, 0.0F)
                vs(LV + 1) = New CustomVertex.PositionColoredTextured(Coin.Right, Coin.Top, COINDEPTH, &HFFFFFFFF, 1.0F, 0.0F)
                vs(LV + 2) = New CustomVertex.PositionColoredTextured(Coin.Left, Coin.Bottom, COINDEPTH, &HFFFFFFFF, 0.0F, 1.0F)
                vs(LV + 3) = New CustomVertex.PositionColoredTextured(Coin.Right, Coin.Bottom, COINDEPTH, &HFFFFFFFF, 1.0F, 1.0F)
            Else
                vs(LV) = New CustomVertex.PositionColoredTextured(Coin.Left, Coin.Top, CAMERADEPTH - 1, &HFFFFFFFF, 0.0F, 0.0F)
                vs(LV + 1) = New CustomVertex.PositionColoredTextured(Coin.Right, Coin.Top, CAMERADEPTH - 1, &HFFFFFFFF, 1.0F, 0.0F)
                vs(LV + 2) = New CustomVertex.PositionColoredTextured(Coin.Left, Coin.Bottom, CAMERADEPTH - 1, &HFFFFFFFF, 0.0F, 1.0F)
                vs(LV + 3) = New CustomVertex.PositionColoredTextured(Coin.Right, Coin.Bottom, CAMERADEPTH - 1, &HFFFFFFFF, 1.0F, 1.0F)

            End If
            LV += 4
        Next


        'vs(1) = New CustomVertex.PositionColored(Coin.Right, Coin.Top, 0, &HFFFFFFFF)
        'vs(2) = New CustomVertex.PositionColored(Coin.Left, Coin.Bottom, 0, &HFFFFFFFF)
        'vs(3) = New CustomVertex.PositionColored(Coin.Right, Coin.Bottom, 0, &HFFFFFFFF)

        vxbf.Unlock()
    End Sub
    Private Sub SetupBillboard(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim vs() As CustomVertex.PositionColoredTextured
        Dim N As Single = 1.0F
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim LV, LI As Integer           'Loop through billboards
        Dim Addend As Single   'Value added to get correct frame for player.
        Dim tool As IMonster, K As Kite
        Dim Vec As Vector3


        Const BBX As Single = 40.0F, BBY As Single = 40.0F, BBZ As Single = 3.0F
        'It returns a vanilla array...
        vs = DirectCast(A, CustomVertex.PositionColoredTextured())

        If Player.IsJumping Then
            If Player.Direc = 1 Then
                Addend = 9.0F
            Else
                Addend = 8.0F
            End If
            If Player.IsShooting Then
                Addend += 20.0F
            End If
        ElseIf Player.IsClimbing Then
            Addend = 16.0F + (Player.AnimCycler \ FRAMETIME)  '10 to convert 0-9 into 0.0-0.9, 2 to expand the frame time.
            If Player.IsShooting Then
                If Player.Direc = 1 Then
                    Addend = 31.0F
                Else
                    Addend = 30.0F
                End If
            End If
        ElseIf Player.Direc = 1 Then
            Addend = 4.0F + (Player.AnimCycler \ FRAMETIME)  '10 to convert 0-9 into 0.0-0.9, 2 to expand the frame time.
            If Player.IsShooting Then
                Addend += 20.0F
            End If
        Else
            Addend = (Player.AnimCycler \ FRAMETIME)
            If Player.IsShooting Then
                Addend += 20.0F
            End If
        End If

        Addend /= FRAMEDIVCONST

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(0.0F, -N, 2.0F * N, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(0.0F, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(0.0F, N, 2.0F * N, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)

        vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 1 / FRAMEDIVCONST, 0.0F)
        vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 1 / FRAMEDIVCONST, 1.0F)

        Vec = DirectCast(Bits(LV), Vector3)

        Mx(LV) = Matrix.Identity
        Mx(LV).Multiply(Matrix.Scaling(4.0F, 16.0F, 1.0F))

        'Mx(LV).Multiply(Matrix.RotationY(Convert.ToSingle(Math.Asin((-CAMERADEPTH) / CAMERADISTANCECONSTANT))))
        'Mx(LV).Multiply(Matrix.RotationY(PiHalf))

        'Mx(LV).Multiply(Matrix.RotationZ(Convert.ToSingle(Math.Atan2(Camera.Y - Vec.Y, Camera.X - Vec.X))))
        'Mx(LV).Multiply(Matrix.RotationZ(-PiHalf))

        Mx(LV).Multiply(Matrix.Translation(Vec.X, Vec.Y, Vec.Z))


        For LV = 0 To Q.Monsters.Count - 1
            tool = DirectCast(Q.Monsters.Item(LV), IMonster)   'Get the next monster.
            LI = LV + 1   'This refers to the place in the vs and Mx arrays that this monster's information will be found.
            If Bits.Count <= LI Then
                Bits.Add(tool.Position)
            Else
                Bits(LI) = tool.Position
            End If
            Vec = DirectCast(Bits(LI), Vector3)

            tool.Draw(vs, LI << 2)

            If TypeOf tool Is Kite Then
                K = DirectCast(tool, Kite)
                Mx(LI) = Matrix.Identity
                Mx(LI).Multiply(Matrix.Translation(-1.0F, -1.0F, 0.0F))
                Mx(LI).Multiply(Matrix.RotationZ(PiHalf - K.head))
                Mx(LI).Multiply(Matrix.RotationY(K.ang))
                Mx(LI).Multiply(Matrix.Translation(1.0F, 1.0F, 0.0F))
                Mx(LI).Multiply(Matrix.Scaling(tool.Width / 2.0F, tool.Height / 2.0F, tool.Height / 2.0F))
                Mx(LI).Multiply(Matrix.Translation(Vec.X, Vec.Y, -Vec.Z))
            Else
                Mx(LI) = Matrix.Identity
                Mx(LI).Multiply(Matrix.Scaling(tool.Width / 2.0F, tool.Height / 2.0F, tool.Height / 2.0F))
                'Mx(LI).Multiply(Matrix.RotationY(Convert.ToSingle(Math.Asin((-CAMERADEPTH) / CAMERADISTANCECONSTANT))))
                'Mx(LI).Multiply(Matrix.RotationY(PiHalf))
                'Mx(LI).Multiply(Matrix.RotationZ(Convert.ToSingle(-PiHalf)))
                Mx(LI).Multiply(Matrix.Translation(Vec.X, Vec.Y, -Vec.Z))
            End If

        Next

        'GetDistance(New Vector3(Bits(LV).X, Bits(LV).Y, Bits(LV).Z), Camera)


        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(0.0F, -N, 2.0F * N, &HFFFFFFFF, 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(0.0F, -N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(0.0F, N, 2.0F * N, &HFFFFFFFF, 1.0F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)




        'Mx(LV) = Matrix.Identity
        'Mx(LV).Multiply(Matrix.Scaling(1.0F, 8.0F, 16.0F))
        'Mx(LV).Multiply(Matrix.RotationY(Convert.ToSingle(Math.Asin((FLOATERDEPTH - Camera.Z) / GetDistance(New Vector3(Floater.X, Floater.Y, FLOATERDEPTH), Camera)))))
        'Mx(LV).Multiply(Matrix.RotationZ(Convert.ToSingle(Math.Atan2(Camera.Y - Floater.Y, Camera.X - Floater.X))))
        'Mx(LV).Multiply(Matrix.Translation(Floater.X, Floater.Y, FLOATERDEPTH))



        vxbf.Unlock()
    End Sub
    Private Sub SetXlucentBuffer(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.PositionColored
        Dim ctr As Integer
        'It returns a vanilla array...
        vs = DirectCast(A, CustomVertex.PositionColored())
        'So we have to convert it to an array of the appropriate vertices.
        Dim LDep As Single = 0.0F
        Const CLR As Integer = &H80FFFFFF
        Const GLSCLR As Integer = &H50FFFFFF

        For Each Lava In Q.Lavas
            vs(ctr) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVACLOSENESS, CLR)
            vs(ctr + 1) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 2) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVACLOSENESS, CLR)
            vs(ctr + 3) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 4) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVACLOSENESS, CLR)
            vs(ctr + 5) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVACLOSENESS, CLR)

            vs(ctr + 6) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVACLOSENESS, CLR)
            vs(ctr + 7) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVACLOSENESS, CLR)
            vs(ctr + 8) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 9) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 10) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 11) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVACLOSENESS, CLR)

            vs(ctr + 12) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVACLOSENESS, CLR)
            vs(ctr + 13) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 14) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 15) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVADEPTH, CLR)
            vs(ctr + 16) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 17) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVADEPTH, CLR)

            vs(ctr + 18) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVACLOSENESS, CLR)
            vs(ctr + 19) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 20) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 21) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVADEPTH, CLR)
            vs(ctr + 22) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 23) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVACLOSENESS, CLR)

            vs(ctr + 24) = New CustomVertex.PositionColored(Lava.Left, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 25) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 26) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVADEPTH, CLR)
            vs(ctr + 27) = New CustomVertex.PositionColored(Lava.Right, Lava.Top, LAVADEPTH, CLR)
            vs(ctr + 28) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVADEPTH, CLR)
            vs(ctr + 29) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVADEPTH, CLR)

            vs(ctr + 30) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 31) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVADEPTH, CLR)
            vs(ctr + 32) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 33) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVADEPTH, CLR)
            vs(ctr + 34) = New CustomVertex.PositionColored(Lava.Right, Lava.Bottom, LAVACLOSENESS, CLR)
            vs(ctr + 35) = New CustomVertex.PositionColored(Lava.Left, Lava.Bottom, LAVADEPTH, CLR)

            ctr += 36
            LDep += 0.0625F
        Next

        LDep = 0.0F
        For Each Water In Q.Waters
            vs(ctr) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERCLOSENESS, CLR)
            vs(ctr + 1) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 2) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERCLOSENESS, CLR)
            vs(ctr + 3) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 4) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERCLOSENESS, CLR)
            vs(ctr + 5) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERCLOSENESS, CLR)

            vs(ctr + 6) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERCLOSENESS, CLR)
            vs(ctr + 7) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERCLOSENESS, CLR)
            vs(ctr + 8) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 9) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 10) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 11) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERCLOSENESS, CLR)

            vs(ctr + 12) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERCLOSENESS, CLR)
            vs(ctr + 13) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 14) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 15) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERDEPTH, CLR)
            vs(ctr + 16) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 17) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERDEPTH, CLR)

            vs(ctr + 18) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERCLOSENESS, CLR)
            vs(ctr + 19) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 20) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 21) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERDEPTH, CLR)
            vs(ctr + 22) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 23) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERCLOSENESS, CLR)

            vs(ctr + 24) = New CustomVertex.PositionColored(Water.Left, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 25) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 26) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERDEPTH, CLR)
            vs(ctr + 27) = New CustomVertex.PositionColored(Water.Right, Water.Top, WATERDEPTH, CLR)
            vs(ctr + 28) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERDEPTH, CLR)
            vs(ctr + 29) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERDEPTH, CLR)

            vs(ctr + 30) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 31) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERDEPTH, CLR)
            vs(ctr + 32) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 33) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERDEPTH, CLR)
            vs(ctr + 34) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, WATERCLOSENESS, CLR)
            vs(ctr + 35) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, WATERDEPTH, CLR)

            ctr += 36
            LDep += 0.0625F
        Next

        For Each Water In Q.Glasses
            vs(ctr) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 1) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 2) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 3) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 4) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 5) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSCLOSENESS, GLSCLR)

            vs(ctr + 6) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 7) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 8) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 9) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 10) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 11) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSCLOSENESS, GLSCLR)

            vs(ctr + 12) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 13) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 14) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 15) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSDEPTH, GLSCLR)
            vs(ctr + 16) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 17) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSDEPTH, GLSCLR)

            vs(ctr + 18) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 19) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 20) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 21) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSDEPTH, GLSCLR)
            vs(ctr + 22) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 23) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSCLOSENESS, GLSCLR)

            vs(ctr + 24) = New CustomVertex.PositionColored(Water.Left, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 25) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 26) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSDEPTH, GLSCLR)
            vs(ctr + 27) = New CustomVertex.PositionColored(Water.Right, Water.Top, GLASSDEPTH, GLSCLR)
            vs(ctr + 28) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSDEPTH, GLSCLR)
            vs(ctr + 29) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSDEPTH, GLSCLR)

            vs(ctr + 30) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 31) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSDEPTH, GLSCLR)
            vs(ctr + 32) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 33) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSDEPTH, GLSCLR)
            vs(ctr + 34) = New CustomVertex.PositionColored(Water.Right, Water.Bottom, GLASSCLOSENESS, GLSCLR)
            vs(ctr + 35) = New CustomVertex.PositionColored(Water.Left, Water.Bottom, GLASSDEPTH, GLSCLR)

            ctr += 36
            LDep += 0.0625F
        Next


        vxbf.Unlock()
    End Sub
    Private Sub SetDynamicBufferData(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.PositionNormalTextured
        Dim ctr As Integer

        Dim LDep As Single = 0.0F
        Const dLDep As Single = 0.0625F
        Const dZWire As Single = -30.0F
        vs = DirectCast(A, CustomVertex.PositionNormalTextured())



        For Each MobileWall In Q.Mobilewalls
            vs(ctr) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 1) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 2) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 3) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 4) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 5) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(ctr + 6) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 7) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 8) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Top, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 9) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 10) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Top, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 11) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(ctr + 12) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 13) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Top, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 14) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 15) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 16) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 17) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Top, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(ctr + 18) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 19) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 20) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 21) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Bottom, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 22) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Top, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 23) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(ctr + 24) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 25) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 26) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 27) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Bottom, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 28) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Right, MobileWall.Loc.Bottom, MOVINGWALLCLOSENESS + LDep, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 29) = New CustomVertex.PositionNormalTextured(MobileWall.Loc.Left, MobileWall.Loc.Bottom, MOVINGWALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            ctr += MOVINGWALLVTXCOUNT
            'LDep += dLDep
        Next

        For Each Datablk In Q.Conveyors
            'If Datablk.Data < 0 Then
            LDep = -Me.Fmd(Tick * Datablk.Data, Datablk.Loc.Width) / Datablk.Loc.Width
            'Else
            '    LDep = Me.Fmd(Tick * Datablk.Data, Datablk.Loc.Width) / Datablk.Loc.Width
            'End If
            vs(ctr) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.5F)
            vs(ctr + 1) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 2) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(ctr + 3) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 4) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.5F)
            vs(ctr + 5) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(ctr + 6) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, LDep, 0.5F)
            vs(ctr + 7) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.5F)
            vs(ctr + 8) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, LDep, 0.0F)
            vs(ctr + 9) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.0F)
            vs(ctr + 10) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, LDep, 0.0F)
            vs(ctr + 11) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.5F)

            vs(ctr + 12) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.5F)
            vs(ctr + 13) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.0F)
            vs(ctr + 14) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, LDep, 0.5F)
            vs(ctr + 15) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, LDep, 0.0F)
            vs(ctr + 16) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, LDep, 0.5F)
            vs(ctr + 17) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.0F)

            vs(ctr + 18) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, LDep, 0.5F)
            vs(ctr + 19) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.5F)
            vs(ctr + 20) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, LDep, 0.0F)
            vs(ctr + 21) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.0F)
            vs(ctr + 22) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, LDep, 0.0F)
            vs(ctr + 23) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.5F)

            vs(ctr + 24) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.0F)
            vs(ctr + 25) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.5F)
            vs(ctr + 26) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, LDep, 0.0F)
            vs(ctr + 27) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, LDep, 0.5F)
            vs(ctr + 28) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, KONVEYCLOSENESS, 0.0F, 0.0F, 0.0F, LDep, 0.0F)
            vs(ctr + 29) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, KONVEYDEPTH, 0.0F, 0.0F, 0.0F, 1.0F + LDep, 0.5F)
            ctr += KONVEYVTXCOUNT
            'LDep += dLDep
        Next

        'For Each Datablk In Q.Videoscreens
        '    LDep = (Tick Mod 64) \ 4
        '    vs(ctr) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Top, VIDEODEPTH, 0.0F, 0.0F, 0.0F, 0.046875F * (Datablk.Data - 7), 0.0625F + 0.03125F * (LDep))
        '    vs(ctr + 1) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, VIDEODEPTH, 0.0F, 0.0F, 0.0F, 0.046875F * (Datablk.Data - 7), 0.0625F + 0.03125F * (LDep + 1))
        '    vs(ctr + 2) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, VIDEODEPTH, 0.0F, 0.0F, 0.0F, 0.046875F * (Datablk.Data - 6), 0.0625F + 0.03125F * (LDep))
        '    vs(ctr + 3) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Bottom, VIDEODEPTH, 0.0F, 0.0F, 0.0F, 0.046875F * (Datablk.Data - 6), 0.0625F + 0.03125F * (LDep + 1))
        '    vs(ctr + 4) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Right, Datablk.Loc.Top, VIDEODEPTH, 0.0F, 0.0F, 0.0F, 0.046875F * (Datablk.Data - 6), 0.0625F + 0.03125F * (LDep))
        '    vs(ctr + 5) = New CustomVertex.PositionNormalTextured(Datablk.Loc.Left, Datablk.Loc.Bottom, VIDEODEPTH, 0.0F, 0.0F, 0.0F, 0.046875F * (Datablk.Data - 7), 0.0625F + 0.03125F * (LDep + 1))
        '    ctr += VIDEOVTXCOUNT

        'Next

        For Each OpSw In Q.RTDs
            vs(ctr) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 1) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 2) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 3) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 4) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 5) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(ctr + 6) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 7) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 8) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 9) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 10) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 11) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(ctr + 12) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 13) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 14) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 15) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 16) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 17) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

            vs(ctr + 18) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 19) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 20) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 21) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 22) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 23) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)

            vs(ctr + 24) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 25) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 26) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 27) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 28) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 29) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

            'LDep += dLDep

            vs(ctr + 30) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 31) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 32) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 33) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 34) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 35) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)

            vs(ctr + 36) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 37) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 38) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 39) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 40) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 41) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)

            vs(ctr + 42) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 43) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 44) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 45) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 46) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 47) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(ctr + 48) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 49) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 50) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 51) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 52) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 53) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(ctr + 54) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 55) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 56) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 57) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 58) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 59) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(ctr + 60) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 61) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 62) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 63) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 64) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 65) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 66) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 67) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 68) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 69) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 70) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 71) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)

            If OpSw.Switch.Left > OpSw.Gate.Left Then
                vs(ctr + 72) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 73) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 74) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 75) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 76) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 77) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            Else
                vs(ctr + 72) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 73) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 74) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 75) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 76) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 77) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            End If

            If OpSw.Switch.Top < OpSw.Gate.Top Then
                vs(ctr + 78) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 79) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 80) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 81) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 82) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 83) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            Else
                vs(ctr + 78) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 79) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 80) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 81) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 82) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 83) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            End If

            vs(ctr + 84) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 85) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 86) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 87) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 88) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 89) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 90) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 91) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 92) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 93) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 94) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 95) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)

            ctr += RTDVTXCOUNT
            'LDep += dLDep


        Next

        For Each OpSw In Q.Pneumos
            vs(ctr) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 1) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 2) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 3) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 4) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 5) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(ctr + 6) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 7) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 8) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 9) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 10) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 11) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)

            vs(ctr + 12) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 13) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 14) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 15) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 16) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 17) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

            vs(ctr + 18) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 19) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 20) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 21) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 22) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Top, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 23) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)

            vs(ctr + 24) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 25) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 26) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 27) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 28) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Right, OpSw.Switch.Bottom, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 29) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left, OpSw.Switch.Bottom, JBDDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)

            'LDep += dLDep

            vs(ctr + 30) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 31) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 32) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 33) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 34) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 35) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)

            vs(ctr + 36) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 37) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 38) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 39) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 40) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 41) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)

            vs(ctr + 42) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 43) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 44) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 45) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 46) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 47) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(ctr + 48) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 49) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 50) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 51) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 52) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Top, WALLDEPTH, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 53) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)

            vs(ctr + 54) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 55) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 56) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 57) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 58) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Right, OpSw.Gate.Bottom, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 1.0F)
            vs(ctr + 59) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left, OpSw.Gate.Bottom, WALLDEPTH, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            'vs(ctr + 60) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 61) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 62) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + 20, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 63) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + 20, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 64) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + 20, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 65) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            'vs(ctr + 66) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Gate.Top + OpSw.Gate.Height / 2, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 67) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + 20, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 68) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top + OpSw.Gate.Height / 2, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 69) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + 20, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 70) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top + OpSw.Gate.Height / 2, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            'vs(ctr + 71) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + 20, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)

            vs(ctr + 60) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 61) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 62) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 63) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 64) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 65) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 66) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 67) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 68) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 69) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 70) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, JBDCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 71) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)

            If OpSw.Switch.Left > OpSw.Gate.Left Then
                vs(ctr + 72) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 73) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 74) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 75) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 76) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 77) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            Else
                vs(ctr + 72) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 73) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 74) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 75) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 76) = New CustomVertex.PositionNormalTextured(OpSw.Switch.Left + 8, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 77) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 6, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            End If

            If OpSw.Switch.Top < OpSw.Gate.Top Then
                vs(ctr + 78) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 79) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 80) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 81) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 82) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 83) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            Else
                vs(ctr + 78) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
                vs(ctr + 79) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
                vs(ctr + 80) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 81) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
                vs(ctr + 82) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 6, OpSw.Switch.Top + 10, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
                vs(ctr + 83) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 10, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            End If

            vs(ctr + 84) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 85) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 86) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 87) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 88) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 89) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 90) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 91) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)
            vs(ctr + 92) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 93) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.125F)
            vs(ctr + 94) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top, WALLCLOSENESS, 0.0F, 0.0F, 0.0F, 0.5F, 0.0F)
            vs(ctr + 95) = New CustomVertex.PositionNormalTextured(OpSw.Gate.Left + 8, OpSw.Gate.Top + 4, WALLCLOSENESS + dZWire, 0.0F, 0.0F, 0.0F, 0.0F, 0.125F)

            ctr += PNEUMOVTXCOUNT
            'LDep += dLDep
        Next

        If Upgrade.Width > 0 Then
            LDep = Me.Fmd(Tick, 60) / 30.0F
            If LDep >= 1.0F Then
                LDep = 2.0F - LDep
            End If
            vs(ctr) = New CustomVertex.PositionNormalTextured(Upgrade.Left, Upgrade.Top, 60.0F * LDep - 30.0F, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
            vs(ctr + 1) = New CustomVertex.PositionNormalTextured(Upgrade.Left, Upgrade.Bottom, 60.0F * LDep - 30.0F, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
            vs(ctr + 2) = New CustomVertex.PositionNormalTextured(Upgrade.Right, Upgrade.Top, 60.0F * LDep - 30.0F, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 3) = New CustomVertex.PositionNormalTextured(Upgrade.Right, Upgrade.Bottom, 60.0F * LDep - 30.0F, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
            vs(ctr + 4) = New CustomVertex.PositionNormalTextured(Upgrade.Right, Upgrade.Top, 60.0F * LDep - 30.0F, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
            vs(ctr + 5) = New CustomVertex.PositionNormalTextured(Upgrade.Left, Upgrade.Bottom, 60.0F * LDep - 30.0F, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
        End If


        NormalizeTriangles(vs, PrimitiveType.TriangleList)

        vxbf.Unlock()
    End Sub
    Private Sub SetSky(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.TransformedColoredTextured

        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        vs(0) = New CustomVertex.TransformedColoredTextured(0.0F, 0.0F, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(1) = New CustomVertex.TransformedColoredTextured(0.0F, Me.Height, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(2) = New CustomVertex.TransformedColoredTextured(Me.Width, 0.0F, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(3) = New CustomVertex.TransformedColoredTextured(Me.Width, Me.Height, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 1.0F)
        vs(4) = vs(2)
        vs(5) = vs(1)

        vxbf.Unlock()
    End Sub
    Private Sub SetBackgrd(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.PositionNormalTextured

        Const EXITCLOSENESS As Single = 0.0F

        vs = DirectCast(A, CustomVertex.PositionNormalTextured())

        'Background
        vs(0) = New CustomVertex.PositionNormalTextured(-500.0F, -1000.0F, BACKGROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
        vs(1) = New CustomVertex.PositionNormalTextured(-500.0F, 1200.0F, BACKGROUNDDEPTH, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
        vs(2) = New CustomVertex.PositionNormalTextured(15000.0F, -1000.0F, BACKGROUNDDEPTH, 0.0F, 0.0F, 0.0F, 4.0F, 0.0F)
        vs(3) = New CustomVertex.PositionNormalTextured(15000.0F, 1200.0F, BACKGROUNDDEPTH, 0.0F, 0.0F, 0.0F, 4.0F, 1.0F)
        vs(4) = vs(2)
        vs(5) = vs(1)

        vs(6) = New CustomVertex.PositionNormalTextured(Exitdoor.Left, Exitdoor.Top, EXITCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F)
        vs(7) = New CustomVertex.PositionNormalTextured(Exitdoor.Left, Exitdoor.Bottom, EXITCLOSENESS, 0.0F, 0.0F, 0.0F, 0.0F, 1.0F)
        vs(8) = New CustomVertex.PositionNormalTextured(Exitdoor.Right, Exitdoor.Top, EXITCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 0.0F)
        vs(9) = New CustomVertex.PositionNormalTextured(Exitdoor.Right, Exitdoor.Bottom, EXITCLOSENESS, 0.0F, 0.0F, 0.0F, 1.0F, 1.0F)
        vs(10) = vs(8)
        vs(11) = vs(7)

        NormalizeTriangles(vs, PrimitiveType.TriangleList)

        vxbf.Unlock()
    End Sub

    Private Sub SetIPGeometry(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.TransformedColoredTextured
        Dim C As Integer
        C = CtrTwo - 200
        If C < 0 Then
            C = 0
        End If

        Dim QLeft, QTop, QRight, QDown As Integer
        If CtrTwo > 600 Then
            QLeft = Me.Width \ 4
            QTop = Me.Height \ 4
            QRight = Me.Width - QLeft
            QDown = Me.Height - QTop
        Else
            QLeft = Me.Width \ 2 - (C * Me.Width \ 1600)
            QTop = Me.Height \ 2 - (C * Me.Height \ 1600)
            QRight = Me.Width - QLeft
            QDown = Me.Height - QTop
        End If



        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        vs(0) = New CustomVertex.TransformedColoredTextured(QLeft, QTop, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(1) = New CustomVertex.TransformedColoredTextured(QLeft, QDown, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(2) = New CustomVertex.TransformedColoredTextured(QRight, QTop, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(3) = New CustomVertex.TransformedColoredTextured(QRight, QDown, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 1.0F)
        vs(4) = vs(2)
        vs(5) = vs(1)

        vxbf.Unlock()

    End Sub
    Private Sub SetTitleGeometry(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.TransformedColoredTextured
        Dim C As Integer
        Dim QLeft, QTop, QRight, QDown As Integer
        Dim LV, Ctr As Integer
        Dim dL As Single = 32.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F
        Dim ChrNum, ChrRow, ChrCol As Integer
        Dim Nums As Integer() '= New Integer(6) {}

        If CtrTwo < 600 Then
            QLeft = 0
            QTop = 600 - CtrTwo
            QRight = Me.Width
            QDown = QTop + Me.Height \ 2
        Else
            QLeft = 0
            QTop = 0
            QRight = Me.Width
            QDown = Me.Height \ 2
        End If


        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        vs(0) = New CustomVertex.TransformedColoredTextured(QLeft, QTop, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(1) = New CustomVertex.TransformedColoredTextured(QLeft, QDown, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(2) = New CustomVertex.TransformedColoredTextured(QRight, QTop, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(3) = New CustomVertex.TransformedColoredTextured(QRight, QDown, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 1.0F)
        vs(4) = vs(2)
        vs(5) = vs(1)

        Nums = New Integer() {32, 78, 69, 87, 32, 32, 32, 32} '... NEW    ...
        If Selexion = 0 Then
            Nums(0) = 64
        End If
        For LV = 0 To 4
            ChrNum = Nums(LV)
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 6) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 7) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 8) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 9) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 10) = vs(6 * LV + 8)
            vs(6 * LV + 11) = vs(6 * LV + 7)
        Next
        Nums = New Integer() {32, 76, 79, 65, 68, 32, 32, 32} '... LOAD   ...
        If Selexion = 1 Then
            Nums(0) = 64
        End If
        For LV = 0 To 4
            ChrNum = Nums(LV)
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 36) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 37) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 38) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 39) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 40) = vs(6 * LV + 38)
            vs(6 * LV + 41) = vs(6 * LV + 37)
        Next
        Nums = New Integer() {32, 79, 80, 84, 73, 79, 78, 83} '... OPTIONS...
        If Selexion = 2 Then
            Nums(0) = 64
        End If
        For LV = 0 To 7
            ChrNum = Nums(LV)
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 66) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 67) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 68) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 69) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 70) = vs(6 * LV + 68)
            vs(6 * LV + 71) = vs(6 * LV + 67)
        Next
        Nums = New Integer() {32, 81, 85, 73, 84} '... QUIT...
        If Selexion = 3 Then
            Nums(0) = 64
        End If
        For LV = 0 To 4
            ChrNum = Nums(LV)
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 114) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 115) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 116) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 117) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 118) = vs(6 * LV + 116)
            vs(6 * LV + 119) = vs(6 * LV + 115)
        Next
        vxbf.Unlock()

    End Sub
    Private Sub SetCloseTitleGeometry(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.TransformedColoredTextured
        Dim C As Integer
        Dim QLeft, QTop, QRight, QDown As Integer
        Dim LV, Ctr As Integer
        Dim dL As Single = 32.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F
        Dim ChrNum, ChrRow, ChrCol As Integer
        Dim Nums As Integer() = New Integer(4) {}

        If CtrTwo < 200 Then
            QLeft = CtrTwo * Me.Width \ 400
            QTop = CtrTwo * Me.Height \ 800
            QRight = Me.Width - QLeft
            QDown = Me.Height \ 2 - QTop
        Else
            QLeft = 0
            QTop = 0
            QRight = 0
            QDown = 0
        End If


        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        vs(0) = New CustomVertex.TransformedColoredTextured(QLeft, QTop, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(1) = New CustomVertex.TransformedColoredTextured(QLeft, QDown, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(2) = New CustomVertex.TransformedColoredTextured(QRight, QTop, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(3) = New CustomVertex.TransformedColoredTextured(QRight, QDown, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 1.0F)
        vs(4) = vs(2)
        vs(5) = vs(1)

        If Selexion = 0 Then
            Nums = New Integer() {64, 78, 69, 87, 32} '... NEW ...
            For LV = 0 To 4
                ChrNum = Nums(LV)
                ChrRow = ChrNum \ 16
                ChrCol = ChrNum Mod 16
                vs(6 * LV + 6) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
                vs(6 * LV + 7) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                vs(6 * LV + 8) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                vs(6 * LV + 9) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                vs(6 * LV + 10) = vs(6 * LV + 8)
                vs(6 * LV + 11) = vs(6 * LV + 7)
            Next
        End If
        If Selexion = 1 Then
            Nums = New Integer() {64, 76, 79, 65, 68} '... LOAD...
            For LV = 0 To 4
                ChrNum = Nums(LV)
                ChrRow = ChrNum \ 16
                ChrCol = ChrNum Mod 16
                vs(6 * LV + 6) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
                vs(6 * LV + 7) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                vs(6 * LV + 8) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                vs(6 * LV + 9) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                vs(6 * LV + 10) = vs(6 * LV + 8)
                vs(6 * LV + 11) = vs(6 * LV + 7)
            Next
        End If
        If Selexion = 2 Then
            Nums = New Integer() {64, 81, 85, 73, 84} '... QUIT...
            For LV = 0 To 4
                ChrNum = Nums(LV)
                ChrRow = ChrNum \ 16
                ChrCol = ChrNum Mod 16
                vs(6 * LV + 6) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
                vs(6 * LV + 7) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                vs(6 * LV + 8) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                vs(6 * LV + 9) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                vs(6 * LV + 10) = vs(6 * LV + 8)
                vs(6 * LV + 11) = vs(6 * LV + 7)
            Next
        End If
        vxbf.Unlock()
    End Sub
    Private Sub SetHelicopter(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.PositionColoredTextured
        Dim C As Integer, X As Single, R As Integer
        Dim QLeft, QTop As Single

        C = CtrTwo Mod 6


        If CtrTwo >= 480 AndAlso CtrTwo < 500 Then
            QLeft = 2400.0F
            QTop = -1200.0F
            X = (CtrTwo - 480.0F) * 20.0F
        ElseIf CtrTwo < 480 Then
            QLeft = 5.0F * CtrTwo
            QTop = -1200.0F
            X = 0
        ElseIf CtrTwo >= 500 Then
            QLeft = 2400.0F
            QTop = -1200.0F
            X = 400.0F
        End If

        If CtrTwo < 980 Then
            R = 0
        ElseIf CtrTwo >= 980 AndAlso CtrTwo <= 1080 Then
            R = (CtrTwo - 980) * 5
            'R = 1
        End If


        vs = DirectCast(A, CustomVertex.PositionColoredTextured())

        vs(0) = New CustomVertex.PositionColoredTextured(QLeft + R, QTop, 0.0F, &HFFFFFFFF, 0.0F, 0.166666672F * C)
        vs(1) = New CustomVertex.PositionColoredTextured(QLeft + R, QTop + 131.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.166666672F * (C + 1))
        vs(2) = New CustomVertex.PositionColoredTextured(QLeft + 414.0F + R, QTop, 0.0F, &HFFFFFFFF, 1.0F, 0.166666672F * C)
        vs(3) = New CustomVertex.PositionColoredTextured(QLeft + 414.0F + R, QTop + 131.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.166666672F * (C + 1))
        vs(4) = vs(2)
        vs(5) = vs(1)

        vs(6) = New CustomVertex.PositionColoredTextured(2600 + R, -1100, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(7) = New CustomVertex.PositionColoredTextured(2600 + R, -1100 + X, 0.0F, &HFFFFFFFF, 0.0F, X / 400.0F)
        vs(8) = New CustomVertex.PositionColoredTextured(2604 + R, -1100, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(9) = New CustomVertex.PositionColoredTextured(2604 + R, -1100 + X, 0.0F, &HFFFFFFFF, 1.0F, X / 400.0F)
        vs(10) = vs(8)
        vs(11) = vs(7)


        vxbf.Unlock()
    End Sub

    Private Sub SetOptionsGeometry(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.TransformedColoredTextured
        Dim C As Integer
        Dim QLeft, QTop, QRight, QDown As Integer
        Dim LV, Ctr As Integer
        Dim dL As Single = 32.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F
        Dim ChrNum, ChrRow, ChrCol As Integer
        Dim Nums As Integer() '= New Integer(6) {}

        QLeft = 0
        QTop = 0
        QRight = Me.Width
        QDown = Me.Height \ 2


        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        vs(0) = New CustomVertex.TransformedColoredTextured(QLeft, QTop, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(1) = New CustomVertex.TransformedColoredTextured(QLeft, QDown, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(2) = New CustomVertex.TransformedColoredTextured(QRight, QTop, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(3) = New CustomVertex.TransformedColoredTextured(QRight, QDown, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 1.0F)
        vs(4) = vs(2)
        vs(5) = vs(1)

        Nums = New Integer() {32, 83, 70, 88, 32, 86, 79, 76, 85, 77, 69} '... SFX VOLUME...
        If Selexion = 0 Then
            Nums(0) = 64
        End If
        For LV = 0 To 10
            ChrNum = Nums(LV)
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 6) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown, 0.0F, 1.0F, &HFFFF00FF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 7) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 8) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown, 0.0F, 1.0F, &HFFFF00FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 9) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 10) = vs(6 * LV + 8)
            vs(6 * LV + 11) = vs(6 * LV + 7)
        Next
        vs(72) = New CustomVertex.TransformedColoredTextured(64 + dL * 14, QDown, 0.0F, 1.0F, &HFF0040FF, 0.0F, 0.0F)
        vs(73) = New CustomVertex.TransformedColoredTextured(64 + dL * 14, QDown + dL, 0.0F, 1.0F, &HFF007FFF, 0.0F, 0.125F)
        vs(74) = New CustomVertex.TransformedColoredTextured(64 + dL * 14 + 2 * Bbx.WaveVol, QDown, 0.0F, 1.0F, &HFF0040FF, 1.0F, 0.0F)
        vs(75) = New CustomVertex.TransformedColoredTextured(64 + dL * 14 + 2 * Bbx.WaveVol, QDown + dL, 0.0F, 1.0F, &HFF007FFF, 1.0F, 0.125F)
        vs(76) = vs(74)
        vs(77) = vs(73)

        Nums = New Integer() {32, 66, 71, 77, 32, 86, 79, 76, 85, 77, 69} '... BGM VOLUME...
        If Selexion = 1 Then
            Nums(0) = 64
        End If
        For LV = 0 To 10
            ChrNum = Nums(LV)
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 78) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, &HFFFF00FF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 79) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 80) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, &HFFFF00FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 81) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 82) = vs(6 * LV + 80)
            vs(6 * LV + 83) = vs(6 * LV + 79)
        Next
        vs(144) = New CustomVertex.TransformedColoredTextured(64 + dL * 14, QDown + dL, 0.0F, 1.0F, &HFF0040FF, 0.0F, 0.0F)
        vs(145) = New CustomVertex.TransformedColoredTextured(64 + dL * 14, QDown + dL + dL, 0.0F, 1.0F, &HFF00FFFF, 0.0F, 0.125F)
        vs(146) = New CustomVertex.TransformedColoredTextured(64 + dL * 14 + 2 * Bbx.MusicVol, QDown + dL, 0.0F, 1.0F, &HFF0040FF, 1.0F, 0.0F)
        vs(147) = New CustomVertex.TransformedColoredTextured(64 + dL * 14 + 2 * Bbx.MusicVol, QDown + dL + dL, 0.0F, 1.0F, &HFF00FFFF, 1.0F, 0.125F)
        vs(148) = vs(146)
        vs(149) = vs(145)

        Nums = New Integer() {32, 67, 79, 78, 84, 82, 79, 76, 83} '... CONTROLS...
        If Selexion = 2 Then
            Nums(0) = 64
        End If
        For LV = 0 To 8
            ChrNum = Nums(LV)
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 150) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, &HFFFF00FF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 151) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 152) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, &HFFFF00FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 153) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 154) = vs(6 * LV + 152)
            vs(6 * LV + 155) = vs(6 * LV + 151)
        Next '203 -> 204


        'Nums = New Integer() {32, 79, 80, 84, 73, 79, 78, 83} '... OPTIONS...
        'If Selexion = 2 Then
        '    Nums(0) = 64
        'End If
        'For LV = 0 To 7
        '    ChrNum = Nums(LV)
        '    ChrRow = ChrNum \ 16
        '    ChrCol = ChrNum Mod 16
        '    vs(6 * LV + 66) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
        '    vs(6 * LV + 67) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
        '    vs(6 * LV + 68) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
        '    vs(6 * LV + 69) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
        '    vs(6 * LV + 70) = vs(6 * LV + 68)
        '    vs(6 * LV + 71) = vs(6 * LV + 67)
        'Next
        'Nums = New Integer() {32, 81, 85, 73, 84} '... QUIT...
        'If Selexion = 3 Then
        '    Nums(0) = 64
        'End If
        'For LV = 0 To 4
        '    ChrNum = Nums(LV)
        '    ChrRow = ChrNum \ 16
        '    ChrCol = ChrNum Mod 16
        '    vs(6 * LV + 114) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
        '    vs(6 * LV + 115) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
        '    vs(6 * LV + 116) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
        '    vs(6 * LV + 117) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL + dL + dL, 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
        '    vs(6 * LV + 118) = vs(6 * LV + 116)
        '    vs(6 * LV + 119) = vs(6 * LV + 115)
        'Next
        vxbf.Unlock()


    End Sub
    Private Sub SetControlsGeometry(ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        Dim vs() As CustomVertex.TransformedColoredTextured
        Dim C As Integer
        Dim QLeft, QTop, QRight, QDown As Integer
        Dim LV, Ctr As Integer
        Dim dL As Single = 16.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F
        Dim ChrNum, ChrRow, ChrCol As Integer
        Dim Nums As Integer() '= New Integer(6) {}
        Dim Strg As String
        Dim CA, CB, CR, CS, CTop, CBtm As Integer

        CA = &HFF808080
        CB = &HFFC0C0C0
        CR = &HFFFFFFFF
        CS = &HFFEEEEEE

        QLeft = 0
        QTop = 0
        QRight = Me.Width
        QDown = Me.Height \ 2


        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        vs(0) = New CustomVertex.TransformedColoredTextured(QLeft, QTop, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(1) = New CustomVertex.TransformedColoredTextured(QLeft, QDown, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(2) = New CustomVertex.TransformedColoredTextured(QRight, QTop, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(3) = New CustomVertex.TransformedColoredTextured(QRight, QDown, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, 1.0F)
        vs(4) = vs(2)
        vs(5) = vs(1)

        CTop = CA
        CBtm = CB
        If Selexion = 0 Then
            Strg = "@UP   -" & UpKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " UP   -" & UpKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV + 6) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 7) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 8) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 9) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 10) = vs(6 * LV + 8)
            vs(6 * LV + 11) = vs(6 * LV + 7)
        Next
        Ctr = 6 * LV + 6

        CTop = CA
        CBtm = CB
        If Selexion = 1 Then
            Strg = "@DOWN -" & DownKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " DOWN -" & DownKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + dL + dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + dL + dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 2 Then
            Strg = "@LEFT -" & LeftKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " LEFT -" & LeftKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 2 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 3 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 2 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 3 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 3 Then
            Strg = "@RIGHT-" & RightKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " RIGHT-" & RightKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 3 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 4 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 3 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 4 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 4 Then
            Strg = "@JUMP -" & JumpKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " JUMP -" & JumpKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 4 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 5 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 4 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 5 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 5 Then
            Strg = "@FIRE -" & FireKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " FIRE -" & FireKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 5 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 6 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 5 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 6 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 6 Then
            Strg = "@RUN  -" & RunKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " RUN  -" & RunKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 6 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 7 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 6 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 7 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 7 Then
            Strg = "@MENU -" & MenuKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " MENU -" & MenuKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 7 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 8 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 7 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 8 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 8 Then
            Strg = "@USE  -" & UseDoorKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " USE  -" & UseDoorKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 8 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 9 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 8 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 9 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 9 Then
            Strg = "@WEAPON WHEEL UP  :" & WeaponNextKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " WEAPON WHEEL UP  :" & WeaponNextKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 9 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 10 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 9 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 10 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 10 Then
            Strg = "@WEAPON WHEEL DOWN:" & WeaponPrevKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " WEAPON WHEEL DOWN:" & WeaponPrevKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 10 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 11 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 10 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 11 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 11 Then
            Strg = "@BULLET WHEEL UP  :" & BulletNextKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " BULLET WHEEL UP  :" & BulletNextKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 11 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 12 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 11 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 12 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 12 Then
            Strg = "@BULLET WHEEL DOWN:" & BulletPrevKey.ToString()
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " BULLET WHEEL DOWN:" & BulletPrevKey.ToString()
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 12 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 13 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 12 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 13 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV


        CTop = CA
        CBtm = CB
        If Selexion = 13 Then
            Strg = "@GO BACK"
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " GO BACK"
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 13 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 14 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 13 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 14 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        CTop = CA
        CBtm = CB
        If Selexion = 14 Then
            Strg = "@SET TO DEFAULT"
            If ControlSet Then
                CTop = CR
                CBtm = CS
            Else
                CTop = CA
                CBtm = CB
            End If
        Else
            Strg = " SET TO DEFAULT"
        End If
        For LV = 0 To Strg.Length - 1
            ChrNum = Convert.ToInt32(Strg.Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 14 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(64 + dL * LV, QDown + 15 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 14 * dL, 0.0F, 1.0F, CTop, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(64 + dL * (LV + 1), QDown + 15 * dL, 0.0F, 1.0F, CBtm, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(Ctr + 6 * LV + 4) = vs(Ctr + 6 * LV + 2)
            vs(Ctr + 6 * LV + 5) = vs(Ctr + 6 * LV + 1)
        Next
        Ctr = Ctr + 6 * LV

        vxbf.Unlock()
    End Sub

    Private Function F(ByVal X As Single, ByVal Y As Single) As Single
        'Try out your own functions of X and Y.
        'Return 0.25F * X * X  'Z = 1/4 (X)... parabolic sheet of paper.
        'Return 0.25F * X * Y   'Origami?  Looks like the beginning of a paper bird.
        'Return 0.25F * X * X + 0.25F * Y * Y  'A hammock with an invisible person sitting in it.
        'Return Convert.ToSingle(Math.Sin(X * PiHalf))  'This looks more sinusoidal at a higher resolution.
        'Return 0.25F * X * X - 0.25F * Y * Y  'Saddle paper.
        'Return Convert.ToSingle(Math.Cos(X * PiHalf)) * Convert.ToSingle(Math.Sin(Y * PiHalf))  'Eggshell bed comforter.
        Return 0.0F

    End Function

    Private Sub Lighting()
        Dim LV, N As Integer
        'Of course, you cannot light an object that has no material -
        'we must define what type of light is reflected (what color we see)
        'from the material.
        'When drawing an object, this is the material that DirectX will put over it...
        'each triangle.
        If LevelMajor = 2 Then
            Wallpaper.Ambient = Color.FromArgb(255, 8, 12, 12)
            Wallpaper.Diffuse = Color.White
        ElseIf LevelMajor = 6 Then
            'Wallpaper.Ambient = Color.FromArgb(255, N, N, N)
            Wallpaper.Ambient = Color.Black
            Wallpaper.Diffuse = Color.White
        Else
            Wallpaper.Ambient = Color.Black
            Wallpaper.Diffuse = Color.White
        End If
        'Wallpaper.Ambient = Color.FromArgb(128, 204, 102, 21)
        'Wallpaper.Diffuse = Color.AntiqueWhite
        Luminous = Wallpaper
        If LevelMajor = 7 Then
            Luminous.Emissive = Color.FromArgb(255, 255, 224, 192)
        Else
            Luminous.Emissive = Color.White
        End If

        LavaMtrl.Ambient = Color.DarkRed
        LavaMtrl.Emissive = Color.Red

        WaterMtrl.Ambient = Color.DarkBlue
        WaterMtrl.Emissive = Color.DarkCyan

        GlasMtrl.Ambient = Color.DarkBlue
        GlasMtrl.Emissive = Color.LightGray

        'D9.Material = Wallpaper


        'This is where we hook up "electricity" to all of the lights here.
        'D9.Lights(0).Type = LightType.Point  'Regular lightbulb.
        'D9.Lights(0).Type = LightType.Point  'Regular lightbulb.
        'D9.Lights(0).Position = New Vector3(0.0F, 0.0F, 2.0F) 'Place the bulb here.
        'D9.Lights(0).Range = 97.6F
        'D9.Lights(0).Attenuation0 = 0.0F
        'D9.Lights(0).Attenuation1 = 0.45F
        'D9.Lights(0).Ambient = Color.White
        ''D9.Lights(0).Attenuation0 = 0.0F
        'D9.Lights(0).Enabled = True
        'D9.Lights(0).Commit()

        If LevelMajor = 0 Then
            D9.Lights(0).Type = LightType.Point
            D9.Lights(0).Direction = New Vector3(0.0F, 1.0F, 0.0F)
            D9.Lights(0).Ambient = Color.White
            D9.Lights(0).Attenuation0 = 0.0F
            D9.Lights(0).Attenuation1 = 0.00075F
            D9.Lights(0).Range = 7000
            D9.Lights(0).InnerConeAngle = PiFourth
            D9.Lights(0).OuterConeAngle = PiHalf
            D9.Lights(0).Position = New Vector3(Player.Loc.X, Player.Loc.Y - 20.0F, -100.0F)
            D9.Lights(0).Specular = Color.White
            D9.Lights(0).Diffuse = Color.White
            D9.Lights(0).Enabled = True
            'D9.Lights(0).
        ElseIf LevelMajor = 2 OrElse LevelMajor = 7 Then
            D9.Lights(0).Type = LightType.Point
            D9.Lights(0).Direction = New Vector3(0.0F, 1.0F, 1.0F)
            D9.Lights(0).Ambient = Color.LightGray
            D9.Lights(0).Attenuation0 = 0.0F
            D9.Lights(0).Attenuation1 = 0.00075F
            D9.Lights(0).Range = 7000
            D9.Lights(0).InnerConeAngle = PiFourth
            D9.Lights(0).OuterConeAngle = PiHalf
            D9.Lights(0).Position = New Vector3(Player.Loc.X, Player.Loc.Y - 20.0F, -100.0F)
            D9.Lights(0).Specular = Color.White
            D9.Lights(0).Diffuse = Color.White
            D9.Lights(0).Enabled = True
        ElseIf LevelMajor = 3 Then
            D9.Lights(0).Type = LightType.Point
            D9.Lights(0).Direction = New Vector3(0.0F, 1.0F, 1.0F)
            D9.Lights(0).Ambient = Color.Bisque
            D9.Lights(0).Attenuation0 = 0.0F
            D9.Lights(0).Attenuation1 = 0.00075F
            D9.Lights(0).Range = 7000
            D9.Lights(0).InnerConeAngle = PiFourth
            D9.Lights(0).OuterConeAngle = PiHalf
            D9.Lights(0).Position = New Vector3(Player.Loc.X, Player.Loc.Y - 20.0F, -100.0F)
            D9.Lights(0).Specular = Color.White
            D9.Lights(0).Diffuse = Color.White
            D9.Lights(0).Enabled = True
        ElseIf LevelMajor = 4 Then
            For LV = 0 To Q.Bulbs.Count - 1
                Backdrop = DirectCast(Q.Bulbs(LV), RectangleF)
                D9.Lights(LV).Type = LightType.Point
                D9.Lights(LV).Attenuation0 = 0.0F
                D9.Lights(LV).Range = 30000
                D9.Lights(LV).Attenuation1 = 0.0005F
                D9.Lights(LV).Attenuation2 = 0.0F
                D9.Lights(LV).Position = New Vector3(Backdrop.Left + BULBHALF, Backdrop.Top + BULBHALF, -100.0F)
                If LV Mod 2 = 1 Then
                    D9.Lights(LV).Ambient = Color.AliceBlue
                    D9.Lights(LV).Specular = Color.AliceBlue
                    D9.Lights(LV).Diffuse = Color.AliceBlue
                Else
                    D9.Lights(LV).Ambient = Color.LightSalmon
                    D9.Lights(LV).Specular = Color.LightSalmon
                    D9.Lights(LV).Diffuse = Color.LightSalmon
                End If
                D9.Lights(LV).Enabled = True
                'D9.Lights(LV).Commit()
            Next
        ElseIf LevelMajor = 5 Then
            D9.Lights(0).Type = LightType.Point
            D9.Lights(0).Direction = New Vector3(0.0F, 1.0F, 1.0F)
            D9.Lights(0).Ambient = Color.LightGoldenrodYellow
            D9.Lights(0).Attenuation0 = 0.0F
            D9.Lights(0).Attenuation1 = 0.00075F
            D9.Lights(0).Range = 7000
            D9.Lights(0).InnerConeAngle = PiFourth
            D9.Lights(0).OuterConeAngle = PiHalf
            D9.Lights(0).Position = New Vector3(Player.Loc.X, Player.Loc.Y - 20.0F, -100.0F)
            D9.Lights(0).Specular = Color.White
            D9.Lights(0).Diffuse = Color.White
            D9.Lights(0).Enabled = True
        ElseIf LevelMajor = 6 Then
            D9.Lights(0).Type = LightType.Point
            D9.Lights(0).Direction = New Vector3(0.0F, 1.0F, 1.0F)
            'D9.Lights(0).Ambient = Color.White
            D9.Lights(0).Attenuation1 = Convert.ToSingle(0.001R * Math.Sin(Tick / 60.0R) + 0.001R)
            D9.Lights(0).Attenuation0 = 0.0F
            D9.Lights(0).Attenuation2 = 0.0F
            D9.Lights(0).Range = 7000
            D9.Lights(0).InnerConeAngle = PiFourth
            D9.Lights(0).OuterConeAngle = PiHalf
            D9.Lights(0).Position = New Vector3(Player.Loc.X, Player.Loc.Y - 20.0F, -100.0F)
            'N = Convert.ToInt32(120.0R * Math.Sin(Tick / 60.0R) + 120)
            'SR.WriteLine(N)
            D9.Lights(0).Ambient = Color.LightGray
            D9.Lights(0).Diffuse = Color.White
            D9.Lights(0).Specular = Color.FromArgb(255, 255, 0, 123)
            D9.Lights(0).Enabled = True

        Else
            For LV = 0 To Q.Bulbs.Count - 1
                Backdrop = DirectCast(Q.Bulbs(LV), RectangleF)
                D9.Lights(LV).Type = LightType.Point
                D9.Lights(LV).Ambient = Color.Cornsilk
                D9.Lights(LV).Attenuation0 = 0.0F
                If LevelMajor = 8 Then
                    D9.Lights(LV).Range = 15000
                    D9.Lights(LV).Attenuation1 = 0.001F
                    D9.Lights(LV).Attenuation2 = 0.0F

                ElseIf LevelMinor = 1 Then
                    D9.Lights(LV).Range = 5000
                    D9.Lights(LV).Attenuation1 = 0.0F
                    D9.Lights(LV).Attenuation2 = 0.000005F

                Else
                    D9.Lights(LV).Range = 30000
                    D9.Lights(LV).Attenuation1 = 0.0005F
                    D9.Lights(LV).Attenuation2 = 0.0F
                End If
                D9.Lights(LV).Position = New Vector3(Backdrop.Left + BULBHALF, Backdrop.Top + BULBHALF, -100.0F)
                D9.Lights(LV).Specular = Color.Cornsilk
                D9.Lights(LV).Diffuse = Color.Cornsilk
                D9.Lights(LV).Enabled = True
                'D9.Lights(LV).Commit()
            Next
        End If


    End Sub
    Private Sub SetMatrices()
        If Player.VisionUp Then
            D9.Transform.Projection = Matrix.PerspectiveFovRH(1.25F, 1.0F, 0.1F, 1000.0F)
        Else
            D9.Transform.Projection = Matrix.PerspectiveFovRH(PiThird, 1.0F, 0.1F, 1000.0F)
            'Pi/3 is the aperture of our camera lens.
        End If
        D9.Transform.View = Matrix.LookAtRH(Camera, FocusPt, CamNormal)



        'Matrix rotation so that our tree rotates by 90.
        Mx(0) = Matrix.RotationX(PiHalf)
        'Stretch the tree so that it is four times as tall.
        Mx(0).Multiply(Matrix.Scaling(1.0F, 1.0F, 4.0F))
        'Move the tree east 2 units and south 2 units.
        Mx(0).Multiply(Matrix.Translation(2.0F, 2.0F, 0.0F))
    End Sub

    Private Sub TextureMeshSetup()
        Dim LV As Integer
        If D9 Is Nothing Then
            Throw New Exception("This doesn't make any sense, but your device is nothing")
        Else
            Bitmech = New Texture(106) {}
            PsTx = New Texture(7) {}
            For LV = 0 To 7
                PsTx(LV) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\c" & LV.ToString() & ".bmp", 64, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.None, &HFFFF00FF)
            Next


            'CustomFont = TextureLoader.FromFile(D9, Application.StartupPath & "\gfont4.bmp", 256, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)

            Bitmech(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sumult.bmp", 512, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\proto.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\bat.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\casukeball.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grimleaper.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shockwave.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\dispatch.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\electrico.bmp", 64, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\deepstep.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\greg.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\atacant.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\squirm.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\risingfire.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rim.bmp", 64, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\fedulus.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\gobblet.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\canonical.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\horolum.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\fly.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\spiderd.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rollingcamera.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(21) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\swivelcamera.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(22) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\motiondetect.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(23) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\liteball.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(24) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\homer.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(25) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pelogic.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(26) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\leach.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(27) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\paint.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(28) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\zone.bmp", 32, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
            Bitmech(29) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\gradibox.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
            Bitmech(30) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sauitek.bmp", 512, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(31) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\toggof.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
            Bitmech(32) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\icio.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(33) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\corven.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(34) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\hoiol.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(35) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\locoleaper.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(36) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\extinguish.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(37) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sscorpi.bmp", 256, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(38) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\vokat.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(39) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\upglist.bmp", 128, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
            Bitmech(40) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\modular.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(41) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\nape.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(42) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\explosio.bmp", 512, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(43) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shabom.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(44) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\caputaru.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(45) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\seso.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(46) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sheltur.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(47) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\surprisefish.bmp", 64, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(48) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\arroglestop.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(49) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\birdd.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(50) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\asterisk.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(51) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\golrb.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(52) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rychyd.bmp", 512, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(53) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sorry.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(54) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ejectent.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(55) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\locus.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(56) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sheila.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(57) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\electrico.bmp", 64, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(58) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pennygun.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(59) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shelshok.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(60) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\wallstep.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(61) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shish.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(62) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\darkfly.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(63) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\exploda.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(64) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pachinko.bmp", 64, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(65) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\bobeck.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(66) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\roto.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(67) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\lowstep.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(68) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forcemod.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(69) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kite.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(70) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\turnscan.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(71) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\gentu.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(72) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\notel.bmp", 512, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(73) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\hoper.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(74) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\gasgene.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(75) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\seancefly.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(76) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\upsilon.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(77) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sunsaws.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(78) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\stone.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(79) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\strafer.bmp", 64, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(80) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\aircanis.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(81) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\maser.bmp", 64, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(82) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\cloud.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(83) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\highstep.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(84) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ragladia.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(85) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\foulbeast.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(86) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forklift.bmp", 512, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(87) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\stroper.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(88) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\locolol.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(89) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\wallmine.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(90) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\seagat.bmp", 192, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(91) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\cemurdo.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(92) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shelis.bmp", 512, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(93) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\plasma.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(94) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\silvo.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(95) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\duletant.bmp", 64, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(96) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pyromist.bmp", 128, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(97) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\spcrock.bmp", 64, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF000000)
            Bitmech(98) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\remouzi.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(99) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\emouzi.bmp", 64, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(100) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\negation.bmp", 512, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(101) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\brassgol.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(102) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sprangala.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(103) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\strongcoffee.bmp", 128, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Bitmech(104) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\smith.bmp", 512, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFF00FF)
            Bitmech(105) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\homunc.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            Bitmech(106) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\final.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)

            Fltr = TextureLoader.FromFile(D9, Application.StartupPath & "\files\mystry.bmp", 16, 16, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Exdo = TextureLoader.FromFile(D9, Application.StartupPath & "\files\exit.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
            IPPicture = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ip.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            TitlePicture = TextureLoader.FromFile(D9, Application.StartupPath & "\files\platform3.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)
            BlockFont = TextureLoader.FromFile(D9, Application.StartupPath & "\files\nofade.bmp", 256, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
            Chopper = TextureLoader.FromFile(D9, Application.StartupPath & "\files\chpz.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)

            Tx = New Texture(20) {}
            Select Case LevelMajor
                Case 0
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sky9.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\johanne.bmp", 512, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\green066.jpg", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metalism.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\built.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\lblue065p.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metalism.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\brick041q.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-817.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\floorafa.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ld32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\roping.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\earthen.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\crating.bmp", 256, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdguard.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shkwvupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdguard.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                Case 1
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sky5.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\cinite.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\green066.jpg", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\wood090p2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\brick041p.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\lblue065p.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metalism.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\brick041q.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-817.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\floorafa.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ld32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\roping.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\earthen.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\crating.bmp", 256, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdguard.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\hdlshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\vizonupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sxtshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pblshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shabmupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shkwvupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    End Select
                Case 2
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\jetsky.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\tundra2.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\green066.jpg", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179s.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-702.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179i.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179.jpg", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-702.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-943.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\floorafa.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ld32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\roping.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\gtile1.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-943.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdguard.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1, 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\icebtupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sprshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammo5upg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\aqbltupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\vrtshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade

                    End Select
                Case 3
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sunseat.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ocean3.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\wood035.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\wood037.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179i.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\birch.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179.jpg", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-702.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\birch.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\discreet.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\red32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grayrope.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal028.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\widshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\timeupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\shtgnupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\scrwrupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\dfensupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\spltrupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    End Select
                Case 4
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sky5.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\gawsall.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal069.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal055.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179i.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179.jpg", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-702.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\birch.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\discreet.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ld32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grayrope.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal028.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\lgjmpupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\alpshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\bkhshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\frmodupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    End Select
                Case 5
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\jetsky.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\coness.jpg", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal069.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\lgray.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rock179i.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-702.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\birch.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\discreet.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ld32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grayrope.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\green066.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal028.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\mtoshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\wljmpupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\fastupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\bladeupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\bmrngupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    End Select
                Case 6
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\jetsky.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\boatview.jpg", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grasnoll.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-703.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\iceknoll.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-702.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\birch.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\brick041p.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\red32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grayrope.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sandwave.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-943.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammo5upg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sprtzupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\naplmupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\evadeupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\fivshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\vlyshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    End Select
                Case 7
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sunseat.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\burnup3.jpg", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\blkgrs.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-705.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\iceknoll.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metalism.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-705.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\birch.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\brick041p.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\red32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grayrope.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sandwave.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-943.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\fastupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\stoneupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\tetshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\abeshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\htrshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\flmpfupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    End Select
                Case 8
                    SkyTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\sky5.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    BackTx = TextureLoader.FromFile(D9, Application.StartupPath & "\files\skypure.jpg", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)
                    Tx(0) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\moonsurf.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ground - unused
                    Tx(1) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal069.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)   'platform
                    Tx(2) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-817.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'wall
                    Tx(3) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\uonlyp.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)       'jumpboard
                    Tx(4) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\iceknoll.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'ice
                    Tx(5) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)    '???
                    Tx(6) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metalism.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFF00FF00)  'trapplatform
                    Tx(7) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-817.jpg", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)  'slopedwall
                    Tx(8) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\forfloor.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(9) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\brick041p.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'extrawall
                    Tx(10) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\kver.bmp", 128, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'conveyor
                    Tx(11) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\red32.bmp", 32, 64, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFFFF)   'ladder
                    Tx(12) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\grayrope.bmp", 16, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)      'rope
                    Tx(13) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\back-817.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'ground
                    Tx(14) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pictoria.bmp", 512, 512, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, &HFFFFFF00) 'background things
                    Tx(15) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\metal025.jpg", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'crate
                    Tx(16) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\rtdboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'rtd and gate
                    Tx(18) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\pnmboat.bmp", 256, 256, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'pneumo and gate
                    Tx(19) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Tx(20) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\IB2.bmp", 128, 128, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0)   'backdrop
                    Select Case LevelMinor
                        Case 1
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\dbljpupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 2
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\plsmaupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 3
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammoupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 4
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\srmshupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 5
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\ammo5upg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case 6
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\lightupg.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                        Case Else
                            Tx(17) = TextureLoader.FromFile(D9, Application.StartupPath & "\files\mystry.bmp", 32, 32, 0, 0, Format.A8B8G8R8, Pool.Managed, Filter.Linear, Filter.Linear, 0) 'upgrade
                    End Select
            End Select

            Mov = Tx(2)

        End If
        'This loads the texture from the file.
        'Ironically, the other four procedures take over 10 nontrivial arguments.
        'D9.SetTexture(0, Tx(0))
        'Sets the active texture.  The 0 is up for discussion.

        Dim Ems() As ExtendedMaterial
        'Merely holds a material and a filename for a texture.

        'MasterMesh = Mesh.FromFile(Application.StartupPath & "\mball.x", MeshFlags.SoftwareProcessing, D9)

        'You can use the one above if you have a material and texture that you will be supplying to the mesh.

        'However, the material and texture can be included within the .x file.

        MM = New MeshPlus(Q.Floaters.Count - 1) {}
        For LV = 0 To MM.GetUpperBound(0)
            Floater = DirectCast(Q.Floaters(LV), LevFloater)
            MM(LV) = New MeshPlus(Application.StartupPath & "\dball.x", MeshFlags.Managed, D9, 3 * PiHalf, 0, PiHalf, FLOATERSIZE >> 2, FLOATERSIZE >> 2, 1.0F, (FLOATERSIZE >> 1) + Floater.Loc.X, (FLOATERSIZE >> 1) + Floater.Loc.Y, FLOATERDEPTH)
            'Create the mesh using our mesh creator class.

        Next

        'Mesh.FromFile("C:\tree.x", MeshFlags.Managed, D9, Ems)


        'All of this information will be used when drawing the mesh.
    End Sub

    Private Sub SetupFonts(ByVal vs() As CustomVertex.TransformedColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        'DisplayFont = New Direct3D.Font(D9, Me.Font)
        Dim LV, Ctr As Integer
        Dim dL As Single = 16.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F


        Dim ChrNum, ChrRow, ChrCol As Integer

        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        For Ctr = 0 To CusLines - 1
            For LV = 0 To Math.Min(CHARACTERCOUNT - 1, CusText(Ctr).Length - 1)
                ChrNum = Convert.ToInt32(CusText(Ctr).Chars(LV))
                ChrRow = ChrNum \ 16
                ChrCol = ChrNum Mod 16
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFF888888, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 4) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 5) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1)
            Next
        Next

        vxbf.Unlock()

    End Sub
    Private Sub SetupLoaderFonts(ByVal vs() As CustomVertex.TransformedColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        'DisplayFont = New Direct3D.Font(D9, Me.Font)
        Dim LV, Ctr As Integer
        Dim dL As Single = 16.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F

        Dim ChrNum, ChrRow, ChrCol As Integer

        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        For Ctr = 0 To 11
            For LV = 0 To Math.Min(CHARACTERCOUNT - 1, CusText(Ctr).Length - 1)
                ChrNum = Convert.ToInt32(CusText(Ctr).Chars(LV))
                ChrRow = ChrNum \ 16
                ChrCol = ChrNum Mod 16
                If Ctr + TopIdx - 2 = Selexion Then
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 4) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 5) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1)
                Else
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFF888888, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 4) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 5) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1)
                End If
            Next
        Next

        vxbf.Unlock()
    End Sub
    Private Sub SetupPauseMenuFonts(ByVal vs() As CustomVertex.TransformedColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim LV, Ctr As Integer
        Dim dL As Single = 16.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F

        Dim ChrNum, ChrRow, ChrCol As Integer

        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        For Ctr = 0 To 7
            For LV = 0 To Math.Min(CHARACTERCOUNT - 1, CusText(Ctr).Length - 1)
                ChrNum = Convert.ToInt32(CusText(Ctr).Chars(LV))
                ChrRow = ChrNum \ 16
                ChrCol = ChrNum Mod 16
                If Ctr = Selexion + 2 Then
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFFFFFF00, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 4) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 5) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1)
                Else
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFF888888, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 4) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2)
                    vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 5) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1)
                End If
            Next
        Next

        vxbf.Unlock()

    End Sub
    Private Sub SetupIntroFonts(ByVal vs() As CustomVertex.TransformedColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        'DisplayFont = New Direct3D.Font(D9, Me.Font)
        Dim LV, Ctr As Integer
        Const dL As Integer = 64
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F
        Dim Swing As Boolean = True

        Const LEVELSTRINGLENGTH As Integer = 10
        Const YOFFSET As Integer = 160
        Const CTRPOINT1 As Integer = (dL * 10 + (400 - dL * 5)) \ 16
        Const CTRPOINT2 As Integer = CTRPOINT1 + 40
        Dim XOffset As Integer
        If CtrTwo < CTRPOINT1 Then
            XOffset = MAPWIDTH - (CtrTwo << 4)
        ElseIf CtrTwo < CTRPOINT2 Then
            XOffset = 80
            Swing = False
        Else
            XOffset = MAPWIDTH - ((CtrTwo - 40) << 4)
        End If

        Dim ChrNum, ChrRow, ChrCol As Integer

        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())


        For LV = 0 To LEVELSTRINGLENGTH - 1
            ChrNum = Convert.ToInt32(CusText(Ctr).Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            If Swing Then
                vs(6 * LV) = New CustomVertex.TransformedColoredTextured(XOffset + dL * LV + 40, YOFFSET + dL * Ctr, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
                vs(6 * LV + 1) = New CustomVertex.TransformedColoredTextured(XOffset + dL * LV, YOFFSET + dL * (Ctr + 1), 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                vs(6 * LV + 2) = New CustomVertex.TransformedColoredTextured(XOffset + dL * (LV + 1) + 40, YOFFSET + dL * Ctr, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                vs(6 * LV + 3) = New CustomVertex.TransformedColoredTextured(XOffset + dL * (LV + 1), YOFFSET + dL * (Ctr + 1), 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            Else
                vs(6 * LV) = New CustomVertex.TransformedColoredTextured(XOffset + dL * LV, YOFFSET + dL * Ctr, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F, ChrRow / 8.0F)
                vs(6 * LV + 1) = New CustomVertex.TransformedColoredTextured(XOffset + dL * LV, YOFFSET + dL * (Ctr + 1), 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                vs(6 * LV + 2) = New CustomVertex.TransformedColoredTextured(XOffset + dL * (LV + 1), YOFFSET + dL * Ctr, 0.0F, 1.0F, &HFF0000FF, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                vs(6 * LV + 3) = New CustomVertex.TransformedColoredTextured(XOffset + dL * (LV + 1), YOFFSET + dL * (Ctr + 1), 0.0F, 1.0F, &HFF00FFFF, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            End If
            vs(6 * LV + 4) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2)
            vs(6 * LV + 5) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1)
        Next

        vxbf.Unlock()

    End Sub
    Private Sub SetupLevelXfer(ByVal vs() As CustomVertex.TransformedColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        'DisplayFont = New Direct3D.Font(D9, Me.Font)
        Dim LV, Ctr As Integer
        Dim dL As Single = 16.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F
        Dim dS As Single = 32.0F

        Dim ChrNum, ChrRow, ChrCol As Integer
        Dim TextEnd As Integer
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        Ctr = 0

        'SR.WriteLine(vs.GetUpperBound(0))

        For LV = 0 To Math.Min(CHARACTERCOUNT - 1, CusText(Ctr).Length - 1)
            'SR.WriteLine(dL)
            'SR.WriteLine(LV)
            'SR.WriteLine(Ctr)
            'SR.WriteLine(ChrCol)
            'SR.WriteLine(ChrRow)
            ChrNum = Convert.ToInt32(CusText(Ctr).Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFF888888, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 4) = vs(6 * LV + 2)
            vs(6 * LV + 5) = vs(6 * LV + 1)

        Next
        TextEnd = CHARACTERCOUNT * 6

        For Ctr = 0 To 7
            For LV = 0 To 7

                If IsLevelClear(Ctr + 1, LV + 1) Then
                    If Ctr = Me.ZoneFromSelexionEx(Selexion) AndAlso LV = Me.SubsecFromSelexionEx(Selexion) Then
                        vs(TextEnd + 6 * LV + 48 * Ctr) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * Ctr, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 1) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFFFFFFFF, 0.0F, (Ctr + 1) / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 2) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * Ctr, 0.0F, 1.0F, &HFFFFFFFF, 1.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 3) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFFFFFFFF, 1.0F, (Ctr + 1) / 8.0F)
                    Else
                        vs(TextEnd + 6 * LV + 48 * Ctr) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * Ctr, 0.0F, 1.0F, &HFF888888, 0.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 1) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFF888888, 0.0F, (Ctr + 1) / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 2) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * Ctr, 0.0F, 1.0F, &HFF888888, 1.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 3) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFF888888, 1.0F, (Ctr + 1) / 8.0F)
                    End If
                Else
                    If Ctr = Me.ZoneFromSelexionEx(Selexion) AndAlso LV = Me.SubsecFromSelexionEx(Selexion) Then
                        vs(TextEnd + 6 * LV + 48 * Ctr) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * Ctr, 0.0F, 1.0F, &HFF551111, 0.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 1) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFF551111, 0.0F, (Ctr + 1) / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 2) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * Ctr, 0.0F, 1.0F, &HFF551111, 1.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 3) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFF551111, 1.0F, (Ctr + 1) / 8.0F)
                    Else
                        vs(TextEnd + 6 * LV + 48 * Ctr) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * Ctr, 0.0F, 1.0F, &HFF111111, 0.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 1) = New CustomVertex.TransformedColoredTextured(100 + dS * LV, 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFF111111, 0.0F, (Ctr + 1) / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 2) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * Ctr, 0.0F, 1.0F, &HFF111111, 1.0F, Ctr / 8.0F)
                        vs(TextEnd + 6 * LV + 48 * Ctr + 3) = New CustomVertex.TransformedColoredTextured(100 + dS * (LV + 1), 100 + dS * (Ctr + 1), 0.0F, 1.0F, &HFF111111, 1.0F, (Ctr + 1) / 8.0F)
                    End If
                End If
                vs(TextEnd + 6 * LV + 48 * Ctr + 4) = vs(TextEnd + 6 * LV + 48 * Ctr + 2)
                vs(TextEnd + 6 * LV + 48 * Ctr + 5) = vs(TextEnd + 6 * LV + 48 * Ctr + 1)
            Next
        Next

        TextEnd = TextEnd + 48 * Ctr
        vs(TextEnd) = New CustomVertex.TransformedColoredTextured(60, 60, 0.0F, 1.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(TextEnd + 1) = New CustomVertex.TransformedColoredTextured(60, 396, 0.0F, 1.0F, &HFFDDDDDD, 0.0F, 1.0F)
        vs(TextEnd + 2) = New CustomVertex.TransformedColoredTextured(396, 60, 0.0F, 1.0F, &HFFDDDDDD, 1.0F, 0.0F)
        vs(TextEnd + 3) = New CustomVertex.TransformedColoredTextured(396, 396, 0.0F, 1.0F, &HFF888888, 1.0F, 1.0F)
        vs(TextEnd + 4) = vs(TextEnd + 2)
        vs(TextEnd + 5) = vs(TextEnd + 1)


        vxbf.Unlock()

    End Sub
    Private Sub SetupSaveFonts(ByVal vs() As CustomVertex.TransformedColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        Dim LV, Ctr As Integer
        Dim dL As Single = 16.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F

        Dim ChrNum, ChrRow, ChrCol As Integer

        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        For Ctr = 0 To 7
            For LV = 0 To Math.Min(CHARACTERCOUNT - 1, CusText(Ctr).Length - 1)
                ChrNum = Convert.ToInt32(CusText(Ctr).Chars(LV))
                ChrRow = ChrNum \ 16
                ChrCol = ChrNum Mod 16
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F, ChrRow / 8.0F + dR)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F + dC, ChrRow / 8.0F)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFF888888, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 4) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 2)
                vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 5) = vs(CHARACTERCOUNT * 6 * Ctr + 6 * LV + 1)
            Next
        Next

        vxbf.Unlock()

    End Sub
    Private Sub SetupWeaponSelect(ByVal vs() As CustomVertex.TransformedColoredTextured, ByVal vxbf As VertexBuffer, ByVal D9 As Device)
        'DisplayFont = New Direct3D.Font(D9, Me.Font)
        Dim LV, Ctr As Integer
        Dim dL As Single = 16.0F
        Dim dR As Single = 1 / 8.0F, dC As Single = 1 / 16.0F
        Dim dS As Single = 32.0F
        Dim LeftOffsetForArrow As Single

        Dim ChrNum, ChrRow, ChrCol As Integer
        Dim TextEnd As Integer
        Dim A As Array = vxbf.Lock(0, LockFlags.None)
        vs = DirectCast(A, CustomVertex.TransformedColoredTextured())

        Ctr = 0

        'SR.WriteLine(vs.GetUpperBound(0))

        For LV = 0 To Math.Min(CHARACTERCOUNT - 1, CusText(Ctr).Length - 1)
            'SR.WriteLine(dL)
            'SR.WriteLine(LV)
            'SR.WriteLine(Ctr)
            'SR.WriteLine(ChrCol)
            'SR.WriteLine(ChrRow)
            ChrNum = Convert.ToInt32(CusText(Ctr).Chars(LV))
            ChrRow = ChrNum \ 16
            ChrCol = ChrNum Mod 16
            vs(6 * LV) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * Ctr, 0.0F, 1.0F, &HFFFFFFFF, ChrCol / 16.0F, ChrRow / 8.0F)
            vs(6 * LV + 1) = New CustomVertex.TransformedColoredTextured(dL * LV, dL * (Ctr + 1), 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F, ChrRow / 8.0F + dR)
            vs(6 * LV + 2) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * Ctr, 0.0F, 1.0F, &HFFDDDDDD, ChrCol / 16.0F + dC, ChrRow / 8.0F)
            vs(6 * LV + 3) = New CustomVertex.TransformedColoredTextured(dL * (LV + 1), dL * (Ctr + 1), 0.0F, 1.0F, &HFF888888, ChrCol / 16.0F + dC, ChrRow / 8.0F + dR)
            vs(6 * LV + 4) = vs(6 * LV + 2)
            vs(6 * LV + 5) = vs(6 * LV + 1)

        Next
        TextEnd = CHARACTERCOUNT * 6

        For LV = 0 To 9

            If Player.OwnedBullets(LV) Then
                If LV = Player.BulletType Then
                    vs(TextEnd + 6 * LV) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * LV, 0.0F, 1.0F, &HFFFFFF00, 0.0F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFFFFFFFF, 0.0F, (LV + 1) / 16.0F)
                    vs(TextEnd + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFFFFFFFF, 0.25F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFFFFFFFF, 0.25F, (LV + 1) / 16.0F)
                Else
                    vs(TextEnd + 6 * LV) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * LV, 0.0F, 1.0F, &HFF888888, 0.0F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF888888, 0.0F, (LV + 1) / 16.0F)
                    vs(TextEnd + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF888888, 0.25F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF888888, 0.25F, (LV + 1) / 16.0F)
                End If
            Else
                If LV = Player.BulletType Then
                    vs(TextEnd + 6 * LV) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * LV, 0.0F, 1.0F, &HFF991111, 0.0F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF991111, 0.0F, (LV + 1) / 16.0F)
                    vs(TextEnd + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF991111, 0.25F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF991111, 0.25F, (LV + 1) / 16.0F)
                Else
                    vs(TextEnd + 6 * LV) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * LV, 0.0F, 1.0F, &HFF111111, 0.0F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(100, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF111111, 0.0F, (LV + 1) / 16.0F)
                    vs(TextEnd + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF111111, 0.25F, LV / 16.0F)
                    vs(TextEnd + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(100 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF111111, 0.25F, (LV + 1) / 16.0F)
                End If
            End If
            vs(TextEnd + 6 * LV + 4) = vs(TextEnd + 6 * LV + 2)
            vs(TextEnd + 6 * LV + 5) = vs(TextEnd + 6 * LV + 1)
            If Player.OwnedWeapons(LV) Then
                If LV = Player.WeaponType Then
                    vs(TextEnd + 60 + 6 * LV) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * LV, 0.0F, 1.0F, &HFFFFFFFF, 0.25F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFFFFFFFF, 0.25F, (LV + 1) / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFFFFFFFF, 0.5F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFFFFFFFF, 0.5F, (LV + 1) / 16.0F)
                Else
                    vs(TextEnd + 60 + 6 * LV) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * LV, 0.0F, 1.0F, &HFF888888, 0.25F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF888888, 0.25F, (LV + 1) / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF888888, 0.5F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF888888, 0.5F, (LV + 1) / 16.0F)
                End If
            Else
                If LV = Player.WeaponType Then
                    vs(TextEnd + 60 + 6 * LV) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * LV, 0.0F, 1.0F, &HFF551111, 0.25F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF551111, 0.25F, (LV + 1) / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF551111, 0.5F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF551111, 0.5F, (LV + 1) / 16.0F)
                Else
                    vs(TextEnd + 60 + 6 * LV) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * LV, 0.0F, 1.0F, &HFF111111, 0.25F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(384, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF111111, 0.25F, (LV + 1) / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF111111, 0.5F, LV / 16.0F)
                    vs(TextEnd + 60 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(384 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF111111, 0.5F, (LV + 1) / 16.0F)
                End If

            End If
            vs(TextEnd + 60 + 6 * LV + 4) = vs(TextEnd + 60 + 6 * LV + 2)
            vs(TextEnd + 60 + 6 * LV + 5) = vs(TextEnd + 60 + 6 * LV + 1)
            If Player.OwnedWeapons(10 + LV) Then
                If 10 + LV = Player.WeaponType Then
                    vs(TextEnd + 120 + 6 * LV) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * LV, 0.0F, 1.0F, &HFFFFFFFF, 0.5F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFFFFFFFF, 0.5F, (LV + 1) / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFFFFFFFF, 0.75F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFFFFFFFF, 0.75F, (LV + 1) / 16.0F)
                Else
                    vs(TextEnd + 120 + 6 * LV) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * LV, 0.0F, 1.0F, &HFF888888, 0.5F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF888888, 0.5F, (LV + 1) / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF888888, 0.75F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF888888, 0.75F, (LV + 1) / 16.0F)
                End If
            Else
                If 10 + LV = Player.WeaponType Then
                    vs(TextEnd + 120 + 6 * LV) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * LV, 0.0F, 1.0F, &HFF551111, 0.5F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF551111, 0.5F, (LV + 1) / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF551111, 0.75F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF551111, 0.75F, (LV + 1) / 16.0F)
                Else
                    vs(TextEnd + 120 + 6 * LV) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * LV, 0.0F, 1.0F, &HFF111111, 0.5F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 1) = New CustomVertex.TransformedColoredTextured(668, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF111111, 0.5F, (LV + 1) / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 2) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * LV, 0.0F, 1.0F, &HFF111111, 0.75F, LV / 16.0F)
                    vs(TextEnd + 120 + 6 * LV + 3) = New CustomVertex.TransformedColoredTextured(668 + dS, 100 + dS * (LV + 1), 0.0F, 1.0F, &HFF111111, 0.75F, (LV + 1) / 16.0F)
                End If

            End If
            vs(TextEnd + 120 + 6 * LV + 4) = vs(TextEnd + 120 + 6 * LV + 2)
            vs(TextEnd + 120 + 6 * LV + 5) = vs(TextEnd + 120 + 6 * LV + 1)
        Next

        If Selexion = 0 Then
            LeftOffsetForArrow = 100.0F
        ElseIf Selexion = 1 Then
            LeftOffsetForArrow = 384.0F
        ElseIf Selexion = 2 Then
            LeftOffsetForArrow = 668.0F
        End If
        vs(TextEnd + 180) = New CustomVertex.TransformedColoredTextured(LeftOffsetForArrow, 70.0F, 0.0F, 1.0F, &HFFFFFFFF, 0.5F, 0.5F)
        vs(TextEnd + 181) = New CustomVertex.TransformedColoredTextured(LeftOffsetForArrow + 32, 70.0F, 0.0F, 1.0F, &HFFFFFFFF, 0.5F, 0.5F)
        vs(TextEnd + 182) = New CustomVertex.TransformedColoredTextured(LeftOffsetForArrow + 16, 90.0F, 0.0F, 1.0F, &HFFFFFFFF, 0.5F, 0.5F)

        vxbf.Unlock()
    End Sub

    Private Sub RenderWeaponSelect()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.BurlyWood, 1.0F, 0)
        D9.BeginScene()

        D9.RenderState.ZBufferWriteEnable = False
        D9.RenderState.AlphaBlendEnable = True

        Me.SetupWeaponSelect(Textspots, TextBfr, D9)

        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CHARACTERCOUNT << 1)

        D9.SetTexture(0, Bitmech(39))
        D9.SetStreamSource(0, TextBfr, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * (CHARACTERCOUNT * 6))
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 61)


        D9.EndScene()
        D9.Present()
    End Sub
    Private Sub RenderGameIntro()
        Dim LV As Integer
        Dim LowBd, UppBd, Idx As Integer  'Lower and upper Ku-values.
        'Idx is the Ku-value.

        Idx = Convert.ToInt32(Player.Loc.X) \ 1000
        If Idx = 0 Then
            LowBd = 0
        Else
            LowBd = Idx - 1
        End If
        If Idx >= KUSize - 1 Then
            UppBd = KUSize - 1
        Else
            UppBd = Idx + 1
        End If

        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.LightSkyBlue, 1.0F, 0)
        D9.BeginScene()

        Lighting()
        SetMatrices()

        D9.RenderState.ZBufferWriteEnable = False
        '************************************
        'Draw the sky to the background.
        '************************************
        Me.SetSky(SkyBuffer, D9)
        D9.SetTexture(0, SkyTx)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, SkyBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)
        '************************************
        'Draw the moving background.
        '************************************
        D9.RenderState.ZBufferWriteEnable = True
        D9.SetTexture(0, BackTx)
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.SetStreamSource(0, Backgrd, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)
        '************************************
        'Draw game landscape.
        '************************************
        D9.Material = Wallpaper
        'Draw platforms.
        D9.SetTexture(0, Tx(1))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * FIELDVERTEXCOUNT)
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).PlatformCount * PLATFORMVTXCOUNT))
        Next
        'Draw walls.
        D9.SetTexture(0, Tx(2))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).WallCount * WALLVTXCOUNT))
        Next
        'Draw extra walls.
        D9.SetTexture(0, Tx(9))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).ExtrawallCount * EXWALLVTXCOUNT))
        Next
        ''Draw ropes
        'D9.SetTexture(0, Tx(12))
        'For LV = LowBd To UppBd
        '    D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
        '    (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
        '    KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
        '    KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + _
        '    KU(LV).LadderCount * LADDERVTXCOUNT))

        '    D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).RopeCount * ROPEVTXCOUNT2))
        'Next
        'Draw main character.
        SetupBillboard(Billboard, D9)
        D9.Material = Luminous
        D9.SetStreamSource(0, Billboard, 0)
        D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
        Dim Lsi As Integer = 0

        LV = 0
        'Draw main character.
        If CtrTwo >= 500 Then
            D9.SetTexture(0, Bitmech(0))
            D9.Transform.World = Mx(LV)
            D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)
        End If
        D9.Transform.World = Matrix.Identity
        'Draw helicopter
        SetHelicopter(ChopBuffer, D9)
        D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
        D9.SetTexture(0, Chopper)
        D9.SetStreamSource(0, ChopBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)
        'Draw helicopter rope.
        D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
        D9.SetTexture(0, Tx(12))
        D9.SetStreamSource(0, ChopBuffer, System.Runtime.InteropServices.Marshal.SizeOf(PCTVertex) * 6)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)


        'Draw font
        D9.RenderState.ZBufferWriteEnable = False

        Me.SetupFonts(Textspots, TextBfr, D9)
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CusLines * CHARACTERCOUNT << 1)
        'Draw coins
        'If Q.Coins.Count > 0 Then
        '    D9.RenderState.PointSpriteEnable = True

        '    Me.SetCoinSprites(Spots, PsB, D9)
        '    D9.SetStreamSource(0, PsB, 0)
        '    D9.VertexFormat = CustomVertex.PositionColored.Format
        '    D9.SetTexture(0, PsTx((Tick And 63) >> 3))
        '    'Sets the active texture.  The 0 is up for discussion.
        '    D9.DrawPrimitives(PrimitiveType.PointList, 0, Q.Coins.Count)

        '    D9.RenderState.PointSpriteEnable = False
        'End If


        D9.EndScene()
        'And we would NOT be drawing here.
        'Typically the line immediately after .EndScene is .Present
        D9.Present()
    End Sub
    Private Sub RenderCloseMS()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.Black, 1.0F, 0)
        D9.BeginScene()

        Me.SetTitleGeometry(TitleBuffer, D9)

        D9.SetTexture(0, TitlePicture)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TitleBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        If Selexion = 0 Then
            D9.SetTexture(0, BlockFont)
            D9.SetStreamSource(0, TitleBuffer, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * 6)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 10)
        ElseIf Selexion = 1 Then
            D9.SetTexture(0, BlockFont)
            D9.SetStreamSource(0, TitleBuffer, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * 36)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 10)
        ElseIf Selexion = 2 Then
            D9.SetTexture(0, BlockFont)
            D9.SetStreamSource(0, TitleBuffer, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * 66)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 16)
        End If


        D9.EndScene()
        D9.Present()

    End Sub
    Private Sub RenderMainScreen()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.Black, 1.0F, 0)
        D9.BeginScene()

        Me.SetTitleGeometry(TitleBuffer, D9)

        D9.SetTexture(0, TitlePicture)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TitleBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        If CtrTwo >= 800 Then
            D9.SetTexture(0, BlockFont)
            D9.SetStreamSource(0, TitleBuffer, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * 6)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 46)

        End If


        D9.EndScene()
        D9.Present()

    End Sub
    Private Sub RenderOptions()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.BurlyWood, 1.0F, 0)
        D9.BeginScene()

        Me.SetOptionsGeometry(OptionBuffer, D9)

        D9.SetTexture(0, TitlePicture)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, OptionBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        D9.SetTexture(0, BlockFont)
        D9.SetStreamSource(0, OptionBuffer, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * 6)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 68)

        D9.EndScene()
        D9.Present()
    End Sub
    Private Sub RenderControls()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.BurlyWood, 1.0F, 0)
        D9.BeginScene()

        Me.SetControlsGeometry(OptionBuffer, D9)

        D9.SetTexture(0, TitlePicture)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, OptionBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        D9.SetTexture(0, BlockFont)
        D9.SetStreamSource(0, OptionBuffer, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * 6)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 798)

        D9.EndScene()
        D9.Present()
    End Sub

    Private Sub RenderMainScreenTransition()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.Black, 1.0F, 0)
        D9.BeginScene()

        Me.SetupFonts(Textspots, TextBfr, D9)
        'Setup the displayfont, the DirectX Font that supposedly will draw text for us.
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CHARACTERCOUNT << 1)

        D9.EndScene()
        D9.Present()

    End Sub
    Private Sub RenderEndCredits()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.Black, 1.0F, 0)
        D9.BeginScene()

        Me.SetupFonts(Textspots, TextBfr, D9)
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CHARACTERCOUNT << 1)


        D9.EndScene()
        D9.Present()

    End Sub
    Private Sub RenderDeath()
        Dim LV As Integer
        Dim LowBd, UppBd, Idx As Integer  'Lower and upper Ku-values.
        'Idx is the Ku-value.

        Idx = Convert.ToInt32(Player.Loc.X) \ 1000
        If Idx = 0 Then
            LowBd = 0
        Else
            LowBd = Idx - 1
        End If
        If Idx >= KUSize - 1 Then
            UppBd = KUSize - 1
        Else
            UppBd = Idx + 1
        End If

        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.LightSkyBlue, 1.0F, 0)

        'When doing more sophisticated drawing, our drawing to the device would be between a .BeginScene and
        'an .EndScene() call.
        D9.BeginScene()

        'Turn on the lights.
        Lighting()
        SetMatrices()

        '************************************
        'Draw the sky to the background.
        '************************************

        D9.RenderState.ZBufferWriteEnable = False

        Me.SetSky(SkyBuffer, D9)

        D9.SetTexture(0, SkyTx)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, SkyBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        '************************************
        'Draw the moving background.
        '************************************

        D9.RenderState.ZBufferWriteEnable = True

        D9.SetTexture(0, BackTx)
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.SetStreamSource(0, Backgrd, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        '************************************
        'Draw the exit door.
        '************************************

        D9.SetTexture(0, Exdo)
        D9.SetStreamSource(0, Backgrd, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * 6)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)




        '************************************
        'Draw game landscape.
        '************************************




        D9.Material = Wallpaper


        RenderAllStatics()

        '************************************
        'Draw dynamic content - lava, etc.
        '************************************

        If Q.Mobilewalls.Count + Q.Conveyors.Count + Q.VideoscreenCount > 0 Then
            Me.SetDynamicBufferData(DynaVxB, D9)

            D9.SetTexture(0, Mov)
            D9.SetStreamSource(0, DynaVxB, 0)
            D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT))

            D9.SetTexture(0, Tx(10))
            D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT)
            D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.Conveyors.Count * KONVEYVTXCOUNT))

            D9.SetTexture(0, Tx(14))
            D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * (Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT + Q.Conveyors.Count * KONVEYVTXCOUNT))
            D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.VideoscreenCount * VIDEOVTXCOUNT))


        End If


        '************************************
        'Draw main character and draw monsters.
        '************************************


        'Applying a world transform allows us to move meshes.
        'By moving the world, then drawing the mesh will cause the world to be changed so that the
        'mesh's default position is not the same as in the original.
        'Resetting the world matrix to the identity fixes everything, except the mesh is now drawn different.
        SetupBillboard(Billboard, D9)
        D9.Material = Luminous
        D9.SetStreamSource(0, Billboard, 0)
        D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
        Dim Lsi As Integer = 0
        LV = 0  'Recycle this integer


        LV += 1

        For Each tool As IMonster In Q.Monsters
            If Lsi <> tool.ID Then
                Lsi = tool.ID
                D9.SetTexture(0, Bitmech(Lsi))
            End If
            D9.Transform.World = Mx(LV)
            D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)

            LV += 1
        Next
        D9.Transform.World = Matrix.Identity

        If Q.Coins.Count > 0 Then
            'D9.RenderState.PointSpriteEnable = True
            Me.SetCoinSprites(Spots, PsB, D9)
            D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
            D9.SetTexture(0, PsTx((Tick And 63) >> 3))
            D9.SetStreamSource(0, PsB, 0)
            For LV = 0 To Q.Coins.Count - 1
                'Sets the active texture.  The 0 is up for discussion.
                D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)


            Next

            'D9.RenderState.PointSpriteEnable = False
        End If




        '************************************
        'Draw water and lava.
        '************************************


        D9.Transform.World = Matrix.Identity
        D9.SetTexture(0, Nothing)
        D9.VertexFormat = CustomVertex.PositionColored.Format

        If Convert.ToBoolean(Q.Lavas.Count Or Q.Waters.Count) Then
            D9.Material = LavaMtrl
            D9.SetStreamSource(0, Xlucent, 0)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, GetPrimitiveCount(PrimitiveType.TriangleList, LAVAVTXCOUNT * Q.Lavas.Count))

            D9.Material = WaterMtrl
            D9.SetStreamSource(0, Xlucent, System.Runtime.InteropServices.Marshal.SizeOf(PCVertex) * LAVAVTXCOUNT * Q.Lavas.Count)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, GetPrimitiveCount(PrimitiveType.TriangleList, WATERVTXCOUNT * Q.Waters.Count))


        End If
        D9.RenderState.ZBufferWriteEnable = False
        'DisplayFont.Begin()
        'DisplayFont.DrawText(Nothing, "Press Esc to End", New Rectangle(Me.ClientRectangle.Location, Me.ClientRectangle.Size), DrawTextFormat.Bottom, &H80FFFFFF)
        'DisplayFont.End()
        D9.Material = Luminous

        '************************************
        'Draw text.
        '************************************



        Me.SetupFonts(Textspots, TextBfr, D9)
        'Setup the displayfont, the DirectX Font that supposedly will draw text for us.
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CusLines * CHARACTERCOUNT << 1)




        D9.EndScene()
        'And we would NOT be drawing here.
        'Typically the line immediately after .EndScene is .Present
        D9.Present()
        'And of course, Present to present what the device has drawn on its own memory
        'The device has its own drawing surface: present just transports it to the screen.


    End Sub
    Private Sub RenderLoader()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.BurlyWood, 1.0F, 0)
        D9.BeginScene()


        D9.RenderState.ZBufferWriteEnable = False

        Me.SetupLoaderFonts(Textspots, TextBfr, D9)
        'Setup the displayfont, the DirectX Font that supposedly will draw text for us.
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 12 * CHARACTERCOUNT << 1)




        D9.EndScene()
        D9.Present()
    End Sub
    Private Sub RenderLevelXfer()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.BurlyWood, 1.0F, 0)
        D9.BeginScene()

        D9.RenderState.ZBufferWriteEnable = False
        D9.RenderState.AlphaBlendEnable = True

        Me.SetupLevelXfer(Textspots, TextBfr, D9)

        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CHARACTERCOUNT << 1)

        D9.SetTexture(0, Bitmech(29))
        D9.SetStreamSource(0, TextBfr, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * ((CHARACTERCOUNT * 6) + 384))
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        D9.SetTexture(0, Bitmech(28))
        D9.SetStreamSource(0, TextBfr, System.Runtime.InteropServices.Marshal.SizeOf(TCVertex) * (CHARACTERCOUNT * 6))
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 128)


        D9.EndScene()
        D9.Present()

    End Sub
    Private Sub RenderPauseMenu()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.BurlyWood, 1.0F, 0)
        D9.BeginScene()


        D9.RenderState.ZBufferWriteEnable = False

        Me.SetupPauseMenuFonts(Textspots, TextBfr, D9)
        'Setup the displayfont, the DirectX Font that supposedly will draw text for us.
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 8 * CHARACTERCOUNT << 1)




        D9.EndScene()
        D9.Present()
    End Sub
    Private Sub RenderSaveName()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.BurlyWood, 1.0F, 0)
        D9.BeginScene()


        D9.RenderState.ZBufferWriteEnable = False

        Me.SetupPauseMenuFonts(Textspots, TextBfr, D9)
        'Setup the displayfont, the DirectX Font that supposedly will draw text for us.
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CHARACTERCOUNT << 1)


        D9.EndScene()
        D9.Present()
    End Sub
    Private Sub RenderIP()
        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.Black, 1.0F, 0)
        D9.BeginScene()


        D9.RenderState.ZBufferWriteEnable = False

        Me.SetIPGeometry(IPBuffer, D9)

        D9.SetTexture(0, IPPicture)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, IPBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)



        D9.EndScene()
        D9.Present()
    End Sub
    Private Sub RenderLevelIntro()
        Dim LV As Integer
        Dim LowBd, UppBd, Idx As Integer  'Lower and upper Ku-values.
        'Idx is the Ku-value.



        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.LightSkyBlue, 1.0F, 0)

        'When doing more sophisticated drawing, our drawing to the device would be between a .BeginScene and
        'an .EndScene() call.
        D9.BeginScene()

        'Turn on the lights.
        Lighting()
        SetMatrices()

        '************************************
        'Draw the sky to the background.
        '************************************

        D9.RenderState.ZBufferWriteEnable = False

        Me.SetSky(SkyBuffer, D9)

        D9.SetTexture(0, SkyTx)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, SkyBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        '************************************
        'Draw the moving background.
        '************************************

        D9.RenderState.ZBufferWriteEnable = True

        D9.SetTexture(0, BackTx)
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.SetStreamSource(0, Backgrd, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        '************************************
        'Draw the exit door.
        '************************************

        D9.SetTexture(0, Exdo)
        D9.SetStreamSource(0, Backgrd, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * 6)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        '************************************
        'Draw game landscape.
        '************************************
        D9.Material = Wallpaper

        RenderAllStatics()

        '************************************
        'Draw dynamic content - lava, etc.
        '************************************

        If Q.Mobilewalls.Count + Q.Conveyors.Count + Q.VideoscreenCount > 0 Then
            Me.SetDynamicBufferData(DynaVxB, D9)

            D9.SetTexture(0, Mov)
            D9.SetStreamSource(0, DynaVxB, 0)
            D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT))

            D9.SetTexture(0, Tx(10))
            D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT)
            D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.Conveyors.Count * KONVEYVTXCOUNT))

            D9.SetTexture(0, Tx(14))
            D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * (Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT + Q.Conveyors.Count * KONVEYVTXCOUNT))
            D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.VideoscreenCount * VIDEOVTXCOUNT))


        End If


        '************************************
        'Draw main character and draw monsters.
        '************************************


        'Applying a world transform allows us to move meshes.
        'By moving the world, then drawing the mesh will cause the world to be changed so that the
        'mesh's default position is not the same as in the original.
        'Resetting the world matrix to the identity fixes everything, except the mesh is now drawn different.
        SetupBillboard(Billboard, D9)
        D9.Material = Luminous
        D9.SetStreamSource(0, Billboard, 0)
        D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
        Dim Lsi As Integer = 0
        LV = 0  'Recycle this integer



        'Draw main character.
        If Player.TemporarilyInvulnerable Mod 2 = 0 Then
            D9.SetTexture(0, Bitmech(0))
            D9.Transform.World = Mx(LV)
            D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)
        End If

        'D9.Transform.World = Mx(LV)
        'D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)


        '************************************
        'Draw water and lava.
        '************************************


        D9.Transform.World = Matrix.Identity
        D9.SetTexture(0, Nothing)
        D9.VertexFormat = CustomVertex.PositionColored.Format

        If Convert.ToBoolean(Q.Lavas.Count Or Q.Waters.Count) Then
            D9.Material = LavaMtrl
            D9.SetStreamSource(0, Xlucent, 0)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, GetPrimitiveCount(PrimitiveType.TriangleList, LAVAVTXCOUNT * Q.Lavas.Count))

            D9.Material = WaterMtrl
            D9.SetStreamSource(0, Xlucent, System.Runtime.InteropServices.Marshal.SizeOf(PCVertex) * LAVAVTXCOUNT * Q.Lavas.Count)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, GetPrimitiveCount(PrimitiveType.TriangleList, WATERVTXCOUNT * Q.Waters.Count))


        End If
        D9.RenderState.ZBufferWriteEnable = False
        'DisplayFont.Begin()
        'DisplayFont.DrawText(Nothing, "Press Esc to End", New Rectangle(Me.ClientRectangle.Location, Me.ClientRectangle.Size), DrawTextFormat.Bottom, &H80FFFFFF)
        'DisplayFont.End()
        D9.Material = Luminous


        '************************************
        'Draw text.
        '************************************



        Me.SetupIntroFonts(Textspots, TextBfr, D9)
        'Setup the displayfont, the DirectX Font that supposedly will draw text for us.
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CusLines * CHARACTERCOUNT << 1)





        D9.EndScene()
        'And we would NOT be drawing here.
        'Typically the line immediately after .EndScene is .Present
        D9.Present()
        'And of course, Present to present what the device has drawn on its own memory
        'The device has its own drawing surface: present just transports it to the screen.

    End Sub
    Private Sub Render()
        Dim LV As Integer
        Dim LowBd, UppBd, Idx As Integer  'Lower and upper Ku-values.
        'Idx is the Ku-value.



        D9.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.LightSkyBlue, 1.0F, 0)

        'When doing more sophisticated drawing, our drawing to the device would be between a .BeginScene and
        'an .EndScene() call.
        D9.BeginScene()

        'Turn on the lights.
        Lighting()
        SetMatrices()

        '************************************
        'Draw the sky to the background.
        '************************************

        D9.RenderState.ZBufferWriteEnable = False

        Me.SetSky(SkyBuffer, D9)

        D9.SetTexture(0, SkyTx)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, SkyBuffer, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)

        '************************************
        'Draw the moving background.
        '************************************

        D9.RenderState.ZBufferWriteEnable = True
        D9.SetTexture(0, BackTx)
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.SetStreamSource(0, Backgrd, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)


        '************************************
        'Draw the exit door.
        '************************************

        D9.SetTexture(0, Exdo)
        D9.SetStreamSource(0, Backgrd, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * 6)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, 2)




        '************************************
        'Draw game landscape.
        '************************************




        D9.Material = Wallpaper


        'Draw field.
        'D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        'D9.SetTexture(0, Tx(0))
        'For LV = LowBd To UppBd
        '    D9.SetStreamSource(0, VxB(LV), 0)
        '    D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, FIELDVERTEXCOUNT))
        'Next


        RenderAllStatics()


        '************************************
        'Draw dynamic content - lava, etc.
        '************************************

        Me.SetDynamicBufferData(DynaVxB, D9)

        D9.SetTexture(0, Mov)
        D9.SetStreamSource(0, DynaVxB, 0)
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT))

        D9.SetTexture(0, Tx(10))
        D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT)
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.Conveyors.Count * KONVEYVTXCOUNT))

        D9.SetTexture(0, Tx(14))
        D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * (Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT + Q.Conveyors.Count * KONVEYVTXCOUNT))
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.VideoscreenCount * VIDEOVTXCOUNT))

        D9.SetTexture(0, Tx(16))
        D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * (Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT + Q.Conveyors.Count * KONVEYVTXCOUNT + Q.VideoscreenCount * VIDEOVTXCOUNT))
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.RTDs.Count * RTDVTXCOUNT))

        D9.SetTexture(0, Tx(18))
        D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * (Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT + Q.Conveyors.Count * KONVEYVTXCOUNT + Q.VideoscreenCount * VIDEOVTXCOUNT + Q.RTDs.Count * RTDVTXCOUNT))
        D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, Q.Pneumos.Count * PNEUMOVTXCOUNT))

        If Upgrade.Width > 0 Then
            D9.SetTexture(0, Tx(17))
            D9.SetStreamSource(0, DynaVxB, System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * (Q.Mobilewalls.Count * MOVINGWALLVTXCOUNT + Q.Conveyors.Count * KONVEYVTXCOUNT + Q.VideoscreenCount * VIDEOVTXCOUNT + Q.RTDs.Count * RTDVTXCOUNT + Q.Pneumos.Count * PNEUMOVTXCOUNT))
            D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, 6))
        End If



        '************************************
        'Draw main character and draw monsters.
        '************************************


        'Applying a world transform allows us to move meshes.
        'By moving the world, then drawing the mesh will cause the world to be changed so that the
        'mesh's default position is not the same as in the original.
        'Resetting the world matrix to the identity fixes everything, except the mesh is now drawn different.
        SetupBillboard(Billboard, D9)
        D9.Material = Luminous
        D9.SetStreamSource(0, Billboard, 0)
        D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
        Dim Lsi As Integer = 0
        LV = 0  'Recycle this integer


        LV += 1

        For Each tool As IMonster In Q.Monsters
            If Lsi <> tool.ID Then
                Lsi = tool.ID
                D9.SetTexture(0, Bitmech(Lsi))
            End If
            D9.Transform.World = Mx(LV)
            D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)

            LV += 1
        Next

        LV = 0
        'Draw main character.
        If Player.TemporarilyInvulnerable Mod 2 = 0 Then
            D9.SetTexture(0, Bitmech(0))
            D9.Transform.World = Mx(LV)
            D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)
        End If

        D9.Transform.World = Matrix.Identity

        If Q.Coins.Count > 0 Then
            'D9.RenderState.PointSpriteEnable = True
            Me.SetCoinSprites(Spots, PsB, D9)
            D9.VertexFormat = CustomVertex.PositionColoredTextured.Format
            D9.SetTexture(0, PsTx((Tick And 63) >> 3))
            D9.SetStreamSource(0, PsB, 0)
            For LV = 0 To Q.Coins.Count - 1
                'Sets the active texture.  The 0 is up for discussion.
                D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)


            Next

            'D9.RenderState.PointSpriteEnable = False
        End If
        'D9.Transform.World = Mx(LV)
        'D9.DrawPrimitives(PrimitiveType.TriangleStrip, LV << 2, 2)

        '************************************
        'Draw floater meshes.
        '************************************

        D9.SetTexture(0, Fltr)
        For LV = 0 To MM.GetUpperBound(0)
            MM(LV).DrawAllSubsetsXF(D9, False, False)
        Next
        'YES ZBufferWrite           NO! PtSprite                ??? AlphaBlend

        'D9.Material = Luminous
        'For LV = 0 To Column.GetUpperBound(0)
        '    Column(LV).DrawAllSubsetsXF(D9, False, False)
        'Next
        D9.Transform.World = Matrix.Identity

        '************************************
        'Draw water and lava.
        '************************************


        D9.Transform.World = Matrix.Identity
        D9.SetTexture(0, Nothing)
        D9.VertexFormat = CustomVertex.PositionColored.Format
        If Convert.ToBoolean(Q.Lavas.Count Or Q.Waters.Count Or Q.Glasses.Count) Then
            Me.SetXlucentBuffer(Xlucent, D9)
            D9.Material = LavaMtrl
            D9.SetStreamSource(0, Xlucent, 0)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, GetPrimitiveCount(PrimitiveType.TriangleList, LAVAVTXCOUNT * Q.Lavas.Count))

            D9.Material = WaterMtrl
            D9.SetStreamSource(0, Xlucent, System.Runtime.InteropServices.Marshal.SizeOf(PCVertex) * LAVAVTXCOUNT * Q.Lavas.Count)
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, GetPrimitiveCount(PrimitiveType.TriangleList, WATERVTXCOUNT * Q.Waters.Count))

            D9.Material = GlasMtrl
            D9.SetStreamSource(0, Xlucent, System.Runtime.InteropServices.Marshal.SizeOf(PCVertex) * (LAVAVTXCOUNT * Q.Lavas.Count + WATERVTXCOUNT * Q.Waters.Count))
            D9.DrawPrimitives(PrimitiveType.TriangleList, 0, GetPrimitiveCount(PrimitiveType.TriangleList, GLASSVTXCOUNT * Q.Glasses.Count))

        End If
        D9.RenderState.ZBufferWriteEnable = False
        'DisplayFont.Begin()
        'DisplayFont.DrawText(Nothing, "Press Esc to End", New Rectangle(Me.ClientRectangle.Location, Me.ClientRectangle.Size), DrawTextFormat.Bottom, &H80FFFFFF)
        'DisplayFont.End()
        D9.Material = Luminous


        '************************************
        'Draw text.
        '************************************



        Me.SetupFonts(Textspots, TextBfr, D9)
        'Setup the displayfont, the DirectX Font that supposedly will draw text for us.
        D9.SetTexture(0, BlockFont)
        D9.VertexFormat = CustomVertex.TransformedColoredTextured.Format
        D9.SetStreamSource(0, TextBfr, 0)
        D9.DrawPrimitives(PrimitiveType.TriangleList, 0, CusLines * CHARACTERCOUNT << 1)


        '************************************
        'Draw coins.
        '************************************





        D9.EndScene()
        'And we would NOT be drawing here.
        'Typically the line immediately after .EndScene is .Present
        D9.Present()
        'And of course, Present to present what the device has drawn on its own memory
        'The device has its own drawing surface: present just transports it to the screen.

    End Sub   'Main game renderer.
    Private Sub RenderAllStatics()
        Dim LV As Integer
        Dim LowBd, UppBd, Idx As Integer  'Lower and upper Ku-values.
        Idx = Convert.ToInt32(Player.Loc.X) \ 1000
        If Idx = 0 Then
            LowBd = 0
        Else
            LowBd = Idx - 1
        End If
        If Idx >= KUSize - 1 Then
            UppBd = KUSize - 1
        Else
            UppBd = Idx + 1
        End If

        D9.SetTexture(0, Tx(6))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT))
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).AftFenceCount * FENCEVTXCOUNT))
        Next


        'Draw backdrop.
        D9.SetTexture(0, Tx(5))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).BackdropCount * BACKDROPVTXCOUNT))
        Next

        'Background crates
        D9.SetTexture(0, Tx(15))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + _
            KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT + KU(LV).AftFenceCount * FENCEVTXCOUNT + KU(LV).FieldFenceCount * FENCEVTXCOUNT + KU(LV).CrateCount * CRATEVTXCOUNT))
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).BackcrateCount * BACKCRATEVTXCOUNT))
        Next


        'Draw platforms.
        'D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.SetTexture(0, Tx(1))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * FIELDVERTEXCOUNT)
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).PlatformCount * PLATFORMVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(6))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + _
            KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT + KU(LV).AftFenceCount * FENCEVTXCOUNT))
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).FieldFenceCount * FENCEVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(15))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + _
            KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT + KU(LV).AftFenceCount * FENCEVTXCOUNT + KU(LV).FieldFenceCount * FENCEVTXCOUNT))
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).CrateCount * CRATEVTXCOUNT))
        Next


        'Draw walls.
        'D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.SetTexture(0, Tx(2))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).WallCount * WALLVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(9))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).ExtrawallCount * EXWALLVTXCOUNT))
        Next

        'Eye Beams
        D9.SetTexture(0, Tx(19))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + _
            KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT + KU(LV).AftFenceCount * FENCEVTXCOUNT + KU(LV).FieldFenceCount * FENCEVTXCOUNT + KU(LV).CrateCount * CRATEVTXCOUNT + _
            KU(LV).BackcrateCount * BACKCRATEVTXCOUNT))
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).EyeBeamCount * EYEBEAMVTXCOUNT))
        Next

        'Cross Stays
        D9.SetTexture(0, Tx(20))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + _
            KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT + KU(LV).AftFenceCount * FENCEVTXCOUNT + KU(LV).FieldFenceCount * FENCEVTXCOUNT + KU(LV).CrateCount * CRATEVTXCOUNT + _
            KU(LV).BackcrateCount * BACKCRATEVTXCOUNT + KU(LV).EyeBeamCount * EYEBEAMVTXCOUNT))
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).CrossStayCount * CROSSSTAYVTXCOUNT))
        Next


        D9.Material = Luminous
        'For LV = 0 To Column.GetUpperBound(0)
        '    Column(LV).DrawAllSubsetsXF(D9, False)
        'Next

        'Draw jumping boards.
        'D9.VertexFormat = CustomVertex.PositionNormalTextured.Format
        D9.SetTexture(0, Tx(3))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).JumpCount * JBDVTXCOUNT))
        Next

        'Draw ices.
        D9.SetTexture(0, Tx(4))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).IceCount * ICEVTXCOUNT))
        Next

        D9.SetTexture(0, Nothing)
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).BulbCount * BULBVTXCOUNT))
        Next

        D9.Material = Wallpaper



        D9.SetTexture(0, Tx(7))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(8))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).ColumnCount * COLUMNVTXCOUNT))
        Next



        D9.SetTexture(0, Tx(11))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).LadderCount * LADDERVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(12))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).RopeCount * ROPEVTXCOUNT2))
        Next

        D9.SetTexture(0, Tx(13))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).GroundCount * GROUNDVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(14))  'Plants
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).PlantCount * PLANTVTXCOUNT))
        Next
        'Statue()
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + _
            KU(LV).PlantCount * PLANTVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).StatueCount * STATUEVTXCOUNT))
        Next
        'Picture
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT + _
            KU(LV).PlantCount * PLANTVTXCOUNT + KU(LV).StatueCount * STATUEVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).PictureCount * PICTUREVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(6))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT))

            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT))
        Next

        D9.SetTexture(0, Tx(6))
        For LV = LowBd To UppBd
            D9.SetStreamSource(0, VxB(LV), System.Runtime.InteropServices.Marshal.SizeOf(Vertex) * _
            (FIELDVERTEXCOUNT + PLATFORMVTXCOUNT * KU(LV).PlatformCount + WALLVTXCOUNT * KU(LV).WallCount + JBDVTXCOUNT * KU(LV).JumpCount + _
            KU(LV).IceCount * ICEVTXCOUNT + KU(LV).BackdropCount * BACKDROPVTXCOUNT + KU(LV).TrapPlatformCount * TRAPPLATFORMVTXCOUNT + _
            KU(LV).SlopedwallCount * SLOPEDWALLVTXCOUNT + KU(LV).ColumnCount * COLUMNVTXCOUNT + KU(LV).ExtrawallCount * EXWALLVTXCOUNT + KU(LV).BulbCount * BULBVTXCOUNT + _
            KU(LV).LadderCount * LADDERVTXCOUNT + KU(LV).RopeCount * ROPEVTXCOUNT2 + KU(LV).GroundCount * GROUNDVTXCOUNT))
            D9.DrawPrimitives(PrT, 0, GetPrimitiveCount(PrT, KU(LV).ForeFenceCount * FOREFENCEVTXCOUNT))
        Next


    End Sub

    Public Function GetDistance(ByVal V1 As Vector3, ByVal V2 As Vector3) As Single
        Return Vector3.Subtract(V2, V1).Length
    End Function

    Private Function GetPrimitiveCount(ByVal PrT As PrimitiveType, ByVal VertexNum As Integer) As Integer
        Select Case PrT
            Case PrimitiveType.LineList
                Return VertexNum \ 2
            Case PrimitiveType.LineStrip
                Return VertexNum - 1
            Case PrimitiveType.PointList
                Return VertexNum
            Case PrimitiveType.TriangleFan
                Return VertexNum - 2
            Case PrimitiveType.TriangleList
                Return VertexNum \ 3
            Case PrimitiveType.TriangleStrip
                Return VertexNum - 2
        End Select
    End Function
    Private Sub NormalizeTriangles(ByRef vs() As CustomVertex.PositionNormalTextured, ByVal PrT As PrimitiveType)
        Dim N As Vector3, LV As Integer
        If PrT = PrimitiveType.TriangleList Then
            For LV = 2 To vs.GetUpperBound(0) Step 3
                N = GetNormal(vs(LV - 2).Position, vs(LV - 1).Position, vs(LV).Position)
                N.Normalize()
                vs(LV - 2).Normal = N
                vs(LV - 1).Normal = N
                vs(LV).Normal = N
            Next
        End If
    End Sub
    Private Function GetNormal(ByVal A As Vector3, ByVal B As Vector3, ByVal C As Vector3) As Vector3
        Dim Pu As Vector3, Pv As Vector3
        Pu = Vector3.Subtract(B, A)  'Final minus initial.
        Pv = Vector3.Subtract(C, A)  'Final minus initial.
        'Pu and Pv are difference vectors that describe the shape and orientation of the triangle.
        'Therefore, the normal is the cross product of these two vectors: N = Pu  Pv
        Return Vector3.Cross(Pu, Pv)
    End Function

    Private Function Fmd(ByVal Nume As Single, ByVal Denom As Single) As Single
        Dim ItE As Double = Math.IEEERemainder(Nume, Denom)
        'Finally figured out how this function works.
        If ItE < 0 Then
            ItE = Denom + ItE
        End If
        Return Convert.ToSingle(ItE)
    End Function

    Private Function TrglDif(ByVal N As Single) As Single   'Squarewave used for generating the trianglewave.
        Dim Res As Single = Fmd(N, 2)  'Fmd now takes a numerator and a denominator.
        If Res >= 0 AndAlso Res < 1 Then
            Return 1
        Else
            Return -1
        End If
    End Function
    Private Function HxgnDif(ByVal N As Single) As Single
        Dim Res As Single = Fmd(N, 4)
        If Res >= 0 AndAlso Res < 1 Then
            Return 2
        ElseIf Res >= 1 AndAlso Res < 2 Then
            Return 0
        ElseIf Res >= 2 AndAlso Res < 3 Then
            Return -2
        Else
            Return 0
        End If
    End Function
    Private Function TrpzDif(ByVal N As Single) As Single
        Dim Res As Single = Fmd(N, 3)
        If Res >= 0 AndAlso Res < 1 Then
            Return 0
        ElseIf Res >= 1 AndAlso Res < 2 Then
            Return 2
        Else
            Return -2
        End If
    End Function
    Private Function SclnDif(ByVal N As Single) As Single
        Dim Res As Single = Fmd(N, 3)
        If Res >= 0 AndAlso Res < 1 Then
            Return 2
        Else
            Return -1
        End If
    End Function

    Public Sub PopulateKUs()
        'There's probably a way I could do this without having this subroutine:
        'The reason for right now is that I need to know how many vertices I'll need for the VertexBuffer before I create it.
        'I use this sub to find out how many objects, thus how many vertices I'll need.  The object vertices themselves will be
        'in the VertexBuffer setup, after the vertex buffer has been created.
        Dim Idx As Integer
        For Each Platform In Q.Platforms
            Idx = Convert.ToInt32(Platform.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Platform " & Platform.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).PlatformCount += 1
            End If
        Next
        For Each Wall In Q.Walls
            Idx = Convert.ToInt32(Wall.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Wall " & Wall.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).WallCount += 1
            End If
        Next
        For Each Lava In Q.Lavas
            Idx = Convert.ToInt32(Lava.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Lava " & Lava.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).LavaCount += 1
            End If
        Next
        For Each Jumpboard In Q.Jumpboards
            Idx = Convert.ToInt32(Jumpboard.Loc.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Jumpboard " & Jumpboard.Loc.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).JumpCount += 1
            End If
        Next
        For Each Ice In Q.Ices
            Idx = Convert.ToInt32(Ice.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Ice " & Ice.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).IceCount += 1
            End If
        Next
        For Each Backdrop In Q.Backdrops
            Idx = Convert.ToInt32(Backdrop.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Backdrop " & Backdrop.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).BackdropCount += 1
            End If
        Next
        For Each TrapPlatform In Q.TrapPlatforms
            Idx = Convert.ToInt32(TrapPlatform.R.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Trapezoid Platform " & TrapPlatform.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).TrapPlatformCount += 1
            End If
        Next
        For Each Slopedwall In Q.Slopedwalls
            Idx = Convert.ToInt32(Slopedwall.R.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Sloped Wall " & Slopedwall.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).SlopedwallCount += 1
            End If
        Next
        For Each Column In Q.Verticals
            Idx = Convert.ToInt32(Column.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Column " & Column.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).ColumnCount += 1
            End If
        Next
        For Each Backdrop In Q.Bulbs
            Idx = Convert.ToInt32(Backdrop.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Bulb " & Backdrop.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).BulbCount += 1
            End If
        Next
        For Each Wall In Q.Extrawalls
            Idx = Convert.ToInt32(Wall.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Extra wall " & Wall.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).ExtrawallCount += 1
            End If
        Next
        For Each Wall In Q.Crates
            Idx = Convert.ToInt32(Wall.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Crate " & Wall.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).CrateCount += 1
            End If
        Next
        For Each Wall In Q.Backcrates
            Idx = Convert.ToInt32(Wall.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Backcrate " & Wall.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).BackcrateCount += 1
            End If
        Next
        For Each Ladder In Q.Ladders
            Idx = Convert.ToInt32(Ladder.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Ladder " & Ladder.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).LadderCount += 1
            End If
        Next
        For Each Ladder In Q.Ropes
            Idx = Convert.ToInt32(Ladder.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Rope " & Ladder.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).RopeCount += 1
            End If
        Next
        For Each Wall In Q.Groundwalls
            Idx = Convert.ToInt32(Wall.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("Ground " & Wall.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).GroundCount += 1
            End If
        Next
        For Each Backdrop In Q.ForeFences
            Idx = Convert.ToInt32(Backdrop.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("forefence " & Backdrop.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).ForeFenceCount += 1
            End If
        Next
        For Each Backdrop In Q.AftFences
            Idx = Convert.ToInt32(Backdrop.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("aftfence " & Backdrop.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).AftFenceCount += 1
            End If
        Next
        For Each Backdrop In Q.FieldFences
            Idx = Convert.ToInt32(Backdrop.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("fieldfence " & Backdrop.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).FieldFenceCount += 1
            End If
        Next
        For Each Backdrop In Q.EyeBeams
            Idx = Convert.ToInt32(Backdrop.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("eyebeam " & Backdrop.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).EyeBeamCount += 1
            End If
        Next
        For Each Backdrop In Q.CrossStays
            Idx = Convert.ToInt32(Backdrop.X) \ 1000
            If Idx >= KUSize Then
                System.Diagnostics.Debug.WriteLine("cross stay " & Backdrop.ToString() & " cannot fit in KUSize of " & KUSize.ToString())
            Else
                KU(Idx).CrossStayCount += 1
            End If
        Next


        Q.PopulateKUs(KU)

    End Sub

    Private Sub LoadLevel(ByVal Filename As String)
        'We need to create a binaryreader to read from the file.
        Dim FS As System.IO.FileStream = New System.IO.FileStream(Filename, IO.FileMode.Open)
        Dim BR As System.IO.BinaryReader = New System.IO.BinaryReader(FS)
        Dim LV As Integer, Dat As RectangleF  'Loop variable.
        Dim Flag As Boolean
        Dim Code As Integer
        Dim L2 As Microsoft.DirectX.Vector2
        Dim FB As FoulBeast, FBT As FoulBeastTentacle

        Player.Loc = New RectangleF(BR.ReadInt32(), BR.ReadInt32(), CHARWIDTH, CHARHEIGHT)
        'Read the player location.
        Player.LeftEnd = BR.ReadInt32() : Player.RightEnd = BR.ReadInt32()
        'And the ending points of the current platform.

        Q.Platforms = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Platforms.Capacity  'Capacity is the number that you've specified in the New arraylist's constructor.
            Q.Platforms.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read four integers from the file and put it into a rectangle.
        Next

        Q.Walls = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Walls.Capacity
            Q.Walls.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next

        Q.Coins = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Coins.Capacity
            Q.Coins.Add(New RectangleF(BR.ReadInt32(), BR.ReadInt32(), COINSIZE, COINSIZE))
            'Coins only need left and top.  Their width and height are defined by coin size.
        Next

        Q.Lavas = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Lavas.Capacity
            Q.Lavas.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read the lavas into the file.
        Next

        Q.Floaters = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Floaters.Capacity
            Q.Floaters.Add(New LevFloater(New RectangleF(BR.ReadInt32(), BR.ReadInt32(), FLOATERSIZE, FLOATERSIZE), _
            BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), _
            CType(BR.ReadInt32(), PeriodicDesignation), CType(BR.ReadInt32(), PeriodicDesignation)))
        Next

        Q.Jumpboards = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Jumpboards.Capacity
            Q.Jumpboards.Add(New JumpingBoard(BR.ReadInt32(), BR.ReadInt32(), JUMPBOARDSIZE, JUMPBOARDSIZE, BR.ReadInt32()))
            'Read jump board from file.  The width and height are determined by our constant.
        Next

        Q.RTDs = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.RTDs.Capacity
            Q.RTDs.Add(New OpenerSwitch(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
        Next
        Q.Pneumos = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Pneumos.Capacity
            Q.Pneumos.Add(New OpenerSwitch(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
        Next

        Q.Mobilewalls = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Mobilewalls.Capacity
            Q.Mobilewalls.Add(New MovingWall(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), _
            BR.ReadInt32(), BR.ReadInt32(), _
            RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32())))
            'Moving wall data from file.
        Next

        Q.Ices = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Ices.Capacity  'Capacity is the number that you've specified in the New arraylist's constructor.
            Q.Ices.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read four integers from the file and put it into a rectangle.
        Next

        Q.Waters = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Waters.Capacity  'Capacity is the number that you've specified in the New arraylist's constructor.
            Q.Waters.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read four integers from the file and put it into a rectangle.
        Next

        Q.Backdrops = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Backdrops.Capacity  'Capacity is the number that you've specified in the New arraylist's constructor.
            Q.Backdrops.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read four integers from the file and put it into a rectangle.
        Next

        Q.TrapPlatforms = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.TrapPlatforms.Capacity
            Q.TrapPlatforms.Add(New StandTrapezoid(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), _
            CType(BR.ReadInt32(), Diagonals), BR.ReadInt32() - 1))
        Next
        Q.TrapPlatforms.Reverse()

        Q.Slopedwalls = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Slopedwalls.Capacity
            Q.Slopedwalls.Add(New StandTrapezoid(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), _
            CType(BR.ReadInt32(), Diagonals), BR.ReadInt32() - 1))
        Next

        Exitdoor = New RectangleF(BR.ReadInt32(), BR.ReadInt32(), EXITSIZE, EXITSIZE)
        Upgrade = New RectangleF(BR.ReadInt32(), BR.ReadInt32(), EXITSIZE, EXITSIZE)

        Q.Verticals = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Verticals.Capacity  'Capacity is the number that you've specified in the New arraylist's constructor.
            Q.Verticals.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read four integers from the file and put it into a rectangle.
        Next

        Q.Extrawalls = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Extrawalls.Capacity
            Q.Extrawalls.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next

        Q.Crates = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Crates.Capacity
            Q.Crates.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next

        Q.Backcrates = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Backcrates.Capacity
            Q.Backcrates.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next


        Q.Conveyors = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Conveyors.Capacity
            Q.Conveyors.Add(New DataBlock(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32() / 2.0F))
            'Read conveyor from file.
        Next

        Q.Bulbs = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Bulbs.Capacity
            Q.Bulbs.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read bulb from file.
        Next

        'Jumpboards = New ArrayList
        'MobileWalls = New ArrayList

        Q.MovingWallBoxCCWcount = BR.ReadInt32()
        For LV = 1 To Q.MovingWallBoxCCWcount
            Q.Mobilewalls.Add(New MovingWallBoxCCW(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), _
            BR.ReadInt32(), BR.ReadInt32(), _
            RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32())))
            'Moving wall data from file.
        Next

        Q.MovingWallBoxCWcount = BR.ReadInt32()
        For LV = 1 To Q.MovingWallBoxCWcount
            Q.Mobilewalls.Add(New MovingWallBoxCW(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), _
            BR.ReadInt32(), BR.ReadInt32(), _
            RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32())))
            'Moving wall data from file.
        Next

        Q.MovingWallCircCount = BR.ReadInt32()
        For LV = 1 To Q.MovingWallCircCount
            Q.Mobilewalls.Add(New MovingWallCirc(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), _
             BR.ReadInt32(), BR.ReadSingle(), _
            RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32())))
            'Moving wall data from file.
        Next

        Q.PistonCount = BR.ReadInt32()
        For LV = 1 To Q.PistonCount
            Q.Mobilewalls.Add(New Piston(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), _
            BR.ReadInt32(), _
            RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32())))
            'Moving wall data from file.
        Next

        Q.Ladders = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Ladders.Capacity  'Capacity is the number that you've specified in the New arraylist's constructor.
            Q.Ladders.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read four integers from the file and put it into a rectangle.
        Next

        Q.Ropes = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Ropes.Capacity  'Capacity is the number that you've specified in the New arraylist's constructor.
            Q.Ropes.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read four integers from the file and put it into a rectangle.
        Next

        Q.Groundwalls = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Groundwalls.Capacity
            Dat = RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32())
            If Dat.X = 0 AndAlso Not Flag Then
                Q.Groundwalls.Capacity += 1
                Q.Groundwalls.Add(RectangleF.FromLTRB(-500, Dat.Top, 0, Dat.Bottom))
                Flag = True
            End If
            Q.Groundwalls.Add(Dat)

            'Same for four integers going into wall rectangles.
        Next

        Q.ForeFences = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.ForeFences.Capacity
            Q.ForeFences.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next
        Q.AftFences = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.AftFences.Capacity
            Q.AftFences.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next
        Q.FieldFences = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.FieldFences.Capacity
            Q.FieldFences.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next
        Q.EyeBeams = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.EyeBeams.Capacity
            Q.EyeBeams.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next
        Q.CrossStays = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.CrossStays.Capacity
            Q.CrossStays.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next
        Q.Glasses = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Glasses.Capacity
            Q.Glasses.Add(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Same for four integers going into wall rectangles.
        Next


        Q.Things = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Things.Capacity
            Q.Things.Add(New DataBlock(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            'Read conveyor from file.
        Next
        Q.Things = New ArrayList
        Q.ArrangeSetThings()

        Q.Monsters = New ArrayList(BR.ReadInt32())
        For LV = 1 To Q.Monsters.Capacity
            Code = BR.ReadInt32()
            If Code = 0 Then
                Q.Monsters.Add(New AcidStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 1 Then
                Q.Monsters.Add(New GrimLeaper(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 2 Then
                Q.Monsters.Add(New DeepStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 3 Then
                Q.Monsters.Add(New Dispatch(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 4 Then
                Q.Monsters.Add(New Jeefly(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 5 Then
                Q.Monsters.Add(New RegBat(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 6 Then
                Q.Monsters.Add(New Gregbird(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 7 Then
                Q.Monsters.Add(New Biotsy(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 8 Then
                Q.Monsters.Add(New RollingCamera(BR.ReadInt32, BR.ReadInt32(), RectangleF.FromLTRB(BR.ReadInt32(), 0, BR.ReadInt32(), 1), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 9 Then
                Q.Monsters.Add(New SwivelCamera(BR.ReadInt32(), BR.ReadInt32, BR.ReadInt32))
            ElseIf Code = 10 Then
                Q.Monsters.Add(New MotionDetector(BR.ReadInt32, BR.ReadInt32, BR.ReadInt32))
            ElseIf Code = 11 Then
                Q.Monsters.Add(New Horion(New RectangleF(BR.ReadInt32(), 0, 1, BR.ReadInt32()), 0, BR.ReadInt32()))
            ElseIf Code = 12 Then
                Q.Monsters.Add(New Pelogic(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 13 Then
                Q.Monsters.Add(New Squirm(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 14 Then
                Q.Monsters.Add(New Leach(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 15 Then
                Q.Monsters.Add(New CasukeBall(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean(), BR.ReadInt32()))
            ElseIf Code = 16 Then
                Q.Monsters.Add(New BoundingBall(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean(), BR.ReadInt32()))
            ElseIf Code = 17 Then
                Q.Monsters.Add(New BoxBat(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 18 Then
                Q.Monsters.Add(New LinearBat(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 19 Then
                Q.Monsters.Add(New SineBat(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 20 Then
                Q.Monsters.Add(New IcyStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 21 Then
                Q.Monsters.Add(New Corven(New RectangleF(BR.ReadInt32(), 0, 1, BR.ReadInt32()), 0))
            ElseIf Code = 22 Then
                Q.Monsters.Add(New Hoiler(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadInt32(), 5))
            ElseIf Code = 23 Then
                Q.Monsters.Add(New LocoLeaper(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 24 Then
                Q.Monsters.Add(New RisingFire(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 25 Then
                Q.Monsters.Add(New SalkScorpion(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 26 Then
                Q.Monsters.Add(New Vokat(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 27 Then
                L2.X = BR.ReadInt32()
                L2.Y = BR.ReadInt32()
                Q.Monsters.Add(New Modular(L2.X, L2.Y))
                Q.Monsters.Add(New ModularBall(L2.X, L2.Y, False))
                Q.Monsters.Add(New ModularBall(L2.X, L2.Y, True))
            ElseIf Code = 28 Then
                Q.Monsters.Add(New SeaStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 29 Then
                Q.Monsters.Add(New ShieldTurret(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 30 Then
                Q.Monsters.Add(New SurpriseFish(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 31 Then
                Q.Monsters.Add(New Arroglestop(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 32 Then
                Q.Monsters.Add(New Opeck(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 33 Then
                Q.Monsters.Add(New Airstar(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32() / 150.0F, BR.ReadInt32()))
            ElseIf Code = 34 Then
                Q.Monsters.Add(New FieryStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 35 Then
                Q.Monsters.Add(New Ejectent(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 36 Then
                Q.Monsters.Add(New Frogetu(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 37 Then
                Q.Monsters.Add(New ShieldBot(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 38 Then
                Q.Monsters.Add(New Electrico(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean(), BR.ReadInt32()))
            ElseIf Code = 39 Then
                Q.Monsters.Add(New PennyRifle(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 40 Then
                Q.Monsters.Add(New ShieldShock(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 41 Then
                Q.Monsters.Add(New Canonical(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean(), BR.ReadInt32()))
            ElseIf Code = 42 Then
                L2.X = BR.ReadInt32()
                L2.Y = BR.ReadInt32()
                Q.Monsters.Add(New WallStep(L2.X, L2.Y, BR.ReadBoolean(), BR.ReadInt32()))
            ElseIf Code = 43 Then
                Q.Monsters.Add(New Spikestrip(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32())))
            ElseIf Code = 44 Then
                Q.Monsters.Add(New DarkFly(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 45 Then
                Q.Monsters.Add(New Exploder(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 46 Then
                Q.Monsters.Add(New Pachinko(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 47 Then
                Q.Monsters.Add(New SteelStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 48 Then
                Q.Monsters.Add(New LowStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 49 Then
                Q.Monsters.Add(New Kite(BR.ReadInt32(), BR.ReadInt32(), BR.ReadSingle(), BR.ReadSingle(), BR.ReadSingle()))
            ElseIf Code = 50 Then
                Q.Monsters.Add(New PivotGunner(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 51 Then
                Q.Monsters.Add(New ArcGunner(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 52 Then
                Q.Monsters.Add(New Gentu(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 53 Then
                Q.Monsters.Add(New FreeModular(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 54 Then
                Q.Monsters.Add(New TurnModular(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 55 Then
                Q.Monsters.Add(New SweepModular(BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean()))
            ElseIf Code = 56 Then
                Q.Monsters.Add(New Gobblet(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 57 Then
                Q.Monsters.Add(New SwingModule(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 58 Then
                Q.Monsters.Add(New SunStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 59 Then
                Q.Monsters.Add(New Trebuchet(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 60 Then
                Q.Monsters.Add(New SummonerFly(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 61 Then
                L2.X = BR.ReadInt32()
                L2.Y = BR.ReadInt32()
                Q.Monsters.Add(New Upsilon(L2.X, L2.Y))
                Q.Monsters.Add(New Upsilon(L2.X, L2.Y - 50))
                Q.Monsters.Add(New Upsilon(L2.X, L2.Y - 100))
                Q.Monsters.Add(New Upsilon(L2.X, L2.Y - 150))
                Q.Monsters.Add(New Upsilon(L2.X, L2.Y - 200))
                Q.Monsters.Add(New Upsilon(L2.X, L2.Y - 250))
            ElseIf Code = 62 Then
                Q.Monsters.Add(New Sunsaws(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 63 Then
                Dat = New RectangleF(BR.ReadInt32(), BR.ReadInt32(), 1, BR.ReadInt32())
                Q.Monsters.Add(New StrafisTrack(Dat.X, Dat.Y, Dat.Height))
                Q.Monsters.Add(New Strafis(Dat.X, Dat.Y, Dat.Height, BR.ReadInt32()))
            ElseIf Code = 64 Then
                Q.Monsters.Add(New Aircan(BR.ReadInt32(), BR.ReadInt32(), New RectangleF(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 65 Then
                Q.Monsters.Add(New Cloud(BR.ReadInt32(), BR.ReadInt32(), Ri))
            ElseIf Code = 66 Then
                Q.Monsters.Add(New HighStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 67 Then
                Q.Monsters.Add(New Faruno(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean(), Ri))
            ElseIf Code = 68 Then
                L2.X = BR.ReadInt32()
                L2.Y = BR.ReadInt32()
                FBT = New FoulBeastTentacle(L2.X, L2.Y)
                FB = New FoulBeast(L2.X, L2.Y, FBT)
                Q.Monsters.Add(FB)
                Q.Monsters.Add(FBT)
                'Add the beast before adding the tentacle.
            ElseIf Code = 69 Then
                Q.Monsters.Add(New Cannone(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 70 Then
                Q.Monsters.Add(New DarkStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 71 Then
                Q.Monsters.Add(New BookooLeap(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 72 Then
                Q.Monsters.Add(New WallMine(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 73 Then
                Q.Monsters.Add(New SeaGatelle(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 74 Then
                Q.Monsters.Add(New Cemurdo(BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 75 Then
                Q.Monsters.Add(New BullFrog(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32()))
            ElseIf Code = 76 Then
                Q.Monsters.Add(New SilverStep(RectangleF.FromLTRB(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()), BR.ReadInt32(), BR.ReadBoolean, BR.ReadInt32()))
            ElseIf Code = 77 Then
                Q.Monsters.Add(New Dilettante(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32() * 1.4F))
            ElseIf Code = 78 Then
                Q.Monsters.Add(New Pyromist(BR.ReadInt32(), BR.ReadInt32(), Ri))
            ElseIf Code = 79 Then
                Q.Monsters.Add(New SpaceRockL(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), Ri))
            ElseIf Code = 80 Then
                Q.Monsters.Add(New RemoUzi(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 81 Then
                Q.Monsters.Add(New EmoUzi(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 82 Then
                Q.Monsters.Add(New BrassGolem(BR.ReadInt32(), BR.ReadInt32() - 48, BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 83 Then
                Q.Monsters.Add(New Springalaru(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), Ri.Next(40, 200)))
            ElseIf Code = 84 Then
                Q.Monsters.Add(New AssaultCamera(BR.ReadInt32, BR.ReadInt32(), RectangleF.FromLTRB(BR.ReadInt32(), 0, BR.ReadInt32(), 1), BR.ReadInt32(), BR.ReadInt32()))
            ElseIf Code = 85 Then
                Q.Monsters.Add(New EvilCoffee(BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadInt32(), BR.ReadBoolean(), BR.ReadInt32()))
            End If
        Next

        'Close the files.
        BR.Close()
        FS.Close()

    End Sub

    Private Sub SetCusText(ByVal FullText As String)
        Dim LV As Integer = 0
        If FullText Is Nothing Then
            CusLines = 0
        ElseIf FullText.Length <= CHARACTERCOUNT Then
            CusText(0) = FullText
            CusLines = 1
        Else
            CusLines = 0
            Do Until LV >= FullText.Length
                If FullText.Length < LV + CHARACTERCOUNT Then
                    CusText(LV \ CHARACTERCOUNT) = FullText.Substring(LV)
                Else
                    CusText(LV \ CHARACTERCOUNT) = FullText.Substring(LV, CHARACTERCOUNT)
                End If
                LV += CHARACTERCOUNT
                CusLines += 1
            Loop
            If CusLines >= CHARLINECOUNT Then
                CusLines = CHARLINECOUNT - 1
            End If
        End If
    End Sub

    Private Sub SaveGame(ByVal Filename As String)
        Dim FS As System.IO.FileStream = New System.IO.FileStream(Filename, IO.FileMode.OpenOrCreate)
        Dim BW As System.IO.BinaryWriter = New IO.BinaryWriter(FS)
        Dim LV As Integer
        AutoSaveLevel = Filename
        For LV = 0 To NUMLEVELSEX
            BW.Write(CompletedLev(LV))
            BW.Write(CoinCount(LV))
            BW.Write(HasUpgrade(LV))
        Next
        For LV = 0 To 9
            BW.Write(Player.OwnedWeapons(LV))
            BW.Write(Player.OwnedWeapons(10 + LV))
            BW.Write(Player.OwnedBullets(LV))
        Next
        BW.Write(Player.Shots.GetUpperBound(0))

        BW.Write(Player.ShotTime)
        BW.Write(Player.ShotSpeed)
        BW.Write(Player.LongJumpEnable)
        BW.Write(Player.DefenseUp)
        BW.Write(Player.EvadeUp)
        BW.Write(Player.VisionUp)
        BW.Write(Player.IceStopEnable)
        BW.Write(Player.FlameRetardEnable)
        BW.Write(Player.WallKickEnable)
        BW.Write(Player.DoubleJumpEnable)
        BW.Write(Player.MysticPowerEnable)
        BW.Write(False)

        BW.Close()
        FS.Close()
    End Sub
    Private Sub LoadGame(ByVal Filename As String)
        Dim FS As System.IO.FileStream = New System.IO.FileStream(Filename, IO.FileMode.OpenOrCreate)
        Dim BR As System.IO.BinaryReader = New IO.BinaryReader(FS)
        Dim LV As Integer, SC As Integer
        AutoSaveLevel = Filename
        For LV = 0 To NUMLEVELSEX
            CompletedLev(LV) = BR.ReadBoolean()
            CoinCount(LV) = BR.ReadInt32()
            HasUpgrade(LV) = BR.ReadBoolean()
        Next
        For LV = 0 To 9
            Player.OwnedWeapons(LV) = BR.ReadBoolean()
            Player.OwnedWeapons(10 + LV) = BR.ReadBoolean()
            Player.OwnedBullets(LV) = BR.ReadBoolean()
        Next
        SC = BR.ReadInt32()
        Player.Shots = New SelfBullet(SC) {}

        Player.ShotTime = BR.ReadInt32()
        Player.ShotSpeed = BR.ReadSingle()
        Player.LongJumpEnable = BR.ReadBoolean()
        Player.DefenseUp = BR.ReadBoolean()
        Player.EvadeUp = BR.ReadBoolean()
        Player.VisionUp = BR.ReadBoolean()
        Player.IceStopEnable = BR.ReadBoolean()
        Player.FlameRetardEnable = BR.ReadBoolean()
        Player.WallKickEnable = BR.ReadBoolean()
        Player.DoubleJumpEnable = BR.ReadBoolean()
        Player.MysticPowerEnable = BR.ReadBoolean()

        UpdateUnlockedRows()

        'CoinCount = New Integer(NUMLEVELSEX) {}
        BR.Close()
        FS.Close()
    End Sub
    Public Sub UpdateUnlockedRows()
        Dim LV, SC, CN, MCN As Integer
        For SC = 0 To 7
            CN = 0
            MCN = 0
            For LV = 0 To 8
                CN += CoinCount(LV + 9 * SC)
                MCN += MaxCoinCount(LV + 9 * SC)
            Next
            If Convert.ToInt32(MCN * 0.9) < CN Then
                Me.RowUnlocked(SC) = True
            End If
        Next
    End Sub

    Private Sub LoadControls()
        Dim S As String, LV As Integer
        Dim SR As IO.StreamReader
        SR = New IO.StreamReader(Application.StartupPath & "\files\ctrls.txt")

        S = SR.ReadLine() : UpKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : DownKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : LeftKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : RightKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : JumpKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : FireKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : RunKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : MenuKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : UseDoorKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)

        S = SR.ReadLine() : WeaponPrevKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : WeaponNextKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : BulletPrevKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)
        S = SR.ReadLine() : BulletNextKey = DirectCast([Enum].Parse(GetType(Keys), S), Keys)

        SR.Close()

    End Sub
    Private Sub SaveControls()
        Dim SW As IO.StreamWriter
        Dim LV As Integer
        SW = New IO.StreamWriter(Application.StartupPath & "\files\ctrls.txt")

        SW.WriteLine(UpKey.ToString())
        SW.WriteLine(DownKey.ToString())
        SW.WriteLine(LeftKey.ToString())
        SW.WriteLine(RightKey.ToString())
        SW.WriteLine(JumpKey.ToString())
        SW.WriteLine(FireKey.ToString())
        SW.WriteLine(RunKey.ToString())
        SW.WriteLine(MenuKey.ToString())
        SW.WriteLine(UseDoorKey.ToString())

        SW.WriteLine(WeaponPrevKey.ToString())
        SW.WriteLine(WeaponNextKey.ToString())
        SW.WriteLine(BulletPrevKey.ToString())
        SW.WriteLine(BulletNextKey.ToString())

        SW.Close()
    End Sub
End Class

Public Interface IMonster

    Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)

    'Sub Draw(ByVal GFX As Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer)
    Sub Draw(ByRef vs() As CustomVertex.PositionColoredTextured, ByVal CI As Integer)

    Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean


    ReadOnly Property Position() As Vector3
    ReadOnly Property Width() As Single
    ReadOnly Property Height() As Single
    ReadOnly Property Rect() As RectangleF

    ReadOnly Property ID() As Integer
    'which texture to use


    Function IntersectsWith(ByVal R As RectangleF) As Boolean
    Function Dead() As Boolean
End Interface
Public Interface IOpticalTool
    Sub Illumination(ByVal Q As LevelSuite, ByVal P As PointF)

End Interface

'Private mr As RectangleF
'Private internalcounter As Integer
'Private goingleft As Boolean
'Private spd As Single

#Region "Unimplemented"
Public Class Fedulus
    Implements IMonster

    Private leftorient As Boolean   'Is fedulus aimed to the left to the left
    Private firefrequency As Integer   'How often will fedulus shoot a flame
    Private firerate As Integer   'How long fedulus will shoot flames
    Private firetime As Integer  'How long fedulus flames are in the air
    Private waittime As Integer  'How long fedulus waits before shooting again.
    Private firing As Boolean   'Is fedulus shooting?

    Private mr As RectangleF
    Private internalcounter As Integer

    Const MYHEIGHT As Integer = 16
    Const MYWIDTH As Integer = 16

    Public Sub New(ByVal X As Single, ByVal Y As Single, ByVal Waits As Integer, ByVal FiresForThisLong As Integer, ByVal ThisOften As Integer, ByVal FlameTimer As Integer, ByVal InLeftDirection As Boolean)
        mr = New RectangleF(X, Y, MYWIDTH, MYHEIGHT)
        Me.waittime = Waits
        Me.firerate = FiresForThisLong
        Me.firefrequency = ThisOften
        Me.firetime = FlameTimer
        Me.leftorient = InLeftDirection
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If leftorient Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, -N, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, N, &HFFFFFFFF, 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, -N, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, N, &HFFFFFFFF, 1.0F, 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, -N, &HFFFFFFFF, 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, -N, &HFFFFFFFF, 1.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, N, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, N, &HFFFFFFFF, 1.0F, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 14
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If firing Then
            If Me.internalcounter Mod Me.firefrequency = 0 Then
                If leftorient Then
                    Q.Monsters.Add(New Bullet(mr.X + (MYWIDTH >> 1), mr.Y + (MYHEIGHT >> 1), -3, 0, 0, 0, firetime))
                Else
                    Q.Monsters.Add(New Bullet(mr.X + (MYWIDTH >> 1), mr.Y + (MYHEIGHT >> 1), 3, 0, 0, 0, firetime))
                End If
            End If
            If Me.internalcounter >= Me.firerate Then
                firing = False
                Me.internalcounter = 0
            End If
        ElseIf Me.internalcounter >= Me.waittime Then
            firing = True
            Me.internalcounter = 0
        End If
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function
End Class
Public Class Atacant
    Implements IMonster

    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingdown As Boolean
    Private OY As Single

    Private spd, amplitude As Single

    Const MYWIDTH As Integer = 18
    Const MYHEIGHT As Integer = 18

    Public Sub New(ByVal X As Single, ByVal Y As Single, ByVal Spd As Single, ByVal Amp As Single)
        mr = New RectangleF(X, Y, MYWIDTH, MYHEIGHT)
        Me.spd = Spd
        Me.amplitude = Amp
        goingdown = True
        OY = Y
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim Ho, Hf As Single
        Ho = 0.25F * ((Me.internalcounter >> 1) Mod 4)
        Hf = 0.25F * (((Me.internalcounter >> 1) Mod 4) + 1)
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, Ho, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, Ho, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, Hf, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, Hf, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 10
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingdown Then
            Me.mr.Y += spd
            If mr.Y - OY >= amplitude Then
                goingdown = False
            End If
        Else
            Me.mr.Y -= spd
            If OY - mr.Y >= amplitude Then
                goingdown = True
            End If
        End If
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function
End Class
#End Region
#Region "Steppers"
Public Class DeepStep
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private intimeout As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 15, MYHEIGHT As Integer = 15
    Const NUMFRAMES As Integer = 24
    Const TIMEOUT As Integer = 80

    Public Sub New(ByVal L As Single, ByVal T As Single, ByVal LeftEnd As Single, ByVal RightEnd As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 1
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYWIDTH - 4, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 1
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If intimeout Then
            internalcounter += 1
            If internalcounter >= TIMEOUT Then
                intimeout = False
            End If
        Else
            If goingleft Then
                mr.Offset(-spd, 0)
                If mr.Left <= le Then
                    goingleft = False
                    intimeout = True
                    internalcounter = 0
                End If
            Else
                mr.Offset(spd, 0)
                If mr.Right >= re Then
                    goingleft = True
                    intimeout = True
                    internalcounter = 0
                End If
            End If

            Me.internalcounter += 1
            If Me.internalcounter >= NUMFRAMES Then
                Me.internalcounter = 0
            End If
        End If

    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, MYHEIGHT, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Integer = Me.internalcounter >> 2
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 8
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class AcidStep
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 2
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 2
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            mr.Offset(-spd, 0)
            If mr.Left <= le Then
                goingleft = False
            End If
        Else
            mr.Offset(spd, 0)
            If mr.Right >= re Then
                goingleft = True
            End If
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, MYHEIGHT, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function




    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(mr.X + MYWIDTH \ 2, mr.Y + MYHEIGHT \ 2, 0.0F)
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 1
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class IcyStep
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private slipping As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single, ls As Single, rs As Single
    Private spd As Single, effspd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        Dim SlipDist As Single
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        effspd = Speed
        hp = 3
        SlipDist = Speed * (8.0F * Speed + 2.0F) / 4.0F
        ls = le + SlipDist
        rs = re - SlipDist
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        Dim SlipDist As Single
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYWIDTH, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        effspd = Speed
        hp = 3
        SlipDist = Speed * (8.0F * Speed + 2.0F) / 4.0F
        ls = le + SlipDist
        rs = re - SlipDist
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If slipping Then
            effspd -= 0.25F
        End If
        If goingleft Then
            mr.Offset(-effspd, 0)
            If effspd <= 0 Then
                goingleft = False
                effspd = spd
                slipping = False
            ElseIf mr.Left < ls Then
                slipping = True
            End If
        Else
            mr.Offset(effspd, 0)
            If effspd <= 0 Then
                goingleft = True
                effspd = spd
                slipping = False
            ElseIf mr.Right > rs Then
                slipping = True
            End If
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, MYHEIGHT, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(mr.X + MYWIDTH \ 2, mr.Y + MYHEIGHT \ 2, 0.0F)
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 32
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class SeaStep
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 3
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 3
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        spd += 1
        goingleft = Not goingleft
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 45
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            mr.Offset(-spd, 0)
            If mr.Left <= le Then
                goingleft = False
            End If
        Else
            mr.Offset(spd, 0)
            If mr.Right >= re Then
                goingleft = True
            End If
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class FieryStep
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 4
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 4
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Q.Monsters.Add(New CustomShot(mr.X + 10, mr.Y, 0, -1, 0, 0, 100, 40, New DirectBullet(0, 0, RectangleF.Empty, 2.5F, 120)))
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 53
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            mr.Offset(-spd, 0)
            If mr.Left <= le Then
                goingleft = False
            End If
        Else
            mr.Offset(spd, 0)
            If mr.Right >= re Then
                goingleft = True
            End If
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SteelStep
    Implements IMonster




    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private intimeout As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd, vt, oy As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32
    Const TIMEOUT As Integer = 80
    Const JUMPSPEED As Single = -3.9F
    Const GRAVITY As Single = -2.0F * JUMPSPEED / (TIMEOUT - 2)

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        oy = mr.Top
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 2
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        oy = mr.Top
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 2
        Me.internalcounter = NUMFRAMES - 1
    End Sub



    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 66
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If intimeout Then
            internalcounter += 1
            If internalcounter >= TIMEOUT Then
                intimeout = False
                mr.Y = oy
            Else
                vt += GRAVITY
                mr.Offset(0, vt)
            End If
        Else
            If goingleft Then
                mr.Offset(-spd, 0)
                If mr.Left <= le Then
                    goingleft = False
                    intimeout = True
                    internalcounter = 0
                    vt = JUMPSPEED
                End If
            Else
                mr.Offset(spd, 0)
                If mr.Right >= re Then
                    goingleft = True
                    intimeout = True
                    internalcounter = 0
                    vt = JUMPSPEED
                End If
            End If

            Me.internalcounter += 1
            If Me.internalcounter >= NUMFRAMES Then
                Me.internalcounter = 0
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SunStep
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft, kill As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 5
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 5
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            mr.Offset(-spd, 0)
            If mr.Left <= le Then
                goingleft = False
            End If
        Else
            mr.Offset(spd, 0)
            If mr.Right >= re Then
                goingleft = True
            End If
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
        If hp <= 0 Then
            kill = True
            If (Q.Monsters.Count Mod 2 = 0) Then
                Q.Monsters.Add(New Napalm(R.Left - 492, R.Top - 1000, 3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left - 472, R.Top - 1000, 3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left - 512, R.Top - 1000, 3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left - 492, R.Top - 1020, 3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left - 492, R.Top - 980, 3, 6, 0, 0, 350))
            Else
                Q.Monsters.Add(New Napalm(R.Left + 508, R.Top - 1000, -3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left + 488, R.Top - 1000, -3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left + 528, R.Top - 1000, -3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left + 508, R.Top - 1020, -3, 6, 0, 0, 350))
                Q.Monsters.Add(New StoneBullet(R.Left + 508, R.Top - 980, -3, 6, 0, 0, 350))
            End If
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, MYHEIGHT, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function




    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(mr.X + MYWIDTH \ 2, mr.Y + MYHEIGHT \ 2, 0.0F)
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 73
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class DarkStep
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private invulnerary As Integer

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 3
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 3
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            mr.Offset(-spd, 0)
            If mr.Left <= le Then
                goingleft = False
            End If
        Else
            mr.Offset(spd, 0)
            If mr.Right >= re Then
                goingleft = True
            End If
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
        If invulnerary > 0 Then
            invulnerary -= 1
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function
    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If invulnerary Mod 2 = 0 Then
            If goingleft Then
                vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(mr.X + MYWIDTH \ 2, mr.Y + MYHEIGHT \ 2, 0.0F)
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 87
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If invulnerary = 0 Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            invulnerary = 32
            Return True
        Else
            Return False
        End If
    End Function

End Class
Public Class SilverStep
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft, speedup As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 6
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 6
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.125F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.125F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.125F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.125F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        speedup = True
        If ISpd > 0 Then
            goingleft = True
        ElseIf ISpd < 0 Then
            goingleft = False
        Else
            goingleft = Not goingleft
        End If
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 94
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim Vx As Single
        If speedup Then
            Vx = 2 * spd
        Else
            Vx = spd
        End If
        If goingleft Then
            mr.Offset(-Vx, 0)
            If mr.Left <= le Then
                goingleft = False
                speedup = False
            End If
        Else
            mr.Offset(Vx, 0)
            If mr.Right >= re Then
                goingleft = True
                speedup = False
            End If
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

End Class

Public Class Vokat
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private intimeout As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 15, MYHEIGHT As Integer = 41
    Const NUMFRAMES As Integer = 24
    Const TIMEOUT As Integer = 80

    Public Sub New(ByVal L As Single, ByVal T As Single, ByVal LeftEnd As Single, ByVal RightEnd As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 5
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT - 4, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 5
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If intimeout Then
            internalcounter += 1
            If internalcounter >= TIMEOUT Then
                intimeout = False
            End If
        Else
            If goingleft Then
                mr.Offset(-spd, 0)
                If mr.Left <= le Then
                    goingleft = False
                    intimeout = True
                    internalcounter = 0
                End If
            Else
                mr.Offset(spd, 0)
                If mr.Right >= re Then
                    goingleft = True
                    intimeout = True
                    internalcounter = 0
                End If
            End If

            Me.internalcounter += 1
            If Me.internalcounter >= NUMFRAMES Then
                Me.internalcounter = 0
            End If
        End If

    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, MYHEIGHT, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Integer = Me.internalcounter >> 2
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 38
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            hp -= 10
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        Else
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        End If
        Return True
    End Function
End Class
Public Class WallStep
    Implements IMonster

    Private mr As RectangleF
    Private goingleft As Boolean
    Private goingup, engaged As Boolean
    Private spd As Single
    Private hp As Integer
    Private ue, de As Single
    Private internalcounter As Integer
    Private intimeout As Boolean

    Const MYWIDTH As Single = 12.0F
    Const MYHEIGHT As Single = 15.0F
    Const NUMFRAMES As Integer = 24
    Const TIMEOUT As Integer = 80

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Goleft As Boolean, ByVal Sp As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        goingleft = Goleft
        spd = Sp
        hp = 1
        internalcounter = 0
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Integer = (Me.internalcounter Mod NUMFRAMES) >> 2
        If goingleft Then
            If goingup Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 1.0F)
            End If
        Else
            'If goingup Then
            '    vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.0F)
            '    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            '    vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.0F)
            '    vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
            'Else
            '    vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            '    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 1.0F)
            '    vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
            '    vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 1.0F)
            'End If
            If goingup Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F / 6.0F, 1.0F)
            End If
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 60
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim Wall As RectangleF
        If engaged Then
            If intimeout Then
                internalcounter += 1
                If internalcounter >= TIMEOUT Then
                    intimeout = False
                End If
            Else
                If goingup Then
                    mr.Offset(0, -spd)
                    If mr.Top < ue Then
                        goingup = False
                        intimeout = True
                        internalcounter = 0
                    End If
                Else
                    mr.Offset(0, spd)
                    If mr.Bottom > de Then
                        goingup = True
                        intimeout = True
                        internalcounter = 0
                    End If
                End If
                internalcounter += 1

                If internalcounter Mod 240 = 90 Then
                    If goingleft Then
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 3.0F, 4.0F, 0.0F, -0.125F, 80))
                    Else
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 3.0F, 4.0F, 0.0F, -0.125F, 80))
                    End If
                End If
            End If


        Else
            If goingleft Then
                mr.Offset(-spd, 0.0F)
                For Each Wall In Q.Walls
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Right - mr.Left, 0)
                        engaged = True
                    End If
                Next
                For Each Wall In Q.Extrawalls
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Right - mr.Left, 0)
                        engaged = True
                    End If
                Next
                For Each Wall In Q.Groundwalls
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Right - mr.Left, 0)
                        engaged = True
                    End If
                Next
                For Each Wall In Q.Crates
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Right - mr.Left, 0)
                        engaged = True
                    End If
                Next
            Else
                mr.Offset(spd, 0.0F)
                For Each Wall In Q.Walls
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Left - mr.Right, 0)
                        engaged = True
                    End If
                Next
                For Each Wall In Q.Extrawalls
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Left - mr.Right, 0)
                        engaged = True
                    End If
                Next
                For Each Wall In Q.Groundwalls
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Left - mr.Right, 0)
                        engaged = True
                    End If
                Next
                For Each Wall In Q.Crates
                    If mr.IntersectsWith(Wall) Then
                        ue = Wall.Top
                        de = Wall.Bottom
                        mr.Offset(Wall.Left - mr.Right, 0)
                        engaged = True
                    End If
                Next

            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class LowStep
    Implements IMonster
    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private intimeout As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 15
    Const NUMFRAMES As Integer = 24
    Const TIMEOUT As Integer = 80
    Public Sub New(ByVal L As Single, ByVal T As Single, ByVal LeftEnd As Single, ByVal RightEnd As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(L, T + 11, MYWIDTH, 4)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 1
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - 4, MYWIDTH, 4)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 1
        Me.internalcounter = NUMFRAMES - 1
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Integer = Me.internalcounter >> 2
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return mr.Height
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 67
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If intimeout Then
            internalcounter += 1
            If internalcounter >= TIMEOUT - 31 Then
                mr.Offset(0, 2.0F)
                mr.Height -= 2
            ElseIf internalcounter >= TIMEOUT - 63 Then
                mr.Offset(0, -2.0F)
                mr.Height += 2
            End If
            If internalcounter >= TIMEOUT Then
                intimeout = False
            End If
        Else
            If goingleft Then
                mr.Offset(-spd, 0)
                If mr.Left <= le Then
                    goingleft = False
                    intimeout = True
                    internalcounter = 0
                End If
            Else
                mr.Offset(spd, 0)
                If mr.Right >= re Then
                    goingleft = True
                    intimeout = True
                    internalcounter = 0
                End If
            End If

            Me.internalcounter += 1
            If Me.internalcounter >= NUMFRAMES Then
                Me.internalcounter = 0
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class HighStep
    Implements IMonster



    'Standard declarations.
    Private mr, ir As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private intimeout As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 15, MYHEIGHT As Integer = 100
    Const NUMFRAMES As Integer = 24
    Const TIMEOUT As Integer = 80

    Public Sub New(ByVal L As Single, ByVal T As Single, ByVal LeftEnd As Single, ByVal RightEnd As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        ir = New RectangleF(L, T, MYWIDTH, 16.0F)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 1
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT - 4, MYWIDTH, MYHEIGHT)
        ir = New RectangleF(mr.X, mr.Y, mr.Width, 16.0F)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 1
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If intimeout Then
            internalcounter += 1
            If internalcounter >= TIMEOUT Then
                intimeout = False
            End If
        Else
            If goingleft Then
                mr.Offset(-spd, 0)
                If mr.Left <= le Then
                    goingleft = False
                    intimeout = True
                    internalcounter = 0
                End If
            Else
                mr.Offset(spd, 0)
                If mr.Right >= re Then
                    goingleft = True
                    intimeout = True
                    internalcounter = 0
                End If
            End If

            Me.internalcounter += 1
            If Me.internalcounter >= NUMFRAMES Then
                Me.internalcounter = 0
            End If
        End If
        ir = New RectangleF(mr.X, mr.Y, mr.Width, 16.0F)

    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, MYHEIGHT, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Integer = Me.internalcounter >> 2
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F / 6.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F / 6.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 6.0F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 83
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return ir
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class

#End Region
#Region "Machines"
Public Class RollingCamera
    Implements IMonster
    Implements IOpticalTool

    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private attacking As Boolean, atktimer As Integer
    Private screwed As Boolean

    Private spd As Single
    Private le, re As Single
    Private orangle As Single

    Private xmultiplier As Single, ymultiplier As Single, auxadd As Single

    Private xorient As Single, yorient As Single

    Const MYWIDTH As Integer = 32
    Const MYHEIGHT As Integer = 32
    Const HALFWIDTH As Integer = MYWIDTH >> 1
    Const HALFHEIGHT As Integer = MYHEIGHT >> 1

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Platform As RectangleF, ByVal Spd As Single, ByVal Angle As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        Me.spd = Spd / 4
        le = Platform.Left
        re = Platform.Right
        orangle = Angle
        xmultiplier = Convert.ToSingle(Math.Sin(orangle * Math.PI / 180))
        ymultiplier = Convert.ToSingle(Math.Cos(orangle * Math.PI / 180))

        xorient = Convert.ToSingle(Math.Sqrt(2) * Math.Sin((-Angle - 45) * Math.PI / 180))
        yorient = Convert.ToSingle(Math.Sqrt(2) * Math.Cos((-Angle - 45) * Math.PI / 180))

    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    mr.Offset(-ScreenLeft, -ScreenTop)

    '    If goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New Rectangle(0, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New Rectangle(0, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)

    'End Sub



    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer
        Const N As Single = 1.0F
        F = internalcounter \ 12
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(1 + xorient, 1 - yorient, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(1 - yorient, 1 - xorient, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(1 + yorient, 1 + xorient, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(1 - xorient, 1 + yorient, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(1 + xorient, 1 - yorient, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(1 - yorient, 1 - xorient, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(1 + yorient, 1 + xorient, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(1 - xorient, 1 + yorient, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 20
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If spd > 0 Then
            If goingleft Then
                mr.Offset(-spd, 0)
                If mr.Left <= le Then
                    goingleft = False
                End If
            Else
                mr.Offset(spd, 0)
                If mr.Right >= re Then
                    goingleft = True
                End If
            End If
        End If
        Me.internalcounter += 1
        If attacking Then
            If Me.internalcounter = 24 Then
                Me.internalcounter = 0
                'Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
                Q.Monsters.Add(New HomingShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8.0F, 0.04F, 120))
                atktimer -= 1
            ElseIf Me.internalcounter = 12 Then
                'Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
                Q.Monsters.Add(New HomingShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8.0F, 0.04F, 120))
                atktimer -= 1
            End If
            If atktimer = 0 Then
                attacking = False
                xmultiplier = Convert.ToSingle(Math.Sin(orangle * Math.PI / 180))
                ymultiplier = Convert.ToSingle(Math.Cos(orangle * Math.PI / 180))
            End If
        ElseIf Not screwed Then
            If Me.internalcounter = 24 Then
                Me.internalcounter = 0
                Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
            ElseIf Me.internalcounter = 12 Then
                Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public Sub Illumination(ByVal Q As LevelSuite, ByVal P As System.Drawing.PointF) Implements IOpticalTool.Illumination
        Dim A, B, C As Microsoft.DirectX.Vector2, L As Single
        A = New Microsoft.DirectX.Vector2(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT)
        B = New Microsoft.DirectX.Vector2(P.X, P.Y)
        C = Microsoft.DirectX.Vector2.Subtract(B, A)
        C.Normalize()
        attacking = True
        atktimer = 2
        xmultiplier = C.X
        ymultiplier = C.Y
        'Q.Monsters.Add(New Bullet(mr.X, mr.Y, vx, Vy, 0, 0, 60))

    End Sub

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Screwer Then
            screwed = True
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return False
        End If
    End Function
End Class
Public Class SwivelCamera
    Implements IMonster
    Implements IOpticalTool

    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private screwed As Boolean

    Private ang As Double
    Private angspd As Double
    Private timeover As Integer
    Private suppressattack As Integer

    Const HALFPI As Double = Math.PI / 2
    Const MYWIDTH As Integer = 32
    Const MYHEIGHT As Integer = 32
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal AngleSpeed As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        angspd = AngleSpeed * Math.PI / 180
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function




    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer
        Const N As Single = 1.0F
        F = internalcounter \ 6
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 21
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If timeover = 0 Then
            If goingleft Then
                ang -= angspd
                If ang <= -HALFPI Then
                    timeover = 60
                    ang = -HALFPI
                End If
            Else
                ang += angspd
                If ang >= HALFPI Then
                    timeover = 60
                    ang = HALFPI
                End If
            End If
        Else
            timeover -= 1
            If timeover = 0 Then
                goingleft = Not goingleft
            End If
        End If
        If suppressattack <> 0 Then
            suppressattack -= 1
        End If
        Me.internalcounter += 1
        If Not screwed Then
            If Me.internalcounter >= 12 Then
                Me.internalcounter = 0
                If timeover > 0 Then
                    If goingleft Then
                        Q.Monsters.Add(New Lightball(mr.X, mr.Y, -8, 0, 0, 0, 50, Me))
                    Else
                        Q.Monsters.Add(New Lightball(mr.X, mr.Y, 8, 0, 0, 0, 50, Me))
                    End If
                Else
                    Q.Monsters.Add(New Lightball(mr.X, mr.Y, 8 * Convert.ToSingle(Math.Sin(ang)), 8 * Convert.ToSingle(Math.Cos(ang)), 0, 0, 50, Me))
                End If
            ElseIf Me.internalcounter = 6 Then
                If timeover > 0 Then
                    If goingleft Then
                        Q.Monsters.Add(New Lightball(mr.X, mr.Y, -8, 0, 0, 0, 50, Me))
                    Else
                        Q.Monsters.Add(New Lightball(mr.X, mr.Y, 8, 0, 0, 0, 50, Me))
                    End If
                Else
                    Q.Monsters.Add(New Lightball(mr.X, mr.Y, 8 * Convert.ToSingle(Math.Sin(ang)), 8 * Convert.ToSingle(Math.Cos(ang)), 0, 0, 50, Me))
                End If
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public Sub Illumination(ByVal Q As LevelSuite, ByVal P As System.Drawing.PointF) Implements IOpticalTool.Illumination
        Dim A, B, C As Microsoft.DirectX.Vector2
        If suppressattack = 0 Then
            A = New Microsoft.DirectX.Vector2(mr.X + 16, mr.Y + 16)
            B = New Microsoft.DirectX.Vector2(P.X, P.Y)
            C = Microsoft.DirectX.Vector2.Subtract(B, A)
            C.Normalize()
            Q.Monsters.Add(New HomingShot(mr.X + 16, mr.Y + 16, 8.0F, 0.04F, 120))
            suppressattack = 6
        End If
    End Sub

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Screwer Then
            screwed = True
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return False
        End If
    End Function
End Class
Public Class MotionDetector
    Implements IMonster


    Private mr As RectangleF
    Private internalcounter As Integer
    Private mem As RectangleF
    Private hp As Integer

    Private dradius As Single

    Const MYWIDTH As Integer = 32
    Const MYHEIGHT As Integer = 32

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal DetectRadius As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        dradius = DetectRadius
        hp = 1
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F


        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 22
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim A, B, C As Microsoft.DirectX.Vector2
        Dim Vx, Vy As Single
        Me.internalcounter += 1
        If Me.internalcounter = 29 Then
            mem = R
        ElseIf Me.internalcounter = 30 Then
            Me.internalcounter = 0
            If R.Left <> mem.Left OrElse R.Top <> mem.Top Then
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                If C.LengthSq < dradius * dradius Then
                    C.Normalize()
                    Vx = dradius * C.X / 60
                    Vy = dradius * C.Y / 60
                    Q.Monsters.Add(New Bullet(mr.X, mr.Y, Vx, Vy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X, mr.Y, -Vx / 6, Vy / 6, 0, 0, 360))
                    Q.Monsters.Add(New Bullet(mr.X, mr.Y, Vx / 6, -Vy / 6, 0, 0, 360))
                    Q.Monsters.Add(New Bullet(mr.X, mr.Y, -Vx / 6, -Vy / 6, 0, 0, 360))
                End If
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Screwer OrElse BY = ShotTypes.Corkwheel OrElse BY = ShotTypes.Stonebullet OrElse BY = ShotTypes.Plasma OrElse BY = ShotTypes.Rubberet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        End If
    End Function
End Class
Public Class RisingFire
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private shooting As Boolean
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 26
    Const TIMEOUT As Integer = 80
    Const RESTTIMEOUT As Integer = 20
    Const SHOTDX As Single = 5

    Public Sub New(ByVal BaseRect As RectangleF, ByVal LeftOffset As Single)
        mr = New RectangleF(BaseRect.X + LeftOffset, BaseRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        hp = 2
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If shooting Then
            If internalcounter > (TIMEOUT >> 2) Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.667F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.667F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.333F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.667F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.667F, 1.0F)

            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.333F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 12
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If shooting Then
            If Me.internalcounter >= TIMEOUT Then
                shooting = False
                Me.internalcounter = 0
            End If
        Else
            If Me.internalcounter >= RESTTIMEOUT Then
                shooting = True
                Me.internalcounter = 0
                Q.Monsters.Add(New Bullet(mr.Left + SHOTDX, mr.Top, 0, -3, 0, 0, 80))
            End If
        End If
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            hp += 3
            shooting = False
            internalcounter = RESTTIMEOUT
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Return True
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Plasma Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function
End Class
Public Class Modular
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private frame As Integer
    Private state As ModuStates
    Private fallspeed, spd As Single

    Private Enum ModuStates
        Standby
        Dropping
        Slowing
        Firing
    End Enum

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const ATKRANGE As Single = 200.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 2
        state = ModuStates.Standby
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse mr.X < 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F

        vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, frame / 4.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, frame / 4.0F, 0.5F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (frame + 1) / 4.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (frame + 1) / 4.0F, 0.5F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 40
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Select Case state
            Case ModuStates.Standby
                If mr.X - R.X < ATKRANGE AndAlso R.X - mr.X < ATKRANGE Then
                    state = ModuStates.Dropping
                    fallspeed = 8
                End If
            Case ModuStates.Dropping
                If R.Y - 72.0F <= mr.Y Then
                    state = ModuStates.Slowing
                Else
                    mr.Y += fallspeed
                End If
            Case ModuStates.Slowing
                fallspeed -= 0.5F
                If fallspeed > 6.0F Then
                    frame = 0
                ElseIf fallspeed > 4.0F Then
                    frame = 1
                ElseIf fallspeed > 2.0F Then
                    frame = 2
                ElseIf fallspeed > 0.0F Then
                    frame = 3
                Else
                    frame = 0
                    spd = 0
                    state = ModuStates.Firing
                End If
                mr.Y += fallspeed
            Case ModuStates.Firing
                If spd >= 4.0F Then
                    mr.X -= 4.0F
                Else
                    spd += 0.25F
                End If
        End Select
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class ModularBall
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private frame As Integer
    Private state As ModuStates
    Private fallspeed, spd As Single
    Private rise As Boolean

    Const ATKRANGE As Single = 200.0F

    Private Enum ModuStates
        Standby
        Dropping
        Slowing
        Firing
    End Enum

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Riser As Boolean)
        mr = New RectangleF(Xo, Yo, 4, 4)
        hp = 1
        rise = Riser
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse mr.X < 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, frame / 4.0F, 0.5F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, frame / 4.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (frame + 1) / 4.0F, 0.5F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (frame + 1) / 4.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If mr.Width >= 30.0F Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Return False
        End If

    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return mr.Height
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 40
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        If mr.Width >= 30 Then
            Return mr.IntersectsWith(R)
        Else
            Return False
        End If
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Select Case state
            Case ModuStates.Standby
                If mr.X - R.X < ATKRANGE AndAlso R.X - mr.X < ATKRANGE Then
                    state = ModuStates.Dropping
                    fallspeed = 8
                End If
            Case ModuStates.Dropping
                If R.Y - 72.0F <= mr.Y Then
                    state = ModuStates.Slowing
                Else
                    mr.Y += fallspeed
                End If
            Case ModuStates.Slowing
                fallspeed -= 0.5F
                If fallspeed > 6.0F Then
                    frame = 0
                ElseIf fallspeed > 4.0F Then
                    frame = 1
                ElseIf fallspeed > 2.0F Then
                    frame = 2
                ElseIf fallspeed > 0.0F Then
                    frame = 3
                Else
                    frame = 0
                    spd = 0
                    state = ModuStates.Firing
                End If
                mr.Y += fallspeed
            Case ModuStates.Firing
                If spd >= 4.0F Then
                    mr.X -= 4.0F
                Else
                    spd += 0.25F
                    mr.X = mr.Left - 0.875F
                    If rise Then
                        mr.Y = mr.Top - 5.0F
                    Else
                        mr.Y = mr.Top + 5.0F
                    End If
                    mr.Width = 4.0F + spd * 7.0F
                    mr.Height = mr.Width
                End If
        End Select
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            If rise Then
                Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, mr.Width - 32)
            Else
                Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 32 - mr.Width)
            End If
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return mr.Width
        End Get
    End Property
End Class
Public Class CasukeBall
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private falling As Boolean, onslope As Boolean
    Private fallspeed As Single
    Private hp As Integer

    Const MYWIDTH As Single = 22, MYHEIGHT As Single = 22
    Const NUMFRAMES As Integer = 4

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYWIDTH, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        Me.internalcounter = NUMFRAMES - 1
        hp = 7
    End Sub
    Public Sub New(ByVal LocX As Single, ByVal LocY As Single, ByVal LE As Single, ByVal RE As Single, ByVal movingleft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(LocX, LocY, MYWIDTH, MYHEIGHT)
        Me.le = LE
        Me.re = RE
        goingleft = movingleft
        spd = Speed
        Me.internalcounter = NUMFRAMES - 1
        hp = 7
    End Sub


    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Const GRAVITY As Single = 0.8F, GRAVITYTH As Single = GRAVITY * 1.5F
        Const TERMINALVELOCITY As Integer = 40

        If goingleft Then
            mr.Offset(-spd, 0)
            For Each Wall As RectangleF In Q.Walls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each opsw As OpenerSwitch In Q.RTDs
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Right - mr.Left, 0)
                    goingleft = False
                End If
            Next
            For Each opsw As OpenerSwitch In Q.Pneumos
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Right - mr.Left, 0)
                    goingleft = False
                End If
            Next
            If mr.Right <= le Then
                falling = True
            End If
        Else
            mr.Offset(spd, 0)
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each opsw As OpenerSwitch In Q.RTDs
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Left - mr.Right, 0)
                    goingleft = True
                End If
            Next
            For Each opsw As OpenerSwitch In Q.Pneumos
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Left - mr.Right, 0)
                    goingleft = True
                End If
            Next
            If mr.Left >= re Then
                falling = True
            End If
        End If
        If falling Then

            mr.Offset(0, fallspeed + GRAVITY / 2.0F)
            'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
            fallspeed += GRAVITY
            'dV = (PlayerVeloc +=) A (GRAVITY)
            'Moves the character according to its velocity, gravity, and location.

            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.

                        falling = False
                        'Hit the ground, we are not jumping.
                        fallspeed = 0

                        le = Platform.Left : re = Platform.Right
                        'Set the endpoints for the platform.
                        mr.Offset(0, Platform.Top - mr.Bottom)
                        'Move the player onto this platform.
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0


                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.

                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.

                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    falling = False
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    falling = False
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    falling = False
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    falling = False
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    falling = False
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next

        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = Me.internalcounter >> 1
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.5F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.5F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.5F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.5F, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 3
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Dim Vx, Vy As Single
        If BY = ShotTypes.ShockWave OrElse BY = ShotTypes.Plasma Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Aquabullet Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Screwer Then
            hp -= 25
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 60))
            Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
            Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 60))
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 7
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Napalm OrElse BY = ShotTypes.Pearl Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        End If
        Return True
    End Function
End Class
Public Class BoundingBall
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private hp As Integer

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private falling As Boolean, onslope As Boolean
    Private fallspeed As Single

    Const MYWIDTH As Single = 22, MYHEIGHT As Single = 22
    Const NUMFRAMES As Integer = 4

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYWIDTH, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        Me.internalcounter = NUMFRAMES - 1
        hp = 7
    End Sub
    Public Sub New(ByVal LocX As Single, ByVal LocY As Single, ByVal LE As Single, ByVal RE As Single, ByVal movingleft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(LocX, LocY, MYWIDTH, MYHEIGHT)
        Me.le = LE
        Me.re = RE
        goingleft = movingleft
        spd = Speed
        Me.internalcounter = NUMFRAMES - 1
        hp = 7
    End Sub

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * F, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Const GRAVITY As Single = 0.7F, GRAVITYTH As Single = GRAVITY * 1.5F
        Const TERMINALVELOCITY As Single = 20.0F

        If goingleft Then
            mr.Offset(-spd, 0)
            For Each Wall As RectangleF In Q.Walls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                End If
            Next
            For Each opsw As OpenerSwitch In Q.RTDs
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Right - mr.Left, 0)
                    goingleft = False
                End If
            Next
            For Each opsw As OpenerSwitch In Q.Pneumos
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Right - mr.Left, 0)
                    goingleft = False
                End If
            Next
            If mr.Right <= le Then
                falling = True
            End If
        Else
            mr.Offset(spd, 0)
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                End If
            Next
            For Each opsw As OpenerSwitch In Q.RTDs
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Left - mr.Right, 0)
                    goingleft = True
                End If
            Next
            For Each opsw As OpenerSwitch In Q.Pneumos
                If mr.IntersectsWith(opsw.Gate) Then
                    mr.Offset(opsw.Gate.Left - mr.Right, 0)
                    goingleft = True
                End If
            Next

            If mr.Left >= re Then
                falling = True
            End If
        End If
        If falling Then

            mr.Offset(0, fallspeed + GRAVITY / 2.0F)
            'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
            fallspeed += GRAVITY
            'dV = (PlayerVeloc +=) A (GRAVITY)
            'Moves the character according to its velocity, gravity, and location.

            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.

                        If fallspeed < 4 Then
                            falling = False
                            'Hit the ground, we are not jumping.
                            fallspeed = 0

                            le = Platform.Left : re = Platform.Right
                            'Set the endpoints for the platform.
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            'Move the player onto this platform.
                        Else
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            'Move the player onto this platform.
                            fallspeed = -(fallspeed * 0.95F)
                            'Bounce back.
                        End If


                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.

                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)

                        fallspeed = 0

                    ElseIf fallspeed < 4 Then
                        falling = False
                        'Stop the jump.
                        fallspeed = 0


                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        fallspeed = -(fallspeed * 0.95F)
                        'Bounce back.
                    End If


                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)
                        fallspeed = 0
                    ElseIf fallspeed < 4 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        fallspeed = -(fallspeed * 0.95F)
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.

                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)

                        fallspeed = 0

                    ElseIf fallspeed < 4 Then
                        falling = False
                        'Stop the jump.
                        fallspeed = 0


                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        fallspeed = -(fallspeed * 0.95F)
                        'Bounce back.
                    End If

                End If

            Next
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Right < 0 OrElse hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = Me.internalcounter >> 1
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.5F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.5F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.5F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.5F, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 3
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(Me.mr.X, Me.mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Dim Vx, Vy As Single
        If BY = ShotTypes.ShockWave OrElse BY = ShotTypes.Plasma Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Aquabullet Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Screwer Then
            hp -= 25
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 60))
            Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
            Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 60))
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 7
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Napalm OrElse BY = ShotTypes.Pearl Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        End If
        Return True
    End Function
End Class
Public Class ShieldTurret
    Implements IMonster
    Private mr As RectangleF
    Private goingleft As Boolean
    Private hp As Integer
    Private internalcounter As Integer
    Private screwed As Boolean

    Private operating As Boolean

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const OPERATIONTIME As Integer = 256
    Const DELAYTIME As Integer = 64
    Const XAZ As Single = 120.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Facingleft As Boolean)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        goingleft = Facingleft
        hp = 1
        operating = False
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim T, B As Single
        If goingleft Then
            T = 0.0F
            B = 0.5F
        Else
            T = 0.5F
            B = 1.0F
        End If
        If operating Then
            If internalcounter = 1 OrElse internalcounter = OPERATIONTIME - 1 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, T)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, B)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, T)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, B)
            ElseIf internalcounter = 2 OrElse internalcounter = OPERATIONTIME - 2 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, T)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, B)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, T)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, B)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, T)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, B)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, T)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, B)
            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, T)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, B)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, T)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, B)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If operating Then
            If BY = ShotTypes.Screwer Then
                screwed = True
                Q.SonicEvents.Add(SonicEventType.ShotRavage)
                Return True
            Else
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            End If
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 46
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If Not operating Then
            If internalcounter >= DELAYTIME Then
                If mr.X - R.X < XAZ AndAlso R.X - mr.X < XAZ Then
                    operating = True
                    internalcounter = 0
                End If
            End If
            internalcounter += 1
        Else
            If internalcounter Mod 64 = 0 AndAlso internalcounter <> 0 Then
                If Not screwed Then
                    If goingleft Then
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -6, 0, 0, 0, 50))
                    Else
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 6, 0, 0, 0, 50))
                    End If
                End If
            End If
            internalcounter += 1
            If internalcounter >= OPERATIONTIME Then
                operating = False
                internalcounter = 0
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Arroglestop
    Implements IMonster

    Private mr As RectangleF
    Private ir As RectangleF
    Private hp As Integer
    Private internalcounter As Integer
    Private standby As Boolean

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 128.0F
    Const XAZ As Single = 200.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        ir = New RectangleF(Xo, Yo, MYWIDTH, 64.0F)
        hp = 4
        standby = True
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F

        If standby Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
        Else
            If internalcounter < 64 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            ElseIf internalcounter < 128 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            End If
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            If hp <= 0 Then
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2.0F, mr.Y, 0.1F, 0.0F, 0.0F, 0.05F, 200))
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2.0F, mr.Y, -0.1F, 0.0F, 0.0F, 0.05F, 200))
            End If
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            standby = True
            Return True
        Else
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            If hp <= 0 Then
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2.0F, mr.Y, 0.1F, 0.0F, 0.0F, 0.05F, 200))
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2.0F, mr.Y, -0.1F, 0.0F, 0.0F, 0.05F, 200))
            End If
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 48
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return ir.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move

        If standby Then
            If R.X - mr.X < XAZ AndAlso mr.X - R.X < XAZ Then
                standby = False
                internalcounter = 0
            End If
        Else
            internalcounter += 1
            If internalcounter >= 128 Then
                If internalcounter Mod 64 = 0 Then
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2.0F, mr.Y, ((internalcounter Mod 5) - 2) * 2.0F, 0.0F, 0.0F, 0.05F, 256))
                End If
            End If
            If R.X - mr.X > XAZ AndAlso mr.X - R.X > XAZ Then
                standby = True
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return ir
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Jeefly
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private screwed, destroy, kill As Boolean

    'Specific
    Private spd As Single
    Private range As Integer
    Private rangectr As Single
    Private shottimer As Integer

    Const MYWIDTH As Integer = 32
    Const MYHEIGHT As Integer = 32

    Public Sub New(ByVal Xo As Integer, ByVal Yo As Integer, ByVal HoverRange As Integer, ByVal Speed As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        range = HoverRange
        spd = Speed
        goingleft = False
        internalcounter = 0
        rangectr = range / spd
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Right < 0 OrElse mr.Top > 800 OrElse kill
    End Function


    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dX As Single, dY As Single, ang As Double
        Dim wall As RectangleF
        If goingleft Then
            mr.Offset(-spd, 0)
        Else
            mr.Offset(spd, 0)
        End If
        If screwed Then
            mr.Offset(0.0F, 1.0F)
        End If
        rangectr -= 1
        If rangectr <= 0 Then
            goingleft = Not goingleft
            rangectr = range / spd
        End If
        Me.internalcounter += 1
        If Me.internalcounter >= 8 Then
            Me.internalcounter = 0
        End If
        shottimer += 1
        If destroy Then
            mr.Offset(0.0F, shottimer Mod 5)
            If shottimer = 80 Then
                Q.AddExplosion(mr.X, mr.Y)
            End If
            For Each wall In Q.Walls
                If wall.IntersectsWith(mr) Then
                    kill = True
                    Q.AddExplosion(mr.X, mr.Y)
                End If
            Next
            For Each wall In Q.Extrawalls
                If wall.IntersectsWith(mr) Then
                    kill = True
                    Q.AddExplosion(mr.X, mr.Y)
                End If
            Next
            For Each wall In Q.Groundwalls
                If wall.IntersectsWith(mr) Then
                    kill = True
                    Q.AddExplosion(mr.X, mr.Y)
                End If
            Next
            For Each wall In Q.Crates
                If wall.IntersectsWith(mr) Then
                    kill = True
                    Q.AddExplosion(mr.X, mr.Y)
                End If
            Next
            For Each wall In Q.Glasses
                If wall.IntersectsWith(mr) Then
                    kill = True
                    Q.AddExplosion(mr.X, mr.Y)
                End If
            Next
            For Each wall In Q.CrossStays
                If wall.IntersectsWith(mr) Then
                    kill = True
                    Q.AddExplosion(mr.X, mr.Y)
                End If
            Next
        Else
            If shottimer = 160 Then
                dX = R.X - mr.X
                dY = R.Y - mr.Y
                ang = Math.Atan2(dY, dX)
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH \ 2, mr.Y + MYHEIGHT \ 2, CSng(3 * Math.Cos(ang)), CSng(3 * Math.Sin(ang)), 0, 0, 160))
            ElseIf shottimer = 200 Then
                dX = R.X - mr.X
                dY = R.Y - mr.Y
                ang = Math.Atan2(dY, dX)
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH \ 2, mr.Y + MYHEIGHT \ 2, CSng(3 * Math.Cos(ang - 0.25)), CSng(3 * Math.Sin(ang - 0.25)), 0, 0, 160))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH \ 2, mr.Y + MYHEIGHT \ 2, CSng(3 * Math.Cos(ang + 0.25)), CSng(3 * Math.Sin(ang + 0.25)), 0, 0, 160))
            ElseIf shottimer = 320 Then
                shottimer = 0
            End If
        End If

    End Sub


    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Integer
        F = Me.internalcounter >> 1
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 18
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Screwer Then
            screwed = True
            shottimer = 700
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Return True
        Else
            destroy = True
            Return True
        End If
    End Function
End Class
Public Class Pelogic
    Implements IMonster
    Private mr As RectangleF
    Private Atkrange As Single
    Private internalcounter As Integer
    Private screwed As Boolean
    Private hp As Integer

    Private Const MYWIDTH As Single = 32.0F
    Private Const MYHEIGHT As Single = 32.0F

    Public Sub New(ByVal X As Single, ByVal Y As Single, ByVal Range As Single)
        mr = New RectangleF(X, Y, MYWIDTH, MYHEIGHT)
        Atkrange = Range
        hp = 10
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim Ho, Hf As Single
        Ho = ((Me.internalcounter \ 20) Mod 4) / 4.0F
        Hf = Ho + 0.25F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, Ho, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, Ho, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, Hf, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, Hf, 1.0F)


    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 25
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If Me.internalcounter >= 80 Then
            Me.internalcounter = 0
            If mr.Y < R.Y Then
                If mr.X > R.X - Atkrange AndAlso mr.X < R.X + Atkrange Then
                    If screwed Then
                        Q.Monsters.Add(New Bullet(mr.X + 8, mr.Y + 16, (mr.X - R.X) / (Convert.ToSingle(Math.Sqrt(0.25 + 2 * (R.Y - mr.Y))) - 0.5F), 0, 0, 1, 100))
                    Else
                        Q.Monsters.Add(New Bullet(mr.X + 8, mr.Y + 16, (R.X - mr.X) / (Convert.ToSingle(Math.Sqrt(0.25 + 2 * (R.Y - mr.Y))) - 0.5F), 0, 0, 1, 100))
                    End If
                End If
            End If
        End If
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Screwer Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            screwed = True
            Return True
        Else
            If Me.internalcounter Mod 40 < 20 Then
                If ISpd < 0 Then
                    Q.SonicEvents.Add(SonicEventType.ShotDefend)
                    Return True
                Else
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                End If
            Else
                If ISpd > 0 Then
                    Q.SonicEvents.Add(SonicEventType.ShotDefend)
                    Return True
                Else
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                End If
            End If
        End If
    End Function
End Class
Public Class Ejectent
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private spd As Single
    Private hp, frame As Integer

    Private jumping As Boolean, onslope As Boolean
    Private fallspeed As Single, initheight As Single

    Const MYWIDTH As Single = 17
    Const MYHEIGHT As Single = 23
    Const STANDHEIGHT As Single = 18
    Const DIFFERENCE As Single = MYHEIGHT - STANDHEIGHT

    Const GRAVITY As Single = 1, GRAVITYTH As Single = GRAVITY * 1.5F, INITVELOC As Single = -16
    Const TIMEOUT As Integer = 32
    Const TERMINALVELOCITY As Single = 20.0F

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        Me.spd = 0
        goingleft = False
        hp = 2
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Bottom >= 800 OrElse hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If frame = 1 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf frame = 3 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf frame = 0 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 54
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If jumping Then
            frame = 0
            If goingleft Then
                mr.Offset(-spd, 0)
                For Each Wall As RectangleF In Q.Walls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Crates
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Glasses
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.CrossStays
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.RTDs
                    'Check with each wall.
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.Pneumos
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(sw.Gate.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
            Else
                mr.Offset(spd, 0)
                For Each Wall As RectangleF In Q.Walls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Crates
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Glasses
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.CrossStays
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.RTDs
                    'Check with each wall.
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.Pneumos
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(sw.Gate.Left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
            End If
            If spd > 0 Then
                spd -= 0.0625F
            End If

            mr.Offset(0, fallspeed)
            fallspeed += GRAVITY
            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, Platform.Top - mr.Bottom)
                        jumping = False

                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Walls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.EyeBeams
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.CrossStays
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Groundwalls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, wall.Top - mr.Bottom)
                        jumping = False
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Glasses
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, wall.Top - mr.Bottom)
                        jumping = False
                    End If
                End If
            Next
            For Each sw As OpenerSwitch In Q.RTDs
                'Looking at each platform in the platform array.
                If mr.Bottom >= sw.Gate.Top AndAlso mr.Top <= sw.Gate.Bottom Then
                    If mr.Left < sw.Gate.Right AndAlso mr.Right > sw.Gate.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, sw.Gate.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, sw.Gate.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each sw As OpenerSwitch In Q.Pneumos
                'Looking at each platform in the platform array.
                If mr.Bottom >= sw.Gate.Top AndAlso mr.Top <= sw.Gate.Bottom Then
                    If mr.Left < sw.Gate.Right AndAlso mr.Right > sw.Gate.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, sw.Gate.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, sw.Gate.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
        Else
            Me.internalcounter += 1
            If Me.internalcounter >= TIMEOUT Then
                jumping = True
                fallspeed = INITVELOC
                initheight = mr.Bottom
                Me.internalcounter = 0
                If Me.internalcounter >= TIMEOUT * 7 \ 8 Then
                    frame = 3
                ElseIf Me.internalcounter >= TIMEOUT * 3 \ 4 Then
                    frame = 2
                ElseIf Me.internalcounter >= TIMEOUT \ 2 Then
                    frame = 1
                Else
                    frame = 0
                End If
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        spd += ISpd
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class ShieldBot
    Implements IMonster


    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter, shotctr As Integer
    Private goingleft, intimeout As Boolean
    Private calibrated As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 32, MYHEIGHT As Integer = 32
    Const NUMFRAMES As Integer = 32, TIMEOUT As Integer = 60, SHOTTIMEOUT As Integer = 100

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 3
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYWIDTH, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 3
        Me.internalcounter = NUMFRAMES - 1
    End Sub


    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Single = (Me.internalcounter >> 2) * 0.25F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.25F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.25F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.25F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If goingleft AndAlso ISpd <= 0.0F Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf Not goingleft AndAlso ISpd >= 0.0F Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 56
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Function CalibrateWalkingRange(ByVal Q As LevelSuite, ByVal Surf As Single, ByVal LEnd As Single, ByVal REnd As Single) As RectangleF
        Dim range, W As RectangleF

        range = RectangleF.FromLTRB(LEnd, Surf, REnd, Surf * 2.0F)

        For Each W In Q.Walls
            If Surf = W.Top Then
                If W.Right = range.Left Then   'touches surface on the left
                    range = RectangleF.FromLTRB(W.Left, range.Top, range.Right, range.Bottom)
                ElseIf W.Left = range.Right Then   'touches surface on the right
                    range = RectangleF.FromLTRB(range.Left, range.Top, W.Right, range.Bottom)
                End If
            Else
                If W.Top < Surf AndAlso W.Bottom > Surf Then
                    If W.Left > range.Left AndAlso W.Left < range.Right Then  'clips part of the right
                        range = RectangleF.FromLTRB(range.Left, range.Top, W.Left, range.Bottom)
                    End If
                    If W.Right > range.Left AndAlso W.Right < range.Right Then   'clips part of the left
                        range = RectangleF.FromLTRB(W.Right, range.Top, range.Right, range.Bottom)
                    End If
                End If
            End If
        Next
        Return range
    End Function
    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim ret As RectangleF
        If calibrated Then
            If intimeout Then
                internalcounter += 1
                If internalcounter >= TIMEOUT Then
                    intimeout = False
                End If
            Else
                If goingleft Then
                    mr.Offset(-spd, 0)
                    If mr.Left <= le Then
                        goingleft = False
                        intimeout = True
                        internalcounter = 0
                    End If
                Else
                    mr.Offset(spd, 0)
                    If mr.Right >= re Then
                        goingleft = True
                        intimeout = True
                        internalcounter = 0
                    End If
                End If

                Me.internalcounter += 1
                If Me.internalcounter >= NUMFRAMES Then
                    Me.internalcounter = 0
                End If
            End If
            shotctr += 1
            If shotctr = SHOTTIMEOUT Then
                shotctr = 0
                Q.Monsters.Add(New Bullet(mr.X + 16.0F, mr.Y, -1.5F, -6.0F, 0.0F, 0.25F, 250))
                Q.Monsters.Add(New Bullet(mr.X + 16.0F, mr.Y, 1.5F, -6.0F, 0.0F, 0.25F, 250))
            End If
        Else
            ret = CalibrateWalkingRange(Q, mr.Bottom, le, re)
            If ret.Left = le AndAlso ret.Right = re Then
                calibrated = True
            End If
            le = ret.Left
            re = ret.Right
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class PennyRifle
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private screwed As Boolean
    Private internalcounter As Integer

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 3
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            screwed = True
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ShockWave OrElse BY = ShotTypes.Plasma Then
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        ElseIf BY = ShotTypes.Rubberet Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 58
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If screwed Then
            internalcounter = 0
            Q.Monsters.Add(New Napalm(mr.X + 16, mr.Y + 16, 0.0F, -5.0F, 0.0F, 0.0F, 90))
            Q.Monsters.Add(New Napalm(mr.X + 16, mr.Y + 16, 0.0F, 5.0F, 0.0F, 0.0F, 90))
            screwed = False
        End If
        If internalcounter = 100 Then
            Q.Monsters.Add(New CustomShot(mr.X + 16, mr.Y + 16, -2.0F, 0.0F, 0.0F, 0.0F, 32, 0, New Bullet(0.0F, 0.0F, 0.0F, 4.0F, 0.0F, 0.0F, 64)))
            Q.Monsters.Add(New CustomShot(mr.X + 16, mr.Y + 16, 2.0F, 0.0F, 0.0F, 0.0F, 32, 0, New Bullet(0.0F, 0.0F, 0.0F, 4.0F, 0.0F, 0.0F, 64)))
            Q.Monsters.Add(New CustomShot(mr.X + 16, mr.Y + 16, 0.0F, 2.0F, 0.0F, 0.0F, 32, 0, New Bullet(0.0F, 0.0F, 0.0F, 4.0F, 0.0F, 0.0F, 64)))
            internalcounter = 0
        Else
            internalcounter += 1
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class ShieldShock
    Implements IMonster
    Private mr As RectangleF
    Private goingleft As Boolean
    Private hp As Integer
    Private internalcounter As Integer
    Private screwed As Boolean

    Private operating As Boolean

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const OPERATIONTIME As Integer = 256
    Const DELAYTIME As Integer = 64
    Const XAZ As Single = 120.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Facingleft As Boolean)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        goingleft = Facingleft
        hp = 5
        operating = False
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim T, B As Single
        If goingleft Then
            T = 0.0F
            B = 0.5F
        Else
            T = 0.5F
            B = 1.0F
        End If
        If operating Then
            If internalcounter = 1 OrElse internalcounter = OPERATIONTIME - 1 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, T)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, B)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, T)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, B)
            ElseIf internalcounter = 2 OrElse internalcounter = OPERATIONTIME - 2 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, T)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, B)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, T)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, B)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, T)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, B)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, T)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, B)
            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, T)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, B)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, T)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, B)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If operating Then
            If BY = ShotTypes.Screwer Then
                screwed = True
                Q.SonicEvents.Add(SonicEventType.ShotRavage)
                Return True
            Else
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            End If
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 59
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If Not operating Then
            If internalcounter >= DELAYTIME Then
                If mr.X - R.X < XAZ AndAlso R.X - mr.X < XAZ Then
                    operating = True
                    internalcounter = 0
                End If
            End If
            internalcounter += 1
        Else
            If internalcounter Mod 128 = 0 AndAlso internalcounter <> 0 Then
                If Not screwed Then
                    If goingleft Then
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -6, -2, 0, 0, 50))
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -6, 2, 0, 0, 50))
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -2, -2, 0, 0, 150))
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -2, 2, 0, 0, 150))
                    Else
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 6, -2, 0, 0, 50))
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 6, 2, 0, 0, 50))
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 2, -2, 0, 0, 150))
                        Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 2, 2, 0, 0, 150))
                    End If
                End If
            End If
            internalcounter += 1
            If internalcounter >= OPERATIONTIME Then
                operating = False
                internalcounter = 0
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Canonical
    Implements IMonster

    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private spd As Single
    Private shooting As Boolean
    Private hp As Integer

    Const MYWIDTH As Integer = 28, MYHEIGHT As Integer = 22
    Const TIMEOUT As Integer = 80
    Const RESTTIMEOUT As Integer = 20

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        goingleft = MovingLeft
        spd = Speed
        hp = 10
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim Ho, Hf As Single
        If goingleft Then
            Ho = 0.0F
        Else
            Ho = 0.5F
        End If
        If shooting Then
            If internalcounter > (TIMEOUT >> 2) Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.333F + Ho, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F + Ho, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F + Ho, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F + Ho, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.167F + Ho, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.167F + Ho, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.333F + Ho, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F + Ho, 1.0F)

            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F + Ho, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F + Ho, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.167F + Ho, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.167F + Ho, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 16
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If shooting Then
            If Me.internalcounter >= TIMEOUT Then
                shooting = False
                Me.internalcounter = 0
            End If
            goingleft = (mr.X > R.X)
        Else
            If Me.internalcounter >= RESTTIMEOUT Then
                shooting = True
                Me.internalcounter = 0
                If goingleft Then
                    Q.Monsters.Add(New Corkwheel(mr.Left, mr.Top, -spd, 2, 0.0F, 0.5F, 200))
                Else
                    Q.Monsters.Add(New Corkwheel(mr.Right, mr.Top, spd, 2, 0.0F, 0.5F, 200))
                End If
            End If
        End If
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.ShockWave OrElse BY = ShotTypes.Plasma Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Aquabullet OrElse BY = ShotTypes.Stonebullet Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Rubberet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            hp -= 50
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ForceMod OrElse BY = ShotTypes.Corkwheel Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True

        End If
    End Function
End Class
Public Class DarkFly
    Implements IMonster
    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter, tick As Integer
    Private attacking As Boolean
    Private overhead As Boolean
    Private done As Boolean
    Private screwed As Boolean
    Private hp As Integer
    'Specific
    Private spd As Single
    Private range As Integer
    Private rangectr As Single
    Private shottimer As Integer

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 150.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 3
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse mr.X < 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Integer
        F = Me.internalcounter >> 1
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.Aquabullet
                hp -= 2
            Case Else
                hp -= 1
        End Select
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 62
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If done Then
            mr.Offset(-2.0F, 0)
        ElseIf overhead Then
            If tick >= 0 AndAlso tick <= 48 AndAlso tick Mod 6 = 0 Then
                Q.Monsters.Add(New DirectBullet(mr.X + 16.0F, mr.Y + 16, R, 3, 100))
            ElseIf tick = 100 Then
                Q.Monsters.Add(New RAShot(mr.X + 16.0F, mr.Y + 16.0F, -4, 0, 0, 0, 225))
                Q.Monsters.Add(New RAShot(mr.X + 16.0F, mr.Y + 16.0F, 4, 0, 0, 0, 225))
                done = True
            End If
            tick += 1
        ElseIf attacking Then
            mr.Offset(-2.0F, 0)
            If mr.X - R.X < 16.0F Then
                overhead = True
            End If
        Else
            If mr.X - R.X < XAZ AndAlso R.X - mr.X < XAZ Then
                attacking = True
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class FreeModular
    Implements IMonster
    Private mr As RectangleF
    Private goleft As Boolean
    Private hp As Integer
    Private internalcounter As Integer
    Private ox, oy As Single

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 300.0F
    Const YAZ As Single = 300.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal FaceLeft As Boolean)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        ox = Xo
        oy = Yo
        goleft = FaceLeft
        hp = 1
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse mr.Left <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        F = (internalcounter \ 2) Mod 4
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFF00, 0.25F * F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * F, 0.5F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 0.5F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 40
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If mr.X - R.X < XAZ Then
            If mr.Y - R.Y < YAZ AndAlso R.Y - mr.Y < YAZ Then
                internalcounter += 1
                mr.Offset(-3.5F, 0.0F)
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class TurnModular
    Implements IMonster
    Private mr As RectangleF
    Private goleft As Boolean
    Private hp As Integer
    Private internalcounter As Integer
    Private ox, oy, vx As Single
    Private slowing As Boolean

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 300.0F
    Const YAZ As Single = 300.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal FaceLeft As Boolean)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        ox = Xo
        oy = Yo
        vx = -3.5F
        goleft = True
        hp = 1
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse mr.Left <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        F = (internalcounter \ 2) Mod 4
        If goleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFF00FF, 0.25F * F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFF00FF, 0.25F * F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFF00FF, 0.25F * (F + 1), 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFF00FF, 0.25F * (F + 1), 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFF00FF, 0.25F * (F + 1), 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFF00FF, 0.25F * (F + 1), 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFF00FF, 0.25F * F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFF00FF, 0.25F * F, 0.5F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 40
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If slowing Then
            If vx >= 3.5F Then
                slowing = False
            End If
            vx += 0.0625F
            If vx > 0 Then
                goleft = False
            End If
        End If
        If mr.X - R.X < XAZ Then
            If mr.Y - R.Y < YAZ AndAlso R.Y - mr.Y < YAZ Then
                internalcounter += 1
                mr.Offset(vx, 0.0F)

                If R.X - mr.X > 150.0F Then
                    slowing = True
                End If
            End If
        ElseIf vx >= 3.5F Then
            hp = 0
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SweepModular
    Implements IMonster
    Private mr As RectangleF
    Private goleft As Boolean
    Private hp As Integer
    Private internalcounter As Integer
    Private ox, oy, vx As Single
    Private slowing As Boolean

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 300.0F
    Const YAZ As Single = 300.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal FaceLeft As Boolean)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        ox = Xo
        oy = Yo
        vx = -3.5F
        goleft = True
        hp = 1
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse mr.Left <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        F = (internalcounter \ 2) Mod 4
        If goleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFF00FFFF, 0.25F * F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFF00FFFF, 0.25F * F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFF00FFFF, 0.25F * (F + 1), 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFF00FFFF, 0.25F * (F + 1), 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFF00FFFF, 0.25F * (F + 1), 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFF00FFFF, 0.25F * (F + 1), 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFF00FFFF, 0.25F * F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFF00FFFF, 0.25F * F, 0.5F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 40
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If slowing Then
            If vx >= 3.5F Then
                slowing = False
            End If
            vx += 0.0625F
            If vx >= 0 Then
                goleft = False
            End If
        End If
        If mr.X - R.X < XAZ Then
            If mr.Y - R.Y < YAZ AndAlso R.Y - mr.Y < YAZ Then
                internalcounter += 1
                mr.Offset(vx, 0.0F)
                If vx > -3.5F Then
                    If R.Y > mr.Y Then
                        mr.Offset(0.0F, 1.0F)
                    Else
                        mr.Offset(0.0F, -1.0F)
                    End If
                End If
                If R.X - mr.X > 150.0F Then
                    slowing = True
                End If
            End If
        ElseIf vx >= 3.5F Then
            hp = 0
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SwingModule
    Implements IMonster

    Private mr As RectangleF
    Private state As BatStates
    Private hp As Integer
    Private internalcounter, outctr As Integer
    Private goingleft As Boolean
    Private swoopheight As Single

    Private Enum BatStates
        Perched = 0
        Descending
        Swooping
        Flying
        'In order of intended occurrence
    End Enum

    Const PERCHWIDTH As Integer = 32
    Const FLYWIDTH As Integer = 32
    Const FLYHEIGHT As Integer = 32
    Const ATKRANGE As Integer = 200

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, PERCHWIDTH, FLYHEIGHT)
        state = BatStates.Perched
        swoopheight = 128.0F
        hp = 3

    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse outctr >= 20
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        F = (internalcounter \ 2) Mod 4
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFF00, 0.25F * F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * F, 0.5F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 0.5F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.Aquabullet
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ForceMod
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Napalm, ShotTypes.Stonebullet
                hp -= 3
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Rubberet
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            Case ShotTypes.Screwer
                hp -= 3
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ShockWave, ShotTypes.Plasma
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
        End Select
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return FLYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 40
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Static dx, dy, dt As Single
        Select Case state
            Case BatStates.Perched
                If R.Right >= mr.Left - ATKRANGE AndAlso R.Left <= mr.Right + ATKRANGE Then
                    goingleft = (R.Left < mr.Left)
                    state = BatStates.Descending
                    mr.Width = FLYWIDTH
                    Me.internalcounter += 1
                End If
            Case BatStates.Descending
                goingleft = (R.Left < mr.Left)
                mr.Y += 8
                If R.Top < mr.Bottom + swoopheight Then
                    state = BatStates.Swooping
                    'System.Diagnostics.Debug.WriteLine("Swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    dy = 3
                End If
                Me.internalcounter += 1
            Case BatStates.Swooping
                dt += 0.25F
                If dt >= 2.0F Then
                    dt = 0.0F
                    If goingleft Then
                        dx -= 0.5F
                    Else
                        dx += 0.5F
                    End If
                    dy -= 0.25F
                    If dy <= 0.0F Then
                        state = BatStates.Flying
                        'System.Diagnostics.Debug.WriteLine("Not-swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    End If
                End If
                mr.Offset(dx, dy)
                Me.internalcounter += 1
            Case BatStates.Flying
                If goingleft Then
                    mr.Offset(-3.0F, 0)
                Else
                    mr.Offset(3.0F, 0)
                End If
                Me.internalcounter += 1
        End Select
        If Me.internalcounter >= 6 Then
            Me.internalcounter = 0
            outctr += 1
            If outctr Mod 5 = 0 Then
                Q.Monsters.Add(New DirectAcceleratorBullet(mr.X, mr.Y, R, 0.1F, 250))
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return FLYWIDTH
        End Get
    End Property
End Class
Public Class Trebuchet
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private shooting As Boolean
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 26
    Const TIMEOUT As Integer = 80
    Const RESTTIMEOUT As Integer = 20
    Const SHOTDX As Single = 5
    Const XAZ As Single = 300.0F
    Const YAZ As Single = 150.0F

    Public Sub New(ByVal BaseRect As RectangleF, ByVal LeftOffset As Single)
        mr = New RectangleF(BaseRect.X + LeftOffset, BaseRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        hp = 2
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If shooting Then
            If internalcounter > (TIMEOUT >> 2) Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.667F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.667F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.333F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.667F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.667F, 1.0F)

            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.333F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 74
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx, dy, ruts As Single
        Const VY As Single = -10.0F
        Const Ac As Single = 0.5F
        Const HIGH As Single = VY * VY / Ac / 2
        If shooting Then
            If Me.internalcounter >= TIMEOUT Then
                shooting = False
                Me.internalcounter = 0
            End If
        Else
            If Me.internalcounter >= RESTTIMEOUT Then
                shooting = True
                Me.internalcounter = 0
                'Sum = (H + L) * ((H - L) / 2D), H and L are speeds, D is acceleration, Sum is distance.
                If R.Left - mr.Left < XAZ AndAlso mr.Left - R.Left < XAZ Then
                    If R.Top - mr.Top < YAZ AndAlso mr.Top - R.Top - 16.0F < HIGH Then
                        'dx = (R.X - mr.X)
                        dy = (R.Y - mr.Y + 16)

                        'Solve for rs in the following eqn:
                        'dy = (dx - rs) * (dx) : dy / dx = dx - rs : rs = dx - (dy / dx)
                        'Or setting vy = -3:
                        'time to reach dy : dy = (R + vy) * (R - vy) / 2A, with R = A * T+vy
                        ' dy = (A * T + 2 * vy) * (A * T) / 2A = (A * T * T / 2) + (vy *  T) : (A/2)a + vyb - dyc
                        'no solution if -2 * dy * A > vy * vy : b = 2 * vy / A, c = -2 * dy / A 
                        'ruts = (-2* vy/ A) & sqr(4 * vyvy / AA + 8 *dy / A) / 2 : (40 & sqr(100 + 16dy)
                        'positive number added to pos or neg number: use positive
                        'dx = (dist / ruts)
                        ruts = Convert.ToSingle((-2 * VY / Ac) + Math.Sqrt(4 * VY * VY / Ac / Ac + 8 * dy / Ac)) / 2.0F
                        dx = (R.X - mr.X) / ruts  'Division by zero protection with constant HIGH.
                        Q.Monsters.Add(New Corkwheel(mr.Left + SHOTDX, mr.Top - 16.0F, dx, VY, 0.0F, Ac, 400))
                        'Q.Monsters.Add(New Bullet(mr.Left + SHOTDX, mr.Top, 0, -3, 0, 0, 80))

                    End If
                End If
            End If
        End If
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            hp += 3
            shooting = False
            internalcounter = RESTTIMEOUT
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Return True
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Plasma Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function

End Class
Public Class StrafisTrack
    Implements IMonster
    Private mr As RectangleF
    Private frame, internalcounter As Integer

    Const MYWIDTH As Single = 24.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Height As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, Height)
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If frame = 0 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return mr.Height
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 79
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        internalcounter += 1
        If internalcounter >= 8 Then
            frame = 1 - frame
            internalcounter = 0
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Strafis
    Implements IMonster
    Private mr As RectangleF
    Private spd As Single
    Private internalcounter, frame As Integer
    Private goingup As Boolean
    Private te, be As Single
    Private hp As Integer

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Height As Single, ByVal Speed As Single)
        te = Yo
        be = Yo + Height
        spd = Speed
        If spd <= 0 Then
            spd = 1
        End If
        mr = New RectangleF(Xo, Yo + Speed, MYWIDTH, MYHEIGHT)
        hp = 9
        goingup = False
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If frame = 0 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.Aquabullet
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            Case ShotTypes.ShockWave, ShotTypes.Napalm, ShotTypes.Rubberet, ShotTypes.Stonebullet
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Screwer
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ForceMod, ShotTypes.Corkwheel
                hp -= 3
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
        End Select
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 79
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingup Then
            mr.Offset(0, -spd - 2)
            If mr.Top < te Then
                goingup = False
            End If
        Else
            mr.Offset(0, spd + 2)
            If mr.Bottom > be Then
                goingup = True
            End If
        End If
        If mr.Bottom - R.Bottom < 1.0F AndAlso R.Bottom - mr.Bottom < 1.0F Then
            If mr.Left > R.Left Then
                Q.Monsters.Add(New Maser(mr.X - 16, mr.Y + 8, -4, 300))
            Else
                Q.Monsters.Add(New Maser(mr.X - 16, mr.Y + 8, 4, 300))
            End If
        End If
        internalcounter += 1
        If internalcounter >= 10 Then
            internalcounter = 0
            frame = 1 - frame
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Cannone
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private shooting As Boolean
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 26
    Const TIMEOUT As Integer = 80
    Const RESTTIMEOUT As Integer = 20
    Const SHOTDX As Single = 5
    Const XAZ As Single = 300.0F
    Const YAZ As Single = 150.0F

    Public Sub New(ByVal BaseRect As RectangleF, ByVal LeftOffset As Single)
        mr = New RectangleF(BaseRect.X + LeftOffset, BaseRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        hp = 2
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If shooting Then
            If internalcounter > (TIMEOUT >> 2) Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.667F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.667F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.333F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.667F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.667F, 1.0F)

            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.333F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.333F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 74
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx, dy, ruts As Single
        Const VY As Single = -10.0F
        Const Ac As Single = 0.5F
        Const HIGH As Single = VY * VY / Ac / 2
        If shooting Then
            If Me.internalcounter >= TIMEOUT Then
                shooting = False
                Me.internalcounter = 0
            End If
        Else
            If Me.internalcounter >= RESTTIMEOUT Then
                shooting = True
                Me.internalcounter = 0
                'Sum = (H + L) * ((H - L) / 2D), H and L are speeds, D is acceleration, Sum is distance.
                If R.Left - mr.Left < XAZ AndAlso mr.Left - R.Left < XAZ Then
                    If R.Top - mr.Top < YAZ AndAlso mr.Top - R.Top - 16.0F < HIGH Then
                        'dx = (R.X - mr.X)
                        dy = (R.Y - mr.Y + 16)

                        'Solve for rs in the following eqn:
                        'dy = (dx - rs) * (dx) : dy / dx = dx - rs : rs = dx - (dy / dx)
                        'Or setting vy = -3:
                        'time to reach dy : dy = (R + vy) * (R - vy) / 2A, with R = A * T+vy
                        ' dy = (A * T + 2 * vy) * (A * T) / 2A = (A * T * T / 2) + (vy *  T) : (A/2)a + vyb - dyc
                        'no solution if -2 * dy * A > vy * vy : b = 2 * vy / A, c = -2 * dy / A 
                        'ruts = (-2* vy/ A) & sqr(4 * vyvy / AA + 8 *dy / A) / 2 : (40 & sqr(100 + 16dy)
                        'positive number added to pos or neg number: use positive
                        'dx = (dist / ruts)
                        ruts = Convert.ToSingle((-2 * VY / Ac) + Math.Sqrt(4 * VY * VY / Ac / Ac + 8 * dy / Ac)) / 2.0F
                        dx = (R.X - mr.X) / ruts  'Division by zero protection with constant HIGH.
                        Q.Monsters.Add(New Napalm(mr.Left + SHOTDX, mr.Top - 16.0F, dx, VY, 0.0F, Ac, 400))
                        'Q.Monsters.Add(New Bullet(mr.Left + SHOTDX, mr.Top, 0, -3, 0, 0, 80))

                    End If
                End If
            End If
        End If
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            hp += 3
            shooting = False
            internalcounter = RESTTIMEOUT
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Return True
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Plasma Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function


End Class
Public Class WallMine
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private counter As Integer

    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 24.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 1
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        If counter >= 48 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        If counter < 48 Then
            Q.AddExplosion(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F)
            Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -12.8F, -12.8F, 0.0F, 0.0F, 5))
            Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 4.0F, -4.0F, 0.0F, 0.0F, 15))
            Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 3.0F, 3.0F, 0.0F, 0.0F, 25))
            Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -2.0F, 2.0F, 0.0F, 0.0F, 30))
        End If
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 89
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dradius, vx, vy As Single
        Dim A, B, C As Microsoft.DirectX.Vector2
        If counter = 0 Then
            A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
            B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
            C = Microsoft.DirectX.Vector2.Subtract(A, B)
            If C.LengthSq < 9000 Then
                Q.AddExplosion(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F)
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -12.8F, -12.8F, 0.0F, 0.0F, 5))
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 4.0F, -4.0F, 0.0F, 0.0F, 15))
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 3.0F, 3.0F, 0.0F, 0.0F, 25))
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -2.0F, 2.0F, 0.0F, 0.0F, 30))
            End If
        End If
        counter += 1
        If counter >= 64 Then
            counter = 0
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Pyromist
    Implements IMonster
    Private mr As RectangleF
    Private hp, internalcounter As Integer
    Private screwed As Boolean
    Private RI As Random

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 1600.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal RNG As Random)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 5
        RI = RNG
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Integer
        F = (Me.internalcounter >> 3) Mod 4
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.ShockWave Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Aquabullet Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Rubberet Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Corkwheel Then
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        ElseIf BY = ShotTypes.Screwer Then
            hp -= 1
            screwed = True
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 96
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If mr.Left - R.Left < XAZ AndAlso R.Left - mr.Left < XAZ Then
            internalcounter += 1
            If internalcounter = 128 Then
                internalcounter = 0
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
                Q.Monsters.Add(New CustomShot(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0, -4, 0, 0, 100, 1, New Bullet(0, 0, (RI.Next(41) - 20) * 0.25F, (RI.Next(41) - 20) * 0.25F, 0, 0.3F, 256)))
            End If
        End If
    End Sub
    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class RemoUzi
    Implements IMonster

    Private mr As RectangleF
    Private hp, internalcounter As Integer
    Private spd, lastx As Single
    Private xaz As Single
    Private inposition, deploying, attacking As Boolean

    Const MYWIDTH As Single = 22.0F
    Const MYHEIGHT As Single = 32.0F
    Const POSTIMER As Integer = 150

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Speed As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 2
        spd = Speed
        xaz = 60
        inposition = False
        deploying = False
        attacking = False
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Integer
        F = (Me.internalcounter >> 2) Mod 4
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 98
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx As Single = lastx - R.X
        If deploying Then
            mr.Offset(-spd, spd)
            internalcounter += 1
            If internalcounter >= POSTIMER * 2.0F / spd Then
                attacking = True
                deploying = False
                internalcounter = 0
            End If
        ElseIf attacking Then
            internalcounter += 1
            If internalcounter Mod 14 = 0 Then
                Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, -spd, 0, 0, 0, 300))
                Q.SonicEvents.Add(SonicEventType.Shoot)
            ElseIf internalcounter Mod 14 = 7 Then
                Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, -spd, 0.6F, 0, 0, 300))
                Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, -spd, -0.6F, 0, 0, 300))
                Q.SonicEvents.Add(SonicEventType.Shoot)
            ElseIf internalcounter = 100 Then
                hp = 0
                Q.AddExplosion(mr.X, mr.Y)
            End If
        ElseIf inposition Then
            dx = R.X - lastx
            xaz = 60 * spd + 80
            If mr.X - R.X < xaz Then
                deploying = True
                internalcounter = 0
            End If
        Else
            mr.Offset(2.0F, -2.0F)
            internalcounter += 1
            If internalcounter >= POSTIMER Then
                inposition = True
                lastx = R.X
            End If

        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class EmoUzi
    Implements IMonster
    Private mr As RectangleF
    Private hp, internalcounter As Integer
    Private spd As Single
    Private fleet As Boolean

    Const MYWIDTH As Single = 22.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 150

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Speed As Single)
        mr = New RectangleF(Xo, Yo, mywidth, myheight)
        hp = 2
        spd = Speed
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Integer
        F = (Me.internalcounter >> 2) Mod 4
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select


    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Plasma Then
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        Else
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 99
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If fleet Then
            mr.Offset(spd, -1.0F)
            internalcounter += 1
            If internalcounter >= 20 AndAlso internalcounter <= 25 Then
                Q.Monsters.Add(New DirectBullet(mr.X, mr.Y, R, (internalcounter - 15) * 0.5F, 500))
            ElseIf internalcounter = 26 Then
                Q.Monsters.Add(New HomingShot(mr.X, mr.Y, spd, 0.02F * spd, 100))
                hp = 0
            End If
        ElseIf mr.X - R.X < XAZ OrElse hp = 1 Then
            fleet = True
            Q.AddNegation(mr.X, mr.Y)

        End If
    End Sub


    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class AssaultCamera
    Implements IMonster
    Implements IOpticalTool

    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private attacking As Boolean, atktimer As Integer
    Private screwed, killed As Boolean

    Private spd As Single
    Private le, re As Single
    Private orangle As Single

    Private xmultiplier As Single, ymultiplier As Single, auxadd As Single

    Private xorient As Single, yorient As Single

    Const MYWIDTH As Integer = 32
    Const MYHEIGHT As Integer = 32
    Const HALFWIDTH As Integer = MYWIDTH >> 1
    Const HALFHEIGHT As Integer = MYHEIGHT >> 1

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Platform As RectangleF, ByVal Spd As Single, ByVal Angle As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        Me.spd = Spd / 4
        le = Platform.Left
        re = Platform.Right
        orangle = Angle
        xmultiplier = Convert.ToSingle(Math.Sin(orangle * Math.PI / 180))
        ymultiplier = Convert.ToSingle(Math.Cos(orangle * Math.PI / 180))

        xorient = Convert.ToSingle(Math.Sqrt(2) * Math.Sin((-Angle - 45) * Math.PI / 180))
        yorient = Convert.ToSingle(Math.Sqrt(2) * Math.Cos((-Angle - 45) * Math.PI / 180))

    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return killed
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer
        Const N As Single = 1.0F
        F = internalcounter \ 12
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(1 + xorient, 1 - yorient, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(1 - yorient, 1 - xorient, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(1 + yorient, 1 + xorient, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(1 - xorient, 1 + yorient, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(1 + xorient, 1 - yorient, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(1 - yorient, 1 - xorient, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(1 + yorient, 1 + xorient, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(1 - xorient, 1 + yorient, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 20
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If spd > 0 Then
            If goingleft Then
                mr.Offset(-spd, 0)
                If mr.Left <= le Then
                    goingleft = False
                End If
            Else
                mr.Offset(spd, 0)
                If mr.Right >= re Then
                    goingleft = True
                End If
            End If
        End If
        Me.internalcounter += 1
        If attacking Then
            If Me.internalcounter = 24 Then
                Me.internalcounter = 0
                'Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
                Q.AddNegation(R.Left - 40.0F, R.Top - 40.0F)
                atktimer -= 1
            ElseIf Me.internalcounter = 12 Then
                'Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
                Q.AddNegation(R.Left - 40.0F, R.Top - 40.0F)
                atktimer -= 1
            End If
            If atktimer = 0 Then
                attacking = False
                xmultiplier = Convert.ToSingle(Math.Sin(orangle * Math.PI / 180))
                ymultiplier = Convert.ToSingle(Math.Cos(orangle * Math.PI / 180))
            End If
        ElseIf Not screwed Then
            If Me.internalcounter = 24 Then
                Me.internalcounter = 0
                Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
            ElseIf Me.internalcounter = 12 Then
                Q.Monsters.Add(New Lightball(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 8 * xmultiplier, 8 * ymultiplier, 0, 0, 50, Me))
            End If
        ElseIf internalcounter > 24 Then
            killed = True
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public Sub Illumination(ByVal Q As LevelSuite, ByVal P As System.Drawing.PointF) Implements IOpticalTool.Illumination
        Dim A, B, C As Microsoft.DirectX.Vector2, L As Single
        A = New Microsoft.DirectX.Vector2(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT)
        B = New Microsoft.DirectX.Vector2(P.X, P.Y)
        C = Microsoft.DirectX.Vector2.Subtract(B, A)
        C.Normalize()
        attacking = True
        atktimer = 2
        xmultiplier = C.X
        ymultiplier = C.Y
        'Q.Monsters.Add(New Bullet(mr.X, mr.Y, vx, Vy, 0, 0, 60))

    End Sub

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Screwer Then
            screwed = True
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return False
        End If
    End Function
End Class

#End Region
#Region "Immaterial"
Public Class Corven
    Implements IMonster

    Private mr As RectangleF
    Private goingleft As Boolean
    Private spd As Single
    Private state As CorvenStates
    Private internalcounter As Integer
    Private oy As Single
    Private frame As Integer

    Private hp As Integer

    Const MYHEIGHT As Single = 20
    Const MYWIDTH As Single = 32
    Const ATKRANGE As Single = 160.0F


    Private Enum CorvenStates
        Waiting
        Dropping
        Rising
        Redropping
        Rerising
    End Enum


    Public Sub New(ByVal Ceiling As RectangleF, ByVal LeftOffset As Integer)
        mr = New RectangleF(Ceiling.Left + LeftOffset, Ceiling.Bottom, MYWIDTH, MYHEIGHT)
        oy = Ceiling.Bottom
        hp = 2
        'state = HoroStates.Waiting
        frame = 0
        state = CorvenStates.Waiting
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Select Case frame
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 33
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If state = CorvenStates.Waiting Then
            If mr.X - R.X < ATKRANGE Then
                If mr.Y < R.Y Then
                    state = CorvenStates.Dropping
                Else
                    internalcounter += 1
                    If internalcounter = 300 Then
                        Q.Monsters.Add(New DirectBullet(mr.X + 10, mr.Y, R, 2, 100))
                        internalcounter = 0
                    End If
                End If

            End If
        ElseIf state = CorvenStates.Dropping Then
            If spd < 4.0F Then
                spd += 0.125F

            End If
            mr.Y += spd
            If mr.Y >= R.Y Then
                state = CorvenStates.Rising
            End If
        ElseIf state = CorvenStates.Rising Then
            spd -= 0.125F
            If spd <= -4.0F Then
                state = CorvenStates.Redropping
            End If
            mr.Y += spd
        ElseIf state = CorvenStates.Redropping Then
            spd += 0.125F
            If spd >= 4.0F Then
                state = CorvenStates.Rerising
            End If
            mr.Y += spd
        ElseIf state = CorvenStates.Rerising Then
            If spd > -4.0F Then
                spd -= 0.125F
            End If
            mr.Y += spd
            If mr.Y <= oy Then
                state = CorvenStates.Waiting
                mr.Y = oy
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Hoiler
    Implements IMonster
    Private mr As RectangleF
    Private vy As Single
    Private vx As Single
    Private internalcounter As Integer

    Private spawnnum As Integer
    Private hp As Integer
    Private jumping As Boolean

    Const MYWIDTH As Integer = 18
    Const MYHEIGHT As Integer = 28
    Const ATKRANGE As Single = 120.0F
    Const GRAVITY As Single = 0.6F
    Const DELAY As Integer = 10

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal Speed As Single, ByVal Count As Integer)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        vx = -Speed
        hp = 1
        spawnnum = Count
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Y > 700 OrElse hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F

        If jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)

        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True

    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 34
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If mr.Left - R.Right < ATKRANGE AndAlso Not jumping Then
            internalcounter += 1
            If internalcounter >= DELAY Then
                jumping = True
                vy = -15.0F
                If spawnnum > 0 Then
                    Q.Monsters.Add(New Hoiler(New RectangleF(mr.Left, mr.Top + MYHEIGHT, MYWIDTH, MYHEIGHT), 0, -0.8F * vx, spawnnum - 1))
                End If
            End If
        End If
        If jumping Then
            mr.Offset(vx, vy)
            vy += GRAVITY
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Horion
    Implements IMonster

    Private mr As RectangleF
    Private goingleft As Boolean
    Private spd As Single
    Private state As HoroStates
    Private internalcounter, hp As Integer
    Private oy As Single

    Const MYHEIGHT As Single = 18
    Const MYWIDTH As Single = 32
    Const ATKRANGE As Single = 60.0F


    Private Enum HoroStates
        Waiting
        Dropping
        Rising
    End Enum

    Public Sub New(ByVal Ceiling As RectangleF, ByVal LeftOffset As Single, ByVal Speed As Single)
        mr = New RectangleF(Ceiling.Left + LeftOffset, Ceiling.Bottom, MYWIDTH, MYHEIGHT)
        oy = Ceiling.Bottom
        state = HoroStates.Waiting
        spd = Speed
        hp = 3
    End Sub


    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim Ho, Hf As Single
        If Me.internalcounter Mod 8 >= 4 Then
            Ho = 0.5F
            Hf = 1.0F
        Else
            Ho = 0.0F
            Hf = 0.5F
        End If
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, Ho, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, Ho, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, Hf, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, Hf, 1.0F)
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 17
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx As Single = mr.X - R.X
        Me.internalcounter += 1
        Select Case state
            Case HoroStates.Waiting
                If dx < ATKRANGE AndAlso -dx < ATKRANGE Then
                    state = HoroStates.Dropping
                End If
            Case HoroStates.Dropping
                mr.Y += 2 * spd
                For Each Platform As RectangleF In Q.Platforms
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= Platform.Top AndAlso mr.Bottom <= Platform.Top Then
                        If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.

                            state = HoroStates.Rising
                            'Hit the ground, we are not jumping.
                            Me.internalcounter = 0

                            mr.Offset(0, Platform.Top - mr.Bottom)
                            'Move the player onto this platform.
                        End If
                    End If
                Next
                For Each Wall As RectangleF In Q.Walls
                    If mr.IntersectsWith(Wall) Then
                        state = HoroStates.Rising
                        Me.internalcounter = 0
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    End If
                Next
                For Each wall As RectangleF In Q.Crates
                    If mr.IntersectsWith(Wall) Then
                        state = HoroStates.Rising
                        Me.internalcounter = 0
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    End If
                Next
                For Each wall As RectangleF In Q.Extrawalls
                    If mr.IntersectsWith(Wall) Then
                        state = HoroStates.Rising
                        Me.internalcounter = 0
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    End If
                Next
                For Each wall As RectangleF In Q.Groundwalls
                    If mr.IntersectsWith(Wall) Then
                        state = HoroStates.Rising
                        Me.internalcounter = 0
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    End If
                Next
                For Each wall As RectangleF In Q.Glasses
                    If mr.IntersectsWith(Wall) Then
                        state = HoroStates.Rising
                        Me.internalcounter = 0
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    End If
                Next
                For Each wall As RectangleF In Q.EyeBeams
                    If mr.IntersectsWith(Wall) Then
                        state = HoroStates.Rising
                        Me.internalcounter = 0
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    End If
                Next
                For Each wall As RectangleF In Q.CrossStays
                    If mr.IntersectsWith(Wall) Then
                        state = HoroStates.Rising
                        Me.internalcounter = 0
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    End If
                Next
            Case HoroStates.Rising
                mr.Y -= spd
                If mr.Y <= oy Then
                    state = HoroStates.Waiting
                End If
        End Select
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Pearl Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        End If
    End Function
End Class
Public Class Squirm
    Implements IMonster

    Private mr As RectangleF
    Private internalcounter As Integer
    Private forwarding, goingleft As Boolean
    Private OY As Single
    Private frame, timing, hp As Integer

    Private spd As Single

    Const SHELLTIMER As Integer = 60
    Const FULLTIMER As Integer = 40
    Const ATKRANGE As Single = 350.0F

    Public Sub New(ByVal X As Single, ByVal Y As Single, ByVal spd As Single)
        mr = New RectangleF(X, Y, 9, 9)
        Me.spd = spd
        hp = 20
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Right < 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim Ho, Hf As Single

        Select Case frame
            Case 0
                Ho = 0.0F
                Hf = 1.0F / 7.0F
            Case 1, 9
                Ho = 1.0F / 7.0F
                Hf = 2.0F / 7.0F
            Case 2, 8
                Ho = 2.0F / 7.0F
                Hf = 3.0F / 7.0F
            Case 3, 7
                Ho = 3.0F / 7.0F
                Hf = 4.0F / 7.0F
            Case 4, 6
                Ho = 4.0F / 7.0F
                Hf = 5.0F / 7.0F
            Case 5
                If (timing >> 2) Mod 2 = 0 Then
                    Ho = 5.0F / 7.0F
                    Hf = 6.0F / 7.0F
                Else
                    Ho = 6.0F / 7.0F
                    Hf = 1.0F
                End If
        End Select

        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, Ho, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, Ho, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, Hf, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, Hf, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Select Case frame
                Case 0
                    Return 9
                Case 1, 9
                    Return 10
                Case 2, 8
                    Return 14
                Case 3, 7
                    Return 18
                Case 4, 6
                    Return 27
                Case 5
                    Return 31
            End Select
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 11
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx As Single
        If forwarding Then
            If goingleft Then
                mr.X -= spd
            Else
                mr.X += spd
            End If
        Else
            dx = mr.X - R.X
            If dx < ATKRANGE AndAlso dx > -ATKRANGE Then
                forwarding = True
            End If
            goingleft = (dx > 0)
        End If

        mr.X += (Me.Width * 0.5F)
        mr.Y += (Me.Height * 0.5F)

        timing += 1
        Select Case frame
            Case 0
                If timing >= SHELLTIMER Then
                    frame = 1
                    timing = 0
                End If
            Case 1 To 4, 6 To 8
                If timing >= 2 Then
                    frame += 1
                    timing = 0
                End If
            Case 5
                If timing >= FULLTIMER Then
                    frame = 6
                    timing = 0
                End If
            Case 9
                If timing >= 2 Then
                    frame = 0
                    timing = 0
                End If
        End Select

        mr.X -= (Me.Width * 0.5F)
        mr.Y -= (Me.Height * 0.5F)

        mr.Width = Me.Width
        mr.Height = Me.Height
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Select Case frame
                Case 0
                    Return 9
                Case 1, 9
                    Return 15
                Case 2, 8
                    Return 17
                Case 3, 7
                    Return 24
                Case 4, 6
                    Return 26
                Case 5
                    Return 26
            End Select
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Pearl Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return False
        End If
    End Function
End Class
Public Class GrimLeaper
    Implements IMonster



    'Standard declarations.
    Private mr, ir As RectangleF
    Private internalcounter, hp As Integer
    Private goingleft As Boolean

    Private jumping As Boolean, onslope As Boolean
    Private fallspeed As Single, initheight As Single

    Const MYWIDTH As Integer = 17
    Const MYHEIGHT As Integer = 23
    Const STANDHEIGHT As Integer = 18
    Const DIFFERENCE As Integer = MYHEIGHT - STANDHEIGHT

    Const GRAVITY As Single = 1, INITVELOC As Single = -16
    Const TIMEOUT As Integer = 16

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        hp = 7
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 1
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If jumping AndAlso goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    ElseIf jumping Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH * 3, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    ElseIf goingleft Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(0, 0, MYWIDTH, STANDHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(MYWIDTH << 1, 0, MYWIDTH, MYHEIGHT), GraphicsUnit.Pixel)
    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub


    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If jumping Then
            mr.Offset(0, fallspeed)
            If mr.Bottom >= initheight Then
                jumping = False
                mr.Offset(0, initheight - mr.Bottom)
            End If
            fallspeed += GRAVITY
        Else
            Me.internalcounter += 1
            If Me.internalcounter >= TIMEOUT Then
                jumping = True
                fallspeed = INITVELOC
                initheight = mr.Bottom
                Me.internalcounter = 0
            End If
        End If
        goingleft = (R.Left < mr.Left)
    End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        ir = RectangleF.FromLTRB(mr.Left + 4, mr.Top + 4, mr.Right - 4, mr.Bottom - 4)
        Return ir.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If jumping AndAlso goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            If jumping Then
                Return MYHEIGHT
            Else
                Return STANDHEIGHT
            End If
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 4
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Pearl Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        End If
    End Function
End Class
Public Class Leach
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter, hp As Integer
    Private goingleft As Boolean
    Private spd As Single

    Private jumping As Boolean, onslope As Boolean
    Private fallspeed As Single, initheight As Single

    Const MYWIDTH As Single = 17
    Const MYHEIGHT As Single = 23
    Const STANDHEIGHT As Single = 18
    Const DIFFERENCE As Single = MYHEIGHT - STANDHEIGHT

    Const GRAVITY As Single = 1, GRAVITYTH As Single = GRAVITY * 1.5F, INITVELOC As Single = -16
    Const TIMEOUT As Integer = 16
    Const TERMINALVELOCITY As Single = 20.0F

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal Spd As Single, ByVal TotheLeft As Boolean)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        Me.spd = Spd
        goingleft = TotheLeft
        hp = 7
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Bottom >= 800 OrElse hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If jumping AndAlso goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 26
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If jumping Then
            If goingleft Then
                mr.Offset(-spd, 0)
                For Each Wall As RectangleF In Q.Walls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Glasses
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Crates
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.CrossStays
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.RTDs
                    'Check with each wall.
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.Pneumos
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(sw.Gate.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
            Else
                mr.Offset(spd, 0)
                For Each Wall As RectangleF In Q.Walls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Crates
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Glasses
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.CrossStays
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.RTDs
                    'Check with each wall.
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.left - mr.Right, 0)
                        'Move the player to the edge of this wall.
                        goingleft = True
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.Pneumos
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(sw.Gate.Left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
            End If

            mr.Offset(0, fallspeed)
            fallspeed += GRAVITY
            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, Platform.Top - mr.Bottom)
                        jumping = False
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Walls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.EyeBeams
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.CrossStays
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Groundwalls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, wall.Top - mr.Bottom)
                        jumping = False
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Glasses
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Top <= wall.Bottom Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, wall.Top - mr.Bottom)
                        jumping = False
                    End If
                End If
            Next
            For Each sw As OpenerSwitch In Q.RTDs
                'Looking at each platform in the platform array.
                If mr.Bottom >= sw.Gate.Top AndAlso mr.Top <= sw.Gate.Bottom Then
                    If mr.Left < sw.Gate.Right AndAlso mr.Right > sw.Gate.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, sw.Gate.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, sw.Gate.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
            For Each sw As OpenerSwitch In Q.Pneumos
                'Looking at each platform in the platform array.
                If mr.Bottom >= sw.Gate.Top AndAlso mr.Top <= sw.Gate.Bottom Then
                    If mr.Left < sw.Gate.Right AndAlso mr.Right > sw.Gate.Left Then
                        If fallspeed > 0 Then
                            mr.Offset(0, sw.Gate.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, sw.Gate.Bottom - mr.Top + GRAVITYTH)
                            fallspeed = 0
                        End If
                    End If
                End If
            Next
        Else
            Me.internalcounter += 1
            If Me.internalcounter >= TIMEOUT Then
                jumping = True
                fallspeed = INITVELOC
                initheight = mr.Bottom
                Me.internalcounter = 0
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Pearl Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        End If
    End Function
End Class
Public Class Airstar
    Implements IMonster

    Private mr As RectangleF
    Private revgrav As Single
    Private spd, maxspd As Single
    Private internalcounter As Integer
    Private upperhalf As Boolean
    Private oy As Single

    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 16.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal RG As Single, ByVal Maxspd As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        revgrav = RG
        Me.maxspd = Maxspd
        Me.spd = Maxspd
        upperhalf = False
        oy = mr.Y
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim cx, cy As Single
        cx = Convert.ToSingle(Math.Cos(internalcounter / 4))
        cy = Convert.ToSingle(Math.Sin(internalcounter / 4))
        vs(CI) = New CustomVertex.PositionColoredTextured(N * cx, N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(N * cy, N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N * -cy, N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N * -cx, N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Q.SonicEvents.Add(SonicEventType.ShotDefend)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 50
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If upperhalf Then
            spd += revgrav
            If spd > maxspd Then
                spd = maxspd
            End If
            If mr.Y >= oy Then
                upperhalf = False
            End If
        Else
            spd -= revgrav
            If spd < -maxspd Then
                spd = -maxspd
            End If
            If mr.Y <= oy Then
                upperhalf = True
            End If
        End If
        mr.Offset(0, spd)
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Electrico
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private CW As Boolean, D As Dirs

    Private around As RectangleF, spd As Single

    Const MYWIDTH As Integer = 15, MYHEIGHT As Integer = 15
    Const NUMFRAMES As Integer = 4

    Enum Dirs
        L = 0
        U
        R
        D
    End Enum

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYWIDTH, MYWIDTH, MYHEIGHT)
        around = SourceRect
        CW = Not MovingLeft
        If MovingLeft Then
            D = Dirs.L
        Else
            D = Dirs.R
        End If
        spd = Speed
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function



    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If D = Dirs.L Then
            mr.Offset(-spd, 0)
            If mr.Right <= around.Left Then
                mr.Offset(around.Left - mr.Right, 0)
                If CW Then
                    D = Dirs.U
                Else
                    D = Dirs.D
                End If
            End If
        ElseIf D = Dirs.U Then
            mr.Offset(0, -spd)
            If mr.Bottom <= around.Top Then
                mr.Offset(0, around.Top - mr.Bottom)
                If CW Then
                    D = Dirs.R
                Else
                    D = Dirs.L
                End If
            End If
        ElseIf D = Dirs.R Then
            mr.Offset(spd, 0)
            If mr.Left >= around.Right Then
                mr.Offset(around.Right - mr.Left, 0)
                If CW Then
                    D = Dirs.D
                Else
                    D = Dirs.U
                End If
            End If
        ElseIf D = Dirs.D Then
            mr.Offset(0, spd)
            If mr.Top >= around.Bottom Then
                mr.Offset(0, around.Bottom - mr.Top)
                If CW Then
                    D = Dirs.L
                Else
                    D = Dirs.R
                End If
            End If

        End If

        For Each Wall As RectangleF In Q.Walls
            If mr.IntersectsWith(Wall) Then
                around = Wall
                Select Case D
                    Case Dirs.D
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        If CW Then
                            D = Dirs.R
                        Else
                            D = Dirs.L
                        End If
                    Case Dirs.L
                        mr.Offset(Wall.Right - mr.Left, 0)
                        If CW Then
                            D = Dirs.D
                        Else
                            D = Dirs.U
                        End If
                    Case Dirs.R
                        mr.Offset(Wall.Left - mr.Right, 0)
                        If CW Then
                            D = Dirs.U
                        Else
                            D = Dirs.D
                        End If
                    Case Dirs.U
                        mr.Offset(0, Wall.Bottom - mr.Top)
                        If CW Then
                            D = Dirs.L
                        Else
                            D = Dirs.R
                        End If
                End Select
            End If
        Next

        Me.internalcounter += 1
        If Me.internalcounter >= 16 Then
            Me.internalcounter = 0
        End If
    End Sub


    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Integer = Me.internalcounter >> 2
        vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, F * 0.25F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, F * 0.25F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 57
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function
End Class
Public Class Spikestrip
    Implements IMonster
    Private mr As RectangleF
    Private internalcounter As Integer

    Public Sub New(ByVal R As RectangleF)
        mr = R
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return False
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Integer = Me.internalcounter
        vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.0F, F * 0.125F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.0F, (F + 1) * 0.125F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 1.0F, F * 0.125F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, (F + 1) * 0.125F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Q.SonicEvents.Add(SonicEventType.ShotDefend)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return mr.Height
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 61
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        internalcounter += 1
        If internalcounter >= 8 Then
            internalcounter = 0
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return mr.Width
        End Get
    End Property
End Class
Public Class Exploder
    Implements IMonster
    Private mr As RectangleF
    Private spd As Single
    Private hp As Integer
    Private goingleft As Boolean
    Private internalcounter As Integer
    Private mystate As mystates

    Private Enum mystates
        Waiting = 0
        Falling
        Flying
    End Enum

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 150.0F
    Const YAZ As Single = 20.0F

    Public Sub New(ByVal xo As Single, ByVal yo As Single, ByVal s As Single)
        mr = New RectangleF(xo, yo, MYWIDTH, MYHEIGHT)
        spd = s
        hp = 1
        mystate = mystates.Waiting
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim cx, cy As Single
        cx = Convert.ToSingle(Math.Cos(internalcounter / 16))
        cy = Convert.ToSingle(Math.Sin(internalcounter / 16))
        vs(CI) = New CustomVertex.PositionColoredTextured(1.0F + N * cx, 1.0F + N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(1.0F + N * cy, 1.0F + N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(1.0F + N * -cy, 1.0F + N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(1.0F + N * -cx, 1.0F + N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.AddExplosion(mr.X + 16.0F, mr.Y + 16.0F)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 63
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If mystate = mystates.Waiting Then
            If mr.X - R.X < XAZ AndAlso R.X - mr.X < XAZ Then
                mystate = mystates.Falling
            End If
        ElseIf mystate = mystates.Falling Then
            mr.Offset(0.0F, 2.0F)
            If R.Y - mr.Y < 0 Then
                mystate = mystates.Flying
                goingleft = (mr.X > R.X)
            End If
        Else
            If goingleft Then
                mr.Offset(-spd, 0.0F)
            Else
                mr.Offset(spd, 0.0F)
            End If
        End If
        internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Pachinko
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private internalcounter As Integer
    Private attacking As Boolean
    Private horizontal As Boolean

    Const MYWIDTH As Single = 40.0F
    Const MYHEIGHT As Single = 40.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 3
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.41F
        Dim cx, cy As Single

        If attacking Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            cx = Convert.ToSingle(Math.Cos(internalcounter / 16))
            cy = Convert.ToSingle(Math.Sin(internalcounter / 16))
            vs(CI) = New CustomVertex.PositionColoredTextured(1.0F + N * cx, 1.0F + N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(1.0F + N * cy, 1.0F + N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(1.0F + N * -cy, 1.0F + N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(1.0F + N * -cx, 1.0F + N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Rubberet Then
            hp -= 5
        Else
            hp -= 1
        End If
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 64
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx, dy As Single
        Dim ang As Double
        If attacking Then
            Select Case internalcounter
                Case 104, 140
                    dx = R.X - mr.X
                    dy = R.Y - mr.Y
                    ang = Math.Atan2(dy, dx)
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang - 0.3)), CSng(3 * Math.Sin(ang - 0.3)), 0, 0, 160))
                Case 108, 136
                    dx = R.X - mr.X
                    dy = R.Y - mr.Y
                    ang = Math.Atan2(dy, dx)
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang - 0.15)), CSng(3 * Math.Sin(ang - 0.15)), 0, 0, 160))
                Case 112, 132
                    dx = R.X - mr.X
                    dy = R.Y - mr.Y
                    ang = Math.Atan2(dy, dx)
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang)), CSng(3 * Math.Sin(ang)), 0, 0, 160))
                Case 116, 128
                    dx = R.X - mr.X
                    dy = R.Y - mr.Y
                    ang = Math.Atan2(dy, dx)
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang + 0.15)), CSng(3 * Math.Sin(ang + 0.15)), 0, 0, 160))
                Case 120, 124
                    dx = R.X - mr.X
                    dy = R.Y - mr.Y
                    ang = Math.Atan2(dy, dx)
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang + 0.3)), CSng(3 * Math.Sin(ang + 0.3)), 0, 0, 160))
                Case Is >= 144
                    internalcounter = 0
                    attacking = False
                    horizontal = Not horizontal
            End Select
            internalcounter += 1

        Else
            If horizontal Then
                If R.X < mr.X Then
                    mr.Offset(-1.0F, 0.0F)
                Else
                    mr.Offset(1.0F, 0.0F)
                End If
            Else
                If R.Y < mr.Y Then
                    mr.Offset(0.0F, -1.0F)
                Else
                    mr.Offset(0.0F, 1.0F)
                End If
            End If
            internalcounter += 1
            If internalcounter >= 100 Then
                attacking = True
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Kite
    Implements IMonster
    Private mr As RectangleF
    Private spd As Single
    Private turn As Single
    Private rotate As Single
    Private hp As Integer
    Private RI As Random

    Public head, ang As Single
    Private dhead, dang As Single

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const PI As Single = 3.14159274F
    Const PIHALF As Single = PI / 2.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal S As Single, ByVal TSpd As Single, ByVal RSpd As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        spd = S
        turn = TSpd / 60.0F
        rotate = RSpd / 256.0F
        hp = 1
        head = -PIHALF / 2
        dhead = TSpd / 60.0F
        ang = 0.0R
        dang = RSpd / 256.0F

        RI = New Random
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim cx, cy As Single

        If ang < PIHALF Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 69
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim cx, cy As Single
        Dim A, B As Double, I As Integer
        cx = Convert.ToSingle(Math.Cos(head))
        cy = Convert.ToSingle(Math.Sin(head))
        mr.Offset(spd * cx, -spd * cy)

        I = RI.Next(128)
        If I = 0 Then
            dhead = -dhead
        End If
        I = RI.Next(128)
        If I = 0 Then
            dang = -dang
        End If



        head += dhead
        ang += dang
        If head > 2 * PI Then
            head -= 2 * PI
        ElseIf head < 0 Then
            head += 2 * PI
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class

Public Class Electrico2
    Implements IMonster
    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 24, MYHEIGHT As Integer = 25
    Const NUMFRAMES As Integer = 32

    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal Pic As Bitmap)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 2
        'pict = Pic
        Me.internalcounter = NUMFRAMES - 1
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        spd = Speed
        hp = 2
        Me.internalcounter = NUMFRAMES - 1
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F

        Select Case internalcounter
            Case 0, 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 2, 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 4, 5
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 6, 7
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 57
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Gobblet
    Implements IMonster

    Private mr As RectangleF
    Private goingleft As Boolean
    Private spd As Single
    Private state As GobbletStates
    Private internalcounter As Integer
    Private oy As Single
    Private hp As Integer

    Private sorx, sory As Single

    Const NEARRANGE As Single = 320.0F
    Const ATKRANGE As Single = 60.0F

    Const SOARSPEED As Single = 11

    Const MYHEIGHT As Single = 16.0F
    Const MYWIDTH As Single = 25.0F

    Enum GobbletStates
        Waiting
        Flying
        Soar
        Dropping
        Dropped
        Rising
    End Enum

    Public Sub New(ByVal X As Single, ByVal Y As Single, ByVal Spd As Single)
        mr = New RectangleF(X, Y, MYWIDTH, MYHEIGHT)
        hp = 4
        Me.spd = Spd
        state = GobbletStates.Waiting
        oy = Y
    End Sub


    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse mr.Y > 700
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = Me.internalcounter Mod 4
        Const N As Single = 1.0F
        If goingleft Then
            If state = GobbletStates.Dropped OrElse state = GobbletStates.Dropping Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.2F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.2F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.2F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.2F, 0.5F)

            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.8F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.8F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            End If
        Else
            If state = GobbletStates.Dropped OrElse state = GobbletStates.Dropping Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.2F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.2F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.2F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.2F, 1.0F)

            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.8F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.8F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            End If
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 15
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx As Single = mr.X - R.X
        Dim fallspeed As Single = spd + spd
        Me.internalcounter += 1

        If state = GobbletStates.Waiting Then
            If dx < NEARRANGE AndAlso -dx < NEARRANGE Then
                state = GobbletStates.Flying
                goingleft = (dx > 0)

            End If
        ElseIf state = GobbletStates.Flying Then
            If goingleft Then
                mr.X -= spd
                If dx < ATKRANGE Then
                    state = GobbletStates.Soar
                    sorx = -SOARSPEED
                    sory = 0
                End If
            Else
                mr.X += spd
                If -dx < ATKRANGE Then
                    state = GobbletStates.Soar
                    sorx = SOARSPEED
                    sory = 0
                End If
            End If
        ElseIf state = GobbletStates.Soar Then
            If goingleft Then
                sorx += 1
                sory -= 1
                mr.X += sorx
                mr.Y += sory
            Else
                sorx -= 1
                sory -= 1
                mr.X += sorx
                mr.Y += sory

            End If
            If sory <= -SOARSPEED Then
                state = GobbletStates.Dropping
            End If
        ElseIf state = GobbletStates.Dropping Then
            mr.Y += fallspeed
            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.

                        state = GobbletStates.Dropped
                        'Hit the ground, we are not jumping.
                        fallspeed = 0
                        Me.internalcounter = 0

                        mr.Offset(0, Platform.Top - mr.Bottom)
                        'Move the player onto this platform.
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    state = GobbletStates.Dropped
                    'Stop the jump.
                    fallspeed = 0
                    Me.internalcounter = 0



                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.

                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    state = GobbletStates.Dropped
                    fallspeed = 0
                    Me.internalcounter = 0
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    state = GobbletStates.Dropped
                    fallspeed = 0
                    Me.internalcounter = 0
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    state = GobbletStates.Dropped
                    fallspeed = 0
                    Me.internalcounter = 0
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    state = GobbletStates.Dropped
                    fallspeed = 0
                    Me.internalcounter = 0
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    state = GobbletStates.Dropped
                    fallspeed = 0
                    Me.internalcounter = 0
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each Wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(Wall) Then
                    state = GobbletStates.Dropped
                    fallspeed = 0
                    Me.internalcounter = 0
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
        ElseIf state = GobbletStates.Dropped Then
            If Me.internalcounter >= 160 Then
                state = GobbletStates.Rising
            End If
        ElseIf state = GobbletStates.Rising Then
            mr.Y -= fallspeed
            If mr.Y <= oy Then
                state = GobbletStates.Waiting
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.ForceMod Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        Else
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        End If
    End Function
End Class
Public Class Upsilon
    Implements IMonster

    Private mr As RectangleF
    Private state As BatStates
    Private hp As Integer
    Private internalcounter, outctr As Integer
    Private goingleft As Boolean
    Private swoopheight As Single

    Private Enum BatStates
        Perched = 0
        Descending
        Swooping
        Flying
        'In order of intended occurrence
    End Enum

    Const PERCHWIDTH As Integer = 32
    Const FLYWIDTH As Integer = 32
    Const FLYHEIGHT As Integer = 32
    Const ATKRANGE As Integer = 200

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, PERCHWIDTH, FLYHEIGHT)
        state = BatStates.Perched
        swoopheight = 128.0F
        hp = 1

    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse outctr >= 20
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        F = (internalcounter \ 2) Mod 4
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F * F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F * F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F * (F + 1), 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F * (F + 1), 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return FLYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 76
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Static dx, dy, dt As Single
        Select Case state
            Case BatStates.Perched
                If R.Right >= mr.Left - ATKRANGE AndAlso R.Left <= mr.Right + ATKRANGE Then
                    goingleft = True
                    state = BatStates.Descending
                    mr.Width = FLYWIDTH
                    Me.internalcounter += 1
                End If
            Case BatStates.Descending
                goingleft = True
                mr.Y += 8
                If R.Top < mr.Bottom + swoopheight Then
                    state = BatStates.Swooping
                    'System.Diagnostics.Debug.WriteLine("Swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    dy = 3
                End If
                Me.internalcounter += 1
            Case BatStates.Swooping
                dt += 0.25F
                If dt >= 2.0F Then
                    dt = 0.0F
                    If goingleft Then
                        dx -= 0.5F
                    Else
                        dx += 0.5F
                    End If
                    dy -= 0.25F
                    If dy <= 0.0F Then
                        state = BatStates.Flying
                        'System.Diagnostics.Debug.WriteLine("Not-swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    End If
                End If
                mr.Offset(dx, dy)
                Me.internalcounter += 1
            Case BatStates.Flying
                If goingleft Then
                    mr.Offset(-3.0F, 0)
                Else
                    mr.Offset(3.0F, 0)
                End If
                Me.internalcounter += 1
        End Select
        If Me.internalcounter >= 6 Then
            Me.internalcounter = 0
            'outctr += 1
            'If outctr Mod 5 = 0 Then
            '    Q.Monsters.Add(New DirectAcceleratorBullet(mr.X, mr.Y, R, 0.1F, 250))
            'End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return FLYWIDTH
        End Get
    End Property

End Class
Public Class Sunsaws
    Implements IMonster
    Private mr As RectangleF
    Private hp As Integer
    Private internalcounter As Integer
    Private goingleft, delayactive, descending As Boolean
    Private isout, falling As Boolean
    Private le, re, fallspeed As Single

    Const WIDTHIN As Single = 8.0F
    Const HEIGHTIN As Single = 8.0F
    Const WIDTHOUT As Single = 16.0F
    Const HEIGHTOUT As Single = 16.0F
    Const XAZ As Single = 300.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, WIDTHIN, HEIGHTIN)
        hp = 4
        isout = False
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        F = (internalcounter \ 4) Mod 3
        If isout Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFF00, 0.25F * F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFF00, 0.125F * F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFF00, 0.125F * F, 0.25F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFF00, 0.125F * (F + 1), 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFF00, 0.125F * (F + 1), 0.25F)
        End If


    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet OrElse BY = ShotTypes.Corkwheel Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ShockWave OrElse BY = ShotTypes.ForceMod OrElse BY = ShotTypes.Napalm OrElse BY = ShotTypes.Plasma Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return False
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            If isout Then
                Return HEIGHTOUT
            Else
                Return HEIGHTIN
            End If
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 77
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Const GRAVITY As Single = 0.8F, GRAVITYTH As Single = GRAVITY * 1.5F
        Const TERMINALVELOCITY As Integer = 40
        Dim wall As RectangleF
        If isout Then
            If delayactive Then
                If internalcounter = 100 Then
                    delayactive = False
                End If
            Else
                If goingleft Then
                    mr.Offset(-4.0F, 0)
                    For Each wall In Q.Walls
                        'Check with each wall.
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Right - mr.Left, 0)
                            'Move the player to the edge of this wall.
                            goingleft = False
                        End If
                    Next
                    For Each wall In Q.Extrawalls
                        'Check with each wall.
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Right - mr.Left, 0)
                            'Move the player to the edge of this wall.
                            goingleft = False
                        End If
                    Next
                    For Each wall In Q.EyeBeams
                        'Check with each wall.
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Right - mr.Left, 0)
                            'Move the player to the edge of this wall.
                            goingleft = False
                        End If
                    Next
                    For Each wall In Q.CrossStays
                        'Check with each wall.
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Right - mr.Left, 0)
                            'Move the player to the edge of this wall.
                            goingleft = False
                        End If
                    Next
                    For Each wall In Q.Groundwalls
                        'Check with each wall.
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Right - mr.Left, 0)
                            'Move the player to the edge of this wall.
                            goingleft = False
                        End If
                    Next
                    For Each wall In Q.Crates
                        'Check with each wall.
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Right - mr.Left, 0)
                            'Move the player to the edge of this wall.
                            goingleft = False
                        End If
                    Next
                    For Each wall In Q.Glasses
                        'Check with each wall.
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Right - mr.Left, 0)
                            'Move the player to the edge of this wall.
                            goingleft = False
                        End If
                    Next
                    If mr.Right <= le Then
                        falling = True
                    End If
                Else
                    mr.Offset(4, 0)
                    For Each wall In Q.Walls
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Left - mr.Right, 0)
                            'Move the player to the edge of this wall.
                            goingleft = True
                        End If
                    Next
                    For Each wall In Q.Extrawalls
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Left - mr.Right, 0)
                            'Move the player to the edge of this wall.
                            goingleft = True
                        End If
                    Next
                    For Each wall In Q.EyeBeams
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Left - mr.Right, 0)
                            'Move the player to the edge of this wall.
                            goingleft = True
                        End If
                    Next
                    For Each wall In Q.CrossStays
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Left - mr.Right, 0)
                            'Move the player to the edge of this wall.
                            goingleft = True
                        End If
                    Next
                    For Each wall In Q.Groundwalls
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Left - mr.Right, 0)
                            'Move the player to the edge of this wall.
                            goingleft = True
                        End If
                    Next
                    For Each wall In Q.Crates
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Left - mr.Right, 0)
                            'Move the player to the edge of this wall.
                            goingleft = True
                        End If
                    Next
                    For Each wall In Q.Glasses
                        If mr.IntersectsWith(wall) Then
                            mr.Offset(wall.Left - mr.Right, 0)
                            'Move the player to the edge of this wall.
                            goingleft = True
                        End If
                    Next
                    If mr.Left >= re Then
                        falling = True
                    End If
                End If
                If falling Then

                    mr.Offset(0, fallspeed + GRAVITY / 2.0F)
                    'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
                    fallspeed += GRAVITY
                    'dV = (PlayerVeloc +=) A (GRAVITY)
                    'Moves the character according to its velocity, gravity, and location.

                    If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                        fallspeed = TERMINALVELOCITY  'Limit the velocity.
                    End If

                    For Each wall In Q.Platforms
                        'Looking at each platform in the platform array.
                        If mr.Bottom >= wall.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= wall.Top Then
                            If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                                'The player has landed on this platform.  He needs to be updated.
                                'See how many advantages there are to using a Rectangle.

                                falling = False
                                'Hit the ground, we are not jumping.
                                fallspeed = 0

                                le = wall.Left : re = wall.Right
                                'Set the endpoints for the platform.
                                mr.Offset(0, wall.Top - mr.Bottom)
                                'Move the player onto this platform.
                            End If
                        End If
                    Next
                    For Each wall In Q.Walls
                        If mr.IntersectsWith(wall) Then
                            'If player is falling, then the player will land on the wall.
                            falling = False
                            'Stop the jump.
                            fallspeed = 0


                            le = wall.Left : re = wall.Right
                            'Set the endpoints.

                            mr.Offset(0, wall.Top - mr.Bottom)
                            'Position the player to stand on this wall.

                        End If
                    Next
                    For Each wall In Q.Groundwalls
                        If mr.IntersectsWith(wall) Then
                            falling = False
                            fallspeed = 0
                            le = wall.Left : re = wall.Right
                            mr.Offset(0, wall.Top - mr.Bottom)
                        End If
                    Next
                    For Each wall In Q.Extrawalls
                        If mr.IntersectsWith(wall) Then
                            falling = False
                            fallspeed = 0
                            le = wall.Left : re = wall.Right
                            mr.Offset(0, wall.Top - mr.Bottom)
                        End If
                    Next
                    For Each wall In Q.Crates
                        If mr.IntersectsWith(wall) Then
                            falling = False
                            fallspeed = 0
                            le = wall.Left : re = wall.Right
                            mr.Offset(0, wall.Top - mr.Bottom)
                        End If
                    Next
                    For Each wall In Q.EyeBeams
                        If mr.IntersectsWith(wall) Then
                            falling = False
                            fallspeed = 0
                            le = wall.Left : re = wall.Right
                            mr.Offset(0, wall.Top - mr.Bottom)
                        End If
                    Next
                    For Each wall In Q.CrossStays
                        If mr.IntersectsWith(wall) Then
                            falling = False
                            fallspeed = 0
                            le = wall.Left : re = wall.Right
                            mr.Offset(0, wall.Top - mr.Bottom)
                        End If
                    Next
                    For Each wall In Q.Glasses
                        If mr.IntersectsWith(wall) Then
                            falling = False
                            fallspeed = 0
                            le = wall.Left : re = wall.Right
                            mr.Offset(0, wall.Top - mr.Bottom)
                        End If
                    Next
                End If
            End If
        ElseIf descending Then
            If goingleft Then
                mr.Offset(-1.0F, 0.0F)
                If delayactive Then
                    If internalcounter = 50 Then
                        goingleft = False
                    End If
                ElseIf R.Left > mr.Left Then
                    delayactive = True
                    internalcounter = 0
                End If

            Else
                mr.Offset(1.0F, 0.0F)
                If delayactive Then
                    If internalcounter = 50 Then
                        goingleft = True
                    End If
                ElseIf R.Left < mr.Left Then
                    delayactive = True
                    internalcounter = 0
                End If
            End If
            For Each wall In Q.Walls
                If mr.IntersectsWith(wall) Then
                    goingleft = Not goingleft
                    delayactive = False
                End If
            Next
            For Each wall In Q.Extrawalls
                If mr.IntersectsWith(wall) Then
                    goingleft = Not goingleft
                    delayactive = False
                End If
            Next
            For Each wall In Q.Crates
                If mr.IntersectsWith(wall) Then
                    goingleft = Not goingleft
                    delayactive = False
                End If
            Next
            For Each wall In Q.Groundwalls
                If mr.IntersectsWith(wall) Then
                    goingleft = Not goingleft
                    delayactive = False
                End If
            Next
            For Each wall In Q.EyeBeams
                If mr.IntersectsWith(wall) Then
                    goingleft = Not goingleft
                    delayactive = False
                End If
            Next
            For Each wall In Q.CrossStays
                If mr.IntersectsWith(wall) Then
                    goingleft = Not goingleft
                    delayactive = False
                End If
            Next
            For Each wall In Q.Glasses
                If mr.IntersectsWith(wall) Then
                    goingleft = Not goingleft
                    delayactive = False
                End If
            Next
            mr.Offset(0.0F, 2.0F)
            For Each wall In Q.Walls
                If mr.IntersectsWith(wall) Then
                    isout = True
                    le = wall.Left
                    re = wall.Right
                    mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                    mr.Inflate(8.0F, 8.0F)
                    delayactive = True
                    internalcounter = 0
                End If
            Next
            For Each wall In Q.Platforms
                If mr.IntersectsWith(wall) Then
                    If mr.Top < wall.Top + 4 Then
                        isout = True
                        le = wall.Left
                        re = wall.Right
                        mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                        mr.Inflate(8.0F, 8.0F)
                        delayactive = True
                        internalcounter = 0
                    End If
                End If
            Next
            For Each wall In Q.Ices
                If mr.IntersectsWith(wall) Then
                    If mr.Top < wall.Top + 4 Then
                        isout = True
                        le = wall.Left
                        re = wall.Right
                        mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                        mr.Inflate(8.0F, 8.0F)
                        delayactive = True
                        internalcounter = 0
                    End If
                End If
            Next
            For Each wall In Q.Extrawalls
                If mr.IntersectsWith(wall) Then
                    isout = True
                    le = wall.Left
                    re = wall.Right
                    mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                    mr.Inflate(8.0F, 8.0F)
                    delayactive = True
                    internalcounter = 0
                End If
            Next
            For Each wall In Q.Crates
                If mr.IntersectsWith(wall) Then
                    isout = True
                    le = wall.Left
                    re = wall.Right
                    mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                    mr.Inflate(8.0F, 8.0F)
                    delayactive = True
                    internalcounter = 0
                End If
            Next
            For Each wall In Q.Groundwalls
                If mr.IntersectsWith(wall) Then
                    isout = True
                    le = wall.Left
                    re = wall.Right
                    mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                    mr.Inflate(8.0F, 8.0F)
                    delayactive = True
                    internalcounter = 0
                End If
            Next
            For Each wall In Q.EyeBeams
                If mr.IntersectsWith(wall) Then
                    isout = True
                    le = wall.Left
                    re = wall.Right
                    mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                    mr.Inflate(8.0F, 8.0F)
                    delayactive = True
                    internalcounter = 0
                End If
            Next
            For Each wall In Q.CrossStays
                If mr.IntersectsWith(wall) Then
                    isout = True
                    le = wall.Left
                    re = wall.Right
                    mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                    mr.Inflate(8.0F, 8.0F)
                    delayactive = True
                    internalcounter = 0
                End If
            Next
            For Each wall In Q.Glasses
                If mr.IntersectsWith(wall) Then
                    isout = True
                    le = wall.Left
                    re = wall.Right
                    mr.Offset(-4.0F, wall.Top - mr.Bottom - 8.0F)
                    mr.Inflate(8.0F, 8.0F)
                    delayactive = True
                    internalcounter = 0
                End If
            Next
        ElseIf R.Left - mr.Left < XAZ AndAlso mr.Left - R.Left < XAZ Then
            goingleft = (mr.Left > R.Left)
            descending = True


        End If
        internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If isout Then
                Return WIDTHOUT
            Else
                Return WIDTHIN
            End If
        End Get
    End Property
End Class
Public Class Aircan
    Implements IMonster
    Private mr As RectangleF
    Private bounds As RectangleF
    Private vx, vy As Single
    Private hp As Integer
    Private kill As Boolean

    Const MYWIDTH As Single = 64.0F
    Const MYHEIGHT As Single = 64.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Boundary As RectangleF, ByVal Speed As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        vx = Speed
        vy = Speed
        bounds = Boundary
        hp = 4
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        If vx > 0 Then
            If vy > 0 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            End If
        Else
            If vy > 0 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            End If
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 80
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim A, B, C As Microsoft.DirectX.Vector2
        Dim cx, cy As Single
        mr.Offset(vx, 0)
        If mr.Right > bounds.Right Then
            vx = -vx
        ElseIf mr.Left < bounds.Left Then
            vx = -vx
        End If
        mr.Offset(0, vy)
        If mr.Bottom > bounds.Bottom Then
            vy = -vy
        ElseIf mr.Top < bounds.Top Then
            vy = -vy
        End If
        If hp <= 0 Then
            kill = True
            A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
            B = New Microsoft.DirectX.Vector2(mr.X + 50, mr.Y + 60)
            C = Microsoft.DirectX.Vector2.Subtract(A, B)
            C.Normalize()
            cx = C.X * 2.5F
            cy = C.Y * 2.5F
            Q.Monsters.Add(New CustomShot(B.X, B.Y, -2.0F, -1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 2.0F, -1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, -2.0F, 1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 2.0F, 1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, 2.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, -2.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, -3.0F, -1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 3.0F, -1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, -3.0F, 1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 3.0F, 1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, 3.0F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, -3.0F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, -1.0F, -0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 1.0F, -0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, -1.0F, 0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 1.0F, 0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, 1.0F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
            Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, -1.0F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))

        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Cloud
    Implements IMonster
    Private mr As RectangleF
    Private hp As Integer
    Private internalcounter, atk As Integer
    Private attacking, done As Boolean
    Private RI As Random

    Const MYWIDTH As Single = 48.0F
    Const MYHEIGHT As Single = 24.0F
    Const XAZ As Single = 400.0F
    Const YDIFF As Single = 200.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal RI As Random)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 1
        Me.RI = RI
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse (internalcounter > 600 AndAlso attacking)
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Select Case (internalcounter \ 6) Mod 4
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            Q.SonicEvents.Add(SonicEventType.ShotThru)
        Else
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 82
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If attacking Then
            Select Case atk
                Case 0
                    If internalcounter Mod 32 = 0 Then
                        Q.Monsters.Add(New ForceMod(mr.X, mr.Y, 0, 5, 0, 0, 200))
                    ElseIf internalcounter Mod 32 = 16 Then
                        Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y, 0, 5, 0, 0, 200))
                    End If
                Case 1
                    If Not done Then
                        If mr.Left < R.Left Then
                            Q.Monsters.Add(New Napalm(mr.X, mr.Y, 0, 3, 0, 0, 70))
                            done = True
                        End If
                    End If
                Case 2
                    If internalcounter Mod 24 = 0 Then
                        Q.Monsters.Add(New Aquabullet(mr.X, mr.Y, Convert.ToSingle(-4 * RI.NextDouble), -4, 0, 0.05, 200))
                    ElseIf internalcounter Mod 24 = 12 Then
                        Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH, mr.Y, Convert.ToSingle(4 * RI.NextDouble), -4, 0, 0.05, 200))
                    End If
                Case 3
                    If Not done Then
                        If mr.Left < R.Left Then
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            Q.Monsters.Add(New CustomShot(mr.X, mr.Y - 500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, 1, 3.0F, 0.0F, 0.03125F, 400)))
                            done = True
                        End If
                    End If
            End Select
            mr.Offset(-1.0F, 0.0F)
        Else
            If R.Left - mr.Left < XAZ AndAlso mr.Left - R.Left < XAZ Then
                attacking = True
                atk = internalcounter Mod 4
                internalcounter = 0
            End If
            If mr.Top < R.Top - YDIFF Then
                mr.Offset(0.0F, 2.0F)
            Else
                mr.Offset(0.0F, -2.0F)
            End If
        End If
        internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Faruno
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft, speedingup As Boolean
    Private frame As Integer
    Private RI As Random
    Private Apeak As Single

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private hp As Integer

    Const MYWIDTH As Integer = 32, MYHEIGHT As Integer = 32
    Const TIMEOUT As Integer = 80
    Public Sub New(ByVal L As Integer, ByVal T As Integer, ByVal LeftEnd As Integer, ByVal RightEnd As Integer, ByVal MovingLeft As Boolean, ByVal Speed As Single, ByVal RI As Random)
        mr = New RectangleF(L, T, MYWIDTH, MYHEIGHT)
        le = LeftEnd
        re = RightEnd
        goingleft = MovingLeft
        spd = Speed
        hp = 7
        'pict = Pic: unused
    End Sub
    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Integer, ByVal MovingLeft As Boolean, ByVal RI As Random)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        'spd = Speed
        hp = 7
        Me.RI = RI
        Apeak = (Me.RI.Next(64) + 1) / 8.0F
    End Sub


    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Select Case frame
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.25F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.25F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.25F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.25F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.25F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.25F)
            Case 4
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.25F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.25F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
            Case 5
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.25F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.25F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            Case 6
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.25F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.25F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
            Case 7
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.25F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.25F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            Case 8
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.75F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
            Case 9
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
            Case 10
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.75F)
            Case 11
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.75F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.75F)
            Case 12
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 13
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 14
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 15
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 84
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            If speedingup Then
                spd -= 0.125F
                If spd <= -Apeak Then
                    speedingup = False
                End If
            Else
                spd += 0.125F
                If spd >= 0 Then
                    Apeak = (Me.RI.Next(64) + 1) / 8.0F
                    speedingup = True
                End If
            End If
        Else
            If speedingup Then
                spd += 0.125F
                If spd >= Apeak Then
                    speedingup = False
                End If
            Else
                spd -= 0.125F
                If spd <= 0 Then
                    Apeak = (Me.RI.Next(64) + 1) / 8.0F
                    speedingup = True
                End If
            End If
        End If
        mr.Offset(spd, 0)
        If mr.Left < le Then
            mr.Offset(-spd, 0)
            goingleft = False
            spd = Math.Abs(spd)
        ElseIf mr.Right > re Then
            mr.Offset(-spd, 0)
            goingleft = True
            spd = -Math.Abs(spd)
        End If
        internalcounter += 1
        frame = (internalcounter \ 2) Mod 16
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Cemurdo
    Implements IMonster

    Private mr As RectangleF
    Private hp, waited As Integer
    Private jumping, readytojump As Boolean
    Private goingleft As Boolean
    Private vx, vy As Single


    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const XAZ As Single = 250.0F
    Const YAZ As Single = 400.0F
    Const GRAVITY As Single = -0.35F, GRAVITYTH As Single = 1.5F * GRAVITY
    Const TERMINALVELOCITY As Single = -20.0F


    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 3
        jumping = True
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        If jumping And goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 91
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If R.Left - mr.Left < XAZ AndAlso mr.Left - R.Left < XAZ Then
            If R.Bottom - mr.Top < YAZ Then
                If Not jumping AndAlso waited = 0 Then
                    readytojump = True
                    waited = 50
                    Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 3.0F, 0.0F, 0.0F, 0.0F, 200))
                    Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, -3.0F, 0.0F, 0.0F, 0.0F, 200))
                    Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 3.0F, 0.0F, 0.0F, 200))
                    Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 2.12F, 2.12F, 0.0F, 0.0F, 200))
                    Q.Monsters.Add(New Bullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, -2.12F, 2.12F, 0.0F, 0.0F, 200))
                End If
            End If
        End If
        If readytojump Then
            waited -= 1
            If waited = 0 Then
                If mr.Left > R.Left Then
                    goingleft = True
                Else
                    goingleft = False
                End If
                vx = 2.0F
                readytojump = False
                jumping = True
                vy = 8.0F
            End If
        ElseIf jumping Then
            If goingleft Then
                mr.Offset(-vx, 0)
                For Each Wall As RectangleF In Q.Walls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Crates
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.CrossStays
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Glasses
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.RTDs
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.Pneumos
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(sw.Gate.Right - mr.Left, 0)
                        goingleft = False
                    End If
                Next
            Else
                mr.Offset(vx, 0)
                For Each Wall As RectangleF In Q.Walls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Crates
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.CrossStays
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Wall As RectangleF In Q.Glasses
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.RTDs
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.Pneumos
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(sw.Gate.Left - mr.Right, 0)
                        goingleft = True
                    End If
                Next
            End If

            mr.Offset(0, vy)
            vy += GRAVITY
            If vy <= TERMINALVELOCITY Then  'We're in TV.
                vy = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each wall As RectangleF In Q.Walls
                If mr.IntersectsWith(wall) Then
                    If vy < 0 Then 'falling up
                        mr.Offset(0, wall.Bottom - mr.Top)
                        jumping = False
                    Else    'jumping down
                        mr.Offset(0, wall.Top - mr.Bottom)
                        vy = 0
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(wall) Then
                    If vy < 0 Then 'falling up
                        mr.Offset(0, wall.Bottom - mr.Top)
                        jumping = False
                    Else    'jumping down
                        mr.Offset(0, wall.Top - mr.Bottom)
                        vy = 0
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                If mr.IntersectsWith(wall) Then
                    If vy < 0 Then 'falling up
                        mr.Offset(0, wall.Bottom - mr.Top)
                        jumping = False
                    Else    'jumping down
                        mr.Offset(0, wall.Top - mr.Bottom)
                        vy = 0
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(wall) Then
                    If vy < 0 Then 'falling up
                        mr.Offset(0, wall.Bottom - mr.Top)
                        jumping = False
                    Else    'jumping down
                        mr.Offset(0, wall.Top - mr.Bottom)
                        vy = 0
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(wall) Then
                    If vy < 0 Then 'falling up
                        mr.Offset(0, wall.Bottom - mr.Top)
                        jumping = False
                    Else    'jumping down
                        mr.Offset(0, wall.Top - mr.Bottom)
                        vy = 0
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(wall) Then
                    If vy < 0 Then 'falling up
                        mr.Offset(0, wall.Bottom - mr.Top)
                        jumping = False
                    Else    'jumping down
                        mr.Offset(0, wall.Top - mr.Bottom)
                        vy = 0
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(wall) Then
                    If vy < 0 Then 'falling up
                        mr.Offset(0, wall.Bottom - mr.Top)
                        jumping = False
                    Else    'jumping down
                        mr.Offset(0, wall.Top - mr.Bottom)
                        vy = 0
                    End If
                End If
            Next

        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Dilettante
    Implements IMonster
    Private spd, targetx As Single
    Private mr As RectangleF
    Private hp, atkcounter As Integer
    Private waiting As Boolean
    Private descending As Boolean

    Const MYWIDTH As Single = 40.0F
    Const MYHEIGHT As Single = 40.0F
    Const XAZ As Single = 100.0F
    Const ATKYDIFF As Single = 200.0F
    Const WAITHIGH As Single = 600.0F
    Const XDIVIDER As Single = 80.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Speed As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 2
        waiting = True
        descending = True
        spd = Speed
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 95
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If waiting Then
            If R.Y - mr.Y > WAITHIGH Then
                mr.Offset(0.0F, 1.0F)
            ElseIf R.Y - mr.Y < WAITHIGH Then
                mr.Offset(0.0F, -2.0F)
            End If
            If R.X - mr.X > XAZ Then
                waiting = False
            End If
        ElseIf descending Then
            mr.Offset(0.0F, 5.0F)
            If R.Top - mr.Top < ATKYDIFF Then
                descending = False
                targetx = XDIVIDER
            End If
        Else
            If R.Left - mr.Left < targetx Then
                atkcounter += 1
                If atkcounter <= 3 Then
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 0.0F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 0.4F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 0.8F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 1.2F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 1.6F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 2.0F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 2.4F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 2.8F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 3.2F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 3.6F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 4.0F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 4.4F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 4.8F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 5.2F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 5.6F, 0.0F, 0.0F, 120))
                    Q.Monsters.Add(New Plasma(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F, 6.0F, 0.0F, 0.0F, 120))
                    targetx -= XDIVIDER
                Else
                    Q.AddShard(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, -2.0F)
                    Q.AddShard(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 2.0F)
                    Q.AddShard(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 0.0F)
                    Q.SonicEvents.Add(SonicEventType.BreakGlass)
                    hp = 0
                End If
            Else
                mr.Offset(spd, 0.0F)
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SpaceRockL
    Implements IMonster

    Private mr As RectangleF
    Private boundary As RectangleF
    Private hp, sc, internalcounter As Integer
    Private spdx, spdy As Single
    Private RI As Random
    Private kill As Boolean

    Const MYWIDTH As Single = 64.0F
    Const MYHEIGHT As Single = 64.0F
    Const BOUNDWIDTH As Single = 1024.0F
    Const BOUNDHEIGHT As Single = 512.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Speed As Single, ByVal SpawnCount As Integer, ByVal RNG As Random)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        boundary = New RectangleF(mr.X - 0.5F * BOUNDWIDTH, mr.Y - 0.5F * BOUNDHEIGHT, BOUNDWIDTH, BOUNDHEIGHT)
        hp = 1
        RI = RNG
        sc = SpawnCount
        spdx = Speed * 0.7F
        spdy = Speed * 0.7F
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim cx, cy As Single
        cx = Convert.ToSingle(Math.Cos(internalcounter / 16))
        cy = Convert.ToSingle(Math.Sin(internalcounter / 16))
        vs(CI) = New CustomVertex.PositionColoredTextured(N * cx, N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(N * cy, N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N * -cy, N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N * -cx, N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 97
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim LV As Integer
        mr.Offset(spdx, 0.0F)
        If mr.Right > boundary.Right Then
            spdx = -spdx
            mr.Offset(spdx, 0.0F)
        ElseIf mr.Left < boundary.Left Then
            spdx = -spdx
            mr.Offset(spdx, 0.0F)
        End If
        mr.Offset(0.0F, spdy)
        If mr.Bottom > boundary.Bottom Then
            spdy = -spdy
            mr.Offset(0.0F, spdy)
        ElseIf mr.Top < boundary.Top Then
            spdy = -spdy
            mr.Offset(0.0F, spdy)
        End If
        internalcounter += 1
        If hp <= 0 Then
            For LV = 1 To sc
                Q.Monsters.Add(New SpaceRockM(mr.X + 16.0F, mr.Y + 16.0F, 2 * spdx, sc, RI, RI.NextDouble * 3.14, boundary))
            Next
            kill = True
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SpaceRockM
    Implements IMonster

    Private mr As RectangleF
    Private boundary As RectangleF
    Private hp, sc, internalcounter As Integer
    Private spdx, spdy As Single
    Private RI As Random
    Private kill As Boolean

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const BOUNDWIDTH As Single = 1024.0F
    Const BOUNDHEIGHT As Single = 512.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Speed As Single, ByVal SpawnCount As Integer, ByVal RNG As Random, ByVal Ang As Double, ByVal Bound As RectangleF)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        boundary = Bound
        hp = 1
        RI = RNG
        sc = SpawnCount
        spdx = Speed * Convert.ToSingle(Math.Cos(Ang))
        spdy = Speed * Convert.ToSingle(Math.Sin(Ang))
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim cx, cy As Single
        cx = Convert.ToSingle(Math.Cos(internalcounter / 16))
        cy = Convert.ToSingle(Math.Sin(internalcounter / 16))
        vs(CI) = New CustomVertex.PositionColoredTextured(N * cx, N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(N * cy, N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N * -cy, N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N * -cx, N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 97
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim LV As Integer
        mr.Offset(spdx, 0.0F)
        If mr.Right > boundary.Right Then
            spdx = -spdx
            mr.Offset(spdx, 0.0F)
        ElseIf mr.Left < boundary.Left Then
            spdx = -spdx
            mr.Offset(spdx, 0.0F)
        End If
        mr.Offset(0.0F, spdy)
        If mr.Bottom > boundary.Bottom Then
            spdy = -spdy
            mr.Offset(0.0F, spdy)
        ElseIf mr.Top < boundary.Top Then
            spdy = -spdy
            mr.Offset(0.0F, spdy)
        End If
        internalcounter += 1
        If hp <= 0 Then
            For LV = 1 To sc
                Q.Monsters.Add(New SpaceRockS(mr.X + 16.0F, mr.Y + 16.0F, 2 * spdx, sc, RI.NextDouble * 3.14, boundary))
            Next
            kill = True
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SpaceRockS
    Implements IMonster

    Private mr As RectangleF
    Private boundary As RectangleF
    Private hp, sc, internalcounter As Integer
    Private spdx, spdy As Single
    Private kill As Boolean

    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 16.0F
    Const BOUNDWIDTH As Single = 1024.0F
    Const BOUNDHEIGHT As Single = 512.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Speed As Single, ByVal SpawnCount As Integer, ByVal Ang As Double, ByVal Bound As RectangleF)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        boundary = Bound
        hp = 1
        sc = SpawnCount
        spdx = Speed * Convert.ToSingle(Math.Cos(Ang))
        spdy = Speed * Convert.ToSingle(Math.Sin(Ang))
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim cx, cy As Single
        cx = Convert.ToSingle(Math.Cos(internalcounter / 16))
        cy = Convert.ToSingle(Math.Sin(internalcounter / 16))
        vs(CI) = New CustomVertex.PositionColoredTextured(N * cx, N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(N * cy, N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N * -cy, N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N * -cx, N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 97
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        mr.Offset(spdx, 0.0F)
        If mr.Right > boundary.Right Then
            spdx = -spdx
            mr.Offset(spdx, 0.0F)
        ElseIf mr.Left < boundary.Left Then
            spdx = -spdx
            mr.Offset(spdx, 0.0F)
        End If
        mr.Offset(0.0F, spdy)
        If mr.Bottom > boundary.Bottom Then
            spdy = -spdy
            mr.Offset(0.0F, spdy)
        ElseIf mr.Top < boundary.Top Then
            spdy = -spdy
            mr.Offset(0.0F, spdy)
        End If
        internalcounter += 1
        If hp <= 0 Then
            Q.Monsters.Add(New StoneBullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, -8.0F, 0.0F, 0.0F, 0.0F, 120))
            Q.Monsters.Add(New StoneBullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 8.0F, 0.0F, 0.0F, 0.0F, 120))
            kill = True
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BrassGolem
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private le, re, lastbtm As Single
    Private enraged, advancing As Boolean
    Private enragedcounter, internalcounter, heatup As Integer

    Const MYWIDTH As Single = 27.0F
    Const MYHEIGHT As Single = 96.0F
    Const HEATPERHIT As Integer = 40
    Const HEATUPENRAGE As Integer = 200
    Const ENRAGETIME As Integer = 120

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Leftend As Single, ByVal Rightend As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 25
        le = Leftend
        re = Rightend
        If mr.X < le Then
            mr.X = le
        ElseIf mr.Right > re Then
            mr.Offset(re - mr.Right, 0.0F)
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Integer
        F = (Me.internalcounter >> 4) Mod 4
        If enraged Then
            Select Case F
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                Case 2
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                Case 3
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            End Select
        Else
            Select Case F
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                Case 2
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                Case 3
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            End Select
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If enraged Then
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        Else
            heatup += HEATPERHIT
            If heatup >= HEATUPENRAGE Then
                enraged = True
                enragedcounter = 0
            End If
            If BY = ShotTypes.ShockWave Then
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            ElseIf BY = ShotTypes.Aquabullet Then
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            ElseIf BY = ShotTypes.Rubberet Then
                hp -= 1
                Return True
            ElseIf BY = ShotTypes.Napalm Then
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return False
            ElseIf BY = ShotTypes.Screwer Then
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            ElseIf BY = ShotTypes.Corkwheel Then
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            ElseIf BY = ShotTypes.ForceMod Then
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            ElseIf BY = ShotTypes.Stonebullet Then
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            ElseIf BY = ShotTypes.Plasma Then
                hp -= 3
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            End If
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 101
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx, dy As Single
        Dim ang As Double
        internalcounter += 1
        If enraged Then
            enragedcounter += 1
            If enragedcounter = 20 Then
                dx = R.X - mr.X
                dy = R.Y - mr.Y
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang - 0.3)), CSng(3 * Math.Sin(ang - 0.3)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang - 0.3)), CSng(2 * Math.Sin(ang - 0.3)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4 * Math.Cos(ang - 0.3)), CSng(4 * Math.Sin(ang - 0.3)), 0, 0, 400))
            ElseIf enragedcounter = 40 Then
                dx = R.X - mr.X
                dy = R.Y - mr.Y
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang - 0.15)), CSng(3 * Math.Sin(ang - 0.15)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang - 0.15)), CSng(2 * Math.Sin(ang - 0.15)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4 * Math.Cos(ang - 0.15)), CSng(4 * Math.Sin(ang - 0.15)), 0, 0, 400))
            ElseIf enragedcounter = 60 Then
                dx = R.X - mr.X
                dy = R.Y - mr.Y
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang)), CSng(3 * Math.Sin(ang)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang)), CSng(2 * Math.Sin(ang)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4 * Math.Cos(ang)), CSng(4 * Math.Sin(ang)), 0, 0, 400))
            ElseIf enragedcounter = 80 Then
                dx = R.X - mr.X
                dy = R.Y - mr.Y
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang + 0.15)), CSng(3 * Math.Sin(ang + 0.15)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang + 0.15)), CSng(2 * Math.Sin(ang + 0.15)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4 * Math.Cos(ang + 0.15)), CSng(4 * Math.Sin(ang + 0.15)), 0, 0, 400))
            ElseIf enragedcounter = 100 Then
                dx = R.X - mr.X
                dy = R.Y - mr.Y
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(3 * Math.Cos(ang + 0.3)), CSng(3 * Math.Sin(ang + 0.3)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang + 0.3)), CSng(2 * Math.Sin(ang + 0.3)), 0, 0, 400))
                Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4 * Math.Cos(ang + 0.3)), CSng(4 * Math.Sin(ang + 0.3)), 0, 0, 400))
            End If
            If enragedcounter >= ENRAGETIME Then
                enraged = False
            End If
        End If
        If lastbtm = R.Bottom Then
            advancing = (R.Bottom = mr.Bottom)
        End If
        lastbtm = R.Bottom
        If advancing Then
            mr.Offset(-0.8F, 0.0F)
            If mr.Left < le Then
                mr.Offset(0.8F, 0.0F)
            End If
        Else
            mr.Offset(2.0F, 0.0F)
            If mr.Right > re Then
                mr.Offset(-2.0F, 0.0F)
            End If
        End If
        If heatup > 0 Then
            heatup -= 1
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Springalaru
    Implements IMonster
    Private mr As RectangleF
    Private hp, zipctr As Integer
    Private spd, rg As Single
    Private goingup, kill, nearactive As Boolean

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Const ZIPFRAME As Integer = 7
    Const ZIPLENGTH As Integer = 8 * ZIPFRAME
    Const VERTSPD As Single = 8.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Speed As Single, ByVal Range As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 2
        spd = Speed
        rg = Range
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill OrElse mr.Bottom >= 700 OrElse mr.Top < -3000
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim tu, td As Single
        If zipctr = 0 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
        Else
            If goingup Then
                tu = 1.0F
                td = 0.5F
            Else
                tu = 0.5F
                td = 1.0F
            End If

            If zipctr < ZIPFRAME Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, tu)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.0F, td)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, tu)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.25F, td)
            ElseIf zipctr < ZIPFRAME << 1 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, tu)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.25F, td)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, tu)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.5F, td)
            ElseIf zipctr < 3 * ZIPFRAME Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, tu)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.5F, td)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, tu)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.75F, td)
            ElseIf zipctr < ZIPFRAME * 5 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, tu)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.75F, td)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, tu)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, td)
            ElseIf zipctr < ZIPFRAME * 6 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, tu)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.5F, td)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, tu)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.75F, td)
            ElseIf zipctr < ZIPFRAME * 7 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, tu)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.25F, td)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, tu)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.5F, td)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, tu)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.0F, td)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, tu)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 0.25F, td)
            End If


        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 102
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move

        If hp <= 0 Then
            Q.Monsters.Add(New DirectBullet(mr.X + 16, mr.Y + 16, R, spd + 1, 300))
            Q.Monsters.Add(New DirectBullet(mr.X + 27, mr.Y + 5, R, spd + 1, 300))
            Q.Monsters.Add(New DirectBullet(mr.X + 5, mr.Y + 27, R, spd + 1, 300))
            Q.Monsters.Add(New DirectBullet(mr.X + 27, mr.Y + 27, R, spd + 1, 300))
            Q.Monsters.Add(New DirectBullet(mr.X + 5, mr.Y + 5, R, spd + 1, 300))
            kill = True
        End If
        If mr.X - R.X < rg AndAlso R.X - mr.X < rg Then
            nearactive = True
        End If
        If zipctr > 0 Then
            zipctr += 1
            If zipctr >= ZIPLENGTH Then
                zipctr = 0
                goingup = Not goingup
            End If
        Else
            If nearactive Then
                If R.X < mr.X Then
                    mr.Offset(-spd, 0.0F)
                    For Each Wall As RectangleF In Q.Walls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Extrawalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Glasses
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Crates
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.EyeBeams
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.CrossStays
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Groundwalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(spd, 0)
                        End If
                    Next
                Else
                    mr.Offset(spd, 0.0F)
                    For Each Wall As RectangleF In Q.Walls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(-spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Extrawalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(-spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Glasses
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(-spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Crates
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(-spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.EyeBeams
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(-spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.CrossStays
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(-spd, 0)
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Groundwalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(-spd, 0)
                        End If
                    Next
                End If
            End If
            If goingup Then
                mr.Offset(0.0F, -VERTSPD)
                For Each wall As RectangleF In Q.Walls
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Extrawalls
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Glasses
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Crates
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.EyeBeams
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.CrossStays
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Groundwalls
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, VERTSPD)
                        End If
                    End If
                Next
            Else
                mr.Offset(0.0F, VERTSPD)
                For Each wall As RectangleF In Q.Walls
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, -VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Extrawalls
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, -VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Glasses
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, -VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Crates
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, -VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.EyeBeams
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, -VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.CrossStays
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, -VERTSPD)
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Groundwalls
                    If mr.Top <= wall.Bottom AndAlso mr.Bottom >= wall.top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            zipctr = 1
                            mr.Offset(0.0F, -VERTSPD)
                        End If
                    End If
                Next
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class EvilCoffee
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private falling As Boolean, onslope As Boolean
    Private fallspeed As Single
    Private hp As Integer
    Private collideX As Single

    Const MYWIDTH As Single = 19.0F, MYHEIGHT As Single = 34.0F, JUMPSPD As Single = -16.0F
    Const NUMFRAMES As Integer = 16

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single, ByVal MovingLeft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYHEIGHT, MYWIDTH, MYHEIGHT)
        le = SourceRect.Left
        re = SourceRect.Right
        goingleft = MovingLeft
        'spd = Speed
        spd = 1
        Me.internalcounter = NUMFRAMES - 1
        hp = 7
    End Sub
    Public Sub New(ByVal LocX As Single, ByVal LocY As Single, ByVal LE As Single, ByVal RE As Single, ByVal movingleft As Boolean, ByVal Speed As Single)
        mr = New RectangleF(LocX, LocY, MYWIDTH, MYHEIGHT)
        Me.le = LE
        Me.re = RE
        goingleft = movingleft
        spd = Speed
        Me.internalcounter = NUMFRAMES - 1
        hp = 7
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Const GRAVITY As Single = 0.7F, GRAVITYTH As Single = GRAVITY * 1.5F
        Const TERMINALVELOCITY As Integer = 40
        Dim collided As Boolean = False

        If goingleft Then
            mr.Offset(-spd, 0)
            For Each Wall As RectangleF In Q.Walls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Glasses
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            If Not falling Then
                If mr.Right <= le Then
                    falling = True
                ElseIf collided Then
                    If mr.X = collideX Then
                        goingleft = False
                        collideX = 0.0F
                    Else
                        collideX = mr.X
                        falling = True
                        fallspeed = JUMPSPD
                    End If
                End If
            End If
        Else
            mr.Offset(spd, 0)
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    collided = True
                End If
            Next
            If Not falling Then
                If mr.Left >= re Then
                    falling = True
                ElseIf collided Then
                    If mr.X = collideX Then
                        goingleft = True
                        collideX = 0.0F
                    Else
                        collideX = mr.X
                        falling = True
                        fallspeed = JUMPSPD
                    End If
                End If
            End If
        End If
        If falling Then

            mr.Offset(0, fallspeed + GRAVITY / 2.0F)
            'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
            fallspeed += GRAVITY
            'dV = (PlayerVeloc +=) A (GRAVITY)
            'Moves the character according to its velocity, gravity, and location.

            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        If fallspeed > 0 Then
                            falling = False
                            fallspeed = 0
                            le = Platform.Left : re = Platform.Right
                            mr.Offset(0, Platform.Top - mr.Bottom)
                        End If
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    If fallspeed > 0 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        fallspeed = 0
                        mr.Offset(0, wall.Bottom - mr.Top + 1.0F)
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    If fallspeed > 0 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        fallspeed = 0
                        mr.Offset(0, wall.Bottom - mr.Top + 1.0F)
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    If fallspeed > 0 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        fallspeed = 0
                        mr.Offset(0, wall.Bottom - mr.Top + 1.0F)
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(Wall) Then
                    If fallspeed > 0 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        fallspeed = 0
                        mr.Offset(0, wall.Bottom - mr.Top + 1.0F)
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    If fallspeed > 0 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        fallspeed = 0
                        mr.Offset(0, wall.Bottom - mr.Top + 1.0F)
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    If fallspeed > 0 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        fallspeed = 0
                        mr.Offset(0, wall.Bottom - mr.Top + 1.0F)
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    If fallspeed > 0 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        fallspeed = 0
                        mr.Offset(0, wall.Bottom - mr.Top + 1.0F)
                    End If
                End If
            Next
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = Me.internalcounter >> 2
        Const N As Single = 1.0F
        If Not goingleft Then
            F += 4
        End If
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.125F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.125F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.125F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.125F, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 103
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Dim Vx, Vy As Single
        If BY = ShotTypes.ShockWave Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Aquabullet Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Screwer Then
            hp -= 25
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 60))
            Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
            Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 60))
            Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 60))
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 7
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
        End If
        Return True
    End Function

End Class
#End Region
#Region "Animal"
Public Class RegBat
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific
    Private state As BatStates
    Private spd As Single
    Private hp As Integer

    Private swoopheight As Single

    'Finite state machine.
    Private Enum BatStates
        Perched = 0
        Descending
        Swooping
        Flying
        'In order of intended occurrence
    End Enum

    Const PERCHWIDTH As Integer = 9
    Const FLYWIDTH As Integer = 26
    Const FLYHEIGHT As Integer = 15
    Const ATKRANGE As Integer = 100
    'Const SWOOPHEIGHT As Integer = 18

    Public Sub New(ByVal Ceiling As RectangleF, ByVal LeftOffset As Single, ByVal Speed As Single)
        mr = New RectangleF(Ceiling.Left + LeftOffset, Ceiling.Bottom, PERCHWIDTH, FLYHEIGHT)
        state = BatStates.Perched
        spd = Speed
        swoopheight = 16 * Speed * Speed + 3 * Speed - 16
        hp = 1
    End Sub

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Dim F As Integer = Me.internalcounter >> 2
    '    mr.Offset(-ScreenLeft, -ScreenTop)
    '    If state = BatStates.Perched Then
    '        GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(104, 0, PERCHWIDTH, FLYHEIGHT), GraphicsUnit.Pixel)
    '    Else
    '        If goingleft Then
    '            GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(FLYWIDTH * F, 0, FLYWIDTH, FLYHEIGHT), GraphicsUnit.Pixel)
    '        Else
    '            GFX.DrawImage(pict, mr.X, mr.Y, New RectangleF(FLYWIDTH * (2 + F), 0, FLYWIDTH, FLYHEIGHT), GraphicsUnit.Pixel)
    '        End If

    '    End If
    '    mr.Offset(ScreenLeft, ScreenTop)
    'End Sub

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Static dx, dy, dt As Single
        Select Case state
            Case BatStates.Perched
                If R.Right >= mr.Left - ATKRANGE AndAlso R.Left <= mr.Right + ATKRANGE Then
                    goingleft = (R.Left < mr.Left)
                    state = BatStates.Descending
                    mr.Width = FLYWIDTH
                End If
            Case BatStates.Descending
                goingleft = (R.Left < mr.Left)
                mr.Y += spd
                If R.Top < mr.Bottom + swoopheight Then
                    state = BatStates.Swooping
                    'System.Diagnostics.Debug.WriteLine("Swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    dy = spd
                End If
                Me.internalcounter += 1
            Case BatStates.Swooping
                dt += 0.25F
                If dt >= 2.0F Then
                    dt = 0.0F
                    If goingleft Then
                        dx -= 0.5F
                    Else
                        dx += 0.5F
                    End If
                    dy -= 0.25F
                    If dy <= 0.0F Then
                        state = BatStates.Flying
                        'System.Diagnostics.Debug.WriteLine("Not-swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    End If
                End If
                mr.Offset(dx, dy)
                Me.internalcounter += 1
            Case BatStates.Flying
                If goingleft Then
                    mr.Offset(-spd * 2.0F, 0)
                Else
                    mr.Offset(spd * 2.0F, 0)
                End If
                Me.internalcounter += 1
        End Select
        If Me.internalcounter >= 6 Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Right < 0 OrElse hp <= 0
    End Function

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F, P As Single = 0.3F
        Const PERCHFRAME As Single = 26.0F / 113.0F
        Dim F As Single = (Me.internalcounter \ 3)

        If state = BatStates.Perched Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            If Not goingleft Then
                F += 2
            End If

            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return FLYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 2
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(Me.mr.X, Me.mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If state = BatStates.Perched Then
                Return PERCHWIDTH
            Else
                Return FLYWIDTH
            End If
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class BoxBat
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific
    Private state As BatStates
    Private spd As Single
    Private hp As Integer

    Private swoopheight As Single

    'Finite state machine.
    Private Enum BatStates
        Perched = 0
        Descending
        Swooping
        Flying
        'In order of intended occurrence
    End Enum

    Const PERCHWIDTH As Integer = 9
    Const FLYWIDTH As Integer = 26
    Const FLYHEIGHT As Integer = 15
    Const ATKRANGE As Integer = 100
    'Const SWOOPHEIGHT As Integer = 18

    Public Sub New(ByVal Ceiling As RectangleF, ByVal LeftOffset As Single, ByVal Speed As Single)
        mr = New RectangleF(Ceiling.Left + LeftOffset, Ceiling.Bottom, PERCHWIDTH, FLYHEIGHT)
        state = BatStates.Perched
        spd = Speed
        swoopheight = -8
        hp = 1
    End Sub

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Static dx, dy, dt As Single
        Select Case state
            Case BatStates.Perched
                If R.Right >= mr.Left - ATKRANGE AndAlso R.Left <= mr.Right + ATKRANGE Then
                    goingleft = (R.Left < mr.Left)
                    state = BatStates.Descending
                    mr.Width = FLYWIDTH
                End If
            Case BatStates.Descending
                goingleft = (R.Left < mr.Left)
                mr.Y += spd
                If R.Top < mr.Bottom + swoopheight Then
                    state = BatStates.Flying
                    'System.Diagnostics.Debug.WriteLine("Swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    dy = spd
                End If
                Me.internalcounter += 1
            Case BatStates.Swooping
                dt += 0.25F
                If dt >= 2.0F Then
                    dt = 0.0F
                    If goingleft Then
                        dx -= 0.5F
                    Else
                        dx += 0.5F
                    End If
                    dy -= 0.25F
                    If dy <= 0.0F Then
                        state = BatStates.Flying
                        'System.Diagnostics.Debug.WriteLine("Not-swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    End If
                End If
                mr.Offset(dx, dy)
                Me.internalcounter += 1
            Case BatStates.Flying
                If goingleft Then
                    mr.Offset(-spd * 2.0F, 0)
                Else
                    mr.Offset(spd * 2.0F, 0)
                End If
                Me.internalcounter += 1
        End Select
        If Me.internalcounter >= 6 Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Right < 0 OrElse hp <= 0
    End Function

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F, P As Single = 0.3F
        Const PERCHFRAME As Single = 26.0F / 113.0F
        Dim F As Single = (Me.internalcounter \ 3)
        If state = BatStates.Perched Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            If Not goingleft Then
                F += 2
            End If

            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 1.0F)
        End If


    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return FLYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 2
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(Me.mr.X, Me.mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If state = BatStates.Perched Then
                Return PERCHWIDTH
            Else
                Return FLYWIDTH
            End If
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class SineBat
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific
    Private state As BatStates
    Private spd As Single
    Private hp As Integer
    Private ang As Single

    Private swoopheight As Single

    'Finite state machine.
    Private Enum BatStates
        Perched = 0
        Descending
        Swooping
        Flying
        'In order of intended occurrence
    End Enum

    Const PERCHWIDTH As Integer = 9
    Const FLYWIDTH As Integer = 26
    Const FLYHEIGHT As Integer = 15
    Const ATKRANGE As Integer = 100
    'Const SWOOPHEIGHT As Integer = 18

    Public Sub New(ByVal Ceiling As RectangleF, ByVal LeftOffset As Single, ByVal Speed As Single)
        mr = New RectangleF(Ceiling.Left + LeftOffset, Ceiling.Bottom, PERCHWIDTH, FLYHEIGHT)
        state = BatStates.Perched
        spd = Speed
        swoopheight = 16
        hp = 1
    End Sub

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Static dx, dy, dt As Single
        Select Case state
            Case BatStates.Perched
                If R.Right >= mr.Left - ATKRANGE AndAlso R.Left <= mr.Right + ATKRANGE Then
                    goingleft = (R.Left < mr.Left)
                    state = BatStates.Descending
                    mr.Width = FLYWIDTH
                End If
            Case BatStates.Descending
                goingleft = (R.Left < mr.Left)
                mr.Y += spd
                If R.Top < mr.Bottom + swoopheight Then
                    state = BatStates.Flying
                    'System.Diagnostics.Debug.WriteLine("Swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    dy = spd
                End If
                Me.internalcounter += 1
            Case BatStates.Swooping
                dt += 0.25F
                If dt >= 2.0F Then
                    dt = 0.0F
                    If goingleft Then
                        dx -= 0.5F
                    Else
                        dx += 0.5F
                    End If
                    dy -= 0.25F
                    If dy <= 0.0F Then
                        state = BatStates.Flying
                        'System.Diagnostics.Debug.WriteLine("Not-swoop at " & mr.Bottom.ToString() & " with speed " & spd.ToString())
                    End If
                End If
                mr.Offset(dx, dy)
                Me.internalcounter += 1
            Case BatStates.Flying
                dy = 8.0F * Convert.ToSingle(Math.Sin(ang))
                If goingleft Then
                    mr.Offset(-spd * 2.0F, dy)
                Else
                    mr.Offset(spd * 2.0F, dy)
                End If
                ang += 0.1F
                Me.internalcounter += 1
        End Select
        If Me.internalcounter >= 6 Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Right < 0 OrElse hp <= 0
    End Function

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F, P As Single = 0.3F
        Const PERCHFRAME As Single = 26.0F / 113.0F
        Dim F As Single = (Me.internalcounter \ 3)

        If state = BatStates.Perched Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            If Not goingleft Then
                F += 2
            End If

            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return FLYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 2
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(Me.mr.X, Me.mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If state = BatStates.Perched Then
                Return PERCHWIDTH
            Else
                Return FLYWIDTH
            End If
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class LinearBat
    Implements IMonster



    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    'Specific
    Private state As BatStates
    Private spd As Single
    Private hp As Integer

    Private swoopheight As Single

    'Finite state machine.
    Private Enum BatStates
        Perched = 0
        Watching
        Swooping
        Flying
        'In order of intended occurrence
    End Enum

    Const PERCHWIDTH As Integer = 9
    Const FLYWIDTH As Integer = 26
    Const FLYHEIGHT As Integer = 15
    Const ATKRANGE As Integer = 100
    'Const SWOOPHEIGHT As Integer = 18

    Public Sub New(ByVal Ceiling As RectangleF, ByVal LeftOffset As Single, ByVal Speed As Single)
        mr = New RectangleF(Ceiling.Left + LeftOffset, Ceiling.Bottom, PERCHWIDTH, FLYHEIGHT)
        state = BatStates.Perched
        spd = Speed
        swoopheight = 16
        hp = 1
    End Sub

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Static dx, dy, dt As Single
        Select Case state
            Case BatStates.Perched
                If R.Right >= mr.Left - ATKRANGE AndAlso R.Left <= mr.Right + ATKRANGE Then
                    goingleft = (R.Left < mr.Left)
                    state = BatStates.Flying
                    mr.Width = FLYWIDTH
                    dy = 0 'Set to let flying procedure know that we have not set these values yet.
                    dt = R.Left
                End If
            Case BatStates.Flying
                'R.Left - dx is the Speed that the player is moving
                'Vertical distance is R.Top - mr.Top
                'Bat speed is spd
                'Bat makes vertical drop in time of T = (R.Top - mr.Top) / spd
                'Bat must move horizontally to fall on top of player.
                'After time of T, player will have moved a distance of (R.Left - dx) * (R.Top - mr.Top) / spd
                'and the player will be located at R.Left + (R.Left - dx) * (R.Top - mr.Top) / spd
                'The bat must move a distance of R.Left + (R.Left - dx) * (R.Top - mr.Top) / spd - mr.Left
                'in a time of T = (R.Top - mr.Top) / spd
                'Horizontal speed of bat is: 
                ' (R.Left + (R.Left - dx) * (R.Top - mr.Top) / spd - mr.Left)
                ' R.Top - mr.Top / spd
                'or (R.Left - mr.Left) * spd / (R.Top - mr.Top) + R.Left - dx
                If dy = 0 AndAlso R.Top <> mr.Top Then
                    dx = (R.Left - mr.Left) * spd / (R.Top - mr.Top)
                    dy = spd
                End If
                mr.Offset(dx, dy)
                Me.internalcounter += 1

        End Select
        If Me.internalcounter >= 6 Then
            Me.internalcounter = 0
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return mr.Right < 0 OrElse hp <= 0
    End Function

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F, P As Single = 0.3F
        Const PERCHFRAME As Single = 26.0F / 113.0F
        Dim F As Single = (Me.internalcounter \ 3)

        If state = BatStates.Perched Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, 4.0F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            If Not goingleft Then
                F += 2
            End If

            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F, 0.0F, &HFFFFFFFF, F * PERCHFRAME, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F, 2.0F, 0.0F, &HFFFFFFFF, (F + 1) * PERCHFRAME, 1.0F)
        End If
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return FLYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 2
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(Me.mr.X, Me.mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If state = BatStates.Perched Then
                Return PERCHWIDTH
            Else
                Return FLYWIDTH
            End If
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class LocoLeaper
    Implements IMonster

    Private hp As Integer

    Private mr As RectangleF
    Private jumping, attacking, goingleft As Boolean
    Private vx, vy As Single
    Private internalcounter As Integer

    Const MYHEIGHT As Single = 21.0F
    Const SITWIDTH As Single = 23.0F
    Const JUMPWIDTH As Single = 36.0F
    Const XAZ As Single = 150.0F
    Const YAZ As Single = 60.0F

    Public Sub New(ByVal Source As RectangleF, ByVal Leftoffset As Single)
        mr = New RectangleF(Source.Left + Leftoffset, Source.Top - MYHEIGHT, SITWIDTH, MYHEIGHT)
        hp = 4
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If jumping AndAlso goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 1
        ElseIf BY = ShotTypes.ShockWave Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 4
        ElseIf BY = ShotTypes.Stonebullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        ElseIf BY = ShotTypes.Plasma Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        Else
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 2
        End If
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 35
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dir As Single
        If attacking Then
            If jumping Then
                mr.X += vx
                For Each Wall As RectangleF In Q.Walls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Crates
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Glasses
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.CrossStays
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    'Check with each wall.
                    If mr.IntersectsWith(Wall) Then
                        mr.Offset(Wall.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.RTDs
                    'Check with each wall.
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next
                For Each Sw As OpenerSwitch In Q.Pneumos
                    'Check with each wall.
                    If mr.IntersectsWith(Sw.Gate) Then
                        mr.Offset(Sw.Gate.Right - mr.Left, 0)
                        'Move the player to the edge of this wall.
                        goingleft = False
                    End If
                Next


                mr.Y += vy
                vy += 0.5F
                For Each Platform As RectangleF In Q.Platforms
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= Platform.Top AndAlso mr.Bottom - vy - 2 <= Platform.Top Then
                        If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            jumping = False

                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Walls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Extrawalls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Crates
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Groundwalls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.EyeBeams
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.CrossStays
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Glasses
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                If jumping = False Then
                    jumping = False
                    If goingleft Then
                        goingleft = False
                    Else
                        attacking = False
                    End If
                ElseIf vy = 0.0F Then
                    internalcounter += 1
                    If mr.X < R.X Then
                        dir = 1.0F
                    Else
                        dir = -1.0F
                    End If
                    Select Case internalcounter Mod 4
                        Case 0, 2
                            Q.Monsters.Add(New Aquabullet(mr.X + 10, mr.Y + 20, 3.0F * dir + 1.0F, 0.0F, 0.0F, 0.0F, 120))
                        Case 1
                            Q.Monsters.Add(New Aquabullet(mr.X + 10, mr.Y + 20, 3.0F * dir + 1.0F, -3.0F, 0.0F, 0.0F, 120))
                        Case 3
                            Q.Monsters.Add(New Aquabullet(mr.X + 10, mr.Y + 20, 3.0F * dir + 1.0F, 3.0F, 0.0F, 0.0F, 120))
                    End Select
                End If
            Else
                jumping = True
                vy = -8.0F
                If goingleft Then
                    vx = -1.0F
                Else
                    vx = 1.0F
                End If
            End If
        Else
            If R.X - mr.X < XAZ AndAlso mr.X - R.X < XAZ AndAlso R.Y - mr.Y < YAZ AndAlso mr.Y - R.Y < YAZ Then
                attacking = True
                goingleft = True

            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If jumping Then
                Return JUMPWIDTH
            Else
                Return SITWIDTH
            End If
        End Get
    End Property
End Class
Public Class Frogetu
    Implements IMonster

    Private hp As Integer

    Private mr As RectangleF
    Private jumping, goingleft As Boolean
    Private vx, vy As Single
    Private internalcounter As Integer

    Const MYHEIGHT As Single = 21.0F
    Const SITWIDTH As Single = 23.0F
    Const JUMPWIDTH As Single = 36.0F
    Const XAZ As Single = 150.0F
    Const YAZ As Single = 60.0F

    Public Sub New(ByVal Source As RectangleF, ByVal Leftoffset As Single)
        mr = New RectangleF(Source.Left + Leftoffset, Source.Top - MYHEIGHT, SITWIDTH, MYHEIGHT)
        hp = 4
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If jumping AndAlso goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 1
        ElseIf BY = ShotTypes.ShockWave Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 4
        ElseIf BY = ShotTypes.Stonebullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        ElseIf BY = ShotTypes.Plasma Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        Else
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 2
        End If
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 55
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dir As Single
        If jumping Then
            mr.Y += vy
            vy += 0.25F
            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - vy - 2 <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, Platform.Top - mr.Bottom)
                        jumping = False

                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Walls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If vy > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + 2)
                            vy = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If vy > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + 2)
                            vy = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        If vy > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + 2)
                            vy = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Groundwalls
                'Looking at each platform in the platform array.
                If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.
                        mr.Offset(0, wall.Top - mr.Bottom)
                        jumping = False
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.EyeBeams
                If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        If vy > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + 2)
                            vy = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.CrossStays
                If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        If vy > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + 2)
                            vy = 0
                        End If
                    End If
                End If
            Next
            For Each wall As RectangleF In Q.Glasses
                If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                    If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                        If vy > 0 Then
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        Else
                            mr.Offset(0, wall.Bottom - mr.Top + 2)
                            vy = 0
                        End If
                    End If
                End If
            Next
            If vy = 0.0F Then
                If internalcounter Mod 8 = 1 Then
                    Q.Monsters.Add(New DirectBullet(mr.X + 10, mr.Y, R, 0.8F, 160))
                    Q.Monsters.Add(New DirectBullet(mr.X + 10, mr.Y, R, 1.6F, 160))
                    Q.Monsters.Add(New DirectBullet(mr.X + 10, mr.Y, R, 2.4F, 160))
                End If
            End If
        ElseIf R.X - mr.X < XAZ AndAlso mr.X - R.X < XAZ AndAlso R.Y - mr.Y < YAZ AndAlso mr.Y - R.Y < YAZ Then
            jumping = True
            internalcounter += 1
            goingleft = (mr.X > R.X)
            vy = -5.25F

        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If jumping Then
                Return JUMPWIDTH
            Else
                Return SITWIDTH
            End If
        End Get
    End Property
End Class
Public Class BookooLeap
    Implements IMonster

    Private hp As Integer

    Private mr As RectangleF
    Private jumping, goingleft, attacking As Boolean
    Private vx, vy As Single
    Private internalcounter As Integer

    Const MYHEIGHT As Single = 21.0F
    Const SITWIDTH As Single = 23.0F
    Const JUMPWIDTH As Single = 36.0F
    Const XAZ As Single = 350.0F
    Const YAZ As Single = 200.0F

    Public Sub New(ByVal Source As RectangleF, ByVal Leftoffset As Single)
        mr = New RectangleF(Source.Left + Leftoffset, Source.Top - MYHEIGHT, SITWIDTH, MYHEIGHT)
        hp = 4
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If jumping AndAlso goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 1
        ElseIf BY = ShotTypes.ShockWave Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 4
        ElseIf BY = ShotTypes.Stonebullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        ElseIf BY = ShotTypes.Plasma Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        Else
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 2
        End If
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 88
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dir As Single
        If attacking Then
            If jumping Then
                mr.X += vx
                If vx < 0 Then
                    For Each Wall As RectangleF In Q.Walls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Extrawalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Crates
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Groundwalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.EyeBeams
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.CrossStays
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Glasses
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.RTDs
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.Pneumos
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                Else
                    For Each Wall As RectangleF In Q.Walls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Extrawalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Crates
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Groundwalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.EyeBeams
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.CrossStays
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Glasses
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.RTDs
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.Pneumos
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                End If



                mr.Y += vy
                vy += 0.5F
                For Each Platform As RectangleF In Q.Platforms
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= Platform.Top AndAlso mr.Bottom - vy - 2 <= Platform.Top Then
                        If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            jumping = False

                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Walls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Extrawalls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Crates
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.EyeBeams
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.CrossStays
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Groundwalls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Glasses
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                If jumping = False Then
                    jumping = False
                    If goingleft Then
                        goingleft = False
                    Else
                        attacking = False
                    End If
                ElseIf vy = 0.0F Then
                    internalcounter += 1
                    If mr.X < R.X Then
                        dir = 1.0F
                    Else
                        dir = -1.0F
                    End If
                    Select Case internalcounter Mod 4
                        Case 0, 2
                            Q.Monsters.Add(New Aquabullet(mr.X + 10, mr.Y + 20, 3.0F * dir + 1.0F, 0.0F, 0.0F, 0.0F, 120))
                        Case 1
                            Q.Monsters.Add(New Aquabullet(mr.X + 10, mr.Y + 20, 3.0F * dir + 1.0F, -3.0F, 0.0F, 0.0F, 120))
                        Case 3
                            Q.Monsters.Add(New Aquabullet(mr.X + 10, mr.Y + 20, 3.0F * dir + 1.0F, 3.0F, 0.0F, 0.0F, 120))
                    End Select
                End If
            Else
                jumping = True
                vy = -8.0F
                If goingleft Then
                    vx = -6.0F
                Else
                    vx = 6.0F
                End If
            End If
        Else
            If R.X - mr.X < XAZ AndAlso mr.X - R.X < XAZ AndAlso R.Y - mr.Y < YAZ AndAlso mr.Y - R.Y < YAZ Then
                attacking = True
                goingleft = True

            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If jumping Then
                Return JUMPWIDTH
            Else
                Return SITWIDTH
            End If
        End Get
    End Property
End Class

Public Class SalkScorpion
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private state As SalkStates
    Private internalcounter As Integer
    Private fallspeed As Single
    Private le, re As Single
    Private goingleft As Boolean

    Const PERCHWIDTH As Single = 24.0F
    Const PERCHHEIGHT As Single = 38.0F
    Const LANDWIDTH As Single = 30.0F
    Const LANDHEIGHT As Single = 22.0F
    Const GRAVITY As Single = 0.5F

    Const XAZ As Single = 72.0F
    Const YAZ As Single = 180.0F

    Private Enum SalkStates
        Perched
        Falling
        Deploying
        Initializing
        Walking
    End Enum

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, PERCHWIDTH, PERCHHEIGHT)
        hp = 2
        state = SalkStates.Perched
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Single

        Select Case state
            Case SalkStates.Perched
                F = ((internalcounter Mod 32) \ 4)
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.125F * F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.125F * F, 0.296875F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.125F * (F + 1), 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.125F * (F + 1), 0.296875F)
            Case SalkStates.Falling
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.296875F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.625F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.625F, 0.296875F)
            Case SalkStates.Deploying
                F = (internalcounter) \ 4
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.125F * F, 0.296875F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.125F * F, 0.59375F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.125F * (F + 1), 0.296875F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.125F * (F + 1), 0.59375F)
            Case SalkStates.Initializing
                F = (internalcounter) \ 4
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.15625F * F, 0.59375F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.15625F * F, 0.765625F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.15625F * (F + 1), 0.59375F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.15625F * (F + 1), 0.765625F)
            Case SalkStates.Walking
                If goingleft Then
                    F = ((internalcounter \ 2) Mod 2)
                Else
                    F = (2 + ((internalcounter \ 2) Mod 2))
                End If
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.15625F * F, 0.765625F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.15625F * F, 0.9375F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.15625F * (F + 1), 0.765625F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.15625F * (F + 1), 0.9375F)

        End Select
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            If state = SalkStates.Walking OrElse state = SalkStates.Initializing Then
                Return LANDHEIGHT
            Else
                Return PERCHHEIGHT
            End If
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 37
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim Vx, Vy As Single
        Dim ir As RectangleF
        Select Case state
            Case SalkStates.Perched
                internalcounter += 1
                If mr.X - R.X < XAZ AndAlso R.X - mr.X < XAZ AndAlso mr.Y - R.Y < YAZ AndAlso R.Y - mr.Y < YAZ Then
                    state = SalkStates.Falling
                End If
            Case SalkStates.Falling
                fallspeed += GRAVITY
                mr.Offset(0, fallspeed)

                For Each Platform As RectangleF In Q.Platforms
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITY <= Platform.Top Then
                        If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.

                            state = SalkStates.Deploying
                            'Hit the ground, we are not jumping.
                            fallspeed = 0
                            internalcounter = 0

                            le = Platform.Left : re = Platform.Right
                            'Set the endpoints for the platform.
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            'Move the player onto this platform.

                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 1, 0, 0, 0, 240))
                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, 1, 0, 0, 240))
                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -1, 0, 0, 0, 240))
                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, -1, 0, 0, 240))
                            Vx = Convert.ToSingle(Math.Cos(Math.PI / 4))
                            Vy = Convert.ToSingle(Math.Sin(Math.PI / 4))
                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vx, Vy, 0, 0, 240))
                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vy, -Vx, 0, 0, 240))
                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vx, -Vy, 0, 0, 240))
                            Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vy, Vx, 0, 0, 240))
                        End If
                    End If
                Next
                For Each Wall As RectangleF In Q.Walls
                    ir = RectangleF.FromLTRB(mr.Left, mr.Bottom - 4, mr.Right, mr.Bottom)
                    If ir.IntersectsWith(Wall) Then
                        'If player is falling, then the player will land on the wall.
                        state = SalkStates.Deploying
                        'Stop the jump.
                        fallspeed = 0
                        internalcounter = 0


                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, 1, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, -1, 0, 0, 240))
                        Vx = Convert.ToSingle(Math.Cos(Math.PI / 4))
                        Vy = Convert.ToSingle(Math.Sin(Math.PI / 4))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vx, Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vy, -Vx, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vx, -Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vy, Vx, 0, 0, 240))
                    End If
                Next
                For Each Wall As RectangleF In Q.Groundwalls
                    ir = RectangleF.FromLTRB(mr.Left, mr.Bottom - 4, mr.Right, mr.Bottom)
                    If ir.IntersectsWith(Wall) Then
                        'If player is falling, then the player will land on the wall.
                        state = SalkStates.Deploying
                        'Stop the jump.
                        fallspeed = 0
                        internalcounter = 0

                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, 1, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, -1, 0, 0, 240))
                        Vx = Convert.ToSingle(Math.Cos(Math.PI / 4))
                        Vy = Convert.ToSingle(Math.Sin(Math.PI / 4))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vx, Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vy, -Vx, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vx, -Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vy, Vx, 0, 0, 240))
                    End If
                Next
                For Each Wall As RectangleF In Q.Extrawalls
                    ir = RectangleF.FromLTRB(mr.Left, mr.Bottom - 4, mr.Right, mr.Bottom)
                    If ir.IntersectsWith(Wall) Then
                        'If player is falling, then the player will land on the wall.
                        state = SalkStates.Deploying
                        'Stop the jump.
                        fallspeed = 0
                        internalcounter = 0

                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, 1, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, -1, 0, 0, 240))
                        Vx = Convert.ToSingle(Math.Cos(Math.PI / 4))
                        Vy = Convert.ToSingle(Math.Sin(Math.PI / 4))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vx, Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vy, -Vx, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vx, -Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vy, Vx, 0, 0, 240))
                    End If
                Next
                For Each Wall As RectangleF In Q.EyeBeams
                    ir = RectangleF.FromLTRB(mr.Left, mr.Bottom - 4, mr.Right, mr.Bottom)
                    If ir.IntersectsWith(Wall) Then
                        'If player is falling, then the player will land on the wall.
                        state = SalkStates.Deploying
                        'Stop the jump.
                        fallspeed = 0
                        internalcounter = 0

                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, 1, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -1, 0, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, 0, -1, 0, 0, 240))
                        Vx = Convert.ToSingle(Math.Cos(Math.PI / 4))
                        Vy = Convert.ToSingle(Math.Sin(Math.PI / 4))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vx, Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, Vy, -Vx, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vx, -Vy, 0, 0, 240))
                        Q.Monsters.Add(New Aquabullet(mr.X + PERCHWIDTH / 2, mr.Y + PERCHHEIGHT / 2, -Vy, Vx, 0, 0, 240))
                    End If
                Next
            Case SalkStates.Deploying
                internalcounter += 1
                If internalcounter >= 17 Then
                    state = SalkStates.Initializing
                    internalcounter = 0
                    mr.Width = LANDWIDTH
                    mr.Height = LANDHEIGHT
                    mr.Offset(0, PERCHHEIGHT - LANDHEIGHT)
                End If
            Case SalkStates.Initializing
                internalcounter += 1
                If internalcounter >= 17 Then
                    state = SalkStates.Walking
                    internalcounter = 0
                    goingleft = (R.X < mr.X)
                End If
            Case SalkStates.Walking
                internalcounter += 1
                If goingleft Then
                    mr.Offset(-1.5F, 0.0F)
                    If mr.Left <= le Then
                        goingleft = False
                    End If
                Else
                    mr.Offset(1.5F, 0.0F)
                    If mr.Right >= re Then
                        goingleft = True
                    End If
                End If
        End Select
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If state = SalkStates.Walking OrElse state = SalkStates.Initializing Then
                Return LANDWIDTH
            Else
                Return PERCHWIDTH
            End If
        End Get
    End Property
End Class
Public Class Gregbird
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter, hp As Integer
    Private goingleft As Boolean
    Private forwarding As Boolean

    'Specific declarations - may require some intervention.

    Private amplitude As Single, spd As Single, OY As Single

    Const MYWIDTH As Integer = 32, MYHEIGHT As Integer = 26
    Const ATKRANGE As Integer = 350

    Public Sub New(ByVal X As Single, ByVal Y As Single, ByVal Spd As Single, ByVal Amp As Single)
        mr = New RectangleF(X, Y, MYWIDTH, MYHEIGHT)
        Me.spd = Spd
        Me.amplitude = Amp
        OY = Y
        hp = 3
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return (forwarding AndAlso mr.Right < 0) OrElse hp = 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim Vo As Single, Vf As Single, Ho As Single, Hf As Single
        If goingleft Then
            Vo = 0.0F
            Vf = 0.5F
        Else
            Vo = 0.5F
            Vf = 1.0F
        End If
        Select Case (Me.internalcounter >> 3) Mod 4
            Case 0
                Ho = 0.0F
                Hf = 0.333F
            Case 2
                Ho = 0.667F
                Hf = 1.0F
            Case 1, 3
                Ho = 0.333F
                Hf = 0.667F
        End Select
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, Ho, Vo)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, Ho, Vf)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, Hf, Vo)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, Hf, Vf)
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 9
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx As Single
        If forwarding Then
            If goingleft Then
                mr.X -= spd
            Else
                mr.X += spd
            End If
        Else
            dx = mr.X - R.X
            If dx < ATKRANGE AndAlso dx > -ATKRANGE Then
                forwarding = True
            End If
            goingleft = (dx > 0)
        End If
        mr.Y = OY + amplitude * Convert.ToSingle(Math.Sin(Me.internalcounter * 0.025F))
        Me.internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Q.Monsters.Add(New Opeck(mr.X, mr.Y))
        Return True
    End Function
End Class
Public Class Dispatch
    Implements IMonster
    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean
    Private shooting As Boolean
    Private screwed As Boolean
    Private hp As Integer

    Const MYWIDTH As Integer = 35, MYHEIGHT As Integer = 36
    Const TIMEOUT As Integer = 48
    Const RESTTIMEOUT As Integer = 16
    Const SHOTHEIGHT As Single = 11

    Public Sub New(ByVal SourceRect As RectangleF, ByVal LeftOffset As Single)
        mr = New RectangleF(SourceRect.Left + LeftOffset, SourceRect.Top - MYWIDTH, MYWIDTH, MYHEIGHT)
        hp = 3
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        goingleft = (R.Left < mr.Left)
        If screwed Then
            If Me.internalcounter = 0 Then
                hp = 0
                Q.AddExplosion(mr.Left, mr.Top + SHOTHEIGHT)
                'Q.SonicEvents.Add(SonicEventType.Explosion)
            End If
        ElseIf shooting Then
            If Me.internalcounter >= TIMEOUT Then
                shooting = False
                Me.internalcounter = 0
            End If
        Else
            If Me.internalcounter >= RESTTIMEOUT Then
                shooting = True
                Me.internalcounter = 0
                If goingleft Then
                    Q.Monsters.Add(New Bullet(mr.Left, mr.Top + SHOTHEIGHT, -4, 0, 0, 0, 60))
                Else
                    Q.Monsters.Add(New Bullet(mr.Right, mr.Top + SHOTHEIGHT, 4, 0, 0, 0, 60))
                End If
            End If
        End If
        Me.internalcounter += 1
    End Sub

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        If goingleft AndAlso shooting Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
        ElseIf shooting Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
        End If

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 6
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(Me.mr.X, Me.mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Screwer Then
            screwed = True
            internalcounter = 0
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        End If
    End Function
End Class
Public Class Biotsy
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer
    Private goingleft As Boolean

    Private spd, hp As Integer
    Private TopRng, BtmRng As Integer
    Private state As SpiderStates

    Const MYWIDTH As Integer = 24
    Const MYHEIGHT As Integer = 24

    Private Enum SpiderStates
        DropSlow
        RiseSlow
        DropQuick
        RiseQuick
    End Enum


    Public Sub New(ByVal Xo As Integer, ByVal Yo As Integer, ByVal Spd As Integer, ByVal FullHeight As Integer)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        Me.spd = Spd
        hp = 1
        Me.TopRng = Yo
        Me.BtmRng = Yo + FullHeight
        state = SpiderStates.DropSlow
        internalcounter = 0
    End Sub



    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer
        Const N As Single = 1.0F
        F = internalcounter \ 5
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 19
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Select Case state
            Case SpiderStates.DropSlow
                mr.Offset(0.0F, spd)
                If mr.Bottom >= BtmRng Then
                    state = SpiderStates.RiseSlow
                End If
            Case SpiderStates.RiseSlow
                mr.Offset(0.0F, -spd)
                If mr.Top <= TopRng Then
                    state = SpiderStates.DropQuick
                End If
            Case SpiderStates.DropQuick
                mr.Offset(0.0F, spd + 2.0F)
                If mr.Bottom >= BtmRng Then
                    state = SpiderStates.RiseQuick
                End If
            Case SpiderStates.RiseQuick
                mr.Offset(0.0F, -spd - 2.0F)
                If mr.Top <= TopRng Then
                    state = SpiderStates.DropSlow
                End If
        End Select
        internalcounter += 1
        If internalcounter >= 20 Then
            internalcounter = 0
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Vector3(Me.mr.X, Me.mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
End Class
Public Class SurpriseFish
    Implements IMonster
    Private mr As RectangleF
    Private hp As Integer
    Private jumping As Boolean
    Private spd As Single
    Private oy As Single
    Private goingleft As Boolean

    Const MYWIDTH As Single = 30.0F
    Const MYHEIGHT As Single = 20.0F
    Const XAZ As Single = 80.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 8
        oy = Yo
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        If goingleft Then
            If jumping Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.5286F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.5286F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 0.5286F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.5286F, 0.5F)
            End If
        Else
            If jumping Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.5286F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.5286F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0, 0, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0, 0.0F, &HFFFFFFFF, 0.5286F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.5286F, 1.0F)
            End If
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ShockWave Then
            hp -= 8
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            hp -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True

        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 47
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx As Single, dy As Single, ang As Double
        If jumping Then
            mr.Offset(0, spd)
            If mr.Y < R.Y Then
                spd += 0.5F
            End If
            If spd > -8.0F Then
                If spd = 0.0F Then
                    dx = R.X - mr.X
                    dy = R.Y - mr.Y
                    ang = Math.Atan2(dy, dx)
                    Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(1.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 160))
                    Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(1.5 * Math.Cos(ang + 0.25F)), CSng(1.5 * Math.Sin(ang + 0.25F)), 0, 0, 160))
                    Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(1.5 * Math.Cos(ang - 0.25F)), CSng(1.5 * Math.Sin(ang - 0.25F)), 0, 0, 160))
                ElseIf spd > 8.0F Then
                    spd = 8.0F
                End If

            End If
            If mr.Y > oy Then
                mr.Y = oy
                jumping = False
            End If
        Else
            If mr.X - R.X < XAZ AndAlso R.X - mr.X < XAZ Then
                If R.Y < mr.Y Then
                    jumping = True
                    spd = -8.0F
                End If
            End If
        End If
        goingleft = (R.X < mr.X)
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Opeck
    Implements IMonster
    Private mr As RectangleF
    Private oy As Single
    Private spd As Single
    Private upperhalf, goingleft As Boolean
    Private internalcounter As Integer
    Private hp As Integer

    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 16.0F
    Const TURNSPEED As Single = 4.0F
    Const REVERSEGRAV As Single = 0.1F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        oy = Yo
        spd = TURNSPEED
        upperhalf = False
        hp = 1
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F

        If goingleft Then
            If (Me.internalcounter Mod 4) >= 2 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            End If
        Else
            If (Me.internalcounter Mod 4) >= 2 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            End If
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 49
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If upperhalf Then
            spd += REVERSEGRAV
            If spd > TURNSPEED Then
                upperhalf = False
            End If
        Else
            spd -= REVERSEGRAV
            If -spd > TURNSPEED Then
                upperhalf = True
            End If
        End If
        mr.Offset(0, spd)
        Me.internalcounter += 1
        goingleft = (R.X < mr.X)
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class PivotGunner
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private goingleft As Boolean
    Private turncounter, chargecounter, atkcounter, frame As Integer

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal goleft As Boolean)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 2
        goingleft = goleft
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return (hp <= 0)
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        If goingleft Then
            Select Case frame
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                Case 2
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                Case 3
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            End Select
        Else
            Select Case frame
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                Case 2
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                Case 3
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            End Select
        End If


    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 70
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            If atkcounter = 0 Then
                If R.Left > mr.Left Then
                    turncounter += 1
                Else
                    turncounter = 0
                    chargecounter += 1
                    If chargecounter >= 128 Then
                        atkcounter = 1
                        chargecounter = 0
                    End If
                    frame = 0
                End If
            Else
                atkcounter += 1
                If atkcounter = 8 Then
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y + 10.0F, -5.0F, 0.0F, 0.0F, 0.0F, 32, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                ElseIf atkcounter = 16 Then
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y + 10.0F, -5.0F, 0.0F, 0.0F, 0.0F, 24, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                ElseIf atkcounter = 24 Then
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y + 10.0F, -5.0F, 0.0F, 0.0F, 0.0F, 16, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                ElseIf atkcounter = 32 Then
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y + 10.0F, -5.0F, 0.0F, 0.0F, 0.0F, 8, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                    atkcounter = 0
                End If
            End If
            If turncounter = 128 Then
                goingleft = False
                turncounter = 0
                frame = 0
            ElseIf turncounter = 96 Then
                frame = 3
            ElseIf turncounter = 64 Then
                frame = 2
            ElseIf turncounter = 32 Then
                frame = 1
            End If
        Else
            If atkcounter = 0 Then
                If R.Left < mr.Left Then
                    turncounter += 1
                Else
                    turncounter = 0
                    frame = 0
                    chargecounter += 1
                    If chargecounter >= 128 Then
                        atkcounter = 1
                        chargecounter = 0
                    End If
                End If
            Else
                atkcounter += 1
                If atkcounter = 8 Then
                    Q.Monsters.Add(New CustomShot(mr.Right, mr.Y + 10.0F, 4.0F, 0.0F, 0.0F, 0.0F, 40, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                ElseIf atkcounter = 16 Then
                    Q.Monsters.Add(New CustomShot(mr.Right, mr.Y + 10.0F, 4.0F, 0.0F, 0.0F, 0.0F, 32, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                ElseIf atkcounter = 24 Then
                    Q.Monsters.Add(New CustomShot(mr.Right, mr.Y + 10.0F, 4.0F, 0.0F, 0.0F, 0.0F, 24, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                ElseIf atkcounter = 32 Then
                    Q.Monsters.Add(New CustomShot(mr.Right, mr.Y + 10.0F, 4.0F, 0.0F, 0.0F, 0.0F, 16, 10, New Bullet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 200)))
                    atkcounter = 0
                End If
            End If
            If turncounter = 128 Then
                goingleft = True
                turncounter = 0
                frame = 0
            ElseIf turncounter = 96 Then
                frame = 3
            ElseIf turncounter = 64 Then
                frame = 2
            ElseIf turncounter = 32 Then
                frame = 1
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class ArcGunner
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private goingleft As Boolean
    Private turncounter, chargecounter, atkcounter, frame As Integer

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal GoLeft As Boolean)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 2
        goingleft = GoLeft
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return (hp <= 0)
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        If goingleft Then
            Select Case frame
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.0F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.0F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 0.25F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 0.25F, 1.0F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.25F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.25F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 0.5F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 0.5F, 0.5F)
                Case 2
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.5F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.5F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 0.75F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 0.75F, 0.5F)
                Case 3
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.75F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.75F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 1.0F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 1.0F, 0.5F)
            End Select
        Else
            Select Case frame
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.0F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.0F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 0.25F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 0.25F, 1.0F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.25F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.25F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 0.5F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 0.5F, 1.0F)
                Case 2
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.5F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.5F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 0.75F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 0.75F, 1.0F)
                Case 3
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFAAAA, 0.75F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFAAAA, 0.75F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFAAAA, 1.0F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFAAAA, 1.0F, 1.0F)
            End Select
        End If


    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 70
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If goingleft Then
            If atkcounter = 0 Then
                If R.Left > mr.Left Then
                    turncounter += 1
                Else
                    turncounter = 0
                    chargecounter += 1
                    If chargecounter >= 128 Then
                        atkcounter = 1
                        chargecounter = 0
                    End If
                    frame = 0
                End If
            Else
                atkcounter += 1
                If atkcounter = 47 Then
                    atkcounter = 0
                ElseIf atkcounter Mod 4 = 0 AndAlso atkcounter > 8 Then
                    Q.Monsters.Add(New Aquabullet(mr.X, mr.Y + 10, -4.0F, 6.0F, 0.0F, -0.125F, 128))

                End If
            End If
            If turncounter = 128 Then
                goingleft = False
                turncounter = 0
                frame = 0
            ElseIf turncounter = 96 Then
                frame = 3
            ElseIf turncounter = 64 Then
                frame = 2
            ElseIf turncounter = 32 Then
                frame = 1
            End If
        Else
            If atkcounter = 0 Then
                If R.Left < mr.Left Then
                    turncounter += 1
                Else
                    turncounter = 0
                    frame = 0
                    chargecounter += 1
                    If chargecounter >= 128 Then
                        atkcounter = 1
                        chargecounter = 0
                    End If
                End If
            Else
                atkcounter += 1
                If atkcounter = 23 Then
                    atkcounter = 0
                ElseIf atkcounter Mod 4 = 0 AndAlso atkcounter > 8 Then
                    Q.Monsters.Add(New Aquabullet(mr.Right, mr.Y + 10, 4.0F, 6.0F, 0.0F, -0.125F, 128))

                End If
            End If
            If turncounter = 128 Then
                goingleft = True
                turncounter = 0
                frame = 0
            ElseIf turncounter = 96 Then
                frame = 3
            ElseIf turncounter = 64 Then
                frame = 2
            ElseIf turncounter = 32 Then
                frame = 1
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Gentu
    Implements IMonster

    Dim mr As RectangleF
    Dim hp As Integer
    Dim active As Boolean
    Dim state, internalcounter As Integer

    Const MYWIDTH As Single = 80.0F
    Const MYHEIGHT As Single = 64.0F
    Const XAZ As Integer = 350

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 25
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If active Then
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 71
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If active Then
            internalcounter += 1
            If internalcounter = 50 Then
                If state = 0 Then
                    Q.Monsters.Add(New GentuShell(mr.X + 10, mr.Y))
                    state = 1
                Else
                    Q.Monsters.Add(New HomingShot(mr.X + 30, mr.Y, 2.5, 0.1, 120))
                    state = 0
                End If
                internalcounter = 0
            End If
        Else
            If mr.X - R.X <= XAZ Then
                active = True
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class GentuShell
    Implements IMonster
    Dim mr As RectangleF
    Dim hp, timeout As Integer

    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 16.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 1
        timeout = 160
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0 OrElse timeout <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFF00FF00, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFF00FF00, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFF00FF00, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFF00FF00, 1.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 63
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        mr.Offset(-1.5F, -0.25F)
        If timeout = 80 Then
            Q.Monsters.Add(New DirectBullet(mr.X, mr.Y, R, 6, 160))
        End If
        timeout -= 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SummonerFly
    Implements IMonster
    Private mr As RectangleF
    Private hp As Integer
    Private internalcounter As Integer

    Const MYWIDTH As Single = 32.0F
    Const MYHEIGHT As Single = 32.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 3
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        F = (internalcounter \ 12) Mod 4
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFF00, 0.25F * F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFF00, 0.25F * (F + 1), 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        hp -= 1
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 75
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim Vx, Vy As Single
        If internalcounter Mod 256 = 255 Then
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 360))
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 360))
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 360))
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 360))
            Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
            Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 360))
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 360))
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 360))
            Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 360))
            Vx = Convert.ToSingle(8 * Math.Cos(Math.PI / 8))
            Vy = Convert.ToSingle(8 * Math.Sin(Math.PI / 8))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, -0.016525F * Vx, -0.016525F * Vy, 160))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, -0.016525F * Vy, 0.016525F * Vx, 160))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0.016525F * Vx, 0.016525F * Vy, 160))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0.016525F * Vy, -0.016525F * Vx, 160))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, -Vy, -0.016525F * Vx, 0.016525F * Vy, 160))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, Vx, -0.016525F * Vy, -0.016525F * Vx, 160))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, Vy, 0.016525F * Vx, -0.016525F * Vy, 160))
            Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, -Vx, 0.016525F * Vy, 0.016525F * Vx, 160))

        End If
        internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class FoulBeastTentacle
    Implements IMonster
    Private mr As RectangleF
    Private charge, deploy As Integer
    Private H As Single
    Public kill As Boolean
    Public internalcounter As Integer
    Public actuated, deploying, charging As Boolean
    Public Host As FoulBeast

    Const MYWIDTH As Single = 68.0F
    Const MAXHEIGHT As Single = 30.5F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, H)
        H = 0
        charge = -1
        kill = False
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        If Not (deploying Or charging) Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.75F)
        ElseIf deploying Then
            If internalcounter <= 7 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.75F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
            End If
        Else
            If internalcounter Mod 8 < 2 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.25F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.25F)
            ElseIf internalcounter Mod 8 = 4 OrElse internalcounter Mod 8 = 5 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 0.25F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.25F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F)
            End If
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet AndAlso charging Then
            Q.SonicEvents.Add(SonicEventType.ShotRavage)
            Host.charging = False
            charging = False
            Host.deploying = False
            deploying = False
            Host.internalcounter = 0
            internalcounter = 0
            Return True
        Else
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MAXHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 85
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx As Single, dy As Single, ang As Double
        If charging AndAlso internalcounter >= 192 Then
            If internalcounter Mod 8 = 0 Then
                dx = R.X - mr.X
                dy = R.Y - mr.Y
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y, CSng(6.5 * Math.Cos(ang)), CSng(6.5 * Math.Sin(ang)), 0, 0, 160))
                Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y, CSng(6.5 * Math.Cos(ang + 0.125F)), CSng(6.5 * Math.Sin(ang + 0.125F)), 0, 0, 160))
                Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH / 2, mr.Y, CSng(6.5 * Math.Cos(ang - 0.125F)), CSng(6.5 * Math.Sin(ang - 0.125F)), 0, 0, 160))
            End If

        End If
        If deploying Then
            If internalcounter >= 8 Then
                H = 9.5F
            End If
        ElseIf charging Then
            H = MAXHEIGHT
        Else
            H = 0
        End If
        mr.Height = H
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class FoulBeast
    Implements IMonster

    Private mr As RectangleF
    Private hp As Integer
    Private Arm As FoulBeastTentacle
    Private kill As Boolean
    Public internalcounter As Integer
    Public actuated, deploying, charging As Boolean

    Const MYWIDTH As Single = 68.0F
    Const MYHEIGHT As Single = 30.5F
    Const XAZ As Single = 380.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal A As FoulBeastTentacle)
        mr = New RectangleF(Xo, Yo + MYHEIGHT, MYWIDTH, MYHEIGHT)
        hp = 80
        Arm = A
        Arm.Host = Me
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function
    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        If Not (deploying Or charging) Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.75F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
        ElseIf deploying Then
            If internalcounter <= 7 Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            End If
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.25F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.25F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 0.5F)
        End If

    End Sub
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.ShockWave, ShotTypes.Plasma
                hp -= 2
            Case ShotTypes.Aquabullet, ShotTypes.Pearl
                hp -= 1
            Case ShotTypes.Corkwheel
                hp -= 3
            Case ShotTypes.ForceMod
                hp -= 5
            Case ShotTypes.Napalm
                hp -= 4
            Case ShotTypes.Rubberet
                hp -= 2
            Case ShotTypes.Screwer
                hp -= 3
            Case ShotTypes.Stonebullet
                hp -= 15
        End Select
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function
    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property
    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 85
        End Get
    End Property
    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If charging Then
            internalcounter += 1
            If internalcounter = 256 Then
                charging = False
                deploying = False
                internalcounter = 0
            End If
        ElseIf deploying Then
            internalcounter += 1
            If internalcounter = 64 Then
                charging = True
                deploying = False
                internalcounter = 0
            End If
        ElseIf actuated Then
            internalcounter += 1
            If internalcounter >= 512 Then
                deploying = True
                charging = False
                internalcounter = 0
            End If
        ElseIf mr.X - R.X < XAZ AndAlso R.X - mr.X < XAZ Then
            actuated = True
        End If
        If internalcounter Mod 64 = 1 OrElse internalcounter Mod 64 = 21 Then
            Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2.0F, mr.Y + MYHEIGHT, -5, -10, 0, 0, 300))
        End If
        If hp <= 0 Then
            kill = True
            Arm.kill = True
        Else
            Arm.actuated = actuated
            Arm.charging = charging
            Arm.deploying = deploying
            Arm.internalcounter = internalcounter
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class SeaGatelle
    Implements IMonster
    Private mr As RectangleF
    Private internalcounter, hp As Integer
    Private goingleft As Boolean

    Const MYWIDTH As Single = 48.0F
    Const MYHEIGHT As Single = 64.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 8
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 1.0F
        Dim F As Integer
        If goingleft Then
            F = (internalcounter \ 4) Mod 4
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F * F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F * F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F * (F + 1), 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F * (F + 1), 1.0F)
        Else
            F = (internalcounter \ 4) Mod 4
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F * (F + 1), 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F * (F + 1), 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F * F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F * F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.ShockWave, ShotTypes.Aquabullet
                hp -= 1
            Case ShotTypes.Napalm
                hp -= 3
            Case ShotTypes.Rubberet
                hp -= 4
            Case Else
                hp -= 2
        End Select
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 90
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        internalcounter += 1
        If goingleft Then
            If internalcounter Mod 128 = 0 Then
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -2.16F, -5.52F, 0.0F, 0.0F, 150))
            ElseIf internalcounter Mod 128 = 126 Then
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -4.24F, -4.24F, 0.0F, 0.0F, 150))
            ElseIf internalcounter Mod 128 = 124 Then
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -5.52F, -2.16F, 0.0F, 0.0F, 150))
            End If
        Else
            If internalcounter Mod 128 = 0 Then
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 2.16F, -5.52F, 0.0F, 0.0F, 150))
            ElseIf internalcounter Mod 128 = 126 Then
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 4.24F, -4.24F, 0.0F, 0.0F, 150))
            ElseIf internalcounter Mod 128 = 124 Then
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 5.52F, -2.16F, 0.0F, 0.0F, 150))
            End If
        End If
        If internalcounter Mod 128 = 64 Then
            goingleft = (R.X < mr.X)
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BullFrog
    Implements IMonster

    Private hp As Integer

    Private mr As RectangleF
    Private jumping, attacking, goingleft As Boolean
    Private vx, vy As Single
    Private internalcounter As Integer

    Const MYHEIGHT As Single = 21.0F
    Const SITWIDTH As Single = 23.0F
    Const JUMPWIDTH As Single = 36.0F
    Const XAZ As Single = 350.0F
    Const YAZ As Single = 600.0F

    Public Sub New(ByVal Source As RectangleF, ByVal Leftoffset As Single)
        mr = New RectangleF(Source.Left + Leftoffset, Source.Top - MYHEIGHT, SITWIDTH, MYHEIGHT)
        hp = 8
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If jumping AndAlso goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        ElseIf jumping Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        ElseIf goingleft Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.2F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.2F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.7F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.7F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Aquabullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 1
        ElseIf BY = ShotTypes.ShockWave Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 4
        ElseIf BY = ShotTypes.Stonebullet Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        ElseIf BY = ShotTypes.Plasma Then
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 10
        Else
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            hp -= 2
        End If
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 35
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If attacking Then
            If jumping Then
                mr.X += vx
                If vx < 0 Then
                    For Each Wall As RectangleF In Q.Walls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Extrawalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Crates
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Groundwalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.EyeBeams
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.CrossStays
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Glasses
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.RTDs
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.Pneumos
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Right - mr.Left, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                Else
                    For Each Wall As RectangleF In Q.Walls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Extrawalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Crates
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Groundwalls
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.EyeBeams
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.CrossStays
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Wall As RectangleF In Q.Glasses
                        If mr.IntersectsWith(Wall) Then
                            mr.Offset(Wall.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.RTDs
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                    For Each Sw As OpenerSwitch In Q.Pneumos
                        If mr.IntersectsWith(Sw.Gate) Then
                            mr.Offset(Sw.Gate.Left - mr.Right, 0)
                            goingleft = Not goingleft
                            vx = -vx
                        End If
                    Next
                End If


                mr.Y += vy
                vy += 0.5F
                For Each Platform As RectangleF In Q.Platforms
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= Platform.Top AndAlso mr.Bottom - vy - 2 <= Platform.Top Then
                        If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            jumping = False

                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Walls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Extrawalls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Crates
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.EyeBeams
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.CrossStays
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Glasses
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            If vy > 0 Then
                                mr.Offset(0, wall.Top - mr.Bottom)
                                jumping = False
                            Else
                                mr.Offset(0, wall.Bottom - mr.Top + 2)
                                vy = 0
                            End If
                        End If
                    End If
                Next
                For Each wall As RectangleF In Q.Groundwalls
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= wall.Top AndAlso mr.Bottom - vy - 2 <= wall.Top Then
                        If mr.Left < wall.Right AndAlso mr.Right > wall.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            mr.Offset(0, wall.Top - mr.Bottom)
                            jumping = False
                        End If
                    End If
                Next
                For Each C As DataBlock In Q.Conveyors
                    'Looking at each platform in the platform array.
                    If mr.Bottom >= C.Loc.Top AndAlso mr.Bottom - vy - 2 <= C.Loc.Top Then
                        If mr.Left < C.Loc.Right AndAlso mr.Right > C.Loc.Left Then
                            'The player has landed on this platform.  He needs to be updated.
                            'See how many advantages there are to using a Rectangle.
                            mr.Offset(0, C.Loc.Top - mr.Bottom)
                            jumping = False
                        End If
                    End If
                Next
                If jumping = False Then
                    jumping = False
                    If goingleft Then
                        goingleft = False
                    Else
                        attacking = False
                    End If
                ElseIf vy = 0.0F Then
                    internalcounter += 1
                    Q.Monsters.Add(New DirectAcceleratorBullet(mr.X + 0.5F * JUMPWIDTH, mr.Y + 0.5F * MYHEIGHT, R, 0.01F, 240))
                End If
            Else
                jumping = True
                vy = -8.0F
                goingleft = (mr.X > R.X)
                If goingleft Then
                    vx = -2.0F
                Else
                    vx = 2.0F
                End If
            End If
        Else
            If R.X - mr.X < XAZ AndAlso mr.X - R.X < XAZ AndAlso R.Y - mr.Y < YAZ AndAlso mr.Y - R.Y < YAZ Then
                attacking = True
            End If
        End If
        If hp <= 0 Then
            Q.Monsters.Add(New Aquabullet(mr.X + 0.5F * JUMPWIDTH, mr.Y + 0.5F * MYHEIGHT, -4.0F, 0.0F, 0.0F, 0.1F, 140))
            Q.Monsters.Add(New Aquabullet(mr.X + 0.5F * JUMPWIDTH, mr.Y + 0.5F * MYHEIGHT, 4.0F, 0.0F, 0.0F, 0.1F, 140))
            Q.Monsters.Add(New Aquabullet(mr.X + 0.5F * JUMPWIDTH, mr.Y + 0.5F * MYHEIGHT, -2.0F, -3.48F, 0.0F, 0.1F, 140))
            Q.Monsters.Add(New Aquabullet(mr.X + 0.5F * JUMPWIDTH, mr.Y + 0.5F * MYHEIGHT, 2.0F, -3.48F, 0.0F, 0.1F, 140))
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            If jumping Then
                Return JUMPWIDTH
            Else
                Return SITWIDTH
            End If
        End Get
    End Property

End Class
#End Region
#Region "Bosses"
Public Class BossOne
    Implements IMonster

    'Boss one
    'This is the first boss of the game.
    'As for any boss, I must plan out all of the moves that the boss will make
    'and any criteria that the boss might need for executing this move.
    'All bosses are finite state machines... thusly I will need an enumeration to
    'delineate all of the available boss moves.
    'Boss one is Sauitek, who was just recently placed in charge of the Doppel Business District
    '(where zone 1 takes place)
    'The thing to note is how bullet hits are rendered in PFT.
    'The character's bullet causes damage everytime it lands (no invulnerability period).
    'The bad guy's bullet causes an invulnerability period.
    '(INVULNTIME) in gameplay constants is currently 30.
    'Pure melee fighting at point blank causes the MC to score 600 hits.
    'To reduce the number of hits, the boss must move and jump... all bosses will move and jump.
    'Either that or the surroundings must move.
    'This will reduce the hits scored on the boss.
    'The boss must also shoot frequently enough... but not too frequently as to the shots
    'not being avoidable.  Some of the best programmed boss will be able to shoot an array of bullets that
    'requires concentration to dodge - or maybe an all out puzzle in some games.
    'With no further ado, Sauitek's moves.

    Private Enum AtkSetOne
        ThreeShots
        HurdledShots
        SixteenPoint
        RightangleShots
        Chartrusen
        Serpentine
        HomingShot
    End Enum
    '[Three shots] - three shots that head straight for the character.  The character should be able to 
    'jump over all three in one jump
    '[Hurdled shots] - one shot low, one shot high.  Normal jump from the base should not touch the top bullet.
    '[16-point sunburst] - sixteen shots go out, separated from each other by 22.5.
    '[Right angle shots] - shot travels straight ahead and moves up or down when player jumps over or goes under.
    'ideally, one will be shot at ground level, and another in the air.
    '[Chartrusen] - 32 shots go out and stop, then all collectively travel directly towards the player.
    '[Box Shot] - 4 shots travel toward player at same velocity.  Havoc if player is at same level.
    '[Serpentine] - 16 slow moving shots travel toward the player at slow speed.  Timed enough so that
    'it's hard to go between and easier to just walk around.
    '[Homing Shot] - a homing shot (gray bullet)

    Public HP As Integer
    Private mr As RectangleF
    Private mystate As StateSetOne
    Private attackset As AtkSetOne
    Private attacked As Boolean

    Private Ri As Random
    Private internalcounter As Integer

    'Set locations:
    'A:     504 - 320
    'B:     632 - 320
    'C:     504 - 520
    'D:     632 - 520

    Private jumping As Boolean
    Private lookleft As Boolean
    Private le, re As Single
    Private veloc As Single

    Private goleft, goright As Boolean


    'Horizontal:    X = X + movespeed
    'Vertical:      Y = Y + veloc + gravity
    'Vx:            V = V + gravity

    Const ENDPTL As Single = 504.0F
    Const ENDPTT As Single = 320.0F
    Const ENDPTR As Single = 632.0F
    Const ENDPTB As Single = 520.0F
    Const ATDELAY As Integer = 40

    Const MYHEIGHT As Single = 32.0F
    Const MYWIDTH As Single = 16.0F

    Const MOVESPEED As Single = 2.0F
    Const GRAVITY As Single = 0.38F
    'Full jump is 44 (45) ticks. (always round final number up for extra frame sake)


    Const JUMPSPEED As Single = -8.38F
    'Using these formulas
    'dX = ET * 0.38 = T(T + 1) * 0.19 =>  0.19T^2 + 0.19T - dX = 0 = T^2 + T - dx/0.19 = 0
    'T = [-1 + Sqrt(1 + 4dx / 0.19)] / 2
    'jump height ~ 96

    Private Enum StateSetOne
        AtA
        AtB
        AtC
        AtD
        FromAtoB
        'Jump from 320 to 368 takes 22 + 27 = 49 => round up to 50
        '368 to 320 is 44 - Formula = 44 - 15 = 29 => rounding up to 30
        'Total time for B to A is 50 + 30 = 80
        FromAtoC
        'Walking from 632 to 672 at speed 2 will take 40 / 2 = 20
        'Fall from 320 to 400 takes 20 => rounding up to 21
        'Walking while falling from 672 to 590 at speed 2 will take 82 / 2 = 41
        'Fall from 472 to 520 takes 16 => rounding up to 17
        'Walking from 590 to 632 at speed 2 will take 42 / 2 = 21
        'Total time for B to D is 20 + 21 + 41 + 17 + 21 = 120
        FromAtoD
        'walking from 632 to 606 at speed 2 will take 26 / 2 = 13
        'fall from 320 to 368 takes 15 => rounding up to 16
        'walking from 606 to 642 at speed 2 takes 36 / 2 = 18
        'falling from 368 to 472 takes 23 => round up to 24
        'walk from 620 to 504 at speed 2 will take 116 / 2 = 58
        'Total time fo B to C is 13 + 16 + 18 + 24 + 58 = 129
        FromBtoA
        'Jump from 320 to 368 takes 22 + 27 = 49 => round up to 50
        '368 to 320 is 44 - Formula = 44 - 15 = 29 => rounding up to 30
        'Total time for B to A is 50 + 30 = 80
        FromBtoC
        'walking from 632 to 606 at speed 2 will take 26 / 2 = 13
        'fall from 320 to 368 takes 15 => rounding up to 16
        'walking from 606 to 642 at speed 2 takes 36 / 2 = 18
        'falling from 368 to 472 takes 23 => round up to 24
        'walk from 620 to 504 at speed 2 will take 116 / 2 = 58
        'Total time fo B to C is 13 + 16 + 18 + 24 + 58 = 129
        FromBtoD
        'Walking from 632 to 672 at speed 2 will take 40 / 2 = 20
        'Fall from 320 to 400 takes 20 => rounding up to 21
        'Walking while falling from 672 to 590 at speed 2 will take 82 / 2 = 41
        'Fall from 472 to 520 takes 16 => rounding up to 17
        'Walking from 590 to 632 at speed 2 will take 42 / 2 = 21
        'Total time for B to D is 20 + 21 + 41 + 17 + 21 = 120
        FromCtoA
        '520 to 472 is 44 - Formula = 44 - 15 = 29 => rounding up to 30
        '472 to 400 is 44 - Formula = 44 - 18 - 26 -> rounding up to 27
        '400 to 320 is 44 - Formula = 44 - 20 = 24 => rounding up to 25
        'Total time foor D to B is 30 + 27 + 25 = 82
        FromCtoB
        '520 to 440 is 25
        '440 to 368 is 44 - Formula = 44 - 18 = 26 -> rounding up to 27
        '368 to 320 is 44 - Formula = 44 - 15 = 29 => rounding up to 30
        'Total time for D to A is 25 + 27 + 30 = 82
        FromCtoD
        'Walking from 632 to 504 at movespeed 2 will take 128 / 2 = 64
        FromDtoA
        '520 to 440 is 25
        '440 to 368 is 44 - Formula = 44 - 18 = 26 -> rounding up to 27
        '368 to 320 is 44 - Formula = 44 - 15 = 29 => rounding up to 30
        'Total time for D to A is 25 + 27 + 30 = 82
        FromDtoB
        '520 to 472 is 44 - Formula = 44 - 15 = 29 => rounding up to 30
        '472 to 400 is 44 - Formula = 44 - 18 - 26 -> rounding up to 27
        '400 to 320 is 44 - Formula = 44 - 20 = 24 => rounding up to 25
        'Total time foor D to B is 30 + 27 + 25 = 82
        FromDtoC
        'Walking from 632 to 504 at movespeed 2 will take 128 / 2 = 64
    End Enum


    Public Sub New(ByVal X As Single, ByVal Y As Single)
        mr = New RectangleF(X, Y, MYWIDTH, MYHEIGHT)
        HP = 40
        mystate = StateSetOne.AtD
        Ri = New Random
        internalcounter = ATDELAY
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return HP <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 2.0F
        Dim F As Single = 0.0F
        If jumping Then
            If Me.lookleft Then
                F = 8.0F
            Else
                F = 9.0F
            End If
            If attacked Then
                F += 20.0F
            End If
        ElseIf lookleft Then
            F = (internalcounter \ 4)
            If attacked Then
                F += 20.0F
            End If
        Else
            F = 4.0F + (internalcounter \ 4)  '10 to convert 0-9 into 0.0-0.9, 2 to expand the frame time.
            If attacked Then
                F += 20.0F
            End If
        End If

        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &H0, F / 32.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F / 32.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.Stonebullet Then
            HP -= 40
        Else
            HP -= 1
        End If
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 30
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.IntersectsWith(mr)
    End Function

    Private Sub Jump()
        veloc = JUMPSPEED
        jumping = True
    End Sub
    Private Sub Attack(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite)
        Dim X As Integer
        Dim A, B, C As Microsoft.DirectX.Vector2
        Dim Vx, Vy As Single
        If internalcounter = ATDELAY - 1 Then
            X = Ri.Next(7)
            Select Case X
                Case 0
                    attackset = AtkSetOne.Chartrusen
                Case 1
                    attackset = AtkSetOne.HomingShot
                Case 2
                    attackset = AtkSetOne.HurdledShots
                Case 3
                    attackset = AtkSetOne.RightangleShots
                Case 4
                    attackset = AtkSetOne.Serpentine
                Case 5
                    attackset = AtkSetOne.SixteenPoint
                Case 6
                    attackset = AtkSetOne.ThreeShots
            End Select
        ElseIf internalcounter = 38 Then
            Select Case attackset
                Case AtkSetOne.Chartrusen
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                    Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                    Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, -Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, -Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                    attacked = True
                Case AtkSetOne.HomingShot
                    Q.Monsters.Add(New HomingShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 3, 0.08, 100))
                    attacked = True
                Case AtkSetOne.HurdledShots
                    A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                    B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                    C = Microsoft.DirectX.Vector2.Subtract(A, B)
                    C.Normalize()
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4 * C.X, 4 * C.Y, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y - 100 + MYHEIGHT / 2, 4 * C.X, 4 * C.Y, 0, 0, 60))
                    attacked = True
                Case AtkSetOne.RightangleShots
                    If R.Left > mr.Left Then
                        Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4.0F, 0.0F, 0.0F, 0.0F, 180))
                    Else
                        Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4.0F, 0.0F, 0.0F, 0.0F, 180))
                    End If
                    attacked = True
                Case AtkSetOne.Serpentine
                    A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                    B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                    C = Microsoft.DirectX.Vector2.Subtract(A, B)
                    C.Normalize()
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4 * C.X, 4 * C.Y, 0, 0, 60))
                    attacked = True
                Case AtkSetOne.SixteenPoint
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 60))
                    Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                    Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 60))
                    Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                    Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, -Vy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, Vx, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, Vy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, -Vx, 0, 0, 60))
                    attacked = True
                Case AtkSetOne.ThreeShots
                    A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                    B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                    C = Microsoft.DirectX.Vector2.Subtract(A, B)
                    C.Normalize()
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4 * C.X, 4 * C.Y, 0, 0, 60))
                    attacked = True
            End Select
        ElseIf internalcounter = 28 Then
            If attackset = AtkSetOne.Serpentine OrElse attackset = AtkSetOne.ThreeShots Then
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                C.Normalize()
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4 * C.X, 4 * C.Y, 0, 0, 60))
                attacked = True
            End If

        ElseIf internalcounter = 18 Then

            If attackset = AtkSetOne.Serpentine OrElse attackset = AtkSetOne.ThreeShots Then
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                C.Normalize()
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4 * C.X, 4 * C.Y, 0, 0, 60))
                attacked = True
            End If
        ElseIf attackset = AtkSetOne.Serpentine AndAlso internalcounter Mod 2 = 0 AndAlso internalcounter >= 8 Then
            If attackset = AtkSetOne.Serpentine OrElse attackset = AtkSetOne.ThreeShots Then
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                C.Normalize()
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4 * C.X, 4 * C.Y, 0, 0, 60))
                attacked = True
            End If

        End If
    End Sub
    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim X As Integer
        Dim RV As AutoSuiteReturn
        attacked = False
        Select Case mystate
            Case StateSetOne.AtD
                If internalcounter <= 0 Then
                    X = Ri.Next(3)
                    If X = 0 Then
                        mystate = StateSetOne.FromDtoA
                        internalcounter = 92
                    ElseIf X = 1 Then
                        mystate = StateSetOne.FromDtoB
                        internalcounter = 92
                    Else
                        mystate = StateSetOne.FromDtoC
                        internalcounter = 64
                    End If
                Else
                    Attack(R, Q)
                End If
            Case StateSetOne.AtC
                If internalcounter <= 0 Then
                    X = Ri.Next(3)
                    If X = 0 Then
                        mystate = StateSetOne.FromCtoA
                        internalcounter = 92
                    ElseIf X = 1 Then
                        mystate = StateSetOne.FromCtoB
                        internalcounter = 92
                    Else
                        mystate = StateSetOne.FromCtoD
                        internalcounter = 64
                    End If
                Else
                    Attack(R, Q)
                End If
            Case StateSetOne.AtB
                If internalcounter <= 0 Then
                    X = Ri.Next(3)
                    If X = 0 Then
                        mystate = StateSetOne.FromBtoA
                        internalcounter = 90
                    ElseIf X = 1 Then
                        mystate = StateSetOne.FromBtoC
                        internalcounter = 141
                    Else
                        mystate = StateSetOne.FromBtoD
                        internalcounter = 120
                    End If
                Else
                    Attack(R, Q)
                End If
            Case StateSetOne.AtA
                If internalcounter <= 0 Then
                    X = Ri.Next(3)
                    If X = 0 Then
                        mystate = StateSetOne.FromAtoB
                        internalcounter = 90
                    ElseIf X = 1 Then
                        mystate = StateSetOne.FromAtoC
                        internalcounter = 120
                    Else
                        mystate = StateSetOne.FromAtoD
                        internalcounter = 141
                    End If
                Else
                    Attack(R, Q)
                End If
            Case StateSetOne.FromAtoB
                If internalcounter = 89 Then
                    goright = True
                    Jump()
                ElseIf internalcounter = 39 Then
                    Jump()
                ElseIf internalcounter = 25 Then
                    goright = False
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtB
                    mr.X = ENDPTR
                    mr.Y = ENDPTT
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromBtoA
                If internalcounter = 89 Then
                    goleft = True
                    Jump()
                ElseIf internalcounter = 39 Then
                    Jump()
                ElseIf internalcounter = 25 Then
                    goleft = False
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtA
                    mr.X = ENDPTL
                    mr.Y = ENDPTT
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromAtoC       '120
                If internalcounter = 119 Then
                    goleft = True
                ElseIf internalcounter = 98 Then
                    goleft = False
                ElseIf internalcounter = 79 Then
                    goright = True
                ElseIf internalcounter = 37 Then
                    goright = False
                ElseIf internalcounter = 20 Then
                    goleft = True
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtC
                    mr.X = ENDPTL
                    mr.Y = ENDPTB
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromBtoD
                If internalcounter = 119 Then
                    goright = True
                ElseIf internalcounter = 98 Then
                    goright = False
                ElseIf internalcounter = 79 Then
                    goleft = True
                ElseIf internalcounter = 37 Then
                    goleft = False
                ElseIf internalcounter = 20 Then
                    goright = True
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtD
                    mr.X = ENDPTR
                    mr.Y = ENDPTB
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromAtoD
                If internalcounter = 140 Then
                    goright = True
                ElseIf internalcounter = 126 Then
                    goright = False
                ElseIf internalcounter = 114 Then
                    goleft = True
                ElseIf internalcounter = 93 Then
                    goleft = False
                ElseIf internalcounter = 71 Then
                    goright = True
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtD
                    mr.X = ENDPTR
                    mr.Y = ENDPTB
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromBtoC
                If internalcounter = 140 Then
                    goleft = True
                ElseIf internalcounter = 126 Then
                    goleft = False
                ElseIf internalcounter = 114 Then
                    goright = True
                ElseIf internalcounter = 93 Then
                    goright = False
                ElseIf internalcounter = 71 Then
                    goleft = True
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtC
                    mr.X = ENDPTL
                    mr.Y = ENDPTB
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromDtoC       '64
                goleft = True
                If internalcounter = 63 Then
                    Jump()
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtC
                    mr.X = ENDPTL
                    mr.Y = ENDPTB
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromDtoB       '82
                If internalcounter = 91 Then
                    Jump()
                ElseIf internalcounter = 61 Then
                    Jump()
                    goright = True
                ElseIf internalcounter = 41 Then
                    goright = False
                ElseIf internalcounter = 34 Then
                    Jump()
                    goleft = True
                ElseIf internalcounter = 14 Then
                    goleft = False
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    'jumping = False
                    mystate = StateSetOne.AtB
                    mr.X = ENDPTR
                    mr.Y = ENDPTT
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromDtoA
                If internalcounter = 91 Then
                    Jump()
                    goright = True
                ElseIf internalcounter = 87 Then
                    goright = False
                ElseIf internalcounter = 66 Then
                    Jump()
                    goleft = True
                ElseIf internalcounter = 39 Then
                    Jump()
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtA
                    mr.X = ENDPTL
                    mr.Y = ENDPTT
                    internalcounter = ATDELAY
                End If

            Case StateSetOne.FromCtoD       '64
                goright = True
                If internalcounter = 63 Then
                    Jump()
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtD
                    mr.X = ENDPTR
                    mr.Y = ENDPTB
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromCtoB
                If internalcounter = 91 Then
                    Jump()
                    goleft = True
                ElseIf internalcounter = 87 Then
                    goleft = False
                ElseIf internalcounter = 66 Then
                    Jump()
                    goright = True
                ElseIf internalcounter = 39 Then
                    Jump()
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    jumping = False
                    mystate = StateSetOne.AtB
                    mr.X = ENDPTR
                    mr.Y = ENDPTT
                    internalcounter = ATDELAY
                End If
            Case StateSetOne.FromCtoA       '82
                If internalcounter = 91 Then
                    Jump()
                ElseIf internalcounter = 61 Then
                    Jump()
                    goleft = True
                ElseIf internalcounter = 41 Then
                    goleft = False
                ElseIf internalcounter = 34 Then
                    Jump()
                    goright = True
                ElseIf internalcounter = 14 Then
                    goright = False
                ElseIf internalcounter <= 0 Then
                    goleft = False
                    goright = False
                    'jumping = False
                    mystate = StateSetOne.AtA
                    mr.X = ENDPTL
                    mr.Y = ENDPTT
                    internalcounter = ATDELAY
                End If
        End Select


        If goleft Then
            mr.Offset(-MOVESPEED, 0)

            If mr.Right < le AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                End If
            Next
        End If
        If goright Then
            mr.Offset(MOVESPEED, 0)

            If mr.Left > re AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each wall As RectangleF In Q.Walls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                End If
            Next
        End If
        If jumping Then
            mr.Offset(0, veloc + GRAVITY)
            veloc += GRAVITY
            mr.Y = Convert.ToSingle(Math.Round(mr.Top * 1000.0F, 0)) / 1000.0F

            RV = LevelSuite.PlatformJump(Q.Platforms, mr, veloc, GRAVITY * 1.5F)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                End If
            End If

            RV = LevelSuite.WallJump(Q.Walls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
        End If


        internalcounter -= 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BossTwo
    Implements IMonster
    Public HP As Integer
    Private Ri As Random
    Private internalcounter, frame As Integer
    Private goingleft As Boolean
    Private state As StateSetTwo
    Private atk As AtkSetTwo

    Private Enum StateSetTwo
        OverheadFlying
        Descending
        OpeningDoor
        Attacking
        ClosingDoor
        Ascending
        FreeFlying
    End Enum
    Private Enum AtkSetTwo
        Spray
        GridBlast
        QuickNape
        Shabom
        HomingShot
        SlowWave
        RAShots
        Boomerang
    End Enum

    Private mr As RectangleF
    Private atksource As PointF

    Const MYHEIGHT As Single = 64.0F
    Const MYWIDTH As Single = 64.0F

    Const LANDINGTOP As Single = 376.0F
    Const FLYINGTOP As Single = 120.0F
    Const LEFTEND As Single = 416.0F
    Const RIGHTEND As Single = 1400.0F

    Const DIVISIONHP As Integer = 60
    Const MAXHP As Integer = 3 * DIVISIONHP
    Const MIDHP As Integer = DIVISIONHP + DIVISIONHP

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        HP = MAXHP
        Ri = New Random
        state = StateSetTwo.ClosingDoor
        goingleft = True
    End Sub

    Private Sub Attack(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim X As Integer
        Dim Vx, Vy As Single, Rd As Double
        If internalcounter = 1 Then
            X = Ri.Next(8)
            Select Case X
                Case 0
                    atk = AtkSetTwo.Spray
                Case 1
                    atk = AtkSetTwo.Boomerang
                Case 2
                    atk = AtkSetTwo.GridBlast
                Case 3
                    atk = AtkSetTwo.HomingShot
                Case 4
                    atk = AtkSetTwo.QuickNape
                Case 5
                    atk = AtkSetTwo.RAShots
                Case 6
                    atk = AtkSetTwo.Shabom
                Case 7
                    atk = AtkSetTwo.SlowWave
            End Select
        ElseIf internalcounter = 2 Then
            Select Case atk
                Case AtkSetTwo.Boomerang
                    If goingleft Then
                        Q.Monsters.Add(New CustomShot(atksource.X, atksource.Y, -3.0F, -3.0F, 0.0F, 0.1F, 60, 0, New Bullet(0, 0, 3.0F, 3.0F, 0.0F, -0.1F, 60)))
                    Else
                        Q.Monsters.Add(New CustomShot(atksource.X, atksource.Y, 3.0F, -3.0F, 0.0F, 0.1F, 60, 0, New Bullet(0, 0, -3.0F, 3.0F, 0.0F, -0.1F, 60)))
                    End If
                Case AtkSetTwo.GridBlast
                    If goingleft Then
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -0.8F, 0.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -1.6F, 0.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -2.4F, 0.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -0.8F, 1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -1.6F, 1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -2.4F, 1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -0.8F, -1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -1.6F, -1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -2.4F, -1.0F, 0.0F, 0.0F, 200))
                    Else
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 0.8F, 0.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 1.6F, 0.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 2.4F, 0.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 0.8F, 1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 1.6F, 1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 2.4F, 1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 0.8F, -1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 1.6F, -1.0F, 0.0F, 0.0F, 200))
                        Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, 2.4F, -1.0F, 0.0F, 0.0F, 200))
                    End If
                Case AtkSetTwo.HomingShot
                    Q.Monsters.Add(New HomingShot(atksource.X, atksource.Y, 5.0F, 0.04F, 180))

                Case AtkSetTwo.QuickNape
                    If goingleft Then
                        Q.Monsters.Add(New Napalm(atksource.X, atksource.Y, -5.0F, -0.125F, 0.0F, 0.0F, 100))
                    Else
                        Q.Monsters.Add(New Napalm(atksource.X, atksource.Y, 5.0F, -0.125F, 0.0F, 0.0F, 100))
                    End If
                Case AtkSetTwo.RAShots
                    If goingleft Then
                        Q.Monsters.Add(New RAShot(atksource.X, atksource.Y, -3.0F, -3.0F, 0.0F, 0.0F, 400))
                    Else
                        Q.Monsters.Add(New RAShot(atksource.X, atksource.Y, 3.0F, -3.0F, 0.0F, 0.0F, 400))
                    End If
                Case AtkSetTwo.Shabom
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 320))
                    Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                    Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 320))
                    Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                    Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, -Vy, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, Vx, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, Vy, 0, 0, 320))
                    Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, -Vx, 0, 0, 320))
                Case AtkSetTwo.SlowWave
                    Rd = 2 * Ri.NextDouble() - 1
                    Vx = Convert.ToSingle(Math.Cos(Rd))
                    Vy = Convert.ToSingle(Math.Sin(Rd))
                    Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, Vx, Vy, 0.0F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -Vx, Vy, 0.0F, 0.0F, 250))
                Case AtkSetTwo.Spray
                    If goingleft Then
                        Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, -2.0F, 0.0F, 0.0F, 0.2F, 300))
                    Else
                        Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, 2.0F, 0.0F, 0.0F, 0.2F, 300))
                    End If
            End Select

        ElseIf internalcounter = 8 Then
            Select Case atk
                Case AtkSetTwo.HomingShot
                    Q.Monsters.Add(New HomingShot(atksource.X, atksource.Y, 3.0F, 0.04F, 140))
                Case AtkSetTwo.QuickNape
                    If goingleft Then
                        Q.Monsters.Add(New Napalm(atksource.X, atksource.Y, -0.9F, -0.125F, 0.0F, 0.0F, 116))
                    Else
                        Q.Monsters.Add(New Napalm(atksource.X, atksource.Y, 0.9F, -0.125F, 0.0F, 0.0F, 116))
                    End If
                Case AtkSetTwo.RAShots
                    If goingleft Then
                        Q.Monsters.Add(New RAShot(atksource.X, atksource.Y, -3.0F, -2.0F, 0.0F, 0.0F, 400))
                    Else
                        Q.Monsters.Add(New RAShot(atksource.X, atksource.Y, 3.0F, -2.0F, 0.0F, 0.0F, 400))
                    End If
                Case AtkSetTwo.Spray
                    If goingleft Then
                        Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, -2.0F, -0.2F * (internalcounter - 2), 0.0F, 0.2F, 300))
                    Else
                        Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, 2.0F, -0.2F * (internalcounter - 2), 0.0F, 0.2F, 300))
                    End If
            End Select
        ElseIf internalcounter = 14 Then
            Select Case atk
                Case AtkSetTwo.HomingShot
                    Q.Monsters.Add(New HomingShot(atksource.X, atksource.Y, 1.0F, 0.1F, 250))
                Case AtkSetTwo.QuickNape
                    If goingleft Then
                        Q.Monsters.Add(New Napalm(atksource.X, atksource.Y, -3.0F, -2.4F, 0.0F, 0.0F, 90))
                    Else
                        Q.Monsters.Add(New Napalm(atksource.X, atksource.Y, 3.0F, -2.4F, 0.0F, 0.0F, 90))
                    End If
                Case AtkSetTwo.RAShots
                    If goingleft Then
                        Q.Monsters.Add(New RAShot(atksource.X, atksource.Y, -3.0F, -1.0F, 0.0F, 0.0F, 400))
                    Else
                        Q.Monsters.Add(New RAShot(atksource.X, atksource.Y, 3.0F, -1.0F, 0.0F, 0.0F, 400))
                    End If
                Case AtkSetTwo.Spray
                    If goingleft Then
                        Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, -2.0F, -0.2F * (internalcounter - 2), 0.0F, 0.2F, 300))
                    Else
                        Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, 2.0F, -0.2F * (internalcounter - 2), 0.0F, 0.2F, 300))
                    End If
            End Select
        ElseIf internalcounter < 32 Then
            If atk = AtkSetTwo.Spray Then
                If goingleft Then
                    Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, -2.0F, -0.2F * (internalcounter - 2), 0.0F, 0.2F, 300))
                Else
                    Q.Monsters.Add(New Aquabullet(atksource.X, atksource.Y, 2.0F, -0.2F * (internalcounter - 2), 0.0F, 0.2F, 300))
                End If
            ElseIf atk = AtkSetTwo.SlowWave Then
                Rd = 2 * Ri.NextDouble() - 1
                Vx = Convert.ToSingle(Math.Cos(Rd))
                Vy = Convert.ToSingle(Math.Sin(Rd))
                Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, Vx, Vy, 0.0F, 0.0F, 600))
                Q.Monsters.Add(New Bullet(atksource.X, atksource.Y, -Vx, Vy, 0.0F, 0.0F, 600))
            End If
        End If
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return HP <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = internalcounter Mod 4
        Const N As Single = 2.0F
        If goingleft Then
            If state = StateSetTwo.ClosingDoor OrElse state = StateSetTwo.OpeningDoor Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * frame)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * (frame + 1))
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.25F * frame)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 0.25F * (frame + 1))
            ElseIf state = StateSetTwo.Attacking Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.25F * F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, 0.0F, 0.25F * (F + 1))
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * (F + 1))
            End If
        Else
            If state = StateSetTwo.ClosingDoor OrElse state = StateSetTwo.OpeningDoor Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.25F * frame)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, 1.0F, 0.25F * (frame + 1))
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * frame)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * (frame + 1))
            ElseIf state = StateSetTwo.Attacking Then
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.75F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.75F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Else
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, 0.5F, 0.25F * (F + 1))
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.25F * F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 0.0F, 0.25F * (F + 1))
            End If
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.ShockWave Then
            HP -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Plasma Then
            HP -= 20
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Aquabullet Then
            HP -= 3
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Rubberet Then
            HP -= 10
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Corkwheel Then
            HP -= 5
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ForceMod Then
            HP -= 6
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Screwer Then
            HP -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Pearl Then
            HP -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True
        End If
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 44
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Select Case state
            Case StateSetTwo.Attacking
                internalcounter += 1
                Attack(R, Q)
                If internalcounter >= 32 Then
                    state = StateSetTwo.ClosingDoor
                    internalcounter = 0
                End If
            Case StateSetTwo.ClosingDoor
                internalcounter += 1
                If internalcounter <= 12 Then
                    frame = 3
                ElseIf internalcounter <= 18 Then
                    frame = 2
                ElseIf internalcounter <= 22 Then
                    frame = 1
                Else
                    frame = 0
                End If
                If internalcounter >= 24 Then
                    state = StateSetTwo.Ascending
                    internalcounter = 0
                End If
            Case StateSetTwo.OpeningDoor
                internalcounter += 1
                If internalcounter <= 2 Then
                    frame = 0
                ElseIf internalcounter <= 6 Then
                    frame = 1
                ElseIf internalcounter <= 12 Then
                    frame = 2
                Else
                    frame = 3
                End If
                If internalcounter >= 24 Then
                    state = StateSetTwo.Attacking
                    internalcounter = 0
                    If goingleft Then
                        atksource = New PointF(mr.X, mr.Y + 30.0F)
                    Else
                        atksource = New PointF(mr.Right, mr.Y + 30.0F)
                    End If
                End If
            Case StateSetTwo.Ascending
                internalcounter += 1
                mr.Offset(0.0F, -8.0F)
                If internalcounter >= 32 Then
                    If HP <= DIVISIONHP Then
                        state = StateSetTwo.FreeFlying
                        internalcounter = 0
                        mr.Y = FLYINGTOP
                        goingleft = (R.X - 56.0F < mr.X)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2.0F, mr.Y + MYHEIGHT / 2.0F, -3.0F, 6.0F, 0.0F, 0.0F, 80))
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2.0F, mr.Y + MYHEIGHT / 2.0F, 3.0F, 6.0F, 0.0F, 0.0F, 80))
                    ElseIf HP <= MIDHP Then
                        state = StateSetTwo.FreeFlying
                        internalcounter = 0
                        mr.Y = FLYINGTOP
                        goingleft = (R.X - 56.0F < mr.X)

                    Else
                        state = StateSetTwo.OverheadFlying
                        internalcounter = 0
                        mr.Y = FLYINGTOP
                    End If
                End If
            Case StateSetTwo.Descending
                internalcounter += 1
                mr.Offset(0.0F, 8.0F)
                If internalcounter >= 32 Then
                    state = StateSetTwo.OpeningDoor
                    internalcounter = 0
                    mr.Y = LANDINGTOP
                End If
            Case StateSetTwo.OverheadFlying
                If goingleft Then
                    mr.Offset(-4.0F, 0.0F)
                    If mr.X <= LEFTEND Then
                        goingleft = False
                        state = StateSetTwo.Descending
                        mr.X = LEFTEND
                        internalcounter = 0
                    End If
                Else
                    mr.Offset(4.0F, 0.0F)
                    If mr.X >= RIGHTEND Then
                        goingleft = True
                        state = StateSetTwo.Descending
                        mr.X = RIGHTEND
                        internalcounter = 0
                    End If
                End If
            Case StateSetTwo.FreeFlying
                If goingleft Then
                    mr.Offset(-4.0F, 0.0F)
                    If mr.X <= LEFTEND OrElse R.X - 28.0F > mr.X Then
                        goingleft = False
                        state = StateSetTwo.Descending
                        internalcounter = 0
                    End If
                Else
                    mr.Offset(4.0F, 0.0F)
                    If mr.X >= RIGHTEND OrElse R.X - 28.0F < mr.X Then
                        goingleft = True
                        state = StateSetTwo.Descending
                        internalcounter = 0
                    End If
                End If
        End Select
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BossThree
    Implements IMonster
    Private mr As RectangleF
    Private Ri As Random
    Private internalcounter As Integer
    Public HP As Integer
    Private goleft, goright, jumping, lookleft, attacked As Boolean
    Private veloc, le, re As Single

    Private Enum Mystates
        Waiting
        JumpingAttack
        RushAttack
        PatientAttack
    End Enum
    Private Enum AtkSetThree
        SpreadShot
        AdjacentShots
        ReverseCharge
        SlowWheel
        FourLine
        VertigoBomb
        RopeFigure
        SixteenShot
        CWStar
        CCWStar
        RisingAttack
        Gutsy
    End Enum

    Private state As Mystates
    Private atk As AtkSetThree

    Const JUMPSPEED As Single = -8.38F
    Const MOVESPEED As Single = 2.0F
    Const GRAVITY As Single = 0.38F
    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 32.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F
    'Full jump is 44 (45) ticks. (always round final number up for extra frame sake)
    Public Sub New(ByVal X As Single, ByVal Y As Single)
        mr = New RectangleF(X, Y, MYWIDTH, MYHEIGHT)
        HP = 200
        Ri = New Random
        state = Mystates.Waiting
        internalcounter = 0
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 2.0F
        Dim F As Single = 0.0F
        If jumping Then
            If Me.lookleft Then
                F = 8.0F
            Else
                F = 9.0F
            End If
            If attacked Then
                F += 20.0F
            End If
        ElseIf lookleft Then
            F = ((internalcounter Mod 16) \ 4)
            If attacked Then
                F += 20.0F
            End If
        Else
            F = 4.0F + ((internalcounter Mod 16) \ 4)  '10 to convert 0-9 into 0.0-0.9, 2 to expand the frame time.
            If attacked Then
                F += 20.0F
            End If
        End If
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F / 32.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F / 32.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.ShockWave Then
            hp -= 1
        ElseIf BY = ShotTypes.Aquabullet Then
            hp -= 1
        ElseIf BY = ShotTypes.Plasma Then
            hp -= 1
        ElseIf BY = ShotTypes.Rubberet Then
            hp -= 4
        ElseIf BY = ShotTypes.Napalm Then
            hp -= 2
        ElseIf BY = ShotTypes.Screwer Then
            hp -= 2
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 10
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 4
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 1
        ElseIf BY = ShotTypes.Pearl Then
            hp -= 100
        End If
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 52
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Attack(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim X As Integer
        Dim Vx, Vy As Single
        X = Ri.Next(4)
        If state = Mystates.RushAttack Then
            Select Case X
                Case 0
                    atk = AtkSetThree.AdjacentShots
                Case 1
                    atk = AtkSetThree.SlowWheel
                Case 2
                    atk = AtkSetThree.SpreadShot
                Case 3
                    atk = AtkSetThree.ReverseCharge
            End Select
        ElseIf state = Mystates.JumpingAttack Then
            Select Case X
                Case 0
                    atk = AtkSetThree.FourLine
                Case 1
                    atk = AtkSetThree.VertigoBomb
                Case 2
                    atk = AtkSetThree.SixteenShot
                Case 3
                    atk = AtkSetThree.RopeFigure
            End Select
        ElseIf state = Mystates.PatientAttack Then
            Select Case X
                Case 0
                    atk = AtkSetThree.CWStar
                Case 1
                    atk = AtkSetThree.CCWStar
                Case 2
                    atk = AtkSetThree.RisingAttack
                Case 3
                    atk = AtkSetThree.Gutsy
            End Select
        End If
        Select Case atk
            Case AtkSetThree.AdjacentShots
                If lookleft Then
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -5, -2, 0, 0, 200))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -5, 2, 0, 0, 200))
                Else
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5, -2, 0, 0, 200))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5, 2, 0, 0, 200))
                End If
            Case AtkSetThree.CWStar
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 4, 0, -0.2F, 0.1F, 150))
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0, 4, -0.1F, -0.2F, 150))
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4, 0, 0.2F, -0.1F, 150))
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0, -4, 0.1F, 0.2F, 150))
            Case AtkSetThree.CCWStar
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 4, 0, -0.2F, -0.1F, 150))
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0, 4, 0.1F, -0.2F, 150))
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4, 0, 0.2F, 0.1F, 150))
                Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0, -4, -0.1F, 0.2F, 150))
            Case AtkSetThree.FourLine
                Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 1.5F, 240))
                Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 3.0F, 120))
                Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 4.5F, 80))
                Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 6.0F, 60))
            Case AtkSetThree.Gutsy
                Q.Monsters.Add(New CasukeBall(mr.X, mr.Y, le, re, True, 3.0F))
                Q.Monsters.Add(New CasukeBall(mr.X, mr.Y, le, re, False, 3.0F))
            Case AtkSetThree.ReverseCharge
                If lookleft Then
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 2.0F, 0, -0.2F, 0.0F, 240))
                Else
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -2.0F, 0, 0.2F, 0.0F, 240))
                End If
            Case AtkSetThree.RisingAttack
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 1.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 2.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 3.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 4.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -1.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -2.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -3.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4.0F, 1.0F, 0.0F, 0.0F, 60, 2, New Bullet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 150)))
            Case AtkSetThree.RopeFigure
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 2.0F, 0.0F, 0.0F, 0.0F, 20, 20, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 2.0F, 2.0F, 0.0F, 0.0F, 20, 24, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, 2.0F, 0.0F, 0.0F, 20, 28, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -2.0F, 2.0F, 0.0F, 0.0F, 20, 32, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -2.0F, 0.0F, 0.0F, 0.0F, 20, 36, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -2.0F, -2.0F, 0.0F, 0.0F, 20, 40, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, -2.0F, 0.0F, 0.0F, 20, 44, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
                Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 2.0F, -2.0F, 0.0F, 0.0F, 20, 48, New DirectBullet(0.0F, 0.0F, R, 3, 150)))
            Case AtkSetThree.SixteenShot
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 180))
                Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 180))
                Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, -Vy, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, Vx, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, Vy, 0, 0, 180))
                Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, -Vx, 0, 0, 180))
            Case AtkSetThree.SlowWheel
                If lookleft Then
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2.0F, mr.Y + MYHEIGHT / 2.0F, 1.0F, -6, 0.0F, 0.5F, 320))
                Else
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2.0F, mr.Y + MYHEIGHT / 2.0F, 1.0F, -6, 0.0F, 0.5F, 320))
                End If
            Case AtkSetThree.SpreadShot
                If lookleft Then
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -5, -4, 0, 0, 200))
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -5, 4, 0, 0, 200))
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -5, 0, 0, 0, 200))
                Else
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5, -4, 0, 0, 200))
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5, 4, 0, 0, 200))
                    Q.Monsters.Add(New Aquabullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5, 0, 0, 0, 200))
                End If
            Case AtkSetThree.VertigoBomb
                Q.Monsters.Add(New Napalm(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, -3.0F, 0.0F, 0.05F, 200))
        End Select
        attacked = True
    End Sub
    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim X As Integer
        Dim RV As AutoSuiteReturn
        attacked = False
        Select Case state
            Case Mystates.Waiting
                If internalcounter = 80 Then
                    internalcounter = 0
                    X = Ri.Next(5)
                    If X = 0 OrElse X = 4 Then
                        state = Mystates.RushAttack
                        lookleft = (R.X < mr.X)
                    ElseIf X = 1 OrElse X = 3 Then
                        state = Mystates.JumpingAttack
                        lookleft = (R.X < mr.X)
                    ElseIf X = 2 Then
                        state = Mystates.PatientAttack
                    End If
                End If
            Case Mystates.RushAttack
                If lookleft Then
                    goleft = True
                    goright = False
                    If mr.X >= 1040 AndAlso mr.X <= 1080 Then
                        jumping = True
                        veloc = JUMPSPEED
                    End If
                Else
                    goright = True
                    goleft = False
                    If mr.X >= 920 AndAlso mr.X <= 960 Then
                        jumping = True
                        veloc = JUMPSPEED
                    End If
                End If
                If internalcounter >= 480 Then
                    state = Mystates.Waiting
                    internalcounter = 0
                    goleft = False
                    goright = False
                ElseIf internalcounter Mod 80 = 40 Then
                    Attack(R, Q)
                End If
            Case Mystates.JumpingAttack
                If internalcounter >= 480 Then
                    state = Mystates.Waiting
                    internalcounter = 0
                    goleft = False
                    goright = False
                ElseIf internalcounter Mod 50 = 1 Then
                    jumping = True
                    veloc = JUMPSPEED
                    lookleft = (R.X < mr.X)
                ElseIf internalcounter Mod 50 = 24 Then
                    Attack(R, Q)
                ElseIf internalcounter Mod 50 >= 25 Then
                    If lookleft Then
                        goleft = True
                        goright = False
                    Else
                        goleft = False
                        goright = True
                    End If
                End If
            Case Mystates.PatientAttack
                If Q.Monsters.Count > 1 Then
                    If Not jumping Then
                        Attack(R, Q)
                        jumping = True
                        veloc = JUMPSPEED
                    End If
                End If
                If internalcounter >= 200 Then
                    state = Mystates.Waiting
                    internalcounter = 0
                    goleft = False
                    goright = False
                End If
        End Select

        If goleft Then
            mr.Offset(-MOVESPEED, 0)

            If mr.Right < le AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
        End If
        If goright Then
            mr.Offset(MOVESPEED, 0)

            If mr.Left > re AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each wall As RectangleF In Q.Walls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
        End If
        If jumping Then
            mr.Offset(0, veloc + GRAVITY)
            veloc += GRAVITY
            mr.Y = Convert.ToSingle(Math.Round(mr.Top * 1000.0F, 0)) / 1000.0F

            RV = LevelSuite.PlatformJump(Q.Platforms, mr, veloc, GRAVITY * 1.5F)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                End If
            End If

            RV = LevelSuite.WallJump(Q.Walls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Extrawalls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Crates, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
        End If
        internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BossFour
    Implements IMonster
    Private mr As RectangleF
    Public HP As Integer
    Private internalcounter As Integer
    Private place, dest As Positions
    Private mystate As BossStates
    Private atk As AtkSetFour
    Private Ri As Random

    Private Enum Positions
        UpperLeft
        UpperMiddle
        UpperRight
        LowerLeft
        LowerMiddle
        LowerRight
    End Enum
    Private Enum BossStates
        Attacking
        Transiting
        TickedOff
    End Enum
    Private Enum AtkSetFour
        Serpentine
        SixteenShot
        DirectNape
        HexagonMist
    End Enum

    Const MYWIDTH As Single = 96.0F
    Const MYHEIGHT As Single = 96.0F
    Const ANGRYBOSSHP As Integer = 72

    Const XA As Single = 544.0F
    Const XC As Single = 1168.0F
    Const XB As Single = (XA + XC) / 2.0F
    Const YA As Single = -40.0F
    Const YB As Single = 160.0F
    '624 and 200 -> 312 and 100 -> 156 and 50 -> 78 and 25

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        HP = 400
        internalcounter = 80
        mystate = BossStates.Attacking
        dest = Positions.LowerRight
        Ri = New Random
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        If internalcounter Mod 16 < 8 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.ShockWave
                hp -= 2
            Case ShotTypes.Aquabullet
                hp -= 1
            Case ShotTypes.Napalm
                hp -= 2
            Case ShotTypes.Rubberet
                hp -= 4
            Case ShotTypes.Screwer
                hp -= 3
            Case Else
                hp -= 3
        End Select
        Q.SonicEvents.Add(SonicEventType.ShotHit)
        Return True
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 65
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim I As Integer
        Dim cx, cy As Single
        Dim A, B, C As Microsoft.DirectX.Vector2

        If mystate = BossStates.Attacking Then
            If internalcounter = 75 Then
                atk = CType(Ri.Next(4), AtkSetFour)
            ElseIf internalcounter = 60 Then
                If atk = AtkSetFour.DirectNape Then
                    A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                    B = New Microsoft.DirectX.Vector2(mr.X + 50, mr.Y + 60)
                    C = Microsoft.DirectX.Vector2.Subtract(A, B)
                    C.Normalize()
                    cx = C.X * 3
                    cy = C.Y * 3
                    Q.Monsters.Add(New Napalm(B.X, B.Y, cx, cy, 0, 0, 200))
                ElseIf atk = AtkSetFour.HexagonMist Then
                    A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                    B = New Microsoft.DirectX.Vector2(mr.X + 50, mr.Y + 60)
                    C = Microsoft.DirectX.Vector2.Subtract(A, B)
                    C.Normalize()
                    cx = C.X * 2.5F
                    cy = C.Y * 2.5F
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, -2.0F, -1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 2.0F, -1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, -2.0F, 1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 2.0F, 1.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, 2.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, -2.0F, 0.0F, 0.0F, 30, 50, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, -3.0F, -1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 3.0F, -1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, -3.0F, 1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 3.0F, 1.5F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, 3.0F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, -3.0F, 0.0F, 0.0F, 30, 10, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, -1.0F, -0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 1.0F, -0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, -1.0F, 0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 1.0F, 0.5F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, 1.0F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                    Q.Monsters.Add(New CustomShot(B.X, B.Y, 0.0F, -1.0F, 0.0F, 0.0F, 30, 90, New Aquabullet(0.0F, 0.0F, cx, cy, 0.0F, 0.0F, 200)))
                ElseIf atk = AtkSetFour.Serpentine Then
                    Q.Monsters.Add(New DirectBullet(mr.X + 50, mr.Y + 60, R, 4, 200))
                ElseIf atk = AtkSetFour.SixteenShot Then
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 240))
                    cx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                    cy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, cx, cy, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, cy, -cx, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -cx, -cy, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -cy, cx, 0, 0, 240))
                    cx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                    cy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, cx, cy, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, cy, -cx, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -cx, -cy, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -cy, cx, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, cx, -cy, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, cy, cx, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -cx, cy, 0, 0, 240))
                    Q.Monsters.Add(New RAShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -cy, -cx, 0, 0, 240))
                End If
            ElseIf internalcounter Mod 2 = 0 AndAlso internalcounter < 60 AndAlso internalcounter >= 52 Then
                If atk = AtkSetFour.Serpentine Then
                    Q.Monsters.Add(New DirectBullet(mr.X + 50, mr.Y + 60, R, internalcounter / 15.0F, 200))
                End If
            ElseIf internalcounter <= 0 Then
                mystate = BossStates.Transiting
                I = Ri.Next(6)
                place = dest
                dest = CType(I, Positions)
                internalcounter = 260
            End If
        ElseIf mystate = BossStates.TickedOff Then
            If internalcounter Mod 5 = 0 Then
                I = Ri.Next(360)
                cx = Convert.ToSingle(Math.Cos(I * Math.PI / 180.0R))
                cy = Convert.ToSingle(Math.Sin(I * Math.PI / 180.0R))
                Q.Monsters.Add(New Rubberet(mr.X + 50, mr.Y + 60, 2 * cx, 2 * cy, 0.0F, 0.0F, 300))
                Q.Monsters.Add(New Rubberet(mr.X + 50, mr.Y + 60, -2 * cx, -2 * cy, 0.0F, 0.0F, 300))
            ElseIf internalcounter <= 0 Then
                mystate = BossStates.Transiting
                I = Ri.Next(6)
                place = dest
                dest = CType(I, Positions)
                internalcounter = 260
            End If
        ElseIf mystate = BossStates.Transiting Then
            Select Case place
                Case Positions.LowerLeft
                    Select Case dest
                        Case Positions.LowerLeft
                            internalcounter = 80
                            mr.X = XA
                            mr.Y = YB
                            If hp <= ANGRYBOSSHP Then
                                mystate = BossStates.TickedOff
                            Else
                                mystate = BossStates.Attacking
                            End If
                        Case Positions.LowerMiddle
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(4.0F, 0.0F)
                            ElseIf internalcounter >= 82 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerRight
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 54 Then
                                mr.Offset(4.0F, 0.0F)
                            ElseIf internalcounter >= 4 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperLeft
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperMiddle
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperRight
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 54 Then
                                mr.Offset(4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If

                    End Select
                Case Positions.LowerMiddle
                    Select Case dest
                        Case Positions.LowerLeft
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(-4.0F, 0.0F)
                            ElseIf internalcounter >= 82 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerMiddle
                            internalcounter = 80
                            mr.X = XB
                            mr.Y = YB
                            If hp <= ANGRYBOSSHP Then
                                mystate = BossStates.TickedOff
                            Else
                                mystate = BossStates.Attacking
                            End If
                        Case Positions.LowerRight
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(4.0F, 0.0F)
                            ElseIf internalcounter >= 82 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperLeft
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(-4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperMiddle
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperRight
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                    End Select
                Case Positions.LowerRight
                    Select Case dest
                        Case Positions.LowerLeft
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 54 Then
                                mr.Offset(-4.0F, 0.0F)
                            ElseIf internalcounter >= 4 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerMiddle
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(-4.0F, 0.0F)
                            ElseIf internalcounter >= 82 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerRight
                            internalcounter = 80
                            mr.X = XC
                            mr.Y = YB
                            If hp <= ANGRYBOSSHP Then
                                mystate = BossStates.TickedOff
                            Else
                                mystate = BossStates.Attacking
                            End If
                        Case Positions.UpperLeft
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 54 Then
                                mr.Offset(-4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperMiddle
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(-4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperRight
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, -4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                    End Select
                Case Positions.UpperLeft
                    Select Case dest
                        Case Positions.LowerLeft
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerMiddle
                            If internalcounter >= 182 Then
                                mr.Offset(4.0F, 0.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerRight
                            If internalcounter >= 104 Then
                                mr.Offset(4.0F, 0.0F)
                            ElseIf internalcounter >= 54 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperLeft
                            internalcounter = 80
                            mr.X = XA
                            mr.Y = YA
                            If hp <= ANGRYBOSSHP Then
                                mystate = BossStates.TickedOff
                            Else
                                mystate = BossStates.Attacking
                            End If
                        Case Positions.UpperMiddle
                            If internalcounter >= 182 Then
                                mr.Offset(4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperRight
                            If internalcounter >= 104 Then
                                mr.Offset(4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                    End Select
                Case Positions.UpperMiddle
                    Select Case dest
                        Case Positions.LowerLeft
                            If internalcounter >= 182 Then
                                mr.Offset(-4.0F, 0.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerMiddle
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerRight
                            If internalcounter >= 182 Then
                                mr.Offset(4.0F, 0.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperLeft
                            If internalcounter >= 182 Then
                                mr.Offset(-4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperMiddle
                            internalcounter = 80
                            mr.X = XB
                            mr.Y = YA
                            If hp <= ANGRYBOSSHP Then
                                mystate = BossStates.TickedOff
                            Else
                                mystate = BossStates.Attacking
                            End If
                        Case Positions.UpperRight
                            If internalcounter >= 182 Then
                                mr.Offset(4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                    End Select
                Case Positions.UpperRight
                    Select Case dest
                        Case Positions.LowerLeft
                            If internalcounter >= 104 Then
                                mr.Offset(-4.0F, 0.0F)
                            ElseIf internalcounter >= 54 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerMiddle
                            If internalcounter >= 182 Then
                                mr.Offset(-4.0F, 0.0F)
                            ElseIf internalcounter >= 132 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.LowerRight
                            If internalcounter >= 210 Then
                                mr.Offset(0.0F, 4.0F)
                            Else
                                internalcounter = 80
                                mr.X = XC
                                mr.Y = YB
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperLeft
                            If internalcounter >= 104 Then
                                mr.Offset(-4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XA
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperMiddle
                            If internalcounter >= 182 Then
                                mr.Offset(-4.0F, 0.0F)
                            Else
                                internalcounter = 80
                                mr.X = XB
                                mr.Y = YA
                                If hp <= ANGRYBOSSHP Then
                                    mystate = BossStates.TickedOff
                                Else
                                    mystate = BossStates.Attacking
                                End If
                            End If
                        Case Positions.UpperRight
                            internalcounter = 80
                            mr.X = XC
                            mr.Y = YA
                            If hp <= ANGRYBOSSHP Then
                                mystate = BossStates.TickedOff
                            Else
                                mystate = BossStates.Attacking
                            End If
                    End Select
            End Select
        End If
        internalcounter -= 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BossFive
    Implements IMonster
    Private mr As RectangleF

    Public HP As Integer
    Private internalcounter As Integer

    Private goleft, goright, jumping, lookleft, attacked As Boolean
    Private veloc, le, re As Single

    Private Enum AtkSetFive
        RainShare
        Serpentine
        LongitudeFan
        LatitudeFan
        NapalmSpray
        NapalmSpritz
        Spread
        ScrewMeteor
        PlasticBeam
        SleepSquare
    End Enum
    Private Enum MyStates
        AttackingA
        AttackingB
        AttackingC
        AtoB
        AtoC
        BtoA
        BtoC
        CtoA
        CtoB
    End Enum

    Private attack As AtkSetFive
    Private state As MyStates

    Private RI As Random

    Const PTAX As Single = 954.0F
    Const PTAY As Single = -200.0F
    Const PTBX As Single = 1204.0F
    Const PTBY As Single = -224.0F
    Const PTCX As Single = 1454.0F
    Const PTCY As Single = -200.0F


    Const JUMPSPEED As Single = -21.08F
    Const MOVESPEED As Single = 2.0F
    Const GRAVITY As Single = 0.38F
    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 32.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F


    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Ri As Random)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        HP = 450
        lookleft = True
        state = MyStates.AttackingB
        internalcounter = 0
        Me.RI = Ri
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 2.0F
        Dim F As Single = 0.0F
        If jumping Then
            If Me.lookleft Then
                F = 8.0F
            Else
                F = 9.0F
            End If
            If attacked Then
                F += 20.0F
            End If
        ElseIf lookleft Then
            F = ((internalcounter Mod 16) \ 4)
            If attacked Then
                F += 20.0F
            End If
        Else
            F = 4.0F + ((internalcounter Mod 16) \ 4)  '10 to convert 0-9 into 0.0-0.9, 2 to expand the frame time.
            If attacked Then
                F += 20.0F
            End If
        End If
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F / 32.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F / 32.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.ShockWave
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Aquabullet
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            Case ShotTypes.Napalm
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Rubberet
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            Case ShotTypes.Screwer
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ForceMod
                hp -= 3
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Corkwheel
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case Else
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True

        End Select
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 72
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim X As Integer
        Dim dx, dy As Single, ang As Double
        Dim A, B, C As Microsoft.DirectX.Vector2
        Dim Vx As Single
        Dim RV As AutoSuiteReturn
        internalcounter += 1
        If state = MyStates.AttackingA OrElse state = MyStates.AttackingB OrElse state = MyStates.AttackingC Then
            If internalcounter = 1 Then
                X = RI.Next(10)
                attack = CType(X, AtkSetFive)
                lookleft = (R.Left < mr.Left)
            ElseIf internalcounter = 2 Then
                Select Case attack
                    Case AtkSetFive.RainShare
                        If lookleft Then
                            Vx = -1
                        Else
                            Vx = 1
                        End If
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                        Q.Monsters.Add(New CustomShot(1204.0F, -500.0F, 12 - Convert.ToSingle(24.0R * RI.NextDouble), 0.0F, 0.0F, 0.0F, 60, RI.Next(500), New Aquabullet(0.0F, 0.0F, Vx, 3.0F, 0.0F, 0.03125F, 400)))
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.LongitudeFan
                        If lookleft Then
                            Vx = -3
                        Else
                            Vx = 3
                        End If
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx, 0.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 50.0F, Vx, 0.0F, 0.0F, 0.01F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 50.0F, Vx, 0.0F, 0.0F, -0.01F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 25.0F, Vx, 0.0F, 0.0F, 0.005F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 25.0F, Vx, 0.0F, 0.0F, -0.005F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 100.0F, Vx, 0.0F, 0.0F, 0.02F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 100.0F, Vx, 0.0F, 0.0F, -0.02F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 75.0F, Vx, 0.0F, 0.0F, 0.015F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 75.0F, Vx, 0.0F, 0.0F, -0.015F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 150.0F, Vx, 0.0F, 0.0F, 0.03F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 150.0F, Vx, 0.0F, 0.0F, -0.03F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 125.0F, Vx, 0.0F, 0.0F, 0.025F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 125.0F, Vx, 0.0F, 0.0F, -0.025F, 250))
                    Case AtkSetFive.LatitudeFan
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5.0F, 0.0F, -0.05F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -5.0F, 0.0F, 0.05F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 4.0F, 0.0F, -0.04F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4.0F, 0.0F, 0.04F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 3.0F, 0.0F, -0.03F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -3.0F, 0.0F, 0.03F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 2.0F, 0.0F, -0.02F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -2.0F, 0.0F, 0.02F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 1.0F, 0.0F, -0.01F, 0.0F, 250))
                        Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -1.0F, 0.0F, 0.01F, 0.0F, 250))
                    Case AtkSetFive.NapalmSpray
                        dx = R.X - mr.X
                        dy = R.Y - mr.Y
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4.5 * Math.Cos(ang - 0.2)), CSng(4.5 * Math.Sin(ang - 0.2)), 0, 0, 350))
                    Case AtkSetFive.NapalmSpritz
                        dx = R.X - mr.X
                        dy = R.Y - (mr.Y - 100)
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2 - 100, CSng(1.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 350))
                    Case AtkSetFive.Spread
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        X = Convert.ToInt32(C.Length)
                        C.Normalize()
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                    Case AtkSetFive.ScrewMeteor
                        If lookleft Then
                            Vx = -3
                        Else
                            Vx = 3
                        End If
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH - 8, mr.Y + HALFHEIGHT, Vx, 4.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH + 8, mr.Y + HALFHEIGHT, Vx, 4.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH - 32, mr.Y + HALFHEIGHT, Vx, 4.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH + 32, mr.Y + HALFHEIGHT, Vx, 4.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 8, Vx, 4.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 8, Vx, 4.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 32, Vx, 4.0F, 0.0F, 0.0F, 250))
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT + 32, Vx, 4.0F, 0.0F, 0.0F, 250))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 2 * C.X, 2 * C.Y, 0.0F, 0.0F, 300))
                    Case AtkSetFive.SleepSquare
                        If lookleft Then
                            Vx = -2
                        Else
                            Vx = 2
                        End If
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx, 0.0F, 0.0F, 0.01F, 200, 1, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.25F, 0.0F, 0.0F, 0.01F, 200, 11, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.25F, 0.0F, 0.0F, 0.01F, 200, 19, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.125F, 0.0F, 0.0F, 0.01F, 200, 2, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.125F, 0.0F, 0.0F, 0.01F, 200, 6, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx, 0.25F, 0.0F, 0.01F, 200, 23, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.25F, 0.25F, 0.0F, 0.01F, 200, 25, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.25F, 0.25F, 0.0F, 0.01F, 200, 21, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.125F, 0.25F, 0.0F, 0.01F, 200, 24, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.125F, 0.25F, 0.0F, 0.01F, 200, 22, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx, -0.25F, 0.0F, 0.01F, 200, 15, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.25F, -0.25F, 0.0F, 0.01F, 200, 13, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.25F, -0.25F, 0.0F, 0.01F, 200, 17, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.125F, -0.25F, 0.0F, 0.01F, 200, 14, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.125F, -0.25F, 0.0F, 0.01F, 200, 16, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx, 0.125F, 0.0F, 0.01F, 200, 8, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.25F, 0.125F, 0.0F, 0.01F, 200, 10, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.25F, 0.125F, 0.0F, 0.01F, 200, 20, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.125F, 0.125F, 0.0F, 0.01F, 200, 9, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.125F, 0.125F, 0.0F, 0.01F, 200, 7, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx, -0.125F, 0.0F, 0.01F, 200, 4, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.25F, -0.125F, 0.0F, 0.01F, 200, 12, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.25F, -0.125F, 0.0F, 0.01F, 200, 18, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx - 0.125F, -0.125F, 0.0F, 0.01F, 200, 3, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                        Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, Vx + 0.125F, -0.125F, 0.0F, 0.01F, 200, 5, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                End Select
            ElseIf internalcounter = 4 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 2 * C.X, 2 * C.Y, 0.0F, 0.005F, 300))
                End Select
            ElseIf internalcounter = 6 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 2 * C.X, 2 * C.Y, 0.0F, 0.01F, 300))
                    Case AtkSetFive.NapalmSpray
                        dx = R.X - mr.X
                        dy = R.Y - mr.Y
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 100))
                    Case AtkSetFive.NapalmSpritz
                        dx = R.X - mr.X
                        dy = R.Y - (mr.Y + 100)
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2 + 100, CSng(1.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 350))
                End Select
            ElseIf internalcounter = 8 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 2 * C.X, 2 * C.Y, 0.0F, 0.015F, 300))
                End Select
            ElseIf internalcounter = 10 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 2 * C.X, 2 * C.Y, 0.0F, 0.02F, 300))
                    Case AtkSetFive.NapalmSpray
                        dx = R.X - mr.X
                        dy = R.Y - mr.Y
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(4.5 * Math.Cos(ang + 0.2)), CSng(4.5 * Math.Sin(ang + 0.2)), 0, 0, 350))
                    Case AtkSetFive.NapalmSpritz
                        dx = R.X - mr.X
                        dy = R.Y - (mr.Y - 60)
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2 - 60, CSng(1.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 350))
                End Select
            ElseIf internalcounter = 12 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 2 * C.X, 2 * C.Y, 0.0F, 0.025F, 300))
                End Select
            ElseIf internalcounter = 14 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 300))
                    Case AtkSetFive.NapalmSpray
                        dx = R.X - mr.X
                        dy = R.Y - mr.Y
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang - 0.2)), CSng(2 * Math.Sin(ang - 0.2)), 0, 0, 100))
                    Case AtkSetFive.NapalmSpritz
                        dx = R.X - mr.X
                        dy = R.Y - (mr.Y + 60)
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2 + 60, CSng(1.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 350))
                End Select
            ElseIf internalcounter = 16 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.005F, 300))
                End Select
            ElseIf internalcounter = 18 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.001F, 300))
                    Case AtkSetFive.NapalmSpray
                        dx = R.X - mr.X
                        dy = R.Y - mr.Y
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang + 0.2)), CSng(2 * Math.Sin(ang + 0.2)), 0, 0, 100))
                    Case AtkSetFive.NapalmSpritz
                        dx = R.X - mr.X
                        dy = R.Y - (mr.Y - 20)
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2 - 20, CSng(1.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 350))
                End Select
            ElseIf internalcounter = 20 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.015F, 300))
                End Select
            ElseIf internalcounter = 22 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.002F, 300))
                    Case AtkSetFive.NapalmSpray
                        dx = R.X - mr.X
                        dy = R.Y - mr.Y
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(2 * Math.Cos(ang)), CSng(2 * Math.Sin(ang)), 0, 0, 100))
                    Case AtkSetFive.NapalmSpritz
                        dx = R.X - mr.X
                        dy = R.Y - (mr.Y + 20)
                        ang = Math.Atan2(dy, dx)
                        Q.Monsters.Add(New Napalm(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2 + 20, CSng(1.5 * Math.Cos(ang)), CSng(1.5 * Math.Sin(ang)), 0, 0, 350))
                End Select
            ElseIf internalcounter = 24 Then
                Select Case attack
                    Case AtkSetFive.Serpentine
                        Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                    Case AtkSetFive.PlasticBeam
                        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                        B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                        C = Microsoft.DirectX.Vector2.Subtract(A, B)
                        C.Normalize()
                        Q.Monsters.Add(New Rubberet(mr.X, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.025F, 300))
                End Select
            ElseIf internalcounter <= 56 AndAlso internalcounter Mod 2 = 0 Then
                If attack = AtkSetFive.Serpentine Then
                    Q.Monsters.Add(New DirectBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 5.0F, 300))
                End If
            ElseIf internalcounter = 58 Then
                internalcounter = 0
                X = RI.Next(2)
                If state = MyStates.AttackingA Then
                    If X = 0 Then
                        state = MyStates.AtoB
                    Else
                        state = MyStates.AtoC
                    End If
                ElseIf state = MyStates.AttackingB Then
                    If X = 0 Then
                        state = MyStates.BtoA
                    Else
                        state = MyStates.BtoC
                    End If
                ElseIf state = MyStates.AttackingC Then
                    If X = 0 Then
                        state = MyStates.CtoA
                    Else
                        state = MyStates.CtoB
                    End If
                End If
            End If
        ElseIf state = MyStates.AtoC Then
            goright = True
            goleft = False
            lookleft = False
            If internalcounter = 1 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 126 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 251 Then
                internalcounter = 0
                goright = False
                state = MyStates.AttackingC
                mr = New RectangleF(PTCX, PTCY, MYWIDTH, MYHEIGHT)
            End If
        ElseIf state = MyStates.AtoB Then
            goright = True
            goleft = False
            lookleft = False
            If internalcounter = 1 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 126 Then
                internalcounter = 0
                goright = False
                state = MyStates.AttackingB
                mr = New RectangleF(PTBX, PTBY, MYWIDTH, MYHEIGHT)
            End If
        ElseIf state = MyStates.BtoA Then
            goright = False
            goleft = True
            lookleft = True
            If internalcounter = 1 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 126 Then
                internalcounter = 0
                goleft = False
                state = MyStates.AttackingA
                mr = New RectangleF(PTAX, PTAY, MYWIDTH, MYHEIGHT)
            End If
        ElseIf state = MyStates.BtoC Then
            goright = True
            goleft = False
            lookleft = False
            If internalcounter = 1 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 126 Then
                internalcounter = 0
                goright = False
                state = MyStates.AttackingC
                mr = New RectangleF(PTCX, PTCY, MYWIDTH, MYHEIGHT)
            End If
        ElseIf state = MyStates.CtoA Then
            goright = False
            goleft = True
            lookleft = True
            If internalcounter = 1 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 126 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 251 Then
                internalcounter = 0
                goleft = False
                state = MyStates.AttackingA
                mr = New RectangleF(PTAX, PTAY, MYWIDTH, MYHEIGHT)
            End If
        ElseIf state = MyStates.CtoB Then
            goright = False
            goleft = True
            lookleft = True
            If internalcounter = 1 Then
                jumping = True
                veloc = JUMPSPEED
            ElseIf internalcounter = 126 Then
                internalcounter = 0
                goleft = False
                state = MyStates.AttackingB
                mr = New RectangleF(PTBX, PTBY, MYWIDTH, MYHEIGHT)
            End If
        End If

        If goleft Then
            mr.Offset(-MOVESPEED, 0)

            If mr.Right < le AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
        End If
        If goright Then
            mr.Offset(MOVESPEED, 0)

            If mr.Left > re AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each wall As RectangleF In Q.Walls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
        End If
        If jumping Then
            mr.Offset(0, veloc + GRAVITY)
            veloc += GRAVITY
            mr.Y = Convert.ToSingle(Math.Round(mr.Top * 1000.0F, 0)) / 1000.0F

            RV = LevelSuite.PlatformJump(Q.Platforms, mr, veloc, GRAVITY * 1.5F)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                End If
            End If

            RV = LevelSuite.WallJump(Q.Walls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Extrawalls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Groundwalls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Crates, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BossSix
    Implements IMonster
    Private mr As RectangleF
    Public HP As Integer
    Private internalcounter, X As Integer
    Private dx, dy As Single
    Private ang As Double
    Private A, B, C As Microsoft.DirectX.Vector2

    Const MYWIDTH As Single = 128.0F
    Const MYHEIGHT As Single = 256.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 400
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        Dim F As Integer
        F = (Me.internalcounter >> 1) Mod 4
        Select Case F
            Case 0
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
            Case 1
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.25F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.25F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
            Case 2
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
            Case 3
                vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.75F, 0.0F)
                vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.75F, 1.0F)
                vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
        End Select
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If ISpd <= 0.0F Then
            Select Case BY
                Case ShotTypes.ShockWave
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                Case ShotTypes.Aquabullet
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                Case ShotTypes.Napalm
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                Case ShotTypes.Rubberet
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                Case ShotTypes.Screwer
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                Case ShotTypes.ForceMod
                    hp -= 2
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
                Case Else
                    hp -= 1
                    Q.SonicEvents.Add(SonicEventType.ShotHit)
                    Return True
            End Select
        Else
            Q.SonicEvents.Add(SonicEventType.ShotDefend)
            Return True

        End If

    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 86
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        mr.Offset(1.0F, 0.0F)
        internalcounter += 1
        'dx = R.X - (mr.X + MYWIDTH)
        'dy = R.Y - (mr.Y + MYHEIGHT / 2.0F)
        'ang = Math.Atan2(dy, dx)
        'Q.Monsters.Add(New Napalm(mr.X + MYWIDTH, mr.Y + MYHEIGHT / 2, CSng(4.5 * Math.Cos(ang - 0.2)), CSng(4.5 * Math.Sin(ang - 0.2)), 0, 0, 350))
        Select Case internalcounter
            Case 1
                dx = R.Left
                dy = R.Top
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 0.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
            Case 5
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 9
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 13
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 17
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 147, 890, 940, 1900, 2200, 2450, 2900
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.0F, 0.1F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.1F, 300)))
            Case 200, 300, 400, 450, 500, 1400, 1700, 2400, 3400, 3500, 3600
                Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH, R.Y, 1.0F, 0.0F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH, R.Y, 2.0F, 0.0F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH, R.Y, 3.0F, 0.0F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New Aquabullet(mr.X + MYWIDTH, R.Y, 4.0F, 0.0F, 0.0F, 0.0F, 400))
            Case 600, 610, 620, 630, 635, 640, 650, 660, 670
                dx = R.Left
                dy = R.Top
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 300))
            Case 800
                dx = R.Left
                dy = R.Top
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 0.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
            Case 804
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 808   'Boom like an 808
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 812
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 816
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 1200, 1500, 2000
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 8.0F, -4.0F, 0.0F, 0.5F, 400))
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 7.0F, -4.0F, 0.0F, 0.5F, 400))
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 6.0F, -4.0F, 0.0F, 0.5F, 400))
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 5.0F, -4.0F, 0.0F, 0.5F, 400))
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 4.0F, -4.0F, 0.0F, 0.5F, 400))
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 3.0F, -4.0F, 0.0F, 0.5F, 400))
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 2.0F, -4.0F, 0.0F, 0.5F, 400))
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Y, 1.0F, -4.0F, 0.0F, 0.5F, 400))
            Case 1850 To 1974
                If internalcounter Mod 4 = 2 Then
                    Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH, mr.Y + 2.0F * (internalcounter - 1850), 4.0F, 0.0F, 0.0F, 0.0F, 400))
                End If
            Case 1111, 2100, 2200, 2250, 2300, 2350
                A = New Microsoft.DirectX.Vector2(R.Left + 500, R.Top)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, X * C.X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, X * C.X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
                A = New Microsoft.DirectX.Vector2(R.Left - 500, R.Top)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, C.X * X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, C.X * X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top + 500)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, C.X * X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, C.X * X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top - 500)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, C.X * X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, C.X * X / 100.0F, C.Y * X / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
            Case 2345
                dx = 2
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.25F, -0.25F, 0.0F, -0.01F, 200, 13, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.25F, -0.125F, 0.0F, -0.01F, 200, 12, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.25F, 0.0F, 0.0F, -0.01F, 200, 11, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.25F, 0.125F, 0.0F, -0.01F, 200, 10, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.25F, 0.25F, 0.0F, -0.01F, 200, 25, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.125F, -0.25F, 0.0F, -0.01F, 200, 14, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.125F, -0.125F, 0.0F, -0.01F, 200, 3, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.125F, 0.0F, 0.0F, -0.01F, 200, 2, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.125F, 0.125F, 0.0F, -0.01F, 200, 9, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx - 0.125F, 0.25F, 0.0F, -0.01F, 200, 24, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx, -0.25F, 0.0F, -0.01F, 200, 15, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx, -0.125F, 0.0F, -0.01F, 200, 4, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx, 0.0F, 0.0F, -0.01F, 200, 1, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx, 0.125F, 0.0F, -0.01F, 200, 8, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx, 0.25F, 0.0F, -0.01F, 200, 23, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.125F, -0.25F, 0.0F, -0.01F, 200, 16, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.125F, -0.125F, 0.0F, -0.01F, 200, 5, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.125F, 0.0F, 0.0F, -0.01F, 200, 6, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.125F, 0.125F, 0.0F, -0.01F, 200, 7, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.125F, 0.25F, 0.0F, -0.01F, 200, 22, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.25F, -0.25F, 0.0F, -0.01F, 200, 17, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.25F, -0.125F, 0.0F, -0.01F, 200, 18, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.25F, 0.0F, 0.0F, -0.01F, 200, 19, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.25F, 0.125F, 0.0F, -0.01F, 200, 20, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + MYHEIGHT, dx + 0.25F, 0.25F, 0.0F, -0.01F, 200, 21, New DirectBullet(0.0F, 0.0F, R, 4.5F, 300)))
            Case 2600 To 2724
                If internalcounter Mod 4 = 2 Then
                    Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH, mr.Y + 3.0F * (internalcounter - 2600), 4.0F, 0.0F, 0.0F, 0.0F, 400))
                End If
            Case 2800, 2810, 2820, 2830, 2835, 2840, 2850, 2860, 2870
                dx = R.Left
                dy = R.Top
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT, 5 * C.X, 5 * C.Y, 0.0F, 0.0F, 300))
            Case 3000
                dx = R.Left
                dy = R.Top
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 0.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
            Case 3004
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3008
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3012
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3016
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3030
                dx = R.Left
                dy = R.Top
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 0.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, -1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 2.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 1.0F, 1.0F, 0.0F, 0.0F, 50, 10, New DirectBullet(0.0F, 0.0F, R, 5.0F, 250)))
            Case 3034
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.75F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3038
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3042
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3046
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New ForceMod(mr.X + MYWIDTH, mr.Y, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 400))
            Case 3100 To 3224
                If internalcounter Mod 4 = 2 Then
                    Q.Monsters.Add(New StoneBullet(mr.X + MYWIDTH, mr.Y + 3.0F * (internalcounter - 3100), 4.0F, 0.0F, 0.0F, 0.0F, 400))
                End If
            Case 1000, 3350, 3550, 3750, 3950, 4150, 4200, 4250
                A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, 6.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, -2.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, 2.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, -6.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, 6.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, -2.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, 2.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, -4.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, -5.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, 5.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, 3.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, -1.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, -5.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, 5.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, -3.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, 3.5F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, 1.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, 6.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, -2.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, 2.0F, 0.0F, 0.0F, 300)))
                Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 300)))
            Case 4000, 4010, 4020, 4030, 4035, 4040, 4050, 4060, 4070, 4100
                dx = R.Left
                dy = R.Top
                A = New Microsoft.DirectX.Vector2(dx, dy)
                B = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT)
                C = Microsoft.DirectX.Vector2.Subtract(A, B)
                X = Convert.ToInt32(C.Length)
                C.Normalize()
                Q.Monsters.Add(New Napalm(mr.X + MYWIDTH, mr.Y + 0.25F * MYHEIGHT, 4 * C.X, 4 * C.Y, 0.0F, 0.0F, 300))

        End Select
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class BossSeven
    Implements IMonster

    Private Enum AtkSetSeven
        PlasticForceField
        CardinalWind
        Twister
        Absolute
        DoubleHandBlaster
        ScrewSeeker
        Abhorration
        TornadoBlow
    End Enum

    Private mr As RectangleF

    Public HP As Integer
    Private internalcounter As Integer
    Private Ri As Random

    Private goleft, goright, jumping, lookleft, attacked As Boolean
    Private veloc, le, re As Single
    Private atk As AtkSetSeven

    Const JUMPSPEED As Single = -8.38F
    Const MOVESPEED As Single = 4.0F
    Const GRAVITY As Single = 0.38F
    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 32.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F
    Const PHASEHP As Integer = 360
    Const MAXHP As Integer = PHASEHP << 2

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal RNG As Random)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = MAXHP
        Ri = RNG
        lookleft = True
        atk = AtkSetSeven.PlasticForceField
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return hp <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 2.0F
        Dim F As Single = 0.0F
        If jumping Then
            If Me.lookleft Then
                F = 8.0F
            Else
                F = 9.0F
            End If
            If attacked Then
                F += 20.0F
            End If
        ElseIf lookleft Then
            F = ((internalcounter Mod 16) \ 4)
            If attacked Then
                F += 20.0F
            End If
        Else
            F = 4.0F + ((internalcounter Mod 16) \ 4)  '10 to convert 0-9 into 0.0-0.9, 2 to expand the frame time.
            If attacked Then
                F += 20.0F
            End If
        End If
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F / 32.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F / 32.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If BY = ShotTypes.ShockWave Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Aquabullet Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Rubberet Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Napalm Then
            Q.SonicEvents.Add(SonicEventType.ShotThru)
            Return False
        ElseIf BY = ShotTypes.Screwer Then
            hp -= 2
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Corkwheel Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.ForceMod Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        ElseIf BY = ShotTypes.Stonebullet Then
            hp -= 4
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        Else
            hp -= 1
            Q.SonicEvents.Add(SonicEventType.ShotHit)
            Return True
        End If

    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 92
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim X As Integer
        Dim dx, dy As Single, ang As Double
        Dim A, B, C As Microsoft.DirectX.Vector2
        Dim Vx As Single
        Dim RV As AutoSuiteReturn
        internalcounter += 1
        Select Case atk
            Case AtkSetSeven.PlasticForceField
                If internalcounter = 30 Then
                    A = New Microsoft.DirectX.Vector2(R.Left + 8, R.Top)
                    B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                    C = Microsoft.DirectX.Vector2.Subtract(A, B)
                    X = Convert.ToInt32(C.Length)
                    C.Normalize()
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, 6.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, -2.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, 2.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Rubberet(0.0F, 0.0F, 0.0F, -4.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, -6.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, 6.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, -2.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, 2.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Rubberet(0.0F, 0.0F, 0.0F, -4.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, -5.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, 5.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, -3.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, 3.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Rubberet(0.0F, 0.0F, 0.0F, -1.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, -5.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, 5.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, -3.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, 3.5F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Rubberet(0.0F, 0.0F, 0.0F, 1.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, -6.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, 6.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, -2.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, 2.0F, 0.0F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Rubberet(0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 0.0F, 300)))
                ElseIf internalcounter = 4 Then
                    jumping = True
                    veloc = JUMPSPEED
                ElseIf internalcounter = 64 Then
                    jumping = True
                    veloc = JUMPSPEED
                ElseIf internalcounter = 120 Then
                    internalcounter = 0
                    atk = AtkSetSeven.CardinalWind
                End If
            Case AtkSetSeven.CardinalWind
                If internalcounter = 1 Then
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5.0F, 0.0F, -0.05F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -5.0F, 0.0F, 0.05F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 4.0F, 0.0F, -0.04F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4.0F, 0.0F, 0.04F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 3.0F, 0.0F, -0.03F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -3.0F, 0.0F, 0.03F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 2.0F, 0.0F, -0.02F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -2.0F, 0.0F, 0.02F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 1.0F, 0.0F, -0.01F, 0.0F, 250))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -1.0F, 0.0F, 0.01F, 0.0F, 250))
                ElseIf internalcounter = 21 Then
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 400, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                ElseIf internalcounter = 41 Then
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y - 180, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 20, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH + 20, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y - 220, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                ElseIf internalcounter = 61 Then
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y - 160, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 20, mr.Y - 220, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH + 20, mr.Y - 220, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y - 240, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 40, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 20, mr.Y - 180, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH + 20, mr.Y - 180, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 40, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                ElseIf internalcounter = 81 Then
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 60, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 40, mr.Y - 220, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH + 40, mr.Y - 220, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 80, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH + 40, mr.Y - 180, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH + 60, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH + 80, mr.Y - 200, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH - 40, mr.Y - 180, 0.0F, 0.0F, 0.0F, 0.0F, 380, 0, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.01F, 200)))
                ElseIf internalcounter = 100 Then
                    Q.Monsters.Add(New Faruno(New RectangleF(le, mr.Bottom, re - le, mr.Bottom + 4), Convert.ToInt32(mr.Left - le), True, Ri))
                ElseIf internalcounter = 120 Then
                    Q.Monsters.Add(New Faruno(New RectangleF(le, mr.Bottom, re - le, mr.Bottom + 4), Convert.ToInt32(mr.Left - le), True, Ri))
                ElseIf internalcounter = 140 Then
                    Q.Monsters.Add(New Faruno(New RectangleF(le, mr.Bottom, re - le, mr.Bottom + 4), Convert.ToInt32(mr.Left - le), True, Ri))
                ElseIf internalcounter = 150 Then
                    Q.Monsters.Add(New DarkFly(mr.X + HALFWIDTH, mr.Y - 200))
                ElseIf internalcounter = 200 Then
                    internalcounter = 0
                    If hp <= 3 * PHASEHP Then
                        atk = AtkSetSeven.Twister
                    Else
                        atk = AtkSetSeven.PlasticForceField
                    End If
                End If
            Case AtkSetSeven.Twister
                If internalcounter = 10 Then
                    Q.Monsters.Add(New Corkwheel(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 4.0F, 0.0F, 0.0F, 0.0F, 500))
                    Q.Monsters.Add(New Corkwheel(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4.0F, 0.0F, 0.0F, 0.0F, 500))
                ElseIf internalcounter = 100 Then
                    Q.Monsters.Add(New SweepModular(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, True))
                ElseIf internalcounter = 120 Then
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -1.0F, 0.0F, 0.0F, 0.0F, 800))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -1.0F, -1.0F, 0.0F, 0.0F, 800))
                    Q.Monsters.Add(New Bullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -1.0F, -2.0F, 0.0F, 0.0F, 800))
                ElseIf internalcounter = 200 Then
                    internalcounter = 0
                    atk = AtkSetSeven.Absolute
                End If
            Case AtkSetSeven.Absolute
                If internalcounter = 25 Then
                    A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
                    B = New Microsoft.DirectX.Vector2(mr.X, mr.Y)
                    C = Microsoft.DirectX.Vector2.Subtract(A, B)
                    X = Convert.ToInt32(C.Length)
                    C.Normalize()
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, -0.5F, -6.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 0.5F, -6.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, -1.0F, -6.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 1.0F, -6.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 1, New Aquabullet(0.0F, 0.0F, 0.0F, -6.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, -0.5F, -5.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 0.5F, -5.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, -1.0F, -5.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 1.0F, -5.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 6, New Aquabullet(0.0F, 0.0F, 0.0F, -5.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, -0.5F, -4.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 0.5F, -4.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, -1.0F, -4.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 1.0F, -4.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 11, New Aquabullet(0.0F, 0.0F, 0.0F, -4.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, -0.5F, -3.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 0.5F, -3.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, -1.0F, -3.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 1.0F, -3.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 16, New Aquabullet(0.0F, 0.0F, 0.0F, -3.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, -0.5F, -2.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 0.5F, -2.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, -1.0F, -2.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 1.0F, -2.0F, 0.05F, 0.0F, 300)))
                    Q.Monsters.Add(New CustomShot(mr.X, mr.Y, C.X * 5.0F, C.Y * 5.0F, 0.0F, 0.0F, X \ 5, 21, New Aquabullet(0.0F, 0.0F, 0.0F, -2.0F, 0.05F, 0.0F, 300)))
                ElseIf internalcounter = 100 Then
                    Q.Monsters.Add(New Ejectent(New RectangleF(le, mr.Bottom, re - le, mr.Bottom + 4), mr.Left - le))
                ElseIf internalcounter = 140 Then
                    Q.Monsters.Add(New Maser(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -1.0F, 800))
                ElseIf internalcounter = 250 Then
                    internalcounter = 0
                    If hp <= 2 * PHASEHP Then
                        atk = AtkSetSeven.DoubleHandBlaster
                    Else
                        atk = AtkSetSeven.PlasticForceField
                    End If
                End If
            Case AtkSetSeven.DoubleHandBlaster
                If internalcounter = 5 Then
                    Q.Monsters.Add(New Maser(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4.0F, 200))
                ElseIf internalcounter = 4 Then
                    jumping = True
                    veloc = JUMPSPEED
                ElseIf internalcounter = 10 Then
                    Q.Monsters.Add(New Maser(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 20, -4.0F, 200))
                ElseIf internalcounter = 15 Then
                    Q.Monsters.Add(New Maser(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 40, -4.0F, 200))
                ElseIf internalcounter = 30 Then
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, -0.5F, -2.0F, 0.0F, 0.0F, 120, 10, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, -0.5F, 2.0F, 0.0F, 0.0F, 120, 160, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 0.5F, -2.0F, 0.0F, 0.0F, 120, 10, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 0.5F, 2.0F, 0.0F, 0.0F, 120, 160, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, -1.0F, -3.0F, 0.0F, 0.0F, 120, 10, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, -1.0F, 3.0F, 0.0F, 0.0F, 120, 160, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 1.0F, -3.0F, 0.0F, 0.0F, 120, 10, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 1.0F, 3.0F, 0.0F, 0.0F, 120, 160, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 0.0F, -3.0F, 0.0F, 0.0F, 120, 10, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 0.0F, 3.0F, 0.0F, 0.0F, 120, 160, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, -0.5F, -4.0F, 0.0F, 0.0F, 120, 10, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, -0.5F, 4.0F, 0.0F, 0.0F, 120, 160, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 0.5F, -4.0F, 0.0F, 0.0F, 120, 10, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 0.5F, 4.0F, 0.0F, 0.0F, 120, 160, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 400, 0.0F, -3.0F, 0.0F, 0.0F, 120, 40, New DirectNape(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, 3.0F, 0.0F, 0.0F, 120, 180, New DirectBullet(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, -3.0F, 0.0F, 0.0F, 120, 60, New DirectBullet(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, 3.0F, 0.0F, 0.0F, 120, 200, New DirectBullet(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, -3.0F, 0.0F, 0.0F, 120, 80, New DirectBullet(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, 3.0F, 0.0F, 0.0F, 120, 220, New DirectBullet(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, -3.0F, 0.0F, 0.0F, 120, 100, New DirectBullet(0.0F, 0.0F, R, 6.0F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 0.0F, 3.0F, 0.0F, 0.0F, 120, 240, New DirectBullet(0.0F, 0.0F, R, 6.0F, 200)))
                ElseIf internalcounter = 100 Then
                    jumping = True
                    veloc = JUMPSPEED
                ElseIf internalcounter = 200 Then
                    internalcounter = 0
                    atk = AtkSetSeven.ScrewSeeker
                End If
            Case AtkSetSeven.ScrewSeeker
                If internalcounter Mod 10 = 0 Then
                    If internalcounter <= 50 Then
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 100, -4.0F, 4.0F, 0.0F, -0.05F, 200))
                    ElseIf internalcounter <= 100 Then
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 180, -5.0F, 5.0F, 0.0F, -0.05F, 200))
                    ElseIf internalcounter <= 150 Then
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 180, -3.0F, 6.0F, 0.0F, -0.1F, 200))
                    Else
                        Q.Monsters.Add(New Screwer(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 180, -5.0F, 2.0F, 0.0F, -0.01F, 200))
                    End If
                ElseIf internalcounter = 205 Then
                    internalcounter = 0
                    If hp <= PHASEHP Then
                        atk = AtkSetSeven.Abhorration
                    Else
                        atk = AtkSetSeven.PlasticForceField
                    End If
                End If
            Case AtkSetSeven.Abhorration
                If internalcounter = 5 Then
                    goleft = True
                ElseIf internalcounter = 1 Then
                    Q.Monsters.Add(New Squirm(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, 5.0F))
                ElseIf internalcounter Mod 20 = 10 Then
                    Q.Monsters.Add(New DirectAcceleratorBullet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, R, 0.04F, 150))
                ElseIf internalcounter = 40 Then
                    jumping = True
                    veloc = JUMPSPEED
                ElseIf internalcounter = 105 Then
                    goright = True
                    goleft = False
                    Q.AddExplosion(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F)
                    Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -12.8F, -12.8F, 0.0F, 0.0F, 5))
                    Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 4.0F, -4.0F, 0.0F, 0.0F, 15))
                    Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, 3.0F, 3.0F, 0.0F, 0.0F, 25))
                    Q.Monsters.Add(New Napalm(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F, -2.0F, 2.0F, 0.0F, 0.0F, 30))
                ElseIf internalcounter = 140 Then
                    jumping = True
                    veloc = JUMPSPEED
                ElseIf internalcounter = 205 Then
                    goright = False
                    lookleft = True
                ElseIf internalcounter = 209 Then
                    internalcounter = 0
                    atk = AtkSetSeven.TornadoBlow
                End If
            Case AtkSetSeven.TornadoBlow
                If internalcounter = 2 Then
                    For X = 5 To 400 Step 5
                        Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.5F + 75, -1.0F, 0.0F, 0.0F, 0.0F, X, 0, New Plasma(0.0F, 0.0F, Ri.Next(9) * 0.5F - 2.0F, -4.5F, 0.0F, 0.1F, 100)))
                    Next
                ElseIf internalcounter = 40 Then
                    Q.Monsters.Add(New Rubberet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4.0F, 0.0F, 0.0F, 0.0F, 800))
                    Q.Monsters.Add(New Rubberet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 25, -4.0F, 0.0F, 0.0F, 0.0F, 800))
                    Q.Monsters.Add(New Rubberet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 50, -4.0F, 0.0F, 0.0F, 0.0F, 800))
                ElseIf internalcounter = 240 Then
                    Q.Monsters.Add(New Rubberet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT, -4.0F, 0.0F, 0.0F, 0.0F, 800))
                    Q.Monsters.Add(New Rubberet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 25, -4.0F, 0.0F, 0.0F, 0.0F, 800))
                    Q.Monsters.Add(New Rubberet(mr.X + HALFWIDTH, mr.Y + HALFHEIGHT - 50, -4.0F, 0.0F, 0.0F, 0.0F, 800))
                ElseIf internalcounter = 400 Then
                    internalcounter = 0
                    atk = AtkSetSeven.PlasticForceField
                End If
        End Select


        If goleft Then
            mr.Offset(-MOVESPEED, 0)

            If mr.Right < le AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
        End If
        If goright Then
            mr.Offset(MOVESPEED, 0)

            If mr.Left > re AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each wall As RectangleF In Q.Walls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
        End If
        If jumping Then
            mr.Offset(0, veloc + GRAVITY)
            veloc += GRAVITY
            mr.Y = Convert.ToSingle(Math.Round(mr.Top * 1000.0F, 0)) / 1000.0F

            RV = LevelSuite.PlatformJump(Q.Platforms, mr, veloc, GRAVITY * 1.5F)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                End If
            End If

            RV = LevelSuite.WallJump(Q.Walls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Extrawalls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Groundwalls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Crates, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class PhaseOne
    Implements IMonster
    Private mr As RectangleF
    Public HP As Integer
    Private internalcounter As Integer
    Private Ri As Random
    Private goleft, goright, jumping, lookleft, attacked, recentlyhit, kill As Boolean
    Private veloc, le, re As Single
    Private petshot As Bullet
    Private hitwaitingtime As Integer

    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 32.0F
    Const MOVESPEED As Single = 4.0F
    Const LEFTHIDINGSPOT As Single = 1192.0F
    Const LEFTWALL As Single = 800.0F
    Const RIGHTWALL As Single = 2000.0F
    Const PT1 As Single = LEFTHIDINGSPOT - 160.0F
    Const PT2 As Single = LEFTHIDINGSPOT + 160.0F
    Const PT3 As Single = RIGHTHIDINGSPOT - 160.0F
    Const PT4 As Single = RIGHTHIDINGSPOT + 160.0F
    Const RIGHTHIDINGSPOT As Single = 1592.0F
    Const MEDIAN As Single = 0.5F * LEFTHIDINGSPOT + 0.5F * RIGHTHIDINGSPOT
    Const JUMPSPEED As Single = -8.38F
    Const HITWAITLENGTH As Integer = 80
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 150
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim N As Single = 2.0F
        Dim F As Single = 0.0F
        If jumping Then
            If Me.lookleft Then
                F = 8.0F
            Else
                F = 9.0F
            End If
            If attacked Then
                F += 20.0F
            End If
        ElseIf lookleft Then
            F = ((internalcounter Mod 16) \ 4)
            If attacked Then
                F += 20.0F
            End If
        Else
            F = 4.0F + ((internalcounter Mod 16) \ 4)  '10 to convert 0-9 into 0.0-0.9, 2 to expand the frame time.
            If attacked Then
                F += 20.0F
            End If
        End If
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F / 32.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F / 32.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, (F + 1) / 32.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        recentlyhit = True
        Select Case BY
            Case ShotTypes.Aquabullet
                hp -= 5
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Corkwheel
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ForceMod
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            Case ShotTypes.Napalm
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Pearl
                hp -= 12
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Plasma
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Rubberet
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Screwer
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ShockWave
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Stonebullet
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
        End Select
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 104
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Const GRAVITY As Single = 0.25F
        Dim RV As AutoSuiteReturn
        Dim Vx, Vy As Single
        If hp <= 0 Then
            kill = True
            Q.Monsters.Add(New PhaseTwo(2100.0F, 224.0F))
            Q.AutoMessages(0) = New AutoMessage(0, 13)
        End If
        If recentlyhit Then
            If mr.Left <= MEDIAN Then
                If mr.Left >= LEFTHIDINGSPOT Then
                    goleft = True
                    goright = False
                    lookleft = True
                    If mr.Left < PT2 AndAlso Not jumping Then
                        jumping = True
                        veloc = JUMPSPEED
                    End If
                Else
                    goright = True
                    goleft = False
                    lookleft = False
                    If mr.Left > PT1 AndAlso Not jumping Then
                        jumping = True
                        veloc = JUMPSPEED
                    End If
                End If
            Else
                If mr.Left >= RIGHTHIDINGSPOT Then
                    goleft = True
                    goright = False
                    lookleft = True
                    If mr.Left < PT4 AndAlso Not jumping Then
                        jumping = True
                        veloc = JUMPSPEED
                    End If
                Else
                    goright = True
                    goleft = False
                    lookleft = False
                    If mr.Left < PT3 AndAlso Not jumping Then
                        jumping = True
                        veloc = JUMPSPEED
                    End If
                End If
            End If
            If mr.Left - LEFTHIDINGSPOT <= MOVESPEED AndAlso LEFTHIDINGSPOT - mr.Left <= MOVESPEED Then
                goleft = False
                goright = False
                hitwaitingtime += 1
            ElseIf mr.Left - RIGHTHIDINGSPOT <= MOVESPEED AndAlso RIGHTHIDINGSPOT - mr.Left <= MOVESPEED Then
                goleft = False
                goright = False
                hitwaitingtime += 1
            End If
            If hitwaitingtime >= HITWAITLENGTH Then
                recentlyhit = False
            End If
        Else
            If lookleft Then
                goleft = True
                goright = False
            Else
                goleft = False
                goright = True
            End If
        End If
        If internalcounter Mod 400 = 0 Then
            petshot = Nothing
            If lookleft Then
                petshot = New Bullet(RIGHTWALL, mr.Y, -4.0F, 0.0F, 0.0F, 0.0F, 400)
                Q.Monsters.Add(petshot)
            Else
                petshot = New Bullet(LEFTWALL, mr.Y, 4.0F, 0.0F, 0.0F, 0.0F, 400)
                Q.Monsters.Add(petshot)
            End If
        ElseIf internalcounter Mod 25 = 0 Then
            Q.AddNegation(petshot.Loc.X, petshot.Loc.Y)
        ElseIf internalcounter Mod 100 = 84 Then
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
            Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
            Vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, -Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, -Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vx, -Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, Vy, Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vx, Vy, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
            Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -Vy, -Vx, 0, 0, 20, 10, New DirectBullet(0, 0, R, 1.0F, 600)))
        End If
        If goleft Then
            mr.Offset(-MOVESPEED, 0)

            If mr.Right < le AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    lookleft = False
                End If
            Next
        End If
        If goright Then
            mr.Offset(MOVESPEED, 0)

            If mr.Left > re AndAlso Not jumping Then
                jumping = True
                veloc = 0
            End If
            For Each wall As RectangleF In Q.Walls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                If mr.IntersectsWith(wall) Then
                    mr.Offset(wall.Left - mr.Right, 0)
                    lookleft = True
                End If
            Next
        End If
        If jumping Then
            mr.Offset(0, veloc + GRAVITY)
            veloc += GRAVITY
            mr.Y = Convert.ToSingle(Math.Round(mr.Top * 1000.0F, 0)) / 1000.0F

            RV = LevelSuite.PlatformJump(Q.Platforms, mr, veloc, GRAVITY * 1.5F)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                End If
            End If

            RV = LevelSuite.WallJump(Q.Walls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Extrawalls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Groundwalls, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
            RV = LevelSuite.WallJump(Q.Crates, mr, veloc, Not jumping)
            If RV.RetState > SuiteReturnType.NoReturn Then
                If RV.RetState = SuiteReturnType.Crush Then
                    jumping = True
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top + 0.6F)
                    veloc = 0
                ElseIf RV.RetState = SuiteReturnType.Landing Then
                    jumping = False
                    le = RV.IncValue.Left
                    re = RV.IncValue.Right
                    mr.Offset(0, RV.IncValue.Top - mr.Bottom)
                ElseIf RV.RetState = SuiteReturnType.HeadBanger Then
                    mr.Offset(0, RV.IncValue.Bottom - mr.Top)
                    veloc = 0
                End If
            End If
        End If
        internalcounter += 1

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class PhaseTwo
    Implements IMonster

    Private internalcounter, frame, ifctr As Integer
    Public HP As Integer
    Private mr As RectangleF
    Private stepleft, stepright, kill As Boolean
    Private atk As AtkSetEight

    Private Enum AtkSetEight
        ReflectBeam
        BeamCannon
        RoomNape
        GuardianSquirm
        MultifacetedBeam
        Chartrusen
        ChromeSpritz
    End Enum

    Const MYWIDTH As Single = 128.0F
    Const MYHEIGHT As Single = 256.0F
    Const ATKCOUNT As Integer = 7
    Const MAXHP As Integer = 2450, QTRHP As Integer = MAXHP \ 4
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = MAXHP
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim F As Single = (frame Mod 4) * 0.25F

        'vs(LV << 2) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 0.0F)
        'vs((LV << 2) + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, Addend + 0.0F, 1.0F)
        'vs((LV << 2) + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 0.0F)
        'vs((LV << 2) + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, Addend + 0.1F, 1.0F)
        If frame < 4 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.25F, 0.5)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F + 0.0F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, N, 0.0F, &HFFFFFFFF, F + 0.0F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, 0.0F, 0.0F, &HFFFFFFFF, F + 0.25F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, F + 0.25F, 1.0F)
        End If

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Select Case BY
            Case ShotTypes.Aquabullet
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Corkwheel
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ForceMod
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Napalm
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Pearl
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Plasma
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Rubberet
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Screwer
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ShockWave
                hp -= 1
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Stonebullet
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
        End Select

    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 105
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx, dy, vx, vy As Single
        Dim ang, theta As Double
        Dim expyr, LV As Integer
        If hp <= 0 Then
            kill = True
            Q.Monsters.Add(New PhaseThree(mr.X + 28.0F, 224.0F))
            Q.AutoMessages(0) = New AutoMessage(0, 14)
        End If
        If Not (stepleft Or stepright) Then
            If mr.Right > 2000 Then
                stepleft = True
            End If
        End If
        If hp < 2150 AndAlso hp > 2100 Then
            stepleft = True
        ElseIf hp < 1620 AndAlso hp > 1600 Then
            stepright = True
        ElseIf hp < 1460 AndAlso hp > 1400 Then
            stepleft = True
        ElseIf hp < 933 AndAlso hp > 900 Then
            stepright = True
        ElseIf hp < 300 AndAlso hp > 275 Then
            stepleft = True
        End If
        If stepleft Then
            mr.Offset(-1.0F, 0.0F)
            ifctr += 1
            If ifctr >= 4 Then
                ifctr = 0
                frame += 1
                If frame = 8 Then
                    frame = 0
                    stepleft = False
                End If
            End If
        ElseIf stepright Then
            If mr.Right < 2000 Then
                mr.Offset(1.0F, 0.0F)
            End If
            ifctr += 1
            If ifctr >= 4 Then
                ifctr = 0
                frame += 1
                If frame = 8 Then
                    frame = 0
                    stepleft = False
                End If
            End If
        End If
        If hp < QTRHP AndAlso internalcounter Mod 2 = 0 Then
            internalcounter += 2
        Else
            internalcounter += 1
        End If
        If atk = AtkSetEight.ReflectBeam Then
            If internalcounter >= 100 AndAlso internalcounter <= 200 AndAlso internalcounter Mod 5 = 0 Then
                dx = R.X - mr.X + 8.0F - MYWIDTH / 2
                dy = R.Y - mr.Y + 8.0F - MYHEIGHT / 4
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Rubberet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 4, CSng(4.5 * Math.Cos(ang)), CSng(4.5 * Math.Sin(ang)), 0, 0, 800))
            ElseIf internalcounter = 250 Then
                atk = CType((hp Mod ATKCOUNT), AtkSetEight)
                internalcounter = 0
            End If
        ElseIf atk = AtkSetEight.BeamCannon Then
            If internalcounter = 50 Then
                Q.Monsters.Add(New ForceMod(mr.X + 16, mr.Y, -2.1F, -0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X, mr.Y + 16, -2.2F, -0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 32, mr.Y, -1.9F, -0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X, mr.Y + 32, -2.2F, 0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 16, mr.Y + 48, -2.1F, 0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 48, mr.Y + 16, -1.8F, -0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 32, mr.Y + 48, -1.9F, 0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 48, mr.Y + 32, -1.8F, 0.1F, 0.0F, 0.0F, 400))
            ElseIf internalcounter = 100 Then
                Q.Monsters.Add(New ForceMod(mr.X + 16, mr.Y + 100, -2.1F, -0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X, mr.Y + 16 + 100, -2.2F, -0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 32, mr.Y + 100, -1.9F, -0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X, mr.Y + 32 + 100, -2.2F, 0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 16, mr.Y + 48 + 100, -2.1F, 0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 48, mr.Y + 16 + 100, -1.8F, -0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 32, mr.Y + 48 + 100, -1.9F, 0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 48, mr.Y + 32 + 100, -1.8F, 0.1F, 0.0F, 0.0F, 400))
            ElseIf internalcounter = 150 Then
                Q.Monsters.Add(New ForceMod(mr.X + 16, mr.Y + 200, -2.1F, -0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X, mr.Y + 16 + 200, -2.2F, -0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 32, mr.Y + 200, -1.9F, -0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X, mr.Y + 32 + 200, -2.2F, 0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 16, mr.Y + 48 + 200, -2.1F, 0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 48, mr.Y + 16 + 200, -1.8F, -0.1F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 32, mr.Y + 48 + 200, -1.9F, 0.2F, 0.0F, 0.0F, 400))
                Q.Monsters.Add(New ForceMod(mr.X + 48, mr.Y + 32 + 200, -1.8F, 0.1F, 0.0F, 0.0F, 400))
            ElseIf internalcounter = 160 Then
                atk = CType((hp Mod ATKCOUNT), AtkSetEight)
                internalcounter = 0
            End If
        ElseIf atk = AtkSetEight.RoomNape Then
            If internalcounter = 100 Then
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                dx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                dy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, -dx, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, -dy, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                dx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                dy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, -dx, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, -dy, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, -dy, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, dx, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, dy, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, -dx, 0, 0, 20, 10, New DirectNape(0, 0, R, 1.0F, 600)))
            ElseIf internalcounter = 250 Then
                atk = CType((hp Mod ATKCOUNT), AtkSetEight)
                internalcounter = 0
            End If
        ElseIf atk = AtkSetEight.GuardianSquirm Then
            If internalcounter = 10 Then
                Q.Monsters.Add(New Squirm(mr.X, mr.Y, 3.0F))
                Q.Monsters.Add(New Squirm(mr.X, mr.Y + 32, 3.0F))
                Q.Monsters.Add(New Squirm(mr.X, mr.Y + 64, 3.0F))
                Q.Monsters.Add(New Squirm(mr.X, mr.Y + 96, 3.0F))
                Q.Monsters.Add(New Squirm(mr.X, mr.Y + 128, 3.0F))
                Q.Monsters.Add(New Squirm(mr.X, mr.Y + 160, 3.0F))
                Q.Monsters.Add(New Squirm(mr.X, mr.Y + 192, 3.0F))
                Q.Monsters.Add(New Squirm(mr.X, mr.Y + 224, 3.0F))
            ElseIf internalcounter = 100 Then
                atk = CType((hp Mod ATKCOUNT), AtkSetEight)
                internalcounter = 0
            End If
        ElseIf atk = AtkSetEight.MultifacetedBeam Then
            If internalcounter = 20 Then
                dx = 500 - mr.X - MYWIDTH / 2
                dy = 0 - mr.Y - MYHEIGHT / 4
                ang = Math.Atan2(dy, dx)
                vx = R.X - 600
                vy = R.Y + 8 - 0
                theta = Math.Atan2(vy, vx)
                expyr = Convert.ToInt32(dx / Math.Cos(ang)) \ 4
                For LV = 0 To 19
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.25F, CSng(4 * Math.Cos(ang)), CSng(4 * Math.Sin(ang)), 0.0F, 0.0F, expyr, 200 + 8 * LV, New Plasma(0, 0, CSng(5 * Math.Cos(theta)), CSng(5 * Math.Sin(theta)), 0, 0, 500)))
                Next
            ElseIf internalcounter = 40 Then
                dx = 500 - mr.X - MYWIDTH / 2
                dy = 1000 - mr.Y - MYHEIGHT / 4
                ang = Math.Atan2(dy, dx)
                vx = R.X - 400
                vy = R.Y + 8 - 1000
                theta = Math.Atan2(vy, vx)
                expyr = Convert.ToInt32(dx / Math.Cos(ang)) \ 4
                For LV = 0 To 19
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.25F, CSng(4 * Math.Cos(ang)), CSng(4 * Math.Sin(ang)), 0.0F, 0.0F, expyr, 200 + 8 * LV, New ForceMod(0, 0, CSng(5 * Math.Cos(theta)), CSng(5 * Math.Sin(theta)), 0, 0, 500)))
                Next
            ElseIf internalcounter = 60 Then
                dx = 1800 - mr.X - MYWIDTH / 2
                dy = 0 - mr.Y - MYHEIGHT / 4
                ang = Math.Atan2(dy, dx)
                vx = R.X - 1800
                vy = R.Y + 100 - 0
                theta = Math.Atan2(vy, vx)
                expyr = Convert.ToInt32(dx / Math.Cos(ang)) \ 4
                For LV = 0 To 19
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH * 0.5F, mr.Y + MYHEIGHT * 0.25F, CSng(4 * Math.Cos(ang)), CSng(4 * Math.Sin(ang)), 0.0F, 0.0F, expyr, 200 + 8 * LV, New Aquabullet(0, 0, CSng(5 * Math.Cos(theta)), CSng(5 * Math.Sin(theta)), 0, 0, 500)))
                Next
            ElseIf internalcounter = 100 Then
                Q.Monsters.Add(New DirectBullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, R, 2.0F, 400))
                Q.Monsters.Add(New DirectBullet(mr.X + 0.5F * MYWIDTH, mr.Y + 0.5F * MYHEIGHT, R, 1.9F, 400))
            ElseIf internalcounter = 250 Then
                atk = CType((hp Mod ATKCOUNT), AtkSetEight)
                internalcounter = 0
            End If
        ElseIf atk = AtkSetEight.Chartrusen Then
            If internalcounter = 10 OrElse internalcounter = 110 Then
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, 20, 10, New DirectBullet(0, 0, R, 4, 60)))
                vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, vx, vy, 0, 0, 20, 10, New DirectNape(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, vy, -vx, 0, 0, 20, 10, New DirectNape(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -vx, -vy, 0, 0, 20, 10, New DirectNape(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -vy, vx, 0, 0, 20, 10, New DirectNape(0, 0, R, 4, 60)))
                vx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                vy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, vx, vy, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, vy, -vx, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -vx, -vy, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -vy, vx, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, vx, -vy, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, vy, vx, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -vx, vy, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
                Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -vy, -vx, 0, 0, 20, 10, New DirectAquaBullet(0, 0, R, 4, 60)))
            ElseIf internalcounter = 150 Then
                atk = CType((hp Mod ATKCOUNT), AtkSetEight)
                internalcounter = 0
            End If
        ElseIf atk = AtkSetEight.ChromeSpritz Then
            If internalcounter = 20 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang - 0.3)), CSng(6 * Math.Sin(ang - 0.3)), 0, 0, 350))
            ElseIf internalcounter = 40 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang + 0.3)), CSng(6 * Math.Sin(ang + 0.3)), 0, 0, 350))
            ElseIf internalcounter = 60 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang - 0.2)), CSng(6 * Math.Sin(ang - 0.2)), 0, 0, 350))
            ElseIf internalcounter = 80 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang + 0.2)), CSng(6 * Math.Sin(ang + 0.2)), 0, 0, 350))
            ElseIf internalcounter = 100 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang - 0.1)), CSng(6 * Math.Sin(ang - 0.1)), 0, 0, 350))
            ElseIf internalcounter = 120 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang + 0.1)), CSng(6 * Math.Sin(ang + 0.1)), 0, 0, 350))
            ElseIf internalcounter = 140 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang + 0.1)), CSng(6 * Math.Sin(ang + 0.1)), 0, 0, 350))
            ElseIf internalcounter = 160 Then
                dx = R.X + 8 - mr.X - MYWIDTH / 2
                dy = R.Y + 8 - mr.Y - MYHEIGHT / 2
                ang = Math.Atan2(dy, dx)
                Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, CSng(6 * Math.Cos(ang)), CSng(6 * Math.Sin(ang)), 0, 0, 350))
            ElseIf internalcounter = 200 Then
                atk = CType((hp Mod ATKCOUNT), AtkSetEight)
                internalcounter = 0
            End If
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class PhaseThree
    Implements IMonster

    Private internalcounter, secondcounter As Integer
    Private specialattack, commuting, goingleft, kill As Boolean
    Private mr As RectangleF
    Public HP As Integer

    Const MYWIDTH As Single = 72.0F
    Const MYHEIGHT As Single = 68.0F
    Const BOTTOMBOUND As Single = 108.0F
    Const TOPBOUND As Single = -892.0F
    Const LEFTBOUND As Single = 800.0F
    Const RIGHTBOUND As Single = 2000.0F - MYWIDTH
    Const HORIZSPEED As Single = 6.0F

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        hp = 400
        goingleft = True
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return kill
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer
        Const N As Single = 1.0F
        F = (internalcounter \ 4) Mod 2
        If mr.X = LEFTBOUND Then
            Select Case F
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.0F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
            End Select
        Else
            Select Case F
                Case 0
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                Case 1
                    vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.5F, 0.5F)
                    vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.5F, 1.0F)
                    vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.5F)
                    vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
            End Select
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        If Not commuting Then
            secondcounter = 128
        End If
        Select Case BY
            Case ShotTypes.Aquabullet
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Corkwheel
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ForceMod
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Napalm
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            Case ShotTypes.Pearl
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Plasma
                Q.SonicEvents.Add(SonicEventType.ShotDefend)
                Return True
            Case ShotTypes.Rubberet
                hp -= 2
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Screwer
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.ShockWave
                hp -= 10
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
            Case ShotTypes.Stonebullet
                hp -= 4
                Q.SonicEvents.Add(SonicEventType.ShotHit)
                Return True
        End Select
    End Function
    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property
    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 106
        End Get
    End Property
    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim dx, dy As Single
        Dim ang As Double
        Dim A, B, C As Integer
        Dim K, L, M As Microsoft.DirectX.Vector2
        If hp <= 0 Then
            kill = True
            Q.AutoMessages(4) = New AutoMessage(0, 15)
        End If
        If mr.Top > BOTTOMBOUND Then
            mr.Offset(0.0F, -4.0F)
            If mr.Top < BOTTOMBOUND Then
                mr.Y = BOTTOMBOUND
            End If
        ElseIf commuting Then
            If goingleft Then
                If mr.Y > TOPBOUND AndAlso mr.X > LEFTBOUND Then
                    mr.Offset(0.0F, -10.0F)
                ElseIf mr.X > LEFTBOUND Then
                    mr.Offset(-HORIZSPEED, 0.0F)
                Else
                    mr.Offset(0.0F, 10.0F)
                    If mr.Top >= BOTTOMBOUND Then
                        mr.Y = BOTTOMBOUND
                        mr.X = LEFTBOUND
                        commuting = False
                        goingleft = False
                        secondcounter = 0
                        specialattack = ((Convert.ToInt32(R.X + R.Y) \ 24) Mod 2 = 0)
                    End If
                End If
            Else
                If mr.Y > TOPBOUND AndAlso mr.X < RIGHTBOUND Then
                    mr.Offset(0.0F, -10.0F)
                ElseIf mr.X < RIGHTBOUND Then
                    mr.Offset(HORIZSPEED, 0.0F)
                Else
                    mr.Offset(0.0F, 10.0F)
                    If mr.Top >= BOTTOMBOUND Then
                        mr.Y = BOTTOMBOUND
                        mr.X = RIGHTBOUND
                        commuting = False
                        goingleft = True
                        secondcounter = 0
                        specialattack = ((Convert.ToInt32(R.X + R.Y) \ 24) Mod 2 = 0)
                    End If
                End If
            End If
        Else
            secondcounter += 1
            If specialattack Then
                If secondcounter = 5 Then
                    If mr.Left = LEFTBOUND Then
                        B = 5
                    Else
                        B = -5
                    End If
                    For A = 6 To 44
                        Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, B, 6.0F, 0.0F, -0.09F, 8 * A, 200 - 8 * A, New DirectNape(0, 0, R, 6.0F, 200)))
                    Next
                ElseIf secondcounter = 100 Then
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 8.0F, -4.0F, 0.0F, 0.5F, 400))
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 7.0F, -4.0F, 0.0F, 0.5F, 400))
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 6.0F, -4.0F, 0.0F, 0.5F, 400))
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 5.0F, -4.0F, 0.0F, 0.5F, 400))
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 4.0F, -4.0F, 0.0F, 0.5F, 400))
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 3.0F, -4.0F, 0.0F, 0.5F, 400))
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 2.0F, -4.0F, 0.0F, 0.5F, 400))
                    Q.Monsters.Add(New Corkwheel(mr.X + MYWIDTH, mr.Bottom, 1.0F, -4.0F, 0.0F, 0.5F, 400))
                ElseIf secondcounter = 50 Then
                    K = New Microsoft.DirectX.Vector2(R.Left + 500, R.Top)
                    L = New Microsoft.DirectX.Vector2(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT)
                    M = Microsoft.DirectX.Vector2.Subtract(K, L)
                    B = Convert.ToInt32(M.Length)
                    M.Normalize()
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, B * M.X / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, B * M.X / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
                    K = New Microsoft.DirectX.Vector2(R.Left - 500, R.Top)
                    M = Microsoft.DirectX.Vector2.Subtract(K, L)
                    B = Convert.ToInt32(M.Length)
                    M.Normalize()
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, M.X * B / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, M.X * B / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
                    K = New Microsoft.DirectX.Vector2(R.Left, R.Top + 500)
                    M = Microsoft.DirectX.Vector2.Subtract(K, L)
                    B = Convert.ToInt32(M.Length)
                    M.Normalize()
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, M.X * B / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, M.X * B / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
                    K = New Microsoft.DirectX.Vector2(R.Left, R.Top - 500)
                    M = Microsoft.DirectX.Vector2.Subtract(K, L)
                    B = Convert.ToInt32(M.Length)
                    M.Normalize()
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, M.X * B / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.1F, 200)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH, mr.Y + 0.5F * MYHEIGHT, M.X * B / 100.0F, M.Y * B / 100.0F, 0.0F, 0.0F, 100, 10, New DirectAcceleratorBullet(0.0F, 0.0F, R, 0.04F, 300)))
                End If
            Else
                If secondcounter = 2 Then
                    A = 50
                    B = 2
                    C = 6
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    dx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                    dy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, -dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, -dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    dx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                    dy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, -dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, -dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, -dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, -dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                ElseIf secondcounter = 20 Then
                    A = 60
                    B = 2
                    C = 6
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 4, 0, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, 4, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -4, 0, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, -4, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    dx = Convert.ToSingle(4 * Math.Cos(Math.PI / 4))
                    dy = Convert.ToSingle(4 * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, -dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, -dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    dx = Convert.ToSingle(4 * Math.Cos(Math.PI / 8))
                    dy = Convert.ToSingle(4 * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, -dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, -dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, -dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, dy, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                    Q.Monsters.Add(New CustomShot(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, -dx, 0, 0, A, B, New DirectBullet(0, 0, R, C, 600)))
                ElseIf secondcounter = 52 Then
                    A = 4
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, A, 0, 0, 0, 60))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, A, 0, 0, 60))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -A, 0, 0, 0, 60))
                    dx = Convert.ToSingle(A * Math.Cos(Math.PI / 4))
                    dy = Convert.ToSingle(A * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, 60))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, 60))
                    dx = Convert.ToSingle(A * Math.Cos(Math.PI / 8))
                    dy = Convert.ToSingle(A * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, 60))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, 60))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, dx, 0, 0, 60))
                    Q.Monsters.Add(New Screwer(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, dy, 0, 0, 60))
                ElseIf secondcounter = 70 Then
                    A = 4
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, A, 0, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, 0, A, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -A, 0, 0, 0, 60))
                    dx = Convert.ToSingle(A * Math.Cos(Math.PI / 4))
                    dy = Convert.ToSingle(A * Math.Sin(Math.PI / 4))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, 60))
                    dx = Convert.ToSingle(A * Math.Cos(Math.PI / 8))
                    dy = Convert.ToSingle(A * Math.Sin(Math.PI / 8))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dx, dy, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dy, dx, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, dy, dx, 0, 0, 60))
                    Q.Monsters.Add(New Bullet(mr.X + MYWIDTH / 2, mr.Y + MYHEIGHT / 2, -dx, dy, 0, 0, 60))
                End If
            End If
            If secondcounter >= 128 Then
                secondcounter = 0
                commuting = True
            End If
        End If

        internalcounter += 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property

End Class
#End Region
#Region "Bullets"
Public Class Bullet
    Implements IMonster
    Private Const RADIUS As Integer = 4

    Public Loc As PointF
    Public Veloc As PointF
    Public Accel As PointF

    Friend Timeout As Integer
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        Loc = New PointF(Xo, Yo)
        Veloc = New PointF(Vxo, Vyo)
        Accel = New PointF(Axo, Ayo)
        Me.Timeout = Timeout
    End Sub

    'Public Sub Draw(ByVal GFX As System.Drawing.Graphics, ByVal ScreenLeft As Integer, ByVal ScreenTop As Integer) Implements IMonster.Draw
    '    Loc.X -= ScreenLeft
    '    Loc.Y -= ScreenTop
    '    GFX.DrawEllipse(Pens.Red, Loc.X - RADIUS, Loc.Y - RADIUS, RADIUS + RADIUS, RADIUS + RADIUS)
    '    Loc.X += ScreenLeft
    '    Loc.Y += ScreenTop
    'End Sub

    Public Overridable Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.Contains(Loc)
    End Function

    Public Overridable Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Veloc.X += Accel.X
        Veloc.Y += Accel.Y
        Loc.X += Veloc.X
        Loc.Y += Veloc.Y
        Timeout -= 1
    End Sub

    Public Overridable Function Dead() As Boolean Implements IMonster.Dead
        Return Timeout <= 0
    End Function

    Public Overridable Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub

    Public Overridable ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return 2 * RADIUS
        End Get
    End Property

    Public Overridable ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 5
        End Get
    End Property

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(Loc.X, Loc.Y, 0.0F)
        End Get
    End Property

    Public Overridable ReadOnly Property Width() As Single Implements IMonster.Width
        Get

            Return 2 * RADIUS
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return New RectangleF(Loc, New SizeF(1, 1))
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function

    Public Overridable Sub Refocus(ByVal R As RectangleF, ByVal Q As LevelSuite)

    End Sub
End Class
Public Class SelfBullet
    Inherits Bullet

    Public Flying As Boolean

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub

    Public Overrides Function IntersectsWith(ByVal R As RectangleF) As Boolean
        Return False
    End Function

    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        MyBase.Move(R, Q)
        If MyBase.Timeout <= 0 Then
            Flying = False
        End If
    End Sub

End Class
Public Class DirectBullet
    Inherits Bullet
    Dim Spd As Single

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal R As RectangleF, ByVal Spd As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Spd, Spd, 0, 0, Timeout)
        Me.Spd = Spd
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Xo, Yo)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Veloc.X = C.X * Spd
        Veloc.Y = C.Y * Spd

    End Sub

    Public Overrides Sub Refocus(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Loc.X, Loc.Y)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Veloc.X = C.X * Spd
        Veloc.Y = C.Y * Spd

    End Sub
End Class
Public Class HomingShot
    Implements IMonster
    Private Const RADIUS As Integer = 4
    Private Const PI As Single = 3.1415926535897931
    Private Const HALFPI As Single = 3.1415926535897931 / 2.0F
    Public Loc As PointF
    Public Spd As Single
    Public Ang As Single  'In radians
    Public AngSpd As Single 'In rad/sec
    Private Timeout As Integer
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Spd As Single, ByVal AngSpd As Single, ByVal Timeout As Integer)
        Loc = New PointF(Xo, Yo)
        Me.Spd = Spd
        Me.AngSpd = AngSpd
        Ang = 1.5F
        Me.Timeout = Timeout
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return Timeout <= 0
    End Function
    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub
    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return 2 * RADIUS
        End Get
    End Property
    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 24
        End Get
    End Property
    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.Contains(Loc)
    End Function
    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim AngDest As Single
        Dim xm, ym As Single
        AngDest = Convert.ToSingle(Math.Atan2(R.Y + R.Height / 2 - Loc.Y, R.X + R.Width / 2 - Loc.X))
        If (Ang - AngDest) < PI Then
            If Ang > AngDest Then
                Ang -= AngSpd
            Else
                Ang += AngSpd
            End If
        Else
            If Ang > AngDest Then
                Ang += AngSpd
            Else
                Ang -= AngSpd
            End If
        End If
        If Ang > PI Then
            Ang -= PI * 2
        ElseIf Ang < -PI Then
            Ang += PI * 2
        End If
        xm = Convert.ToSingle(Math.Cos(Ang))
        ym = Convert.ToSingle(Math.Sin(Ang))
        Loc.X += Spd * xm
        Loc.Y += Spd * ym
        Timeout -= 1
    End Sub
    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(Loc.X, Loc.Y, 0.0F)
        End Get
    End Property
    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return 2 * RADIUS
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return New RectangleF(Loc, New SizeF(1, 1))
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function
End Class
Public Class CustomShot
    Implements IMonster
    'A custom shot is a shot that will travel in some direction (points)
    'at some speed (Veloc) for some time (Timeout)... then stop
    'for a set amount of time   '(Delay)
    'When this delay occurs, the custom shot will be 'dead'
    'and will release the monster - preferably a bullet (Spawn)
    Private Const RADIUS As Integer = 4

    Public Loc As PointF
    Public Veloc As PointF
    Public Accel As PointF
    Public Spawn As Bullet

    Friend Timeout As Integer
    Friend Delay As Integer

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer, ByVal Delay As Integer, ByVal Spew As Bullet)
        Loc = New PointF(Xo, Yo)
        Veloc = New PointF(Vxo, Vyo)
        Accel = New PointF(Axo, Ayo)
        Me.Timeout = Timeout
        Me.Delay = Delay
        Spawn = Spew
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return Timeout <= 0 AndAlso Delay <= 0
    End Function
    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function
    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return 2 * RADIUS
        End Get
    End Property
    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 27
        End Get
    End Property
    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return R.Contains(Loc)

    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        If Timeout > 0 Then
            Veloc.X += Accel.X
            Veloc.Y += Accel.Y
            Loc.X += Veloc.X
            Loc.Y += Veloc.Y
            Timeout -= 1
            If Timeout = 0 Then
                Delay -= 1
                If Delay <= 0 Then
                    Spawn.Loc.X = Loc.X
                    Spawn.Loc.Y = Loc.Y
                    Spawn.Refocus(R, Q)
                    Q.Monsters.Add(Spawn)
                End If
            End If
        Else
            Delay -= 1
            If Delay <= 0 Then
                Spawn.Loc.X = Loc.X
                Spawn.Loc.Y = Loc.Y
                Spawn.Refocus(R, Q)
                Q.Monsters.Add(Spawn)
            End If
        End If

    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(Loc.X, Loc.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return New RectangleF(Loc, New SizeF(1, 1))
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return 2 * RADIUS
        End Get
    End Property
End Class
Public Class RAShot
    Inherits Bullet

    Public Sub New(ByVal xo As Single, ByVal yo As Single, ByVal vxo As Single, ByVal vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(xo, yo, vxo, vyo, Axo, Ayo, Timeout)

    End Sub

    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Veloc.X += Accel.X
        Veloc.Y += Accel.Y
        Loc.X += Veloc.X
        Loc.Y += Veloc.Y
        Timeout -= 1
        If Loc.X > R.Left AndAlso Loc.X < R.Right Then
            If Veloc.X >= 0 Then
                If R.Top > Loc.X Then
                    Q.Monsters.Add(New Bullet(Loc.X, Loc.Y, 0, -Veloc.X, 0, 0, Timeout))
                Else
                    Q.Monsters.Add(New Bullet(Loc.X, Loc.Y, 0, Veloc.X, 0, 0, Timeout))
                End If
            Else
                If R.Top > Loc.X Then
                    Q.Monsters.Add(New Bullet(Loc.X, Loc.Y, 0, Veloc.X, 0, 0, Timeout))
                Else
                    Q.Monsters.Add(New Bullet(Loc.X, Loc.Y, 0, -Veloc.X, 0, 0, Timeout))
                End If
            End If
            Timeout = 0
        End If
    End Sub

    Public Overrides Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer)
        Const N As Single = 2.0F
        Dim cx, cy As Single
        cx = Convert.ToSingle(Math.Cos(Timeout / 25))
        cy = Convert.ToSingle(Math.Sin(Timeout / 25))
        vs(CI) = New CustomVertex.PositionColoredTextured(N * cx, N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(N * cy, N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N * -cy, N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N * -cx, N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub

    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 31
        End Get
    End Property
End Class
Public Class Aquabullet
    Inherits Bullet

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub

    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 36
        End Get
    End Property
End Class
Public Class ForceMod
    Inherits Bullet
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 68
        End Get
    End Property
End Class
Public Class SelfAquabullet
    Inherits SelfBullet
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub

    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 36
        End Get
    End Property

End Class
Public Class SelfForceMod
    Inherits SelfBullet
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub

    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 68
        End Get
    End Property


End Class
Public Class Napalm
    Inherits Bullet

    Private crashed As Boolean

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub

    Public Overrides Function Dead() As Boolean
        Return Timeout <= 0 OrElse crashed
    End Function

    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove
        MyBase.Move(R, Q)

        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Glasses
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        If crashed Then
            Q.AddExplosion(Loc.X, Loc.Y)
        ElseIf Timeout <= 0 Then
            Q.AddExplosion(Loc.X, Loc.Y)
        End If
    End Sub

    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 41
        End Get
    End Property

End Class
Public Class Rubberet
    Inherits Bullet

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub

    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove

        Veloc.X += Accel.X
        Loc.X += Veloc.X

        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Glasses
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next


        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        Veloc.Y += Accel.Y
        Loc.Y += Veloc.Y
        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Glasses
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        Timeout -= 1
    End Sub

    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 43
        End Get
    End Property

End Class
Public Class SelfNapalm
    Inherits SelfBullet
    Private crashed As Boolean
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides Function Dead() As Boolean
        Return Timeout <= 0 OrElse crashed
    End Function
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 41
        End Get
    End Property
    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove
        MyBase.Move(R, Q)

        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Glasses
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        If crashed Then
            Q.AddExplosion(Loc.X, Loc.Y)
            Flying = False
        ElseIf Timeout <= 0 Then
            Q.AddExplosion(Loc.X, Loc.Y)
            Flying = False

        End If
    End Sub

End Class
Public Class SelfRubberet
    Inherits SelfBullet
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 43
        End Get
    End Property
    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove
        Veloc.X += Accel.X
        Loc.X += Veloc.X

        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.Glasses
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.X = -Veloc.X
                Loc.X += Veloc.X
            End If
        Next
        Veloc.Y += Accel.Y
        Loc.Y += Veloc.Y
        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.Glasses
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            ElseIf opsw.Switch.Contains(Loc) Then
                Veloc.Y = -Veloc.Y
                Loc.Y += Veloc.Y
            End If
        Next
        Timeout -= 1
        If MyBase.Timeout <= 0 Then
            Flying = False
        End If

    End Sub

End Class
Public Class Screwer
    Inherits Bullet

    Private crashed As Boolean
    Private mr As RectangleF
    Const MYWIDTH As Single = 8.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const MYHEIGHT As Single = 8.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove
        If crashed Then
            Timeout -= 1
        Else
            MyBase.Move(R, Q)
            mr.X = Loc.X - HALFWIDTH
            mr.Y = Loc.Y - HALFHEIGHT

            For Each Wall In Q.Walls
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Extrawalls
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Groundwalls
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.EyeBeams
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.CrossStays
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Glasses
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Crates
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Datablk In Q.Conveyors
                If Datablk.Loc.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each MobileWall In Q.Mobilewalls
                If MobileWall.Loc.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each opsw In Q.RTDs
                If opsw.Gate.Contains(Loc) Then
                    crashed = True
                ElseIf opsw.Switch.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each opsw In Q.Pneumos
                If opsw.Gate.Contains(Loc) Then
                    crashed = True
                ElseIf opsw.Switch.Contains(Loc) Then
                    crashed = True
                End If
            Next
            If crashed Then
                Veloc.X -= Accel.X
                Veloc.Y -= Accel.Y
                Loc.X -= (2.0F * Veloc.X)
                Loc.Y -= (2.0F * Veloc.Y)
                mr.X = Loc.X - HALFWIDTH
                mr.Y = Loc.Y - HALFHEIGHT
            End If

        End If

    End Sub
    Public Overrides Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean
        Return mr.IntersectsWith(R)
    End Function
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 51
        End Get
    End Property
    Public Overrides Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer)
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub


End Class
Public Class SelfScrewer
    Inherits SelfBullet

    Private crashed As Boolean
    Private mr As RectangleF
    Const MYWIDTH As Single = 8.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const MYHEIGHT As Single = 8.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub




    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove
        If crashed Then
            Timeout -= 1
            If Timeout <= 0 Then
                Flying = False
            End If
        Else
            MyBase.Move(R, Q)

            For Each Wall In Q.Walls
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Extrawalls
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Groundwalls
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.CrossStays
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.EyeBeams
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Glasses
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Wall In Q.Crates
                If Wall.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each Datablk In Q.Conveyors
                If Datablk.Loc.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each MobileWall In Q.Mobilewalls
                If MobileWall.Loc.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each opsw In Q.RTDs
                If opsw.Gate.Contains(Loc) Then
                    crashed = True
                ElseIf opsw.Switch.Contains(Loc) Then
                    crashed = True
                End If
            Next
            For Each opsw In Q.Pneumos
                If opsw.Gate.Contains(Loc) Then
                    crashed = True
                ElseIf opsw.Switch.Contains(Loc) Then
                    crashed = True
                End If
            Next
            If crashed Then
                Veloc.X -= Accel.X
                Veloc.Y -= Accel.Y
                Loc.X -= (2.0F * Veloc.X)
                Loc.Y -= (2.0F * Veloc.Y)
                mr.X = Loc.X - HALFWIDTH
                mr.Y = Loc.Y - HALFHEIGHT
            End If
        End If

    End Sub
    Public Overrides Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean
        Return mr.IntersectsWith(R)
    End Function
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 51
        End Get
    End Property
    Public Overrides Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer)
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub


End Class
Public Class Lightball
    Implements IMonster
    Public Event FoundPlayer(ByVal Q As LevelSuite, ByVal P As PointF)

    Private Const RADIUS As Integer = 4

    Public Loc As PointF
    Public Veloc As PointF
    Public Accel As PointF

    Private attenuate As Boolean
    Private src As IOpticalTool
    Private Timeout As Integer
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer, ByRef Source As IOpticalTool)
        Loc = New PointF(Xo, Yo)
        Veloc = New PointF(Vxo, Vyo)
        Accel = New PointF(Axo, Ayo)
        Me.Timeout = Timeout
        src = Source
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return attenuate Or Timeout <= 0

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return RADIUS << 1
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 23
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return False
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Dim W, X, C, E, P As RectangleF
        Dim Max As Integer
        Dim LV As Integer = 0
        Veloc.X += Accel.X
        Veloc.Y += Accel.Y
        Loc.X += Veloc.X
        Loc.Y += Veloc.Y
        Timeout -= 1

        Max = Math.Max(Math.Max(Math.Max(Math.Max(Q.Walls.Count, Q.Extrawalls.Count), Q.Crates.Count), Q.RTDs.Count), Q.Pneumos.Count)

        Do
            If LV < Q.Walls.Count Then
                W = DirectCast(Q.Walls(LV), RectangleF)
            Else
                W = RectangleF.Empty
            End If
            If LV < Q.Extrawalls.Count Then
                X = DirectCast(Q.Extrawalls(LV), RectangleF)
            Else
                X = RectangleF.Empty
            End If
            If LV < Q.Crates.Count Then
                C = DirectCast(Q.Crates(LV), RectangleF)
            Else
                C = RectangleF.Empty
            End If
            If LV < Q.RTDs.Count Then
                E = DirectCast(Q.RTDs(LV), OpenerSwitch).Gate
            Else
                E = RectangleF.Empty
            End If
            If LV < Q.Pneumos.Count Then
                E = DirectCast(Q.Pneumos(LV), OpenerSwitch).Gate
            Else
                E = RectangleF.Empty
            End If
            If W.Contains(Loc) OrElse X.Contains(Loc) OrElse C.Contains(Loc) OrElse E.Contains(Loc) OrElse P.Contains(Loc) Then
                attenuate = True
            End If

            LV += 1
        Loop Until attenuate OrElse LV >= Max
        If R.Contains(Loc) Then
            src.Illumination(Q, Loc)
        End If
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(Loc.X, Loc.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return RADIUS << 1
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return New RectangleF(Loc, New SizeF(1, 1))
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function
End Class
Public Class DirectAcceleratorBullet
    Inherits Bullet
    Dim Acc As Single

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal R As RectangleF, ByVal Acc As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, 0, 0, Acc, Acc, Timeout)
        Me.Acc = Acc
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Xo, Yo)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Accel.X = C.X * Acc
        Accel.Y = C.Y * Acc

    End Sub

    Public Overrides Sub Refocus(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Loc.X, Loc.Y)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Accel.X = C.X * Acc
        Accel.Y = C.Y * Acc

    End Sub
End Class
Public Class Corkwheel
    Implements IMonster

    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private falling As Boolean, onslope As Boolean
    Private fallspeed As Single, g, fa As Single

    Private Timeout As Integer

    Const MYWIDTH As Integer = 14, MYHEIGHT As Integer = 14
    Const NUMFRAMES As Integer = 8

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        spd = Vxo
        fallspeed = Vyo
        g = Ayo
        fa = Axo
        Me.Timeout = Timeout
        falling = True
    End Sub

    Public Function Dead() As Boolean Implements IMonster.Dead
        Return Timeout <= 0

    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = Me.internalcounter >> 2
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.25F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.25F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 1.0F)

    End Sub

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 13
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        Const GRAVITY As Single = 0.8F, GRAVITYTH As Single = GRAVITY * 1.5F
        Const TERMINALVELOCITY As Integer = 40

        mr.Offset(spd, 0)
        For Each Wall As RectangleF In Q.Walls
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Extrawalls
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.EyeBeams
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.CrossStays
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Groundwalls
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Crates
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Glasses
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    spd = -spd
                End If
            End If
        Next
        If mr.Right <= le Then
            falling = True
        ElseIf mr.Left >= re Then
            falling = True
        End If

        If falling Then

            mr.Offset(0, fallspeed + GRAVITY / 2.0F)
            'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
            fallspeed += GRAVITY
            'dV = (PlayerVeloc +=) A (GRAVITY)
            'Moves the character according to its velocity, gravity, and location.

            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.

                        falling = False
                        'Hit the ground, we are not jumping.
                        fallspeed = 0

                        le = Platform.Left : re = Platform.Right
                        'Set the endpoints for the platform.
                        mr.Offset(0, Platform.Top - mr.Bottom)
                        'Move the player onto this platform.
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    falling = False
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
            For Each wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(Wall) Then
                    falling = False
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    mr.Offset(0, Wall.Top - mr.Bottom)
                End If
            Next
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
        Me.Timeout -= 1
        spd += fa
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)

        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property
    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function
End Class
Public Class SelfCorkwheel
    Inherits SelfBullet
    'Standard declarations.
    Private mr As RectangleF
    Private internalcounter As Integer

    'Specific declarations - may require some intervention.

    Private le As Single, re As Single
    Private spd As Single
    Private falling As Boolean, onslope As Boolean
    Private fallspeed As Single, fa As Single


    Const MYWIDTH As Integer = 14, MYHEIGHT As Integer = 14
    Const NUMFRAMES As Integer = 8

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
        spd = Vxo
        fallspeed = Vyo
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
    End Sub

    Public Overrides Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer)
        Dim F As Integer = Me.internalcounter >> 2
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.25F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.25F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 1.0F)

    End Sub
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 13
        End Get
    End Property
    Public Overrides Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean
        Return mr.IntersectsWith(R)
    End Function
    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Gravityth As Single = MyBase.Accel.Y * 1.5F
        Const TERMINALVELOCITY As Integer = 40

        mr.Offset(spd, 0)
        For Each Wall As RectangleF In Q.Walls
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Extrawalls
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.EyeBeams
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.CrossStays
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Groundwalls
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Crates
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        For Each wall As RectangleF In Q.Glasses
            'Check with each wall.
            If mr.IntersectsWith(Wall) Then
                If spd < 0 Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                ElseIf spd >= 0 Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    spd = -spd
                End If
            End If
        Next
        If mr.Right <= le Then
            falling = True
        ElseIf mr.Left >= re Then
            falling = True
        End If

        If falling Then

            mr.Offset(0, fallspeed + MyBase.Accel.Y / 2.0F)
            'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
            fallspeed += MyBase.Accel.Y
            'dV = (PlayerVeloc +=) A (GRAVITY)
            'Moves the character according to its velocity, gravity, and location.

            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - Gravityth <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.

                        falling = False
                        'Hit the ground, we are not jumping.
                        fallspeed = 0

                        le = Platform.Left : re = Platform.Right
                        'Set the endpoints for the platform.
                        mr.Offset(0, Platform.Top - mr.Bottom)
                        'Move the player onto this platform.
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
            For Each wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.
                    falling = False
                    'Stop the jump.
                    fallspeed = 0
                    le = Wall.Left : re = Wall.Right
                    'Set the endpoints.
                    mr.Offset(0, Wall.Top - mr.Bottom)
                    'Position the player to stand on this wall.
                End If
            Next
        End If

        Me.internalcounter += 1
        If Me.internalcounter >= NUMFRAMES Then
            Me.internalcounter = 0
        End If
        Me.Timeout -= 1
        If Timeout <= 0 Then
            Flying = False
        End If
        Veloc.X += Accel.X

        Loc.X = mr.X
        Loc.Y = mr.Y
    End Sub
    Public Overrides ReadOnly Property Height() As Single
        Get
            Return MYHEIGHT
        End Get
    End Property
    Public Overrides ReadOnly Property Width() As Single
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class StoneBullet
    Inherits Bullet
    Private crashed As Boolean
    Private mr As RectangleF
    Const MYWIDTH As Single = 12.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const MYHEIGHT As Single = 12.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove
        MyBase.Move(R, Q)
        mr.X = Loc.X - HALFWIDTH
        mr.Y = Loc.Y - HALFHEIGHT

        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.EyeBeams
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        If crashed Then
            Loc.X -= (2.0F * Veloc.X)
            Loc.Y -= (2.0F * Veloc.Y)
            Veloc.X = -Veloc.X / 4.0F
            Veloc.Y = -Veloc.Y / 4.0F
            Accel.Y = 0.5F
            mr.X = Loc.X - HALFWIDTH
            mr.Y = Loc.Y - HALFHEIGHT
        End If


    End Sub
    Public Overrides Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean
        Return mr.IntersectsWith(R)
    End Function
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 78
        End Get
    End Property
    Public Overrides Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer)
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub

End Class
Public Class SelfStoneBullet
    Inherits SelfBullet

    Private crashed As Boolean
    Private mr As RectangleF
    Const MYWIDTH As Single = 12.0F
    Const HALFWIDTH As Single = MYWIDTH / 2.0F
    Const MYHEIGHT As Single = 12.0F
    Const HALFHEIGHT As Single = MYHEIGHT / 2.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub




    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim Wall As RectangleF, opsw As OpenerSwitch, Datablk As DataBlock, MobileWall As IWallmove
        MyBase.Move(R, Q)
        If Timeout <= 0 Then
            Flying = False
        End If

        For Each Wall In Q.Walls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Extrawalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Groundwalls
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.EyeBeams
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.CrossStays
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Wall In Q.Crates
            If Wall.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each Datablk In Q.Conveyors
            If Datablk.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each MobileWall In Q.Mobilewalls
            If MobileWall.Loc.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.RTDs
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        For Each opsw In Q.Pneumos
            If opsw.Gate.Contains(Loc) Then
                crashed = True
            ElseIf opsw.Switch.Contains(Loc) Then
                crashed = True
            End If
        Next
        If crashed Then
            Loc.X -= (2.0F * Veloc.X)
            Loc.Y -= (2.0F * Veloc.Y)
            Veloc.X = -Veloc.X / 4.0F
            Veloc.Y = -Veloc.Y / 4.0F
            Accel.Y = 0.5F
            mr.X = Loc.X - HALFWIDTH
            mr.Y = Loc.Y - HALFHEIGHT
        End If

    End Sub
    Public Overrides Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean
        Return mr.IntersectsWith(R)
    End Function
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 78
        End Get
    End Property
    Public Overrides Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer)
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(-N, -N, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(-N, N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N, -N, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N, N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub


End Class
Public Class Maser
    Implements IMonster
    Private mr As RectangleF
    Private velx As Single
    Private timeout As Integer

    Const MYWIDTH As Single = 64.0F
    Const MYHEIGHT As Single = 16.0F
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vx As Single, ByVal TimeOut As Integer)
        mr = New RectangleF(Xo, Yo, MYWIDTH, MYHEIGHT)
        velx = Vx
        Me.timeout = TimeOut
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return timeout <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 1.0F
        vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 81
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        mr.Offset(velx, 0)
        timeout -= 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Plasma
    Inherits Bullet

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 93
        End Get
    End Property
End Class
Public Class SelfPlasma
    Inherits SelfBullet

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 93
        End Get
    End Property

End Class
Public Class SelfPearl
    Inherits SelfBullet
    Private attenuate As Boolean

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single, ByVal Vyo As Single, ByVal Axo As Single, ByVal Ayo As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Vxo, Vyo, Axo, Ayo, Timeout)
    End Sub
    Public Overrides ReadOnly Property ID() As Integer
        Get
            Return 23
        End Get
    End Property

    Public Overrides Sub Move(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim W, X, C, E, P As RectangleF
        Dim Max As Integer
        Dim LV As Integer = 0
        MyBase.Move(R, Q)

        Max = Math.Max(Math.Max(Math.Max(Math.Max(Q.Walls.Count, Q.Extrawalls.Count), Q.Crates.Count), Q.RTDs.Count), Q.Pneumos.Count)

        Do
            If LV < Q.Walls.Count Then
                W = DirectCast(Q.Walls(LV), RectangleF)
            Else
                W = RectangleF.Empty
            End If
            If LV < Q.Extrawalls.Count Then
                X = DirectCast(Q.Extrawalls(LV), RectangleF)
            Else
                X = RectangleF.Empty
            End If
            If LV < Q.Crates.Count Then
                C = DirectCast(Q.Crates(LV), RectangleF)
            Else
                C = RectangleF.Empty
            End If
            If LV < Q.RTDs.Count Then
                E = DirectCast(Q.RTDs(LV), OpenerSwitch).Gate
            Else
                E = RectangleF.Empty
            End If
            If LV < Q.Pneumos.Count Then
                E = DirectCast(Q.Pneumos(LV), OpenerSwitch).Gate
            Else
                E = RectangleF.Empty
            End If
            If W.Contains(Loc) OrElse X.Contains(Loc) OrElse C.Contains(Loc) OrElse E.Contains(Loc) OrElse P.Contains(Loc) Then
                attenuate = True
            End If

            LV += 1
        Loop Until attenuate OrElse LV >= Max
        If attenuate Then
            Flying = False
        End If
    End Sub

End Class
Public Class DirectNape
    Inherits Napalm
    Dim Spd As Single

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal R As RectangleF, ByVal Spd As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Spd, Spd, 0, 0, Timeout)
        Me.Spd = Spd
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Xo, Yo)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Veloc.X = C.X * Spd
        Veloc.Y = C.Y * Spd

    End Sub

    Public Overrides Sub Refocus(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Loc.X, Loc.Y)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Veloc.X = C.X * Spd
        Veloc.Y = C.Y * Spd

    End Sub
End Class
Public Class DirectAquaBullet
    Inherits Aquabullet
    Dim Spd As Single
    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal R As RectangleF, ByVal Spd As Single, ByVal Timeout As Integer)
        MyBase.New(Xo, Yo, Spd, Spd, 0, 0, Timeout)
        Me.Spd = Spd
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Xo, Yo)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Veloc.X = C.X * Spd
        Veloc.Y = C.Y * Spd

    End Sub

    Public Overrides Sub Refocus(ByVal R As RectangleF, ByVal Q As LevelSuite)
        Dim A, B, C As Microsoft.DirectX.Vector2

        A = New Microsoft.DirectX.Vector2(R.Left, R.Top)
        B = New Microsoft.DirectX.Vector2(Loc.X, Loc.Y)
        C = Microsoft.DirectX.Vector2.Subtract(A, B)
        C.Normalize()
        Veloc.X = C.X * Spd
        Veloc.Y = C.Y * Spd

    End Sub


End Class

#End Region
#Region "Gallery"
Public Class Explosion
    Implements IMonster

    Private mr As RectangleF
    Private timeout As Integer

    Const MYWIDTH As Single = 128.0F
    Const MYHEIGHT As Single = 128.0F
    Const FULLTIME As Integer = 32

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo - MYWIDTH / 2.0F, Yo - MYHEIGHT / 2.0F, MYWIDTH, MYHEIGHT)
        timeout = FULLTIME
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return timeout <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = 7 - (timeout \ 4)
        Dim N As Single = 1.0F

        If F >= 4 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, (F - 4) * 0.25F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, (F - 4) * 0.25F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F - 3) * 0.25F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F - 3) * 0.25F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.25F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.25F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 0.5F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 42

        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return (timeout Mod 10 = 1) AndAlso mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        timeout -= 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Negation
    Implements IMonster

    Private mr As RectangleF
    Private timeout As Integer

    Const MYWIDTH As Single = 128.0F
    Const MYHEIGHT As Single = 128.0F
    Const FULLTIME As Integer = 32

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single)
        mr = New RectangleF(Xo - MYWIDTH / 2.0F, Yo - MYHEIGHT / 2.0F, MYWIDTH, MYHEIGHT)
        timeout = FULLTIME
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return timeout <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Dim F As Integer = 7 - (timeout \ 4)
        Dim N As Single = 1.0F

        If F >= 4 Then
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, (F - 4) * 0.25F, 0.5F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, (F - 4) * 0.25F, 1.0F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F - 3) * 0.25F, 0.5F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F - 3) * 0.25F, 1.0F)
        Else
            vs(CI) = New CustomVertex.PositionColoredTextured(0.0F, 0.0F, 0.0F, &HFFFFFFFF, F * 0.25F, 0.0F)
            vs(CI + 1) = New CustomVertex.PositionColoredTextured(0.0F, 2.0F * N, 0.0F, &HFFFFFFFF, F * 0.25F, 0.5F)
            vs(CI + 2) = New CustomVertex.PositionColoredTextured(2.0F * N, 0.0F, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 0.0F)
            vs(CI + 3) = New CustomVertex.PositionColoredTextured(2.0F * N, 2.0F * N, 0.0F, &HFFFFFFFF, (F + 1) * 0.25F, 0.5F)
        End If
    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False
    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 100

        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return (timeout Mod 10 = 1) AndAlso mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        timeout -= 1
    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr
        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH
        End Get
    End Property
End Class
Public Class Shard
    Implements IMonster

    Private mr As RectangleF
    Private timeout As Integer
    Private ang, dang As Single
    Private fallspeed, le, re, vx As Single
    Private falling, goingleft As Boolean

    Const MYWIDTH As Single = 16.0F
    Const MYHEIGHT As Single = 16.0F
    Const FULLTIME As Integer = 256

    Public Sub New(ByVal Xo As Single, ByVal Yo As Single, ByVal Vxo As Single)
        mr = New RectangleF(Xo - MYWIDTH / 2.0F, Yo - MYHEIGHT / 2.0F, MYWIDTH, MYHEIGHT)
        timeout = FULLTIME
        dang = 1.0F
        vx = Vxo
        goingleft = (Vxo < 0)
        falling = True
    End Sub
    Public Function Dead() As Boolean Implements IMonster.Dead
        Return timeout <= 0
    End Function

    Public Sub Draw(ByRef vs() As Microsoft.DirectX.Direct3D.CustomVertex.PositionColoredTextured, ByVal CI As Integer) Implements IMonster.Draw
        Const N As Single = 2.0F
        Dim cx, cy As Single
        cx = Convert.ToSingle(Math.Cos(ang / 4))
        cy = Convert.ToSingle(Math.Sin(ang / 4))
        vs(CI) = New CustomVertex.PositionColoredTextured(N * cx, N * cy, 0.0F, &HFFFFFFFF, 0.0F, 0.0F)
        vs(CI + 1) = New CustomVertex.PositionColoredTextured(N * cy, N * -cx, 0.0F, &HFFFFFFFF, 0.0F, 1.0F)
        vs(CI + 2) = New CustomVertex.PositionColoredTextured(N * -cy, N * cx, 0.0F, &HFFFFFFFF, 1.0F, 0.0F)
        vs(CI + 3) = New CustomVertex.PositionColoredTextured(N * -cx, N * -cy, 0.0F, &HFFFFFFFF, 1.0F, 1.0F)

    End Sub

    Public Function Harm(ByVal Q As LevelSuite, ByVal BY As ShotTypes, ByVal ISpd As Single) As Boolean Implements IMonster.Harm
        Return False

    End Function

    Public ReadOnly Property Height() As Single Implements IMonster.Height
        Get
            Return MYHEIGHT
        End Get
    End Property

    Public ReadOnly Property ID() As Integer Implements IMonster.ID
        Get
            Return 50
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As System.Drawing.RectangleF) As Boolean Implements IMonster.IntersectsWith
        Return (timeout Mod 10 = 1) AndAlso mr.IntersectsWith(R)
    End Function

    Public Sub Move(ByVal R As System.Drawing.RectangleF, ByVal Q As LevelSuite) Implements IMonster.Move
        timeout -= 1
        ang += dang

        Const GRAVITY As Single = 0.7F, GRAVITYTH As Single = GRAVITY * 1.5F
        Const TERMINALVELOCITY As Single = 20.0F

        If goingleft Then
            mr.Offset(vx, 0)
            For Each Wall As RectangleF In Q.Walls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Glasses
                'Check with each wall.
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Right - mr.Left, 0)
                    'Move the player to the edge of this wall.
                    goingleft = False
                    vx = -vx
                End If
            Next
            If mr.Right <= le Then
                falling = True
            End If
        Else
            mr.Offset(vx, 0)
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.EyeBeams
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.CrossStays
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                    vx = -vx
                End If
            Next
            For Each Wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(Wall) Then
                    mr.Offset(Wall.Left - mr.Right, 0)
                    'Move the player to the edge of this wall.
                    goingleft = True
                    vx = -vx
                End If
            Next
            If mr.Left >= re Then
                falling = True
            End If
        End If
        If falling Then

            mr.Offset(0, fallspeed + GRAVITY / 2.0F)
            'dD (.OffSet) = Vo (PlayerVeloc) + 0.5 * A (GRAVITY >> 1)
            fallspeed += GRAVITY
            'dV = (PlayerVeloc +=) A (GRAVITY)
            'Moves the character according to its velocity, gravity, and location.

            If fallspeed >= TERMINALVELOCITY Then  'We're in TV.
                fallspeed = TERMINALVELOCITY  'Limit the velocity.
            End If

            For Each Platform As RectangleF In Q.Platforms
                'Looking at each platform in the platform array.
                If mr.Bottom >= Platform.Top AndAlso mr.Bottom - fallspeed - GRAVITYTH <= Platform.Top Then
                    If mr.Left < Platform.Right AndAlso mr.Right > Platform.Left Then
                        'The player has landed on this platform.  He needs to be updated.
                        'See how many advantages there are to using a Rectangle.

                        If fallspeed < 4 Then
                            falling = False
                            'Hit the ground, we are not jumping.
                            fallspeed = 0

                            le = Platform.Left : re = Platform.Right
                            'Set the endpoints for the platform.
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            'Move the player onto this platform.
                        Else
                            mr.Offset(0, Platform.Top - mr.Bottom)
                            'Move the player onto this platform.
                            fallspeed = -(fallspeed * 0.95F)
                            dang *= 0.5F
                            vx *= 0.5F
                            'Bounce back.
                        End If


                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Walls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.

                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)

                        fallspeed = 0

                    ElseIf fallspeed < 4 Then
                        falling = False
                        'Stop the jump.
                        fallspeed = 0


                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        fallspeed = -(fallspeed * 0.95F)
                        dang *= 0.5F
                        vx *= 0.5F
                        'Bounce back.
                    End If


                End If
            Next
            For Each Wall As RectangleF In Q.Groundwalls
                If mr.IntersectsWith(Wall) Then
                    'If player is falling, then the player will land on the wall.

                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)

                        fallspeed = 0

                    ElseIf fallspeed < 4 Then
                        falling = False
                        'Stop the jump.
                        fallspeed = 0


                        le = Wall.Left : re = Wall.Right
                        'Set the endpoints.

                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        'Position the player to stand on this wall.
                        fallspeed = -(fallspeed * 0.95F)
                        dang *= 0.5F
                        vx *= 0.5F
                        'Bounce back.
                    End If

                End If

            Next
            For Each Wall As RectangleF In Q.Extrawalls
                If mr.IntersectsWith(Wall) Then
                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)
                        fallspeed = 0
                    ElseIf fallspeed < 4 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        fallspeed = -(fallspeed * 0.95F)
                        dang *= 0.5F
                        vx *= 0.5F
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Crates
                If mr.IntersectsWith(Wall) Then
                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)
                        fallspeed = 0
                    ElseIf fallspeed < 4 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        fallspeed = -(fallspeed * 0.95F)
                        dang *= 0.5F
                        vx *= 0.5F
                    End If
                End If
            Next
            For Each Wall As RectangleF In Q.Glasses
                If mr.IntersectsWith(Wall) Then
                    If fallspeed < 0 Then
                        mr.Offset(0, wall.Bottom - mr.Top)
                        fallspeed = 0
                    ElseIf fallspeed < 4 Then
                        falling = False
                        fallspeed = 0
                        le = Wall.Left : re = Wall.Right
                        mr.Offset(0, Wall.Top - mr.Bottom)
                    Else
                        mr.Offset(0, Wall.Top - mr.Bottom)
                        fallspeed = -(fallspeed * 0.95F)
                        dang *= 0.5F
                        vx *= 0.5F
                    End If
                End If
            Next
        End If



    End Sub

    Public ReadOnly Property Position() As Microsoft.DirectX.Vector3 Implements IMonster.Position
        Get
            Return New Microsoft.DirectX.Vector3(mr.X, mr.Y, 0.0F)
        End Get
    End Property

    Public ReadOnly Property Rect() As System.Drawing.RectangleF Implements IMonster.Rect
        Get
            Return mr

        End Get
    End Property

    Public ReadOnly Property Width() As Single Implements IMonster.Width
        Get
            Return MYWIDTH

        End Get
    End Property
End Class
#End Region

'For LV = 0 To MM.GetUpperBound(0)
'    MM(LV).DrawAllSubsetsXF(D9)
'Next
'D9.Transform.World = Matrix.Identity
'YES ZBufferWrite           NO! PtSprite                ??? AlphaBlend


'NO! ZBufferWrite           YES PtSprite                ??? AlphaBlend


'If NUp Then
'    If AngleEv < PiHalf - (Pi / 30) Then
'        AngleEv += (Pi / 30)
'    End If
'    FocusPt.X = Camera.X + Convert.ToSingle(Math.Cos(AnglePl) * Math.Cos(AngleEv))
'    FocusPt.Z = Camera.Z + Convert.ToSingle(Math.Sin(AnglePl) * Math.Cos(AngleEv))
'    FocusPt.Y = Camera.Y + Convert.ToSingle(Math.Sin(AngleEv))

'End If
'If NDwn Then
'    If AngleEv > (Pi / 30) - PiHalf Then

'        AngleEv -= (Pi / 30)
'    End If
'    FocusPt.X = Camera.X + Convert.ToSingle(Math.Cos(AnglePl) * Math.Cos(AngleEv))
'    FocusPt.Z = Camera.Z + Convert.ToSingle(Math.Sin(AnglePl) * Math.Cos(AngleEv))
'    FocusPt.Y = Camera.Y + Convert.ToSingle(Math.Sin(AngleEv))
'End If
'If Dwn Then
'    dX = Convert.ToSingle(Math.Sin(AnglePl))
'    dY = Convert.ToSingle(Math.Cos(AnglePl))
'    Camera.X += dX
'    Camera.Z -= dY
'    FocusPt.X += dX
'    FocusPt.Z -= dY

'End If
'If Up Then
'    dX = Convert.ToSingle(Math.Sin(AnglePl))
'    dY = Convert.ToSingle(Math.Cos(AnglePl))
'    Camera.X -= dX
'    Camera.Z += dY
'    FocusPt.X -= dX
'    FocusPt.Z += dY

'End If
'If NLft Then
'    AnglePl += (Pi / 30)
'    FocusPt.X = Camera.X + Convert.ToSingle(Math.Cos(AnglePl))
'    FocusPt.Z = Camera.Z + Convert.ToSingle(Math.Sin(AnglePl))
'End If
'If NRght Then
'    AnglePl -= (Pi / 30)
'    FocusPt.X = Camera.X + Convert.ToSingle(Math.Cos(AnglePl))
'    FocusPt.Z = Camera.Z + Convert.ToSingle(Math.Sin(AnglePl))
'End If




'For Each Platform In Q.Platforms

'    If Player.Veloc > 0 AndAlso Player.Loc.Bottom >= Platform.Top AndAlso Player.Loc.Bottom - Player.Veloc - GRAVITYTH <= Platform.Top Then
'        If Player.Loc.Left < Platform.Right AndAlso Player.Loc.Right > Platform.Left Then
'            'The player has landed on this platform.  He needs to be updated.
'            'See how many advantages there are to using a Rectangle.
'            Player.IsJumping = False
'            'Hit the ground, we are not jumping.
'            Player.LeftEnd = Platform.Left : Player.RightEnd = Platform.Right
'            'Set the current end points.
'            Player.AnimCycler = 0
'            'Reset cycler.
'            Player.OnIce = False : Player.HorizontVeloc = 0
'            'We are no longer on ice.
'            Player.Loc.Offset(0, Platform.Top - Player.Loc.Bottom)
'            'Move the player onto this platform.
'            OnPlatform = True
'            'This is used to fix other problems with landing on platforms and colliding with walls.
'        End If



'    End If
'Next

'For Each TrapPlatform In Q.TrapPlatforms
'    Sh = TrapPlatform.HeightForRect(Player.Loc)

'    If Player.Veloc > 0 AndAlso Player.Loc.Bottom >= Sh AndAlso Player.Loc.Bottom - Player.Veloc - GRAVITYTH <= Sh Then
'        If Player.Loc.Left < TrapPlatform.R.Right AndAlso Player.Loc.Right > TrapPlatform.R.Left Then
'            'The player has landed on a platform.
'            'Sh is the calculated height that the player must be at to be on this sloped platform.
'            'Advantages of structures.

'            Player.IsJumping = False
'            'Hit the ground, we are not jumping.
'            Player.OnIce = False : Player.HorizontVeloc = 0
'            'We are no longer on ice.
'            Player.LeftEnd = TrapPlatform.R.Left : Player.RightEnd = TrapPlatform.R.Right
'            'Set the endpoints for the platform.
'            Player.AnimCycler = 0
'            'Reset cycler.
'            Player.Loc.Offset(0, Sh - Player.Loc.Bottom)
'            'Move the player onto this platform.
'            Player.OnSlope = True
'            'Is player walking on something sloped?
'            Player.LH = TrapPlatform.LeftHeight
'            Player.RH = TrapPlatform.RightHeight
'            'The interpolation will occur in the character movement places, after the movement checks.
'            OnPlatform = True
'            'This is used to fix other problems with landing on platforms and colliding with walls.
'        End If
'    End If
'Next


'For Each Ice In Q.Ices
'    'Looking at each platform in the platform array.
'    If Player.Veloc > 0 AndAlso Player.Loc.Bottom >= Ice.Top AndAlso Player.Loc.Bottom - Player.Veloc - GRAVITYTH <= Ice.Top Then
'        If Player.Loc.Left < Ice.Right AndAlso Player.Loc.Right > Ice.Left Then
'            'The player has landed on this platform.  He needs to be updated.
'            'See how many advantages there are to using a Rectangle.

'            Player.IsJumping = False
'            'Hit the ground, we are not jumping.
'            Player.OnIce = True
'            'Just landed on some ice.
'            Player.LeftEnd = Ice.Left : Player.RightEnd = Ice.Right
'            'Set the endpoints for the platform.
'            Player.AnimCycler = 0
'            'Reset cycler.

'            If Lft Then
'                If GoFast Then
'                    Player.HorizontVeloc = -FASTMOVESPEED
'                Else
'                    Player.HorizontVeloc = -MOVESPEED
'                End If
'            ElseIf Rght Then
'                If GoFast Then
'                    Player.HorizontVeloc = FASTMOVESPEED
'                Else
'                    Player.HorizontVeloc = MOVESPEED
'                End If
'            End If
'            'The above lines set the initial movement speed when the player initially lands on the ice.

'            Player.Loc.Offset(0, Ice.Top - Player.Loc.Bottom)
'            'Move the player onto this platform.
'        End If
'    End If
'Next

'For Each Wall In Q.Walls
'    If Player.Loc.IntersectsWith(Wall) Then
'        If OnPlatform Then  'If the player has landed on a platform.
'            Player.IsJumping = True
'            'Knock the player off the platform and make the player
'            Player.OnSlope = False
'            'Not on a slopy surface anymore.
'            Player.Loc.Offset(0, Wall.Bottom - Player.Loc.Top + GRAVITYTH)
'            'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
'            Player.Veloc = 0
'            'Make the player start falling.
'        ElseIf Player.Veloc > 0 Then
'            'If player is falling, then the player will land on the wall.
'            Player.IsJumping = False
'            'Stop the jump.
'            Player.LeftEnd = Wall.Left : Player.RightEnd = Wall.Right
'            'Set the endpoints.
'            Player.AnimCycler = 0
'            'Reset the cycler.
'            Player.OnIce = False : Player.HorizontVeloc = 0
'            'We are no longer on ice.
'            Player.Loc.Offset(0, Wall.Top - Player.Loc.Bottom)
'            'Position the player to stand on this wall.
'        Else
'            'The player has jumped up into the wall.
'            Player.Loc.Offset(0, Wall.Bottom - Player.Loc.Top)
'            'Move the player to under the wall.
'            Player.Veloc = 0
'            'Make the player start falling down.
'        End If
'    End If

'Next

'For Each MobileWall In Q.Mobilewalls
'    If Player.Loc.IntersectsWith(MobileWall.Loc) Then
'        If OnPlatform Then  'If the player has landed on a platform.
'            Player.IsJumping = True
'            'Knock the player off the platform and make the player
'            Player.OnSlope = False
'            'Not on a slopy surface anymore.
'            Player.Loc.Offset(0, MobileWall.Loc.Bottom - Player.Loc.Top + GRAVITYTH)
'            'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
'            Player.Veloc = 0
'            'Make the player start falling.

'        ElseIf Player.Veloc > 0 Then
'            'If player is falling, then the player will land on the wall.
'            Player.IsJumping = False
'            'Stop the jump.
'            Player.LeftEnd = MobileWall.Loc.Left : Player.RightEnd = MobileWall.Loc.Right
'            'Set the endpoints.
'            MobileIndex = Q.Mobilewalls.IndexOf(MobileWall)
'            'Set the index of this mobile wall in the array.
'            Player.AnimCycler = 0
'            'Reset the cycler.
'            Player.OnIce = False : Player.HorizontVeloc = 0
'            'We are no longer on ice.
'            Player.Loc.Offset(0, MobileWall.Loc.Top - Player.Loc.Bottom)
'            'Position the player to stand on this wall.
'        Else
'            'The player has jumped up into the wall.
'            Player.Loc.Offset(0, MobileWall.Loc.Bottom - Player.Loc.Top)
'            'Move the player to under the wall.
'            Player.Veloc = 0
'            'Make the player start falling down.
'        End If
'    End If
'Next

'For Each Slopedwall In Q.Slopedwalls
'    If Slopedwall.IntersectsWith(Player.Loc) Then
'        If OnPlatform Then  'If the player has landed on a platform.
'            Player.IsJumping = True
'            'Knock the player off the platform and make the player
'            Player.OnSlope = False
'            'Not on a slopy surface anymore.
'            Player.Loc.Offset(0, Slopedwall.R.Bottom - Player.Loc.Top + GRAVITYTH)
'            'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
'            Player.Veloc = 0
'            'Make the player start falling.
'        ElseIf Player.Veloc > 0 Then
'            'If player is falling, then the player will land on the wall.
'            Player.IsJumping = False
'            'Stop the jump.
'            Player.OnIce = False : Player.HorizontVeloc = 0
'            'No longer on ice.
'            Player.LeftEnd = Slopedwall.R.Left : Player.RightEnd = Slopedwall.R.Right
'            'Set the endpoints.
'            Player.AnimCycler = 0
'            'Reset the cycler.
'            Sh = Slopedwall.HeightForRect(Player.Loc)
'            Player.Loc.Offset(0, Sh - Player.Loc.Bottom)
'            'Position the player to stand on this wall.
'            Player.OnSlope = True
'            'Is player walking on something sloped?
'            Player.LH = Slopedwall.LeftHeight
'            Player.RH = Slopedwall.RightHeight
'            'The interpolation will occur in the character movement places, after the movement checks.
'        Else
'            'The player has jumped up into the wall.
'            Player.Loc.Offset(0, Slopedwall.R.Bottom - Player.Loc.Top)
'            'Move the player to under the wall.
'            Player.Veloc = 0
'            'Make the player start falling down.        
'        End If


'    End If
'Next

'For Each Jumpboard In Q.Jumpboards
'    If Player.Loc.IntersectsWith(Jumpboard.Loc) Then
'        If OnPlatform Then  'If the player has landed on a platform.
'            Player.IsJumping = True
'            'Knock the player off the platform and make the player
'            Player.OnSlope = False
'            'Not on a slopy surface anymore.
'            Player.Loc.Offset(0, Jumpboard.Loc.Bottom - Player.Loc.Top + GRAVITYTH)
'            'Move the player to under the wall.  We need GRAVITYTH so that the player climb up to the platform.
'            Player.Veloc = 0
'            'Make the player start falling.

'        ElseIf Player.Veloc > 0 Then
'            'If player is falling, then the player will land on the wall.
'            Player.Veloc = Jumpboard.ReboundVeloc / 3.2F
'            'Bounce the player off of this board.
'            Player.Loc.Offset(0, Jumpboard.Loc.Top - Player.Loc.Bottom)
'            'Position the player to stand on this wall.
'        Else
'            'The player has jumped up into the wall.
'            Player.Loc.Offset(0, Jumpboard.Loc.Bottom - Player.Loc.Top)
'            'Move the player to under the wall.
'            Player.Veloc = 0
'            'Make the player start falling down.
'        End If
'    End If
'Next

Public Enum MeshTypes
    Cylinder
    Sphere
End Enum

Public Class MeshPlus
    Implements IDisposable

    Const Pi As Single = 3.1415926535897931
    Const PiHalf As Single = Pi / 2

    Public MasterMesh As Mesh
    Public MMMtrls() As Material
    Public MMTxtrs() As Texture
    Public ReadOnly SubsetsEx As Integer  'Number of subsets - 1.
    Public RSTMatrix As Matrix  'Rotation, scale, and translate matrix.

    Public Sub New(ByVal Filename As String, ByVal Options As MeshFlags, ByVal D As Device)
        Dim Ems() As ExtendedMaterial, LV As Integer
        'Merely holds a material and a filename for a texture.

        'MasterMesh = Mesh.FromFile(Application.StartupPath & "\mball.x", MeshFlags.SoftwareProcessing, D9)

        'You can use the one above if you have a material and texture that you will be supplying to the mesh.

        'However, the material and texture can be included within the .x file.

        MasterMesh = Mesh.FromFile(Filename, Options, D, Ems)
        'Loads the mesh from file.  .x data goes into the Extended Material array.

        'The number of entries corresponds to the number of materials and textures.

        SubsetsEx = Ems.GetUpperBound(0)
        MMMtrls = New Material(SubsetsEx) {}
        MMTxtrs = New Texture(SubsetsEx) {}

        For LV = 0 To SubsetsEx
            MMMtrls(LV) = Ems(LV).Material3D
            'Set the material.  3D means DX3D.

            MMMtrls(LV).Ambient = MMMtrls(LV).Diffuse
            'Ambient property is lost within the loader or in the .x file.
            'But anyway, this property must be set in VB.

            'Not every mesh will have a texture, so meshes without textures will have Texture = Nothing
            If Not (Ems(LV).TextureFilename Is Nothing) Then
                MMTxtrs(LV) = TextureLoader.FromFile(D, Ems(LV).TextureFilename)
            End If
        Next

        'All of this information will be used when drawing the mesh.
    End Sub
    Public Sub New(ByVal Filename As String, ByVal Options As MeshFlags, ByVal D As Device, ByVal Mx As Matrix)
        Dim Ems() As ExtendedMaterial, LV As Integer
        MasterMesh = Mesh.FromFile(Filename, Options, D, Ems)

        SubsetsEx = Ems.GetUpperBound(0)
        MMMtrls = New Material(SubsetsEx) {}
        MMTxtrs = New Texture(SubsetsEx) {}

        For LV = 0 To SubsetsEx
            MMMtrls(LV) = Ems(LV).Material3D

            MMMtrls(LV).Ambient = MMMtrls(LV).Diffuse

            If Not (Ems(LV).TextureFilename Is Nothing) Then
                MMTxtrs(LV) = TextureLoader.FromFile(D, Ems(LV).TextureFilename)
            End If
        Next

        Me.RSTMatrix = Mx
        'Viewpoint matrix.
        'All of this information will be used when drawing the mesh.
    End Sub
    Public Sub New(ByVal Filename As String, ByVal Options As MeshFlags, ByVal D As Device, ByVal RotateYZ As Single, ByVal RotateXY As Single, ByVal RotateXZ As Single, ByVal SX As Single, ByVal SY As Single, ByVal SZ As Single, ByVal X As Single, ByVal Y As Single, ByVal Z As Single)
        Dim Ems() As ExtendedMaterial, LV As Integer

        MasterMesh = Mesh.FromFile(Filename, Options, D, Ems)

        SubsetsEx = Ems.GetUpperBound(0)
        MMMtrls = New Material(SubsetsEx) {}
        MMTxtrs = New Texture(SubsetsEx) {}

        For LV = 0 To SubsetsEx
            MMMtrls(LV) = Ems(LV).Material3D

            MMMtrls(LV).Ambient = MMMtrls(LV).Diffuse

            If Not (Ems(LV).TextureFilename Is Nothing) Then
                MMTxtrs(LV) = TextureLoader.FromFile(D, Ems(LV).TextureFilename)
            End If
        Next

        'First, we apply the rotations:
        RSTMatrix = Matrix.RotationX(RotateYZ)
        RSTMatrix.Multiply(Matrix.RotationZ(RotateXY))
        RSTMatrix.Multiply(Matrix.RotationY(RotateXZ))

        'Next, we do the scaling:
        RSTMatrix.Multiply(Matrix.Scaling(SX, SY, SZ))

        'Then, we do the translation:
        RSTMatrix.Multiply(Matrix.Translation(X, Y, Z))

    End Sub
    Public Sub New(ByVal Filename As String, ByVal Options As MeshFlags, ByVal D As Device, ByVal RotateYZ As Single, ByVal RotateXY As Single, ByVal RotateXZ As Single, ByVal S As Vector3, ByVal P As Vector3)
        Dim Ems() As ExtendedMaterial, LV As Integer

        MasterMesh = Mesh.FromFile(Filename, Options, D, Ems)

        SubsetsEx = Ems.GetUpperBound(0)
        MMMtrls = New Material(SubsetsEx) {}
        MMTxtrs = New Texture(SubsetsEx) {}

        For LV = 0 To SubsetsEx
            MMMtrls(LV) = Ems(LV).Material3D

            MMMtrls(LV).Ambient = MMMtrls(LV).Diffuse

            If Not (Ems(LV).TextureFilename Is Nothing) Then
                MMTxtrs(LV) = TextureLoader.FromFile(D, Ems(LV).TextureFilename)
            End If
        Next

        'First, we apply the rotations:
        RSTMatrix = Matrix.RotationX(RotateYZ)
        RSTMatrix.Multiply(Matrix.RotationZ(RotateXY))
        RSTMatrix.Multiply(Matrix.RotationY(RotateXZ))

        'Next, we do the scaling:
        RSTMatrix.Multiply(Matrix.Scaling(S))

        'Then, we do the translation:
        RSTMatrix.Multiply(Matrix.Translation(P))
    End Sub
    Public Sub New(ByVal MeshType As MeshTypes, ByVal Options As MeshFlags, ByVal D As Device, ByVal RotateYZ As Single, ByVal RotateXY As Single, ByVal RotateXZ As Single, ByVal S As Vector3, ByVal P As Vector3)
        Dim Ems() As ExtendedMaterial, LV As Integer

        If MeshType = MeshTypes.Cylinder Then
            MasterMesh = Mesh.Cylinder(D, 1.0F, 0.5F, 1.0F, 6, 4)
        ElseIf MeshType = MeshTypes.Sphere Then
            MasterMesh = Mesh.Sphere(D, 1.0F, 4, 4)
        End If

        MMMtrls = New Material(0) {}
        MMTxtrs = New Texture(0) {}

        'For LV = 0 To SubsetsEx
        '    MMMtrls(LV) = Ems(LV).Material3D

        '    MMMtrls(LV).Ambient = MMMtrls(LV).Diffuse

        '    If Not (Ems(LV).TextureFilename Is Nothing) Then
        '        MMTxtrs(LV) = TextureLoader.FromFile(D, Ems(LV).TextureFilename)
        '    End If
        'Next

        'First, we apply the rotations:
        RSTMatrix = Matrix.RotationX(RotateYZ)
        RSTMatrix.Multiply(Matrix.RotationZ(RotateXY))
        RSTMatrix.Multiply(Matrix.RotationY(RotateXZ))

        'Next, we do the scaling:
        RSTMatrix.Multiply(Matrix.Scaling(S))

        'Then, we do the translation:
        RSTMatrix.Multiply(Matrix.Translation(P))

        'mastermesh.VertexBuffer


    End Sub

    Public Sub DrawAllSubsets(ByVal D As Device)
        'Each subset must be drawn separately and is what we extracted data from.

        Dim LV As Integer
        For LV = 0 To SubsetsEx
            D.Material = MMMtrls(LV)
            'Set the appropriate material for this subset.
            If MMTxtrs(LV) Is Nothing Then
                D.SetTexture(0, Nothing)
                'Clear unset texture.
            Else
                D.SetTexture(0, MMTxtrs(LV))
                'And the appropriate texture.
            End If

            'Then draw the mesh subset.
            MasterMesh.DrawSubset(LV)

        Next
    End Sub
    Public Sub DrawAllSubsetsXF(ByVal D As Device, ByVal DrawTxs As Boolean, ByVal UseMtrl As Boolean)
        'Each subset must be drawn separately and is what we extracted data from.

        Dim LV As Integer
        D.Transform.World = RSTMatrix

        If DrawTxs Then
            For LV = 0 To SubsetsEx
                If UseMtrl Then
                    D.Material = MMMtrls(LV)
                    'Set the appropriate material for this subset.

                End If
                If MMTxtrs(LV) Is Nothing Then
                    D.SetTexture(0, Nothing)
                    'Clear unset texture.
                Else
                    D.SetTexture(0, MMTxtrs(LV))
                    'And the appropriate texture.
                End If

                'Then draw the mesh subset.
                MasterMesh.DrawSubset(LV)

            Next
        Else
            For LV = 0 To SubsetsEx
                If UseMtrl Then
                    D.Material = MMMtrls(LV)
                    'Set the appropriate material for this subset.

                End If


                'Then draw the mesh subset.
                MasterMesh.DrawSubset(LV)

            Next
        End If


    End Sub

    Public Sub Dispose() Implements System.IDisposable.Dispose
        Dim LV As Integer
        For LV = 0 To SubsetsEx
            MMTxtrs(LV).Dispose()
        Next
        MasterMesh.Dispose()
    End Sub
End Class

Public Class Sonic3
    'String.empty?

    Public Const SND_APPLICATION As Integer = &H80          ' look for application specific association
    Public Const SND_ALIAS As Integer = &H10000    ' name is a WIN.INI [sounds] entry
    Public Const SND_ALIAS_ID As Integer = &H110000   ' name is a WIN.INI [sounds] entry identifier
    Public Const SND_ASYNC As Integer = &H1        ' play asynchronously
    Public Const SND_FILENAME As Integer = &H20000    ' name is a file name
    Public Const SND_LOOP As Integer = &H8        ' loop the sound until next sndPlaySound
    Public Const SND_MEMORY As Integer = &H4        ' lpszSoundName points to a memory file
    Public Const SND_NODEFAULT As Integer = &H2        ' silence not default, if sound not found
    Public Const SND_NOSTOP As Integer = &H10       ' don't stop any currently playing sound
    Public Const SND_NOWAIT As Integer = &H2000     ' don't wait if the driver is busy
    Public Const SND_PURGE As Integer = &H40              ' purge non-static events for task
    Public Const SND_RESOURCE As Integer = &H40004    ' name is a resource name or atom
    Public Const SND_SYNC As Integer = &H0        ' play synchronously (default)

    Private Declare Function PlaySound Lib "winmm.dll" Alias "PlaySoundA" (ByVal lpszName As String, ByVal hModule As Integer, ByVal dwFlags As Integer) As Integer
    Private WithEvents Bbx As Microsoft.DirectX.AudioVideoPlayback.Audio

    Public Prevent As Boolean
    Public MusicVol, WaveVol As Integer

    Dim Mfn As String

    Dim DS As Microsoft.DirectX.DirectSound.Device
    Dim Trx() As Microsoft.DirectX.DirectSound.SecondaryBuffer
    Public Sub PlayWave(ByVal Filename As String, ByVal Trxnum As Integer, ByVal PlayBool As Boolean)
        Trx(Trxnum) = New Microsoft.DirectX.DirectSound.SecondaryBuffer(Filename, DS)
        If PlayBool Then
            Trx(Trxnum).Play(0, Microsoft.DirectX.DirectSound.BufferPlayFlags.Default)

        End If
    End Sub
    Public Sub PlayWave(ByVal Trxnum As Integer, ByVal PlayBool As Boolean)
        Try
            If PlayBool Then
                Trx(Trxnum).SetCurrentPosition(0)
                Trx(Trxnum).Play(0, Microsoft.DirectX.DirectSound.BufferPlayFlags.Default)
            End If
        Catch ex As Exception

        End Try
    End Sub
    Public Sub PlayWaveLoop(ByVal Filename As String, ByVal Trxnum As Integer, ByVal PlayBool As Boolean)
        Trx(Trxnum) = New Microsoft.DirectX.DirectSound.SecondaryBuffer(Filename, DS)
        If PlayBool Then
            Trx(Trxnum).Play(0, Microsoft.DirectX.DirectSound.BufferPlayFlags.Looping)
        End If
    End Sub
    Public Sub PlayWaveLoop(ByVal trxnum As Integer, ByVal PlayBool As Boolean)
        If PlayBool Then
            Trx(trxnum).Play(0, Microsoft.DirectX.DirectSound.BufferPlayFlags.Looping)
        End If
    End Sub
    Public Sub StopWave(ByVal Trxnum As Integer)
        Trx(Trxnum).Stop()
    End Sub
    Public Function PlayMusic(ByVal Filename As String) As Double

        PlayMusic = StopMusic()
        If Not Prevent Then
            If Bbx Is Nothing Then
                Bbx = Bbx.FromFile(Filename, True)
                Mfn = Filename
                SetMusicVolume(MusicVol)
            ElseIf Not Bbx.Playing Then
                Bbx.Open(Filename, True)
            End If
        End If

    End Function
    Public Function PlayMusic(ByVal Filename As String, ByVal StartAt As Double) As Double

        PlayMusic = StopMusic()
        If Not Prevent Then
            If Bbx Is Nothing Then
                Bbx = Bbx.FromFile(Filename, False)
                Mfn = Filename
                SetMusicVolume(MusicVol)
                Bbx.CurrentPosition = StartAt
                Bbx.Play()
            ElseIf Not Bbx.Playing Then
                Bbx.Open(Filename, False)
                Bbx.CurrentPosition = StartAt
                Bbx.Play()
            End If
        End If

    End Function
    Public Function StopMusic() As Double
        If Not Bbx Is Nothing AndAlso Not Bbx.Stopped Then
            StopMusic = Bbx.CurrentPosition
            Bbx.Stop()

        End If
    End Function
    Public Sub SetWaveVolume(ByVal Percent As Integer)
        Dim LV As Integer
        If Percent > 100 Then
            WaveVol = 100
        ElseIf Percent < 0 Then
            WaveVol = 0
        Else
            WaveVol = Percent
        End If

        Dim Wv As Integer = (Percent - 100) * 100  '0 to 100 -> -100 to 0 -> -10000 to 0
        For LV = 0 To Trx.GetUpperBound(0)
            Trx(LV).Volume = Wv
        Next
    End Sub
    Public Sub SetMusicVolume(ByVal Percent As Integer)
        Dim LV As Integer
        If Percent > 100 Then
            MusicVol = 100
        ElseIf Percent < 0 Then
            MusicVol = 0
        Else
            MusicVol = Percent
        End If

        Dim Wv As Integer = (Percent - 100) * 10  '0 to 100 -> -100 to 0 -> -10000 to 0
        If Not (Bbx Is Nothing) Then
            If Not Mfn.EndsWith("mid") Then
                Bbx.Volume = Wv
            End If
        End If
    End Sub
    Public Sub Silence()
        Dim LV As Integer
        PlaySound(String.Empty, 0, Me.SND_PURGE)
        For LV = 0 To Trx.GetUpperBound(0)
            StopWave(LV)
        Next
        StopMusic()
    End Sub
    Public Sub New(ByVal NumTraxEx As Integer, ByVal WindowHandle As IntPtr)
        Mfn = String.Empty
        DS = New Microsoft.DirectX.DirectSound.Device
        DS.SetCooperativeLevel(WindowHandle, DirectSound.CooperativeLevel.Priority)
        Trx = New Microsoft.DirectX.DirectSound.SecondaryBuffer(NumTraxEx) {}
        SetWaveVolume(80)
        SetMusicVolume(80)
    End Sub
    Public Sub New(ByVal WindowHandle As IntPtr, ByVal TrackNameList() As String)
        Dim LV As Integer
        Mfn = String.Empty
        DS = New Microsoft.DirectX.DirectSound.Device
        DS.SetCooperativeLevel(WindowHandle, DirectSound.CooperativeLevel.Priority)
        Trx = New Microsoft.DirectX.DirectSound.SecondaryBuffer(TrackNameList.GetUpperBound(0)) {}
        For LV = 0 To Trx.GetUpperBound(0)
            Trx(LV) = New Microsoft.DirectX.DirectSound.SecondaryBuffer(TrackNameList(LV), DS)
        Next
        SetWaveVolume(80)
        SetMusicVolume(80)
    End Sub

    Protected Overrides Sub Finalize()
        Dim LV As Integer
        MyBase.Finalize()
        If Not Bbx Is Nothing Then
            Bbx.Dispose()
            Bbx = Nothing
        End If
        If Not DS Is Nothing Then
            For LV = 0 To Trx.GetUpperBound(0)
                Trx(LV) = Nothing
            Next
            DS = Nothing
        End If
    End Sub

    Private Sub Bbx_Ending(ByVal sender As Object, ByVal e As System.EventArgs) Handles Bbx.Ending
        'Bbx.Play()
        'Diagnostics.Debug.WriteLine("Ended.")
        Bbx.CurrentPosition = 0.0#
        Bbx.Play()
    End Sub

    Private Sub Bbx_Pausing(ByVal sender As Object, ByVal e As System.EventArgs) Handles Bbx.Pausing
        'Diagnostics.Debug.WriteLine("Paused.")

    End Sub

    Private Sub Bbx_Starting(ByVal sender As Object, ByVal e As System.EventArgs) Handles Bbx.Starting
        'Diagnostics.Debug.WriteLine("Started.")

    End Sub

    Private Sub Bbx_Stopping(ByVal sender As Object, ByVal e As System.EventArgs) Handles Bbx.Stopping
        'Diagnostics.Debug.WriteLine("Stopped.")

    End Sub

    Private Sub Bbx_Disposing(ByVal sender As Object, ByVal e As System.EventArgs) Handles Bbx.Disposing
        'Diagnostics.Debug.WriteLine("Disposed.")
    End Sub
End Class
'Added options selection to menu


Public Enum Diagonals
    UpLeft = 0
    UpRight = 1
    DownLeft = 2
    DownRight = 3
End Enum

Public Structure StandTrapezoid
    Private mr As RectangleF
    Private dg As Diagonals
    Private mx As Single
    Private pts() As PointF

    Public Property R() As RectangleF
        Get
            Return mr
        End Get
        Set(ByVal Value As RectangleF)
            mr = Value
            UpdatePts()
        End Set
    End Property

    Public Property Diag() As Diagonals
        Get
            Return dg
        End Get
        Set(ByVal Value As Diagonals)
            dg = Value
            UpdatePts()
        End Set
    End Property

    Public Property MiddleValue() As Single
        Get
            Return mx
        End Get
        Set(ByVal Value As Single)
            If Value > mr.Bottom Then
                mx = mr.Bottom
            ElseIf Value < mr.Top Then
                mx = mr.Top
            Else
                mx = Value
            End If
            UpdatePts()
        End Set
    End Property

    Public Function HeightForRect(ByVal R As RectangleF) As Single
        Dim L As Single, Percent As Double
        If dg = Diagonals.UpLeft Then
            L = R.Right
            Percent = (L - mr.Left) / mr.Width
            If Percent < 1.0R Then
                Return mx + Convert.ToInt32(Percent * (mr.Top - mx))
            Else
                Return mr.Top
            End If
        Else
            L = R.Left
            Percent = (L - mr.Left) / mr.Width
            If Percent > 0.0R Then
                Return mr.Top + Convert.ToInt32(Percent * (mx - mr.Top))
            Else
                Return mr.Top
            End If
        End If
    End Function

    Public Function HorValueForRect(ByVal R As RectangleF, ByVal LeftSide As Boolean) As Single
        Dim T As Single, Percent As Double
        If dg = Diagonals.UpLeft Then
            If LeftSide Then
                T = R.Bottom
                Percent = (T - mr.Top) / (mx - mr.Top)
                If Percent < 1.0R Then
                    Return mr.Right - Convert.ToInt32(Percent * mr.Width)
                Else
                    Return mr.Left
                End If
            Else
                Return mr.Right
            End If
        Else
            If LeftSide Then
                Return mr.Left
            Else
                T = R.Bottom
                Percent = (T - mr.Top) / (mx - mr.Top)
                If Percent < 1.0R Then
                    Return mr.Left + Convert.ToInt32(Percent * mr.Width)
                Else
                    Return mr.Right
                End If
            End If
        End If
    End Function

    Public ReadOnly Property LeftHeight() As Single
        Get
            If dg = Diagonals.UpLeft Then
                Return mx
            Else
                Return mr.Top
            End If
        End Get
    End Property
    Public ReadOnly Property RightHeight() As Single
        Get
            If dg = Diagonals.UpLeft Then
                Return mr.Top
            Else
                Return mx
            End If
        End Get
    End Property

    Public Function IntersectsWith(ByVal R As RectangleF) As Boolean
        Dim Hgt As Single
        If mr.IntersectsWith(R) Then
            Hgt = HeightForRect(R)
            If R.Bottom > Hgt Then
                IntersectsWith = True
            End If
        End If
    End Function


    Public Sub New(ByVal R As RectangleF, ByVal Diag As Diagonals, ByVal MiddleValue As Integer)
        mr = R
        dg = Diag
        If MiddleValue > mr.Bottom Then
            mx = mr.Bottom
        ElseIf MiddleValue < mr.Top Then
            mx = mr.Top
        Else
            mx = MiddleValue
        End If
        UpdatePts()
    End Sub

    Public Sub Offset(ByVal dx As Single, ByVal dy As Single)
        mr.Offset(dx, dy)
        pts(0).X += dx
        pts(0).Y += dy
        pts(1).X += dx
        pts(1).Y += dy

        pts(2).X += dx
        pts(2).Y += dy
        pts(3).X += dx
        pts(3).Y += dy


        mx += dy
    End Sub

    Public Sub Draw(ByVal GFX As Graphics, ByVal P As Pen)
        GFX.DrawPolygon(P, pts)
    End Sub
    Public Sub Fill(ByVal GFX As Graphics, ByVal B As Brush)
        GFX.FillPolygon(B, pts)
    End Sub

    Private Sub UpdatePts()
        If pts Is Nothing Then
            pts = New PointF(3) {}
        End If
        Select Case dg
            Case Diagonals.UpLeft
                pts(0) = New PointF(mr.Right, mr.Top)
                pts(1) = New PointF(mr.Left, mx)
                pts(2) = New PointF(mr.Left, mr.Bottom)
                pts(3) = New PointF(mr.Right, mr.Bottom)
            Case Diagonals.UpRight
                pts(0) = New PointF(mr.Left, mr.Top)
                pts(1) = New PointF(mr.Left, mr.Bottom)
                pts(2) = New PointF(mr.Right, mr.Bottom)
                pts(3) = New PointF(mr.Right, mx)
            Case Diagonals.DownLeft
                pts(0) = New PointF(mr.Left, mr.Top)
                pts(1) = New PointF(mr.Right, mr.Top)
                pts(2) = New PointF(mr.Right, mr.Bottom)
                pts(3) = New PointF(mr.Left, mx)
            Case Diagonals.DownRight
                pts(0) = New PointF(mr.Left, mr.Top)
                pts(1) = New PointF(mr.Left, mr.Bottom)
                pts(2) = New PointF(mr.Right, mx)
                pts(3) = New PointF(mr.Right, mr.Top)
        End Select
    End Sub
End Structure

'Drop of height H.  Must hit target at distance of R.  Initial velocity is only in the horizontal direction Vx.
'Every tick, Vy = Vy + A, X = X + Vx, Y = Y + Vy
'Time to reach height difference H depends only on A.
'Progression of Y: 0 1 3 6 10 15 21 28 36 45
'H = 0.5T(T+1) = 0.5T + 0.5T, 0 = 0.5T + 0.5T - H
'T = -0.5 + sqr[0.25 + 2H] 
'This is how long the shot will be in the air, and the shot must travel the distance of R in this time of T, thus, the value of Vx
'is equal to R/T = R / [sqr(0.25 + 2H) - 0.5]